@@ -6,11 +6,11 @@
  * Authors:
  *   Mitsuru Oka
  *   Lauris Kaplinski <lauris@kaplinski.com>
+ *   bulia byak <buliabyak@users.sf.net>
  *
- * Copyright (C) 2000 Lauris Kaplinski
+ * Copyright (C) 2000-2002 Lauris Kaplinski
  * Copyright (C) 2000-2001 Ximian, Inc.
  * Copyright (C) 2002 Mitsuru Oka
- * Copyright (C) 2002 Lauris Kaplinski
  *
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
@@ -32,11 +32,11 @@
 #include "desktop-affine.h"
 #include "snap.h"
 #include "pixmaps/cursor-arc.xpm"
-#include "sp-desktop-widget.h"
+#include "desktop-widget.h"
 #include "sp-metrics.h"
 #include "knotholder.h"
 #include "xml/repr.h"
-#include "xml/sp-repr-event-vector.h"
+#include "xml/node-event-vector.h"
 #include "object-edit.h"
 #include "prefs-utils.h"
 #include "widgets/spw-utilities.h"
@@ -110,6 +110,9 @@ static void sp_arc_context_init(SPArcContext *arc_context)
     event_context->within_tolerance = false;
     event_context->item_to_select = NULL;
 
+    event_context->shape_repr = NULL;
+    event_context->shape_knot_holder = NULL;
+
     arc_context->item = NULL;
 
     new (&arc_context->sel_changed_connection) sigc::connection();
@@ -120,18 +123,20 @@ static void sp_arc_context_dispose(GObject *object)
     SPEventContext *ec = SP_EVENT_CONTEXT(object);
     SPArcContext *ac = SP_ARC_CONTEXT(object);
 
+    ec->enableGrDrag(false);
+
     ac->sel_changed_connection.disconnect();
     ac->sel_changed_connection.~connection();
 
-    if (ac->knot_holder) {
-        sp_knot_holder_destroy(ac->knot_holder);
-        ac->knot_holder = NULL;
+    if (ec->shape_knot_holder) {
+        sp_knot_holder_destroy(ec->shape_knot_holder);
+        ec->shape_knot_holder = NULL;
     }
 
-    if (ac->repr) { // remove old listener
-        sp_repr_remove_listener_by_data(ac->repr, ec);
-        sp_repr_unref(ac->repr);
-        ac->repr = 0;
+    if (ec->shape_repr) { // remove old listener
+        sp_repr_remove_listener_by_data(ec->shape_repr, ec);
+        sp_repr_unref(ec->shape_repr);
+        ec->shape_repr = 0;
     }
 
     /* fixme: This is necessary because we do not grab */
@@ -144,29 +149,10 @@ static void sp_arc_context_dispose(GObject *object)
     G_OBJECT_CLASS(parent_class)->dispose(object);
 }
 
-static void shape_event_attr_changed(SPRepr * repr, const gchar * name, const gchar * old_value, const gchar * new_value, bool is_interactive, gpointer data)
-{
-    SPArcContext *ac = SP_ARC_CONTEXT(data);
-    SPEventContext *ec = SP_EVENT_CONTEXT(ac);
-
-    if (ac->knot_holder) {
-        sp_knot_holder_destroy(ac->knot_holder);
-    }
-    ac->knot_holder = NULL;
-
-    SPDesktop *desktop = ec->desktop;
-
-    SPItem *item = SP_DT_SELECTION(desktop)->singleItem();
-
-    if (item) {
-        ac->knot_holder = sp_item_knot_holder(item, desktop);
-    }
-}
-
-static SPReprEventVector shape_repr_events = {
+static Inkscape::XML::NodeEventVector ec_shape_repr_events = {
     NULL, /* child_added */
     NULL, /* child_removed */
-    shape_event_attr_changed,
+    ec_shape_event_attr_changed,
     NULL, /* content_changed */
     NULL  /* order_changed */
 };
@@ -176,31 +162,31 @@ static SPReprEventVector shape_repr_events = {
 destroys old and creates new knotholder.
 */
 void
-sp_arc_context_selection_changed(SPSelection * selection, gpointer data)
+sp_arc_context_selection_changed(Inkscape::Selection * selection, gpointer data)
 {
     SPArcContext *ac = SP_ARC_CONTEXT(data);
     SPEventContext *ec = SP_EVENT_CONTEXT(ac);
 
-    if (ac->knot_holder) { // desktroy knotholder
-        sp_knot_holder_destroy(ac->knot_holder);
-        ac->knot_holder = NULL;
+    if (ec->shape_knot_holder) { // desktroy knotholder
+        sp_knot_holder_destroy(ec->shape_knot_holder);
+        ec->shape_knot_holder = NULL;
     }
 
-    if (ac->repr) { // remove old listener
-        sp_repr_remove_listener_by_data(ac->repr, ec);
-        sp_repr_unref(ac->repr);
-        ac->repr = 0;
+    if (ec->shape_repr) { // remove old listener
+        sp_repr_remove_listener_by_data(ec->shape_repr, ec);
+        sp_repr_unref(ec->shape_repr);
+        ec->shape_repr = 0;
     }
 
     SPItem *item = selection ? selection->singleItem() : NULL;
     if (item) {
-        ac->knot_holder = sp_item_knot_holder(item, ec->desktop);
-        SPRepr *repr = SP_OBJECT_REPR(item);
-        if (repr) {
-            ac->repr = repr;
-            sp_repr_ref(repr);
-            sp_repr_add_listener(repr, &shape_repr_events, ec);
-            sp_repr_synthesize_events(repr, &shape_repr_events, ec);
+        ec->shape_knot_holder = sp_item_knot_holder(item, ec->desktop);
+        Inkscape::XML::Node *shape_repr = SP_OBJECT_REPR(item);
+        if (shape_repr) {
+            ec->shape_repr = shape_repr;
+            sp_repr_ref(shape_repr);
+            sp_repr_add_listener(shape_repr, &ec_shape_repr_events, ec);
+            sp_repr_synthesize_events(shape_repr, &ec_shape_repr_events, ec);
         }
 
 
@@ -218,13 +204,13 @@ sp_arc_context_setup(SPEventContext *ec)
     SPItem *item = SP_DT_SELECTION(ec->desktop)->singleItem();
 
     if (item) {
-        ac->knot_holder = sp_item_knot_holder(item, ec->desktop);
-        SPRepr *repr = SP_OBJECT_REPR(item);
-        if (repr) {
-            ac->repr = repr;
-            sp_repr_ref(repr);
-            sp_repr_add_listener(repr, &shape_repr_events, ec);
-            sp_repr_synthesize_events(repr, &shape_repr_events, ec);
+        ec->shape_knot_holder = sp_item_knot_holder(item, ec->desktop);
+        Inkscape::XML::Node *shape_repr = SP_OBJECT_REPR(item);
+        if (shape_repr) {
+            ec->shape_repr = shape_repr;
+            sp_repr_ref(shape_repr);
+            sp_repr_add_listener(shape_repr, &ec_shape_repr_events, ec);
+            sp_repr_synthesize_events(shape_repr, &ec_shape_repr_events, ec);
         }
     }
 
@@ -235,6 +221,10 @@ sp_arc_context_setup(SPEventContext *ec)
         ec->enableSelectionCue();
     }
 
+    if (prefs_get_int_attribute("tools.shapes", "gradientdrag", 0) != 0) {
+        ec->enableGrDrag();
+    }
+
     ac->_message_context = new Inkscape::MessageContext(SP_VIEW(ec->desktop)->messageStack());
 }
 
@@ -254,8 +244,8 @@ static gint sp_arc_context_item_handler(SPEventContext *event_context, SPItem *i
                 event_context->yp = (gint) event->button.y;
                 event_context->within_tolerance = true;
 
-                // remember clicked item, disregarding groups
-                event_context->item_to_select = sp_desktop_item_at_point(desktop, NR::Point(event->button.x, event->button.y), TRUE);
+                // remember clicked item, disregarding groups, honoring Alt
+                event_context->item_to_select = sp_event_context_find_item (desktop, NR::Point(event->button.x, event->button.y), event->button.state, TRUE);
 
                 ret = TRUE;
             }
@@ -276,6 +266,8 @@ static gint sp_arc_context_root_handler(SPEventContext *event_context, GdkEvent
     static gboolean dragging;
 
     SPDesktop *desktop = event_context->desktop;
+    Inkscape::Selection *selection = SP_DT_SELECTION (desktop);
+
     SPArcContext *ac = SP_ARC_CONTEXT(event_context);
 
     event_context->tolerance = prefs_get_int_attribute_limited("options.dragtolerance", "value", 0, 0, 100);
@@ -290,8 +282,8 @@ static gint sp_arc_context_root_handler(SPEventContext *event_context, GdkEvent
                 event_context->yp = (gint) event->button.y;
                 event_context->within_tolerance = true;
 
-                // remember clicked item, disregarding groups
-                event_context->item_to_select = sp_desktop_item_at_point(desktop, NR::Point(event->button.x, event->button.y), TRUE);
+                // remember clicked item, disregarding groups, honoring Alt
+                event_context->item_to_select = sp_event_context_find_item (desktop, NR::Point(event->button.x, event->button.y), event->button.state, TRUE);
 
                 dragging = TRUE;
                 /* Position center */
@@ -333,10 +325,14 @@ static gint sp_arc_context_root_handler(SPEventContext *event_context, GdkEvent
                     sp_arc_finish(ac);
                 } else if (event_context->item_to_select) {
                     // no dragging, select clicked item if any
-                    SP_DT_SELECTION(desktop)->setItem(event_context->item_to_select);
+                    if (event->button.state & GDK_SHIFT_MASK) {
+                        selection->toggle(event_context->item_to_select);
+                    } else {
+                        selection->set(event_context->item_to_select);
+                    }
                 } else {
                     // click in an empty space
-                    SP_DT_SELECTION(desktop)->clear();
+                    selection->clear();
                 }
                 event_context->xp = 0;
                 event_context->yp = 0;
@@ -346,7 +342,7 @@ static gint sp_arc_context_root_handler(SPEventContext *event_context, GdkEvent
             sp_canvas_item_ungrab(SP_CANVAS_ITEM(desktop->acetate), event->button.time);
             break;
         case GDK_KEY_PRESS:
-            switch (event->key.keyval) {
+            switch (get_group0_keyval (&event->key)) {
                 case GDK_Alt_L:
                 case GDK_Alt_R:
                 case GDK_Control_L:
@@ -430,7 +426,7 @@ static void sp_arc_drag(SPArcContext *ac, NR::Point pt, guint state)
         }
 
         /* Create object */
-        SPRepr *repr = sp_repr_new("svg:path");
+        Inkscape::XML::Node *repr = sp_repr_new("svg:path");
         sp_repr_set_attr(repr, "sodipodi:type", "arc");
 
         /* Set style */
@@ -504,7 +500,7 @@ static void sp_arc_drag(SPArcContext *ac, NR::Point pt, guint state)
 
     GString *xs = SP_PX_TO_METRIC_STRING(fabs(x1-x0), sp_desktop_get_default_metric(desktop));
     GString *ys = SP_PX_TO_METRIC_STRING(fabs(y1-y0), sp_desktop_get_default_metric(desktop));
-    ac->_message_context->setF(Inkscape::NORMAL_MESSAGE, _("<b>Ellipse</b>: %s x %s; with <b>Ctrl</b> to make circle or integer-ratio ellipse; with <b>Shift</b> to draw around the starting point"), xs->str, ys->str);
+    ac->_message_context->setF(Inkscape::NORMAL_MESSAGE, _("<b>Ellipse</b>: %s &#215; %s; with <b>Ctrl</b> to make circle or integer-ratio ellipse; with <b>Shift</b> to draw around the starting point"), xs->str, ys->str);
     g_string_free(xs, FALSE);
     g_string_free(ys, FALSE);
 }
@@ -518,7 +514,7 @@ static void sp_arc_finish(SPArcContext *ac)
 
         SP_OBJECT(ac->item)->updateRepr();
 
-        SP_DT_SELECTION(desktop)->setItem(ac->item);
+        SP_DT_SELECTION(desktop)->set(ac->item);
         sp_document_done(SP_DT_DOCUMENT(desktop));
 
         ac->item = NULL;
