@@ -25,7 +25,7 @@
 #include "display/guideline.h"
 #include "helper/unit-menu.h"
 #include "helper/units.h"
-#include "libnr/nr-matrix-ops.h"
+#include "libnr/nr-point-matrix-ops.h"
 #include "widgets/icon.h"
 #include "inkscape-private.h"
 #include "desktop.h"
@@ -35,7 +35,7 @@
 #include "desktop-affine.h"
 #include "desktop-handles.h"
 #include "event-context.h"
-#include "sp-desktop-widget.h"
+#include "desktop-widget.h"
 #include "sp-metrics.h"
 #include "sp-item.h"
 #include "desktop-events.h"
@@ -90,7 +90,7 @@ sp_dt_ruler_event (GtkWidget *widget, GdkEvent *event, SPDesktopWidget *dtw, gbo
 	int wx, wy;
 
 	SPDesktop *desktop = dtw->desktop;
-	SPRepr *repr = SP_OBJECT_REPR (desktop->namedview);
+	Inkscape::XML::Node *repr = SP_OBJECT_REPR (desktop->namedview);
 
 	gdk_window_get_pointer (GTK_WIDGET (dtw->canvas)->window, &wx, &wy, NULL);
 	NR::Point const event_win(wx, wy);
@@ -144,13 +144,13 @@ sp_dt_ruler_event (GtkWidget *widget, GdkEvent *event, SPDesktopWidget *dtw, gbo
 			       : wx )
 			     >= 0 )
 			{
-				SPRepr *repr = sp_repr_new("sodipodi:guide");
+				Inkscape::XML::Node *repr = sp_repr_new("sodipodi:guide");
 				sp_repr_set_attr (repr, "orientation", (horiz) ? "horizontal" : "vertical");
 				double const guide_pos_dt = event_dt[ horiz
 								      ? NR::Y
 								      : NR::X ];
 				sp_repr_set_double(repr, "position", guide_pos_dt);
-				sp_repr_append_child (SP_OBJECT_REPR (desktop->namedview), repr);
+				SP_OBJECT_REPR (desktop->namedview)->appendChild(repr);
 				sp_repr_unref (repr);
 				sp_document_done (SP_DT_DOCUMENT (desktop));
 			}
@@ -359,7 +359,7 @@ guide_dialog_response (GtkDialog *dialog, gint response, gpointer data)
 static void
 sp_dt_simple_guide_dialog (SPGuide *guide, SPDesktop *desktop)
 {
-	GtkWidget * pix, * b1, * b2, * b3, * b4,* but;
+	GtkWidget * b1, * b2, * b3, * b4,* but;
 
 	if (!GTK_IS_WIDGET (d)) {
 		GtkObject *a;
@@ -386,10 +386,6 @@ sp_dt_simple_guide_dialog (SPGuide *guide, SPDesktop *desktop)
 		gtk_box_pack_end (GTK_BOX (b1), b2, TRUE, TRUE, 0);
 		gtk_widget_show (b2);
     
-		//pixmap
-		pix = sp_icon_new (32, "guide_dialog");
-		gtk_box_pack_start (GTK_BOX (b1), pix, TRUE, TRUE, 0);
-		gtk_widget_show (pix);
 		//labels
 		b3 = gtk_hbox_new (FALSE,4);
 		gtk_box_pack_start (GTK_BOX (b2), b3, TRUE, TRUE, 0);
@@ -422,7 +418,7 @@ sp_dt_simple_guide_dialog (SPGuide *guide, SPDesktop *desktop)
 
 		// unitmenu
 		/* fixme: We should allow percents here too, as percents of the canvas size */
-		u = sp_unit_selector_new (SP_UNIT_ABSOLUTE);
+		u = sp_unit_selector_new (SP_UNIT_ABSOLUTE | SP_UNIT_DEVICE);
 		sp_unit_selector_set_unit (SP_UNIT_SELECTOR(u), sp_desktop_get_default_unit (desktop));
     
 		// spinbutton
