@@ -3,9 +3,10 @@
 /*
  * Editable view and widget implementation
  *
- * Author:
+ * Authors:
  *   Lauris Kaplinski <lauris@kaplinski.com>
  *   MenTaLguY <mental@rydia.net>
+ *   bulia byak <buliabyak@users.sf.net>
  *
  * Copyright (C) 2004 MenTaLguY
  * Copyright (C) 1999-2002 Lauris Kaplinski
@@ -14,7 +15,9 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
 
 #include <math.h>
 #include <string.h>
@@ -37,7 +40,7 @@
 #include "display/sodipodi-ctrlrect.h"
 #include "display/sp-canvas-util.h"
 #include "helper/units.h"
-#include "libnr/nr-matrix-ops.h"
+#include "libnr/nr-point-matrix-ops.h"
 #include "widgets/button.h"
 #include "widgets/ruler.h"
 #include "widgets/icon.h"
@@ -55,7 +58,7 @@
 #include "document.h"
 #include "selection.h"
 #include "select-context.h"
-#include "sp-desktop-widget.h"
+#include "desktop-widget.h"
 #include "sp-namedview.h"
 #include "sp-text.h"
 #include "sp-tspan.h"
@@ -70,7 +73,7 @@
 #include "message-stack.h"
 #include "message-context.h"
 #include "object-hierarchy.h"
-#include "xml/sp-repr-event-vector.h"
+#include "xml/node-event-vector.h"
 
 #include "file.h"
 
@@ -98,7 +101,7 @@ static void sp_desktop_document_resized (SPView *view, SPDocument *doc, gdouble
 static SPView *sp_desktop_new (SPNamedView *nv, SPCanvas *canvas);
 
 static void sp_dt_namedview_modified (SPNamedView *nv, guint flags, SPDesktop *desktop);
-static void sp_desktop_selection_modified (SPSelection *selection, guint flags, SPDesktop *desktop);
+static void sp_desktop_selection_modified (Inkscape::Selection *selection, guint flags, SPDesktop *desktop);
 
 static void sp_dt_update_snap_distances (SPDesktop *desktop);
 
@@ -190,6 +193,7 @@ sp_desktop_class_init (SPDesktopClass *klass)
 static void
 sp_desktop_init (SPDesktop *desktop)
 {
+    desktop->_dlg_mgr = NULL;
     desktop->namedview = NULL;
     desktop->selection = NULL;
     desktop->acetate = NULL;
@@ -211,12 +215,15 @@ sp_desktop_init (SPDesktop *desktop)
 
     desktop->is_fullscreen = FALSE;
 
+    desktop->gr_item = NULL;
+    desktop->gr_point_num = 0;
+    desktop->gr_fill_or_stroke = true;
+
     new (&desktop->sel_modified_connection) sigc::connection();
 
     new (&desktop->sel_changed_connection) sigc::connection();
 
 
-
     new (&desktop->_set_colorcomponent_signal) sigc::signal<bool, ColorComponent, float, bool, bool>();
 
     new (&desktop->_set_style_signal) sigc::signal<bool, const SPCSSAttr *, StopOnTrue>();
@@ -224,8 +231,6 @@ sp_desktop_init (SPDesktop *desktop)
     new (&desktop->_layer_changed_signal) sigc::signal<void, SPObject *>();
 
 
-
-
     desktop->_guides_message_context = new Inkscape::MessageContext(desktop->messageStack());
 
     desktop->current = sp_repr_css_attr_inherited (inkscape_get_repr (INKSCAPE, "desktop"), "style");
@@ -297,6 +302,7 @@ SPObject *SPDesktop::currentLayer() {
 void SPDesktop::setCurrentLayer(SPObject *object) {
     g_return_if_fail(SP_IS_GROUP(object));
     g_return_if_fail( currentRoot() == object || currentRoot()->isAncestorOf(object));
+    // printf("Set Layer to ID: %s\n", SP_OBJECT_ID(object));
     _layer_hierarchy->setBottom(object);
 }
 
@@ -401,6 +407,9 @@ sp_desktop_new (SPNamedView *namedview, SPCanvas *canvas)
     /* Setup widget */
     SPDesktop *desktop = (SPDesktop *) g_object_new (SP_TYPE_DESKTOP, NULL);
 
+    /* Setup Dialog Manager */
+    desktop->_dlg_mgr = new Inkscape::UI::Dialog::DialogManager();
+
     desktop->dkey = sp_item_display_key_new (1);
 
     /* Connect document */
@@ -436,7 +445,7 @@ sp_desktop_new (SPNamedView *namedview, SPCanvas *canvas)
     desktop->sketch = (SPCanvasGroup *) sp_canvas_item_new (desktop->main, SP_TYPE_CANVAS_GROUP, NULL);
     desktop->controls = (SPCanvasGroup *) sp_canvas_item_new (desktop->main, SP_TYPE_CANVAS_GROUP, NULL);
 
-    desktop->selection = new SPSelection (desktop);
+    desktop->selection = new Inkscape::Selection (desktop);
 
     /* Push select tool to the bottom of stack */
     // FIXME: this is the only call to this.  Everything else seems to just
@@ -454,9 +463,12 @@ sp_desktop_new (SPNamedView *namedview, SPCanvas *canvas)
        0 is used, then the constructor for a shadow is not initialized.
     */
 
-    if ( desktop->namedview->pageshadow != 0 ) {
-        sp_ctrlrect_set_shadow (SP_CTRLRECT (desktop->page_border), desktop->namedview->pageshadow, 0x3f3f3fff);
+    if ( desktop->namedview->pageshadow != 0 && 
+         desktop->namedview->showpageshadow ) {
+        sp_ctrlrect_set_shadow (SP_CTRLRECT (desktop->page_border), 
+                                desktop->namedview->pageshadow, 0x3f3f3fff);
     }
+    
 
     /* Connect event for page resize */
     desktop->doc2dt[5] = sp_document_height (document);
@@ -491,6 +503,14 @@ sp_desktop_new (SPNamedView *namedview, SPCanvas *canvas)
     /* Ugly hack */
     sp_dt_namedview_modified (desktop->namedview, SP_OBJECT_MODIFIED_FLAG, desktop);
 
+    /* Set up notification of rebuilding the document, this allows
+       for saving object related settings in the document. */
+    desktop->_reconstruction_start_connection =
+        document->connectReconstructionStart(sigc::bind(sigc::ptr_fun(&SPDesktop::_reconstruction_start), desktop));
+    desktop->_reconstruction_finish_connection =
+        document->connectReconstructionFinish(sigc::bind(sigc::ptr_fun(&SPDesktop::_reconstruction_finish), desktop));
+    desktop->_reconstruction_old_layer_id = NULL;
+
     // ?
     // sp_active_desktop_set (desktop);
     inkscape_add_desktop (desktop);
@@ -510,7 +530,8 @@ sp_dt_namedview_modified (SPNamedView *nv, guint flags, SPDesktop *desktop)
         /* Show/hide page background */
         if (nv->pagecolor & 0xff) {
             sp_canvas_item_show (desktop->table);
-            sp_ctrlrect_set_color ((SPCtrlRect *) desktop->table, 0x00000000, TRUE, nv->pagecolor);
+            sp_ctrlrect_set_color ((SPCtrlRect *) desktop->table, 0x00000000, 
+                TRUE, nv->pagecolor);
             sp_canvas_item_move_to_z (desktop->table, 0);
         } else {
             sp_canvas_item_hide (desktop->table);
@@ -521,22 +542,41 @@ sp_dt_namedview_modified (SPNamedView *nv, guint flags, SPDesktop *desktop)
             // show
             sp_canvas_item_show (desktop->page_border);
             // set color and shadow
-            sp_ctrlrect_set_color ((SPCtrlRect *) desktop->page_border, nv->bordercolor, FALSE, 0x00000000);
+            sp_ctrlrect_set_color ((SPCtrlRect *) desktop->page_border, 
+			    nv->bordercolor, FALSE, 0x00000000);
             if (nv->pageshadow)
-                sp_ctrlrect_set_shadow ((SPCtrlRect *)desktop->page_border, nv->pageshadow, nv->bordercolor);
+                sp_ctrlrect_set_shadow ((SPCtrlRect *)desktop->page_border, 
+				nv->pageshadow, nv->bordercolor);
             // place in the z-order stack
             if (nv->borderlayer == SP_BORDER_LAYER_BOTTOM) {
                  sp_canvas_item_move_to_z (desktop->page_border, 2);
             } else {
                 int order = sp_canvas_item_order (desktop->page_border);
                 int morder = sp_canvas_item_order (desktop->drawing);
-                if (morder > order) sp_canvas_item_raise (desktop->page_border, morder - order);
+                if (morder > order) sp_canvas_item_raise (desktop->page_border,
+				    morder - order);
             }
         } else {
                 sp_canvas_item_hide (desktop->page_border);
                 if (nv->pageshadow)
-                    sp_ctrlrect_set_shadow ((SPCtrlRect *)desktop->page, 0, 0x00000000);
+                    sp_ctrlrect_set_shadow ((SPCtrlRect *)desktop->page, 0, 
+				    0x00000000);
         }
+	
+        /* Show/hide page shadow */
+        if (nv->showpageshadow && nv->pageshadow) {
+            // show
+            sp_ctrlrect_set_shadow ((SPCtrlRect *)desktop->page_border, 
+                            nv->pageshadow, nv->bordercolor);
+        } else {
+            // hide
+            sp_ctrlrect_set_shadow ((SPCtrlRect *)desktop->page_border, 0, 
+                            0x00000000);
+        }
+
+        
+        
+        
     }
 }
 
@@ -608,7 +648,6 @@ sp_desktop_change_document (SPDesktop *desktop, SPDocument *document)
     g_return_if_fail (desktop != NULL);
     g_return_if_fail (SP_IS_DESKTOP (desktop));
     g_return_if_fail (document != NULL);
-    g_return_if_fail (SP_IS_DOCUMENT (document));
 
     /* unselect everything before switching documents */
     SP_DT_SELECTION (desktop)->clear();
@@ -619,7 +658,7 @@ sp_desktop_change_document (SPDesktop *desktop, SPDocument *document)
 /* Private methods */
 
 static void
-sp_desktop_selection_modified (SPSelection *selection, guint flags, SPDesktop *desktop)
+sp_desktop_selection_modified (Inkscape::Selection *selection, guint flags, SPDesktop *desktop)
 {
     sp_desktop_update_scrollbars (desktop);
 }
@@ -640,7 +679,7 @@ sp_desktop_set_event_context (SPDesktop *dt, GtkType type, const gchar *config)
         g_object_unref (G_OBJECT (ec));
     }
 
-    SPRepr *repr = (config) ? inkscape_get_repr (INKSCAPE, config) : NULL;
+    Inkscape::XML::Node *repr = (config) ? inkscape_get_repr (INKSCAPE, config) : NULL;
     ec = sp_event_context_new (type, dt, repr, SP_EVENT_CONTEXT_STATIC);
     ec->next = dt->event_context;
     dt->event_context = ec;
@@ -652,7 +691,7 @@ void
 sp_desktop_push_event_context (SPDesktop *dt, GtkType type, const gchar *config, unsigned int key)
 {
     SPEventContext *ref, *ec;
-    SPRepr *repr;
+    Inkscape::XML::Node *repr;
 
     if (dt->event_context && dt->event_context->key == key) return;
     ref = dt->event_context;
@@ -901,7 +940,7 @@ sp_desktop_widget_init (SPDesktopWidget *dtw)
     dtw->hruler = sp_hruler_new ();
     dtw->hruler_box = eventbox;
     sp_ruler_set_metric (GTK_RULER (dtw->hruler), SP_PT);
-    gtk_tooltips_set_tip (dtw->tt, dtw->hruler_box, sp_unit_get_plural (&sp_unit_get_by_id(SP_UNIT_PT)), NULL);
+    gtk_tooltips_set_tip (dtw->tt, dtw->hruler_box, gettext(sp_unit_get_plural (&sp_unit_get_by_id(SP_UNIT_PT))), NULL);
     gtk_container_add (GTK_CONTAINER (eventbox), dtw->hruler);
     gtk_table_attach (GTK_TABLE (tbl), eventbox, 1, 2, 0, 1, (GtkAttachOptions)(GTK_FILL), (GtkAttachOptions)(GTK_FILL), widget->style->xthickness, 0);
     g_signal_connect (G_OBJECT (eventbox), "button_press_event", G_CALLBACK (sp_dt_hruler_event), dtw);
@@ -913,7 +952,7 @@ sp_desktop_widget_init (SPDesktopWidget *dtw)
     dtw->vruler = sp_vruler_new ();
     dtw->vruler_box = eventbox;
     sp_ruler_set_metric (GTK_RULER (dtw->vruler), SP_PT);
-    gtk_tooltips_set_tip (dtw->tt, dtw->vruler_box, sp_unit_get_plural (&sp_unit_get_by_id(SP_UNIT_PT)), NULL);
+    gtk_tooltips_set_tip (dtw->tt, dtw->vruler_box, gettext(sp_unit_get_plural (&sp_unit_get_by_id(SP_UNIT_PT))), NULL);
     gtk_container_add (GTK_CONTAINER (eventbox), GTK_WIDGET (dtw->vruler));
     gtk_table_attach (GTK_TABLE (tbl), eventbox, 0, 1, 1, 2, (GtkAttachOptions)(GTK_FILL), (GtkAttachOptions)(GTK_FILL), 0, widget->style->ythickness);
     g_signal_connect (G_OBJECT (eventbox), "button_press_event", G_CALLBACK (sp_dt_vruler_event), dtw);
@@ -926,7 +965,7 @@ sp_desktop_widget_init (SPDesktopWidget *dtw)
     gtk_table_attach (GTK_TABLE (tbl), dtw->hscrollbar, 1, 2, 2, 3, (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)(GTK_FILL), 0, 0);
     /* Vertical scrollbar and the sticky zoom button */
     dtw->vscrollbar_box = gtk_vbox_new (FALSE, 0);
-    dtw->sticky_zoom = sp_button_new_from_data (10,
+    dtw->sticky_zoom = sp_button_new_from_data ( GTK_ICON_SIZE_MENU,
                                                  SP_BUTTON_TYPE_TOGGLE,
                                                  NULL,
                                                  "sticky_zoom",
@@ -953,7 +992,7 @@ sp_desktop_widget_init (SPDesktopWidget *dtw)
     // zoom status spinbutton
     dtw->zoom_status = gtk_spin_button_new_with_range (log(SP_DESKTOP_ZOOM_MIN)/log(2), log(SP_DESKTOP_ZOOM_MAX)/log(2), 0.1);
     gtk_tooltips_set_tip (dtw->tt, dtw->zoom_status, _("Zoom"), NULL);
-    gtk_widget_set_usize (dtw->zoom_status, STATUS_ZOOM_WIDTH, -1);
+    gtk_widget_set_size_request (dtw->zoom_status, STATUS_ZOOM_WIDTH, -1);
     gtk_entry_set_width_chars (GTK_ENTRY (dtw->zoom_status), 6);
     gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (dtw->zoom_status), FALSE);
     gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (dtw->zoom_status), GTK_UPDATE_ALWAYS);
@@ -977,7 +1016,7 @@ sp_desktop_widget_init (SPDesktopWidget *dtw)
     eventbox = gtk_event_box_new ();
     gtk_container_add (GTK_CONTAINER (eventbox), dtw->coord_status);
     gtk_tooltips_set_tip (dtw->tt, eventbox, _("Cursor coordinates"), NULL);
-    gtk_widget_set_usize (dtw->coord_status, STATUS_COORD_WIDTH, -1);
+    gtk_widget_set_size_request (dtw->coord_status, STATUS_COORD_WIDTH, -1);
     sp_set_font_size (dtw->coord_status, STATUS_COORD_FONT_SIZE);
     gtk_box_pack_start (GTK_BOX (dtw->statusbar), eventbox, FALSE, FALSE, 1);
 
@@ -1036,9 +1075,9 @@ sp_desktop_widget_set_title (SPDesktopWidget *dtw)
                                : g_basename(uri) );
         GString *name = g_string_new ("");
         if (dtw->desktop->number > 1) {
-            g_string_sprintf (name, _("%s: %d - Inkscape"), fname, dtw->desktop->number);
+            g_string_printf (name, _("%s: %d - Inkscape"), fname, dtw->desktop->number);
         } else {
-            g_string_sprintf (name, _("%s - Inkscape"), fname);
+            g_string_printf (name, _("%s - Inkscape"), fname);
         }
         gtk_window_set_title (window, name->str);
         g_string_free (name, TRUE);
@@ -1168,10 +1207,11 @@ sp_dtw_desktop_deactivate (SPDesktop *desktop, SPDesktopWidget *dtw)
 static gboolean
 sp_dtw_desktop_shutdown (SPView *view, SPDesktopWidget *dtw)
 {
-    SPDocument *doc = SP_VIEW_DOCUMENT (view);
-
-    if (doc && (((GObject *) doc)->ref_count == 1)) {
-        if (sp_repr_attr (sp_document_repr_root (doc), "sodipodi:modified") != NULL) {
+    SPDesktop *desktop=SP_DESKTOP(view);
+    g_assert(desktop != NULL);
+    if (inkscape_is_sole_desktop_for_document(*desktop)) {
+        SPDocument *doc=SP_VIEW_DOCUMENT (view);
+        if (sp_document_repr_root(doc)->attribute("sodipodi:modified") != NULL) {
             GtkWidget *dialog;
 
             dialog = gtk_message_dialog_new(
@@ -1228,7 +1268,7 @@ sp_dtw_desktop_shutdown (SPView *view, SPDesktopWidget *dtw)
         }
         /* Code to check data loss */
         bool allow_data_loss = FALSE;
-        while (sp_repr_attr (sp_document_repr_root (doc), "inkscape:dataloss") != NULL && allow_data_loss == FALSE) {
+        while (sp_document_repr_root(doc)->attribute("inkscape:dataloss") != NULL && allow_data_loss == FALSE) {
             GtkWidget *dialog;
 
             dialog = gtk_message_dialog_new(
@@ -1244,7 +1284,7 @@ sp_dtw_desktop_shutdown (SPView *view, SPDesktopWidget *dtw)
                 _("<span weight=\"bold\" size=\"larger\">The file \"%s\" was saved with a format (%s) that may cause data loss!</span>\n\n"
                   "Do you want to save this file in another format?"),
                 SP_DOCUMENT_NAME(doc),
-                Inkscape::Extension::db.get(sp_repr_attr(sp_document_repr_root(doc), "inkscape:output_extension"))->get_name());
+                Inkscape::Extension::db.get(sp_document_repr_root(doc)->attribute("inkscape:output_extension"))->get_name());
 
             /* FIXME !!! Gtk 2.3+ gives us gtk_message_dialog_set_markup() (and actually even
                gtk_message_dialog_new_with_markup(..., format, ...)!) -- until then, we will have
@@ -1453,7 +1493,7 @@ void SPDesktop::_layer_hierarchy_changed(SPObject *top, SPObject *bottom,
     desktop->_layer_changed_signal.emit(bottom);
 }
 
-void SPDesktop::_selection_changed(SPSelection *selection, SPDesktop *desktop)
+void SPDesktop::_selection_changed(Inkscape::Selection *selection, SPDesktop *desktop)
 {
     // TODO - only change the layer for single selections, or what?
     // This seems reasonable -- for multiple selections there can be many
@@ -1467,6 +1507,46 @@ void SPDesktop::_selection_changed(SPSelection *selection, SPDesktop *desktop)
     }
 }
 
+void
+SPDesktop::_reconstruction_start (SPDesktop * desktop)
+{
+    // printf("Desktop, starting reconstruction\n");
+    desktop->_reconstruction_old_layer_id = g_strdup(SP_OBJECT_ID(desktop->currentLayer()));
+    desktop->_layer_hierarchy->setBottom(desktop->currentRoot());
+
+    /*
+    GSList const * selection_objs = desktop->selection->list();
+    for (; selection_objs != NULL; selection_objs = selection_objs->next) {
+
+    }
+    */
+    desktop->selection->clear();
+
+    // printf("Desktop, starting reconstruction end\n");
+}
+
+void
+SPDesktop::_reconstruction_finish (SPDesktop * desktop)
+{
+    // printf("Desktop, finishing reconstruction\n");
+    if (desktop->_reconstruction_old_layer_id == NULL)
+        return;
+
+    SPObject * newLayer = SP_OBJECT_DOCUMENT(desktop->namedview)->getObjectById(desktop->_reconstruction_old_layer_id);
+    if (newLayer != NULL)
+        desktop->setCurrentLayer(newLayer);
+
+    g_free(desktop->_reconstruction_old_layer_id);
+    desktop->_reconstruction_old_layer_id = NULL;
+    // printf("Desktop, finishing reconstruction end\n");
+    return;
+}
+
+void SPDesktop::emitToolSubselectionChanged(gpointer data) {
+	_tool_subselection_changed.emit(data);
+	inkscape_subselection_changed (this);
+}
+
 void SPDesktopWidget::setMessage(Inkscape::MessageType type, const gchar *message)
 {
     GtkLabel *sb=GTK_LABEL(this->select_status);
@@ -1484,8 +1564,8 @@ sp_desktop_widget_namedview_modified (SPNamedView *nv, guint flags, SPDesktopWid
         sp_ruler_set_metric (GTK_RULER (dtw->vruler), sp_desktop_get_default_metric(dtw->desktop));
         sp_ruler_set_metric (GTK_RULER (dtw->hruler), sp_desktop_get_default_metric(dtw->desktop));
 
-        gtk_tooltips_set_tip (dtw->tt, dtw->hruler_box, sp_unit_get_plural (sp_desktop_get_default_unit(dtw->desktop)), NULL);
-        gtk_tooltips_set_tip (dtw->tt, dtw->vruler_box, sp_unit_get_plural (sp_desktop_get_default_unit(dtw->desktop)), NULL);
+        gtk_tooltips_set_tip (dtw->tt, dtw->hruler_box, gettext(sp_unit_get_plural (sp_desktop_get_default_unit(dtw->desktop))), NULL);
+        gtk_tooltips_set_tip (dtw->tt, dtw->vruler_box, gettext(sp_unit_get_plural (sp_desktop_get_default_unit(dtw->desktop))), NULL);
 
         sp_desktop_widget_update_rulers (dtw);
     }
