@@ -5,7 +5,10 @@
  *
  * Authors:
  *   Lauris Kaplinski <lauris@kaplinski.com>
- *
+ *   Jon Phillips <jon@rejon.org>
+ *   bulia byak <buliabyak@users.sf.net>
+ * 
+ * Copyright (C) Jon Phillips 2005
  * Copyright (C) Lauris Kaplinski 2000-2002
  *
  * Released under GNU GPL, read the file 'COPYING' for more information
@@ -28,9 +31,7 @@
 #include "helper/units.h"
 #include "helper/window.h"
 #include "svg/svg.h"
-#include "widgets/sp-color-selector.h"
-#include "widgets/sp-color-notebook.h"
-#include "widgets/sp-color-preview.h"
+#include "color-picker.h"
 #include "../inkscape.h"
 #include "../document.h"
 #include "../desktop.h"
@@ -45,7 +46,7 @@
 #include "../interface.h"
 
 #include "../xml/repr.h"
-#include "../xml/sp-repr-event-vector.h"
+#include "../xml/node-event-vector.h"
 
 #include "desktop-properties.h"
 #include "svg/stringstream.h"
@@ -64,17 +65,6 @@ static void sp_dtw_deactivate_desktop(Inkscape::Application *inkscape,
 static void sp_dtw_update(GtkWidget *dialog,
                           SPDesktop *desktop);
 
-static GtkWidget *sp_color_picker_new(gchar *colorkey, gchar *alphakey,
-                                      gchar *title, guint32 rgba);
-
-static void sp_color_picker_set_rgba32(GtkWidget *cp, guint32 rgba);
-static void sp_color_picker_clicked(GObject *cp, void *data);
-
-static void sp_color_picker_button(GtkWidget *dialog, GtkWidget *t,
-                                   gchar const *label, gchar *key,
-                                   gchar *color_dialog_label,
-                                   gchar *opacity_key, int row);
-
 static GtkWidget *dlg = NULL;
 static GtkTooltips *tooltips = NULL;
 static win_data wd;
@@ -202,14 +192,18 @@ static PaperSize const inkscape_papers[] = {
 };
 
 static void
-docoptions_event_attr_changed(SPRepr *, gchar const *, gchar const *, gchar const *, bool, gpointer)
+docoptions_event_attr_changed(Inkscape::XML::Node *, gchar const *, gchar const *, gchar const *, bool, gpointer)
 {
+    if (g_object_get_data(G_OBJECT(dlg), "update")) {
+        return;
+    }
+
     if (dlg) {
         sp_dtw_update(dlg, SP_ACTIVE_DESKTOP);
     }
 }
 
-static SPReprEventVector docoptions_repr_events = {
+static Inkscape::XML::NodeEventVector docoptions_repr_events = {
     NULL, /* child_added */
     NULL, /* child_removed */
     docoptions_event_attr_changed,
@@ -257,7 +251,7 @@ sp_dtw_whatever_toggled(GtkToggleButton *tb, GtkWidget *dialog)
 
     SPDocument *doc = SP_DT_DOCUMENT(dt);
 
-    SPRepr *repr = SP_OBJECT_REPR(dt->namedview);
+    Inkscape::XML::Node *repr = SP_OBJECT_REPR(dt->namedview);
     gchar const *key = (gchar const *)gtk_object_get_data(GTK_OBJECT(tb), "key");
 
     gtk_object_set_data(GTK_OBJECT(dlg), "update", GINT_TO_POINTER(TRUE));
@@ -287,7 +281,7 @@ sp_dtw_border_layer_toggled(GtkToggleButton *tb, GtkWidget *dialog)
 
     SPDocument *doc = SP_DT_DOCUMENT(dt);
 
-    SPRepr *repr = SP_OBJECT_REPR(dt->namedview);
+    Inkscape::XML::Node *repr = SP_OBJECT_REPR(dt->namedview);
 
     gboolean saved = sp_document_get_undo_sensitive(doc);
     sp_document_set_undo_sensitive(doc, FALSE);
@@ -299,6 +293,7 @@ sp_dtw_border_layer_toggled(GtkToggleButton *tb, GtkWidget *dialog)
 }
 
 
+
 /**
  * \brief  Writes the change into the corresponding attribute of the
  *         sodipodi:namedview element.
@@ -317,7 +312,7 @@ sp_dtw_whatever_changed(GtkAdjustment *adjustment, GtkWidget *dialog)
     }
     SPDocument *doc = SP_DT_DOCUMENT(dt);
 
-    SPRepr *repr = SP_OBJECT_REPR(dt->namedview);
+    Inkscape::XML::Node *repr = SP_OBJECT_REPR(dt->namedview);
     gchar const *key = (gchar const *)gtk_object_get_data(GTK_OBJECT(adjustment), "key");
     SPUnitSelector *us = (SPUnitSelector *) gtk_object_get_data(GTK_OBJECT(adjustment),
                                                                 "unit_selector");
@@ -419,7 +414,7 @@ sp_doc_dialog_whatever_changed(GtkAdjustment *adjustment, GtkWidget *dialog)
 
     SPDocument *doc = SP_DT_DOCUMENT(dt);
 
-    SPRepr *repr = sp_document_repr_root(doc);
+    Inkscape::XML::Node *repr = sp_document_repr_root(doc);
     gchar const *key = (gchar const *) gtk_object_get_data(GTK_OBJECT(adjustment), "key");
     SPUnitSelector *us = (SPUnitSelector *) gtk_object_get_data(GTK_OBJECT(adjustment),
                                                                 "unit_selector");
@@ -447,23 +442,6 @@ sp_doc_dialog_whatever_changed(GtkAdjustment *adjustment, GtkWidget *dialog)
 
     sp_repr_set_attr(repr, key, os.str().c_str());
 
-    /* Save this for later
-    if (!strcmp(key, "width") || !strcmp(key, "height")) {
-
-        //A short-term hack to set the viewBox to be 1.25 x the size
-        // of the page.
-        gdouble vbWidth  = (gdouble)g_ascii_strtod(sp_repr_attr(repr, "width"), NULL)  * 1.25;
-        gdouble vbHeight = (gdouble)g_ascii_strtod(sp_repr_attr(repr, "height"), NULL) * 1.25;
-        std::ostringstream os;
-        os.imbue(std::locale::classic());
-        os.setf(std::ios::showpoint);
-        os.precision(8);
-        os << "0 0 " << vbWidth << " " << vbHeight;
-        gchar const *strVal = (gchar const *)os.str().c_str();
-        sp_repr_set_attr(repr, "viewBox", g_strdup(strVal));
-
-    }
-    */
     sp_document_done(doc);
 }
 
@@ -480,7 +458,7 @@ sp_dtw_grid_snap_distance_changed(GtkAdjustment *adjustment,
         return;
     }
 
-    SPRepr *repr = SP_OBJECT_REPR(dt->namedview);
+    Inkscape::XML::Node *repr = SP_OBJECT_REPR(dt->namedview);
 
     SPUnitSelector *us = (SPUnitSelector *) gtk_object_get_data(GTK_OBJECT(dialog),
                                                                 "grid_snap_units");
@@ -505,7 +483,7 @@ sp_dtw_grid_emp_spacing_changed (GtkAdjustment *adjustment,
         return;
     }
 
-    SPRepr *repr = SP_OBJECT_REPR(dt->namedview);
+    Inkscape::XML::Node *repr = SP_OBJECT_REPR(dt->namedview);
 
     Inkscape::SVGOStringStream os;
     int value = int(adjustment->value);
@@ -529,7 +507,7 @@ sp_dtw_guides_snap_distance_changed(GtkAdjustment *adjustment,
         return;
     }
 
-    SPRepr *repr = SP_OBJECT_REPR(dt->namedview);
+    Inkscape::XML::Node *repr = SP_OBJECT_REPR(dt->namedview);
 
     SPUnitSelector *us = (SPUnitSelector *) gtk_object_get_data(GTK_OBJECT(dialog),
                                                                 "guide_snap_units");
@@ -829,7 +807,7 @@ static gboolean set_doc_units (SPUnitSelector *,
 
     SPDocument *doc = SP_DT_DOCUMENT(dt);
 
-    SPRepr *repr = SP_OBJECT_REPR(dt->namedview);
+    Inkscape::XML::Node *repr = SP_OBJECT_REPR(dt->namedview);
 
     gboolean saved = sp_document_get_undo_sensitive(doc);
     sp_document_set_undo_sensitive(doc, FALSE);
@@ -910,9 +888,9 @@ sp_desktop_dialog(void)
 
         /* Checkbuttons */
         // FIXME: gray out snapping when grid is off
-        spw_vbox_checkbutton(dlg, v, _("Show grid"), "showgrid", cb);
-        spw_vbox_checkbutton(dlg, v, _("Snap bounding boxes to grid"), "inkscape:grid-bbox", cb);
-        spw_vbox_checkbutton(dlg, v, _("Snap points to grid"), "inkscape:grid-points", cb);
+        spw_vbox_checkbutton(dlg, v, _("Show grid"), _("Show or hide grid"), "showgrid", cb);
+        spw_vbox_checkbutton(dlg, v, _("Snap bounding boxes to grid"), _("Snap the edges of the object bounding boxes"), "inkscape:grid-bbox", cb);
+        spw_vbox_checkbutton(dlg, v, _("Snap points to grid"), _("Snap path nodes, text baselines, ellipse centers, etc."), "inkscape:grid-points", cb);
 
         /*   Commenting out until Nathan implements the grids -- bryce
          *   spw_checkbutton(dlg, t, _("Iso grid"), "isogrid", 0, row, 0, cb);
@@ -951,14 +929,14 @@ sp_desktop_dialog(void)
         spw_unit_selector(dlg, t, _("Snap distance:"), "gridtolerance", row++, us,
                           G_CALLBACK(sp_dtw_grid_snap_distance_changed));
 
-        sp_color_picker_button(dlg, t, _("Minor grid line color:"), "gridcolor",
-                               _("Grid color"), "gridopacity", row++);
+        sp_color_picker_button(NULL, true, dlg, t, _("Grid line color:"), "gridcolor",
+                               _("Grid line color"), _("Color of grid lines"), "gridopacity", row++);
 
-        sp_color_picker_button(dlg, t, _("Major grid line color:"), "gridempcolor",
-                               _("Grid emphasis color"), "gridempopacity", row++);
+        sp_color_picker_button(NULL, true, dlg, t, _("Major grid line color:"), "gridempcolor",
+                               _("Major grid line color"), _("Color of the major (highlighted) grid lines"), "gridempopacity", row++);
 
         if (1) {
-            spw_label(t, _("Major grid line spacing:"), 0, row);
+            spw_label(t, _("Major grid line every:"), 0, row);
             GtkObject * a = gtk_adjustment_new (0.0, 0.0, 25.0, 1.0, 1.0, 1.0);
             gtk_object_set_data(GTK_OBJECT(a), (const gchar *)"key", (gpointer)"gridempspacing");
             gtk_object_set_data(GTK_OBJECT(dlg), "gridempspacing", a);
@@ -968,7 +946,7 @@ sp_desktop_dialog(void)
             GtkWidget * sb = gtk_spin_button_new (GTK_ADJUSTMENT(a), 1.0, 0);
             gtk_widget_show(sb);
 
-            // TRANSLATORS: This belongs to the "Major grid line spacing:" string,
+            // TRANSLATORS: This belongs to the "Major grid line every:" string,
             //  see grid settings in the "Document Preferences" dialog
             GtkWidget * label = gtk_label_new(_("lines"));
             gtk_widget_show(label);
@@ -993,9 +971,9 @@ sp_desktop_dialog(void)
         /* Checkbuttons */
         // FIXME: gray out snapping when guides are off
         cb = G_CALLBACK(sp_dtw_whatever_toggled);
-        spw_vbox_checkbutton(dlg, v, _("Show guides"), "showguides", cb);
-        spw_vbox_checkbutton(dlg, v, _("Snap bounding boxes to guides"), "inkscape:guide-bbox", cb);
-        spw_vbox_checkbutton(dlg, v, _("Snap points to guides"), "inkscape:guide-points", cb);
+        spw_vbox_checkbutton(dlg, v, _("Show guides"), _("Show or hide guides"), "showguides", cb);
+        spw_vbox_checkbutton(dlg, v, _("Snap bounding boxes to guides"),  _("Snap the edges of the object bounding boxes"), "inkscape:guide-bbox", cb);
+        spw_vbox_checkbutton(dlg, v, _("Snap points to guides"), _("Snap path nodes, text baselines, ellipse centers, etc."), "inkscape:guide-points", cb);
 
         t = gtk_table_new(4, 2, FALSE);
         gtk_widget_show(t);
@@ -1013,11 +991,11 @@ sp_desktop_dialog(void)
                           row++, us,
                           G_CALLBACK(sp_dtw_guides_snap_distance_changed));
 
-        sp_color_picker_button(dlg, t, _("Guide color:"), "guidecolor",
-                               _("Guideline color"), "guideopacity", row++);
+        sp_color_picker_button(NULL, true, dlg, t, _("Guide color:"), "guidecolor",
+                               _("Guideline color"), _("Color of guidelines"), "guideopacity", row++);
 
-        sp_color_picker_button(dlg, t, _("Highlight color:"), "guidehicolor",
-                               _("Highlighted guideline color"),
+        sp_color_picker_button(NULL, true, dlg, t, _("Highlight color:"), "guidehicolor",
+                               _("Highlighted guideline color"), _("Color of a guideline when it is under mouse"),
                                "guidehiopacity", row++);
 
         row=0;
@@ -1032,8 +1010,9 @@ sp_desktop_dialog(void)
         gtk_notebook_prepend_page(GTK_NOTEBOOK(nb), t, l);
         gtk_notebook_set_current_page(GTK_NOTEBOOK(nb), 0);
 
-        sp_color_picker_button(dlg, t, _("Background (also for export):"),
-                               "pagecolor", _("Background color"),
+        sp_color_picker_button(NULL, true, dlg, t, _("Background:"),
+                               "pagecolor", _("Background color"), 
+                               _("Color and transparency of the page background (also used for bitmap export)"),
                                "inkscape:pageopacity", 0);
 
         cb = G_CALLBACK(sp_dtw_whatever_toggled);
@@ -1044,28 +1023,35 @@ sp_desktop_dialog(void)
         spw_checkbutton(dlg, t, _("Border on top of drawing"),
                         "borderlayer", 0, 2, 0, cb);
 
-        sp_color_picker_button(dlg, t, _("Border color:"),
+        sp_color_picker_button(NULL, true, dlg, t, _("Border color:"),
                                "bordercolor", _("Canvas border color"),
+                               _("Color of the canvas border"),
                                "borderopacity", 4);
 
+	/* Page Shadow toggle */
+        cb = G_CALLBACK(sp_dtw_whatever_toggled);
+        spw_checkbutton(dlg, t, _("Show page shadow"),
+		                  "inkscape:showpageshadow", 0, 5, 0, cb);
+
+	
         l = gtk_label_new(_("Default units:"));
         gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
         gtk_widget_show(l);
-        gtk_table_attach (GTK_TABLE (t), l, 0, 1, 5, 6,
+        gtk_table_attach (GTK_TABLE (t), l, 0, 1, 6, 7,
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)0, 0, 0);
         GtkWidget *doc_units = sp_unit_selector_new(SP_UNIT_ABSOLUTE | SP_UNIT_DEVICE);
         gtk_tooltips_set_tip(tooltips, doc_units, _("Units for the tool controls, ruler, and the statusbar"), NULL);
         g_signal_connect(G_OBJECT(doc_units), "set_unit", G_CALLBACK(set_doc_units), NULL);
         gtk_object_set_data (GTK_OBJECT (dlg), "doc_units", doc_units);
         gtk_widget_show(doc_units);
-        gtk_table_attach (GTK_TABLE (t), doc_units, 1, 2, 5, 6,
+        gtk_table_attach (GTK_TABLE (t), doc_units, 1, 2, 6, 7,
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)0, 0, 0);
 
         // The following comes from the former "document settings" dialog
 
         GtkWidget *vb = gtk_vbox_new(FALSE, 4);
         gtk_widget_show(vb);
-        gtk_table_attach(GTK_TABLE(t), vb, 0, 2, 6, 7,
+        gtk_table_attach(GTK_TABLE(t), vb, 0, 2, 7, 8,
                          (GtkAttachOptions)( GTK_EXPAND | GTK_FILL ),
                          (GtkAttachOptions)0, 0, 0);
 
@@ -1467,6 +1453,10 @@ sp_dtw_update(GtkWidget *dialog, SPDesktop *desktop)
             gtk_widget_set_sensitive(GTK_WIDGET(w), TRUE);
         }
 
+        o = (GtkObject *)gtk_object_get_data(GTK_OBJECT(dialog), 
+		"inkscape:showpageshadow");
+        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(o), nv->showpageshadow);
+	
         cp = GTK_WIDGET(gtk_object_get_data(GTK_OBJECT(dialog), "pagecolor"));
         sp_color_picker_set_rgba32(cp, nv->pagecolor);
         w = GTK_WIDGET(g_object_get_data(G_OBJECT(cp), "window"));
@@ -1546,232 +1536,6 @@ sp_dtw_update(GtkWidget *dialog, SPDesktop *desktop)
     }
 }
 
-
-static void
-sp_color_picker_button(GtkWidget *dialog, GtkWidget *t,
-                       gchar const *label, gchar *key,
-                       gchar *color_dialog_label,
-                       gchar *opacity_key,
-                       int row)
-{
-    GtkWidget *l = gtk_label_new(label);
-    gtk_misc_set_alignment(GTK_MISC(l), 1.0, 0.5);
-    gtk_widget_show(l);
-    gtk_table_attach(GTK_TABLE(t), l, 0, 1, row, row+1,
-                     (GtkAttachOptions)( GTK_EXPAND | GTK_FILL ),
-                     (GtkAttachOptions)0, 0, 0);
-
-    GtkWidget *cp = sp_color_picker_new(key, opacity_key, color_dialog_label, 0);
-    gtk_widget_show(cp);
-    gtk_table_attach(GTK_TABLE(t), cp, 1, 2, row, row+1,
-                     (GtkAttachOptions)( GTK_EXPAND | GTK_FILL ),
-                     (GtkAttachOptions)0, 0, 0);
-
-    g_object_set_data(G_OBJECT(dialog), key, cp);
-
-} // end of sp_color_picker_button
-
-
-
-static void
-sp_color_picker_destroy(GtkObject *cp, gpointer data)
-{
-    GtkObject *w = (GtkObject *) g_object_get_data(G_OBJECT(cp), "window");
-
-    if (w) {
-        g_assert ( G_IS_OBJECT (w) );
-        gtk_object_destroy(w);
-    }
-
-} // end of sp_color_picker_destroy
-
-
-
-/**
- * \brief  Creates a new color picker for the desktop properties dialog.
- *
- */
-static GtkWidget *
-sp_color_picker_new(gchar *colorkey, gchar *alphakey,
-                    gchar *title, guint32 rgba)
-{
-    GtkWidget *b = gtk_button_new();
-
-    g_object_set_data(G_OBJECT(b), "title", title);
-
-    GtkWidget *cpv = sp_color_preview_new(rgba);
-
-    gtk_widget_show(cpv);
-    gtk_container_add(GTK_CONTAINER(b), cpv);
-    g_object_set_data(G_OBJECT(b), "preview", cpv);
-
-    g_object_set_data(G_OBJECT(b), "colorkey", colorkey);
-    g_object_set_data(G_OBJECT(b), "alphakey", alphakey);
-
-    g_signal_connect(G_OBJECT(b), "destroy",
-                     G_CALLBACK(sp_color_picker_destroy), NULL);
-    g_signal_connect(G_OBJECT(b), "clicked",
-                     G_CALLBACK(sp_color_picker_clicked), NULL);
-
-    return b;
-
-} // end of sp_color_picker_new
-
-
-
-static void
-sp_color_picker_set_rgba32(GtkWidget *cp, guint32 rgba)
-{
-    g_assert ( G_IS_OBJECT (cp) );
-
-    SPColorPreview *cpv = (SPColorPreview *)g_object_get_data(G_OBJECT(cp), "preview");
-    g_assert ( G_IS_OBJECT (cpv) );
-    sp_color_preview_set_rgba32(cpv, rgba);
-
-    SPColorSelector *csel = (SPColorSelector *)g_object_get_data(G_OBJECT(cp), "selector");
-
-    if (csel) {
-        g_assert ( G_IS_OBJECT (csel) );
-
-        SPColor color;
-        sp_color_set_rgb_rgba32(&color, rgba);
-        csel->base->setColorAlpha(color, SP_RGBA32_A_F(rgba));
-    }
-
-    g_object_set_data(G_OBJECT(cp), "color", GUINT_TO_POINTER(rgba));
-
-} // end of sp_color_picker_set_rgba32
-
-
-
-static void
-sp_color_picker_window_destroy(GtkObject *object, GObject *cp)
-{
-    /* remove window object */
-    GtkWidget *w = (GtkWidget*) g_object_get_data(G_OBJECT(cp), "window");
-    if (w) {
-        sp_signal_disconnect_by_data(INKSCAPE, w);
-        gtk_widget_destroy(GTK_WIDGET(w));
-    }
-
-    g_object_set_data(G_OBJECT(cp), "window", NULL);
-    g_object_set_data(G_OBJECT(cp), "selector", NULL);
-
-} // end of sp_color_picker_window_destroy
-
-
-
-static void
-sp_color_picker_color_mod(SPColorSelector *csel, GObject *cp)
-{
-    if (g_object_get_data(G_OBJECT(dlg), "update")) {
-        return;
-    }
-
-    SPColor color;
-    float alpha;
-    csel->base->getColorAlpha(color, &alpha);
-    guint32 rgba = sp_color_get_rgba32_falpha(&color, alpha);
-
-    g_object_set_data(G_OBJECT(cp), "color", GUINT_TO_POINTER(rgba));
-
-    SPColorPreview *cpv = (SPColorPreview *)g_object_get_data(G_OBJECT(cp), "preview");
-    gchar *colorkey = (gchar *)g_object_get_data(G_OBJECT(cp), "colorkey");
-    gchar *alphakey = (gchar *)g_object_get_data(G_OBJECT(cp), "alphakey");
-    sp_color_preview_set_rgba32(cpv, rgba);
-
-    if (!SP_ACTIVE_DESKTOP) {
-        return;
-    }
-
-    SPRepr *repr = SP_OBJECT_REPR(SP_ACTIVE_DESKTOP->namedview);
-
-    gchar c[32];
-    sp_svg_write_color(c, 32, rgba);
-    sp_repr_set_attr(repr, colorkey, c);
-
-    if (alphakey) {
-        sp_repr_set_double(repr, alphakey, (rgba & 0xff) / 255.0);
-    }
-
-    if (!SP_ACTIVE_DESKTOP) {
-        return;
-    }
-
-    sp_document_done(SP_DT_DOCUMENT(SP_ACTIVE_DESKTOP));
-
-} // end of sp_color_picker_color_mod
-
-
-
-static void
-sp_color_picker_window_close(GtkButton *button, GtkWidget *w)
-{
-    gtk_widget_destroy(w);
-}
-
-
-
-static void
-sp_color_picker_clicked(GObject *cp, void *data)
-{
-    GtkWidget *w = (GtkWidget *) g_object_get_data(cp, "window");
-
-    if (!w) {
-        w = sp_window_new(NULL, TRUE);
-        gtk_window_set_title( GTK_WINDOW(w), (gchar *)g_object_get_data(cp, "title"));
-        gtk_container_set_border_width(GTK_CONTAINER(w), 4);
-        g_object_set_data(cp, "window", w);
-
-        gtk_window_set_position(GTK_WINDOW(w), GTK_WIN_POS_CENTER);
-        sp_transientize(w);
-
-        gtk_signal_connect(GTK_OBJECT(w), "event", GTK_SIGNAL_FUNC(sp_dialog_event_handler), w);
-
-        g_signal_connect(G_OBJECT(INKSCAPE), "dialogs_hide", G_CALLBACK(sp_dialog_hide), w);
-        g_signal_connect(G_OBJECT(INKSCAPE), "dialogs_unhide", G_CALLBACK(sp_dialog_unhide), w);
-
-        g_signal_connect(G_OBJECT(w), "destroy", G_CALLBACK(sp_color_picker_window_destroy), cp);
-
-        GtkWidget *vb = gtk_vbox_new(FALSE, 4);
-        gtk_container_add(GTK_CONTAINER(w), vb);
-
-        GtkWidget *csel = sp_color_selector_new(SP_TYPE_COLOR_NOTEBOOK,
-                                                SP_COLORSPACE_TYPE_UNKNOWN);
-        gtk_box_pack_start(GTK_BOX(vb), csel, TRUE, TRUE, 0);
-        guint32 rgba = GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(cp), "color"));
-
-        SPColor color;
-        sp_color_set_rgb_rgba32(&color, rgba);
-        SP_COLOR_SELECTOR(csel)->base->setColorAlpha(color,
-                                                     SP_RGBA32_A_F(rgba));
-        g_signal_connect(G_OBJECT(csel), "dragged",
-                         G_CALLBACK(sp_color_picker_color_mod), cp);
-        g_signal_connect(G_OBJECT(csel), "changed",
-                         G_CALLBACK(sp_color_picker_color_mod), cp);
-
-        g_object_set_data(cp, "selector", csel);
-
-        GtkWidget *hs = gtk_hseparator_new();
-        gtk_box_pack_start(GTK_BOX(vb), hs, FALSE, FALSE, 0);
-
-        GtkWidget *hb = gtk_hbox_new(FALSE, 0);
-        gtk_box_pack_start(GTK_BOX(vb), hb, FALSE, FALSE, 0);
-
-        GtkWidget *b = gtk_button_new_with_label(_("Close"));
-        gtk_box_pack_end(GTK_BOX(hb), b, FALSE, FALSE, 0);
-        g_signal_connect(G_OBJECT(b), "clicked",
-                         G_CALLBACK(sp_color_picker_window_close), w);
-
-        gtk_widget_show_all(w);
-
-    } else {
-        gtk_window_present(GTK_WINDOW(w));
-    }
-
-} // end of sp_color_picker_clicked
-
-
 /*
   Local Variables:
   mode:c++
