@@ -11,7 +11,9 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
 
 #include <glib.h>
 #include <gdk/gdkkeysyms.h>
@@ -26,6 +28,7 @@
 #include "desktop.h"
 #include "inkscape-private.h"
 #include "prefs-utils.h"
+#include "event-context.h"
 
 #include "dialog-events.h"
 
@@ -101,7 +104,7 @@ sp_dialog_event_handler (GtkWindow *win, GdkEvent *event, gpointer data)
     
         case GDK_KEY_PRESS:
         
-            switch (event->key.keyval) {
+            switch (get_group0_keyval (&event->key)) {
                 case GDK_Escape: 
                     sp_dialog_defocus (win);
                     ret = TRUE; 
@@ -149,10 +152,6 @@ sp_dialog_event_handler (GtkWindow *win, GdkEvent *event, gpointer data)
 void
 sp_transientize (GtkWidget *dialog)
 {
-    if (!gtk_window_get_modal(GTK_WINDOW (dialog))) {
-        gtk_window_set_type_hint (GTK_WINDOW (dialog), GDK_WINDOW_TYPE_HINT_UTILITY);
-    }
-
     if (prefs_get_int_attribute ( "options.dialogsskiptaskbar", "value", 0)) {
         gtk_window_set_skip_taskbar_hint (GTK_WINDOW (dialog), TRUE);
     }
@@ -177,11 +176,9 @@ sp_transientize (GtkWidget *dialog)
                                                (G_OBJECT (SP_ACTIVE_DESKTOP),
                                            "window") );
         }
-        
 #endif
-    
+   
     }
-    
 } // end of sp_transientize()
 
 
