@@ -7,14 +7,17 @@
 /*
  * Authors:
  *   Lauris Kaplinski <lauris@kaplinski.com>
+ *   bulia byak <buliabyak@users.sf.net>
  *
- * Copyright (C) 1999-2002 Authors
+ * Copyright (C) 1999-2005 Authors
  * Copyright (C) 2001-2002 Ximian, Inc.
  *
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
 
 #include <iostream>
 #include <math.h>
@@ -129,7 +132,7 @@ static const char * selection_names[SELECTION_NUMBER_OF] = {
 
 /** The names on the buttons for the various selection types. */
 static const char * selection_labels[SELECTION_NUMBER_OF] = {
-    N_("Page"), N_("Drawing"), N_("Selection"), N_("Custom")};
+    N_("_Page"), N_("_Drawing"), N_("_Selection"), N_("_Custom")};
 
 static void
 sp_export_dialog_destroy ( GtkObject *object, gpointer data )
@@ -193,12 +196,7 @@ sp_export_spinbutton_new ( gchar *key, float val, float min, float max,
                            int digits, unsigned int sensitive,
                            GCallback cb, GtkWidget *dlg )
 {
-
-    GtkWidget *l, *sb;
-    GtkObject *a;
-    int pos;
-
-    a = gtk_adjustment_new (val, min, max, step, page, page);
+    GtkObject *a = gtk_adjustment_new (val, min, max, step, page, page);
     gtk_object_set_data (a, "key", key);
     gtk_object_set_data (GTK_OBJECT (dlg), (const gchar *)key, a);
     
@@ -207,11 +205,13 @@ sp_export_spinbutton_new ( gchar *key, float val, float min, float max,
                                           GTK_ADJUSTMENT (a) );
     }
     
-    pos = 0;
+    int pos = 0;
+
+    GtkWidget *l = NULL;
 
     if (ll) {
     
-        l = gtk_label_new ((const gchar *)ll);
+        l = gtk_label_new_with_mnemonic ((const gchar *)ll);
         gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
         gtk_table_attach ( GTK_TABLE (t), l, x + pos, x + pos + 1, y, y + 1, 
                            (GtkAttachOptions)0, (GtkAttachOptions)0, 0, 0 );
@@ -220,22 +220,25 @@ sp_export_spinbutton_new ( gchar *key, float val, float min, float max,
     
     }
 
-    sb = gtk_spin_button_new (GTK_ADJUSTMENT (a), 1.0, digits);
+    GtkWidget *sb = gtk_spin_button_new (GTK_ADJUSTMENT (a), 1.0, digits);
     gtk_table_attach ( GTK_TABLE (t), sb, x + pos, x + pos + 1, y, y + 1, 
                        (GtkAttachOptions)0, (GtkAttachOptions)0, 0, 0 );
-    gtk_widget_set_usize (sb, 64, -1);
+    gtk_widget_set_size_request (sb, 64, -1);
     gtk_widget_set_sensitive (sb, sensitive);
     pos += 1;
 
+    if (ll) { gtk_label_set_mnemonic_widget (GTK_LABEL(l), sb); }
+
     if (lr) {
     
-        l = gtk_label_new ((const gchar *)lr);
+        l = gtk_label_new_with_mnemonic ((const gchar *)lr);
         gtk_misc_set_alignment (GTK_MISC (l), 0.0, 0.5);
         gtk_table_attach ( GTK_TABLE (t), l, x + pos, x + pos + 1, y, y + 1, 
                            (GtkAttachOptions)0, (GtkAttachOptions)0, 0, 0 );
         gtk_widget_set_sensitive (l, sensitive);
         pos += 1;
-    
+
+        gtk_label_set_mnemonic_widget (GTK_LABEL(l), sb);
     }
 
     if (cb) 
@@ -256,6 +259,7 @@ sp_export_dialog_area_frame (GtkWidget * dlg)
 
     /* Units box */
     unitbox = gtk_hbox_new (FALSE, 0);
+    gtk_container_set_border_width (GTK_CONTAINER (unitbox), 4);
     /* gets added to the vbox later, but the unit selector is needed
        earlier than that */
 
@@ -269,11 +273,12 @@ sp_export_dialog_area_frame (GtkWidget * dlg)
     gtk_object_set_data (GTK_OBJECT (dlg), "units", us);
 
     hb = gtk_hbox_new (TRUE, 0);
+    gtk_container_set_border_width (GTK_CONTAINER (hb), 4);
     gtk_box_pack_start(GTK_BOX(vb), hb, FALSE, FALSE, 3);
 
     for (int i = 0; i < SELECTION_NUMBER_OF; i++) {
-        b = gtk_toggle_button_new_with_label (_(selection_labels[i]));
-        gtk_object_set_data (GTK_OBJECT (b), "key", (void *)i);
+        b = gtk_toggle_button_new_with_mnemonic (_(selection_labels[i]));
+        gtk_object_set_data (GTK_OBJECT (b), "key", GINT_TO_POINTER(i));
         gtk_object_set_data (GTK_OBJECT (dlg), selection_names[i], b);
         gtk_box_pack_start (GTK_BOX (hb), b, FALSE, TRUE, 0);
         gtk_signal_connect ( GTK_OBJECT (b), "clicked", 
@@ -293,33 +298,33 @@ sp_export_dialog_area_frame (GtkWidget * dlg)
     gtk_table_set_col_spacings (GTK_TABLE (t), 4);
     gtk_container_set_border_width (GTK_CONTAINER (t), 4);
 
-    sp_export_spinbutton_new ( "x0", 0.0, -10000.0, 10000.0, 0.1, 1.0, us, 
-                               t, 0, 0, _("x0:"), NULL, 2, 1,
+    sp_export_spinbutton_new ( "x0", 0.0, -1000000.0, 1000000.0, 0.1, 1.0, us, 
+                               t, 0, 0, _("_x0:"), NULL, 2, 1,
                                G_CALLBACK ( sp_export_area_x_value_changed), 
                                dlg );
 
-    sp_export_spinbutton_new ( "x1", 0.0, -10000.0, 10000.0, 0.1, 1.0, us, 
-                               t, 2, 0, _("x1:"), NULL, 2, 1,
+    sp_export_spinbutton_new ( "x1", 0.0, -1000000.0, 1000000.0, 0.1, 1.0, us, 
+                               t, 2, 0, _("x_1:"), NULL, 2, 1,
                                G_CALLBACK (sp_export_area_x_value_changed), 
                                dlg );
 
-    sp_export_spinbutton_new ( "width", 0.0, -10000.0, 10000.0, 0.1, 1.0, 
+    sp_export_spinbutton_new ( "width", 0.0, -1000000.0, 1000000.0, 0.1, 1.0, 
                                us, t, 4, 0, _("Width:"), NULL, 2, 1,
                                G_CALLBACK 
                                    (sp_export_area_width_value_changed), 
                                dlg );
 
-    sp_export_spinbutton_new ( "y0", 0.0, -10000.0, 10000.0, 0.1, 1.0, us, 
-                               t, 0, 1, _("y0:"), NULL, 2, 1,
+    sp_export_spinbutton_new ( "y0", 0.0, -1000000.0, 1000000.0, 0.1, 1.0, us, 
+                               t, 0, 1, _("_y0:"), NULL, 2, 1,
                                G_CALLBACK (sp_export_area_y_value_changed), 
                                dlg );
 
-    sp_export_spinbutton_new ( "y1", 0.0, -10000.0, 10000.0, 0.1, 1.0, us, 
-                               t, 2, 1, _("y1:"), NULL, 2, 1,
+    sp_export_spinbutton_new ( "y1", 0.0, -1000000.0, 1000000.0, 0.1, 1.0, us, 
+                               t, 2, 1, _("y_1:"), NULL, 2, 1,
                                G_CALLBACK (sp_export_area_y_value_changed), 
                                dlg );
 
-    sp_export_spinbutton_new ( "height", 0.0, -10000.0, 10000.0, 0.1, 1.0, 
+    sp_export_spinbutton_new ( "height", 0.0, -1000000.0, 1000000.0, 0.1, 1.0, 
                                us, t, 4, 1, _("Height:"), NULL, 2, 1,
                                G_CALLBACK 
                                    (sp_export_area_height_value_changed), 
@@ -328,8 +333,6 @@ sp_export_dialog_area_frame (GtkWidget * dlg)
     /* Adding in the unit box */
     gtk_box_pack_start(GTK_BOX(vb), unitbox, FALSE, FALSE, 0);
 
-    gtk_widget_show_all (f);
-
     return f;
 } // end of sp_export_dialog_area_frame
 
@@ -338,7 +341,7 @@ void
 sp_export_dialog (void)
 {
     if (!dlg) {
-        GtkWidget *vb, *f, *t, *hb, *fe, *hs, *b;
+        GtkWidget *vb, *hb;
 
         gchar title[500];
         sp_ui_dialog_title_string (Inkscape::Verb::get(SP_VERB_FILE_EXPORT), title);
@@ -368,10 +371,9 @@ sp_export_dialog (void)
         wd.win = dlg;
         wd.stop = 0;
         
-        
         g_signal_connect   ( G_OBJECT (INKSCAPE), "activate_desktop", 
                              G_CALLBACK (sp_transientize_callback), &wd);
-                             
+                            
         gtk_signal_connect ( GTK_OBJECT (dlg), "event", 
                              GTK_SIGNAL_FUNC (sp_dialog_event_handler), dlg);
                              
@@ -390,152 +392,163 @@ sp_export_dialog (void)
         g_signal_connect   ( G_OBJECT (INKSCAPE), "dialogs_unhide", 
                              G_CALLBACK (sp_dialog_unhide), dlg);
 
+        GtkTooltips *tt = gtk_tooltips_new();
                              
         vb = gtk_vbox_new (FALSE, 4);
-        gtk_widget_show (vb);
-        gtk_container_set_border_width (GTK_CONTAINER (vb), 4);
+        gtk_container_set_border_width (GTK_CONTAINER (vb), 0);
         gtk_container_add (GTK_CONTAINER (dlg), vb);
 
         /* Export area frame */
-        f = sp_export_dialog_area_frame(dlg);
-        gtk_box_pack_start (GTK_BOX (vb), f, FALSE, FALSE, 0);
+        {
+            GtkWidget *f = sp_export_dialog_area_frame(dlg);
+            gtk_box_pack_start (GTK_BOX (vb), f, FALSE, FALSE, 0);
+        }
 
         /* Bitmap size frame */
-        f = gtk_frame_new (_("Bitmap size"));
-        gtk_box_pack_start (GTK_BOX (vb), f, FALSE, FALSE, 0);
-        t = gtk_table_new (2, 5, FALSE);
-        gtk_table_set_row_spacings (GTK_TABLE (t), 4);
-        gtk_table_set_col_spacings (GTK_TABLE (t), 4);
-        gtk_container_set_border_width (GTK_CONTAINER (t), 4);
-        gtk_container_add (GTK_CONTAINER (f), t);
+        {
+            GtkWidget *f = gtk_frame_new (_("Bitmap size"));
+            gtk_box_pack_start (GTK_BOX (vb), f, FALSE, FALSE, 0);
+            GtkWidget *t = gtk_table_new (2, 5, FALSE);
+            gtk_table_set_row_spacings (GTK_TABLE (t), 4);
+            gtk_table_set_col_spacings (GTK_TABLE (t), 4);
+            gtk_container_set_border_width (GTK_CONTAINER (t), 4);
+            gtk_container_add (GTK_CONTAINER (f), t);
 
-        sp_export_spinbutton_new ( "bmwidth", 16.0, 1.0, 1000000.0, 1.0, 10.0, 
-                                   NULL, t, 0, 0,
-                                   _("Width:"), _("pixels"), 0, 1,
-                                   G_CALLBACK 
+            sp_export_spinbutton_new ( "bmwidth", 16.0, 1.0, 1000000.0, 1.0, 10.0, 
+                                       NULL, t, 0, 0,
+                                       _("_Width:"), _("pixels at"), 0, 1,
+                                       G_CALLBACK 
                                        (sp_export_bitmap_width_value_changed), 
-                                   dlg );
+                                       dlg );
 
-        sp_export_spinbutton_new ( "xdpi", 
-                                   prefs_get_double_attribute 
+            sp_export_spinbutton_new ( "xdpi", 
+                                       prefs_get_double_attribute 
                                        ( "dialogs.export.defaultxdpi", 
                                          "value", DPI_BASE), 
-                                   1.0, 9600.0, 0.1, 1.0, NULL, t, 3, 0,
-                                   NULL, _("dpi"), 2, 1,
-                                   G_CALLBACK (sp_export_xdpi_value_changed), 
-                                   dlg );
+                                       1.0, 9600.0, 0.1, 1.0, NULL, t, 3, 0,
+                                       NULL, _("dp_i"), 2, 1,
+                                       G_CALLBACK (sp_export_xdpi_value_changed), 
+                                       dlg );
 
-        sp_export_spinbutton_new ( "bmheight", 16.0, 1.0, 1000000.0, 1, 10.0, 
-                                   NULL, t, 0, 1, _("Height:"), _("pixels"), 
-                                   0, 0, NULL, dlg );
+            sp_export_spinbutton_new ( "bmheight", 16.0, 1.0, 1000000.0, 1, 10.0, 
+                                       NULL, t, 0, 1, _("Height:"), _("pixels at"), 
+                                       0, 0, NULL, dlg );
 
-        /*
-         * TODO: Nees fixing: there's no way to set ydpi currently, so we use  
-         *       the defaultxdpi value here, too...
-         */
-        sp_export_spinbutton_new ( "ydpi", prefs_get_double_attribute 
-                                               ( "dialogs.export.defaultxdpi", 
-                                                 "value", DPI_BASE), 
-                                   1.0, 9600.0, 0.1, 1.0, NULL, t, 3, 1,
-                                   NULL, _("dpi"), 2, 0, NULL, dlg );
+            /*
+             * TODO: Nees fixing: there's no way to set ydpi currently, so we use  
+             *       the defaultxdpi value here, too...
+             */
+            sp_export_spinbutton_new ( "ydpi", prefs_get_double_attribute 
+                                       ( "dialogs.export.defaultxdpi", 
+                                         "value", DPI_BASE), 
+                                       1.0, 9600.0, 0.1, 1.0, NULL, t, 3, 1,
+                                       NULL, _("dpi"), 2, 0, NULL, dlg );
+        }
 
-        gtk_widget_show_all (f);
-
-        
         /* File entry */
-        f = gtk_frame_new (_("Filename"));
-        gtk_box_pack_start (GTK_BOX (vb), f, FALSE, FALSE, 0);
-        fe = gtk_entry_new ();
-
-        /*
-         * set the default filename to be that of the current path + document 
-         * with .png extension
-         *
-         * One thing to notice here is that this filename may get
-         * overwritten, but it won't happen here.  The filename gets
-         * written into the text field, but then the button to select
-         * the area gets set.  In that code the filename can be changed
-         * if there are some with presidence in the document.  So, while
-         * this code sets the name first, it may not be the one users
-         * really see.
-         */
-        if (SP_ACTIVE_DOCUMENT && SP_DOCUMENT_URI (SP_ACTIVE_DOCUMENT))
         {
-            gchar *name;
-            SPDocument * doc = SP_ACTIVE_DOCUMENT;
-            const gchar *uri = SP_DOCUMENT_URI (doc);
-            SPRepr * repr = sp_document_repr_root(doc);
-            const gchar * text_extension = sp_repr_attr(repr, "inkscape:output_extension");
-            Inkscape::Extension::Output * oextension = NULL;
+            GtkWidget *frame = gtk_frame_new ("");
+            GtkWidget *flabel = gtk_label_new_with_mnemonic (_("_Filename"));
+            gtk_frame_set_label_widget (GTK_FRAME(frame), flabel);
+            gtk_box_pack_start (GTK_BOX (vb), frame, FALSE, FALSE, 0);
 
-            if (text_extension != NULL) {
-                oextension = dynamic_cast<Inkscape::Extension::Output *>(Inkscape::Extension::db.get(text_extension));
+            GtkWidget *fe = gtk_entry_new ();
+
+            /*
+             * set the default filename to be that of the current path + document 
+             * with .png extension
+             *
+             * One thing to notice here is that this filename may get
+             * overwritten, but it won't happen here.  The filename gets
+             * written into the text field, but then the button to select
+             * the area gets set.  In that code the filename can be changed
+             * if there are some with presidence in the document.  So, while
+             * this code sets the name first, it may not be the one users
+             * really see.
+             */
+            if (SP_ACTIVE_DOCUMENT && SP_DOCUMENT_URI (SP_ACTIVE_DOCUMENT))
+            {
+                gchar *name;
+                SPDocument * doc = SP_ACTIVE_DOCUMENT;
+                const gchar *uri = SP_DOCUMENT_URI (doc);
+                Inkscape::XML::Node * repr = sp_document_repr_root(doc);
+                const gchar * text_extension = repr->attribute("inkscape:output_extension");
+                Inkscape::Extension::Output * oextension = NULL;
+
+                if (text_extension != NULL) {
+                    oextension = dynamic_cast<Inkscape::Extension::Output *>(Inkscape::Extension::db.get(text_extension));
+                }
+
+                if (oextension != NULL) {
+                    gchar * old_extension = oextension->get_extension();
+                    if (g_str_has_suffix(uri, old_extension)) {
+                        gchar * uri_copy;
+                        gchar * extension_point;
+                        gchar * final_name;
+
+                        uri_copy = g_strdup(uri);
+                        extension_point = g_strrstr(uri_copy, old_extension);
+                        extension_point[0] = '\0';
+
+                        final_name = g_strconcat(uri_copy, ".png", NULL);
+                        gtk_entry_set_text (GTK_ENTRY (fe), final_name);
+
+                        g_free(final_name);
+                        g_free(uri_copy);
+                    }
+                } else {
+                    name = g_strconcat(uri, ".png", NULL);
+                    gtk_entry_set_text (GTK_ENTRY (fe), name);
+                    g_free(name);
+                } 
+
+                doc_export_name = g_strdup(gtk_entry_get_text(GTK_ENTRY(fe)));
+            }
+            g_signal_connect ( G_OBJECT (fe), "changed", 
+                               G_CALLBACK (sp_export_filename_modified), dlg);
+        
+            hb = gtk_hbox_new (FALSE, 5);
+            gtk_container_add (GTK_CONTAINER (frame), hb);
+            gtk_container_set_border_width (GTK_CONTAINER (hb), 4);
+
+            {
+                GtkWidget *b = gtk_button_new_with_mnemonic (_("_Browse..."));
+                gtk_box_pack_end (GTK_BOX (hb), b, FALSE, FALSE, 4);
+                g_signal_connect ( G_OBJECT (b), "clicked", 
+                                   G_CALLBACK (sp_export_browse_clicked), NULL );
             }
 
-            if (oextension != NULL) {
-                gchar * old_extension = oextension->get_extension();
-                if (g_str_has_suffix(uri, old_extension)) {
-                    gchar * uri_copy;
-                    gchar * extension_point;
-                    gchar * final_name;
+            gtk_box_pack_start (GTK_BOX (hb), fe, TRUE, TRUE, 0);
+            gtk_object_set_data (GTK_OBJECT (dlg), "filename", fe);
+            gtk_object_set_data (GTK_OBJECT (dlg), "filename-modified", (gpointer)FALSE);
+            original_name = g_strdup(gtk_entry_get_text (GTK_ENTRY (fe)));
+            // pressing enter in the filename field is the same as clicking export:
+            g_signal_connect ( G_OBJECT (fe), "activate", 
+                               G_CALLBACK (sp_export_export_clicked), dlg );
+            // focus is in the filename initially:
+            gtk_widget_grab_focus (GTK_WIDGET (fe));
 
-                    uri_copy = g_strdup(uri);
-                    extension_point = g_strrstr(uri_copy, old_extension);
-                    extension_point[0] = '\0';
-
-                    final_name = g_strconcat(uri_copy, ".png", NULL);
-                    gtk_entry_set_text (GTK_ENTRY (fe), final_name);
-
-                    g_free(final_name);
-                    g_free(uri_copy);
-                }
-            } else {
-                name = g_strconcat(uri, ".png", NULL);
-                gtk_entry_set_text (GTK_ENTRY (fe), name);
-                g_free(name);
-            } 
-
-            doc_export_name = g_strdup(gtk_entry_get_text(GTK_ENTRY(fe)));
+            // mnemonic in frame label moves focus to filename:
+            gtk_label_set_mnemonic_widget (GTK_LABEL(flabel), fe);
         }
-        g_signal_connect ( G_OBJECT (fe), "changed", 
-                           G_CALLBACK (sp_export_filename_modified), dlg);
-
-        
-        hb = gtk_hbox_new (FALSE, 5);
-        gtk_container_add (GTK_CONTAINER (f), hb);
-        gtk_container_set_border_width (GTK_CONTAINER (hb), 4);
-
-        b = gtk_button_new_with_label (_("Browse..."));
-        gtk_box_pack_end (GTK_BOX (hb), b, FALSE, FALSE, 4);
-        g_signal_connect ( G_OBJECT (b), "clicked", 
-                           G_CALLBACK (sp_export_browse_clicked), NULL );
-
-        gtk_box_pack_start (GTK_BOX (hb), fe, TRUE, TRUE, 0);
-        gtk_object_set_data (GTK_OBJECT (dlg), "filename", fe);
-        gtk_object_set_data (GTK_OBJECT (dlg), "filename-modified", (gpointer)FALSE);
-        original_name = g_strdup(gtk_entry_get_text (GTK_ENTRY (fe)));
-        gtk_widget_show_all (f);
-        // pressing enter in the filename field is the same as clicking export:
-        g_signal_connect ( G_OBJECT (fe), "activate", 
-                           G_CALLBACK (sp_export_export_clicked), dlg );
-        // focus is in the filename initially:
-        gtk_widget_grab_focus (GTK_WIDGET (fe));
 
         /* Buttons */
         hb = gtk_hbox_new (FALSE, 0);
-        gtk_widget_show (hb);
         gtk_box_pack_end (GTK_BOX (vb), hb, FALSE, FALSE, 0);
 
-        b = gtk_button_new_with_label (_("Export"));
-        gtk_widget_show (b);
-        gtk_box_pack_end (GTK_BOX (hb), b, FALSE, FALSE, 0);
-        gtk_signal_connect ( GTK_OBJECT (b), "clicked", 
-                             GTK_SIGNAL_FUNC (sp_export_export_clicked), dlg );
+        {
+            GtkWidget *b = gtk_button_new ();
+            GtkWidget *l = gtk_label_new ("");
+            gtk_label_set_markup_with_mnemonic (GTK_LABEL(l), _(" <b>_Export</b> "));
+            gtk_container_add (GTK_CONTAINER(b), l);
+            gtk_tooltips_set_tip (tt, b, _("Export the bitmap file with these settings"), NULL);
+            gtk_signal_connect ( GTK_OBJECT (b), "clicked", 
+                                 GTK_SIGNAL_FUNC (sp_export_export_clicked), dlg );
+            gtk_box_pack_end (GTK_BOX (hb), b, FALSE, FALSE, 0);
+        }
 
-        hs = gtk_hseparator_new ();
-        gtk_widget_show (hs);
-        gtk_box_pack_end (GTK_BOX (vb), hs, FALSE, FALSE, 0);
-    
+        gtk_widget_show_all (vb);
+  
     } // end of if (!dlg)
 
     sp_export_find_default_selection(dlg);
@@ -595,7 +608,7 @@ sp_export_selection_changed ( Inkscape::Application *inkscape,
                               GtkObject *base )
 {
     selection_type current_key;
-    current_key = (selection_type)((int)gtk_object_get_data(GTK_OBJECT(base), "selection-type"));
+    current_key = (selection_type)(GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(base), "selection-type")));
 
     if ((current_key == SELECTION_DRAWING || current_key == SELECTION_PAGE) &&
             (SP_DT_SELECTION(SP_ACTIVE_DESKTOP))->isEmpty() == false &&
@@ -606,7 +619,7 @@ sp_export_selection_changed ( Inkscape::Application *inkscape,
     }
     was_empty = (SP_DT_SELECTION(SP_ACTIVE_DESKTOP))->isEmpty();
 
-    current_key = (selection_type)((int)gtk_object_get_data(GTK_OBJECT(base), "selection-type"));
+    current_key = (selection_type)(GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(base), "selection-type")));
 
     if (inkscape &&
             SP_IS_INKSCAPE (inkscape) &&
@@ -627,7 +640,7 @@ sp_export_selection_modified ( Inkscape::Application *inkscape,
                                GtkObject *base )
 {
     selection_type current_key;
-    current_key = (selection_type)((int)gtk_object_get_data(GTK_OBJECT(base), "selection-type"));
+    current_key = (selection_type)(GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(base), "selection-type")));
 
     switch (current_key) {
         case SELECTION_DRAWING:
@@ -664,8 +677,8 @@ sp_export_area_toggled (GtkToggleButton *tb, GtkObject *base)
         return;
 
     selection_type key, old_key;
-    key = (selection_type)((int)gtk_object_get_data (GTK_OBJECT (tb), "key"));
-    old_key = (selection_type)((int)gtk_object_get_data(GTK_OBJECT(base), "selection-type"));
+    key = (selection_type)(GPOINTER_TO_INT(gtk_object_get_data (GTK_OBJECT (tb), "key")));
+    old_key = (selection_type)(GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(base), "selection-type")));
 
     /* Ignore all "turned off" events unless we're the only active button */
     if (!gtk_toggle_button_get_active (tb) ) {
@@ -757,19 +770,19 @@ sp_export_area_toggled (GtkToggleButton *tb, GtkObject *base)
             case SELECTION_PAGE:
             case SELECTION_DRAWING: {
                 SPDocument * doc = SP_ACTIVE_DOCUMENT;
-                SPRepr * repr = sp_document_repr_root(doc);
+                Inkscape::XML::Node * repr = sp_document_repr_root(doc);
                 const gchar * dpi_string;
 
-                filename = sp_repr_attr(repr, "inkscape:export-filename");
+                filename = repr->attribute("inkscape:export-filename");
 
                 dpi_string = NULL;
-                dpi_string = sp_repr_attr(repr, "inkscape:export-xdpi");
+                dpi_string = repr->attribute("inkscape:export-xdpi");
                 if (dpi_string != NULL) {
                     xdpi = atof(dpi_string);
                 }
 
                 dpi_string = NULL;
-                dpi_string = sp_repr_attr(repr, "inkscape:export-ydpi");
+                dpi_string = repr->attribute("inkscape:export-ydpi");
                 if (dpi_string != NULL) {
                     ydpi = atof(dpi_string);
                 }
@@ -798,17 +811,17 @@ sp_export_area_toggled (GtkToggleButton *tb, GtkObject *base)
                             ydpi_search;
                             reprlst = reprlst->next) {
                         const gchar * dpi_string;
-                        SPRepr * repr = (SPRepr *)reprlst->data;
+                        Inkscape::XML::Node * repr = (Inkscape::XML::Node *)reprlst->data;
 
                         if (filename_search) {
-                            filename = sp_repr_attr(repr, "inkscape:export-filename");
+                            filename = repr->attribute("inkscape:export-filename");
                             if (filename != NULL)
                                 filename_search = FALSE;
                         }
 
                         if (xdpi_search) {
                             dpi_string = NULL;
-                            dpi_string = sp_repr_attr(repr, "inkscape:export-xdpi");
+                            dpi_string = repr->attribute("inkscape:export-xdpi");
                             if (dpi_string != NULL) {
                                 xdpi = atof(dpi_string);
                                 xdpi_search = FALSE;
@@ -817,7 +830,7 @@ sp_export_area_toggled (GtkToggleButton *tb, GtkObject *base)
 
                         if (ydpi_search) {
                             dpi_string = NULL;
-                            dpi_string = sp_repr_attr(repr, "inkscape:export-ydpi");
+                            dpi_string = repr->attribute("inkscape:export-ydpi");
                             if (dpi_string != NULL) {
                                 ydpi = atof(dpi_string);
                                 ydpi_search = FALSE;
@@ -831,9 +844,9 @@ sp_export_area_toggled (GtkToggleButton *tb, GtkObject *base)
                         const gchar * id = NULL;
                         reprlst = SP_DT_SELECTION(SP_ACTIVE_DESKTOP)->reprList();
                         for(; reprlst != NULL; reprlst = reprlst->next) {
-                            SPRepr * repr = (SPRepr *)reprlst->data;
-                            if (sp_repr_attr(repr, "id")) {
-                                id = sp_repr_attr(repr, "id");
+                            Inkscape::XML::Node * repr = (Inkscape::XML::Node *)reprlst->data;
+                            if (repr->attribute("id")) {
+                                id = repr->attribute("id");
                                 break;
                             }
                         }
@@ -859,7 +872,7 @@ sp_export_area_toggled (GtkToggleButton *tb, GtkObject *base)
 
                         if (directory == NULL) {
                             // std::cout << "Home Directory" << std::endl;
-                            directory = g_strdup(g_get_home_dir());
+                            directory = homedir_path(NULL);
                         }
 
                         gchar * id_ext = g_strconcat(id, ".png", NULL);
@@ -939,40 +952,37 @@ sp_export_progress_callback (float value, void *data)
 static void
 sp_export_export_clicked (GtkButton *button, GtkObject *base)
 {
-    GtkWidget *fe;
-    const gchar *filename;
-    float x0, y0, x1, y1, xdpi, ydpi;
-    int width, height;
-
     if (!SP_ACTIVE_DESKTOP) return;
 
-    fe = (GtkWidget *)gtk_object_get_data (base, "filename");
-    filename = gtk_entry_get_text (GTK_ENTRY (fe));
+    GtkWidget *fe = (GtkWidget *)gtk_object_get_data(base, "filename");
+    gchar const *filename = gtk_entry_get_text(GTK_ENTRY(fe));
 
-    x0 = sp_export_value_get_px (base, "x0");
-    y0 = sp_export_value_get_px (base, "y0");
-    x1 = sp_export_value_get_px (base, "x1");
-    y1 = sp_export_value_get_px (base, "y1");
-    xdpi = sp_export_value_get (base, "xdpi");
-    ydpi = sp_export_value_get (base, "ydpi");
-    width = (int) (sp_export_value_get (base, "bmwidth") + 0.5);
-    height = (int) (sp_export_value_get (base, "bmheight") + 0.5);
+    float const x0 = sp_export_value_get_px(base, "x0");
+    float const y0 = sp_export_value_get_px(base, "y0");
+    float const x1 = sp_export_value_get_px(base, "x1");
+    float const y1 = sp_export_value_get_px(base, "y1");
+    float const xdpi = sp_export_value_get(base, "xdpi");
+    float const ydpi = sp_export_value_get(base, "ydpi");
+    int const width = int(sp_export_value_get(base, "bmwidth") + 0.5);
+    int const height = int(sp_export_value_get(base, "bmheight") + 0.5);
 
-    if (filename == NULL || strlen (filename) == 0) {
-        sp_ui_error_dialog (_("You have to enter a filename"));
+    if (filename == NULL || *filename == '\0') {
+        sp_ui_error_dialog(_("You have to enter a filename"));
         return;
     }
-    
+
     if (!((x1 > x0) && (y1 > y0) && (width > 0) && (height > 0))) {
         sp_ui_error_dialog (_("The chosen area to be exported is invalid"));
         return;
     }
 
-    gchar * dirname = g_dirname(filename);
-    if (dirname == NULL || !Inkscape::IO::file_test(dirname, (GFileTest)(G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR))) {
-        gchar * error;
-        gchar * safeDir = Inkscape::IO::sanitizeString(dirname);
-        error = g_strdup_printf(_("Directory %s does not exist or is not a directory.\n"), safeDir);
+    gchar *dirname = g_dirname(filename);
+    if ( dirname == NULL
+         || !Inkscape::IO::file_test(dirname, (GFileTest)(G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR)) )
+    {
+        gchar *safeDir = Inkscape::IO::sanitizeString(dirname);
+        gchar *error = g_strdup_printf(_("Directory %s does not exist or is not a directory.\n"),
+                                       safeDir);
         sp_ui_error_dialog(error);
         g_free(safeDir);
         g_free(error);
@@ -993,8 +1003,8 @@ sp_export_export_clicked (GtkButton *button, GtkObject *base)
     gtk_window_set_resizable (GTK_WINDOW (dlg), FALSE);
     g_object_set_data ((GObject *) base, "progress", prg);
     fn = g_path_get_basename (filename);
-    text = g_strdup_printf ( _("Exporting [%d x %d] %s"), 
-                             width, height, fn);
+    text = g_strdup_printf ( _("Exporting %s (%d x %d)"), 
+                             fn, width, height);
     g_free (fn);
     gtk_progress_bar_set_text ((GtkProgressBar *) prg, text);
     g_free (text);
@@ -1036,28 +1046,28 @@ sp_export_export_clicked (GtkButton *button, GtkObject *base)
     g_object_set_data (G_OBJECT (base), "cancel", (gpointer) 0);
 
     /* Setup the values in the document */
-    switch ((selection_type)((int)gtk_object_get_data(GTK_OBJECT(base), "selection-type"))) {
+    switch ((selection_type)(GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(base), "selection-type")))) {
         case SELECTION_PAGE:
         case SELECTION_DRAWING: {
             SPDocument * doc = SP_ACTIVE_DOCUMENT;
-            SPRepr * repr = sp_document_repr_root(doc);
+            Inkscape::XML::Node * repr = sp_document_repr_root(doc);
             bool modified = FALSE;
             const gchar * temp_string;
 
             gboolean saved = sp_document_get_undo_sensitive(doc);
             sp_document_set_undo_sensitive(doc, FALSE);
 
-            temp_string = sp_repr_attr(repr, "inkscape:export-filename");
+            temp_string = repr->attribute("inkscape:export-filename");
             if (temp_string == NULL || strcmp(temp_string, filename)) {
                 sp_repr_set_attr(repr, "inkscape:export-filename", filename);
                 modified = TRUE;
             }
-            temp_string = sp_repr_attr(repr, "inkscape:export-xdpi");
+            temp_string = repr->attribute("inkscape:export-xdpi");
             if (temp_string == NULL || xdpi != atof(temp_string)) {
                 sp_repr_set_double(repr, "inkscape:export-xdpi", xdpi);
                 modified = TRUE;
             }
-            temp_string = sp_repr_attr(repr, "inkscape:export-ydpi");
+            temp_string = repr->attribute("inkscape:export-ydpi");
             if (temp_string == NULL || xdpi != atof(temp_string)) {
                 sp_repr_set_double(repr, "inkscape:export-ydpi", ydpi);
                 modified = TRUE;
@@ -1078,25 +1088,25 @@ sp_export_export_clicked (GtkButton *button, GtkObject *base)
             reprlst = SP_DT_SELECTION(SP_ACTIVE_DESKTOP)->reprList();
 
             for(; reprlst != NULL; reprlst = reprlst->next) {
-                SPRepr * repr = (SPRepr *)reprlst->data;
+                Inkscape::XML::Node * repr = (Inkscape::XML::Node *)reprlst->data;
                 const gchar * temp_string;
 
-                if (sp_repr_attr(repr, "id") == NULL ||
-                        !(g_strrstr(filename, sp_repr_attr(repr,"id")) != NULL &&
+                if (repr->attribute("id") == NULL ||
+                        !(g_strrstr(filename, repr->attribute("id")) != NULL &&
                           (!SP_DOCUMENT_URI(SP_ACTIVE_DOCUMENT) || 
                             strcmp(g_dirname(filename), g_dirname(SP_DOCUMENT_URI(SP_ACTIVE_DOCUMENT))) == 0))) {
-                    temp_string = sp_repr_attr(repr, "inkscape:export-filename");
+                    temp_string = repr->attribute("inkscape:export-filename");
                     if (temp_string == NULL || strcmp(temp_string, filename)) {
                         sp_repr_set_attr(repr, "inkscape:export-filename", filename);
                         modified = TRUE;
                     }
                 }
-                temp_string = sp_repr_attr(repr, "inkscape:export-xdpi");
+                temp_string = repr->attribute("inkscape:export-xdpi");
                 if (temp_string == NULL || xdpi != atof(temp_string)) {
                     sp_repr_set_double(repr, "inkscape:export-xdpi", xdpi);
                     modified = TRUE;
                 }
-                temp_string = sp_repr_attr(repr, "inkscape:export-ydpi");
+                temp_string = repr->attribute("inkscape:export-ydpi");
                 if (temp_string == NULL || xdpi != atof(temp_string)) {
                     sp_repr_set_double(repr, "inkscape:export-ydpi", ydpi);
                     modified = TRUE;
@@ -1104,7 +1114,7 @@ sp_export_export_clicked (GtkButton *button, GtkObject *base)
             }
 
             if (modified) {
-                SPRepr * repr = sp_document_repr_root(doc);
+                Inkscape::XML::Node * repr = sp_document_repr_root(doc);
                 sp_repr_set_attr(repr, "sodipodi:modified", "TRUE");
             }
 
@@ -1135,8 +1145,8 @@ sp_export_browse_clicked (GtkButton *button, gpointer userdata)
 
     filename = gtk_entry_get_text (GTK_ENTRY (fe));
 
-    if(strlen(filename) == 0) {
-        filename = g_build_filename (g_get_home_dir(), G_DIR_SEPARATOR_S, NULL);
+    if (*filename == '\0') {
+        filename = homedir_path(NULL);
     }
 
     gtk_file_selection_set_filename (GTK_FILE_SELECTION (fs), filename);
@@ -1172,8 +1182,9 @@ sp_export_browse_store (GtkButton *button, gpointer userdata)
     fe = (GtkWidget *)g_object_get_data (G_OBJECT (dlg), "filename");
 
     file = gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));
-
-    gtk_entry_set_text (GTK_ENTRY (fe), file);
+    gchar * utf8file = g_filename_to_utf8( file, -1, NULL, NULL, NULL );
+    gtk_entry_set_text (GTK_ENTRY (fe), utf8file);
+    g_free(utf8file);
 
     g_object_set_data (G_OBJECT (dlg), "filename", fe);
 
@@ -1229,7 +1240,7 @@ sp_export_detect_size(GtkObject * base) {
     current_bbox.round(2);
     // std::cout << "Current " << current_bbox;
 
-    this_test[0] = (selection_type)((int)gtk_object_get_data(GTK_OBJECT(base), "selection-type"));
+    this_test[0] = (selection_type)(GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(base), "selection-type")));
     for (int i = 0; i < SELECTION_NUMBER_OF; i++) {
         this_test[i + 1] = test_order[i];
     }
@@ -1298,7 +1309,7 @@ sp_export_detect_size(GtkObject * base) {
 
     /* We're now using a custom size, not a fixed one */
     /* printf("Detecting state: %s\n", selection_names[key]); */
-    selection_type old = (selection_type)((int)gtk_object_get_data(GTK_OBJECT(base), "selection-type"));
+    selection_type old = (selection_type)(GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(base), "selection-type")));
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(gtk_object_get_data(base, selection_names[old])), FALSE);
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(gtk_object_get_data(base, selection_names[key])), TRUE);
     gtk_object_set_data(GTK_OBJECT(base), "selection-type", (gpointer)key);
