@@ -156,7 +156,7 @@ bool SVGPreview::setFileName(Glib::ustring &theFileName)
 
     fileName = Glib::filename_to_utf8(fileName);
 
-    SPDocument *doc = sp_document_new (fileName.c_str(), 0, 0);
+    SPDocument *doc = sp_document_new (fileName.c_str(), 0);
     if (!doc) {
         g_warning("SVGView: error loading document '%s'\n", fileName.c_str());
         return false;
@@ -177,7 +177,7 @@ bool SVGPreview::setFromMem(char const *xmlBuffer)
         return false;
 
     gint len = (gint)strlen(xmlBuffer);
-    SPDocument *doc = sp_document_new_from_mem(xmlBuffer, len, 0, 0);
+    SPDocument *doc = sp_document_new_from_mem(xmlBuffer, len, 0);
     if (!doc) {
         g_warning("SVGView: error loading buffer '%s'\n",xmlBuffer);
         return false;
@@ -542,7 +542,7 @@ bool SVGPreview::set(Glib::ustring &fileName, int dialogType)
             return FALSE;
             }
         long fileLen = info.st_size;
-        if (fileLen > 0x100000L)
+        if (fileLen > 0x150000L)
             {
             showingNoPreview = false;
             showTooLarge(fileLen);
@@ -616,7 +616,7 @@ public:
 
     gchar *getFilename();
 
-
+    Glib::SListHandle<Glib::ustring> getFilenames ();
 protected:
 
 
@@ -818,7 +818,8 @@ FileOpenDialogImpl::FileOpenDialogImpl(char const *dir,
 
 
     /* One file at a time */
-    set_select_multiple(false);
+    /* And also Multiple Files */
+    set_select_multiple(true);
 
     /* Initalize to Autodetect */
     extension = NULL;
@@ -831,7 +832,14 @@ FileOpenDialogImpl::FileOpenDialogImpl(char const *dir,
 
     /* Set the pwd and/or the filename */
     if (dir != NULL)
-        set_current_folder(dir);
+    {
+        Glib::ustring udir(dir);
+        Glib::ustring::size_type len = udir.length();
+        // leaving a trailing backslash on the directory name leads to the infamous
+        // double-directory bug on win32
+        if (len != 0 && udir[len - 1] == '\\') udir.erase(len - 1);
+        set_current_folder(udir.c_str());
+    }
 
     //###### Add the file types menu
     createFilterMenu();
@@ -947,7 +955,13 @@ FileOpenDialogImpl::getFilename (void)
 }
 
 
-
+/**
+ * To Get Multiple filenames selected at-once.
+ */
+Glib::SListHandle<Glib::ustring>FileOpenDialogImpl::getFilenames()
+{    
+    return get_filenames();
+}
 
 
 
@@ -1231,7 +1245,14 @@ FileSaveDialogImpl::FileSaveDialogImpl(char const *dir,
 
     /* Set the pwd and/or the filename */
     if (dir != NULL)
-        set_current_folder(dir);
+    {
+        Glib::ustring udir(dir);
+        Glib::ustring::size_type len = udir.length();
+        // leaving a trailing backslash on the directory name leads to the infamous
+        // double-directory bug on win32
+        if (len != 0 && udir[len - 1] == '\\') udir.erase(len - 1);
+        set_current_folder(udir.c_str());
+    }
 
     //###### Add the file types menu
     //createFilterMenu();
