@@ -35,10 +35,10 @@ public :
     virtual Gtk::Container & get_main_widget() {return _widget;}
 
 private :
-    DialogFind();    
+    DialogFind();
     virtual ~DialogFind(){};
     void addSearchField(Glib::ustring label, Glib::ustring tiptext, int line);
-    Gtk::VBox _widget; 
+    Gtk::VBox _widget;
     Gtk::Table _fieldsTable;
     Docker *_pDocker;
     Gtk::Tooltips _tooltips;
@@ -72,12 +72,9 @@ DialogFind::DialogFind():
     _fieldsTable(5, 4, true),
     _pDocker(0)
 {
-    PixBufFactory::ID id ("inkscape_options", 60, 20);
-    Gtk::Image*  pImage = 
-        Gtk::manage( new Gtk::Image(
-                         PixBufFactory::get().
-                         getIcon(id)));
-    _widget.pack_start(*pImage);
+    Glib::ustring id("inkscape_options");
+    Gtk::Widget*  pIcon = Gtk::manage( sp_icon_get_icon(id, GTK_ICON_SIZE_LARGE_TOOLBAR) );
+    _widget.pack_start(*pIcon);
     _widget.pack_start(_fieldsTable);
     addSearchField("_Text", _("Find objects by their text content (exact or partial match)"), 0);
     addSearchField("_ID", _("Find objects by the value of the id attribute (exact or partial match)"), 1);
@@ -90,7 +87,7 @@ DialogFind::DialogFind():
 
 DialogFind & DialogFind::get()
 {
-    static DialogFind &da = *(new DialogFind());    
+    static DialogFind &da = *(new DialogFind());
     return da;
 }
 
@@ -188,7 +185,7 @@ item_id_match (SPItem *item, const gchar *id, bool exact)
     if (SP_IS_STRING(item)) // SPStrings have "on demand" ids which are useless for searching
         return false;
 
-    const gchar *item_id = sp_repr_attr (SP_OBJECT_REPR (item), "id");
+    const gchar *item_id = (SP_OBJECT_REPR (item))->attribute("id");
     if (item_id == NULL)
         return false;
 
@@ -206,7 +203,7 @@ item_text_match (SPItem *item, const gchar *text, bool exact)
     if (SP_OBJECT_REPR (item) == NULL)
         return false;
 
-    const gchar *item_text = sp_repr_content (SP_OBJECT_REPR (item));
+    const gchar *item_text = SP_OBJECT_REPR (item)->content();
     if (item_text == NULL)
         return false;
 
@@ -224,7 +221,7 @@ item_style_match (SPItem *item, const gchar *text, bool exact)
     if (SP_OBJECT_REPR (item) == NULL)
         return false;
 
-    const gchar *item_text = sp_repr_attr (SP_OBJECT_REPR (item), "style");
+    const gchar *item_text = (SP_OBJECT_REPR (item))->attribute("style");
     if (item_text == NULL)
         return false;
 
@@ -242,10 +239,10 @@ item_attr_match (SPItem *item, const gchar *name, bool exact)
         return false;
 
     if (exact) {
-        const gchar *attr_value = sp_repr_attr (SP_OBJECT_REPR (item), name);
+        const gchar *attr_value = (SP_OBJECT_REPR (item))->attribute(name);
         return ((bool) (attr_value != NULL));
     } else {
-        return sp_repr_has_attr (SP_OBJECT_REPR (item), name);
+        return SP_OBJECT_REPR (item)->matchAttributeName(name);
     }
 }
 
@@ -272,13 +269,13 @@ filter_onefield (GSList *l, GObject *dlg, const gchar *field, bool (*match_funct
 }
 
 
-bool 
+bool
 type_checkbox (GtkWidget *widget, const gchar *data)
 {
     return  gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (gtk_object_get_data (GTK_OBJECT (widget), data)));
 }
 
-bool 
+bool
 item_type_match (SPItem *item, GtkWidget *widget)
 {
     SPDesktop *desktop = SP_ACTIVE_DESKTOP;
@@ -323,9 +320,9 @@ filter_types (GSList *l, GObject *dlg, bool (*match_function)(SPItem *, GtkWidge
     GtkWidget *widget = GTK_WIDGET (gtk_object_get_data (GTK_OBJECT (dlg), "types"));
 
     GtkWidget *alltypes = GTK_WIDGET (gtk_object_get_data (GTK_OBJECT (widget), "all"));
-    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (alltypes))) 
+    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (alltypes)))
         return l;
-    
+
 
     GSList *n = NULL;
     for (GSList *i = l; i != NULL; i = i->next) {
@@ -356,9 +353,9 @@ all_items (SPObject *r, GSList *l, bool hidden, bool locked)
     SPDesktop *desktop = SP_ACTIVE_DESKTOP;
 
     if (SP_IS_DEFS(r))
-        return l; // we're not interested in items in defs 
+        return l; // we're not interested in items in defs
 
-    if (!strcmp (sp_repr_name (SP_OBJECT_REPR (r)), "svg:metadata"))
+    if (!strcmp (SP_OBJECT_REPR (r)->name(), "svg:metadata"))
         return l; // we're not interested in metadata
 
     for (SPObject *child = sp_object_first_child(r); child; child = SP_OBJECT_NEXT (child)) {
@@ -373,7 +370,7 @@ all_items (SPObject *r, GSList *l, bool hidden, bool locked)
 }
 
 GSList *
-all_selection_items (SPSelection *s, GSList *l, SPObject *ancestor, bool hidden, bool locked)
+all_selection_items (Inkscape::Selection *s, GSList *l, SPObject *ancestor, bool hidden, bool locked)
 {
     SPDesktop *desktop = SP_ACTIVE_DESKTOP;
 
@@ -433,9 +430,9 @@ void sp_find_dialog_find(GObject *, GObject *dlg)
                                                  count),
                                         count, all, exact? _("exact") : _("partial"));
 
-        SPSelection *selection = SP_DT_SELECTION (desktop);
+        Inkscape::Selection *selection = SP_DT_SELECTION (desktop);
         selection->clear();
-        selection->setItemList(n);
+        selection->setList(n);
         scroll_to_show_item (desktop, SP_ITEM(n->data));
     } else {
         desktop->messageStack()->flash(Inkscape::WARNING_MESSAGE, _("No objects found"));
@@ -472,7 +469,7 @@ sp_find_new_searchfield (GtkWidget *dlg, GtkWidget *vb, const gchar *label, cons
 {
     GtkWidget *hb = gtk_hbox_new (FALSE, 0);
     GtkWidget *l = gtk_label_new_with_mnemonic (label);
-    gtk_widget_set_usize (l, FIND_LABELWIDTH, -1);
+    gtk_widget_set_size_request (l, FIND_LABELWIDTH, -1);
     gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
     gtk_box_pack_start (GTK_BOX (hb), l, FALSE, FALSE, 0);
 
@@ -505,7 +502,7 @@ toggle_alltypes (GtkToggleButton *tb, gpointer data)
         gtk_widget_hide_all (alltypes_pane);
     } else {
         gtk_widget_show_all (alltypes_pane);
-        
+
         // excplicit toggle to make sure its handler gets called, no matter what was the original state
         gtk_toggle_button_toggled (GTK_TOGGLE_BUTTON (gtk_object_get_data (GTK_OBJECT (data), "shapes")));
         gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (gtk_object_get_data (GTK_OBJECT (data), "shapes")), TRUE);
@@ -538,7 +535,7 @@ toggle_shapes (GtkToggleButton *tb, gpointer data)
 
 
 GtkWidget *
-sp_find_types_checkbox (GtkWidget *w, const gchar *data, gboolean active, 
+sp_find_types_checkbox (GtkWidget *w, const gchar *data, gboolean active,
                         GtkTooltips *tt, const gchar *tip,
                         const gchar *label,
                         void (*toggled)(GtkToggleButton *, gpointer))
@@ -561,7 +558,7 @@ sp_find_types_checkbox (GtkWidget *w, const gchar *data, gboolean active,
 }
 
 GtkWidget *
-sp_find_types_checkbox_indented (GtkWidget *w, const gchar *data, gboolean active, 
+sp_find_types_checkbox_indented (GtkWidget *w, const gchar *data, gboolean active,
                                  GtkTooltips *tt, const gchar *tip,
                                  const gchar *label,
                                  void (*toggled)(GtkToggleButton *, gpointer), guint indent)
@@ -572,7 +569,7 @@ sp_find_types_checkbox_indented (GtkWidget *w, const gchar *data, gboolean activ
     { // empty label for indent
         GtkWidget *l = gtk_label_new ("");
         gtk_widget_show (l);
-        gtk_widget_set_usize (l, FIND_LABELWIDTH + indent, -1);
+        gtk_widget_set_size_request (l, FIND_LABELWIDTH + indent, -1);
         gtk_box_pack_start (GTK_BOX (hb), l, FALSE, FALSE, 0);
     }
 
@@ -598,14 +595,14 @@ sp_find_types ()
         {
             GtkWidget *l = gtk_label_new_with_mnemonic (_("T_ype: "));
             gtk_widget_show (l);
-            gtk_widget_set_usize (l, FIND_LABELWIDTH, -1);
+            gtk_widget_set_size_request (l, FIND_LABELWIDTH, -1);
             gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
             gtk_box_pack_start (GTK_BOX (hb), l, FALSE, FALSE, 0);
         }
 
         GtkWidget *alltypes = sp_find_types_checkbox (vb, "all", TRUE, tt, _("Search in all object types"), _("All types"), toggle_alltypes);
         gtk_box_pack_start (GTK_BOX (hb), alltypes, FALSE, FALSE, 0);
-   
+
         gtk_box_pack_start (GTK_BOX (vb), hb, FALSE, FALSE, 0);
     }
 
@@ -626,7 +623,7 @@ sp_find_types ()
             { // empty label for alignment
                 GtkWidget *l = gtk_label_new ("");
                 gtk_widget_show (l);
-                gtk_widget_set_usize (l, FIND_LABELWIDTH + 20, -1);
+                gtk_widget_set_size_request (l, FIND_LABELWIDTH + 20, -1);
                 gtk_box_pack_start (GTK_BOX (hb), l, FALSE, FALSE, 0);
             }
 
@@ -719,20 +716,20 @@ sp_find_dialog_old (void)
         } else {
             gtk_window_set_position(GTK_WINDOW(dlg), GTK_WIN_POS_CENTER);
         }
-        if (w && h) 
+        if (w && h)
             gtk_window_resize ((GtkWindow *) dlg, w, h);
-        
+
         sp_transientize (dlg);
         wd.win = dlg;
         wd.stop = 0;
         g_signal_connect   ( G_OBJECT (INKSCAPE), "activate_desktop", G_CALLBACK (sp_transientize_callback), &wd );
-                             
+
         gtk_signal_connect ( GTK_OBJECT (dlg), "event", GTK_SIGNAL_FUNC (sp_dialog_event_handler), dlg);
-                             
+
         gtk_signal_connect ( GTK_OBJECT (dlg), "destroy", G_CALLBACK (sp_find_dialog_destroy), NULL );
         gtk_signal_connect ( GTK_OBJECT (dlg), "delete_event", G_CALLBACK (sp_find_dialog_delete), dlg);
         g_signal_connect   ( G_OBJECT (INKSCAPE), "shut_down", G_CALLBACK (sp_find_dialog_delete), dlg);
-                             
+
         g_signal_connect   ( G_OBJECT (INKSCAPE), "dialogs_hide", G_CALLBACK (sp_dialog_hide), dlg);
         g_signal_connect   ( G_OBJECT (INKSCAPE), "dialogs_unhide", G_CALLBACK (sp_dialog_unhide), dlg);
 
@@ -756,7 +753,7 @@ sp_find_dialog_old (void)
         gtk_box_pack_start (GTK_BOX (vb), types, FALSE, FALSE, 0);
 
         {
-            GtkWidget *w = gtk_hseparator_new (); 
+            GtkWidget *w = gtk_hseparator_new ();
             gtk_widget_show (w);
             gtk_box_pack_start (GTK_BOX (vb), w, FALSE, FALSE, 3);
 
@@ -811,7 +808,7 @@ sp_find_dialog_old (void)
     gtk_widget_show((GtkWidget *) dlg);
     gtk_window_present ((GtkWindow *) dlg);
     sp_find_dialog_reset (NULL, G_OBJECT (dlg));
-    
+
     return dlg;
 }
 
