@@ -6,15 +6,17 @@
  * Authors:
  *   Lauris Kaplinski <lauris@kaplinski.com>
  *   Frank Felfe <innerspace@iname.com>
+ *   bulia byak <buliabyak@users.sf.net>
  *
- * Copyright (C) 1999-2002 authors
+ * Copyright (C) 1999-2005 authors
  * Copyright (C) 2001-2002 Ximian, Inc.
  *
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
-
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
 
 #include <glib.h>
 #include <gdk/gdkkeysyms.h>
@@ -40,34 +42,28 @@
 #include "desktop-style.h"
 #include "document.h"
 
-
 static GtkWidget *dlg = NULL;
 static win_data wd;
 
 // impossible original values to make sure they are read from prefs
-static gint x = -1000, y = -1000, w = 0, h = 0; 
+static gint x = -1000, y = -1000, w = 0, h = 0;
 static gchar *prefs_path = "dialogs.fillstroke";
 
-
 static void sp_fillstroke_selection_modified ( Inkscape::Application *inkscape, SPDesktop *desktop, guint flags, GtkObject *base );
 static void sp_fillstroke_selection_changed ( Inkscape::Application *inkscape, SPDesktop *desktop, GtkObject *base );
 static void sp_fillstroke_opacity_changed (GtkAdjustment *a, SPWidget *dlg);
 
-
 static void
 sp_object_properties_dialog_destroy (GtkObject *object, gpointer data)
 {
     sp_signal_disconnect_by_data (INKSCAPE, dlg);
     wd.win = dlg = NULL;
     wd.stop = 0;
-    
 }
 
-
-
 static gboolean
-sp_object_properties_dialog_delete ( GtkObject *object, 
-                                     GdkEvent *event, 
+sp_object_properties_dialog_delete ( GtkObject *object,
+                                     GdkEvent *event,
                                      gpointer data )
 {
 
@@ -85,34 +81,31 @@ sp_object_properties_dialog_delete ( GtkObject *object,
 
 
 int
-sp_object_properties_page( GtkWidget *nb, 
+sp_object_properties_page( GtkWidget *nb,
                            GtkWidget *page,
-                           char *label, 
-                           char *dlg_name, 
+                           char *label,
+                           char *dlg_name,
                            char *label_image )
 {
-
     GtkWidget *hb, *l, *px;
 
     hb = gtk_hbox_new (FALSE, 0);
     gtk_widget_show (hb);
 
-    px = sp_icon_new_scaled (16, label_image);
+    px = sp_icon_new( GTK_ICON_SIZE_MENU, label_image );
     gtk_widget_show (px);
-    gtk_box_pack_start (GTK_BOX (hb), px, FALSE, FALSE, 0);
+    gtk_box_pack_start (GTK_BOX (hb), px, FALSE, FALSE, 2);
 
-    l = gtk_label_new (_(label));
+    l = gtk_label_new_with_mnemonic (label);
     gtk_widget_show (l);
     gtk_box_pack_start (GTK_BOX (hb), l, FALSE, FALSE, 0);
 
     gtk_widget_show (page);
     gtk_notebook_append_page (GTK_NOTEBOOK (nb), page, hb);
     gtk_object_set_data (GTK_OBJECT (dlg), dlg_name, page);
-    
-    return 0;
-    
-}
 
+    return 0;
+}
 
 void
 sp_object_properties_dialog (void)
@@ -130,7 +123,7 @@ sp_object_properties_dialog (void)
             w = prefs_get_int_attribute (prefs_path, "w", 0);
             h = prefs_get_int_attribute (prefs_path, "h", 0);
         }
-        if (x != 0 || y != 0) 
+        if (x != 0 || y != 0)
             gtk_window_move ((GtkWindow *) dlg, x, y);
         else
             gtk_window_set_position(GTK_WINDOW(dlg), GTK_WIN_POS_CENTER);
@@ -138,21 +131,21 @@ sp_object_properties_dialog (void)
         sp_transientize (dlg);
         wd.win = dlg;
         wd.stop = 0;
-        
+
         g_signal_connect ( G_OBJECT (INKSCAPE), "activate_desktop", G_CALLBACK (sp_transientize_callback), &wd );
-                           
+
         gtk_signal_connect ( GTK_OBJECT (dlg), "event", GTK_SIGNAL_FUNC (sp_dialog_event_handler), dlg );
 
         gtk_signal_connect ( GTK_OBJECT (dlg), "destroy", G_CALLBACK (sp_object_properties_dialog_destroy), dlg );
         gtk_signal_connect ( GTK_OBJECT (dlg), "delete_event", G_CALLBACK (sp_object_properties_dialog_delete), dlg );
         g_signal_connect ( G_OBJECT (INKSCAPE), "shut_down", G_CALLBACK (sp_object_properties_dialog_delete), dlg );
-                           
+
         g_signal_connect ( G_OBJECT (INKSCAPE), "dialogs_hide", G_CALLBACK (sp_dialog_hide), dlg );
         g_signal_connect ( G_OBJECT (INKSCAPE), "dialogs_unhide", G_CALLBACK (sp_dialog_unhide), dlg );
 
         GtkWidget *vb = gtk_vbox_new (FALSE, 0);
         gtk_widget_show (vb);
-        gtk_container_add (GTK_CONTAINER (dlg), vb); 
+        gtk_container_add (GTK_CONTAINER (dlg), vb);
 
         GtkWidget *nb = gtk_notebook_new ();
         gtk_widget_show (nb);
@@ -162,21 +155,21 @@ sp_object_properties_dialog (void)
         /* Fill page */
         {
         GtkWidget *page = sp_fill_style_widget_new ();
-        sp_object_properties_page(nb, page, _("Fill"), "fill",
+        sp_object_properties_page(nb, page, _("_Fill"), "fill",
                     INKSCAPE_STOCK_PROPERTIES_FILL_PAGE);
         }
 
         /* Stroke paint page */
         {
         GtkWidget *page = sp_stroke_style_paint_widget_new ();
-        sp_object_properties_page(nb, page, _("Stroke paint"), "stroke-paint",
+        sp_object_properties_page(nb, page, _("Stroke _paint"), "stroke-paint",
                     INKSCAPE_STOCK_PROPERTIES_STROKE_PAINT_PAGE);
         }
 
         /* Stroke line page */
         {
         GtkWidget *page = sp_stroke_style_line_widget_new ();
-        sp_object_properties_page(nb, page, _("Stroke style"), "stroke-line",
+        sp_object_properties_page(nb, page, _("Stroke st_yle"), "stroke-line",
                     INKSCAPE_STOCK_PROPERTIES_STROKE_PAGE);
         }
 
@@ -187,11 +180,11 @@ sp_object_properties_dialog (void)
     gtk_object_set_data (GTK_OBJECT (dlg), "master_opacity", o_vb);
 
     GtkWidget *l_hb = gtk_hbox_new (FALSE, 4);
-    GtkWidget *l = gtk_label_new (_("Master opacity"));
+    GtkWidget *l = gtk_label_new_with_mnemonic (_("Master _opacity"));
     gtk_misc_set_alignment (GTK_MISC (l), 0.0, 1.0);
     gtk_box_pack_start (GTK_BOX (l_hb), l, FALSE, FALSE, 4);
     gtk_box_pack_start (GTK_BOX (o_vb), l_hb, FALSE, FALSE, 0);
-    
+
     GtkWidget *hb = gtk_hbox_new (FALSE, 4);
     gtk_box_pack_start (GTK_BOX (o_vb), hb, FALSE, FALSE, 0);
 
@@ -201,11 +194,12 @@ sp_object_properties_dialog (void)
     GtkWidget *s = gtk_hscale_new (GTK_ADJUSTMENT (a));
     gtk_scale_set_draw_value (GTK_SCALE (s), FALSE);
     gtk_box_pack_start (GTK_BOX (hb), s, TRUE, TRUE, 4);
+    gtk_label_set_mnemonic_widget (GTK_LABEL(l), s);
 
-    GtkWidget *sb = gtk_spin_button_new (GTK_ADJUSTMENT (a), 0.01, 2);
+    GtkWidget *sb = gtk_spin_button_new (GTK_ADJUSTMENT (a), 0.01, 3);
     gtk_box_pack_start (GTK_BOX (hb), sb, FALSE, FALSE, 0);
 
-    gtk_signal_connect ( a, "value_changed", 
+    gtk_signal_connect ( a, "value_changed",
                          GTK_SIGNAL_FUNC (sp_fillstroke_opacity_changed),
                          dlg );
 
@@ -219,16 +213,15 @@ sp_object_properties_dialog (void)
         sp_fillstroke_selection_changed ( NULL, NULL, NULL );
 
         gtk_widget_show (dlg);
-        
+
     } else {
-        
+
         gtk_window_present (GTK_WINDOW (dlg));
     }
 
 } // end of sp_object_properties_dialog()
 
 
-
 void sp_object_properties_stroke (void)
 {
     GtkWidget *nb;
@@ -240,8 +233,6 @@ void sp_object_properties_stroke (void)
     gtk_notebook_set_page (GTK_NOTEBOOK (nb), 1);
 }
 
-
-
 void sp_object_properties_fill (void)
 {
     GtkWidget *nb;
@@ -251,30 +242,29 @@ void sp_object_properties_fill (void)
     nb = (GtkWidget *)gtk_object_get_data (GTK_OBJECT (dlg), "notebook");
 
     gtk_notebook_set_page (GTK_NOTEBOOK (nb), 0);
-    
 }
 
 
 static void
-sp_fillstroke_selection_modified ( Inkscape::Application *inkscape, 
-                              SPDesktop *desktop, 
+sp_fillstroke_selection_modified ( Inkscape::Application *inkscape,
+                              SPDesktop *desktop,
                               guint flags,
                               GtkObject *base )
 {
-    sp_fillstroke_selection_changed ( NULL, NULL, NULL ); 
+    sp_fillstroke_selection_changed ( NULL, NULL, NULL );
 }
 
 
 static void
-sp_fillstroke_selection_changed ( Inkscape::Application *inkscape, 
-                              SPDesktop *desktop, 
+sp_fillstroke_selection_changed ( Inkscape::Application *inkscape,
+                              SPDesktop *desktop,
                               GtkObject *base )
 {
     if (gtk_object_get_data (GTK_OBJECT (dlg), "blocked"))
         return;
     gtk_object_set_data (GTK_OBJECT (dlg), "blocked", GUINT_TO_POINTER (TRUE));
 
-    SPSelection *sel = SP_DT_SELECTION (SP_ACTIVE_DESKTOP);
+    Inkscape::Selection *sel = SP_DT_SELECTION (SP_ACTIVE_DESKTOP);
 
     GtkWidget *opa = GTK_WIDGET (gtk_object_get_data (GTK_OBJECT (dlg), "master_opacity"));
     GtkAdjustment *a = GTK_ADJUSTMENT(gtk_object_get_data(GTK_OBJECT(dlg), "master_opacity_adjustment"));
@@ -311,8 +301,8 @@ sp_fillstroke_opacity_changed (GtkAdjustment *a, SPWidget *dlg)
     gtk_object_set_data (GTK_OBJECT (dlg), "blocked", GUINT_TO_POINTER (TRUE));
 
     SPCSSAttr *css = sp_repr_css_attr_new ();
-    
-    Inkscape::SVGOStringStream os;	
+
+    Inkscape::SVGOStringStream os;
     os << CLAMP (a->value, 0.0, 1.0);
     sp_repr_css_set_property (css, "opacity", os.str().c_str());
 
@@ -323,7 +313,7 @@ sp_fillstroke_opacity_changed (GtkAdjustment *a, SPWidget *dlg)
     sp_document_maybe_done (SP_DT_DOCUMENT (SP_ACTIVE_DESKTOP), "fillstroke:opacity");
 
     gtk_object_set_data (GTK_OBJECT (dlg), "blocked", GUINT_TO_POINTER (FALSE));
-} 
+}
 
 
 /*
