@@ -6,9 +6,10 @@
  * Authors:
  *   Kees Cook <kees@outflux.net>
  *
- * Copyright (C) Kees Cook 2004
+ * Copyright (C) 2004 Kees Cook <kees@outflux.net>
  *
  * Released under GNU GPL, read the file 'COPYING' for more information
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -166,6 +167,16 @@ struct rdf_double_t rdf_license_pd [] = {
     { NULL, NULL }
 };
 
+struct rdf_double_t rdf_license_freeart [] = {
+    { "cc:permits", "http://web.resource.org/cc/Reproduction", },
+    { "cc:permits", "http://web.resource.org/cc/Distribution", },
+    { "cc:permits", "http://web.resource.org/cc/DerivativeWorks", },
+    { "cc:requires", "http://web.resource.org/cc/ShareAlike", },
+    { "cc:requires", "http://web.resource.org/cc/Notice", },
+    { "cc:requires", "http://web.resource.org/cc/Attribution", },
+    { NULL, NULL }
+};
+
 struct rdf_license_t rdf_licenses [] = {
     { "CC Attribution", 
       "http://creativecommons.org/licenses/by/2.0/",
@@ -173,27 +184,27 @@ struct rdf_license_t rdf_licenses [] = {
     },
 
     { "CC Attribution-ShareAlike", 
-      "http://creativecommons.org/licenses/by/2.0/",
+      "http://creativecommons.org/licenses/by-sa/2.0/",
       rdf_license_cc_a_sa,
     },
 
     { "CC Attribution-NoDerivs", 
-      "http://creativecommons.org/licenses/by/2.0/",
+      "http://creativecommons.org/licenses/by-nd/2.0/",
       rdf_license_cc_a_nd,
     },
 
     { "CC Attribution-NonCommercial", 
-      "http://creativecommons.org/licenses/by/2.0/",
+      "http://creativecommons.org/licenses/by-nc/2.0/",
       rdf_license_cc_a_nc,
     },
 
     { "CC Attribution-NonCommercial-ShareAlike", 
-      "http://creativecommons.org/licenses/by/2.0/",
+      "http://creativecommons.org/licenses/by-nc-sa/2.0/",
       rdf_license_cc_a_nc_sa,
     },
 
     { "CC Attribution-NonCommercial-NoDerivs", 
-      "http://creativecommons.org/licenses/by/2.0/",
+      "http://creativecommons.org/licenses/by-nc-nd/2.0/",
       rdf_license_cc_a_nc_nd,
     },
 
@@ -212,6 +223,11 @@ struct rdf_license_t rdf_licenses [] = {
       rdf_license_pd,
     },
 
+    { "FreeArt",
+      "http://artlibre.org/licence.php/lalgb.html",
+      rdf_license_freeart,
+    },
+
     { NULL, NULL, rdf_license_empty, }
 };
 
@@ -437,20 +453,20 @@ rdf_string(struct rdf_t * rdf)
  *  
  */
 const gchar *
-rdf_get_repr_text ( SPRepr * repr, struct rdf_work_entity_t * entity )
+rdf_get_repr_text ( Inkscape::XML::Node * repr, struct rdf_work_entity_t * entity )
 {
     g_return_val_if_fail (repr != NULL, NULL);
     g_return_val_if_fail (entity != NULL, NULL);
     static gchar * bag = NULL;
     gchar * holder = NULL;
 
-    SPRepr * temp=NULL;
+    Inkscape::XML::Node * temp=NULL;
     switch (entity->datatype) {
         case RDF_CONTENT:
             temp = sp_repr_children(repr);
             if ( temp == NULL ) return NULL;
             
-            return sp_repr_content(temp);
+            return temp->content();
 
         case RDF_AGENT:
             temp = sp_repr_lookup_name ( repr, "cc:Agent", 1 );
@@ -462,10 +478,10 @@ rdf_get_repr_text ( SPRepr * repr, struct rdf_work_entity_t * entity )
             temp = sp_repr_children(temp);
             if ( temp == NULL ) return NULL;
 
-            return sp_repr_content(temp);
+            return temp->content();
 
         case RDF_RESOURCE:
-            return sp_repr_attr(repr, "rdf:resource");
+            return repr->attribute("rdf:resource");
 
         case RDF_XML:
             return "xml goes here";
@@ -481,13 +497,13 @@ rdf_get_repr_text ( SPRepr * repr, struct rdf_work_entity_t * entity )
                 temp = sp_repr_children(repr);
                 if ( temp == NULL ) return NULL;
             
-                return sp_repr_content(temp);
+                return temp->content();
             }
 
             for ( temp = sp_repr_children(temp) ;
                   temp ;
                   temp = sp_repr_next(temp) ) {
-                if (!strcmp(sp_repr_name(temp),"rdf:li") &&
+                if (!strcmp(temp->name(),"rdf:li") &&
                     temp->firstChild()) {
                     const gchar * str = temp->firstChild()->content();
                     if (bag) {
@@ -509,7 +525,7 @@ rdf_get_repr_text ( SPRepr * repr, struct rdf_work_entity_t * entity )
 }
 
 unsigned int
-rdf_set_repr_text ( SPRepr * repr,
+rdf_set_repr_text ( Inkscape::XML::Node * repr,
                     struct rdf_work_entity_t * entity,
                     gchar const * text )
 {
@@ -520,9 +536,9 @@ rdf_set_repr_text ( SPRepr * repr,
     gchar** strlist = NULL;
     int i;
 
-    SPRepr * temp=NULL;
-    SPRepr * child=NULL;
-    SPRepr * parent=repr;
+    Inkscape::XML::Node * temp=NULL;
+    Inkscape::XML::Node * child=NULL;
+    Inkscape::XML::Node * parent=repr;
     switch (entity->datatype) {
         case RDF_CONTENT:
             temp = sp_repr_children(parent);
@@ -530,13 +546,14 @@ rdf_set_repr_text ( SPRepr * repr,
                 temp = sp_repr_new_text( text );
                 g_return_val_if_fail (temp != NULL, 0);
 
-                sp_repr_append_child ( parent, temp );
+                parent->appendChild(temp);
                 sp_repr_unref ( temp );
 
                 return TRUE;
             }
             else {
-                return sp_repr_set_content( temp, text );
+                temp->setContent(text);
+				return TRUE;
             }
 
         case RDF_AGENT:
@@ -545,7 +562,7 @@ rdf_set_repr_text ( SPRepr * repr,
                 temp = sp_repr_new ( "cc:Agent" );
                 g_return_val_if_fail (temp != NULL, 0);
 
-                sp_repr_append_child ( parent, temp );
+                parent->appendChild(temp);
                 sp_repr_unref ( temp );
             }
             parent = temp;
@@ -555,7 +572,7 @@ rdf_set_repr_text ( SPRepr * repr,
                 temp = sp_repr_new ( "dc:title" );
                 g_return_val_if_fail (temp != NULL, 0);
 
-                sp_repr_append_child ( parent, temp );
+                parent->appendChild(temp);
                 sp_repr_unref ( temp );
             }
             parent = temp;
@@ -565,13 +582,14 @@ rdf_set_repr_text ( SPRepr * repr,
                 temp = sp_repr_new_text( text );
                 g_return_val_if_fail (temp != NULL, 0);
 
-                sp_repr_append_child ( parent, temp );
+                parent->appendChild(temp);
                 sp_repr_unref ( temp );
 
                 return TRUE;
             }
             else {
-                return sp_repr_set_content( temp, text );
+                temp->setContent(text);
+				return TRUE;
             }
 
         case RDF_RESOURCE:
@@ -592,7 +610,7 @@ rdf_set_repr_text ( SPRepr * repr,
                 temp = sp_repr_new ( "rdf:Bag" );
                 g_return_val_if_fail (temp != NULL, 0);
 
-                sp_repr_append_child ( parent, temp );
+                parent->appendChild(temp);
                 sp_repr_unref ( temp );
             }
             parent = temp;
@@ -609,13 +627,13 @@ rdf_set_repr_text ( SPRepr * repr,
                 temp = sp_repr_new ( "rdf:li" );
                 g_return_val_if_fail (temp != NULL, 0);
 
-                sp_repr_append_child ( parent, temp );
+                parent->appendChild(temp);
                 sp_repr_unref ( temp );
 
                 child = sp_repr_new_text( g_strstrip(str) );
                 g_return_val_if_fail (child != NULL, 0);
 
-                sp_repr_append_child ( temp, child );
+                temp->appendChild(child);
                 sp_repr_unref ( child );
             }
             g_strfreev( strlist );
@@ -628,34 +646,34 @@ rdf_set_repr_text ( SPRepr * repr,
     return 0;
 }
 
-SPRepr *
+Inkscape::XML::Node *
 rdf_get_rdf_root_repr ( SPDocument * doc, bool build )
 {
     g_return_val_if_fail (doc        != NULL, NULL);
     g_return_val_if_fail (doc->rroot != NULL, NULL);
 
-    SPRepr * rdf = sp_repr_lookup_name ( doc->rroot, XML_TAG_NAME_RDF );
+    Inkscape::XML::Node * rdf = sp_repr_lookup_name ( doc->rroot, XML_TAG_NAME_RDF );
 
     if (rdf == NULL) {
         //printf("missing XML '%s'\n",XML_TAG_NAME_RDF);
         if (!build) return NULL;
 
-        SPRepr * svg = sp_repr_lookup_name ( doc->rroot, XML_TAG_NAME_SVG );
+        Inkscape::XML::Node * svg = sp_repr_lookup_name ( doc->rroot, XML_TAG_NAME_SVG );
         g_return_val_if_fail ( svg != NULL, NULL );
 
-        SPRepr * parent = sp_repr_lookup_name ( svg, XML_TAG_NAME_METADATA );
+        Inkscape::XML::Node * parent = sp_repr_lookup_name ( svg, XML_TAG_NAME_METADATA );
         if ( parent == NULL ) {
             parent = sp_repr_new( XML_TAG_NAME_METADATA );
             g_return_val_if_fail ( parent != NULL, NULL);
 
-            sp_repr_append_child(svg, parent);
+            svg->appendChild(parent);
             sp_repr_unref(parent);
         }
 
         rdf = sp_repr_new( XML_TAG_NAME_RDF );
         g_return_val_if_fail (rdf != NULL, NULL);
 
-        sp_repr_append_child(parent, rdf);
+        parent->appendChild(rdf);
         sp_repr_unref(rdf);
     }
 
@@ -663,37 +681,37 @@ rdf_get_rdf_root_repr ( SPDocument * doc, bool build )
      * some implementations do not put RDF stuff inside <metadata>,
      * so we need to check for it and add it if we don't see it
      */
-    SPRepr * want_metadata = sp_repr_parent ( rdf );
+    Inkscape::XML::Node * want_metadata = sp_repr_parent ( rdf );
     g_return_val_if_fail (want_metadata != NULL, NULL);
-    if (strcmp( sp_repr_name(want_metadata), XML_TAG_NAME_METADATA )) {
-            SPRepr * metadata = sp_repr_new( XML_TAG_NAME_METADATA );
+    if (strcmp( want_metadata->name(), XML_TAG_NAME_METADATA )) {
+            Inkscape::XML::Node * metadata = sp_repr_new( XML_TAG_NAME_METADATA );
             g_return_val_if_fail (metadata != NULL, NULL);
 
             /* attach the metadata node */
-            sp_repr_append_child ( want_metadata, metadata );
+            want_metadata->appendChild(metadata);
             sp_repr_unref ( metadata );
 
             /* move the RDF into it */
             sp_repr_ref ( rdf );
             sp_repr_unparent ( rdf );
-            sp_repr_append_child ( metadata, rdf );
+            metadata->appendChild(rdf);
             sp_repr_unref ( rdf );
     }
     
     return rdf;
 }
 
-SPRepr *
+Inkscape::XML::Node *
 rdf_get_xml_repr( SPDocument * doc, gchar const * name, bool build )
 {
     g_return_val_if_fail (name       != NULL, NULL);
     g_return_val_if_fail (doc        != NULL, NULL);
     g_return_val_if_fail (doc->rroot != NULL, NULL);
 
-    SPRepr * rdf = rdf_get_rdf_root_repr ( doc, build );
+    Inkscape::XML::Node * rdf = rdf_get_rdf_root_repr ( doc, build );
     if (!rdf) return NULL;
 
-    SPRepr * xml = sp_repr_lookup_name ( rdf, name );
+    Inkscape::XML::Node * xml = sp_repr_lookup_name ( rdf, name );
     if (xml == NULL) {
         //printf("missing XML '%s'\n",name);
         if (!build) return NULL;
@@ -703,24 +721,24 @@ rdf_get_xml_repr( SPDocument * doc, gchar const * name, bool build )
 
         sp_repr_set_attr ( xml, "rdf:about", "" );
 
-        sp_repr_append_child(rdf, xml);
+        rdf->appendChild(xml);
         sp_repr_unref(xml);
     }
 
     return xml;
 }
 
-SPRepr *
+Inkscape::XML::Node *
 rdf_get_work_repr( SPDocument * doc, gchar const * name, bool build )
 {
     g_return_val_if_fail (name       != NULL, NULL);
     g_return_val_if_fail (doc        != NULL, NULL);
     g_return_val_if_fail (doc->rroot != NULL, NULL);
 
-    SPRepr * work = rdf_get_xml_repr ( doc, XML_TAG_NAME_WORK, build );
+    Inkscape::XML::Node * work = rdf_get_xml_repr ( doc, XML_TAG_NAME_WORK, build );
     if (!work) return NULL;
 
-    SPRepr * item = sp_repr_lookup_name ( work, name, 1 );
+    Inkscape::XML::Node * item = sp_repr_lookup_name ( work, name, 1 );
     if (item == NULL) {
         //printf("missing XML '%s'\n",name);
         if (!build) return NULL;
@@ -728,7 +746,7 @@ rdf_get_work_repr( SPDocument * doc, gchar const * name, bool build )
         item = sp_repr_new( name );
         g_return_val_if_fail (item != NULL, NULL);
 
-        sp_repr_append_child(work, item);
+        work->appendChild(item);
         sp_repr_unref(item);
     }
 
@@ -750,7 +768,7 @@ rdf_get_work_entity(SPDocument * doc, struct rdf_work_entity_t * entity)
     if ( entity == NULL ) return NULL;
     //printf("want '%s'\n",entity->title);
 
-    SPRepr * item;
+    Inkscape::XML::Node * item;
     if ( entity->datatype == RDF_XML ) {
         item = rdf_get_xml_repr ( doc, entity->tag, FALSE );
     }
@@ -787,7 +805,7 @@ rdf_set_work_entity(SPDocument * doc, struct rdf_work_entity_t * entity,
         text);
     */
 
-    SPRepr * item = rdf_get_work_repr( doc, entity->tag, TRUE );
+    Inkscape::XML::Node * item = rdf_get_work_repr( doc, entity->tag, TRUE );
     g_return_val_if_fail ( item != NULL, 0 );
 
     return rdf_set_repr_text ( item, entity, text );
@@ -796,7 +814,7 @@ rdf_set_work_entity(SPDocument * doc, struct rdf_work_entity_t * entity,
 #undef DEBUG_MATCH
 
 bool
-rdf_match_license ( SPRepr * repr, struct rdf_license_t * license )
+rdf_match_license ( Inkscape::XML::Node * repr, struct rdf_license_t * license )
 {
     g_assert ( repr != NULL );
     g_assert ( license != NULL );
@@ -813,15 +831,15 @@ rdf_match_license ( SPRepr * repr, struct rdf_license_t * license )
     }
     bool * matched = (bool*)calloc(count,sizeof(bool));
 
-    for (SPRepr * current = sp_repr_children ( repr );
+    for (Inkscape::XML::Node * current = sp_repr_children ( repr );
          current;
          current = sp_repr_next ( current ) ) {
 
-        gchar const * attr = sp_repr_attr ( current, "rdf:resource" );
+        gchar const * attr = current->attribute("rdf:resource");
         if ( attr == NULL ) continue;
 
 #ifdef DEBUG_MATCH
-        printf("\texamining '%s' => '%s'\n", sp_repr_name ( current ), attr);
+        printf("\texamining '%s' => '%s'\n", current->name(), attr);
 #endif
 
         bool found_match=FALSE;
@@ -830,11 +848,11 @@ rdf_match_license ( SPRepr * repr, struct rdf_license_t * license )
             if (matched[i]) continue;
 
 #ifdef DEBUG_MATCH
-            printf("\t\t'%s' vs '%s'\n", sp_repr_name ( current ), license->details[i].name);
+            printf("\t\t'%s' vs '%s'\n", current->name(), license->details[i].name);
             printf("\t\t'%s' vs '%s'\n", attr, license->details[i].resource);
 #endif
 
-            if (!strcmp( sp_repr_name ( current ),
+            if (!strcmp( current->name(),
                          license->details[i].name ) &&
                 !strcmp( attr,
                          license->details[i].resource )) {
@@ -889,7 +907,7 @@ rdf_match_license ( SPRepr * repr, struct rdf_license_t * license )
 struct rdf_license_t *
 rdf_get_license(SPDocument * document)
 {
-    SPRepr * repr = rdf_get_xml_repr ( document, XML_TAG_NAME_LICENSE, FALSE );
+    Inkscape::XML::Node * repr = rdf_get_xml_repr ( document, XML_TAG_NAME_LICENSE, FALSE );
     if (repr) {
         for (struct rdf_license_t * license = rdf_licenses;
              license->name; license++ ) {
@@ -914,7 +932,7 @@ void
 rdf_set_license(SPDocument * document, struct rdf_license_t * license)
 {
     // drop old license section
-    SPRepr * repr = rdf_get_xml_repr ( document, XML_TAG_NAME_LICENSE, FALSE );
+    Inkscape::XML::Node * repr = rdf_get_xml_repr ( document, XML_TAG_NAME_LICENSE, FALSE );
     if (repr) sp_repr_unparent(repr);
 
     if (!license) return;
@@ -927,11 +945,11 @@ rdf_set_license(SPDocument * document, struct rdf_license_t * license)
 
     for (struct rdf_double_t * detail = license->details;
          detail->name; detail++) {
-        SPRepr * child = sp_repr_new( detail->name );
+        Inkscape::XML::Node * child = sp_repr_new( detail->name );
         g_assert ( child != NULL );
 
         sp_repr_set_attr ( child, "rdf:resource", detail->resource );
-        sp_repr_append_child ( repr, child );
+        repr->appendChild(child);
         sp_repr_unref ( child );
     }
 }
@@ -955,7 +973,7 @@ rdf_set_defaults ( SPDocument * document )
     if (!sp_item_group_get_child_by_name ((SPGroup *) document->root, NULL,
                                           XML_TAG_NAME_METADATA)) {
         // create repr
-        SPRepr * rnew = sp_repr_new (XML_TAG_NAME_METADATA);
+        Inkscape::XML::Node * rnew = sp_repr_new (XML_TAG_NAME_METADATA);
         // insert into the document
         sp_repr_add_child (document->rroot, rnew, NULL);
         // clean up
