@@ -5,6 +5,7 @@
  *
  * Author:
  *   Lauris Kaplinski <lauris@ximian.com>
+ *   bulia byak <buliabyak@users.sf.net>
  *
  * Copyright (C) 1999-2002 Lauris Kaplinski
  * Copyright (C) 2000-2001 Ximian, Inc.
@@ -12,7 +13,9 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
 
 #include <string.h>
 #include <libnrtype/FontFactory.h>
@@ -40,6 +43,8 @@ extern "C" {
 #include "../selection.h"
 #include "../style.h"
 #include "../sp-text.h"
+#include "../sp-flowtext.h"
+#include "../text-editing.h"
 #include "../inkscape-stock.h"
 #include <libnrtype/font-style-to-pos.h>
 
@@ -55,8 +60,9 @@ extern "C" {
 
 #define VB_MARGIN 4
 
-static void sp_text_edit_dialog_modify_selection (Inkscape::Application *inkscape, SPSelection *sel, guint flags, GtkWidget *dlg);
-static void sp_text_edit_dialog_change_selection (Inkscape::Application *inkscape, SPSelection *sel, GtkWidget *dlg);
+static void sp_text_edit_dialog_selection_modified (Inkscape::Application *inkscape, Inkscape::Selection *sel, guint flags, GtkWidget *dlg);
+static void sp_text_edit_dialog_selection_changed (Inkscape::Application *inkscape, Inkscape::Selection *sel, GtkWidget *dlg);
+static void sp_text_edit_dialog_subselection_changed ( Inkscape::Application *inkscape, SPDesktop *desktop, GtkWidget *dlg);
 
 static void sp_text_edit_dialog_set_default (GtkButton *button, GtkWidget *dlg);
 static void sp_text_edit_dialog_apply (GtkButton *button, GtkWidget *dlg);
@@ -69,7 +75,7 @@ static void sp_text_edit_dialog_font_changed (SPFontSelector *fontsel, font_inst
 static void sp_text_edit_dialog_any_toggled (GtkToggleButton *tb, GtkWidget *dlg);
 static void sp_text_edit_dialog_line_spacing_changed (GtkEditable *editable, GtkWidget *dlg);
 
-static SPText *sp_ted_get_selected_text_item (void);
+static SPItem *sp_ted_get_selected_text_item (void);
 static unsigned sp_ted_get_selected_text_count (void);
 
 
@@ -267,7 +273,7 @@ sp_text_edit_dialog (void)
 
                     // horizontal
                     {
-                        GtkWidget *px = sp_icon_new_scaled ( SP_ICON_SIZE_NOTEBOOK,
+                        GtkWidget *px = sp_icon_new( GTK_ICON_SIZE_LARGE_TOOLBAR,
                                                       INKSCAPE_STOCK_WRITING_MODE_LR );
                         GtkWidget *b = group = gtk_radio_button_new (NULL);
                         gtk_tooltips_set_tip (tt, b, _("Horizontal text"), NULL);
@@ -281,7 +287,7 @@ sp_text_edit_dialog (void)
 
                     // vertical
                     {
-                        GtkWidget *px = sp_icon_new_scaled ( SP_ICON_SIZE_NOTEBOOK,
+                        GtkWidget *px = sp_icon_new( GTK_ICON_SIZE_LARGE_TOOLBAR,
                                                       INKSCAPE_STOCK_WRITING_MODE_TB );
                         GtkWidget *b = gtk_radio_button_new (gtk_radio_button_group (GTK_RADIO_BUTTON (group)));
                         gtk_tooltips_set_tip (tt, b, _("Vertical text"), NULL);
@@ -313,7 +319,7 @@ sp_text_edit_dialog (void)
                     gtk_combo_set_value_in_list ((GtkCombo *) c, FALSE, FALSE);
                     gtk_combo_set_use_arrows ((GtkCombo *) c, TRUE);
                     gtk_combo_set_use_arrows_always ((GtkCombo *) c, TRUE);
-                    gtk_widget_set_size_request (c, 64, -1);
+                    gtk_widget_set_size_request (c, 90, -1);
 
                     { /* Setup strings */
                         GList *sl = NULL;
@@ -359,6 +365,7 @@ sp_text_edit_dialog (void)
 
             GtkTextBuffer *tb = gtk_text_buffer_new (NULL);
             GtkWidget *txt = gtk_text_view_new_with_buffer (tb);
+            gtk_text_view_set_wrap_mode ((GtkTextView *) txt, GTK_WRAP_WORD);
 #ifdef WITH_GTKSPELL
             GError *error = NULL;
             char *errortext = NULL;
@@ -415,9 +422,10 @@ sp_text_edit_dialog (void)
         }
 
         g_signal_connect ( G_OBJECT (INKSCAPE), "modify_selection", 
-                           G_CALLBACK (sp_text_edit_dialog_modify_selection), dlg);
+                           G_CALLBACK (sp_text_edit_dialog_selection_modified), dlg);
         g_signal_connect ( G_OBJECT (INKSCAPE), "change_selection", 
-                           G_CALLBACK (sp_text_edit_dialog_change_selection), dlg);
+                           G_CALLBACK (sp_text_edit_dialog_selection_changed), dlg);
+        g_signal_connect (INKSCAPE, "change_subselection", G_CALLBACK (sp_text_edit_dialog_subselection_changed), dlg);
 
         gtk_widget_show_all (dlg);
 
@@ -431,8 +439,8 @@ sp_text_edit_dialog (void)
 
 
 static void
-sp_text_edit_dialog_modify_selection ( Inkscape::Application *inkscape, 
-                                       SPSelection *sel, 
+sp_text_edit_dialog_selection_modified ( Inkscape::Application *inkscape, 
+                                       Inkscape::Selection *sel, 
                                        guint flags, 
                                        GtkWidget *dlg )
 {
@@ -448,21 +456,25 @@ sp_text_edit_dialog_modify_selection ( Inkscape::Application *inkscape,
     
     sp_text_edit_dialog_read_selection (dlg, style, content);
 
-} // end of sp_text_edit_dialog_modify_selection()
+} 
 
 
 
 static void
-sp_text_edit_dialog_change_selection ( Inkscape::Application *inkscape, 
-                                       SPSelection *sel, 
+sp_text_edit_dialog_selection_changed ( Inkscape::Application *inkscape, 
+                                       Inkscape::Selection *sel, 
                                        GtkWidget *dlg )
 {
     sp_text_edit_dialog_read_selection (dlg, TRUE, TRUE);
 }
 
+static void sp_text_edit_dialog_subselection_changed ( Inkscape::Application *inkscape, SPDesktop *desktop, GtkWidget *dlg )
+{
+    sp_text_edit_dialog_read_selection (dlg, TRUE, FALSE);
+}
 
 static void
-sp_text_edit_dialog_update_object_text ( SPText *text )
+sp_text_edit_dialog_update_object_text ( SPItem *text )
 {
         GtkTextBuffer *tb;
         GtkTextIter start, end;
@@ -474,7 +486,7 @@ sp_text_edit_dialog_update_object_text ( SPText *text )
         if (gtk_text_buffer_get_modified (tb)) {
             gtk_text_buffer_get_bounds (tb, &start, &end);
             str = gtk_text_buffer_get_text (tb, &start, &end, TRUE);
-            sp_text_set_repr_text_multiline (text, str);
+            sp_te_set_repr_text_multiline (text, str);
             g_free (str);    
             gtk_text_buffer_set_modified (tb, FALSE);
         }
@@ -520,13 +532,16 @@ sp_get_text_dialog_style ()
         
         if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (b))) {
             sp_repr_css_set_property (css, "text-anchor", "start");
+            sp_repr_css_set_property (css, "text-align", "start");
         } else {
             b = (GtkWidget*)g_object_get_data ( G_OBJECT (dlg), 
                                                 "text_anchor_middle");
             if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (b))) {
                 sp_repr_css_set_property (css, "text-anchor", "middle");
+                sp_repr_css_set_property (css, "text-align", "center");
             } else {
                 sp_repr_css_set_property (css, "text-anchor", "end");
+                sp_repr_css_set_property (css, "text-align", "end");
             }
         }
         
@@ -573,26 +588,27 @@ sp_text_edit_dialog_apply (GtkButton *button, GtkWidget *dlg)
 
     GtkWidget *apply = (GtkWidget*)g_object_get_data (G_OBJECT (dlg), "apply");
     GtkWidget *def = (GtkWidget*)g_object_get_data (G_OBJECT (dlg), "default");
+    SPDesktop *desktop = SP_ACTIVE_DESKTOP;
 
     unsigned items = 0;
-    const GSList *item_list = SP_DT_SELECTION(SP_ACTIVE_DESKTOP)->itemList();
-    bool text_changed = false, style_changed = false;
+    const GSList *item_list = SP_DT_SELECTION(desktop)->itemList();
 
     SPCSSAttr *css = sp_get_text_dialog_style ();
 
+    sp_desktop_set_style(desktop, css, true);
+
     for (; item_list != NULL; item_list = item_list->next) { 
         // apply style to the reprs of all text objects in the selection
         if (SP_IS_TEXT (item_list->data)) {
-            SPText *text = SP_TEXT(item_list->data);
 
-        // FIXME: use sp_desktop_set_style instead!
-        sp_desktop_apply_css_recursive (SP_OBJECT(text), css, true);
-        // backwards compatibility:
-        sp_repr_set_attr (SP_OBJECT_REPR(text), "sodipodi:linespacing", sp_repr_css_property (css, "line-height", NULL));
+            // backwards compatibility:
+            sp_repr_set_attr (SP_OBJECT_REPR(item_list->data), "sodipodi:linespacing", sp_repr_css_property (css, "line-height", NULL));
 
             ++items;
-            style_changed = true;
         }
+        else if (SP_IS_FLOWTEXT (item_list->data))
+            // no need to set sodipodi:linespacing, because Inkscape never supported it on flowtext
+            ++items;
     }
     
     if (items == 0) { 
@@ -602,16 +618,13 @@ sp_text_edit_dialog_apply (GtkButton *button, GtkWidget *dlg)
     } else if (items == 1) {
         /* exactly one text object; now set its text, too */
         SPItem *item = SP_DT_SELECTION(SP_ACTIVE_DESKTOP)->singleItem();
-        if (SP_IS_TEXT (item)) {
-            sp_text_edit_dialog_update_object_text (SP_TEXT(item));
-            text_changed = true;
+        if (SP_IS_TEXT (item) || SP_IS_FLOWTEXT(item)) {
+            sp_text_edit_dialog_update_object_text (item);
         }
     } 
 
     // complete the transaction
-    if (style_changed || text_changed) {
-        sp_document_done (SP_DT_DOCUMENT (SP_ACTIVE_DESKTOP));
-    }
+    sp_document_done (SP_DT_DOCUMENT (SP_ACTIVE_DESKTOP));
 
     gtk_widget_set_sensitive (apply, FALSE);
 
@@ -628,8 +641,6 @@ sp_text_edit_dialog_close (GtkButton *button, GtkWidget *dlg)
     gtk_widget_destroy (GTK_WIDGET (dlg));
 }
 
-
-
 static void
 sp_text_edit_dialog_read_selection ( GtkWidget *dlg, 
                                      gboolean dostyle, 
@@ -649,10 +660,9 @@ sp_text_edit_dialog_read_selection ( GtkWidget *dlg,
 
     GtkTextBuffer *tb = (GtkTextBuffer*)g_object_get_data (G_OBJECT (dlg), "text");
 
-    SPText *text = sp_ted_get_selected_text_item ();
+    SPItem *text = sp_ted_get_selected_text_item ();
 
-    SPRepr *repr;
-    SPStyle *style;
+    Inkscape::XML::Node *repr;
     if (text)
     {
         guint items = sp_ted_get_selected_text_count ();
@@ -663,11 +673,10 @@ sp_text_edit_dialog_read_selection ( GtkWidget *dlg,
         }
         gtk_widget_set_sensitive (apply, FALSE);
         gtk_widget_set_sensitive (def, TRUE);
-        style = SP_OBJECT_STYLE (text);
 
         if (docontent) {
             gchar *str;
-            str = sp_text_get_string_multiline (text);
+            str = sp_te_get_string_multiline (text);
 
             if (str) {
                 int pos;
@@ -691,77 +700,81 @@ sp_text_edit_dialog_read_selection ( GtkWidget *dlg,
         gtk_widget_set_sensitive (textw, FALSE);
         gtk_widget_set_sensitive (apply, FALSE);
         gtk_widget_set_sensitive (def, FALSE);
-        repr = inkscape_get_repr (INKSCAPE, "tools.text");
-        if (repr) {
-            gtk_widget_set_sensitive (notebook, TRUE);
-            style = sp_style_new ();
-            sp_style_read_from_repr (style, repr);
-        } else {
-            gtk_widget_set_sensitive (notebook, FALSE);
-            style = sp_style_new ();
-        }
     }
 
     if (dostyle) {
-        GtkWidget *b, *combo;
-        const gchar *sstr;
 
-        font_instance *font = (font_factory::Default())->Face ( style->text->font_family.value, font_style_to_pos(*style) );
+        // create temporary style
+        SPStyle *query = sp_style_new ();
+        // query style from desktop into it. This returns a result flag and fills query with the style of subselection, if any, or selection
+        int result_family = sp_desktop_query_style (SP_ACTIVE_DESKTOP, query, QUERY_STYLE_PROPERTY_FONTFAMILY); 
+        int result_style = sp_desktop_query_style (SP_ACTIVE_DESKTOP, query, QUERY_STYLE_PROPERTY_FONTSTYLE); 
+        int result_numbers = sp_desktop_query_style (SP_ACTIVE_DESKTOP, query, QUERY_STYLE_PROPERTY_FONTNUMBERS); 
 
+        // If querying returned nothing, read the style from the text tool prefs (default style for new texts)
+        if (result_family == QUERY_STYLE_NOTHING || result_style == QUERY_STYLE_NOTHING || result_numbers == QUERY_STYLE_NOTHING) {
+            repr = inkscape_get_repr (INKSCAPE, "tools.text");
+            if (repr) {
+                gtk_widget_set_sensitive (notebook, TRUE);
+                sp_style_read_from_repr (query, repr);
+            } else {
+                gtk_widget_set_sensitive (notebook, FALSE);
+            }
+        }
+
+        // FIXME: process result_family/style == QUERY_STYLE_MULTIPLE_DIFFERENT by showing "Many" in the lists
+        font_instance *font = (font_factory::Default())->Face ( query->text->font_family.value, font_style_to_pos(*query) );
         if (font) {
             // the font is oversized, so we need to pass the true size separately
-            sp_font_selector_set_font (SP_FONT_SELECTOR (fontsel), font, style->font_size.computed);
+            sp_font_selector_set_font (SP_FONT_SELECTOR (fontsel), font, query->font_size.computed);
             sp_font_preview_set_font (SP_FONT_PREVIEW (preview), font, SP_FONT_SELECTOR(fontsel));
 						font->Unref();
 						font=NULL;
         }
 
-        if (style->text_anchor.computed == SP_CSS_TEXT_ANCHOR_START) {
-            b = (GtkWidget*)g_object_get_data ( G_OBJECT (dlg), 
-                                                "text_anchor_start" );
-        } else if (style->text_anchor.computed == SP_CSS_TEXT_ANCHOR_MIDDLE) {
-            b = (GtkWidget*)g_object_get_data ( G_OBJECT (dlg), 
-                                                "text_anchor_middle" );
+        GtkWidget *b;
+        if (query->text_anchor.computed == SP_CSS_TEXT_ANCHOR_START) {
+            b = (GtkWidget*)g_object_get_data ( G_OBJECT (dlg), "text_anchor_start" );
+        } else if (query->text_anchor.computed == SP_CSS_TEXT_ANCHOR_MIDDLE) {
+            b = (GtkWidget*)g_object_get_data ( G_OBJECT (dlg), "text_anchor_middle" );
         } else {
-            b = (GtkWidget*)g_object_get_data ( G_OBJECT (dlg), 
-                                                "text_anchor_end" );
+            b = (GtkWidget*)g_object_get_data ( G_OBJECT (dlg), "text_anchor_end" );
         }
+        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (b), TRUE);
+       
+        if (query->writing_mode.computed == SP_CSS_WRITING_MODE_LR_TB) {
+            b = (GtkWidget*)g_object_get_data ( G_OBJECT (dlg), INKSCAPE_STOCK_WRITING_MODE_LR );
+        } else {
+            b = (GtkWidget*)g_object_get_data ( G_OBJECT (dlg), INKSCAPE_STOCK_WRITING_MODE_TB );
+        }
+        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (b), TRUE);
 
-        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (b), TRUE);
-        
-        if (style->writing_mode.computed == SP_CSS_WRITING_MODE_LR) {
-            b = (GtkWidget*)g_object_get_data ( G_OBJECT (dlg), 
-                                                INKSCAPE_STOCK_WRITING_MODE_LR );
-        } else {
-            b = (GtkWidget*)g_object_get_data ( G_OBJECT (dlg), 
-                                                INKSCAPE_STOCK_WRITING_MODE_TB );
-        }
-        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (b), TRUE);
-        combo = (GtkWidget*)g_object_get_data ( G_OBJECT (dlg), 
-                                                "line_spacing" );
-        sstr = (repr) ? sp_repr_attr (repr, "sodipodi:linespacing") : NULL;
-        gtk_entry_set_text ((GtkEntry *) ((GtkCombo *) (combo))->entry, 
-                    (sstr) ? sstr : (const gchar *) "100%");
+        GtkWidget *combo = (GtkWidget*)g_object_get_data ( G_OBJECT (dlg), "line_spacing" );
+        double height;
+        if (query->line_height.normal) height = Inkscape::Text::Layout::LINE_HEIGHT_NORMAL;
+        else height = query->line_height.computed;
+        gchar *sstr = g_strdup_printf ("%d%%", (int) floor(height * 100 + 0.5));
+        gtk_entry_set_text ((GtkEntry *) ((GtkCombo *) (combo))->entry, sstr);
+        g_free(sstr);
+
+        g_free (query);
     }
 
     g_object_set_data (G_OBJECT (dlg), "blocked", NULL);
-   
-} // end of sp_text_edit_dialog_read_selection()
-
+}
 
 
 static void
 sp_text_edit_dialog_text_changed (GtkTextBuffer *tb, GtkWidget *dlg)
 {
     GtkWidget *textw, *preview, *apply, *def;
-    SPText *text;
     GtkTextIter start, end;
     gchar *str;
 
     if (g_object_get_data (G_OBJECT (dlg), "blocked")) 
         return;
 
-    text = sp_ted_get_selected_text_item ();
+    SPItem *text = sp_ted_get_selected_text_item ();
 
     textw = (GtkWidget*)g_object_get_data (G_OBJECT (dlg), "textw");
     preview = (GtkWidget*)g_object_get_data (G_OBJECT (dlg), "preview");
@@ -793,12 +806,11 @@ sp_text_edit_dialog_font_changed ( SPFontSelector *fsel,
                                    GtkWidget *dlg )
 {
     GtkWidget *preview, *apply, *def;
-    SPText *text;
 
     if (g_object_get_data (G_OBJECT (dlg), "blocked")) 
         return;
 
-    text = sp_ted_get_selected_text_item ();
+    SPItem *text = sp_ted_get_selected_text_item ();
 
     preview = (GtkWidget*)g_object_get_data (G_OBJECT (dlg), "preview");
     apply = (GtkWidget*)g_object_get_data (G_OBJECT (dlg), "apply");
@@ -819,12 +831,11 @@ static void
 sp_text_edit_dialog_any_toggled (GtkToggleButton *tb, GtkWidget *dlg)
 {
     GtkWidget *apply, *def;
-    SPText *text;
 
     if (g_object_get_data (G_OBJECT (dlg), "blocked"))
         return;
 
-    text = sp_ted_get_selected_text_item ();
+    SPItem *text = sp_ted_get_selected_text_item ();
 
     apply = (GtkWidget*)g_object_get_data (G_OBJECT (dlg), "apply");
     def = (GtkWidget*)g_object_get_data (G_OBJECT (dlg), "default");
@@ -841,12 +852,11 @@ static void
 sp_text_edit_dialog_line_spacing_changed (GtkEditable *editable, GtkWidget *dlg)
 {
     GtkWidget *apply, *def;
-    SPText *text;
 
     if (g_object_get_data ((GObject *) dlg, "blocked")) 
         return;
 
-    text = sp_ted_get_selected_text_item ();
+    SPItem *text = sp_ted_get_selected_text_item ();
 
     apply = (GtkWidget*)g_object_get_data (G_OBJECT (dlg), "apply");
     def = (GtkWidget*)g_object_get_data (G_OBJECT (dlg), "default");
@@ -859,7 +869,7 @@ sp_text_edit_dialog_line_spacing_changed (GtkEditable *editable, GtkWidget *dlg)
 
 
 
-static SPText *
+static SPItem *
 sp_ted_get_selected_text_item (void)
 {
     if (!SP_ACTIVE_DESKTOP)
@@ -869,8 +879,8 @@ sp_ted_get_selected_text_item (void)
          item != NULL;
          item = item->next)
     {
-        if (SP_IS_TEXT(item->data))
-            return SP_TEXT (item->data);
+        if (SP_IS_TEXT(item->data) || SP_IS_FLOWTEXT(item->data))
+            return SP_ITEM (item->data);
     }
 
     return NULL;
@@ -890,7 +900,7 @@ sp_ted_get_selected_text_count (void)
          item != NULL;
          item = item->next)
     {
-        if (SP_IS_TEXT(item->data))
+        if (SP_IS_TEXT(item->data) || SP_IS_FLOWTEXT(item->data))
             ++items;
     }
 
