@@ -6,14 +6,17 @@
  * Authors:
  *   Lauris Kaplinski <lauris@kaplinski.com>
  *   MenTaLguY <mental@rydia.net>
+ *   bulia byak <buliabyak@users.sf.net>
  *
- * Copyright (C) 1999-2002 Authors
+ * Copyright (C) 1999-2005 Authors
  * Copyright (C) 2004 David Turner
  *
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
 
 #include <sigc++/sigc++.h>
 #include <string.h>
@@ -44,6 +47,7 @@
 #include "../sp-tspan.h"
 #include "../sp-root.h"
 #include "../sp-defs.h"
+#include "../event-context.h"
 #include "in-dt-coordsys.h"
 
 
@@ -72,6 +76,7 @@ struct EditableDest {
 
 static GtkWidget * dlg = NULL;
 static sigc::connection sel_changed_connection;
+static sigc::connection document_uri_set_connection;
 static win_data wd;
 // impossible original values to make sure they are read from prefs
 static gint x = -1000, y = -1000, w = 0, h = 0;
@@ -92,19 +97,19 @@ static gint blocked = 0;
 static SPDesktop * current_desktop = NULL;
 static SPDocument * current_document = NULL;
 static gint selected_attr = 0;
-static SPRepr * selected_repr = NULL;
+static Inkscape::XML::Node * selected_repr = NULL;
 
 static void sp_xmltree_desktop_change ( Inkscape::Application *inkscape,  SPDesktop *desktop, GtkWidget *dialog );
 
 static void set_tree_desktop (SPDesktop * desktop);
 static void set_tree_document (SPDocument * document);
-static void set_tree_repr (SPRepr * repr);
+static void set_tree_repr (Inkscape::XML::Node * repr);
 
-static void set_tree_select (SPRepr * repr);
-static void propagate_tree_select (SPRepr * repr);
+static void set_tree_select (Inkscape::XML::Node * repr);
+static void propagate_tree_select (Inkscape::XML::Node * repr);
 
-static SPRepr * get_dt_select (void);
-static void set_dt_select (SPRepr * repr);
+static Inkscape::XML::Node * get_dt_select (void);
+static void set_dt_select (Inkscape::XML::Node * repr);
 
 static void on_tree_select_row (GtkCTree * tree, GtkCTreeNode * node, gint column, gpointer data);
 static void on_tree_unselect_row (GtkCTree * tree, GtkCTreeNode * node, gint column, gpointer data);
@@ -138,8 +143,8 @@ static void on_attr_unselect_row_clear_text (GtkCList *list, gint row, gint colu
 
 static void on_editable_changed_enable_if_valid_xml_name (GtkEditable * editable, gpointer data);
 
-static void on_desktop_selection_changed (SPSelection * selection);
-static void on_document_uri_set (SPDocument * document, const gchar * uri, gpointer data);
+static void on_desktop_selection_changed (Inkscape::Selection * selection);
+static void on_document_uri_set (gchar const *uri, SPDocument * document);
 
 static void on_clicked_get_editable_text (GtkWidget * widget, gpointer data);
 
@@ -299,7 +304,7 @@ sp_xml_tree_dialog (void)
                 NULL,
                 _("New element node"),
                 NULL,
-                sp_icon_new_scaled( SP_ICON_SIZE_NOTEBOOK, 
+                sp_icon_new( GTK_ICON_SIZE_LARGE_TOOLBAR,
                                     INKSCAPE_STOCK_ADD_XML_ELEMENT_NODE ),
                 G_CALLBACK (cmd_new_element_node),
                 NULL);
@@ -320,7 +325,7 @@ sp_xml_tree_dialog (void)
 
         button = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar),
                 NULL, _("New text node"), NULL,
-                sp_icon_new_scaled( SP_ICON_SIZE_NOTEBOOK, 
+                sp_icon_new( GTK_ICON_SIZE_LARGE_TOOLBAR,
                              INKSCAPE_STOCK_ADD_XML_TEXT_NODE ),
                 G_CALLBACK (cmd_new_text_node),
                 NULL);
@@ -341,7 +346,7 @@ sp_xml_tree_dialog (void)
 
         button = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar),
                 NULL, _("Duplicate node"), NULL,
-                sp_icon_new_scaled( SP_ICON_SIZE_NOTEBOOK, 
+                sp_icon_new( GTK_ICON_SIZE_LARGE_TOOLBAR,
                              INKSCAPE_STOCK_DUPLICATE_XML_NODE ),
                 G_CALLBACK (cmd_duplicate_node),
                 NULL);
@@ -362,7 +367,7 @@ sp_xml_tree_dialog (void)
 
         button = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar),
                 NULL, _("Delete node"), NULL,
-                sp_icon_new_scaled( SP_ICON_SIZE_NOTEBOOK, 
+                sp_icon_new( GTK_ICON_SIZE_LARGE_TOOLBAR,
                              INKSCAPE_STOCK_DELETE_XML_NODE ),
                                            G_CALLBACK (cmd_delete_node), NULL );
 
@@ -462,7 +467,7 @@ sp_xml_tree_dialog (void)
 
         button = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar),
                 NULL, _("Delete attribute"), NULL,
-                sp_icon_new_scaled( SP_ICON_SIZE_NOTEBOOK, 
+                sp_icon_new( GTK_ICON_SIZE_LARGE_TOOLBAR,
                              INKSCAPE_STOCK_DELETE_XML_ATTRIBUTE ),
                 (GCallback) cmd_delete_attr, NULL);
 
@@ -628,7 +633,7 @@ static gboolean
 sp_xml_tree_key_press (GtkWidget *widget, GdkEventKey *event)
 {
 
-    unsigned int shortcut = event->keyval |
+    unsigned int shortcut = get_group0_keyval (event) |
         ( event->state & GDK_SHIFT_MASK ?
           SP_SHORTCUT_SHIFT_MASK : 0 ) |
         ( event->state & GDK_CONTROL_MASK ?
@@ -685,14 +690,13 @@ set_tree_document (SPDocument * document)
         return;
 
     if (current_document) {
-        sp_signal_disconnect_by_data (current_document, dlg);
+        document_uri_set_connection.disconnect();
     }
     current_document = document;
     if (current_document) {
-        g_signal_connect ( G_OBJECT (current_document), "uri_set",
-                           G_CALLBACK (on_document_uri_set), dlg );
-        on_document_uri_set ( current_document,
-                              SP_DOCUMENT_URI (current_document), dlg );
+
+        document_uri_set_connection = current_document->connectURISet(sigc::bind(sigc::ptr_fun(&on_document_uri_set), current_document));
+        on_document_uri_set(SP_DOCUMENT_URI (current_document), current_document);
         set_tree_repr (sp_document_repr_root (current_document));
 
     } else {
@@ -703,7 +707,7 @@ set_tree_document (SPDocument * document)
 
 
 void
-set_tree_repr (SPRepr * repr)
+set_tree_repr (Inkscape::XML::Node * repr)
 {
     if ( repr == selected_repr )
         return;
@@ -727,7 +731,7 @@ set_tree_repr (SPRepr * repr)
 
 
 void
-set_tree_select (SPRepr * repr)
+set_tree_select (Inkscape::XML::Node * repr)
 {
     if (selected_repr) {
         sp_repr_unref (selected_repr);
@@ -762,15 +766,15 @@ set_tree_select (SPRepr * repr)
 
 
 void
-propagate_tree_select (SPRepr * repr)
+propagate_tree_select (Inkscape::XML::Node * repr)
 {
-    if (repr && repr->type() == SP_XML_ELEMENT_NODE) {
+    if (repr && repr->type() == Inkscape::XML::ELEMENT_NODE) {
         sp_xmlview_attr_list_set_repr (attributes, repr);
     } else {
         sp_xmlview_attr_list_set_repr (attributes, NULL);
     }
 
-    if (repr && ( repr->type() == SP_XML_TEXT_NODE || repr->type() == SP_XML_COMMENT_NODE ) ) {
+    if (repr && ( repr->type() == Inkscape::XML::TEXT_NODE || repr->type() == Inkscape::XML::COMMENT_NODE ) ) {
         sp_xmlview_content_set_repr (content, repr);
     } else {
         sp_xmlview_content_set_repr (content, NULL);
@@ -778,7 +782,7 @@ propagate_tree_select (SPRepr * repr)
 }
 
 
-SPRepr *
+Inkscape::XML::Node *
 get_dt_select (void)
 {
     if (!current_desktop)
@@ -790,16 +794,16 @@ get_dt_select (void)
 
 
 void
-set_dt_select (SPRepr *repr)
+set_dt_select (Inkscape::XML::Node *repr)
 {
     if (!current_desktop)
         return;
 
-    SPSelection *selection = SP_DT_SELECTION(current_desktop);
+    Inkscape::Selection *selection = SP_DT_SELECTION(current_desktop);
 
     SPObject *object;
     if (repr) {
-        while ( ( repr->type() != SP_XML_ELEMENT_NODE )
+        while ( ( repr->type() != Inkscape::XML::ELEMENT_NODE )
                 && sp_repr_parent(repr) )
         {
             repr = sp_repr_parent(repr);
@@ -821,7 +825,7 @@ set_dt_select (SPRepr *repr)
             /* TODO: when a tspan is highlighted, set selection to its parent
              * text
              */
-            selection->setItem(SP_ITEM(object));
+            selection->set(SP_ITEM(object));
     }
     blocked--;
 
@@ -837,7 +841,7 @@ on_tree_select_row ( GtkCTree * tree,
     if (blocked)
         return;
 
-    SPRepr *repr = sp_xmlview_tree_node_get_repr(SP_XMLVIEW_TREE(tree), node);
+    Inkscape::XML::Node *repr = sp_xmlview_tree_node_get_repr(SP_XMLVIEW_TREE(tree), node);
     g_assert( repr != NULL );
 
     if ( selected_repr == repr )
@@ -863,7 +867,7 @@ on_tree_unselect_row ( GtkCTree * tree,
                        gint column,
                        gpointer data )
 {
-    SPRepr * repr;
+    Inkscape::XML::Node * repr;
 
     if (blocked)
         return;
@@ -961,10 +965,10 @@ on_tree_select_row_enable_if_element ( GtkCTree * tree,
                                        gint column,
                                        gpointer data )
 {
-    SPRepr * repr;
+    Inkscape::XML::Node * repr;
     repr = sp_xmlview_tree_node_get_repr (SP_XMLVIEW_TREE (tree), node);
 
-    if (repr->type() == SP_XML_ELEMENT_NODE) {
+    if (repr->type() == Inkscape::XML::ELEMENT_NODE) {
         gtk_widget_set_sensitive (GTK_WIDGET (data), TRUE);
     } else {
         gtk_widget_set_sensitive (GTK_WIDGET (data), FALSE);
@@ -977,10 +981,10 @@ void
 on_tree_select_row_show_if_element ( GtkCTree * tree, GtkCTreeNode * node,
                                      gint column, gpointer data)
 {
-    SPRepr * repr;
+    Inkscape::XML::Node * repr;
     repr = sp_xmlview_tree_node_get_repr (SP_XMLVIEW_TREE (tree), node);
 
-    if (repr->type() == SP_XML_ELEMENT_NODE) {
+    if (repr->type() == Inkscape::XML::ELEMENT_NODE) {
         gtk_widget_show (GTK_WIDGET (data));
     } else {
         gtk_widget_hide (GTK_WIDGET (data));
@@ -993,10 +997,10 @@ void
 on_tree_select_row_show_if_text ( GtkCTree * tree, GtkCTreeNode * node,
                                   gint column, gpointer data )
 {
-    SPRepr * repr;
+    Inkscape::XML::Node * repr;
     repr = sp_xmlview_tree_node_get_repr (SP_XMLVIEW_TREE (tree), node);
 
-    if ( repr->type() == SP_XML_TEXT_NODE || repr->type() == SP_XML_COMMENT_NODE ) {
+    if ( repr->type() == Inkscape::XML::TEXT_NODE || repr->type() == Inkscape::XML::COMMENT_NODE ) {
         gtk_widget_show (GTK_WIDGET (data));
     } else {
         gtk_widget_hide (GTK_WIDGET (data));
@@ -1014,16 +1018,16 @@ xml_tree_node_mutable ( GtkCTreeNode * node )
 
     // if not in base level (where namedview, defs, etc go), we're mutable
     if (GTK_CTREE_ROW(GTK_CTREE_ROW (node)->parent)->parent) {
-        return true; 
+        return true;
     }
 
-    SPRepr * repr;
+    Inkscape::XML::Node * repr;
     repr = sp_xmlview_tree_node_get_repr (SP_XMLVIEW_TREE (tree), node);
     g_assert (repr);
 
     // don't let "defs" or "namedview" disappear
-    if ( !strcmp(sp_repr_name(repr),"svg:defs") ||
-         !strcmp(sp_repr_name(repr),"sodipodi:namedview") ) {
+    if ( !strcmp(repr->name(),"svg:defs") ||
+         !strcmp(repr->name(),"sodipodi:namedview") ) {
         return false;
     }
 
@@ -1106,7 +1110,7 @@ on_attr_row_changed ( GtkCList *list, gint row, gpointer data )
         /*
         // get current attr values
         const gchar * name = g_quark_to_string (sp_xmlview_attr_list_get_row_key (list, row));
-        const gchar * value = sp_repr_attr (selected_repr, name);
+        const gchar * value = selected_repr->attribute(name);
 
         g_warning("value: '%s'",value);
 
@@ -1160,7 +1164,7 @@ on_attr_select_row_set_value_content ( GtkCList *list, gint row, gint column,
     const gchar *name, *value;
     tb = gtk_text_view_get_buffer (GTK_TEXT_VIEW (data));
     name = g_quark_to_string (sp_xmlview_attr_list_get_row_key (list, row));
-    value = sp_repr_attr (selected_repr, name);
+    value = selected_repr->attribute(name);
     if (!value) {
         value = "";
     }
@@ -1175,10 +1179,10 @@ on_tree_select_row_enable_if_indentable ( GtkCTree * tree, GtkCTreeNode * node,
     gboolean indentable = FALSE;
 
     if ( xml_tree_node_mutable( node ) ) {
-        SPRepr * repr, * prev;
+        Inkscape::XML::Node * repr, * prev;
         repr = sp_xmlview_tree_node_get_repr (SP_XMLVIEW_TREE (tree), node);
 
-        SPRepr *parent=repr->parent();
+        Inkscape::XML::Node *parent=repr->parent();
         if ( parent && repr != parent->firstChild() ) {
             g_assert (parent->firstChild());
 
@@ -1187,7 +1191,7 @@ on_tree_select_row_enable_if_indentable ( GtkCTree * tree, GtkCTreeNode * node,
                   prev && prev->next() != repr ;
                   prev = prev->next() );
 
-            if (prev && prev->type() == SP_XML_ELEMENT_NODE) {
+            if (prev && prev->type() == Inkscape::XML::ELEMENT_NODE) {
                 indentable = TRUE;
             }
         }
@@ -1204,10 +1208,10 @@ on_tree_select_row_enable_if_not_first_child ( GtkCTree * tree,
                                                gint column,
                                                gpointer data )
 {
-    SPRepr * repr;
+    Inkscape::XML::Node * repr;
     repr = sp_xmlview_tree_node_get_repr (SP_XMLVIEW_TREE (tree), node);
 
-    SPRepr *parent=repr->parent();
+    Inkscape::XML::Node *parent=repr->parent();
     if ( parent && repr != parent->firstChild() ) {
         gtk_widget_set_sensitive (GTK_WIDGET (data), TRUE);
     } else {
@@ -1222,10 +1226,10 @@ on_tree_select_row_enable_if_not_last_child ( GtkCTree * tree,
                                               GtkCTreeNode * node,
                                               gint column, gpointer data )
 {
-    SPRepr * repr;
+    Inkscape::XML::Node * repr;
     repr = sp_xmlview_tree_node_get_repr (SP_XMLVIEW_TREE (tree), node);
 
-    SPRepr *parent=repr->parent();
+    Inkscape::XML::Node *parent=repr->parent();
     if ( parent && parent->parent() && repr->next() ) {
         gtk_widget_set_sensitive (GTK_WIDGET (data), TRUE);
     } else {
@@ -1310,7 +1314,7 @@ on_editable_changed_enable_if_valid_xml_name ( GtkEditable * editable,
 
 
 void
-on_desktop_selection_changed (SPSelection * selection)
+on_desktop_selection_changed (Inkscape::Selection * selection)
 {
     if (!blocked++) {
         set_tree_select (get_dt_select ());
@@ -1319,9 +1323,7 @@ on_desktop_selection_changed (SPSelection * selection)
 }
 
 
-void
-on_document_uri_set ( SPDocument * document, const gchar * uri, gpointer data )
-{
+void on_document_uri_set(gchar const *uri, SPDocument *document) {
     gchar *t;
     gchar title[500];
 
@@ -1407,10 +1409,10 @@ cmd_new_element_node (GtkObject * object, gpointer data)
     g_assert (selected_repr != NULL);
 
     if (name.text) {
-        SPRepr * new_repr;
+        Inkscape::XML::Node * new_repr;
         new_repr = sp_repr_new (name.text);
         g_free (name.text);
-        sp_repr_append_child (selected_repr, new_repr);
+        selected_repr->appendChild(new_repr);
         set_tree_select (new_repr);
         set_dt_select (new_repr);
     }
@@ -1422,12 +1424,12 @@ cmd_new_element_node (GtkObject * object, gpointer data)
 void
 cmd_new_text_node (GtkObject * object, gpointer data)
 {
-    SPRepr * text;
+    Inkscape::XML::Node * text;
 
     g_assert (selected_repr != NULL);
 
     text = sp_repr_new_text ("");
-    sp_repr_append_child (selected_repr, text);
+    selected_repr->appendChild(text);
 
     sp_document_done (current_document);
 
@@ -1441,14 +1443,14 @@ cmd_new_text_node (GtkObject * object, gpointer data)
 void
 cmd_duplicate_node (GtkObject * object, gpointer data)
 {
-    SPRepr * parent;
-    SPRepr * dup;
+    Inkscape::XML::Node * parent;
+    Inkscape::XML::Node * dup;
     GtkCTreeNode * node;
 
     g_assert (selected_repr != NULL);
 
     parent = sp_repr_parent (selected_repr);
-    dup = sp_repr_duplicate (selected_repr);
+    dup = selected_repr->duplicate();
     sp_repr_add_child (parent, dup, selected_repr);
 
     sp_document_done (current_document);
@@ -1479,6 +1481,12 @@ cmd_delete_attr (GtkObject * object, gpointer data)
     g_assert (selected_attr != 0);
     sp_repr_set_attr (selected_repr, g_quark_to_string (selected_attr), NULL);
 
+    SPObject *updated=current_document->getObjectByRepr(selected_repr);
+    if (updated) {
+        // force immediate update of dependant attributes
+        updated->updateRepr();
+    }
+
     sp_document_done (current_document);
 }
 
@@ -1500,7 +1508,7 @@ cmd_set_attr (GtkObject * object, gpointer data)
     value = gtk_text_buffer_get_text ( gtk_text_view_get_buffer (attr_value),
                                        &start, &end, TRUE );
 
-    if (!sp_repr_set_attr (selected_repr, name, value, true)) {
+    if (!sp_repr_set_attr (selected_repr, name, value)) {
         gchar * message = g_strdup_printf(_("Cannot set <b>%s</b>: Another element with value <b>%s</b> already exists!"), name, value);
         _message_stack->flash(Inkscape::WARNING_MESSAGE, message);
         g_free(message);
@@ -1509,6 +1517,12 @@ cmd_set_attr (GtkObject * object, gpointer data)
     g_free (name);
     g_free (value);
 
+    SPObject *updated=current_document->getObjectByRepr(selected_repr);
+    if (updated) {
+        // force immediate update of dependant attributes
+        updated->updateRepr();
+    }
+
     sp_document_done (current_document);
 
     /* TODO: actually, the row won't have been created yet.  why? */
@@ -1524,7 +1538,7 @@ cmd_set_attr (GtkObject * object, gpointer data)
 void
 cmd_raise_node (GtkObject * object, gpointer data)
 {
-    SPRepr * before, * parent, * ref;
+    Inkscape::XML::Node * before, * parent, * ref;
     g_assert (selected_repr != NULL);
 
     parent = sp_repr_parent (selected_repr);
@@ -1551,7 +1565,7 @@ cmd_raise_node (GtkObject * object, gpointer data)
 void
 cmd_lower_node (GtkObject * object, gpointer data)
 {
-    SPRepr * parent;
+    Inkscape::XML::Node * parent;
     g_assert (selected_repr != NULL);
     g_return_if_fail (selected_repr->next() != NULL);
     parent = sp_repr_parent (selected_repr);
@@ -1570,7 +1584,7 @@ void
 cmd_indent_node (GtkObject * object, gpointer data)
 {
     int ok;
-    SPRepr * prev, * parent, * repr, * ref;
+    Inkscape::XML::Node * prev, * parent, * repr, * ref;
 
     repr = selected_repr;
     g_assert (repr != NULL);
@@ -1583,7 +1597,7 @@ cmd_indent_node (GtkObject * object, gpointer data)
         prev = prev->next();
     }
     g_return_if_fail (prev != NULL);
-    g_return_if_fail (prev->type() == SP_XML_ELEMENT_NODE);
+    g_return_if_fail (prev->type() == Inkscape::XML::ELEMENT_NODE);
 
     if (prev->firstChild()) {
         for ( ref = prev->firstChild() ; ref->next() ; ref = ref->next() );
@@ -1609,7 +1623,7 @@ cmd_indent_node (GtkObject * object, gpointer data)
 void
 cmd_unindent_node (GtkObject * object, gpointer data)
 {
-    SPRepr * grandparent, * parent, * repr;
+    Inkscape::XML::Node * grandparent, * parent, * repr;
     int ok;
 
     repr = selected_repr;
