@@ -167,6 +167,9 @@ copy_without_nans_or_adjacent_duplicates(NR::Point const src[], unsigned src_len
     return dest_len;
 }
 
+/**
+ * \pre data is uniqued, i.e. not exist i: data[i] == data[i + 1].
+ */
 gint
 sp_bezier_fit_cubic_full(NR::Point bezier[], int split_points[],
                          NR::Point const data[], gint const len,
@@ -592,7 +595,7 @@ NewtonRaphsonRootFind(BezierCurve const Q, NR::Point const &P, gdouble const u)
         }
     }
 
-    if (!finite(improved_u)) {
+    if (!isFinite(improved_u)) {
         improved_u = u;
     } else if ( improved_u < 0.0 ) {
         improved_u = 0.0;
@@ -817,7 +820,7 @@ chord_length_parameterize(NR::Point const d[], gdouble u[], unsigned const len)
     /* Then scale to [0.0 .. 1.0]. */
     gdouble tot_len = u[len - 1];
     g_return_if_fail( tot_len != 0 );
-    if (finite(tot_len)) {
+    if (isFinite(tot_len)) {
         for (unsigned i = 1; i < len; ++i) {
             u[i] /= tot_len;
         }
@@ -828,6 +831,17 @@ chord_length_parameterize(NR::Point const d[], gdouble u[], unsigned const len)
         }
     }
 
+    /* It's been reported that u[len - 1] can differ from 1.0 on some systems (amd64),
+       despite it having been calculated as x / x where x is isFinite and non-zero. */
+    if (u[len - 1] != 1) {
+        double const diff = u[len - 1] - 1;
+        if (fabs(diff) > 1e-13) {
+            g_warning("u[len - 1] = %19g (= 1 + %19g), expecting exactly 1",
+                      u[len - 1], diff);
+        }
+        u[len - 1] = 1;
+    }
+
 #ifdef BEZIER_DEBUG
     g_assert( u[0] == 0.0 && u[len - 1] == 1.0 );
     for (unsigned i = 1; i < len; i++) {
