@@ -15,21 +15,26 @@
 
 #include <map>
 #include <sigc++/sigc++.h>
-#include "xml/sp-repr-action-fns.h"
+#include "xml/event-fns.h"
 #include "sp-defs.h"
 #include "sp-root.h"
 #include "document.h"
 
 #define SP_DOCUMENT_DEFS(d) ((SPObject *) SP_ROOT (SP_DOCUMENT_ROOT (d))->defs)
 
-class SPReprAction;
+namespace Inkscape {
+namespace XML {
+class Event;
+}
+}
+
 
 struct SPDocumentPrivate {
 	typedef std::map<GQuark, SPDocument::IDChangedSignal> IDChangedSignalMap;
 	typedef std::map<GQuark, SPDocument::ResourcesChangedSignal> ResourcesChangedSignalMap;
 
 	GHashTable *iddef;	/**< Dictionary of id -> SPObject mappings */
-	GHashTable *reprdef;   /**< Dictionary of SPRepr -> SPObject mappings */
+	GHashTable *reprdef;   /**< Dictionary of Inkscape::XML::Node -> SPObject mappings */
 
 	/** Dictionary of signals for id changes */
 	IDChangedSignalMap id_changed_signals;
@@ -39,9 +44,15 @@ struct SPDocumentPrivate {
 	GHashTable *resources;
 	ResourcesChangedSignalMap resources_changed_signals;
 
+	SPDocument::ModifiedSignal modified_signal;
+	SPDocument::URISetSignal uri_set_signal;
+	SPDocument::ResizedSignal resized_signal;
+	SPDocument::ReconstructionStart _reconstruction_start_signal;
+	SPDocument::ReconstructionFinish  _reconstruction_finish_signal;
+
 	/* Undo/Redo state */
 	guint sensitive: 1; /* If we save actions to undo stack */
-	SPReprAction * partial; /* partial undo log when interrupted */
+	Inkscape::XML::Event * partial; /* partial undo log when interrupted */
 	int history_size;
 	GSList * undo; /* Undo stack of reprs */
 	GSList * redo; /* Redo stack of reprs */
