@@ -5,7 +5,10 @@
  *
  * Authors:
  *   Lauris Kaplinski <lauris@kaplinski.com>
+ *   MenTaLguY <mental@rydia.net>
+ *   bulia byak <buliabyak@users.sf.net>
  *
+ * Copyright (C) 2004-2005 MenTaLguY
  * Copyright (C) 1999-2002 Lauris Kaplinski
  * Copyright (C) 2000-2001 Ximian, Inc.
  *
@@ -15,12 +18,14 @@
 #define noSP_DOCUMENT_DEBUG_IDLE
 #define noSP_DOCUMENT_DEBUG_UNDO
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
 #include <glib.h>
 #include <string.h>
 #include <gtk/gtkmain.h>
 #include "xml/repr.h"
-#include "xml/sp-repr-action-fns.h"
+#include "xml/event-fns.h"
 #include "helper/sp-marshal.h"
 #include <glibmm/i18n.h>
 #include "inkscape-private.h"
@@ -50,99 +55,30 @@
 #include <locale>
 #include <sstream>
 
-enum {
-	MODIFIED,
-	URI_SET,
-	RESIZED,
-	LAST_SIGNAL
-};
-
-static void sp_document_class_init(SPDocumentClass *klass);
-static void sp_document_init(SPDocument *document);
-static void sp_document_dispose(GObject *object);
-
 static gint sp_document_idle_handler(gpointer data);
 
 gboolean sp_document_resource_list_free(gpointer key, gpointer value, gpointer data);
 
-static GObjectClass *parent_class;
-static guint signals[LAST_SIGNAL] = {0};
 static gint doc_count = 0;
 
-GType
-sp_document_get_type (void)
-{
-	static GType type = 0;
-	if (!type) {
-		GTypeInfo info = {
-			sizeof (SPDocumentClass),
-			NULL, NULL,
-			(GClassInitFunc) sp_document_class_init,
-			NULL, NULL,
-			sizeof (SPDocument),
-			4,
-			(GInstanceInitFunc) sp_document_init,
-			NULL
-		};
-		type = g_type_register_static (G_TYPE_OBJECT, "SPDocument", &info, (GTypeFlags)0);
-	}
-	return type;
-}
-
-static void
-sp_document_class_init(SPDocumentClass *klass)
-{
-	GObjectClass *object_class = (GObjectClass *) klass;
-
-	parent_class = (GObjectClass*)g_type_class_peek_parent (klass);
-
-	signals[MODIFIED] = g_signal_new ("modified",
-					    G_TYPE_FROM_CLASS(klass),
-					    G_SIGNAL_RUN_FIRST,
-					    G_STRUCT_OFFSET (SPDocumentClass, modified),
-					    NULL, NULL,
-					    sp_marshal_NONE__UINT,
-					    G_TYPE_NONE, 1,
-					    G_TYPE_UINT);
-	signals[URI_SET] =    g_signal_new ("uri_set",
-					    G_TYPE_FROM_CLASS(klass),
-					    G_SIGNAL_RUN_FIRST,
-					    G_STRUCT_OFFSET (SPDocumentClass, uri_set),
-					    NULL, NULL,
-					    sp_marshal_NONE__POINTER,
-					    G_TYPE_NONE, 1,
-					    G_TYPE_POINTER);
-	signals[RESIZED] =    g_signal_new ("resized",
-					    G_TYPE_FROM_CLASS(klass),
-					    G_SIGNAL_RUN_FIRST,
-					    G_STRUCT_OFFSET (SPDocumentClass, uri_set),
-					    NULL, NULL,
-					    sp_marshal_NONE__DOUBLE_DOUBLE,
-					    G_TYPE_NONE, 2,
-					    G_TYPE_DOUBLE, G_TYPE_DOUBLE);
-	object_class->dispose = sp_document_dispose;
-}
-
-static void
-sp_document_init (SPDocument *doc)
-{
+SPDocument::SPDocument() {
 	SPDocumentPrivate *p;
 
-	doc->advertize = FALSE;
-	doc->keepalive = FALSE;
-	doc->virgin    = TRUE;
+	keepalive = FALSE;
+	virgin    = TRUE;
 
-	doc->modified_id = 0;
+	modified_id = 0;
 
-	doc->rdoc = NULL;
-	doc->rroot = NULL;
-	doc->root = NULL;
+	rdoc = NULL;
+	rroot = NULL;
+	root = NULL;
+	style_cascade = cr_cascade_new(NULL, NULL, NULL);
 
-	doc->uri = NULL;
-	doc->base = NULL;
-	doc->name = NULL;
+	uri = NULL;
+	base = NULL;
+	name = NULL;
 
-	doc->_collection_queue = NULL;
+	_collection_queue = NULL;
 
 	p = new SPDocumentPrivate();
 
@@ -157,7 +93,67 @@ sp_document_init (SPDocument *doc)
 	p->undo = NULL;
 	p->redo = NULL;
 
-	doc->priv = p;
+	priv = p;
+}
+
+SPDocument::~SPDocument() {
+	collectOrphans();
+
+	if (priv) {
+		inkscape_remove_document (this);
+
+		if (priv->partial) {
+			sp_repr_free_log (priv->partial);
+			priv->partial = NULL;
+		}
+
+		sp_document_clear_redo (this);
+		sp_document_clear_undo (this);
+
+		if (root) {
+			sp_object_invoke_release (root);
+			g_object_unref (G_OBJECT (root));
+			root = NULL;
+		}
+
+		if (priv->iddef) g_hash_table_destroy (priv->iddef);
+		if (priv->reprdef) g_hash_table_destroy (priv->reprdef);
+
+		if (rdoc) Inkscape::GC::release(rdoc);
+
+		/* Free resources */
+		g_hash_table_foreach_remove (priv->resources, sp_document_resource_list_free, this);
+		g_hash_table_destroy (priv->resources);
+
+		delete priv;
+		priv = NULL;
+	}
+
+	cr_cascade_unref(style_cascade);
+	style_cascade = NULL;
+
+	if (name) {
+		g_free (name);
+		name = NULL;
+	}
+	if (base) {
+		g_free (base);
+		base = NULL;
+	}
+	if (uri) {
+		g_free (uri);
+		uri = NULL;
+	}
+
+	if (modified_id) {
+		gtk_idle_remove (modified_id);
+		modified_id = 0;
+	}
+
+	if (keepalive) {
+		inkscape_unref ();
+		keepalive = FALSE;
+	}
 }
 
 void SPDocument::queueForOrphanCollection(SPObject *object) {
@@ -181,93 +177,21 @@ void SPDocument::collectOrphans() {
 	}
 }
 
-/**
- *  This routine removes the given document object from the application,
- *  clearing the undo/redo information, releasing its root object, releasing
- *  other resources, and destructing its data members.
- */
-static void
-sp_document_dispose (GObject *object)
-{
-	SPDocument *doc = (SPDocument *) object;
-	SPDocumentPrivate *priv = doc->priv;
-
-	doc->collectOrphans();
-
-	if (priv) {
-		inkscape_remove_document (doc);
-
-		if (priv->partial) {
-			sp_repr_free_log (priv->partial);
-			priv->partial = NULL;
-		}
-
-		sp_document_clear_redo (doc);
-		sp_document_clear_undo (doc);
-
-		if (doc->root) {
-			sp_object_invoke_release (doc->root);
-			g_object_unref (G_OBJECT (doc->root));
-			doc->root = NULL;
-		}
-
-		if (priv->iddef) g_hash_table_destroy (priv->iddef);
-		if (priv->reprdef) g_hash_table_destroy (priv->reprdef);
-
-		if (doc->rdoc) sp_repr_document_unref (doc->rdoc);
-
-		/* Free resources */
-		g_hash_table_foreach_remove (priv->resources, sp_document_resource_list_free, doc);
-		g_hash_table_destroy (priv->resources);
-
-		delete priv;
-		doc->priv = NULL;
-	}
-
-	if (doc->name) {
-		g_free (doc->name);
-		doc->name = NULL;
-	}
-	if (doc->base) {
-		g_free (doc->base);
-		doc->base = NULL;
-	}
-	if (doc->uri) {
-		g_free (doc->uri);
-		doc->uri = NULL;
-	}
-
-	if (doc->modified_id) {
-		gtk_idle_remove (doc->modified_id);
-		doc->modified_id = 0;
-	}
-
-	if (doc->keepalive) {
-		inkscape_unref ();
-		doc->keepalive = FALSE;
-	}
-
-	G_OBJECT_CLASS (parent_class)->dispose (object);
-}
-
-
 static SPDocument *
-sp_document_create (SPReprDoc *rdoc,
+sp_document_create (Inkscape::XML::Document *rdoc,
 		    const gchar *uri,
 		    const gchar *base,
 		    const gchar *name,
-		    unsigned int advertize,
 		    unsigned int keepalive)
 {
 	SPDocument *document;
-	SPRepr *rroot;
-	SPVersion sodipodi_version;
+	Inkscape::XML::Node *rroot;
+	Inkscape::Version sodipodi_version;
 
 	rroot = sp_repr_document_root (rdoc);
 
-	document = (SPDocument*)g_object_new (SP_TYPE_DOCUMENT, NULL);
+	document = new SPDocument();
 
-	document->advertize = advertize;
 	document->keepalive = keepalive;
 
 	document->rdoc = rdoc;
@@ -299,8 +223,8 @@ sp_document_create (SPReprDoc *rdoc,
 	/* fixme: Again, I moved these here to allow version determining in ::build (Lauris) */
 
 	/* Quick hack 2 - get default image size into document */
-	if (!sp_repr_attr (rroot, "width")) sp_repr_set_attr (rroot, "width", A4_WIDTH_STR);
-	if (!sp_repr_attr (rroot, "height")) sp_repr_set_attr (rroot, "height", A4_HEIGHT_STR);
+	if (!rroot->attribute("width")) sp_repr_set_attr (rroot, "width", A4_WIDTH_STR);
+	if (!rroot->attribute("height")) sp_repr_set_attr (rroot, "height", A4_HEIGHT_STR);
         /* End of quick hack 2 */
 
 	/* Quick hack 3 - Set uri attributes */
@@ -323,8 +247,8 @@ sp_document_create (SPReprDoc *rdoc,
 	// creating namedview
 	if (!sp_item_group_get_child_by_name ((SPGroup *) document->root, NULL, "sodipodi:namedview")) {
 		// if there's none in the document already,
-		SPRepr *r = NULL;
-		SPRepr *rnew = NULL;
+		Inkscape::XML::Node *r = NULL;
+		Inkscape::XML::Node *rnew = NULL;
 		r = inkscape_get_repr (INKSCAPE, "template.base");
 		// see if there's a template with id="base" in the preferences 
 		if (!r) {
@@ -333,7 +257,7 @@ sp_document_create (SPReprDoc *rdoc,
 			sp_repr_set_attr (rnew, "id", "base");
 		} else {
 			// otherwise, take from preferences
-			rnew = sp_repr_duplicate (r);
+			rnew = r->duplicate();
 		}
 		// insert into the document
 		sp_repr_add_child (rroot, rnew, NULL);
@@ -343,7 +267,7 @@ sp_document_create (SPReprDoc *rdoc,
 	
 	/* Defs */
 	if (!SP_ROOT (document->root)->defs) {
-		SPRepr *r;
+		Inkscape::XML::Node *r;
 		r = sp_repr_new ("svg:defs");
 		sp_repr_add_child (rroot, r, NULL);
 		sp_repr_unref (r);
@@ -372,15 +296,15 @@ sp_document_create (SPReprDoc *rdoc,
 }
 
 SPDocument *
-sp_document_new (const gchar *uri, unsigned int advertize, unsigned int keepalive, bool make_new)
+sp_document_new (const gchar *uri, unsigned int keepalive, bool make_new)
 {
 	SPDocument *doc;
-	SPReprDoc *rdoc;
+	Inkscape::XML::Document *rdoc;
 	gchar *base = NULL;
         gchar *name = NULL;
 
 	if (uri) {
-		SPRepr *rroot;
+		Inkscape::XML::Node *rroot;
 		gchar *s, *p;
 		/* Try to fetch repr from file */
 		rdoc = sp_repr_read_file (uri, SP_SVG_NS_URI);
@@ -389,7 +313,7 @@ sp_document_new (const gchar *uri, unsigned int advertize, unsigned int keepaliv
 		rroot = sp_repr_document_root (rdoc);
 		/* If xml file is not svg, return NULL without warning */
 		/* fixme: destroy document */
-		if (strcmp (sp_repr_name (rroot), "svg:svg") != 0) return NULL;
+		if (strcmp (rroot->name(), "svg:svg") != 0) return NULL;
 		s = g_strdup (uri);
 		p = strrchr (s, '/');
 		if (p) {
@@ -414,7 +338,7 @@ sp_document_new (const gchar *uri, unsigned int advertize, unsigned int keepaliv
         //# These should be set by now
         g_assert (name);
 
-	doc = sp_document_create (rdoc, uri, base, name, advertize, keepalive);
+	doc = sp_document_create (rdoc, uri, base, name, keepalive);
 
 	g_free (base);
 	g_free (name);
@@ -423,11 +347,11 @@ sp_document_new (const gchar *uri, unsigned int advertize, unsigned int keepaliv
 }
 
 SPDocument *
-sp_document_new_from_mem (const gchar *buffer, gint length, unsigned int advertize, unsigned int keepalive)
+sp_document_new_from_mem (const gchar *buffer, gint length, unsigned int keepalive)
 {
 	SPDocument *doc;
-	SPReprDoc *rdoc;
-	SPRepr *rroot;
+	Inkscape::XML::Document *rdoc;
+	Inkscape::XML::Node *rroot;
 	gchar *name;
 
 	rdoc = sp_repr_read_mem (buffer, length, SP_SVG_NS_URI);
@@ -438,23 +362,26 @@ sp_document_new_from_mem (const gchar *buffer, gint length, unsigned int adverti
 	rroot = sp_repr_document_root (rdoc);
 	/* If xml file is not svg, return NULL without warning */
 	/* fixme: destroy document */
-	if (strcmp (sp_repr_name (rroot), "svg:svg") != 0) return NULL;
+	if (strcmp (rroot->name(), "svg:svg") != 0) return NULL;
 
 	name = g_strdup_printf (_("Memory document %d"), ++doc_count);
 
-	doc = sp_document_create (rdoc, NULL, NULL, name, advertize, keepalive);
+	doc = sp_document_create (rdoc, NULL, NULL, name, keepalive);
 
 	return doc;
 }
 
+SPDocument *sp_document_new_dummy() {
+	SPDocument *document = new SPDocument();
+	inkscape_add_document(document);
+	return document;
+}
+
 SPDocument *
 sp_document_ref (SPDocument *doc)
 {
 	g_return_val_if_fail (doc != NULL, NULL);
-	g_return_val_if_fail (SP_IS_DOCUMENT (doc), NULL);
-
-	g_object_ref (G_OBJECT (doc));
-
+	Inkscape::GC::anchor(doc);
 	return doc;
 }
 
@@ -462,17 +389,13 @@ SPDocument *
 sp_document_unref (SPDocument *doc)
 {
 	g_return_val_if_fail (doc != NULL, NULL);
-	g_return_val_if_fail (SP_IS_DOCUMENT (doc), NULL);
-
-	g_object_unref (G_OBJECT (doc));
-
+	Inkscape::GC::release(doc);
 	return NULL;
 }
 
 gdouble sp_document_width(SPDocument *document)
 {
 	g_return_val_if_fail (document != NULL, 0.0);
-	g_return_val_if_fail (SP_IS_DOCUMENT (document), 0.0);
 	g_return_val_if_fail (document->priv != NULL, 0.0);
 	g_return_val_if_fail (document->root != NULL, 0.0);
 
@@ -482,7 +405,6 @@ gdouble sp_document_width(SPDocument *document)
 gdouble sp_document_height(SPDocument *document)
 {
 	g_return_val_if_fail (document != NULL, 0.0);
-	g_return_val_if_fail (SP_IS_DOCUMENT (document), 0.0);
 	g_return_val_if_fail (document->priv != NULL, 0.0);
 	g_return_val_if_fail (document->root != NULL, 0.0);
 
@@ -492,7 +414,6 @@ gdouble sp_document_height(SPDocument *document)
 void sp_document_set_uri(SPDocument *document, gchar const *uri)
 {
 	g_return_if_fail (document != NULL);
-	g_return_if_fail (SP_IS_DOCUMENT (document));
 
 	if (document->name) {
 		g_free (document->name);
@@ -527,7 +448,7 @@ void sp_document_set_uri(SPDocument *document, gchar const *uri)
 	}
 
 	// Update saveable repr attributes.
-	SPRepr *repr = sp_document_repr_root(document);
+	Inkscape::XML::Node *repr = sp_document_repr_root(document);
 	// changing uri in the document repr must not be not undoable
 	gboolean saved = sp_document_get_undo_sensitive(document);
 	sp_document_set_undo_sensitive (document, FALSE);
@@ -537,16 +458,65 @@ void sp_document_set_uri(SPDocument *document, gchar const *uri)
 	sp_repr_set_attr (repr, "sodipodi:docname", document->name);
 	sp_document_set_undo_sensitive (document, saved);
 
-	g_signal_emit (G_OBJECT (document), signals [URI_SET], 0, document->uri);
+	document->priv->uri_set_signal.emit(document->uri);
 }
 
 void
 sp_document_set_size_px (SPDocument *doc, gdouble width, gdouble height)
 {
 	g_return_if_fail (doc != NULL);
-	g_return_if_fail (SP_IS_DOCUMENT (doc));
 
-	g_signal_emit (G_OBJECT (doc), signals [RESIZED], 0, width, height);
+	doc->priv->resized_signal.emit(width, height);
+}
+
+sigc::connection SPDocument::connectModified(SPDocument::ModifiedSignal::slot_type slot)
+{
+	return priv->modified_signal.connect(slot);
+}
+
+sigc::connection SPDocument::connectURISet(SPDocument::URISetSignal::slot_type slot)
+{
+	return priv->uri_set_signal.connect(slot);
+}
+
+sigc::connection SPDocument::connectResized(SPDocument::ResizedSignal::slot_type slot)
+{
+	return priv->resized_signal.connect(slot);
+}
+
+sigc::connection
+SPDocument::connectReconstructionStart(SPDocument::ReconstructionStart::slot_type slot)
+{
+	return priv->_reconstruction_start_signal.connect(slot);
+}
+
+void
+SPDocument::emitReconstructionStart (void)
+{
+	// printf("Starting Reconstruction\n");
+	priv->_reconstruction_start_signal.emit();
+	return;
+}
+
+sigc::connection
+SPDocument::connectReconstructionFinish (SPDocument::ReconstructionFinish::slot_type  slot)
+{
+	return priv->_reconstruction_finish_signal.connect(slot);
+}
+
+void
+SPDocument::emitReconstructionFinish (void)
+{
+	// printf("Finishing Reconstruction\n");
+	priv->_reconstruction_finish_signal.emit();
+	return;
+}
+
+
+void SPDocument::_emitModified() {
+	static const guint flags = SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_CHILD_MODIFIED_FLAG | SP_OBJECT_PARENT_MODIFIED_FLAG;
+	root->emitModified(0);
+	priv->modified_signal.emit(flags);
 }
 
 void SPDocument::bindObjectToId(gchar const *id, SPObject *object) {
@@ -583,7 +553,7 @@ sigc::connection SPDocument::connectIdChanged(const gchar *id, SPDocument::IDCha
 	return priv->id_changed_signals[g_quark_from_string(id)].connect(slot);
 }
 
-void SPDocument::bindObjectToRepr(SPRepr *repr, SPObject *object) {
+void SPDocument::bindObjectToRepr(Inkscape::XML::Node *repr, SPObject *object) {
 	if (object) {
 		g_assert(g_hash_table_lookup(priv->reprdef, repr) == NULL);
 		g_hash_table_insert(priv->reprdef, repr, object);
@@ -593,7 +563,7 @@ void SPDocument::bindObjectToRepr(SPRepr *repr, SPObject *object) {
 	}
 }
 
-SPObject *SPDocument::getObjectByRepr(SPRepr *repr) {
+SPObject *SPDocument::getObjectByRepr(Inkscape::XML::Node *repr) {
 	g_return_val_if_fail (repr != NULL, NULL);
 	return (SPObject*)g_hash_table_lookup(priv->reprdef, repr);
 }
@@ -628,20 +598,16 @@ sp_document_ensure_up_to_date (SPDocument *doc)
 		if (doc->root->uflags) {
 			SPItemCtx ctx;
 			ctx.ctx.flags = 0;
-			nr_matrix_set_identity (&ctx.i2doc);
+			ctx.i2doc = NR::identity();
 			/* Set up viewport in case svg has it defined as percentages */
 			ctx.vp.x0 = 0.0;
 			ctx.vp.y0 = 0.0;
-			ctx.vp.x1 = 21.0 * PX_PER_CM; 
-			ctx.vp.y1 = 29.7 * PX_PER_CM;
-			nr_matrix_set_identity (&ctx.i2vp);
+			ctx.vp.x1 = 210 * PX_PER_MM;
+			ctx.vp.y1 = 297 * PX_PER_MM;
+			ctx.i2vp = NR::identity();
 			doc->root->updateDisplay((SPCtx *)&ctx, 0);
 		}
-		/* Emit "modified" signal on objects */
-		doc->root->emitModified(0);
-		/* Emit our own "modified" signal */
-		g_signal_emit (G_OBJECT (doc), signals [MODIFIED], 0,
-			       SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_CHILD_MODIFIED_FLAG | SP_OBJECT_PARENT_MODIFIED_FLAG);
+		doc->_emitModified();
 	}
 	if (doc->modified_id) {
 		/* Remove handler */
@@ -657,7 +623,7 @@ sp_document_idle_handler (gpointer data)
 	SPDocument *doc;
 	int repeat;
 
-	doc = SP_DOCUMENT (data);
+	doc = static_cast<SPDocument *>(data);
 
 #ifdef SP_DOCUMENT_DEBUG_IDLE
 	g_print ("->\n");
@@ -667,13 +633,13 @@ sp_document_idle_handler (gpointer data)
 	if (doc->root->uflags) {
 		SPItemCtx ctx;
 		ctx.ctx.flags = 0;
-		nr_matrix_set_identity (&ctx.i2doc);
+		ctx.i2doc = NR::identity();
 		/* Set up viewport in case svg has it defined as percentages */
 		ctx.vp.x0 = 0.0;
 		ctx.vp.y0 = 0.0;
 		ctx.vp.x1 = 21.0 * PX_PER_CM;
 		ctx.vp.y1 = 29.7 * PX_PER_CM;
-		nr_matrix_set_identity (&ctx.i2vp);
+		ctx.i2vp = NR::identity();
 
 		gboolean saved = sp_document_get_undo_sensitive(doc);
 		sp_document_set_undo_sensitive (doc, FALSE);
@@ -684,20 +650,7 @@ sp_document_idle_handler (gpointer data)
 		/* if (doc->root->uflags & SP_OBJECT_MODIFIED_FLAG) return TRUE; */
 	}
 
-	/* Emit "modified" signal on objects */
-	doc->root->emitModified(0);
-
-#ifdef SP_DOCUMENT_DEBUG_IDLE
-	g_print ("\n->");
-#endif
-
-	/* Emit our own "modified" signal */
-	g_signal_emit (G_OBJECT (doc), signals [MODIFIED], 0,
-			 SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_CHILD_MODIFIED_FLAG | SP_OBJECT_PARENT_MODIFIED_FLAG);
-
-#ifdef SP_DOCUMENT_DEBUG_IDLE
-	g_print (" S ->\n");
-#endif
+	doc->_emitModified();
 
 	repeat = (doc->root->uflags || doc->root->mflags);
 	if (!repeat) doc->modified_id = 0;
@@ -884,7 +837,6 @@ GSList *
 sp_document_items_in_box (SPDocument *document, unsigned int dkey, NRRect const *box)
 {
 	g_return_val_if_fail (document != NULL, NULL);
-	g_return_val_if_fail (SP_IS_DOCUMENT (document), NULL);
 	g_return_val_if_fail (document->priv != NULL, NULL);
 	g_return_val_if_fail (box != NULL, NULL);
 
@@ -903,7 +855,6 @@ GSList *
 sp_document_partial_items_in_box (SPDocument *document, unsigned int dkey, NRRect const *box)
 {
 	g_return_val_if_fail (document != NULL, NULL);
-	g_return_val_if_fail (SP_IS_DOCUMENT (document), NULL);
 	g_return_val_if_fail (document->priv != NULL, NULL);
 	g_return_val_if_fail (box != NULL, NULL);
 
@@ -916,7 +867,6 @@ sp_document_item_at_point(SPDocument *document, unsigned const key, NR::Point co
 			  gboolean const into_groups, SPItem *upto)
 {
 	g_return_val_if_fail (document != NULL, NULL);
-	g_return_val_if_fail (SP_IS_DOCUMENT (document), NULL);
 	g_return_val_if_fail (document->priv != NULL, NULL);
 
  	return find_item_at_point (key, SP_GROUP (document->root), p, into_groups, false, upto);
@@ -926,7 +876,6 @@ SPItem*
 sp_document_group_at_point (SPDocument *document, unsigned int key, NR::Point const p)
 {
 	g_return_val_if_fail (document != NULL, NULL);
-	g_return_val_if_fail (SP_IS_DOCUMENT (document), NULL);
 	g_return_val_if_fail (document->priv != NULL, NULL);
 
  	return find_group_at_point (key, SP_GROUP (document->root), p);
@@ -942,12 +891,14 @@ sp_document_add_resource (SPDocument *document, const gchar *key, SPObject *obje
 	GQuark q=g_quark_from_string(key);
 
 	g_return_val_if_fail (document != NULL, FALSE);
-	g_return_val_if_fail (SP_IS_DOCUMENT (document), FALSE);
 	g_return_val_if_fail (key != NULL, FALSE);
 	g_return_val_if_fail (*key != '\0', FALSE);
 	g_return_val_if_fail (object != NULL, FALSE);
 	g_return_val_if_fail (SP_IS_OBJECT (object), FALSE);
 
+	if (SP_OBJECT_IS_CLONED (object)) 
+		return FALSE;
+
 	rlist = (GSList*)g_hash_table_lookup (document->priv->resources, key);
 	g_return_val_if_fail (!g_slist_find (rlist, object), FALSE);
 	rlist = g_slist_prepend (rlist, object);
@@ -965,7 +916,6 @@ sp_document_remove_resource (SPDocument *document, const gchar *key, SPObject *o
 	GQuark q=g_quark_from_string(key);
 
 	g_return_val_if_fail (document != NULL, FALSE);
-	g_return_val_if_fail (SP_IS_DOCUMENT (document), FALSE);
 	g_return_val_if_fail (key != NULL, FALSE);
 	g_return_val_if_fail (*key != '\0', FALSE);
 	g_return_val_if_fail (object != NULL, FALSE);
@@ -986,7 +936,6 @@ const GSList *
 sp_document_get_resource_list (SPDocument *document, const gchar *key)
 {
 	g_return_val_if_fail (document != NULL, NULL);
-	g_return_val_if_fail (SP_IS_DOCUMENT (document), NULL);
 	g_return_val_if_fail (key != NULL, NULL);
 	g_return_val_if_fail (*key != '\0', NULL);
 
@@ -1011,4 +960,60 @@ sp_document_resource_list_free (gpointer key, gpointer value, gpointer data)
 	return TRUE;
 }
 
+unsigned int
+count_objects_recursive (SPObject *obj, unsigned int count)
+{
+	count ++; // obj itself
+
+	for (SPObject *i = sp_object_first_child(obj); i != NULL; i = SP_OBJECT_NEXT(i)) {
+		count = count_objects_recursive (i, count);
+	}
+
+	return count;
+}
+
+unsigned int
+objects_in_document (SPDocument *document)
+{
+	return count_objects_recursive (SP_DOCUMENT_ROOT(document), 0);
+}
+
+void
+vacuum_document_recursive (SPObject *obj)
+{
+	if (SP_IS_DEFS(obj)) {
+		for (SPObject *def = obj->firstChild(); def; def = SP_OBJECT_NEXT(def)) {
+                 /* fixme: some inkscape-internal nodes in the future might not be collectable */
+			def->requestOrphanCollection();
+		}
+	} else {
+		for (SPObject *i = sp_object_first_child(obj); i != NULL; i = SP_OBJECT_NEXT(i)) {
+			vacuum_document_recursive (i);
+		}
+	}
+}
+
+unsigned int
+vacuum_document (SPDocument *document)
+{
+	unsigned int start = objects_in_document (document);
+	unsigned int end;
+	unsigned int newend = start;
+
+	unsigned int iterations = 0;
+
+	do {
+		end = newend;
+
+		vacuum_document_recursive (SP_DOCUMENT_ROOT(document));
+		document->collectOrphans();
+		iterations ++;
+
+		newend = objects_in_document (document);
+
+	} while (iterations < 100 && newend < end);
+
+	return start - newend;
+}
+
 
