@@ -4,9 +4,11 @@
 /*
  * Typed SVG document implementation
  *
- * Author:
+ * Authors:
  *   Lauris Kaplinski <lauris@kaplinski.com>
+ *   MenTaLguY <mental@rydia.net>
  *
+ * Copyright (C) 2004-2005 MenTaLguY
  * Copyright (C) 1999-2002 Lauris Kaplinski
  * Copyright (C) 2000-2001 Ximian, Inc.
  *
@@ -20,24 +22,51 @@
 #include <sigc++/class_slot.h>
 
 #include "forward.h"
+#include "libcroco/cr-cascade.h"
 #include "libnr/nr-forward.h"
-#include "xml/xml-forward.h"
+
+#include "gc-managed.h"
+#include "gc-finalized.h"
+#include "gc-anchored.h"
+
+namespace Inkscape {
+namespace XML {
+class Node;
+}
+}
+
+namespace Inkscape {
+namespace XML {
+class Document;
+}
+}
+
+namespace Inkscape { class Selection; }
 
 class SPDocumentPrivate;
 
-struct SPDocument {
+struct SPDocument : public Inkscape::GC::Managed<>,
+                    public Inkscape::GC::Finalized,
+		    public Inkscape::GC::Anchored
+{
 	typedef sigc::signal<void, SPObject *> IDChangedSignal;
 	typedef sigc::signal<void> ResourcesChangedSignal;
+	typedef sigc::signal<void, guint> ModifiedSignal;
+	typedef sigc::signal<void, gchar const *> URISetSignal;
+	typedef sigc::signal<void, double, double> ResizedSignal;
+	typedef sigc::signal<void> ReconstructionStart;
+	typedef sigc::signal<void> ReconstructionFinish;
 
-	GObject object;
+	SPDocument();
+	~SPDocument();
 
-	unsigned int advertize : 1;
 	unsigned int keepalive : 1;
 	unsigned int virgin    : 1; /* Has the document never been touched? */
 
-	SPReprDoc *rdoc; /* Our SPReprDoc */
-	SPRepr *rroot; /* Root element of SPReprDoc */
+	Inkscape::XML::Document *rdoc; /* Our Inkscape::XML::Document */
+	Inkscape::XML::Node *rroot; /* Root element of Inkscape::XML::Document */
 	SPObject *root; /* Our SPRoot */
+	CRCascade *style_cascade;
 
 	gchar *uri; /* URI string or NULL */
 	gchar *base;
@@ -50,33 +79,42 @@ struct SPDocument {
 	/* Handler ID */
 	guint modified_id;
 
+	sigc::connection connectModified(ModifiedSignal::slot_type slot);
+	sigc::connection connectURISet(URISetSignal::slot_type slot);
+	sigc::connection connectResized(ResizedSignal::slot_type slot);
+
 	void bindObjectToId(gchar const *id, SPObject *object);
 	SPObject *getObjectById(gchar const *id);
-	sigc::connection connectIdChanged(const gchar *id, SPDocument::IDChangedSignal::slot_type slot);
+	sigc::connection connectIdChanged(const gchar *id, IDChangedSignal::slot_type slot);
 
-	void bindObjectToRepr(SPRepr *repr, SPObject *object);
-	SPObject *getObjectByRepr(SPRepr *repr);
+	void bindObjectToRepr(Inkscape::XML::Node *repr, SPObject *object);
+	SPObject *getObjectByRepr(Inkscape::XML::Node *repr);
 
 	void queueForOrphanCollection(SPObject *object);
 	void collectOrphans();
 
+	void _emitModified();
+
 	GSList *_collection_queue;
-};
 
-struct SPDocumentClass {
-	GObjectClass parent_class;
+private:
+	SPDocument(SPDocument const &); // no copy
+	void operator=(SPDocument const &); // no assign
 
-	void (* modified) (SPDocument *document, guint flags);
-	void (* uri_set) (SPDocument *document, const gchar *uri);
-	void (* resized) (SPDocument *document, gdouble width, gdouble height);
+public:
+	sigc::connection connectReconstructionStart(ReconstructionStart::slot_type slot);
+	sigc::connection connectReconstructionFinish (ReconstructionFinish::slot_type  slot);
+	void emitReconstructionStart (void);
+	void emitReconstructionFinish  (void);
 };
 
 /* Fetches document from URI, or creates new, if NULL
  * Public document appear in document list
  */
 
-SPDocument *sp_document_new (const gchar *uri, unsigned int advertize, unsigned int keepalive, bool make_new = false);
-SPDocument *sp_document_new_from_mem (const gchar *buffer, gint length, unsigned int advertize, unsigned int keepalive);
+SPDocument *sp_document_new (const gchar *uri, unsigned int keepalive, bool make_new = false);
+SPDocument *sp_document_new_from_mem (const gchar *buffer, gint length, unsigned int keepalive);
+SPDocument *sp_document_new_dummy(); 
 
 SPDocument *sp_document_ref (SPDocument *doc);
 SPDocument *sp_document_unref (SPDocument *doc);
@@ -85,16 +123,16 @@ SPDocument *sp_document_unref (SPDocument *doc);
  * Access methods
  */
 
-#define sp_document_repr_doc(d) (SP_DOCUMENT (d)->rdoc)
-#define sp_document_repr_root(d) (SP_DOCUMENT (d)->rroot)
-#define sp_document_root(d) (SP_DOCUMENT (d)->root)
-#define SP_DOCUMENT_ROOT(d)  (SP_DOCUMENT (d)->root)
+#define sp_document_repr_doc(d) (d->rdoc)
+#define sp_document_repr_root(d) (d->rroot)
+#define sp_document_root(d) (d->root)
+#define SP_DOCUMENT_ROOT(d)  (d->root)
 
 gdouble sp_document_width (SPDocument * document);
 gdouble sp_document_height (SPDocument * document);
-#define SP_DOCUMENT_URI(d) (SP_DOCUMENT (d)->uri)
-#define SP_DOCUMENT_NAME(d) (SP_DOCUMENT (d)->name)
-#define SP_DOCUMENT_BASE(d) (SP_DOCUMENT (d)->base)
+#define SP_DOCUMENT_URI(d) (d->uri)
+#define SP_DOCUMENT_NAME(d) (d->name)
+#define SP_DOCUMENT_BASE(d) (d->base)
 
 /*
  * Dictionary
@@ -110,11 +148,11 @@ gboolean sp_document_get_undo_sensitive (SPDocument const * document);
 void sp_document_clear_undo (SPDocument * document);
 void sp_document_clear_redo (SPDocument * document);
 
-void sp_document_child_added (SPDocument *doc, SPObject *object, SPRepr *child, SPRepr *ref);
-void sp_document_child_removed (SPDocument *doc, SPObject *object, SPRepr *child, SPRepr *ref);
+void sp_document_child_added (SPDocument *doc, SPObject *object, Inkscape::XML::Node *child, Inkscape::XML::Node *ref);
+void sp_document_child_removed (SPDocument *doc, SPObject *object, Inkscape::XML::Node *child, Inkscape::XML::Node *ref);
 void sp_document_attr_changed (SPDocument *doc, SPObject *object, const gchar *key, const gchar *oldval, const gchar *newval);
 void sp_document_content_changed (SPDocument *doc, SPObject *object, const gchar *oldcontent, const gchar *newcontent);
-void sp_document_order_changed (SPDocument *doc, SPObject *object, SPRepr *child, SPRepr *oldref, SPRepr *newref);
+void sp_document_order_changed (SPDocument *doc, SPObject *object, Inkscape::XML::Node *child, Inkscape::XML::Node *oldref, Inkscape::XML::Node *newref);
 
 /* Object modification root handler */
 void sp_document_request_modified (SPDocument *doc);
@@ -166,6 +204,6 @@ SPItem * sp_document_group_at_point (SPDocument *document, unsigned int key,  NR
 void sp_document_set_uri (SPDocument *document, const gchar *uri);
 void sp_document_set_size_px (SPDocument *doc, gdouble width, gdouble height);
 
-
+unsigned int vacuum_document (SPDocument *document);
 
 #endif
