@@ -6,12 +6,13 @@
  * Authors:
  *   Mitsuru Oka <oka326@parkcity.ne.jp>
  *   Lauris Kaplinski <lauris@kaplinski.com>
+ *   bulia byak <buliabyak@users.sf.net>
  *
  * The original dynadraw code:
  *   Paul Haeberli <paul@sgi.com>
  *
  * Copyright (C) 1998 The Free Software Foundation
- * Copyright (C) 1999-2002 authors
+ * Copyright (C) 1999-2005 authors
  * Copyright (C) 2001-2002 Ximian, Inc.
  *
  * Released under GNU GPL, read the file 'COPYING' for more information
@@ -52,7 +53,7 @@
 #include "desktop-affine.h"
 #include "desktop-style.h"
 #include "snap.h"
-#include "sp-desktop-widget.h"
+#include "desktop-widget.h"
 #include "dyna-draw-context.h"
 #include "widgets/spw-utilities.h"
 #include "libnr/n-art-bpath.h"
@@ -567,7 +568,7 @@ sp_dyna_draw_context_root_handler(SPEventContext *event_context,
         }
         break;
     case GDK_KEY_PRESS:
-        switch (event->key.keyval) {
+        switch (get_group0_keyval (&event->key)) {
         case GDK_Up:
         case GDK_KP_Up:
             if (!MOD__CTRL_ONLY) {
@@ -660,7 +661,7 @@ set_to_accumulated(SPDynaDrawContext *dc)
 
         if (!dc->repr) {
             /* Create object */
-            SPRepr *repr = sp_repr_new("svg:path");
+            Inkscape::XML::Node *repr = sp_repr_new("svg:path");
 
             /* Set style */
             sp_desktop_apply_style_tool (desktop, repr, "tools.calligraphic", false);
@@ -671,7 +672,7 @@ set_to_accumulated(SPDynaDrawContext *dc)
             sp_repr_unref(dc->repr);
             item->transform = SP_ITEM(desktop->currentRoot())->getRelativeTransform(desktop->currentLayer());
             item->updateRepr();
-            SP_DT_SELECTION(desktop)->setRepr(dc->repr);
+            SP_DT_SELECTION(desktop)->set(dc->repr);
         }
         abp = nr_artpath_affine(sp_curve_first_bpath(dc->accumulated), sp_desktop_dt2root_affine(desktop));
         str = sp_svg_write_path(abp);
