@@ -8,6 +8,7 @@
  */
 
 #include <helper/action.h>
+#include <document.h>
 
 #include "prefdialog.h"
 
@@ -21,7 +22,7 @@ namespace Extension {
 
 Effect * Effect::_last_effect = NULL;
 
-Effect::Effect (SPRepr * in_repr, Implementation::Implementation * in_imp)
+Effect::Effect (Inkscape::XML::Node * in_repr, Implementation::Implementation * in_imp)
     : Extension(in_repr, in_imp), _verb(get_id(), get_name(), NULL, NULL, this)
 {
     return;
@@ -47,14 +48,14 @@ Effect::prefs (SPView * doc)
         set_state(Extension::STATE_LOADED);
     if (!loaded()) return false;
 
-    Gdk::NativeWindow plug;
-    plug = imp->prefs_effect(this, doc);
-    if (plug == 0) {
-        std::cout << "No preferences for Effect" << std::endl;
+    Gtk::Widget * controls;
+    controls = imp->prefs_effect(this, doc);
+    if (controls == NULL) {
+        // std::cout << "No preferences for Effect" << std::endl;
         return true;
     }
 
-    PrefDialog * dialog = new PrefDialog(this->get_name(), plug);
+    PrefDialog * dialog = new PrefDialog(this->get_name(), controls);
     int response = dialog->run();
     dialog->hide();
 
@@ -65,6 +66,16 @@ Effect::prefs (SPView * doc)
     return false;
 }
 
+/**
+    \brief  The function that 'does' the effect itself
+    \param  doc  The SPView to do the effect on
+
+    This function first insures that the extension is loaded, and if not,
+    loads it.  It then calls the implemention to do the actual work.  It
+    also resets the last effect pointer to be this effect.  Finally, it
+    executes a \c sp_document_done to commit the changes to the undo
+    stack.
+*/
 void
 Effect::effect (SPView * doc)
 {
@@ -73,7 +84,11 @@ Effect::effect (SPView * doc)
     if (!loaded()) return;
 
     _last_effect = this;
-    return imp->effect(this, doc);
+    imp->effect(this, doc);
+
+    sp_document_done(doc->doc);
+
+    return;
 }
 
 
@@ -100,7 +115,7 @@ Effect::EffectVerb::perform (SPAction *action, void * data, void *pdata)
     if (effect == NULL) return;
     if (current_view == NULL) return;
 
-    std::cout << "Executing: " << effect->get_name() << std::endl;
+    // std::cout << "Executing: " << effect->get_name() << std::endl;
     if (effect->prefs(current_view))
         effect->effect(current_view);
 
