@@ -16,13 +16,16 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
 
 #include <ostream>
 #include <fstream>
 #include <string.h>
 #include <stdlib.h>
 
+#include "path-prefix.h"
 #include "xml/repr.h"
 #include <glibmm/i18n.h>
 #include "dir-util.h"
@@ -37,17 +40,21 @@
 #include "db.h"
 #include "dependency.h"
 #include "timer.h"
+#include "parameter.h"
 
 namespace Inkscape {
 namespace Extension {
 
 /* Inkscape::Extension::Extension */
 
+std::vector<const gchar *> Extension::search_path;
 std::ofstream Extension::error_file;
 
+Parameter * param_shared (const gchar * name, GSList * list);
+
 /**
     \return  none
-    \brief   Constructs an Extension from a SPRepr
+    \brief   Constructs an Extension from a Inkscape::XML::Node
     \param   in_repr    The repr that should be used to build it
 
     This function is the basis of building an extension for Inkscape.  It
@@ -56,7 +63,7 @@ std::ofstream Extension::error_file;
     not related to the module directly.  If the Repr does not include
     a name and an ID the module will be left in an errored state.
 */
-Extension::Extension (SPRepr * in_repr, Implementation::Implementation * in_imp)
+Extension::Extension (Inkscape::XML::Node * in_repr, Implementation::Implementation * in_imp)
 {
     repr = in_repr;
     sp_repr_ref(in_repr);
@@ -74,20 +81,23 @@ Extension::Extension (SPRepr * in_repr, Implementation::Implementation * in_imp)
 
 //  printf("Extension Constructor: ");
     if (repr != NULL) {
-        SPRepr *child_repr = sp_repr_children(repr);
+        Inkscape::XML::Node *child_repr = sp_repr_children(repr);
         /* TODO: Handle what happens if we don't have these two */
         while (child_repr != NULL) {
-            if (!strcmp(sp_repr_name(child_repr), "id")) {
-                gchar const *val = sp_repr_content(sp_repr_children(child_repr));
+            if (!strcmp(child_repr->name(), "id")) {
+                gchar const *val = sp_repr_children(child_repr)->content();
                 id = g_strdup (val);
             } /* id */
-            if (!strcmp(sp_repr_name(child_repr), "name")) {
-                name = g_strdup (sp_repr_content(sp_repr_children(child_repr)));
+            if (!strcmp(child_repr->name(), "name")) {
+                name = g_strdup (sp_repr_children(child_repr)->content());
             } /* name */
-            if (!strcmp(sp_repr_name(child_repr), "param")) {
-                make_param(child_repr);
+            if (!strcmp(child_repr->name(), "param")) {
+				Parameter * param;
+				param = Parameter::make(child_repr, this);
+				if (param != NULL)
+					parameters = g_slist_append(parameters, param);
             } /* param */
-            if (!strcmp(sp_repr_name(child_repr), "dependency")) {
+            if (!strcmp(child_repr->name(), "dependency")) {
                 _deps.push_back(new Dependency(child_repr));
             } /* param */
             child_repr = sp_repr_next(child_repr);
@@ -280,7 +290,7 @@ Extension::printFailure (Glib::ustring reason)
     \return  The XML tree that is used to define the extension
     \brief   A getter for the internal Repr, does not add a reference.
 */
-SPRepr *
+Inkscape::XML::Node *
 Extension::get_repr (void)
 {
     return repr;
@@ -344,102 +354,6 @@ Extension::deactivated (void)
 	return get_state() == STATE_DEACTIVATED;
 }
 
-/**
-    \return None
-    \brief  This function creates a parameter that can be used later.  This
-            is typically done in the creation of the extension and defined
-            in the XML file describing the extension (it's private so people
-            have to use the system) :)
-    \param  paramrepr  The XML describing the parameter
-
-    This function first grabs all of the data out of the Repr and puts
-    it into local variables.  Actually, these are just pointers, and the
-    data is not duplicated so we need to be careful with it.  If there
-    isn't a name or a type in the XML, then no parameter is created as
-    the function just returns.
-
-    From this point on, we're pretty committed as we've allocated an
-    object and we're starting to fill it.  The name is set first, and
-    is created with a strdup to actually allocate memory for it.  Then
-    there is a case statement (roughly because strcmp requires 'ifs')
-    based on what type of parameter this is.  Depending which type it
-    is, the value is interpreted differently, but they are relatively
-    straight forward.  In all cases the value is set to the default
-    value from the XML and the type is set to the interpreted type.
-
-    Finally the allocated parameter is put into the GSList that is called
-    parameteres.
-
-    \todo This function should pull up parameters that are stored
-    in the preferences somewhere.  This needs to be figured out.
-*/
-void
-Extension::make_param (SPRepr * paramrepr)
-{
-    const char * name;
-    const char * type;
-    const char * defaultval;
-	gchar * param_name;
-    Extension::param_t * param;
-
-    name = sp_repr_attr(paramrepr, "name");
-    type = sp_repr_attr(paramrepr, "type");
-    // defaultval = sp_repr_content(paramrepr);
-    defaultval = sp_repr_content(sp_repr_children(paramrepr));
-
-    /* In this case we just don't have enough information */
-    if (name == NULL || type == NULL) {
-        return;
-    }
-
-    param = new param_t();
-    param->name = g_strdup(name);
-	param_name = g_strdup_printf("%s.%s", id, name);
-    if (!strcmp(type, "boolean")) {
-        param->type = Extension::PARAM_BOOL;
-        if (defaultval != NULL && !strcmp(defaultval, "TRUE")) {
-            param->val.t_bool = TRUE;
-        } else {
-            param->val.t_bool = FALSE;
-        }
-		param->val.t_bool = prefs_get_int_attribute("extensions", param_name, (gint)param->val.t_bool);
-    } else if (!strcmp(type, "int")) { 
-        param->type = Extension::PARAM_INT;
-        if (defaultval != NULL) {
-            param->val.t_int = atoi(defaultval);
-        } else {
-            param->val.t_int = 0;
-        }
-		param->val.t_int = prefs_get_int_attribute("extensions", param_name, (gint)param->val.t_int);
-    } else if (!strcmp(type, "float")) { 
-        param->type = Extension::PARAM_FLOAT;
-		// std::cout << "Float value: " << defaultval;
-        if (defaultval != NULL) {
-            param->val.t_float = atof(defaultval);
-        } else {
-            param->val.t_float = 0.0;
-        }
-		param->val.t_float = prefs_get_double_attribute("extensions", param_name, (gfloat)param->val.t_float);
-		// std::cout << " after: " << param->val.t_float << std::endl;
-    } else if (!strcmp(type, "string")) { 
-		const gchar * temp_str;
-        param->type = Extension::PARAM_STRING;
-
-		temp_str = prefs_get_string_attribute("extensions", param_name);
-		if (temp_str == NULL)
-			temp_str = defaultval;
-
-        param->val.t_string = g_strdup(temp_str);
-    } else {
-		g_free(param_name);
-        return;
-    }
-	g_free(param_name);
-
-    parameters = g_slist_append(parameters, param);
-    return;
-}
-
 /**
     \return    Parameter structure with a name of 'name'
     \brief     This function looks through the linked list for a parameter
@@ -466,10 +380,10 @@ Extension::make_param (SPRepr * paramrepr)
     name then that param_t is returned.  Otherwise, this function is
     called again with g_slist_next as a parameter.
 */
-inline Extension::param_t *
-Extension::param_shared (const gchar * name, GSList * list)
+Parameter *
+param_shared (const gchar * name, GSList * list)
 {
-    Extension::param_t * output;
+    Parameter * output;
     
     if (name == NULL) {
         throw Extension::param_not_exist();
@@ -478,12 +392,12 @@ Extension::param_shared (const gchar * name, GSList * list)
         throw Extension::param_not_exist();
     }
 
-    output = static_cast<Extension::param_t *>(list->data);
-    if (!strcmp(output->name, name)) {
+    output = static_cast<Parameter *>(list->data);
+    if (!strcmp(output->name(), name)) {
         return output;
     }
 
-    return Extension::param_shared(name, g_slist_next(list));
+    return param_shared(name, g_slist_next(list));
 }
 
 /**
@@ -493,31 +407,16 @@ Extension::param_shared (const gchar * name, GSList * list)
     \param    name    The name of the parameter to get
 	\param    doc    The document to look in for document specific parameters
 
-    To get the parameter to be used the function param_shared is called.
-    This function is inline so it shouldn't cause the stack to build
-    or anything like that.  If it can't find the parameter, it will
-    throw and exception - we aren't catching that because we want
-    the calling function to catch it.
-
-    Next up, the parameter that we got, we're making sure that it is
-    a string parameter.  If it isn't, then we throw a param_wrong_type
-    exception.
-    
-    Finally, if everything is okay, the string value that is stored in
-    the parameter is placed in value.
+	Look up in the parameters list, then execute the function on that
+	found parameter.
 */
 const gchar *
-Extension::get_param_string (const gchar * name, const SPReprDoc * doc)
+Extension::get_param_string (const gchar * name, const Inkscape::XML::Document * doc)
 {
-    Extension::param_t * param;
+    Parameter * param;
     
-    param = Extension::param_shared(name, parameters);
-
-    if (param->type != Extension::PARAM_STRING) {
-        throw Extension::param_wrong_type();
-    }
-
-    return param->val.t_string;
+    param = param_shared(name, parameters);
+	return param->get_string(doc);
 }
 
 /**
@@ -527,31 +426,16 @@ Extension::get_param_string (const gchar * name, const SPReprDoc * doc)
     \param    name    The name of the parameter to get
 	\param    doc    The document to look in for document specific parameters
 
-    To get the parameter to be used the function param_shared is called.
-    This function is inline so it shouldn't cause the stack to build
-    or anything like that.  If it can't find the parameter, it will
-    throw and exception - we aren't catching that because we want
-    the calling function to catch it.
-
-    Next up, the parameter that we got, we're making sure that it is
-    a bool parameter.  If it isn't, then we throw a param_wrong_type
-    exception.
-    
-    Finally, if everything is okay, the boolean value that is stored in
-    the parameter is placed in value.
+	Look up in the parameters list, then execute the function on that
+	found parameter.
 */
 bool
-Extension::get_param_bool (const gchar * name, const SPReprDoc * doc)
+Extension::get_param_bool (const gchar * name, const Inkscape::XML::Document * doc)
 {
-    Extension::param_t * param;
+    Parameter * param;
     
-    param = Extension::param_shared(name, parameters);
-
-    if (param->type != Extension::PARAM_BOOL) {
-        throw Extension::param_wrong_type();
-    }
-
-    return param->val.t_bool;
+    param = param_shared(name, parameters);
+    return param->get_bool(doc);
 }
 
 /**
@@ -561,31 +445,16 @@ Extension::get_param_bool (const gchar * name, const SPReprDoc * doc)
     \param    name    The name of the parameter to get
 	\param    doc    The document to look in for document specific parameters
 
-    To get the parameter to be used the function param_shared is called.
-    This function is inline so it shouldn't cause the stack to build
-    or anything like that.  If it can't find the parameter, it will
-    throw and exception - we aren't catching that because we want
-    the calling function to catch it.
-
-    Next up, the parameter that we got, we're making sure that it is
-    a integer parameter.  If it isn't, then we throw a param_wrong_type
-    exception.
-    
-    Finally, if everything is okay, the integer value that is stored in
-    the parameter is placed in value.
+	Look up in the parameters list, then execute the function on that
+	found parameter.
 */
 int
-Extension::get_param_int (const gchar * name, const SPReprDoc * doc)
+Extension::get_param_int (const gchar * name, const Inkscape::XML::Document * doc)
 {
-    Extension::param_t * param;
+    Parameter * param;
     
-    param = Extension::param_shared(name, parameters);
-
-    if (param->type != Extension::PARAM_INT) {
-        throw Extension::param_wrong_type();
-    }
-
-    return param->val.t_int;
+    param = param_shared(name, parameters);
+    return param->get_int(doc);
 }
 
 /**
@@ -595,31 +464,15 @@ Extension::get_param_int (const gchar * name, const SPReprDoc * doc)
     \param    name    The name of the parameter to get
 	\param    doc    The document to look in for document specific parameters
 
-    To get the parameter to be used the function param_shared is called.
-    This function is inline so it shouldn't cause the stack to build
-    or anything like that.  If it can't find the parameter, it will
-    throw and exception - we aren't catching that because we want
-    the calling function to catch it.
-
-    Next up, the parameter that we got, we're making sure that it is
-    a float parameter.  If it isn't, then we throw a param_wrong_type
-    exception.
-    
-    Finally, if everything is okay, the float value that is stored in
-    the parameter is placed in value.
+	Look up in the parameters list, then execute the function on that
+	found parameter.
 */
 float
-Extension::get_param_float (const gchar * name, const SPReprDoc * doc)
+Extension::get_param_float (const gchar * name, const Inkscape::XML::Document * doc)
 {
-    Extension::param_t * param;
-    
-    param = Extension::param_shared(name, parameters);
-
-    if (param->type != Extension::PARAM_FLOAT) {
-        throw Extension::param_wrong_type();
-    }
-
-    return param->val.t_float;
+    Parameter * param;
+    param = param_shared(name, parameters);
+    return param->get_float(doc);
 }
 
 /**
@@ -630,38 +483,15 @@ Extension::get_param_float (const gchar * name, const SPReprDoc * doc)
     \param    value   The value to set the parameter to
 	\param    doc    The document to look in for document specific parameters
 
-    To get the parameter to be used the function param_shared is called.
-    This function is inline so it shouldn't cause the stack to build
-    or anything like that.  If it can't find the parameter, it will
-    throw and exception - we aren't catching that because we want
-    the calling function to catch it.
-
-    Next up, the parameter that we got, we're making sure that it is
-    a boolean parameter.  If it isn't, then we throw a param_wrong_type
-    exception.
-    
-    Finally, if everything is okay, the boolean value that was passed
-	in is placed in the param.
+	Look up in the parameters list, then execute the function on that
+	found parameter.
 */
 bool
-Extension::set_param_bool (const gchar * name, bool value, SPReprDoc * doc)
+Extension::set_param_bool (const gchar * name, bool value, Inkscape::XML::Document * doc)
 {
-    Extension::param_t * param;
-	gchar * param_name;
-    
-    param = Extension::param_shared(name, parameters);
-
-    if (param->type != Extension::PARAM_BOOL) {
-        throw Extension::param_wrong_type();
-    }
-
-    param->val.t_bool = value;
-
-	param_name = g_strdup_printf("%s.%s", id, name);
-	prefs_set_int_attribute("extensions", param_name, value == TRUE ? 1 : 0);
-	g_free(param_name);
-
-    return value;
+    Parameter * param;
+    param = param_shared(name, parameters);
+	return param->set_bool(value, doc);
 }
 
 /**
@@ -672,38 +502,15 @@ Extension::set_param_bool (const gchar * name, bool value, SPReprDoc * doc)
     \param    value   The value to set the parameter to
 	\param    doc    The document to look in for document specific parameters
 
-    To get the parameter to be used the function param_shared is called.
-    This function is inline so it shouldn't cause the stack to build
-    or anything like that.  If it can't find the parameter, it will
-    throw and exception - we aren't catching that because we want
-    the calling function to catch it.
-
-    Next up, the parameter that we got, we're making sure that it is
-    a integer parameter.  If it isn't, then we throw a param_wrong_type
-    exception.
-    
-    Finally, if everything is okay, the integer value that was passed
-	in is placed in the param.
+	Look up in the parameters list, then execute the function on that
+	found parameter.
 */
 int
-Extension::set_param_int (const gchar * name, int value, SPReprDoc * doc)
+Extension::set_param_int (const gchar * name, int value, Inkscape::XML::Document * doc)
 {
-    Extension::param_t * param;
-	gchar * param_name;
-    
-    param = Extension::param_shared(name, parameters);
-
-    if (param->type != Extension::PARAM_INT) {
-        throw Extension::param_wrong_type();
-    }
-
-    param->val.t_int = value;
-
-	param_name = g_strdup_printf("%s.%s", id, name);
-	prefs_set_int_attribute("extensions", param_name, value);
-	g_free(param_name);
-
-    return value;
+    Parameter * param;
+    param = param_shared(name, parameters);
+	return param->set_int(value, doc);
 }
 
 /**
@@ -714,38 +521,15 @@ Extension::set_param_int (const gchar * name, int value, SPReprDoc * doc)
     \param    value   The value to set the parameter to
 	\param    doc    The document to look in for document specific parameters
 
-    To get the parameter to be used the function param_shared is called.
-    This function is inline so it shouldn't cause the stack to build
-    or anything like that.  If it can't find the parameter, it will
-    throw and exception - we aren't catching that because we want
-    the calling function to catch it.
-
-    Next up, the parameter that we got, we're making sure that it is
-    a integer parameter.  If it isn't, then we throw a param_wrong_type
-    exception.
-    
-    Finally, if everything is okay, the integer value that was passed
-	in is placed in the param.
+	Look up in the parameters list, then execute the function on that
+	found parameter.
 */
 float
-Extension::set_param_float (const gchar * name, float value, SPReprDoc * doc)
+Extension::set_param_float (const gchar * name, float value, Inkscape::XML::Document * doc)
 {
-    Extension::param_t * param;
-	gchar * param_name;
-    
-    param = Extension::param_shared(name, parameters);
-
-    if (param->type != Extension::PARAM_FLOAT) {
-        throw Extension::param_wrong_type();
-    }
-
-    param->val.t_float = value;
-
-	param_name = g_strdup_printf("%s.%s", id, name);
-	prefs_set_double_attribute("extensions", param_name, value);
-	g_free(param_name);
-
-    return value;
+    Parameter * param;
+    param = param_shared(name, parameters);
+	return param->set_float(value, doc);
 }
 
 /**
@@ -756,48 +540,15 @@ Extension::set_param_float (const gchar * name, float value, SPReprDoc * doc)
     \param    value   The value to set the parameter to
 	\param    doc    The document to look in for document specific parameters
 
-	First this function makes sure that the incoming string is not
-	NULL.  The value can't be set to NULL.
-
-    To get the parameter to be used the function param_shared is called.
-    This function is inline so it shouldn't cause the stack to build
-    or anything like that.  If it can't find the parameter, it will
-    throw and exception - we aren't catching that because we want
-    the calling function to catch it.
-
-    Next up, the parameter that we got, we're making sure that it is
-    a string parameter.  If it isn't, then we throw a param_wrong_type
-    exception.
-    
-    Finally, if everything is okay, the previous value is free'd and
-	the incoming value is duplicated and placed in the parameter.
+	Look up in the parameters list, then execute the function on that
+	found parameter.
 */
 const gchar *
-Extension::set_param_string (const gchar * name, const gchar * value, SPReprDoc * doc)
+Extension::set_param_string (const gchar * name, const gchar * value, Inkscape::XML::Document * doc)
 {
-    Extension::param_t * param;
-	gchar * param_name;
-    
-    if (value == NULL) {
-        /* This probably isn't a good error, but the calling function
-           will find out soon enough it doesn't have data there ;) */
-        return value;
-    }
-
-    param = Extension::param_shared(name, parameters);
-
-    if (param->type != Extension::PARAM_STRING) {
-        throw Extension::param_wrong_type();
-    }
-
-    g_free(param->val.t_string);
-    param->val.t_string = g_strdup(value);
-
-	param_name = g_strdup_printf("%s.%s", id, name);
-	prefs_set_string_attribute("extensions", param_name, value);
-	g_free(param_name);
-
-    return value;
+    Parameter * param;
+    param = param_shared(name, parameters);
+	return param->set_string(value, doc);
 }
 
 /** \brief A function to open the error log file. */
@@ -822,6 +573,60 @@ Extension::error_file_close (void)
 	error_file.close();
 };
 
+/** \brief  A function to automatically generate a GUI using the parameters
+	\return Generated widget
+
+	This function just goes through each parameter, and calls it's 'get_widget'
+	function to get each widget.  Then, each of those is placed into
+	a Gtk::VBox, which is then returned to the calling function.
+
+	If there are no parameters, this function just returns NULL.
+*/
+Gtk::Widget *
+Extension::autogui (void)
+{
+	if (g_slist_length(parameters) == 0) return NULL;
+
+	Gtk::VBox * vbox = new Gtk::VBox();
+    vbox = new Gtk::VBox();
+
+	for (GSList * list = parameters; list != NULL; list = g_slist_next(list)) {
+		Parameter * param = reinterpret_cast<Parameter *>(list->data);
+		Gtk::Widget * widg = param->get_widget();
+		if (widg != NULL)
+			vbox->pack_start(*widg, true, true);
+	}
+
+	vbox->show();
+	return vbox;
+};
+
+/**
+	\brief  A function to get the parameters in a string form
+	\return A string with all the parameters as command line arguements
+
+	I don't really like this function, but it works for now.
+
+	\todo  Do this better.
+*/
+Glib::ustring *
+Extension::paramString (void)
+{
+	Glib::ustring * param_string = new Glib::ustring("");
+
+	for (GSList * list = parameters; list != NULL; list = g_slist_next(list)) {
+		Parameter * param = reinterpret_cast<Parameter *>(list->data);
+
+		*param_string += " --";
+		*param_string += param->name();
+		*param_string += "=";
+		Glib::ustring * paramstr = param->string();
+		*param_string += *paramstr;
+		delete paramstr;
+	}
+
+	return param_string;
+}
+
 }  /* namespace Extension */
 }  /* namespace Inkscape */
-
