@@ -17,6 +17,7 @@
 #include <ostream>
 #include <fstream>
 #include <vector>
+#include <gtkmm/widget.h>
 #include <glibmm/ustring.h>
 #include "xml/repr.h"
 #include <extension/extension-forward.h>
@@ -30,13 +31,17 @@
 #define SP_MODULE_KEY_INPUT_DEFAULT SP_MODULE_KEY_AUTODETECT
 /** The key for outputing standard W3C SVG */
 #define SP_MODULE_KEY_OUTPUT_SVG "org.inkscape.output.svg.plain"
+#define SP_MODULE_KEY_OUTPUT_SVGZ "org.inkscape.output.svgz.plain"
 /** This is an output file that has SVG data with the Sodipodi namespace extensions */
 #define SP_MODULE_KEY_OUTPUT_SVG_INKSCAPE "org.inkscape.output.svg.inkscape"
+#define SP_MODULE_KEY_OUTPUT_SVGZ_INKSCAPE "org.inkscape.output.svgz.inkscape"
 /** Which output module should be used? */
 #define SP_MODULE_KEY_OUTPUT_DEFAULT SP_MODULE_KEY_AUTODETECT
 
 /** Defines the key for Postscript printing */
 #define SP_MODULE_KEY_PRINT_PS    "org.inkscape.print.ps"
+/** Defines the key for LaTeX printing */
+#define SP_MODULE_KEY_PRINT_LATEX    "org.inkscape.print.latex"
 /** Defines the key for printing with GNOME Print */
 #define SP_MODULE_KEY_PRINT_GNOME "org.inkscape.print.gnome"
 /** Defines the key for printing under Win32 */
@@ -74,8 +79,9 @@ public:
     typedef enum {
         STATE_LOADED,      /**< The extension has been loaded successfully */
         STATE_UNLOADED,    /**< The extension has not been loaded */
-        STATE_DEACTIVATED, /**< The extension is missing something which makes it unusable */
+        STATE_DEACTIVATED  /**< The extension is missing something which makes it unusable */
     } state_t;
+    static std::vector<const gchar *> search_path; /**< A vector of paths to search for extensions */
 
 private:
     gchar     *id;                        /**< The unique identifier for the Extension */
@@ -85,12 +91,12 @@ private:
     static std::ofstream error_file;      /**< This is the place where errors get reported */
 
 protected:
-    SPRepr *repr;                         /**< The XML description of the Extension */
+    Inkscape::XML::Node *repr;                         /**< The XML description of the Extension */
     Implementation::Implementation * imp; /**< An object that holds all the functions for making this work */
     ExpirationTimer * timer;              /**< Timeout to unload after a given time */
 
 public:
-                  Extension    (SPRepr * in_repr,
+                  Extension    (Inkscape::XML::Node * in_repr,
                                 Implementation::Implementation * in_imp);
     virtual      ~Extension    (void);
 
@@ -98,7 +104,7 @@ public:
     state_t       get_state    (void);
     bool          loaded       (void);
     virtual bool  check        (void);
-    SPRepr *      get_repr     (void);
+    Inkscape::XML::Node *      get_repr     (void);
     gchar *       get_id       (void);
     gchar *       get_name     (void);
     void          deactivate   (void);
@@ -107,42 +113,11 @@ public:
 
 
 /* Parameter Stuff */
-    /* This is what allows modules to save values that are persistent
-       through a reset and while running.  These values should be created
-       when the extension is initialized, but will pull from previously
-       set values if they are available.
-     */
 private:
     GSList * parameters; /**< A table to store the parameters for this extension.
                               This only gets created if there are parameters in this
                               extension */
-    /** This is an enum to define the parameter type */
-    enum param_type_t {
-        PARAM_BOOL,   /**< Boolean parameter */
-        PARAM_INT,    /**< Integer parameter */
-        PARAM_FLOAT,  /**< Float parameter */
-        PARAM_STRING, /**< String parameter */
-        PARAM_CNT     /**< How many types are there? */
-    };
-    /** A union to select between the types of parameters.  They will
-        probbably all fit within a pointer of various systems, but making
-        them a union ensures this */
-    union param_switch_t {
-        bool t_bool;      /**< To get a boolean use this */
-        int  t_int;       /**< If you want an integer this is your variable */
-        float t_float;    /**< This guy stores the floating point numbers */
-        gchar * t_string; /**< Strings are here */
-    };
-    /** The class that actually stores the value and type of the
-        variable.  It should really take up a very small space.  It's
-        important to note that the name is stored as the key to the
-        hash table. */
-    class param_t {
-    public:
-        gchar * name;        /**< The name of this parameter */
-        param_type_t type;   /**< What type of variable */
-        param_switch_t val;  /**< Here is the actual value */
-    };
+
 public:
     class param_wrong_type {}; /**< An error class for when a parameter is
                                     called on a type it is not */
@@ -152,35 +127,41 @@ public:
                                     already exists, but the user doesn't
                                     want to overwrite it */
 private:
-    void             make_param       (SPRepr * paramrepr);
+    void             make_param       (Inkscape::XML::Node * paramrepr);
+#if 0
     inline param_t * param_shared     (const gchar * name,
                                        GSList * list);
+#endif
 public:
     bool             get_param_bool   (const gchar * name,
-                                       const SPReprDoc *   doc = NULL);
+                                       const Inkscape::XML::Document *   doc = NULL);
     int              get_param_int    (const gchar * name,
-                                       const SPReprDoc *   doc = NULL);
+                                       const Inkscape::XML::Document *   doc = NULL);
     float            get_param_float  (const gchar * name,
-                                       const SPReprDoc *   doc = NULL);
+                                       const Inkscape::XML::Document *   doc = NULL);
     const gchar *    get_param_string (const gchar * name,
-                                       const SPReprDoc *   doc = NULL);
+                                       const Inkscape::XML::Document *   doc = NULL);
     bool             set_param_bool   (const gchar * name,
                                        bool          value,
-                                       SPReprDoc *   doc = NULL);
+                                       Inkscape::XML::Document *   doc = NULL);
     int              set_param_int    (const gchar * name,
                                        int           value,
-                                       SPReprDoc *   doc = NULL);
+                                       Inkscape::XML::Document *   doc = NULL);
     float            set_param_float  (const gchar * name,
                                        float         value,
-                                       SPReprDoc *   doc = NULL);
+                                       Inkscape::XML::Document *   doc = NULL);
     const gchar *    set_param_string (const gchar * name,
                                        const gchar * value,
-                                       SPReprDoc *   doc = NULL);
+                                       Inkscape::XML::Document *   doc = NULL);
 
     /* Error file handling */
 public:
     static void      error_file_open  (void);
     static void      error_file_close (void);
+
+public:
+    Gtk::Widget *    autogui (void);
+    Glib::ustring *  paramString (void);
 };
 
 
