@@ -42,18 +42,18 @@ Implementation::check (Inkscape::Extension::Extension * module) {
 	return TRUE;
 } /* Implemenation::check */
 
-GtkDialog *
+Gtk::Widget *
 Implementation::prefs_input (Inkscape::Extension::Input * module, const gchar * filename) {
 	return NULL;
 } /* Implementation::prefs_input */
 
 SPDocument *
 Implementation::open (Inkscape::Extension::Input * module, const gchar * filename) {
-	/* throw open_failed(); */
-	return NULL;
+    /* throw open_failed(); */
+    return NULL;
 } /* Implementation::open */
 
-GtkDialog *
+Gtk::Widget *
 Implementation::prefs_output (Inkscape::Extension::Output * module) {
 	return NULL;
 } /* Implementation::prefs_output */
@@ -64,7 +64,7 @@ Implementation::save (Inkscape::Extension::Output * module, SPDocument * doc, co
 	return;
 } /* Implementation::save */
 
-Gdk::NativeWindow
+Gtk::Widget *
 Implementation::prefs_effect (Inkscape::Extension::Effect * module, SPView * view) {
 	return (Gdk::NativeWindow)0;
 } /* Implementation::prefs_effect */
