@@ -14,6 +14,7 @@
 
 #include <gtk/gtkdialog.h>
 #include <gdkmm/types.h>
+#include <gtkmm/widget.h>
 
 #include <forward.h>
 #include <extension/extension-forward.h>
@@ -31,6 +32,11 @@ namespace Implementation {
  */
 class Implementation {
 public:
+    /* ----- Constructor / destructor ----- */
+    Implementation() {}
+    
+    virtual ~Implementation() {}
+
     /* ----- Basic functions for all Extension ----- */
     virtual bool load(Inkscape::Extension::Extension *module);
 
@@ -42,7 +48,7 @@ public:
 
     /* ----- Input functions ----- */
     /** Find out information about the file. */
-    virtual GtkDialog *prefs_input(Inkscape::Extension::Input *module,
+    virtual Gtk::Widget *prefs_input(Inkscape::Extension::Input *module,
                              gchar const *filename);
 
     virtual SPDocument *open(Inkscape::Extension::Input *module,
@@ -50,12 +56,12 @@ public:
 
     /* ----- Output functions ----- */
     /** Find out information about the file. */
-    virtual GtkDialog *prefs_output(Inkscape::Extension::Output *module);
+    virtual Gtk::Widget *prefs_output(Inkscape::Extension::Output *module);
     virtual void save(Inkscape::Extension::Output *module, SPDocument *doc, gchar const *filename);
 
     /* ----- Effect functions ----- */
     /** Find out information about the file. */
-    virtual Gdk::NativeWindow prefs_effect(Inkscape::Extension::Effect *module, SPView * view);
+    virtual Gtk::Widget * prefs_effect(Inkscape::Extension::Effect *module, SPView * view);
     /* TODO: need to figure out what we need here */
 
     virtual void effect(Inkscape::Extension::Effect *module,
