@@ -14,7 +14,13 @@
 #define __INKSCAPE_EXTENSION_IMPEMENTATION_SCRIPT_H__
 
 #include "implementation.h"
-#include "xml/xml-forward.h"
+
+namespace Inkscape {
+namespace XML {
+class Node;
+}
+}
+
 
 namespace Inkscape {
 namespace Extension {
@@ -31,28 +37,30 @@ private:
                                     data */
     /** This function actually does the work, everything else is preparing
         for this function.  It is the core here */
-    void          execute      (gchar const *command,
+    int           execute      (gchar const *command,
                                 gchar const *filein,
                                 gchar const *fileout);
     /** Just a quick function to find and resolve relative paths for
         the incoming scripts */
-    gchar *       solve_reldir (SPRepr *reprin);
+    gchar *       solve_reldir (Inkscape::XML::Node *reprin);
     bool          check_existance (gchar const *command);
+    void          copy_doc (Inkscape::XML::Node * olddoc, Inkscape::XML::Node * newdoc);
+    void          checkStderr (gchar * filename, Gtk::MessageType type, gchar * message);
 
 public:
                           Script       (void);
     virtual bool          load         (Inkscape::Extension::Extension *module);
     virtual void          unload       (Inkscape::Extension::Extension *module);
     virtual bool          check        (Inkscape::Extension::Extension *module);
-    virtual GtkDialog *   prefs_input  (Inkscape::Extension::Input *module,
+    virtual Gtk::Widget * prefs_input  (Inkscape::Extension::Input *module,
                                         gchar const *filename);
     virtual SPDocument *  open         (Inkscape::Extension::Input *module,
                                         gchar const *filename);
-    virtual GtkDialog *   prefs_output (Inkscape::Extension::Output *module);
+    virtual Gtk::Widget * prefs_output (Inkscape::Extension::Output *module);
     virtual void          save         (Inkscape::Extension::Output *module,
                                         SPDocument *doc,
                                         gchar const *filename);
-    virtual Gdk::NativeWindow
+    virtual Gtk::Widget *
                           prefs_effect (Inkscape::Extension::Effect *module,
                                         SPView * view);
     virtual void          effect       (Inkscape::Extension::Effect *module,
