@@ -5,6 +5,7 @@
  *
  * Author:
  *   Lauris Kaplinski <lauris@kaplinski.com>
+ *   bulia byak <buliabyak@users.sf.net>
  *
  * Basic printing code, EXCEPT image and
  * ascii85 filter is in public domain
@@ -21,7 +22,9 @@
 
 /* Plain Print */
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
 
 #include <string.h>
 #include <ctype.h>
@@ -33,6 +36,8 @@
 #include <libnr/n-art-bpath.h>
 #include <libnr/nr-macros.h>
 #include <libnr/nr-matrix.h>
+#include <libnr/nr-matrix-ops.h>
+#include <libnr/nr-point-matrix-ops.h>
 
 #include <glib.h>
 #include <gtk/gtkstock.h>
@@ -55,6 +60,11 @@
 #include "sp-item.h"
 #include "style.h"
 #include "inkscape_version.h"
+#include <sp-gradient.h>
+#include <sp-gradient-vector.h>
+#include <sp-linear-gradient.h>
+#include <sp-pattern.h>
+#include <sp-radial-gradient.h>
 
 #include <libnrtype/FontFactory.h>
 #include <libnrtype/font-instance.h>
@@ -104,7 +114,7 @@ PrintPS::setup(Inkscape::Extension::Print * mod)
     int response;
     unsigned int ret;
 #ifdef TED
-    SPRepr *repr;
+    Inkscape::XML::Node *repr;
 #endif
     bool p2bm;
 
@@ -143,8 +153,8 @@ PrintPS::setup(Inkscape::Extension::Print * mod)
     rb = gtk_radio_button_new_with_label(NULL, _("Print using PostScript operators"));
     gtk_tooltips_set_tip((GtkTooltips *) tt, rb,
                          _("Use PostScript vector operators. The resulting image is usually smaller "
-                           "in file size and can be arbitrarily scaled, but alpha transparency, "
-                           "gradients and patterns will be lost."), NULL);
+                           "in file size and can be arbitrarily scaled, but alpha transparency "
+                           "and patterns will be lost."), NULL);
     if (!p2bm) gtk_toggle_button_set_active((GtkToggleButton *) rb, TRUE);
     gtk_box_pack_start(GTK_BOX(vb), rb, FALSE, FALSE, 0);
     rb = gtk_radio_button_new_with_label(gtk_radio_button_get_group((GtkRadioButton *) rb), _("Print as bitmap"));
@@ -161,7 +171,7 @@ PrintPS::setup(Inkscape::Extension::Print * mod)
     gtk_combo_set_value_in_list(GTK_COMBO(combo), FALSE, FALSE);
     gtk_combo_set_use_arrows(GTK_COMBO(combo), TRUE);
     gtk_combo_set_use_arrows_always(GTK_COMBO(combo), TRUE);
-    gtk_widget_set_usize(combo, 64, -1);
+    gtk_widget_set_size_request(combo, 64, -1);
     gtk_tooltips_set_tip((GtkTooltips *) tt, GTK_COMBO(combo)->entry,
                          _("Preferred resolution (dots per inch) of bitmap"), NULL);
     /* Setup strings */
@@ -218,6 +228,8 @@ PrintPS::setup(Inkscape::Extension::Print * mod)
         _dpi = (unsigned int) MAX((int)(atof(sstr)), 1);
         /* Arrgh, have to do something */
         fn = gtk_entry_get_text(GTK_ENTRY(e));
+        /* skip leading whitespace, bug #1068483 */
+        while (fn && *fn==' ') { fn++; }
         /* g_print("Printing to %s\n", fn); */
 
         mod->set_param_bool("bitmap", _bitmap);
@@ -238,6 +250,10 @@ PrintPS::begin(Inkscape::Extension::Print *mod, SPDocument *doc)
     int res;
     FILE *osf, *osp;
     const gchar * fn;
+    gboolean epsexport=false;
+
+    _latin1_encoded_fonts.clear();
+    _newlatin1font_proc_defined = false;
 
     fn = mod->get_param_string("destination");
 
@@ -273,6 +289,7 @@ PrintPS::begin(Inkscape::Extension::Print *mod, SPDocument *doc)
             _stream = osp;
         } else if (*fn == '>') {
             fn += 1;
+            epsexport = g_str_has_suffix(fn,".eps");
             while (isspace(*fn)) fn += 1;
             Inkscape::IO::dump_fopen_call(fn, "K");
             osf = Inkscape::IO::fopen_utf8name(fn, "w+");
@@ -311,7 +328,10 @@ PrintPS::begin(Inkscape::Extension::Print *mod, SPDocument *doc)
 #endif
     }
 
-    res = fprintf(_stream, "%%!PS-Adobe-2.0\n");
+    if (epsexport)
+        res = fprintf(_stream, "%%!PS-Adobe-3.0 EPSF-3.0\n");
+    else
+        res = fprintf(_stream, "%%!PS-Adobe-3.0\n");
     /* flush this to test output stream as early as possible */
     if (fflush(_stream)) {
         /*g_print("caught error in sp_module_print_plain_begin\n");*/
@@ -373,6 +393,7 @@ PrintPS::begin(Inkscape::Extension::Print *mod, SPDocument *doc)
                  (w > 0. && h > 0.) // empty documents fail this sanity check, have w<0, h<0
                  && (w > h)   // implies, but does not prove, the user wanted landscape
                  && (w > 600) // approximate maximum printable width of an A4
+                 && (!epsexport) // eps should always be portrait
              )
              ? true : false;
         }
@@ -508,6 +529,7 @@ PrintPS::finish(Inkscape::Extension::Print *mod)
     /* fixme: should really use pclose for popen'd streams */
     fclose(_stream);
     _stream = 0;
+    _latin1_encoded_fonts.clear();
 
     return res;
 }
@@ -548,14 +570,103 @@ PrintPS::comment(Inkscape::Extension::Print *mod, const char * comment)
 }
 
 void
-PrintPS::print_fill_style(SVGOStringStream &os, const SPStyle *style)
+PrintPS::print_fill_style(SVGOStringStream &os, const SPStyle *style, const NRRect *pbox)
 {
-    g_return_if_fail(style->fill.type == SP_PAINT_TYPE_COLOR);
+    g_return_if_fail(style->fill.type == SP_PAINT_TYPE_COLOR || SP_IS_GRADIENT (SP_STYLE_FILL_SERVER (style)));
+    
+    if (style->fill.type == SP_PAINT_TYPE_COLOR)
+    {
+        float rgb[3];
+        sp_color_get_rgb_floatv(&style->fill.value.color, rgb);
 
-    float rgb[3];
-    sp_color_get_rgb_floatv(&style->fill.value.color, rgb);
+        os << rgb[0] << " " << rgb[1] << " " << rgb[2] << " setrgbcolor\n";
 
-    os << rgb[0] << " " << rgb[1] << " " << rgb[2] << " setrgbcolor\n";
+    } else if (SP_IS_GRADIENT (SP_STYLE_FILL_SERVER (style))) {
+        if (SP_IS_LINEARGRADIENT (SP_STYLE_FILL_SERVER (style))) {
+
+            SPLinearGradient *lg=SP_LINEARGRADIENT(SP_STYLE_FILL_SERVER (style));
+            NR::Point p1 (lg->x1.computed, lg->y1.computed);
+            NR::Point p2 (lg->x2.computed, lg->y2.computed);
+            if (pbox && SP_GRADIENT(lg)->units == SP_GRADIENT_UNITS_OBJECTBOUNDINGBOX) {
+                // convert to userspace
+                NR::Matrix bbox2user(pbox->x1 - pbox->x0, 0, 0, pbox->y1 - pbox->y0, pbox->x0, pbox->y0);
+                p1 *= bbox2user;
+                p2 *= bbox2user;
+            }
+
+            os << "<<\n/ShadingType 2\n/ColorSpace /DeviceRGB\n";
+            os << "/Coords [" << p1[NR::X] << " " << p1[NR::Y] << " " << p2[NR::X] << " " << p2[NR::Y] <<"]\n";
+            os << "/Extend [true true]\n";
+            os << "/Domain [0 1]\n";
+            os << "/Function <<\n/FunctionType 3\n/Functions\n[\n";
+
+            sp_gradient_ensure_vector(SP_GRADIENT(lg)); // when exporting from commandline, vector is not built
+            for (gint i = 0; unsigned(i) < lg->vector.stops.size() - 1; i++) {
+                float rgb[3];
+                sp_color_get_rgb_floatv(&lg->vector.stops[i].color, rgb);
+                os << "<<\n/FunctionType 2\n/Domain [0 1]\n";
+                os << "/C0 [" << rgb[0] << " " << rgb[1] << " " << rgb[2] << "]\n";
+                sp_color_get_rgb_floatv(&lg->vector.stops[i+1].color, rgb);
+                os << "/C1 [" << rgb[0] << " " << rgb[1] << " " << rgb[2] << "]\n";
+                os << "/N 1\n>>\n";
+            }
+            os << "]\n/Domain [0 1]\n";
+            os << "/Bounds [ ";
+            for (gint i=0;unsigned(i)<lg->vector.stops.size()-2;i++) {
+                os << lg->vector.stops[i+1].offset <<" ";
+            }
+            os << "]\n";
+            os << "/Encode [ ";
+            for (gint i=0;unsigned(i)<lg->vector.stops.size()-1;i++) {
+                os << "0 1 ";
+            }
+            os << "]\n";
+            os << ">>\n>>\n";
+
+        } else if (SP_IS_RADIALGRADIENT (SP_STYLE_FILL_SERVER (style))) {
+
+            SPRadialGradient *rg=SP_RADIALGRADIENT(SP_STYLE_FILL_SERVER (style));
+            NR::Point c (rg->cx.computed, rg->cy.computed);
+            NR::Point f (rg->fx.computed, rg->fy.computed);
+            double r = rg->r.computed;
+            if (pbox && SP_GRADIENT(rg)->units == SP_GRADIENT_UNITS_OBJECTBOUNDINGBOX) {
+                // convert to userspace
+                NR::Matrix bbox2user(pbox->x1 - pbox->x0, 0, 0, pbox->y1 - pbox->y0, pbox->x0, pbox->y0);
+                c *= bbox2user;
+                f *= bbox2user;
+                r *= bbox2user.expansion();
+            }
+
+            os << "<<\n/ShadingType 3\n/ColorSpace /DeviceRGB\n";
+            os << "/Coords ["<< f[NR::X] <<" "<< f[NR::Y] <<" 0 "<< c[NR::X] <<" "<< c[NR::Y] <<" "<< r <<"]\n";
+            os << "/Extend [true true]\n";
+            os << "/Domain [0 1]\n";
+            os << "/Function <<\n/FunctionType 3\n/Functions\n[\n";
+
+            sp_gradient_ensure_vector(SP_GRADIENT(rg)); // when exporting from commandline, vector is not built
+            for (gint i = 0; unsigned(i) < rg->vector.stops.size() - 1; i++) {
+                float rgb[3];
+                sp_color_get_rgb_floatv(&rg->vector.stops[i].color, rgb);
+                os << "<<\n/FunctionType 2\n/Domain [0 1]\n";
+                os << "/C0 [" << rgb[0] << " " << rgb[1] << " " << rgb[2] << "]\n";
+                sp_color_get_rgb_floatv(&rg->vector.stops[i+1].color, rgb);
+                os << "/C1 [" << rgb[0] << " " << rgb[1] << " " << rgb[2] << "]\n";
+                os << "/N 1\n>>\n";
+            }
+            os << "]\n/Domain [0 1]\n";
+            os << "/Bounds [ ";
+            for (gint i=0;unsigned(i)<rg->vector.stops.size()-2;i++) {
+                os << rg->vector.stops[i+1].offset <<" ";
+            }
+            os << "]\n";
+            os << "/Encode [ ";
+            for (gint i=0;unsigned(i)<rg->vector.stops.size()-1;i++) {
+                os << "0 1 ";
+            }
+            os << "]\n";
+            os << ">>\n>>\n";
+        }
+    }
 }
 
 void
@@ -566,18 +677,31 @@ PrintPS::print_stroke_style(SVGOStringStream &os, const SPStyle *style)
 
     os << rgb[0] << " " << rgb[1] << " " << rgb[2] << " setrgbcolor\n";
 
+    // There are rare cases in which for a solid line stroke_dasharray_set is true. To avoid 
+    // invalid PS-lines such as "[0.0000000 0.0000000] 0.0000000 setdash", which should be "[] 0 setdash",
+    // we first check if all components of stroke_dash.dash are 0.
+    bool LineSolid = true;
     if (style->stroke_dasharray_set &&
         style->stroke_dash.n_dash &&
-        style->stroke_dash.dash) {
-        int i;
-        os << "[";
-        for (i = 0; i < style->stroke_dash.n_dash; i++) {
-            if ((i)) {
-                os << " ";
+        style->stroke_dash.dash ) {
+        int i = 0;
+        while (LineSolid && (i < style->stroke_dash.n_dash)) {
+                i++;
+                if (style->stroke_dash.dash[i] > 0.00000001)
+                    LineSolid = false;
+        }
+        if (!LineSolid) {
+            os << "[";
+            for (i = 0; i < style->stroke_dash.n_dash; i++) {
+                if (i > 0) {
+                    os << " ";
+                }
+                os << style->stroke_dash.dash[i];
             }
-            os << style->stroke_dash.dash[i];
+            os << "] " << style->stroke_dash.offset << " setdash\n";
+        } else {
+            os << "[] 0 setdash\n";
         }
-        os << "] " << style->stroke_dash.offset << " setdash\n";
     } else {
         os << "[] 0 setdash\n";
     }
@@ -595,20 +719,53 @@ PrintPS::fill(Inkscape::Extension::Print *mod, const NRBPath *bpath, const NRMat
     if (!_stream) return 0; // XXX: fixme, returning -1 as unsigned.
     if (_bitmap) return 0;
 
-    if (style->fill.type == SP_PAINT_TYPE_COLOR) {
+    if (style->fill.type == SP_PAINT_TYPE_COLOR || SP_IS_GRADIENT (SP_STYLE_FILL_SERVER (style))) {
         Inkscape::SVGOStringStream os;
 
-        print_fill_style(os, style);
+        os << "gsave\n";
+
+        print_fill_style(os, style, pbox);
 
         print_bpath(os, bpath->path);
 
         if (style->fill_rule.value == SP_WIND_RULE_EVENODD) {
-            os << "eofill\n";
+            if (style->fill.type == SP_PAINT_TYPE_COLOR) {
+                os << "eofill\n";
+            } else if (SP_IS_GRADIENT (SP_STYLE_FILL_SERVER (style))) {
+                SPGradient *g=SP_GRADIENT(SP_STYLE_FILL_SERVER (style));
+                os << "eoclip\n";
+                if (g->gradientTransform_set) {
+                    os << "gsave [" << g->gradientTransform[0] << " " << g->gradientTransform[1] 
+                        << " " << g->gradientTransform[2] << " " << g->gradientTransform[3] 
+                        << " " << g->gradientTransform[4] << " " << g->gradientTransform[5] << "] concat\n"; 
+                }
+                os << "shfill\n";
+                if (g->gradientTransform_set) {
+                    os << "grestore\n";
+                }
+            }
         } else {
-            os << "fill\n";
+            if (style->fill.type == SP_PAINT_TYPE_COLOR) {
+                os << "fill\n";
+            } else if (SP_IS_GRADIENT (SP_STYLE_FILL_SERVER (style))) {
+                SPGradient *g=SP_GRADIENT(SP_STYLE_FILL_SERVER (style));
+                os << "clip\n";
+                if (g->gradientTransform_set) {
+                    os << "gsave [" << g->gradientTransform[0] << " " << g->gradientTransform[1] 
+                        << " " << g->gradientTransform[2] << " " << g->gradientTransform[3] 
+                        << " " << g->gradientTransform[4] << " " << g->gradientTransform[5] << "] concat\n"; 
+                }
+                os << "shfill\n";
+                if (g->gradientTransform_set) {
+                    os << "grestore\n";
+                }
+            }
         }
+
+        os << "grestore\n";
+
         fprintf(_stream, "%s", os.str().c_str());
-    }
+    }        
 
     return 0;
 }
@@ -724,40 +881,66 @@ PrintPS::text(Inkscape::Extension::Print *mod, const char *text, NR::Point p,
     Inkscape::SVGOStringStream os;
 
     // Escape chars
-    // FlowResOut feeds us text char by char
-    if (!strcmp(text, "(")) {
-        text = "\\(";
-    } else if (!strcmp(text, ")")) {
-        text = "\\)";
-    } else if (!strcmp(text, "\\")) {
-        text = "\\\\";
+    Inkscape::SVGOStringStream escaped_text;
+    escaped_text << std::oct;
+    for (const gchar *p_text = text ; *p_text ; p_text = g_utf8_next_char(p_text)) {
+        gunichar c = g_utf8_get_char(p_text);
+        if (c == '\\' || c == ')' || c == '(')
+            escaped_text << '\\' << static_cast<char>(c);
+        else if (c >= 0x80)
+            escaped_text << '\\' << c;
+        else
+            escaped_text << static_cast<char>(c);
     }
 
+    os << "gsave\n";
+
     // set font
     const char *fn = PSFontName(style);
-    os << "/" << fn << " findfont\n";
+    if (_latin1_encoded_fonts.find(fn) == _latin1_encoded_fonts.end()) {
+        if (!_newlatin1font_proc_defined) {
+            // input: newfontname, existingfontname
+            // output: new font object, also defined to newfontname
+            os << "/newlatin1font "         // name of the proc
+                  "{findfont dup length dict copy "     // load the font and create a copy of it
+                  "dup /Encoding ISOLatin1Encoding put "     // change the encoding in the copy
+                  "definefont} def\n";      // create the new font and leave it on the stack, define the proc
+            _newlatin1font_proc_defined = true;
+        }
+        os << "/" << fn << "-ISOLatin1 /" << fn << " newlatin1font\n";
+        _latin1_encoded_fonts.insert(fn);
+    } else
+        os << "/" << fn << "-ISOLatin1 findfont\n";
     os << style->font_size.computed << " scalefont\n";
     os << "setfont\n";
     g_free((void *) fn);
 
-    if (style->fill.type == SP_PAINT_TYPE_COLOR) {
+    if (style->fill.type == SP_PAINT_TYPE_COLOR || SP_IS_GRADIENT (SP_STYLE_FILL_SERVER (style))) {
+
         // set fill style
-        print_fill_style(os, style);
-        // paint fill
+        print_fill_style(os, style, NULL); 
+        // FIXME: we don't know the pbox of text, so have to pass NULL. This means gradients with
+        // bbox units won't work with text. However userspace gradients don't work with text either
+        // (text is black) for some reason.
+
         os << "newpath\n";
         os << p[NR::X] << " " << p[NR::Y] << " moveto\n";
-        os << "(" << text << ") show\n";
+        os << "(" << escaped_text.str() << ") show\n";
     }
 
     if (style->stroke.type == SP_PAINT_TYPE_COLOR) {
+
         // set stroke style
         print_stroke_style(os, style);
+
         // paint stroke
         os << "newpath\n";
         os << p[NR::X] << " " << p[NR::Y] << " moveto\n";
-        os << "(" << text << ") false charpath stroke\n";
+        os << "(" << escaped_text.str() << ") false charpath stroke\n";
     }
 
+    os << "grestore\n";
+
     fprintf(_stream, "%s", os.str().c_str());
 
     return 0;
