@@ -32,14 +32,14 @@
 #include "effect.h"
 #include "print.h"
 #include "implementation/script.h"
-#include "implementation/plugin.h"
+/* #include "implementation/plugin.h" */
 
 namespace Inkscape {
 namespace Extension {
 
 static void open_internal(Inkscape::Extension::Extension *in_plug, gpointer in_data);
 static void save_internal(Inkscape::Extension::Extension *in_plug, gpointer in_data);
-static Extension *build_from_reprdoc(SPReprDoc *doc, Implementation::Implementation *in_imp);
+static Extension *build_from_reprdoc(Inkscape::XML::Document *doc, Implementation::Implementation *in_imp);
 
 /**
  * \return   A new document created from the filename passed in
@@ -90,10 +90,8 @@ open(Extension *key, gchar const *filename)
         throw Input::open_failed();
     }
 
-    GtkDialog *prefs = imod->prefs(filename);
-    if (prefs != NULL) {
-        gtk_dialog_run(prefs);
-    }
+    if (!imod->prefs(filename))
+        return NULL;
 
     SPDocument *doc = imod->open(filename);
     if (!doc) {
@@ -110,7 +108,7 @@ open(Extension *key, gchar const *filename)
 
     /* This kinda overkill as most of these are already set, but I want
        to make sure for this release -- TJG */
-    SPRepr *repr = sp_document_repr_root(doc);
+    Inkscape::XML::Node *repr = sp_document_repr_root(doc);
     gboolean saved = sp_document_get_undo_sensitive(doc);
     sp_document_set_undo_sensitive(doc, FALSE);
     sp_repr_set_attr(repr, "sodipodi:modified", NULL);
@@ -148,7 +146,7 @@ open_internal(Extension *in_plug, gpointer in_data)
         gchar const *filename = (gchar const *)parray[0];
         Input **pimod = (Input **)parray[1];
 
-        // skip all the rest if we already found someone to open it
+        // skip all the rest if we already found a function to open it
         // since they're ordered by preference now.
         if (!*pimod) {
             gchar const *ext = dynamic_cast<Input *>(in_plug)->get_extension();
@@ -227,17 +225,8 @@ save(Extension *key, SPDocument *doc, gchar const *filename, bool setextension,
         throw Output::save_failed();
     }
 
-    GtkDialog *prefs = omod->prefs();
-    if (prefs != NULL) {
-        gint response;
-
-        response = gtk_dialog_run(prefs);
-        gtk_widget_hide(GTK_WIDGET(prefs));
-
-        if (response != GTK_RESPONSE_OK) {
-            return;
-        }
-    }
+    if (!omod->prefs())
+        return;
 
     gchar *fileName = NULL;
     if (setextension) {
@@ -326,7 +315,7 @@ get_print(gchar const *key)
 
 /**
  * \return   The built module
- * \brief    Creates a module from a SPReprDoc describing the module
+ * \brief    Creates a module from a Inkscape::XML::Document describing the module
  * \param    doc  The XML description of the module
  *
  * This function basically has two segments.  The first is that it goes through the Repr tree
@@ -340,11 +329,11 @@ get_print(gchar const *key)
  * case could apply to modules that are built in (like the SVG load/save functions).
  */
 static Extension *
-build_from_reprdoc(SPReprDoc *doc, Implementation::Implementation *in_imp)
+build_from_reprdoc(Inkscape::XML::Document *doc, Implementation::Implementation *in_imp)
 {
     enum {
         MODULE_EXTENSION,
-        MODULE_PLUGIN,
+        /* MODULE_PLUGIN, */
         MODULE_UNKNOWN_IMP
     } module_implementation_type = MODULE_UNKNOWN_IMP;
     enum {
@@ -357,19 +346,19 @@ build_from_reprdoc(SPReprDoc *doc, Implementation::Implementation *in_imp)
 
     g_return_val_if_fail(doc != NULL, NULL);
 
-    SPRepr *repr = sp_repr_document_root(doc);
+    Inkscape::XML::Node *repr = sp_repr_document_root(doc);
 
     /* sp_repr_print(repr); */
 
-    if (strcmp(sp_repr_name(repr), "inkscape-extension")) {
-        g_warning("Extension definition started with <%s> instead of <inkscape-extension>.  Extension will not be created.\n", sp_repr_name(repr));
+    if (strcmp(repr->name(), "inkscape-extension")) {
+        g_warning("Extension definition started with <%s> instead of <inkscape-extension>.  Extension will not be created.\n", repr->name());
         return NULL;
     }
 
-    SPRepr *child_repr = sp_repr_children(repr);
+    Inkscape::XML::Node *child_repr = sp_repr_children(repr);
     while (child_repr != NULL) {
-        char const *element_name = sp_repr_name(child_repr);
-        /* printf("Child: %s\n", sp_repr_name(child_repr)); */
+        char const *element_name = child_repr->name();
+        /* printf("Child: %s\n", child_repr->name()); */
         if (!strcmp(element_name, "input")) {
             module_functional_type = MODULE_INPUT;
         } else if (!strcmp(element_name, "output")) {
@@ -380,11 +369,13 @@ build_from_reprdoc(SPReprDoc *doc, Implementation::Implementation *in_imp)
             module_functional_type = MODULE_PRINT;
         } else if (!strcmp(element_name, "script")) {
             module_implementation_type = MODULE_EXTENSION;
+#if 0
         } else if (!strcmp(element_name, "plugin")) {
             module_implementation_type = MODULE_PLUGIN;
+#endif
         }
 
-        //SPRepr *old_repr = child_repr;
+        //Inkscape::XML::Node *old_repr = child_repr;
         child_repr = sp_repr_next(child_repr);
         //sp_repr_unref(old_repr);
     }
@@ -397,11 +388,13 @@ build_from_reprdoc(SPReprDoc *doc, Implementation::Implementation *in_imp)
                 imp = static_cast<Implementation::Implementation *>(script);
                 break;
             }
+#if 0
             case MODULE_PLUGIN: {
                 Implementation::Plugin *plugin = new Implementation::Plugin();
                 imp = static_cast<Implementation::Implementation *>(plugin);
                 break;
             }
+#endif
             default: {
                 imp = NULL;
                 break;
@@ -450,9 +443,9 @@ build_from_file(gchar const *filename)
 {
     /* TODO: Need to define namespace here, need to write the
        DTD in general for this stuff */
-    SPReprDoc *doc = sp_repr_read_file(filename, NULL);
+    Inkscape::XML::Document *doc = sp_repr_read_file(filename, NULL);
     Extension *ext = build_from_reprdoc(doc, NULL);
-    sp_repr_document_unref(doc);
+    Inkscape::GC::release(doc);
     if (ext == NULL)
         g_warning("Unable to create extension from definition file %s.\n", filename);
     return ext;
@@ -470,9 +463,9 @@ build_from_file(gchar const *filename)
 Extension *
 build_from_mem(gchar const *buffer, Implementation::Implementation *in_imp)
 {
-    SPReprDoc *doc = sp_repr_read_mem(buffer, strlen(buffer), NULL);
+    Inkscape::XML::Document *doc = sp_repr_read_mem(buffer, strlen(buffer), NULL);
     Extension *ext = build_from_reprdoc(doc, in_imp);
-    sp_repr_document_unref(doc);
+    Inkscape::GC::release(doc);
     return ext;
 }
 
