@@ -6,8 +6,9 @@
  * Authors:
  *   Lauris Kaplinski <lauris@kaplinski.com>
  *   Chema Celorio <chema@celorio.com>
+ *   bulia byak <buliabyak@users.sf.net>
  *
- * Copyright (C) 1999-2002 Authors
+ * Copyright (C) 1999-2005 Authors
  * Copyright (C) 2001-2002 Ximian, Inc.
  * Copyright (C) 2004 David Turner
  *
@@ -20,7 +21,9 @@
  * the API, and two or more .cpp files for the implementations.
  */
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
 
 #include <string.h>
 #include <time.h>
@@ -30,11 +33,11 @@
 #include <gtkmm.h>
 
 #include "macros.h"
-#include "xml/repr-get-children.h"
+#include "xml/node-fns.h"
 #include "document.h"
 #include "document-private.h"
 #include "selection-chemistry.h"
-#include "view.h"
+#include "ui/view/view.h"
 #include "dir-util.h"
 #include "helper/png-write.h"
 #include "dialogs/export.h"
@@ -44,6 +47,7 @@
 #include "selection.h"
 #include "sp-image.h"
 #include "interface.h"
+#include "style.h"
 #include "print.h"
 #include "file.h"
 #include "dialogs/dialog-events.h"
@@ -67,7 +71,6 @@
 
 #include "uri.h"
 
-
 /**
  * 'Current' paths.  Used to remember which directory
  * had the last file accessed.
@@ -96,7 +99,7 @@ void dump_ustr(Glib::ustring const &ustr);
 void
 sp_file_new(gchar const *templ)
 {
-    SPDocument *doc = sp_document_new(templ, TRUE, TRUE, true);
+    SPDocument *doc = sp_document_new(templ, TRUE, true);
     g_return_if_fail(doc != NULL);
 
     SPViewWidget *dtw = sp_desktop_widget_new(sp_document_namedview(doc, NULL));
@@ -110,7 +113,11 @@ sp_file_new(gchar const *templ)
 void
 sp_file_new_default()
 {
-    char *default_template = g_build_filename(INKSCAPE_TEMPLATESDIR, "/default.svg", NULL);
+    // TRANSLATORS: default.svg is localizable - this is the name of the default document
+    //  template. This way you can localize the default pagesize, translate the name of
+    //  the default layer, etc. If you wish to localize this file, please create a
+    //  localized share/templates/default.xx.svg file, where xx is your language code.
+    char *default_template = g_build_filename(INKSCAPE_TEMPLATESDIR, _("default.svg"), NULL);
     if (Inkscape::IO::file_test(default_template, G_FILE_TEST_IS_REGULAR)) {
         sp_file_new(default_template);
     } else {
@@ -159,8 +166,7 @@ sp_file_open(gchar const *uri, Inkscape::Extension::Extension *key, bool add_to_
         if (existing && existing->virgin) {
             // If the current desktop is empty, open the document there
             sp_desktop_change_document(desktop, doc);
-        }
-        else {
+        } else {
             // create a whole new desktop and window
             SPViewWidget *dtw = sp_desktop_widget_new(sp_document_namedview(doc, NULL));
             sp_create_window(dtw, TRUE);
@@ -201,7 +207,7 @@ sp_file_revert_dialog()
     SPDocument *doc = SP_DT_DOCUMENT(desktop);
     g_assert(doc != NULL);
 
-    SPRepr     *repr = sp_document_repr_root(doc);
+    Inkscape::XML::Node     *repr = sp_document_repr_root(doc);
     g_assert(repr != NULL);
 
     gchar const *uri = doc->uri;
@@ -211,7 +217,7 @@ sp_file_revert_dialog()
     }
 
     bool do_revert = true;
-    if (sp_repr_attr(repr, "sodipodi:modified") != NULL) {
+    if (repr->attribute("sodipodi:modified") != NULL) {
         gchar *text = g_strdup_printf(_("Changes will be lost!  Are you sure you want to reload document %s?"), uri);
 
         GtkWidget *dialog = gtk_message_dialog_new(
@@ -269,7 +275,8 @@ void dump_ustr(Glib::ustring const &ustr)
     Glib::ustring::size_type const dataLen = ustr.length();
     Glib::ustring::size_type const cstrLen = strlen(cstr);
 
-    g_message("   size: %d\n   length: %d\n   bytes: %d\n    clen: %d", ustr.size(), dataLen, byteLen, cstrLen );
+    g_message("   size: %lu\n   length: %lu\n   bytes: %lu\n    clen: %lu",
+              gulong(ustr.size()), gulong(dataLen), gulong(byteLen), gulong(cstrLen) );
     g_message( "  ASCII? %s", (ustr.is_ascii() ? "yes":"no") );
     g_message( "  UTF-8? %s", (ustr.validate() ? "yes":"no") );
 
@@ -330,6 +337,7 @@ static Inkscape::UI::Dialogs::FileOpenDialog *openDialogInstance = NULL;
 
 /**
  *  Display an file Open selector.  Open a document if OK is pressed.
+ *  Can select single or multiple files for opening.
  */
 void
 sp_file_open_dialog(gpointer object, gpointer data)
@@ -364,6 +372,66 @@ sp_file_open_dialog(gpointer object, gpointer data)
     g_free(open_path);
 
     if (!success) return;
+
+    // Code to check & open iff multiple files.
+    Glib::SListHandle<Glib::ustring> flist=openDialogInstance->getFilenames();
+    GSList *list=flist.data();
+
+    if(g_slist_length(list)>1)
+    {
+        gchar *fileName=NULL;
+
+        while(list!=NULL)
+        {
+
+#ifdef INK_DUMP_FILENAME_CONV
+            g_message(" FileName: %s",(const char *)list->data);           
+#endif
+
+            fileName=(gchar *)g_strdup((gchar *)list->data);
+
+            if (fileName && !g_file_test(fileName,G_FILE_TEST_IS_DIR)) {
+                gsize bytesRead = 0;
+                gsize bytesWritten = 0;
+                GError *error = NULL;
+#ifdef INK_DUMP_FILENAME_CONV
+                dump_str( fileName, "A file pre  is " );
+#endif
+                gchar *newFileName = g_filename_to_utf8(fileName,
+                                                -1,
+                                                        &bytesRead,
+                                                        &bytesWritten,
+                                                        &error);
+                if ( newFileName != NULL ) {
+                    g_free(fileName);
+                    fileName = newFileName;
+#ifdef INK_DUMP_FILENAME_CONV
+                    dump_str( fileName, "A file post is " );
+#endif
+                } else {
+                    // TODO: bulia, please look over
+                    g_warning( "ERROR CONVERTING OPEN FILENAME TO UTF-8" );
+                }
+
+#ifdef INK_DUMP_FILENAME_CONV                
+                g_message("Opening File %s\n",fileName);
+#endif
+
+                sp_file_open(fileName, selection);
+                g_free(fileName);
+            }
+            else
+            {
+                g_message("Cannot Open Directory %s\n",fileName);
+            }
+            
+            list=list->next;
+        }
+
+        return;
+    }
+
+
     if (fileName) {
         gsize bytesRead = 0;
         gsize bytesWritten = 0;
@@ -421,36 +489,20 @@ void
 sp_file_vacuum()
 {
     SPDocument *doc = SP_ACTIVE_DOCUMENT;
-    SPDefs *defs = SP_ROOT(SP_DOCUMENT_ROOT(doc))->defs;
 
-    int before = 0;
-    for (SPObject *i = sp_object_first_child(defs); i != NULL; i = SP_OBJECT_NEXT(i)) {
-        before++;
-    }
-
-    for (SPObject *def = defs->firstChild(); def; def = SP_OBJECT_NEXT(def)) {
-        /* fixme: some inkscape-internal nodes in the future might not be collectable */
-        def->requestOrphanCollection();
-    }
+    unsigned int diff = vacuum_document (doc);
 
     sp_document_done(doc);
 
-    int after = 0;
-    for (SPObject *i = sp_object_first_child(defs); i != NULL; i = SP_OBJECT_NEXT(i)) {
-        after++;
-    }
-
     SPDesktop *dt = SP_ACTIVE_DESKTOP;
-    if (before - after > 0) {
-        int diff = before - after;
-
+    if (diff > 0) {
         dt->messageStack()->flashF(Inkscape::NORMAL_MESSAGE,
-                ngettext("Removed <b>%i</b> unused item in &lt;defs&gt;.",
-                         "Removed <b>%i</b> unused items in &lt;defs&gt;.",
+                ngettext("Removed <b>%i</b> unused definition in &lt;defs&gt;.",
+                         "Removed <b>%i</b> unused definitions in &lt;defs&gt;.",
                          diff),
                 diff);
     } else {
-        dt->messageStack()->flash(Inkscape::NORMAL_MESSAGE,  _("No unused items in &lt;defs&gt;."));
+        dt->messageStack()->flash(Inkscape::NORMAL_MESSAGE,  _("No unused definitions in &lt;defs&gt;."));
     }
 }
 
@@ -505,13 +557,13 @@ static Inkscape::UI::Dialogs::FileSaveDialog *saveDialogInstance = NULL;
 gboolean
 sp_file_save_dialog(SPDocument *doc)
 {
-    SPRepr *repr = sp_document_repr_root(doc);
+    Inkscape::XML::Node *repr = sp_document_repr_root(doc);
     gchar const *default_extension = NULL;
     gchar *save_loc;
     Inkscape::Extension::Output *extension;
     gchar *save_path = NULL;
 
-    default_extension = sp_repr_attr(repr, "inkscape:output_extension");
+    default_extension = repr->attribute("inkscape:output_extension");
     if (default_extension == NULL) {
         default_extension = prefs_get_string_attribute("dialogs.save_as", "default");
     }
@@ -552,7 +604,7 @@ sp_file_save_dialog(SPDocument *doc)
             g_free(temp_filename);
         }
     } else {
-        save_loc = g_strdup(doc->uri); /* \todo should use a getter */
+        save_loc = g_path_get_dirname(doc->uri); /* \todo should use a getter */
     }
 
     { // convert save_loc from utf-8 to locale
@@ -585,7 +637,7 @@ sp_file_save_dialog(SPDocument *doc)
                  (char const *) _("Select file to save to"),
                  default_extension
             );
-    }
+    } // FIXME: else (i.e. if reshowing an already shown dialog) save_loc is not used, it thus always displays the previously opened dir
     bool success = saveDialogInstance->show();
     char *fileName = ( success
                        ? g_strdup(saveDialogInstance->getFilename())
@@ -651,17 +703,17 @@ sp_file_save_document(SPDocument *doc)
 {
     gboolean success = TRUE;
 
-    SPRepr *repr = sp_document_repr_root(doc);
+    Inkscape::XML::Node *repr = sp_document_repr_root(doc);
 
-    gchar const *fn = sp_repr_attr(repr, "sodipodi:modified");
+    gchar const *fn = repr->attribute("sodipodi:modified");
     if (fn != NULL) {
         if (doc->uri == NULL
-            || sp_repr_attr(repr, "inkscape:output_extension") == NULL)
+            || repr->attribute("inkscape:output_extension") == NULL)
         {
             return sp_file_save_dialog(doc);
         } else {
             fn = g_strdup(doc->uri);
-            gchar const *ext = sp_repr_attr(repr, "inkscape:output_extension");
+            gchar const *ext = repr->attribute("inkscape:output_extension");
             success = file_save(doc, fn, Inkscape::Extension::db.get(ext), FALSE);
             g_free((void *) fn);
         }
@@ -727,7 +779,7 @@ file_import(SPDocument *in_doc, gchar const *uri, Inkscape::Extension::Extension
     if (doc != NULL) {
         // the import extension has passed us a document, now we need to embed it into our document
         if ( 0 ) {
-//            const gchar *docbase = sp_repr_attr( sp_repr_document_root( sp_repr_document( repr ) ), "sodipodi:docbase" );
+//            const gchar *docbase = (sp_repr_document_root( sp_repr_document( repr ))->attribute("sodipodi:docbase" );
             g_message(" settings  uri  [%s]", doc->uri );
             g_message("           base [%s]", doc->base );
             g_message("           name [%s]", doc->name );
@@ -739,33 +791,38 @@ file_import(SPDocument *in_doc, gchar const *uri, Inkscape::Extension::Extension
         // move imported defs to our document's defs
         SPObject *in_defs = SP_DOCUMENT_DEFS(in_doc);
         SPObject *defs = SP_DOCUMENT_DEFS(doc);
-        SPRepr *last_def = sp_repr_last_child(SP_OBJECT_REPR(in_defs));
+        Inkscape::XML::Node *last_def = SP_OBJECT_REPR(in_defs)->lastChild();
         for (SPObject *child = sp_object_first_child(defs);
              child != NULL; child = SP_OBJECT_NEXT(child))
         {
             // FIXME: in case of id conflict, newly added thing will be re-ided and thus likely break a reference to it from imported stuff
-            sp_repr_add_child(SP_OBJECT_REPR(in_defs), sp_repr_duplicate(SP_OBJECT_REPR(child)), last_def);
+            sp_repr_add_child(SP_OBJECT_REPR(in_defs), SP_OBJECT_REPR(child)->duplicate(), last_def);
         }
 
-        SPRepr *repr = sp_document_repr_root(doc);
         guint items_count = 0;
         for (SPObject *child = sp_object_first_child(SP_DOCUMENT_ROOT(doc));
              child != NULL; child = SP_OBJECT_NEXT(child)) {
             if (SP_IS_ITEM(child))
                 items_count ++;
         }
-        gchar const *style = sp_repr_attr(repr, "style");
+        SPCSSAttr *style = sp_css_attr_from_object (SP_DOCUMENT_ROOT (doc));
 
         SPObject *new_obj = NULL;
 
-        if (style || items_count > 1) {
+        if ((style && style->firstChild()) || items_count > 1) {
             // create group
-            SPRepr *newgroup = sp_repr_new("svg:g");
-            sp_repr_set_attr(newgroup, "style", style);
+            Inkscape::XML::Node *newgroup = sp_repr_new("svg:g");
+            sp_repr_css_set (newgroup, style, "style");
 
             for (SPObject *child = sp_object_first_child(SP_DOCUMENT_ROOT(doc)); child != NULL; child = SP_OBJECT_NEXT(child) ) {
                 if (SP_IS_ITEM(child)) {
-                    sp_repr_append_child(newgroup, sp_repr_duplicate(SP_OBJECT_REPR(child)));
+                    Inkscape::XML::Node *newchild = SP_OBJECT_REPR(child)->duplicate();
+
+                    // convert layers to groups; FIXME: add "preserve layers" mode where each layer
+                    // from impot is copied to the same-named layer in host
+                    sp_repr_set_attr (newchild, "inkscape:groupmode", NULL);
+
+                    newgroup->appendChild(newchild);
                 }
             }
 
@@ -783,7 +840,8 @@ file_import(SPDocument *in_doc, gchar const *uri, Inkscape::Extension::Extension
             // just add one item
             for (SPObject *child = sp_object_first_child(SP_DOCUMENT_ROOT(doc)); child != NULL; child = SP_OBJECT_NEXT(child) ) {
                 if (SP_IS_ITEM(child)) {
-                    SPRepr *newitem = sp_repr_duplicate(SP_OBJECT_REPR(child));
+                    Inkscape::XML::Node *newitem = SP_OBJECT_REPR(child)->duplicate();
+                    sp_repr_set_attr (newitem, "inkscape:groupmode", NULL);
 
                     if (desktop) {
                         // Add it to the current layer
@@ -798,10 +856,12 @@ file_import(SPDocument *in_doc, gchar const *uri, Inkscape::Extension::Extension
             }
         }
 
+        if (style) sp_repr_css_attr_unref (style);
+
         // select and move the imported item
         if (new_obj && SP_IS_ITEM(new_obj)) {
-            SPSelection *selection = SP_DT_SELECTION(desktop);
-            selection->setItem(SP_ITEM(new_obj));
+            Inkscape::Selection *selection = SP_DT_SELECTION(desktop);
+            selection->set(SP_ITEM(new_obj));
 
             // To move the imported object, we must temporarily set the "transform pattern with
             // object" option.
@@ -837,7 +897,7 @@ void
 sp_file_import(GtkWidget *widget)
 {
     SPDocument *doc = SP_ACTIVE_DOCUMENT;
-    if (!SP_IS_DOCUMENT(doc))
+    if (!doc)
         return;
 
     if (!importDialogInstance) {
@@ -1017,7 +1077,6 @@ sp_export_png_file(SPDocument *doc, gchar const *filename,
 {
     int write_status = TRUE;
     g_return_val_if_fail(doc != NULL, FALSE);
-    g_return_val_if_fail(SP_IS_DOCUMENT(doc), FALSE);
     g_return_val_if_fail(filename != NULL, FALSE);
     g_return_val_if_fail(width >= 1, FALSE);
     g_return_val_if_fail(height >= 1, FALSE);
@@ -1216,14 +1275,14 @@ void Inkscape::IO::fixupHrefs( SPDocument *doc, const gchar *base, gboolean spns
 
     GSList const *images = sp_document_get_resource_list(doc, "image");
     for (GSList const *l = images; l != NULL; l = l->next) {
-        SPRepr *ir = SP_OBJECT_REPR(l->data);
+        Inkscape::XML::Node *ir = SP_OBJECT_REPR(l->data);
 
-        const gchar *href = sp_repr_attr(ir, "xlink:href");
+        const gchar *href = ir->attribute("xlink:href");
 
         // First try to figure out an absolute path to the asset
         //g_message("image href [%s]", href );
         if (spns && !g_path_is_absolute(href)) {
-            const gchar *absref = sp_repr_attr(ir, "sodipodi:absref");
+            const gchar *absref = ir->attribute("sodipodi:absref");
             //g_message("      absr [%s]", absref );
 
             if ( absref && Inkscape::IO::file_test(absref, G_FILE_TEST_EXISTS) )
