@@ -17,7 +17,7 @@
 
 #include <libnr/nr-macros.h>
 
-#include "action.h"
+#include "helper/action.h"
 
 static void sp_action_class_init (SPActionClass *klass);
 static void sp_action_init (SPAction *action);
@@ -50,11 +50,17 @@ sp_action_class_init (SPActionClass *klass)
 	parent_class = (NRActiveObjectClass *) (((NRObjectClass *) klass)->parent);
 
 	object_class->finalize = sp_action_finalize;
+	object_class->cpp_ctor = NRObject::invoke_ctor<SPAction>;
 }
 
 static void
 sp_action_init (SPAction *action)
 {
+	action->sensitive = 0;
+	action->active = 0;
+	action->view = NULL;
+	action->id = action->name = action->tip = NULL;
+	action->image = NULL;
 }
 
 static void
@@ -73,14 +79,14 @@ sp_action_finalize (NRObject *object)
 }
 
 SPAction *
-sp_action_setup (SPAction *action,
-                 SPView *view,
-		 const gchar *id,
-		 const gchar *name,
-		 const gchar *tip,
-		 const gchar *image)
+sp_action_new(SPView *view,
+              const gchar *id,
+              const gchar *name,
+              const gchar *tip,
+              const gchar *image,
+              Inkscape::Verb * verb)
 {
-	nr_object_setup ((NRObject *) action, SP_TYPE_ACTION);
+	SPAction *action = (SPAction *)nr_object_new(SP_TYPE_ACTION);
 
 	action->view = view;
 	action->sensitive = TRUE;
@@ -88,13 +94,14 @@ sp_action_setup (SPAction *action,
 	if (name) action->name = strdup (name);
 	if (tip) action->tip = strdup (tip);
 	if (image) action->image = strdup (image);
+	action->verb = verb;
 
 	return action;
 }
 
 /**
 	\return   None
-	\breif    Executes an action
+	\brief    Executes an action
 	\param    action   The action to be executed
 	\param    data     Data that is passed into the action.  This depends
 	                   on the situation that the action is used in.
