@@ -5,8 +5,9 @@
  *
  * Authors:
  *   Lauris Kaplinski <lauris@kaplinski.com>
+ *   bulia byak <buliabyak@users.sf.net>
  *
- * Copyright (C) 1999-2003 authors
+ * Copyright (C) 1999-2005 authors
  * g++ port Copyright (C) 2003 Nathan Hurst
  *
  * Released under GNU GPL, read the file 'COPYING' for more information
@@ -23,6 +24,10 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
+#include "debug/logger.h"
+#include "debug/simple-event.h"
+#include "debug/event-tracker.h"
+
 #ifndef WIN32
 #include <unistd.h>
 #define HAS_PROC_SELF_EXE  //to get path of executable
@@ -34,7 +39,8 @@ using Inkscape::Extension::Internal::PrintWin32;
 
 #include <direct.h>
 #define _WIN32_IE 0x0400
-#define HAS_SHGetSpecialFolderPath
+//#define HAS_SHGetSpecialFolderPath
+#define HAS_SHGetSpecialFolderLocation
 #define HAS_GetModuleFileName
 #include <shlobj.h> //to get appdata path
 #endif
@@ -76,19 +82,18 @@ static Inkscape::Application *inkscape = NULL;
 #include "preferences-skeleton.h"
 
 enum {
-    MODIFY_SELECTION,
-    CHANGE_SELECTION,
-    SET_SELECTION,
-    SET_EVENTCONTEXT,
-    NEW_DESKTOP,
-    DESTROY_DESKTOP,
-    ACTIVATE_DESKTOP,
-    DEACTIVATE_DESKTOP,
-    NEW_DOCUMENT,
-    DESTROY_DOCUMENT,
-    SHUTDOWN_SIGNAL,
-    DIALOGS_HIDE,
-    DIALOGS_UNHIDE,
+    MODIFY_SELECTION, // global: one of selections modified
+    CHANGE_SELECTION, // global: one of selections changed
+    CHANGE_SUBSELECTION, // global: one of subselections (text selection, gradient handle, etc) changed
+    SET_SELECTION, // global: one of selections set
+    SET_EVENTCONTEXT, // tool switched
+    ACTIVATE_DESKTOP, // some desktop got focus
+    DEACTIVATE_DESKTOP, // some desktop lost focus
+    SHUTDOWN_SIGNAL, // inkscape is quitting
+    DIALOGS_HIDE, // user pressed F12
+    DIALOGS_UNHIDE, // user pressed F12
+    EXTERNAL_CHANGE, // a document was changed by some external means (undo or XML editor); this
+                     // may not be reflected by a selection change and thus needs a separate signal
     LAST_SIGNAL
 };
 
@@ -99,6 +104,8 @@ enum {
 # FORWARD DECLARATIONS
 ################################*/
 
+gboolean inkscape_app_use_gui( Inkscape::Application const * app );
+
 static void inkscape_class_init (Inkscape::ApplicationClass *klass);
 static void inkscape_init (SPObject *object);
 static void inkscape_dispose (GObject *object);
@@ -106,7 +113,7 @@ static void inkscape_dispose (GObject *object);
 static void inkscape_activate_desktop_private (Inkscape::Application *inkscape, SPDesktop *desktop);
 static void inkscape_deactivate_desktop_private (Inkscape::Application *inkscape, SPDesktop *desktop);
 
-static void inkscape_init_config (SPReprDoc *doc, const gchar *config_name, const gchar *skeleton, 
+static void inkscape_init_config (Inkscape::XML::Document *doc, const gchar *config_name, const gchar *skeleton, 
 				  unsigned int skel_size,
 				  const gchar *e_mkdir,
 				  const gchar *e_notdir,
@@ -120,7 +127,7 @@ static void inkscape_init_preferences (Inkscape::Application * inkscape);
 
 struct Inkscape::Application {
     GObject object;
-    SPReprDoc *preferences;
+    Inkscape::XML::Document *preferences;
     gboolean save_preferences;
     GSList *documents;
     GSList *desktops;
@@ -134,20 +141,19 @@ struct Inkscape::ApplicationClass {
     GObjectClass object_class;
 
     /* Signals */
-    void (* change_selection) (Inkscape::Application * inkscape, SPSelection * selection);
-    void (* modify_selection) (Inkscape::Application * inkscape, SPSelection * selection, guint flags);
-    void (* set_selection) (Inkscape::Application * inkscape, SPSelection * selection);
+    void (* change_selection) (Inkscape::Application * inkscape, Inkscape::Selection * selection);
+    void (* change_subselection) (Inkscape::Application * inkscape, SPDesktop *desktop);
+    void (* modify_selection) (Inkscape::Application * inkscape, Inkscape::Selection * selection, guint flags);
+    void (* set_selection) (Inkscape::Application * inkscape, Inkscape::Selection * selection);
     void (* set_eventcontext) (Inkscape::Application * inkscape, SPEventContext * eventcontext);
-    void (* new_desktop) (Inkscape::Application * inkscape, SPDesktop * desktop);
-    void (* destroy_desktop) (Inkscape::Application * inkscape, SPDesktop * desktop);
     void (* activate_desktop) (Inkscape::Application * inkscape, SPDesktop * desktop);
     void (* deactivate_desktop) (Inkscape::Application * inkscape, SPDesktop * desktop);
-    void (* new_document) (Inkscape::Application *inkscape, SPDocument *doc);
     void (* destroy_document) (Inkscape::Application *inkscape, SPDocument *doc);
     void (* color_set) (Inkscape::Application *inkscape, SPColor *color, double opacity);
     void (* shut_down) (Inkscape::Application *inkscape);
     void (* dialogs_hide) (Inkscape::Application *inkscape);
     void (* dialogs_unhide) (Inkscape::Application *inkscape);
+    void (* external_change) (Inkscape::Application *inkscape);
 };
 
 static GObjectClass * parent_class;
@@ -217,6 +223,14 @@ inkscape_class_init (Inkscape::ApplicationClass * klass)
                                sp_marshal_NONE__POINTER,
                                G_TYPE_NONE, 1,
                                G_TYPE_POINTER);
+    inkscape_signals[CHANGE_SUBSELECTION] = g_signal_new ("change_subselection",
+                               G_TYPE_FROM_CLASS (klass),
+                               G_SIGNAL_RUN_FIRST,
+                               G_STRUCT_OFFSET (Inkscape::ApplicationClass, change_subselection),
+                               NULL, NULL,
+                               sp_marshal_NONE__POINTER,
+                               G_TYPE_NONE, 1,
+                               G_TYPE_POINTER);
     inkscape_signals[SET_SELECTION] =    g_signal_new ("set_selection",
                                G_TYPE_FROM_CLASS (klass),
                                G_SIGNAL_RUN_FIRST,
@@ -233,22 +247,6 @@ inkscape_class_init (Inkscape::ApplicationClass * klass)
                                sp_marshal_NONE__POINTER,
                                G_TYPE_NONE, 1,
                                G_TYPE_POINTER);
-    inkscape_signals[NEW_DESKTOP] =      g_signal_new ("new_desktop",
-                               G_TYPE_FROM_CLASS (klass),
-                               G_SIGNAL_RUN_FIRST,
-                               G_STRUCT_OFFSET (Inkscape::ApplicationClass, new_desktop),
-                               NULL, NULL,
-                               sp_marshal_NONE__POINTER,
-                               G_TYPE_NONE, 1,
-                               G_TYPE_POINTER);
-    inkscape_signals[DESTROY_DESKTOP] =  g_signal_new ("destroy_desktop",
-                               G_TYPE_FROM_CLASS (klass),
-                               G_SIGNAL_RUN_FIRST,
-                               G_STRUCT_OFFSET (Inkscape::ApplicationClass, destroy_desktop),
-                               NULL, NULL,
-                               sp_marshal_NONE__POINTER,
-                               G_TYPE_NONE, 1,
-                               G_TYPE_POINTER);
     inkscape_signals[ACTIVATE_DESKTOP] = g_signal_new ("activate_desktop",
                                G_TYPE_FROM_CLASS (klass),
                                G_SIGNAL_RUN_FIRST,
@@ -265,22 +263,6 @@ inkscape_class_init (Inkscape::ApplicationClass * klass)
                                sp_marshal_NONE__POINTER,
                                G_TYPE_NONE, 1,
                                G_TYPE_POINTER);
-    inkscape_signals[NEW_DOCUMENT] =     g_signal_new ("new_document",
-                               G_TYPE_FROM_CLASS (klass),
-                               G_SIGNAL_RUN_FIRST,
-                               G_STRUCT_OFFSET (Inkscape::ApplicationClass, new_document),
-                               NULL, NULL,
-                               sp_marshal_NONE__POINTER,
-                               G_TYPE_NONE, 1,
-                               G_TYPE_POINTER);
-    inkscape_signals[DESTROY_DOCUMENT] = g_signal_new ("destroy_document",
-                               G_TYPE_FROM_CLASS (klass),
-                               G_SIGNAL_RUN_FIRST,
-                               G_STRUCT_OFFSET (Inkscape::ApplicationClass, destroy_document),
-                               NULL, NULL,
-                               sp_marshal_NONE__POINTER,
-                               G_TYPE_NONE, 1,
-                               G_TYPE_POINTER);
     inkscape_signals[SHUTDOWN_SIGNAL] =        g_signal_new ("shut_down",
                                G_TYPE_FROM_CLASS (klass),
                                G_SIGNAL_RUN_FIRST,
@@ -302,6 +284,13 @@ inkscape_class_init (Inkscape::ApplicationClass * klass)
                                NULL, NULL,
                                g_cclosure_marshal_VOID__VOID,
                                G_TYPE_NONE, 0);
+    inkscape_signals[EXTERNAL_CHANGE] =   g_signal_new ("external_change",
+                               G_TYPE_FROM_CLASS (klass),
+                               G_SIGNAL_RUN_FIRST,
+                               G_STRUCT_OFFSET (Inkscape::ApplicationClass, external_change),
+                               NULL, NULL,
+                               g_cclosure_marshal_VOID__VOID,
+                               G_TYPE_NONE, 0);
 
     object_class->dispose = inkscape_dispose;
 
@@ -323,6 +312,8 @@ inkscape_init (SPObject * object)
 
     inkscape->documents = NULL;
     inkscape->desktops = NULL;
+
+    inkscape->dialogs_toggle = TRUE;
 }
 
 
@@ -332,7 +323,8 @@ inkscape_dispose (GObject *object)
     Inkscape::Application *inkscape = (Inkscape::Application *) object;
 
     while (inkscape->documents) {
-        g_object_unref (G_OBJECT (inkscape->documents->data));
+        // we don't otherwise unref, so why here?
+        sp_document_unref((SPDocument *)inkscape->documents->data);
     }
 
     g_assert (!inkscape->desktops);
@@ -340,7 +332,7 @@ inkscape_dispose (GObject *object)
     if (inkscape->preferences && inkscape->save_preferences) {
         /* fixme: This is not the best place */
         inkscape_save_preferences (inkscape);
-        sp_repr_document_unref (inkscape->preferences);
+        Inkscape::GC::release(inkscape->preferences);
         inkscape->preferences = NULL;
         inkscape->save_preferences = FALSE;
     }
@@ -387,6 +379,10 @@ inkscape_deactivate_desktop_private (Inkscape::Application *inkscape, SPDesktop
 static void
 inkscape_segv_handler (int signum)
 {
+    using Inkscape::Debug::SimpleEvent;
+    using Inkscape::Debug::EventTracker;
+    using Inkscape::Debug::Logger;
+
     static gint recursion = FALSE;
 
     /* let any SIGABRTs seen from within this handler dump core */
@@ -398,6 +394,9 @@ inkscape_segv_handler (int signum)
     }
     recursion = TRUE;
 
+    EventTracker<SimpleEvent<Inkscape::Debug::Event::CORE> > tracker("crash");
+    tracker.set<SimpleEvent<> >("emergency-save");
+
     fprintf(stderr, "\nEmergency save activated!\n");
 
     time_t sptime = time (NULL);
@@ -410,10 +409,10 @@ inkscape_segv_handler (int signum)
     GSList *failednames = NULL;
     for (GSList *l = inkscape->documents; l != NULL; l = l->next) {
         SPDocument *doc;
-        SPRepr *repr;
+        Inkscape::XML::Node *repr;
         doc = (SPDocument *) l->data;
         repr = sp_document_repr_root (doc);
-        if (sp_repr_attr (repr, "sodipodi:modified")) {
+        if (repr->attribute("sodipodi:modified")) {
             const gchar *docname, *d0, *d;
             gchar n[64], c[1024];
             FILE *file;
@@ -441,7 +440,7 @@ inkscape_segv_handler (int signum)
             if (!docname || !*docname) docname = "emergency";
             // try saving to the profile location
             g_snprintf (c, 1024, "%.256s.%s.%d", docname, sptstr, count);
-            gchar * location = profile_path(c);
+            gchar * location = homedir_path(c);
             Inkscape::IO::dump_fopen_call(location, "E");
             file = Inkscape::IO::fopen_utf8name(location, "w");
             g_free(location);
@@ -462,7 +461,7 @@ inkscape_segv_handler (int signum)
                 savednames = g_slist_prepend (savednames, g_strdup (c));
                 fclose (file);
             } else {
-                docname = sp_repr_attr (repr, "sodipodi:docname");
+                docname = repr->attribute("sodipodi:docname");
                 failednames = g_slist_prepend (failednames, (docname) ? g_strdup (docname) : g_strdup (_("Untitled document")));
             }
             count++;
@@ -540,11 +539,21 @@ inkscape_segv_handler (int signum)
         }
     }
     *(b + pos) = '\0';
-    GtkWidget *msgbox = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, "%s", b);
-    gtk_dialog_run (GTK_DIALOG (msgbox));
-    gtk_widget_destroy (msgbox);
+
+    if ( inkscape_get_instance() && inkscape_app_use_gui( inkscape_get_instance() ) ) {
+        GtkWidget *msgbox = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, "%s", b);
+        gtk_dialog_run (GTK_DIALOG (msgbox));
+        gtk_widget_destroy (msgbox);
+    }
+    else
+    {
+        g_message( "Error: %s", b );
+    }
     g_free (b);
 
+    tracker.clear();
+    Logger::shutdown();
+
     (* segv_handler) (signum);
 }
 
@@ -598,6 +607,11 @@ inkscape_get_instance()
         return inkscape;
 }
 
+gboolean inkscape_app_use_gui( Inkscape::Application const * app )
+{
+    return app->use_gui;
+}
+
 /**
  * Preference management
  * We use '.' as separator
@@ -605,7 +619,7 @@ inkscape_get_instance()
  * Returns TRUE if the config file was successfully loaded, FALSE if not.
  */
 static gboolean
-inkscape_load_config (const gchar *filename, SPReprDoc *config, const gchar *skeleton, 
+inkscape_load_config (const gchar *filename, Inkscape::XML::Document *config, const gchar *skeleton, 
 		      unsigned int skel_size, const gchar *e_notreg, const gchar *e_notxml, 
 		      const gchar *e_notsp, const gchar *warn)
 {
@@ -628,7 +642,7 @@ inkscape_load_config (const gchar *filename, SPReprDoc *config, const gchar *ske
         return FALSE;
     }
 
-    SPReprDoc *doc = sp_repr_read_file (fn, NULL);
+    Inkscape::XML::Document *doc = sp_repr_read_file (fn, NULL);
     if (doc == NULL) {
         /* Not an valid xml file */
         gchar *safeFn = Inkscape::IO::sanitizeString(fn);
@@ -640,20 +654,20 @@ inkscape_load_config (const gchar *filename, SPReprDoc *config, const gchar *ske
         return FALSE;
     }
 
-    SPRepr *root = sp_repr_document_root (doc);
-    if (strcmp (sp_repr_name (root), "inkscape")) {
+    Inkscape::XML::Node *root = sp_repr_document_root (doc);
+    if (strcmp (root->name(), "inkscape")) {
         gchar *safeFn = Inkscape::IO::sanitizeString(fn);
         GtkWidget *w = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, e_notsp, safeFn, warn);
         gtk_dialog_run (GTK_DIALOG (w));
         gtk_widget_destroy (w);
-        sp_repr_document_unref (doc);
+        Inkscape::GC::release(doc);
         g_free(safeFn);
         g_free (fn);
         return FALSE;
     }
 
-    sp_repr_document_merge (config, doc, "id");
-    sp_repr_document_unref (doc);
+    config->root()->mergeFrom(doc->root(), "id");
+    Inkscape::GC::release(doc);
     g_free (fn);
     return TRUE;
 }
@@ -699,15 +713,15 @@ inkscape_save_preferences (Inkscape::Application * inkscape)
 /**
  * We use '.' as separator
  */
-SPRepr *
+Inkscape::XML::Node *
 inkscape_get_repr (Inkscape::Application *inkscape, const gchar *key)
 {
     if (key == NULL) {
         return NULL;
     }
 
-    SPRepr *repr = sp_repr_document_root (inkscape->preferences);
-    g_assert (!(strcmp (sp_repr_name (repr), "inkscape")));
+    Inkscape::XML::Node *repr = sp_repr_document_root (inkscape->preferences);
+    g_assert (!(strcmp (repr->name(), "inkscape")));
 
     gchar const *s = key;
     while ((s) && (*s)) {
@@ -721,9 +735,9 @@ inkscape_get_repr (Inkscape::Application *inkscape, const gchar *key)
             len = strlen (s);
         }
         
-        SPRepr* child;
+        Inkscape::XML::Node* child;
         for (child = repr->firstChild(); child != NULL; child = child->next()) {
-            gchar const *id = sp_repr_attr (child, "id");
+            gchar const *id = child->attribute("id");
             if ((id) && (strlen (id) == len) && (!strncmp (id, s, len)))
             {
                 break;
@@ -742,7 +756,7 @@ inkscape_get_repr (Inkscape::Application *inkscape, const gchar *key)
 
 
 void
-inkscape_selection_modified (SPSelection *selection, guint flags)
+inkscape_selection_modified (Inkscape::Selection *selection, guint flags)
 {
     g_return_if_fail (inkscape != NULL);
     g_return_if_fail (selection != NULL);
@@ -754,7 +768,7 @@ inkscape_selection_modified (SPSelection *selection, guint flags)
 
 
 void
-inkscape_selection_changed (SPSelection * selection)
+inkscape_selection_changed (Inkscape::Selection * selection)
 {
     g_return_if_fail (inkscape != NULL);
     g_return_if_fail (selection != NULL);
@@ -764,9 +778,19 @@ inkscape_selection_changed (SPSelection * selection)
     }
 }
 
+void
+inkscape_subselection_changed (SPDesktop *desktop)
+{
+    g_return_if_fail (inkscape != NULL);
+
+    if (DESKTOP_IS_ACTIVE (desktop)) {
+        g_signal_emit (G_OBJECT (inkscape), inkscape_signals[CHANGE_SUBSELECTION], 0, desktop);
+    }
+}
+
 
 void
-inkscape_selection_set (SPSelection * selection)
+inkscape_selection_set (Inkscape::Selection * selection)
 {
     g_return_if_fail (inkscape != NULL);
     g_return_if_fail (selection != NULL);
@@ -802,8 +826,6 @@ inkscape_add_desktop (SPDesktop * desktop)
 
     inkscape->desktops = g_slist_append (inkscape->desktops, desktop);
 
-    g_signal_emit (G_OBJECT (inkscape), inkscape_signals[NEW_DESKTOP], 0, desktop);
-
     if (DESKTOP_IS_ACTIVE (desktop)) {
         g_signal_emit (G_OBJECT (inkscape), inkscape_signals[ACTIVATE_DESKTOP], 0, desktop);
         g_signal_emit (G_OBJECT (inkscape), inkscape_signals[SET_EVENTCONTEXT], 0, SP_DT_EVENTCONTEXT (desktop));
@@ -839,8 +861,6 @@ inkscape_remove_desktop (SPDesktop * desktop)
         }
     }
 
-    g_signal_emit (G_OBJECT (inkscape), inkscape_signals[DESTROY_DESKTOP], 0, desktop);
-
     inkscape->desktops = g_slist_remove (inkscape->desktops, desktop);
 
     // if this was the last desktop, shut down the program
@@ -1029,6 +1049,13 @@ inkscape_dialogs_toggle ()
     }
 }
 
+void
+inkscape_external_change ()
+{
+    g_return_if_fail (inkscape != NULL);
+
+    g_signal_emit (G_OBJECT (inkscape), inkscape_signals[EXTERNAL_CHANGE], 0);
+}
 
 /**
  * fixme: These need probably signals too
@@ -1038,13 +1065,10 @@ inkscape_add_document (SPDocument *document)
 {
     g_return_if_fail (inkscape != NULL);
     g_return_if_fail (document != NULL);
-    g_return_if_fail (SP_IS_DOCUMENT (document));
 
     g_assert (!g_slist_find (inkscape->documents, document));
 
     inkscape->documents = g_slist_append (inkscape->documents, document);
-
-    g_signal_emit (G_OBJECT (inkscape), inkscape_signals[NEW_DOCUMENT], 0, document);
 }
 
 
@@ -1054,12 +1078,9 @@ inkscape_remove_document (SPDocument *document)
 {
     g_return_if_fail (inkscape != NULL);
     g_return_if_fail (document != NULL);
-    g_return_if_fail (SP_IS_DOCUMENT (document));
 
     g_assert (g_slist_find (inkscape->documents, document));
 
-    g_signal_emit (G_OBJECT (inkscape), inkscape_signals[DESTROY_DOCUMENT], 0, document);
-
     inkscape->documents = g_slist_remove (inkscape->documents, document);
 
     return;
@@ -1085,7 +1106,20 @@ inkscape_active_document (void)
     return NULL;
 }
 
-
+bool inkscape_is_sole_desktop_for_document(SPDesktop const &desktop) {
+    SPDocument *document = SP_VIEW_DOCUMENT(&desktop);
+    if (!document) {
+        return false;
+    }
+    for ( GSList *iter = inkscape->desktops ; iter ; iter = iter->next ) {
+        SPDesktop *other_desktop=(SPDesktop *)iter->data;
+        SPDocument *other_document=SP_VIEW_DOCUMENT(other_desktop);
+        if ( other_document == document && other_desktop != &desktop ) {
+            return false;
+        }
+    }
+    return true;
+}
 
 SPEventContext *
 inkscape_active_event_context (void)
@@ -1104,7 +1138,7 @@ inkscape_active_event_context (void)
 #####################*/
 
 static void 
-inkscape_init_config (SPReprDoc *doc, const gchar *config_name, const gchar *skeleton, 
+inkscape_init_config (Inkscape::XML::Document *doc, const gchar *config_name, const gchar *skeleton, 
 		      unsigned int skel_size,
 		      const gchar *e_mkdir, 
 		      const gchar *e_notdir, 
@@ -1238,6 +1272,36 @@ inkscape_exit (Inkscape::Application *inkscape)
     gtk_main_quit ();
 }
 
+gchar *
+homedir_path(const char *filename)
+{
+    static const gchar *homedir = NULL;
+    if (!homedir) {
+        homedir = g_get_home_dir();
+        gchar* utf8Path = g_filename_to_utf8( homedir, -1, NULL, NULL, NULL );
+        if ( utf8Path )
+        {
+                homedir = utf8Path;
+                if (!g_utf8_validate(homedir, -1, NULL)) {
+                    g_warning( "g_get_home_dir() post A IS NOT UTF-8" );
+                }
+        }
+    }
+    if (!homedir) {
+        gchar * path = g_path_get_dirname(INKSCAPE->argv0);
+        gchar* utf8Path = g_filename_to_utf8( path, -1, NULL, NULL, NULL );
+        g_free(path);
+        if ( utf8Path )
+        {
+            homedir = utf8Path;
+            if (!g_utf8_validate(homedir, -1, NULL)) {
+                g_warning( "g_get_home_dir() post B IS NOT UTF-8" );
+            }
+        }
+    }
+    return g_build_filename(homedir, filename, NULL);
+}
+
 
 /**
  * Get, or guess, or decide the location where the preferences.xml
@@ -1246,68 +1310,59 @@ inkscape_exit (Inkscape::Application *inkscape)
 gchar *
 profile_path(const char *filename)
 {
-    static const gchar *homedir = NULL;
-    if (!homedir) {
-#ifdef HAS_SHGetSpecialFolderPath
+    static const gchar *prefdir = NULL;
+    if (!prefdir) {
+#ifdef HAS_SHGetSpecialFolderLocation
         // prefer c:\Documents and Settings\UserName\Application Data\ to
-        // c:\Documents and Settings\userName\; this
-        // closes bug #933461
-        if (!homedir) {
-            gchar * utf8Path = NULL;
-            if ( PrintWin32::is_os_wide() )
-            {
-                wchar_t pathBuf[MAX_PATH+1];
-                g_assert(sizeof(wchar_t) == sizeof(gunichar2));
-                if (SHGetSpecialFolderPathW(NULL, pathBuf, CSIDL_APPDATA, 1))
-                {
-                    utf8Path = g_utf16_to_utf8( (gunichar2*)(&pathBuf[0]), -1, NULL, NULL, NULL );
+        // c:\Documents and Settings\userName\;
+        if (!prefdir) {
+            ITEMIDLIST *pidl = 0;
+            if ( SHGetSpecialFolderLocation( NULL, CSIDL_APPDATA, &pidl ) == NOERROR ) {
+                gchar * utf8Path = NULL;
+
+                if ( PrintWin32::is_os_wide() ) {
+                    wchar_t pathBuf[MAX_PATH+1];
+                    g_assert(sizeof(wchar_t) == sizeof(gunichar2));
+
+                    if ( SHGetPathFromIDListW( pidl, pathBuf ) ) {
+                        utf8Path = g_utf16_to_utf8( (gunichar2*)(&pathBuf[0]), -1, NULL, NULL, NULL );
+                    }
+                } else {
+                    char pathBuf[MAX_PATH+1];
+
+                    if ( SHGetPathFromIDListA( pidl, pathBuf ) ) {
+                        utf8Path = g_filename_to_utf8( pathBuf, -1, NULL, NULL, NULL );
+                    }
                 }
-            }
-            else
-            {
-                char pathBuf[MAX_PATH+1];
-                if (SHGetSpecialFolderPathA(NULL, pathBuf, CSIDL_APPDATA, 1))
-                {
-                    utf8Path = g_filename_to_utf8( pathBuf, -1, NULL, NULL, NULL );
+
+                if ( utf8Path ) {
+                    if (!g_utf8_validate(utf8Path, -1, NULL)) {
+                        g_warning( "SHGetPathFromIDList%c() resulted in invalid UTF-8", (PrintWin32::is_os_wide() ? 'W' : 'A') );
+                        g_free( utf8Path );
+                        utf8Path = 0;
+                    } else {
+                        prefdir = utf8Path;
+                    }
                 }
-            }
-            if ( utf8Path )
-            {
-                homedir = utf8Path;
+
+
+                /* not compiling yet...
+
+                // Remember to free the list pointer
+                IMalloc * imalloc = 0;
+                if ( SHGetMalloc(&imalloc) == NOERROR) {
+                    imalloc->lpVtbl->Free( imalloc, pidl );
+                    imalloc->lpVtbl->Release( imalloc );
+                }
+                */
             }
         }
 #endif
-        if (!homedir) {
-            homedir = g_get_home_dir();
-            if (!g_utf8_validate(homedir, -1, NULL)) {
-                g_warning( "g_get_home_dir() IS NOT UTF-8" );
-            }
-            gchar* utf8Path = g_filename_to_utf8( homedir, -1, NULL, NULL, NULL );
-            if ( utf8Path )
-            {
-                homedir = utf8Path;
-                if (!g_utf8_validate(homedir, -1, NULL)) {
-                    g_warning( "g_get_home_dir() post A IS NOT UTF-8" );
-                }
-            }
-        }
-        if (!homedir) {
-            gchar * path = g_path_get_dirname(INKSCAPE->argv0);
-            if (!g_utf8_validate(path, -1, NULL)) {
-                g_warning( "g_path_get_dirname() IS NOT UTF-8" );
-            }
-            gchar* utf8Path = g_filename_to_utf8( path, -1, NULL, NULL, NULL );
-            g_free(path);
-            if ( utf8Path )
-            {
-                homedir = utf8Path;
-                if (!g_utf8_validate(homedir, -1, NULL)) {
-                    g_warning( "g_get_home_dir() post B IS NOT UTF-8" );
-                }
-            }
+        if (!prefdir) {
+            prefdir = homedir_path(NULL);
         }
     }
-    return g_build_filename(homedir, INKSCAPE_PROFILE_DIR, filename, NULL);
+    return g_build_filename(prefdir, INKSCAPE_PROFILE_DIR, filename, NULL);
 }
 
 
