@@ -13,7 +13,13 @@
  */
 
 #include "forward.h"
-#include "xml/xml-forward.h"
+
+namespace Inkscape {
+namespace XML {
+class Node;
+}
+}
+
 
 #define INKSCAPE inkscape_get_instance()
 
@@ -22,7 +28,7 @@ void inkscape_application_init (const gchar *argv0, gboolean use_gui);
 /* Preference management */
 gboolean inkscape_load_preferences (Inkscape::Application * inkscape);
 gboolean inkscape_save_preferences (Inkscape::Application * inkscape);
-SPRepr *inkscape_get_repr (Inkscape::Application *inkscape, const gchar *key);
+Inkscape::XML::Node *inkscape_get_repr (Inkscape::Application *inkscape, const gchar *key);
 
 Inkscape::Application *inkscape_get_instance();
 
@@ -35,6 +41,9 @@ SPDocument * inkscape_active_document (void);
 #define SP_ACTIVE_DESKTOP inkscape_active_desktop ()
 SPDesktop * inkscape_active_desktop (void);
 
+bool inkscape_is_sole_desktop_for_document(SPDesktop const &desktop);
+
+gchar *homedir_path(const char *filename);
 gchar *profile_path(const char *filename);
 
 void inkscape_switch_desktops_next ();
@@ -44,6 +53,9 @@ void inkscape_dialogs_hide ();
 void inkscape_dialogs_unhide ();
 void inkscape_dialogs_toggle ();
 
+void inkscape_external_change ();
+void inkscape_subselection_changed (SPDesktop *desktop);
+
 /*
  * fixme: This has to be rethought
  */
