@@ -27,7 +27,9 @@
  */
 
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
 
 #include <string.h>
 #include <sys/stat.h>
@@ -226,7 +228,6 @@ main (int argc, const char **argv)
 			
 			ss.doc = sp_document_new_from_mem ((const gchar *)gba->data, 
 							   gba->len,
-							   TRUE, 
 							   TRUE);
 			gchar *last_filename = jar_file_reader.get_last_filename();
 			if (ss.doc) {
@@ -249,7 +250,7 @@ main (int argc, const char **argv)
 		}
 		
 		ss.slides[ss.length++] = strdup (argv[i]);
-		ss.doc = sp_document_new (ss.slides[ss.current], TRUE, TRUE, false);
+		ss.doc = sp_document_new (ss.slides[ss.current], TRUE, false);
 		
 		if (!ss.doc && ++ss.current >= ss.length) {
 		    /* No loadable documents */
@@ -372,7 +373,7 @@ sp_svgview_show_next (struct SPSlideShow *ss)
     doc = NULL;
     current = ss->current;
     while (!doc && (current < ss->length - 1)) {
-	doc = sp_document_new (ss->slides[++current], TRUE, TRUE, false);
+	doc = sp_document_new (ss->slides[++current], TRUE, false);
     }
     if (doc) {
 	sp_view_set_document (SP_VIEW_WIDGET_VIEW (ss->view), doc);
@@ -390,7 +391,7 @@ sp_svgview_show_prev (struct SPSlideShow *ss)
     doc = NULL;
     current = ss->current;
     while (!doc && (current > 0)) {
-	doc = sp_document_new (ss->slides[--current], TRUE, TRUE, false);
+	doc = sp_document_new (ss->slides[--current], TRUE, false);
     }
     if (doc) {
 	sp_view_set_document (SP_VIEW_WIDGET_VIEW (ss->view), doc);
@@ -406,7 +407,7 @@ sp_svgview_goto_first (struct SPSlideShow *ss)
     SPDocument *doc = NULL;
     int current = 0;
     for ( ; !doc && (current < ss->length); current++) {
-	doc = sp_document_new (ss->slides[current], TRUE, TRUE, false);
+	doc = sp_document_new (ss->slides[current], TRUE, false);
     }
     if (doc) {
 	sp_view_set_document (SP_VIEW_WIDGET_VIEW (ss->view), doc);
@@ -422,7 +423,7 @@ sp_svgview_goto_last (struct SPSlideShow *ss)
     SPDocument *doc = NULL;
     int current = ss->length - 1;
     for ( ; !doc && (current >= 0); current--) {
-	doc = sp_document_new (ss->slides[current], TRUE, TRUE, false);
+	doc = sp_document_new (ss->slides[current], TRUE, false);
     }
     if (doc) {
 	sp_view_set_document (SP_VIEW_WIDGET_VIEW (ss->view), doc);
@@ -450,11 +451,11 @@ is_jar(char const *filename)
 
 static void usage()
 {
-    fprintf(stdout, 
+    fprintf(stderr, 
 	    "Usage: inkview [FILES ...]\n"
-	    "\twhere FILES are SVG (.svg)"
+	    "\twhere FILES are SVG (.svg or .svgz)"
 #ifdef WITH_INKJAR
-	    "or archives of svgs (.sxw, .jar)"
+	    "or archives of SVGs (.sxw, .jar)"
 #endif
 	    "\n");
     exit(1);
@@ -467,9 +468,7 @@ void inkscape_ref (void) {}
 void inkscape_unref (void) {}
 void inkscape_add_document (SPDocument *document) {}
 void inkscape_remove_document (SPDocument *document) {}
-SPRepr *inkscape_get_repr (Inkscape::Application *inkscape, const gchar *key) {return NULL;}
-#include "widgets/menu.h"
-void sp_menu_append (SPMenu *menu, const gchar *name, const gchar *tip, const void *data) {}
+Inkscape::XML::Node *inkscape_get_repr (Inkscape::Application *inkscape, const gchar *key) {return NULL;}
 #endif
 
 
