@@ -6,17 +6,21 @@
  * Authors:
  *   Lauris Kaplinski <lauris@kaplinski.com>
  *   Frank Felfe <innerspace@iname.com>
+ *   bulia byak <buliabyak@users.sf.net>
  *
- * Copyright (C) 1999-2002 authors
+ * Copyright (C) 1999-2005 authors
  * Copyright (C) 2001-2002 Ximian, Inc.
  * Copyright (C) 2004 David Turner
  *
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
 
 #include <string.h>
+#include <vector>
 #include <glib.h>
 #include <glib-object.h>
 #include <gtk/gtk.h>
@@ -44,7 +48,7 @@
 #include "path-chemistry.h"
 #include "zoom-context.h"
 #include "svg-view.h"
-#include "sp-desktop-widget.h"
+#include "desktop-widget.h"
 #include "sp-item-group.h"
 #include "sp-namedview.h"
 
@@ -56,13 +60,16 @@
 #include "helper/window.h"
 
 #include "io/sys.h"
+#include "io/stringstream.h"
+#include "io/base64stream.h"
 
 #include "dialogs/dialog-events.h"
 
 #include "message-context.h"
 
-/* Define this to have an Effects menu in the menu bar. */
-// #define WITH_EFFECT_MENU 1
+
+using Inkscape::IO::StringOutputStream;
+using Inkscape::IO::Base64OutputStream;
 
 /* forward declaration */
 static gint sp_ui_delete (GtkWidget *widget, GdkEvent *event, SPView *view);
@@ -71,15 +78,23 @@ static gint sp_ui_delete (GtkWidget *widget, GdkEvent *event, SPView *view);
 typedef enum {
   URI_LIST,
   SVG_XML_DATA,
-  SVG_DATA
+  SVG_DATA,
+  PNG_DATA,
+  JPEG_DATA,
+  IMAGE_DATA
 } ui_drop_target_info;
 
 static GtkTargetEntry ui_drop_target_entries [] = {
   {"text/uri-list", 0, URI_LIST},
   {"image/svg+xml", 0, SVG_XML_DATA},
   {"image/svg",     0, SVG_DATA},
+  {"image/png",     0, PNG_DATA},
+  {"image/jpeg",    0, JPEG_DATA},
 };
 
+static GtkTargetEntry* completeDropTargets = 0;
+static int completeDropTargetsCount = 0;
+
 #define ENTRIES_SIZE(n) sizeof(n)/sizeof(n[0])
 static guint nui_drop_target_entries = ENTRIES_SIZE(ui_drop_target_entries);
 static void sp_ui_import_files(gchar * buffer);
@@ -123,10 +138,42 @@ sp_create_window (SPViewWidget *vw, gboolean editable)
 	gtk_box_pack_end (GTK_BOX (hb), GTK_WIDGET (vw), TRUE, TRUE, 0);
 	gtk_widget_show (GTK_WIDGET (vw));
 
+
+    if ( completeDropTargets == 0 || completeDropTargetsCount == 0 )
+    {
+        std::vector<gchar*> types;
+
+        GSList* list = gdk_pixbuf_get_formats();
+        while ( list ) {
+            int i = 0;
+            GdkPixbufFormat* one = (GdkPixbufFormat*)list->data;
+            gchar** typesXX = gdk_pixbuf_format_get_mime_types(one);
+            for ( i = 0; typesXX[i]; i++ ) {
+                types.push_back(g_strdup(typesXX[i]));
+            }
+            g_strfreev(typesXX);
+
+            list = g_slist_next(list);
+        }
+        completeDropTargetsCount = nui_drop_target_entries + types.size();
+        completeDropTargets = new GtkTargetEntry[completeDropTargetsCount];
+        for ( int i = 0; i < (int)nui_drop_target_entries; i++ ) {
+            completeDropTargets[i] = ui_drop_target_entries[i];
+        }
+        int pos = nui_drop_target_entries;
+
+        for (std::vector<gchar*>::iterator it = types.begin() ; it != types.end() ; it++) {
+            completeDropTargets[pos].target = *it;
+            completeDropTargets[pos].flags = 0;
+            completeDropTargets[pos].info = IMAGE_DATA;
+            pos++;
+        }
+    }
+
 	gtk_drag_dest_set(w,
 			  GTK_DEST_DEFAULT_ALL,
-			  ui_drop_target_entries,
-			  nui_drop_target_entries,
+			  completeDropTargets,
+			  completeDropTargetsCount,
 			  GdkDragAction (GDK_ACTION_COPY | GDK_ACTION_MOVE));
 	g_signal_connect (G_OBJECT(w),
 			   "drag_data_received",
@@ -205,19 +252,34 @@ sp_ui_close_all (void)
     /* Iterate through all the windows, destroying each in the order they
        become active */
     while (SP_ACTIVE_DESKTOP) {
-    GtkWidget *w;
-    w = (GtkWidget*)g_object_get_data (G_OBJECT (SP_ACTIVE_DESKTOP), "window");
-    if (sp_view_shutdown (SP_VIEW (SP_ACTIVE_DESKTOP))) {
-        /* The user cancelled the operation, so end doing the close */
-        return FALSE;
-    }
-    gtk_widget_destroy (w);
+        GtkWidget *w;
+        w = (GtkWidget*)g_object_get_data (G_OBJECT (SP_ACTIVE_DESKTOP), "window");
+        if (sp_view_shutdown (SP_VIEW (SP_ACTIVE_DESKTOP))) {
+            /* The user cancelled the operation, so end doing the close */
+            return FALSE;
+        }
+ //       gtk_widget_destroy (w);
+
+       // Spackle for Bug [ 1217361 ] "freeze on quitting with transform dialog"
+       // The following code is intended to be temporary and IS NOT NEEDED once this part
+       // of Inkscape is de-bugged, when the single line above should be re-instituted.
+
+       // At present, SP_ACTIVE_DESKTOP does not guarantee to return a gtk_widget: If we find
+       // that we have been issued an object other than a widget we assume that something is
+       // wrong upstream and quit immediately.
+       if( GTK_IS_WIDGET(w) ) {
+ //           fprintf( stderr, "sp_ui_close_all( ) About to call  gtk_widget_destroy ( %08p )\n", w );
+            gtk_widget_destroy (w);
+        } else {
+            g_warning( "Desktop has passed us a non-widget! : %p", w );
+            gtk_main_quit();
+            break;
+        }
     }
 
     return TRUE;
 }
 
-
 static gint
 sp_ui_delete (GtkWidget *widget, GdkEvent *event, SPView *view)
 {
@@ -274,7 +336,7 @@ sp_ui_menuitem_add_icon ( GtkWidget *item, gchar * icon_name )
 {
     GtkWidget *icon;
 
-    icon = sp_icon_new_scaled (SP_ICON_SIZE_BUTTON, icon_name);
+    icon = sp_icon_new( GTK_ICON_SIZE_MENU, icon_name );
     gtk_widget_show (icon);
     gtk_image_menu_item_set_image ((GtkImageMenuItem *) item, icon);
 } // end of sp_ui_menu_add_icon
@@ -328,11 +390,11 @@ sp_ui_menu_key_press (GtkMenuItem *item, GdkEventKey *event, void *data)
     if (event->state & (GDK_SHIFT_MASK | GDK_CONTROL_MASK | GDK_MOD1_MASK)) {
         unsigned int shortcut;
 
-        shortcut = event->keyval;
+        shortcut = get_group0_keyval (event);
         if (event->state & GDK_SHIFT_MASK) shortcut |= SP_SHORTCUT_SHIFT_MASK;
         if (event->state & GDK_CONTROL_MASK) shortcut |= SP_SHORTCUT_CONTROL_MASK;
         if (event->state & GDK_MOD1_MASK) shortcut |= SP_SHORTCUT_ALT_MASK;
-        sp_shortcut_set (shortcut, (Inkscape::Verb *)((int)data), true);
+        sp_shortcut_set (shortcut, (Inkscape::Verb *)data, true);
     }
 }
 
@@ -645,14 +707,20 @@ sp_menu_append_new_templates (GtkWidget *menu, SPView *view)
     for (const gchar *file = g_dir_read_name (dir); file != NULL; file = g_dir_read_name (dir)) {
         if (!g_str_has_suffix (file, ".svg"))
             continue; // skip non-svg files
-        if (g_str_has_suffix (file, "default.svg"))
-            continue; // skip default.svg - it's in the menu already
+
+        gchar *basename = g_path_get_basename (file);
+        if (g_str_has_suffix (basename, ".svg") && g_str_has_prefix (basename, "default."))
+            continue; // skip default.*.svg (i.e. default.svg and translations) - it's in the menu already
 
         const gchar *filepath = g_build_filename(INKSCAPE_TEMPLATESDIR, file, NULL);
-        const gchar *filename =  g_filename_to_utf8(g_strndup (file, strlen(file) - 4),  -1, NULL, NULL, NULL);
-
+        gchar * dupfile = g_strndup(file, strlen(file) - 4);
+        gchar *filename =  g_filename_to_utf8(dupfile,  -1, NULL, NULL, NULL);
+        g_free(dupfile);
         GtkWidget *item = gtk_menu_item_new_with_label (filename);
+        g_free(filename);
+
         gtk_widget_show(item);
+        // how does "filepath" ever get freed?
         g_signal_connect(G_OBJECT(item),
                          "activate",
                          G_CALLBACK(sp_file_new_from_template),
@@ -705,7 +773,7 @@ sp_menu_append_recent_documents (GtkWidget *menu, SPView* /* view */)
 static void
 sp_ui_file_menu (GtkMenu *fm, SPDocument *doc, SPView *view)
 {
-    sp_ui_menu_append_submenu (fm, view, sp_menu_append_new_templates, _("_New"), _("Create new document"), "file_templates" );
+    sp_ui_menu_append_submenu (fm, view, sp_menu_append_new_templates, _("_New"), _("Create new document"), NULL);
 
     static Inkscape::Verb * file_verbs_one[] = {
 	Inkscape::Verb::get(SP_VERB_FILE_OPEN),
@@ -812,6 +880,11 @@ sp_ui_layer_menu (GtkMenu *menu, SPDocument *doc, SPView *view)
 
         Inkscape::Verb::get(SP_VERB_NONE),
 
+        Inkscape::Verb::get(SP_VERB_LAYER_MOVE_TO_NEXT),
+        Inkscape::Verb::get(SP_VERB_LAYER_MOVE_TO_PREV),
+
+        Inkscape::Verb::get(SP_VERB_NONE),
+
         Inkscape::Verb::get(SP_VERB_LAYER_RAISE),
         Inkscape::Verb::get(SP_VERB_LAYER_LOWER),
         Inkscape::Verb::get(SP_VERB_LAYER_TO_TOP),
@@ -831,6 +904,7 @@ sp_ui_object_menu (GtkMenu *menu, SPDocument *doc, SPView *view)
 {
     static Inkscape::Verb * selection[] = {
         Inkscape::Verb::get(SP_VERB_DIALOG_FILL_STROKE),
+        Inkscape::Verb::get(SP_VERB_DIALOG_SWATCHES),
         Inkscape::Verb::get(SP_VERB_DIALOG_ITEM),
         Inkscape::Verb::get(SP_VERB_NONE),
 
@@ -853,6 +927,7 @@ sp_ui_object_menu (GtkMenu *menu, SPDocument *doc, SPView *view)
         Inkscape::Verb::get(SP_VERB_NONE),
         Inkscape::Verb::get(SP_VERB_DIALOG_TRANSFORM),
         Inkscape::Verb::get(SP_VERB_DIALOG_ALIGN_DISTRIBUTE),
+        Inkscape::Verb::get(SP_VERB_SELECTION_GRIDTILE),
 
         Inkscape::Verb::get(SP_VERB_LAST)
     };
@@ -866,7 +941,7 @@ sp_ui_path_menu (GtkMenu *menu, SPDocument *doc, SPView *view)
     static Inkscape::Verb * selection[] = {
         Inkscape::Verb::get(SP_VERB_OBJECT_TO_CURVE),
         Inkscape::Verb::get(SP_VERB_SELECTION_OUTLINE),
-        Inkscape::Verb::get(SP_VERB_SELECTION_POTRACE),
+        Inkscape::Verb::get(SP_VERB_SELECTION_TRACE),
 
         Inkscape::Verb::get(SP_VERB_NONE),
         Inkscape::Verb::get(SP_VERB_SELECTION_UNION),
@@ -889,7 +964,6 @@ sp_ui_path_menu (GtkMenu *menu, SPDocument *doc, SPView *view)
         Inkscape::Verb::get(SP_VERB_NONE),
         Inkscape::Verb::get(SP_VERB_SELECTION_SIMPLIFY),
         Inkscape::Verb::get(SP_VERB_SELECTION_REVERSE),
-        Inkscape::Verb::get(SP_VERB_SELECTION_CLEANUP),
         Inkscape::Verb::get(SP_VERB_LAST)
     };
     sp_ui_menu_append (menu, selection, view);
@@ -938,12 +1012,13 @@ sp_ui_view_menu (GtkMenu *menu, SPDocument *doc, SPView *view)
         Inkscape::Verb::get(SP_VERB_NONE),
 	Inkscape::Verb::get(SP_VERB_VIEW_NEW),
 	// Inkscape::Verb::get(SP_VERB_VIEW_NEW_PREVIEW),
+        Inkscape::Verb::get(SP_VERB_VIEW_ICON_PREVIEW),
         Inkscape::Verb::get(SP_VERB_LAST)
     };
 
     sp_ui_menu_append (menu, view_verbs1, view);
 
-    GtkWidget *item_showhide = sp_ui_menu_append_item (menu, NULL, _("_Show/Hide"), _("Show or hide parts of the document window (differently for normal and fullscreen modes)"), view, NULL, NULL);
+    GtkWidget *item_showhide = sp_ui_menu_append_item (menu, NULL, _("S_how/Hide"), _("Show or hide parts of the document window (differently for normal and fullscreen modes)"), view, NULL, NULL);
     GtkMenu *m = (GtkMenu *) gtk_menu_new ();
 
 //    sp_ui_menu_append_check_item_from_verb (m, view, _("_Menu"), _("Show or hide the menu bar"), "menu",
@@ -976,6 +1051,11 @@ sp_ui_text_menu (GtkMenu *menu, SPDocument *doc, SPView *view)
         Inkscape::Verb::get(SP_VERB_SELECTION_TEXTTOPATH),
         Inkscape::Verb::get(SP_VERB_SELECTION_TEXTFROMPATH),
 
+        Inkscape::Verb::get(SP_VERB_NONE),
+        Inkscape::Verb::get(SP_VERB_OBJECT_FLOW_TEXT),
+        Inkscape::Verb::get(SP_VERB_OBJECT_UNFLOW_TEXT),
+        Inkscape::Verb::get(SP_VERB_OBJECT_FLOWTEXT_TO_TEXT),
+
         Inkscape::Verb::get(SP_VERB_NONE),
         Inkscape::Verb::get(SP_VERB_SELECTION_REMOVE_KERNS),
 
@@ -985,7 +1065,6 @@ sp_ui_text_menu (GtkMenu *menu, SPDocument *doc, SPView *view)
     sp_ui_menu_append (menu, text_verbs, view);
 }
 
-#ifdef WITH_EFFECT_MENU
 /** Creates the effects menu.
     \param  menu  The menu to append to.
     \param  doc   Document being used.
@@ -1020,7 +1099,6 @@ sp_ui_effect_menu (GtkMenu *menu, SPDocument *doc, SPView *view)
 
     return;
 }
-#endif /* WITH_EFFECT_MENU */
 
 static void
 sp_ui_help_menu (GtkMenu *fm, SPDocument *doc, SPView *view)
@@ -1044,7 +1122,11 @@ sp_ui_help_menu (GtkMenu *fm, SPDocument *doc, SPView *view)
     };
 
     static Inkscape::Verb * help_verbs_two[] = {
-        Inkscape::Verb::get(SP_VERB_HELP_ABOUT), Inkscape::Verb::get(SP_VERB_LAST)
+//        Inkscape::Verb::get(SP_VERB_HELP_ABOUT_EXTENSIONS),
+        Inkscape::Verb::get(SP_VERB_HELP_MEMORY),
+        Inkscape::Verb::get(SP_VERB_HELP_ABOUT),
+        Inkscape::Verb::get(SP_VERB_SHOW_LICENSE),
+        Inkscape::Verb::get(SP_VERB_LAST)
     };
 
     sp_ui_menu_append (fm, help_verbs_one, view);
@@ -1121,13 +1203,13 @@ sp_ui_main_menubar (SPView *view)
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (mitem), GTK_WIDGET (menu));
 	gtk_menu_shell_append (GTK_MENU_SHELL (mbar), mitem);
 
-#ifdef WITH_EFFECT_MENU
-	mitem = gtk_menu_item_new_with_mnemonic (_("Effects"));
-	menu = gtk_menu_new ();
-	sp_ui_effect_menu (GTK_MENU (menu), NULL, view);
-	gtk_menu_item_set_submenu (GTK_MENU_ITEM (mitem), GTK_WIDGET (menu));
-	gtk_menu_shell_append (GTK_MENU_SHELL (mbar), mitem);
-#endif
+        if (prefs_get_int_attribute("extensions", "show-effects-menu", 0)) {
+            mitem = gtk_menu_item_new_with_mnemonic (_("Effects"));
+            menu = gtk_menu_new ();
+            sp_ui_effect_menu (GTK_MENU (menu), NULL, view);
+            gtk_menu_item_set_submenu (GTK_MENU_ITEM (mitem), GTK_WIDGET (menu));
+            gtk_menu_shell_append (GTK_MENU_SHELL (mbar), mitem);
+        }
 
 	mitem = gtk_menu_item_new_with_mnemonic (_("_Help"));
 	menu = gtk_menu_new ();
@@ -1142,10 +1224,6 @@ static void leave_group(GtkMenuItem *, SPDesktop *desktop) {
     desktop->setCurrentLayer(SP_OBJECT_PARENT(desktop->currentLayer()));
 }
 
-static void leave_all_groups(GtkMenuItem *, SPDesktop *desktop) {
-    desktop->setCurrentLayer(desktop->currentRoot());
-}
-
 static void enter_group(GtkMenuItem *mi, SPDesktop *desktop) {
     desktop->setCurrentLayer(reinterpret_cast<SPObject *>(g_object_get_data(G_OBJECT(mi), "group")));
     SP_DT_SELECTION(desktop)->clear();
@@ -1218,10 +1296,6 @@ sp_ui_context_menu (SPView *view, SPItem *item)
                 gtk_menu_shell_append(GTK_MENU_SHELL(m), w);
 
             }
-            GtkWidget *w = gtk_menu_item_new_with_label(_("Go to root"));
-            g_signal_connect(G_OBJECT(w), "activate", GCallback(leave_all_groups), dt);
-            gtk_widget_show(w);
-            gtk_menu_shell_append(GTK_MENU_SHELL(m), w);
         }
 
 	return m;
@@ -1245,25 +1319,25 @@ sp_ui_drag_data_received (GtkWidget * widget,
 
 		SPDocument *doc = SP_ACTIVE_DOCUMENT;
 
-		SPReprDoc *rnewdoc = sp_repr_read_mem (svgdata, data->length, SP_SVG_NS_URI);
+		Inkscape::XML::Document *rnewdoc = sp_repr_read_mem (svgdata, data->length, SP_SVG_NS_URI);
 
 		if (rnewdoc == NULL) {
 			sp_ui_error_dialog (_("Could not parse SVG data"));
 			return;
 		}
 
-		SPRepr *repr = sp_repr_document_root (rnewdoc);
-		const gchar *style = sp_repr_attr (repr, "style");
+		Inkscape::XML::Node *repr = sp_repr_document_root (rnewdoc);
+		const gchar *style = repr->attribute("style");
 
-		SPRepr *newgroup = sp_repr_new ("svg:g");
+		Inkscape::XML::Node *newgroup = sp_repr_new ("svg:g");
 		sp_repr_set_attr (newgroup, "style", style);
 
-		for (SPRepr *child = repr->firstChild(); child != NULL; child = child->next()) {
-			SPRepr * newchild = sp_repr_duplicate (child);
-			sp_repr_append_child (newgroup, newchild);
+		for (Inkscape::XML::Node *child = repr->firstChild(); child != NULL; child = child->next()) {
+			Inkscape::XML::Node * newchild = child->duplicate();
+			newgroup->appendChild(newchild);
 		}
 
-		sp_repr_document_unref (rnewdoc);
+		Inkscape::GC::release(rnewdoc);
 
             SPDesktop *desktop = SP_ACTIVE_DESKTOP;
             // Add it to the current layer
@@ -1275,9 +1349,66 @@ sp_ui_drag_data_received (GtkWidget * widget,
 
 		break;
 	case URI_LIST:
+    {
 		gchar *uri = (gchar *)data->data;
 		sp_ui_import_files(uri);
-		break;
+    }
+    break;
+
+    case PNG_DATA:
+    case JPEG_DATA:
+    case IMAGE_DATA:
+    {
+        char tmp[1024];
+
+        StringOutputStream outs;
+        Base64OutputStream b64out(outs);
+        b64out.setColumnWidth(0);
+
+        SPDocument *doc = SP_ACTIVE_DOCUMENT;
+
+        Inkscape::XML::Node *newImage = sp_repr_new ("svg:image");
+
+        for ( int i = 0; i < data->length; i++ ) {
+            b64out.put( data->data[i] );
+        }
+        b64out.close();
+
+
+        Glib::ustring str = outs.getString();
+
+        snprintf( tmp, sizeof(tmp), "data:%s;base64,", gdk_atom_name(data->type) );
+        str.insert( 0, tmp );
+        sp_repr_set_attr( newImage, "xlink:href", str.c_str() );
+
+        GError* error = NULL;
+        GdkPixbufLoader* loader = gdk_pixbuf_loader_new_with_mime_type( gdk_atom_name(data->type), &error );
+        if ( loader ) {
+            error = NULL;
+            if ( gdk_pixbuf_loader_write( loader, data->data, data->length, &error) ) {
+                GdkPixbuf* pbuf = gdk_pixbuf_loader_get_pixbuf(loader);
+                if ( pbuf ) {
+                    int width = gdk_pixbuf_get_width(pbuf);
+                    int height = gdk_pixbuf_get_height(pbuf);
+                    snprintf( tmp, sizeof(tmp), "%d", width );
+                    sp_repr_set_attr( newImage, "width", tmp );
+
+                    snprintf( tmp, sizeof(tmp), "%d", height );
+                    sp_repr_set_attr( newImage, "height", tmp );
+                }
+            }
+        }
+
+        SPDesktop *desktop = SP_ACTIVE_DESKTOP;
+
+        // Add it to the current layer
+        desktop->currentLayer()->appendChildRepr(newImage);
+
+        sp_repr_unref( newImage );
+        sp_document_done( doc );
+    }
+    break;
+
 	}
 }
 
@@ -1305,7 +1436,7 @@ static void
 sp_ui_import_one_file(char const *filename)
 {
     SPDocument *doc = SP_ACTIVE_DOCUMENT;
-    if (!SP_IS_DOCUMENT(doc)) return;
+    if (!doc) return;
 
     if (filename == NULL) return;
 
