@@ -14,8 +14,9 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
+#include <gtk/gtkstyle.h>
+
 #include "forward.h"
-#include <verbs.h>
 
 
 /**
@@ -28,6 +29,12 @@ void sp_create_window (SPViewWidget *vw, gboolean editable);
  */
 void sp_ui_close_view (GtkWidget *widget);
 
+/**
+ *
+ */
+void sp_ui_new_view (void);
+void sp_ui_new_view_preview (void);
+
 /**
  *
  */
@@ -53,7 +60,7 @@ void sp_menu_append_recent_documents (GtkWidget *menu);
 /**
  *
  */
-void sp_ui_dialog_title_string (sp_verb_t verb, gchar* c);
+void sp_ui_dialog_title_string (Inkscape::Verb * verb, gchar* c);
 
 
 /**
@@ -62,6 +69,8 @@ void sp_ui_dialog_title_string (sp_verb_t verb, gchar* c);
 void sp_ui_error_dialog (const gchar * message);
 bool sp_ui_overwrite_file (const gchar * filename);
 
+void sp_ui_shortcut_string (unsigned int shortcut, gchar* c);
+
 #endif
 
 /*
