@@ -5,8 +5,9 @@
  *
  * Authors:
  *   Lauris Kaplinski <lauris@kaplinski.com>
+ *   bulia byak <buliabyak@users.sf.net>
  *
- * Copyright (C) 1999-2002 authors
+ * Copyright (C) 1999-2005 authors
  * Copyright (C) 2001-2002 Ximian, Inc.
  *
  * Released under GNU GPL, read the file 'COPYING' for more information
@@ -43,6 +44,9 @@
 
 static bool nograb = false;
 
+static gboolean grabbed = FALSE;
+static gboolean moved = FALSE;
+
 static gint xp = 0, yp = 0; // where drag started
 static gint tolerance = 0;
 static bool within_tolerance = false;
@@ -69,6 +73,7 @@ enum {
 enum {
 	EVENT,
 	CLICKED,
+	DOUBLECLICKED,
 	GRABBED,
 	UNGRABBED,
 	MOVED,
@@ -249,6 +254,14 @@ sp_knot_class_init (SPKnotClass * klass)
 					    sp_marshal_NONE__UINT,
 					    G_TYPE_NONE, 1,
 					    G_TYPE_UINT);
+	knot_signals[DOUBLECLICKED] = g_signal_new ("doubleclicked",
+					    G_TYPE_FROM_CLASS(klass),
+					    G_SIGNAL_RUN_FIRST,
+					    G_STRUCT_OFFSET (SPKnotClass, doubleclicked),
+					    NULL, NULL,
+					    sp_marshal_NONE__UINT,
+					    G_TYPE_NONE, 1,
+					    G_TYPE_UINT);
 	knot_signals[GRABBED] = g_signal_new ("grabbed",
 					    G_TYPE_FROM_CLASS(klass),
 					    G_SIGNAL_RUN_FIRST,
@@ -334,8 +347,10 @@ sp_knot_dispose (GObject * object)
 {
 	SPKnot *knot = (SPKnot *) object;
 
-	/* ungrab pointer if still grabbed by mouseover, find a different way */
-	if (gdk_pointer_is_grabbed ()) gdk_pointer_ungrab (GDK_CURRENT_TIME);
+	if ((knot->flags & SP_KNOT_GRABBED) && gdk_pointer_is_grabbed ()) {
+		// This happens e.g. when deleting a node in node tool while dragging it
+		gdk_pointer_ungrab (GDK_CURRENT_TIME);
+	}
 
 	if (knot->item) {
 		gtk_object_destroy (GTK_OBJECT (knot->item));
@@ -456,12 +471,28 @@ static void sp_knot_get_property(GObject *, guint, GValue *, GParamSpec *)
 	g_assert_not_reached ();
 }
 
+void sp_knot_start_dragging (SPKnot *knot, NR::Point p, gint x, gint y, guint32 etime)
+{
+	// save drag origin
+	xp = x; 
+	yp = y;
+	within_tolerance = true;
+
+	knot->grabbed_rel_pos = p - knot->pos;
+	knot->drag_origin = knot->pos;
+	if (!nograb) {
+		sp_canvas_item_grab (knot->item,
+				     KNOT_EVENT_MASK,
+				     knot->cursor[SP_KNOT_STATE_DRAGGING],
+				     etime);
+	}
+	sp_knot_set_flag (knot, SP_KNOT_GRABBED, TRUE);
+	grabbed = TRUE;
+}
+
 static int
 sp_knot_handler (SPCanvasItem *item, GdkEvent *event, SPKnot *knot)
 {
-	static gboolean grabbed = FALSE;
-	static gboolean moved = FALSE;
-
 	g_assert (knot != NULL);
 	g_assert (SP_IS_KNOT (knot));
 
@@ -476,26 +507,22 @@ sp_knot_handler (SPCanvasItem *item, GdkEvent *event, SPKnot *knot)
 	if (consumed) return TRUE;
 
 	switch (event->type) {
+	case GDK_2BUTTON_PRESS:
+		if (event->button.button == 1) {
+			g_signal_emit (G_OBJECT (knot), knot_signals[DOUBLECLICKED], 0, event->button.state);
+
+			grabbed = FALSE;
+			moved = FALSE;
+			consumed = TRUE;
+		}
+		break;
 	case GDK_BUTTON_PRESS:
 		if (event->button.button == 1) {
 			NR::Point p;
 
-			// save drag origin
-			xp = (gint) event->button.x; 
-			yp = (gint) event->button.y;
-			within_tolerance = true;
-
 			p = sp_desktop_w2d_xy_point (knot->desktop, NR::Point(event->button.x, event->button.y));
-			knot->grabbed_rel_pos = p - knot->pos;
-			knot->drag_origin = knot->pos;
-			if (!nograb) {
-				sp_canvas_item_grab (knot->item,
-						     KNOT_EVENT_MASK,
-						     knot->cursor[SP_KNOT_STATE_DRAGGING],
-						     event->button.time);
-			}
-			sp_knot_set_flag (knot, SP_KNOT_GRABBED, TRUE);
-			grabbed = TRUE;
+
+			sp_knot_start_dragging (knot, p, (gint) event->button.x, (gint) event->button.y, event->button.time);
 			consumed = TRUE;
 		}
 		break;
@@ -536,6 +563,7 @@ sp_knot_handler (SPCanvasItem *item, GdkEvent *event, SPKnot *knot)
 			     && ( abs( (gint) event->motion.y - yp ) < tolerance ) ) {
 				break; // do not drag if we're within tolerance from origin
 			}
+
 			// Once the user has moved farther than tolerance from the original location 
 			// (indicating they intend to move the object, not click), then always process the 
 			// motion notify coordinates as given (no snapping back to origin)
@@ -584,7 +612,7 @@ sp_knot_handler (SPCanvasItem *item, GdkEvent *event, SPKnot *knot)
 		consumed = TRUE;
 		break;
 	case GDK_KEY_PRESS: // keybindings for knot
-		switch (event->key.keyval) {  
+		switch (get_group0_keyval (&event->key)) {  
 		case GDK_Escape:
 			sp_knot_set_flag (knot, SP_KNOT_GRABBED, FALSE);
 			if (!nograb) {
@@ -722,6 +750,18 @@ sp_knot_set_position (SPKnot * knot, NR::Point *p, guint state)
 		       state);
 }
 
+void
+sp_knot_moveto (SPKnot * knot, NR::Point *p)
+{
+	g_return_if_fail (knot != NULL);
+	g_return_if_fail (SP_IS_KNOT (knot));
+
+	knot->pos = *p;
+
+	if (knot->item)
+		SP_CTRL (knot->item)->moveto (*p);
+}
+
 NR::Point sp_knot_position (SPKnot const * knot)
 {
 	g_assert(knot != NULL);
