@@ -2,6 +2,9 @@
  *  FontFactory.cpp
  *  testICU
  *
+ *   Authors:
+ *     fred
+ *     bulia byak <buliabyak@users.sf.net>
  *
  */
 
@@ -311,14 +314,10 @@ font_factory::~font_factory(void)
 	//	g_object_unref(fontContext);
 }
 
-font_instance* font_factory::FaceFromDescr(const char* descr)
+font_instance* font_factory::FaceFromDescr(const char* family, const char* style)
 {
-	PangoFontDescription* temp_descr=pango_font_description_from_string(descr);
-        // workaround for bug #1025565.
-        if (pango_font_description_get_family(temp_descr)==NULL) {
-            pango_font_description_set_family(temp_descr,descr);
-            //g_warning(_("Font '%s' didn't have a default family\n"),descr);
-        }
+	PangoFontDescription* temp_descr=pango_font_description_from_string(style);
+    pango_font_description_set_family(temp_descr,family);
 	font_instance *res=Face(temp_descr);
 	pango_font_description_free(temp_descr);
 	return res;
@@ -461,11 +460,11 @@ void font_factory::UnrefFace(font_instance* who)
 	if ( loadedFaces.find(who->descr) == loadedFaces.end() ) {
 		// not found
 		char* tc=pango_font_description_to_string(who->descr);
-		g_warning("unrefFace %x=%s: failed\n",(unsigned int)who,tc);
+		g_warning("unrefFace %p=%s: failed\n",who,tc);
 		free(tc);
 	} else {
 		loadedFaces.erase(loadedFaces.find(who->descr));
-		//			printf("unrefFace %x: success\n",who);
+		//			printf("unrefFace %p: success\n",who);
 	}
 }
 
