@@ -2,6 +2,10 @@
  *  FontInstance.cpp
  *  testICU
  *
+ *   Authors:
+ *     fred
+ *     bulia byak <buliabyak@users.sf.net>
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -44,7 +48,7 @@ size_t  font_style_hash::operator()(const font_style &x) const {
 	h*=12186;
 	h+=n;
 	if ( x.stroke_width >= 0.01 ) {
-		n=x.stroke_cap*10+x.stroke_join;
+		n=x.stroke_cap*10+x.stroke_join+(int)(x.stroke_miter_limit*100);
 		h*=12186;
 		h+=n;
 		if ( x.nbDash > 0 ) {
@@ -83,6 +87,7 @@ bool  font_style_equal::operator()(const font_style &a,const font_style &b) {
 	
 	if ( a.stroke_cap != b.stroke_cap ) return false;
 	if ( a.stroke_join != b.stroke_join ) return false;
+    if ( fabs(a.stroke_miter_limit-b.stroke_miter_limit) > 0.01) return false;
 	if ( a.nbDash != b.nbDash ) return false;
 	if ( a.nbDash <= 0 ) return true;
 	if ( fabs(a.dash_offset-b.dash_offset) < 0.01 ) {
@@ -456,6 +461,26 @@ bool font_instance::FontMetrics(double &ascent,double &descent,double &leading)
 	return true;
 }
 
+bool font_instance::FontSlope(double &run, double &rise)
+{
+    run = 0.0;
+    rise = 1.0;
+
+    theFace=pango_ft2_font_get_face(pFont);
+
+	if ( theFace->units_per_EM == 0 ) return false; // bitmap font
+	
+	if ( pFont == NULL ) return false;
+	
+	if ( theFace == NULL ) return false;
+
+    TT_HoriHeader *hhea = (TT_HoriHeader*)FT_Get_Sfnt_Table(theFace, ft_sfnt_hhea);
+    if (hhea == NULL) return false;
+    run = hhea->caret_Slope_Run;
+    rise = hhea->caret_Slope_Rise;
+	return true;
+}
+
 NR::Rect font_instance::BBox(int glyph_id)
 {
 	int no=-1;
@@ -540,7 +565,7 @@ double font_instance::Advance(int glyph_id,bool vertical)
 }	
 
 
-raster_font* font_instance::RasterFont(const NR::Matrix &trs,double stroke_width,bool vertical,JoinType stroke_join,ButtType stroke_cap)
+raster_font* font_instance::RasterFont(const NR::Matrix &trs,double stroke_width,bool vertical,JoinType stroke_join,ButtType stroke_cap,float miter_limit)
 {
 	font_style  nStyle;
 	nStyle.transform=trs;
@@ -559,6 +584,9 @@ raster_font* font_instance::RasterFont(const font_style &inStyle)
 	raster_font  *res=NULL;
 	double *savDashes=NULL;
 	font_style nStyle=inStyle;
+    // for some evil reason font_style doesn't have a copy ctor, so the
+    // stuff that should be done there is done here instead (because the
+    // raster_font ctor copies nStyle).
 	if ( nStyle.stroke_width > 0 && nStyle.nbDash > 0 && nStyle.dashes ) {
 		savDashes=nStyle.dashes;
 		nStyle.dashes=(double*)malloc(nStyle.nbDash*sizeof(double));
@@ -572,9 +600,9 @@ raster_font* font_instance::RasterFont(const font_style &inStyle)
 		res=nR;
 		if ( res ) Ref();
 	} else {
-		if ( nStyle.dashes ) free(nStyle.dashes); // since they're not taken by a new rasterfont
 		res=loadedStyles[nStyle];
 		res->Ref();
+		if ( nStyle.dashes ) free(nStyle.dashes); // since they're not taken by a new rasterfont
 	}
 	nStyle.dashes=savDashes;
 	return res;
