@@ -62,6 +62,7 @@
 #include "unit-constants.h"
 
 #include "svg/svg.h"
+#include "svg/stringstream.h"
 
 #include "inkscape-private.h"
 #include "inkscape-stock.h"
@@ -74,9 +75,12 @@
 
 #include "io/sys.h"
 
+#include "debug/logger.h"
+
 #include <extension/extension.h>
 #include <extension/system.h>
 #include <extension/db.h>
+#include <extension/output.h>
 
 #ifdef WIN32
 #include "extension/internal/win32.h"
@@ -84,6 +88,7 @@
 #include "extension/init.h"
 
 #include <glibmm/i18n.h>
+#include <gtkmm/main.h>
 
 #ifndef HAVE_BIND_TEXTDOMAIN_CODESET
 #define bind_textdomain_codeset(p,c)
@@ -92,8 +97,8 @@
 #define gtk_window_set_default_icon_from_file(f,v)
 #endif
 
+#include "application/application.h"
 
-#include <gtkmm/main.h>
 enum {
     SP_ARG_NONE,
     SP_ARG_NOGUI,
@@ -103,6 +108,7 @@ enum {
     SP_ARG_EXPORT_PNG,
     SP_ARG_EXPORT_DPI,
     SP_ARG_EXPORT_AREA,
+    SP_ARG_EXPORT_AREA_DRAWING,
     SP_ARG_EXPORT_WIDTH,
     SP_ARG_EXPORT_HEIGHT,
     SP_ARG_EXPORT_ID,
@@ -111,15 +117,28 @@ enum {
     SP_ARG_EXPORT_BACKGROUND,
     SP_ARG_EXPORT_BACKGROUND_OPACITY,
     SP_ARG_EXPORT_SVG,
+    SP_ARG_EXPORT_PS,
+    SP_ARG_EXPORT_EPS,
+    SP_ARG_EXPORT_TEXT_TO_PATH,
+    SP_ARG_EXPORT_BBOX_PAGE,
+    SP_ARG_EXTENSIONDIR,
     SP_ARG_SLIDESHOW,
-    SP_ARG_BITMAP_ICONS,
+    SP_ARG_QUERY_X,
+    SP_ARG_QUERY_Y,
+    SP_ARG_QUERY_WIDTH,
+    SP_ARG_QUERY_HEIGHT,
+    SP_ARG_QUERY_ID,
     SP_ARG_VERSION,
+    SP_ARG_NEW_GUI,
+    SP_ARG_VACUUM_DEFS,
     SP_ARG_LAST
 };
 
 int sp_main_gui(int argc, char const **argv);
 int sp_main_console(int argc, char const **argv);
 static void sp_do_export_png(SPDocument *doc);
+static void do_export_ps(SPDocument* doc, gchar const* uri, char const *mime);
+static void do_query_dimension (SPDocument *doc, bool extent, NR::Dim2 const axis, const gchar *id);
 
 
 static gchar *sp_global_printer = NULL;
@@ -127,6 +146,7 @@ static gboolean sp_global_slideshow = FALSE;
 static gchar *sp_export_png = NULL;
 static gchar *sp_export_dpi = NULL;
 static gchar *sp_export_area = NULL;
+static gboolean sp_export_area_drawing = FALSE;
 static gchar *sp_export_width = NULL;
 static gchar *sp_export_height = NULL;
 static gchar *sp_export_id = NULL;
@@ -135,6 +155,22 @@ static gchar *sp_export_background_opacity = NULL;
 static gboolean sp_export_use_hints = FALSE;
 static gboolean sp_export_id_only = FALSE;
 static gchar *sp_export_svg = NULL;
+static gchar *sp_export_ps = NULL;
+static gchar *sp_export_eps = NULL;
+static gboolean sp_export_text_to_path = FALSE;
+static gboolean sp_export_bbox_page = FALSE;
+static gboolean sp_query_x = FALSE;
+static gboolean sp_query_y = FALSE;
+static gboolean sp_query_width = FALSE;
+static gboolean sp_query_height = FALSE;
+static gchar *sp_query_id = NULL;
+static int sp_new_gui = FALSE;
+static gboolean sp_vacuum_defs = FALSE;
+
+static gchar *sp_export_png_utf8 = NULL;
+static gchar *sp_export_svg_utf8 = NULL;
+static gchar *sp_global_printer_utf8 = NULL;
+
 
 static GSList *sp_process_args(poptContext ctx);
 struct poptOption options[] = {
@@ -175,9 +211,14 @@ struct poptOption options[] = {
 
     {"export-area", 'a', 
      POPT_ARG_STRING, &sp_export_area, SP_ARG_EXPORT_AREA,
-     N_("Exported area in SVG pixels (default is full document; 0,0 is lower-left corner)"),
+     N_("Exported area in SVG pixels (default is the canvas; 0,0 is lower-left corner)"),
      N_("x0:y0:x1:y1")},
 
+    {"export-area-drawing", 'D', 
+     POPT_ARG_NONE, &sp_export_area_drawing, SP_ARG_EXPORT_AREA_DRAWING,
+     N_("Exported area is the entire drawing (not canvas)"),
+     NULL},
+
     {"export-width", 'w', 
      POPT_ARG_STRING, &sp_export_width, SP_ARG_EXPORT_WIDTH,
      N_("The width of exported bitmap in pixels (overrides export-dpi)"), 
@@ -198,7 +239,7 @@ struct poptOption options[] = {
      // TRANSLATORS: this means: "Only export the object whose id is given in --export-id".
      //  See "man inkscape" for details.
      N_("Export just the object with export-id, hide all others (only with export-id)"), 
-     N_("ID")},
+     NULL},
 
     {"export-use-hints", 't', 
      POPT_ARG_NONE, &sp_export_use_hints, SP_ARG_EXPORT_USE_HINTS,
@@ -220,20 +261,82 @@ struct poptOption options[] = {
      N_("Export document to plain SVG file (no sodipodi or inkscape namespaces)"), 
      N_("FILENAME")},
 
+    {"export-ps", 'P',
+     POPT_ARG_STRING, &sp_export_ps, SP_ARG_EXPORT_PS,
+     N_("Export document to a PS file"),
+     N_("FILENAME")},
+
+    {"export-eps", 'E',
+     POPT_ARG_STRING, &sp_export_eps, SP_ARG_EXPORT_EPS,
+     N_("Export document to an EPS file"),
+     N_("FILENAME")},
+
+    {"export-text-to-path", 'T',
+     POPT_ARG_NONE, &sp_export_text_to_path, SP_ARG_EXPORT_TEXT_TO_PATH,
+     N_("Convert text object to paths on export (EPS)"),
+     NULL},
+
+    {"export-bbox-page", 'B',
+     POPT_ARG_NONE, &sp_export_bbox_page, SP_ARG_EXPORT_BBOX_PAGE,
+     N_("Export files with the bounding box set to the page size (EPS)"),
+     NULL},
+
+    {"query-x", 'X',
+     POPT_ARG_NONE, &sp_query_x, SP_ARG_QUERY_X,
+     // TRANSLATORS: "--query-id" is an Inkscape command line option; see "inkscape --help"
+     N_("Query the X coordinate of the drawing or, if specified, of the object with --query-id"),
+     NULL},
+
+    {"query-y", 'Y',
+     POPT_ARG_NONE, &sp_query_y, SP_ARG_QUERY_Y,
+     // TRANSLATORS: "--query-id" is an Inkscape command line option; see "inkscape --help"
+     N_("Query the Y coordinate of the drawing or, if specified, of the object with --query-id"),
+     NULL},
+
+    {"query-width", 'W',
+     POPT_ARG_NONE, &sp_query_width, SP_ARG_QUERY_WIDTH,
+     // TRANSLATORS: "--query-id" is an Inkscape command line option; see "inkscape --help"
+     N_("Query the width of the drawing or, if specified, of the object with --query-id"),
+     NULL},
+
+    {"query-height", 'H',
+     POPT_ARG_NONE, &sp_query_height, SP_ARG_QUERY_HEIGHT,
+     // TRANSLATORS: "--query-id" is an Inkscape command line option; see "inkscape --help"
+     N_("Query the height of the drawing or, if specified, of the object with --query-id"),
+     NULL},
+
+    {"query-id", 'I', 
+     POPT_ARG_STRING, &sp_query_id, SP_ARG_QUERY_ID,
+     N_("The ID of the object whose dimensions are queried"), 
+     N_("ID")},
+
+    {"extension-directory", 'x',
+     POPT_ARG_NONE, NULL, SP_ARG_EXTENSIONDIR,
+     // TRANSLATORS: this option makes Inkscape print the name (path) of the extension directory
+     N_("Print out the extension directory and exit"),
+     NULL},
+
     {"slideshow", 's', 
      POPT_ARG_NONE, &sp_global_slideshow, SP_ARG_SLIDESHOW,
      N_("Show given files one-by-one, switch to next on any key/mouse event"), 
      NULL},
 
+    {"new-gui", 'G', 
+     POPT_ARG_NONE, &sp_new_gui, SP_ARG_NEW_GUI,
+     N_("Use the new Gtkmm GUI interface"),
+     NULL},
+
+    {"vacuum-defs", 0,
+     POPT_ARG_NONE, &sp_vacuum_defs, SP_ARG_VACUUM_DEFS,
+     N_("Remove unused definitions from the defs section(s) of the document"),
+     NULL},
+
     POPT_AUTOHELP POPT_TABLEEND
 };
 
 int
-main(int argc, char const **argv)
+main(int argc, char **argv)
 {
-    gboolean use_gui;
-    gint result, i;
-
 #ifdef HAVE_FPSETMASK
     /* This is inherited from Sodipodi code, where it was in #ifdef __FreeBSD__.  It's probably
        safe to remove: the default mask is already 0 in C99, and in current FreeBSD according to
@@ -261,6 +364,9 @@ main(int argc, char const **argv)
 
     Inkscape::GC::init();
 
+    Inkscape::Debug::Logger::init();
+
+    gboolean use_gui;
 #ifndef WIN32
     use_gui = (getenv("DISPLAY") != NULL);
 #else
@@ -279,7 +385,7 @@ main(int argc, char const **argv)
     use_gui = TRUE;
 #endif
     /* Test whether with/without GUI is forced */
-    for (i = 1; i < argc; i++) {
+    for (int i = 1; i < argc; i++) {
         if (!strcmp(argv[i], "-z")
             || !strcmp(argv[i], "--without-gui")
             || !strcmp(argv[i], "-p")
@@ -289,7 +395,22 @@ main(int argc, char const **argv)
             || !strcmp(argv[i], "-l")
             || !strncmp(argv[i], "--export-plain-svg", 12)
             || !strcmp(argv[i], "-i")
+            || !strncmp(argv[i], "--export-area-drawing", 21)
+            || !strcmp(argv[i], "-D")
             || !strncmp(argv[i], "--export-id", 12)
+            || !strcmp(argv[i], "-P")
+            || !strncmp(argv[i], "--export-ps", 11)
+            || !strcmp(argv[i], "-E")
+            || !strncmp(argv[i], "--export-eps", 12)
+            || !strcmp(argv[i], "-W")
+            || !strncmp(argv[i], "--query-width", 13)
+            || !strcmp(argv[i], "-H")
+            || !strncmp(argv[i], "--query-height", 14)
+            || !strcmp(argv[i], "-X")
+            || !strncmp(argv[i], "--query-x", 13)
+            || !strcmp(argv[i], "-Y")
+            || !strncmp(argv[i], "--query-y", 14)
+            || !strcmp(argv[i], "--vacuum-defs")
            )
         {
             /* main_console handles any exports -- not the gui */
@@ -298,26 +419,53 @@ main(int argc, char const **argv)
         } else if (!strcmp(argv[i], "-g") || !strcmp(argv[i], "--with-gui")) {
             use_gui = TRUE;
             break;
+        } else if (!strcmp(argv[i], "-G") || !strcmp(argv[i], "--new-gui")) {
+            sp_new_gui = TRUE;
+            break;
         }
     }
 
-    if (use_gui) {
-        result = sp_main_gui(argc, argv);
-    } else {
-        result = sp_main_console(argc, argv);
-    }
+    // TODO:  Should this be a static object (see inkscape.cpp)?
+    Inkscape::NSApplication::Application app(argc, argv, use_gui, sp_new_gui);
 
-    return result;
+    return app.run();
 }
 
-int
-sp_main_gui(int argc, char const **argv)
+void fixupSingleFilename( gchar **orig, gchar **spare )
 {
-    GSList *fl = NULL;
+    if ( orig && *orig && **orig ) {
+        GError *error = NULL;
+        gchar *newFileName = Inkscape::IO::filename_to_utf8_fallback(*orig, -1, NULL, NULL, &error);
+        if ( newFileName )
+        {
+            *orig = newFileName;
+            if ( spare ) {
+                *spare = newFileName;
+            }
+//             g_message("Set a replacement fixup");
+        }
+    }
+}
 
-    //gtk_init(&argc, const_cast<char ***>(&argv));
-    Gtk::Main main_instance (&argc, const_cast<char ***>(&argv));
+GSList *fixupFilenameEncoding( GSList* fl )
+{
+    GSList *newFl = NULL;
+    while ( fl ) {
+        gchar *fn = static_cast<gchar*>(fl->data);
+        fl = g_slist_remove( fl, fl->data );
+        gchar *newFileName = Inkscape::IO::filename_to_utf8_fallback(fn, -1, NULL, NULL, NULL);
+        if ( newFileName ) {
+            g_free( fn );
+            fn = newFileName;
+            newFileName = 0;
+        }
+        newFl = g_slist_append( newFl, fn );
+    }
+    return newFl;
+}
 
+int sp_common_main( int argc, char const **argv, GSList **flDest )
+{
     /* fixme: Move these to some centralized location (Lauris) */
     sp_object_type_register("sodipodi:namedview", SP_TYPE_NAMEDVIEW);
     sp_object_type_register("sodipodi:guide", SP_TYPE_GUIDE);
@@ -331,13 +479,44 @@ sp_main_gui(int argc, char const **argv)
     poptContext ctx = poptGetContext(NULL, argc, argv, options, 0);
     poptSetOtherOptionHelp(ctx, _("[OPTIONS...] [FILE...]\n\nAvailable options:"));
     g_return_val_if_fail(ctx != NULL, 1);
+
     /* Collect own arguments */
-    fl = sp_process_args(ctx);
+    GSList *fl = sp_process_args(ctx);
     poptFreeContext(ctx);
 
     // now switch gettext back to UTF-8 (for GUI)
     bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
 
+    // Now let's see if the file list still holds up
+    fl = fixupFilenameEncoding( fl );
+
+    // Check the globals for filename-fixup
+    fixupSingleFilename( &sp_export_png, &sp_export_png_utf8 );
+    fixupSingleFilename( &sp_export_svg, &sp_export_svg_utf8 );
+    fixupSingleFilename( &sp_global_printer, &sp_global_printer_utf8 );
+
+    // Return the list if wanted, else free it up.
+    if ( flDest ) {
+        *flDest = fl;
+        fl = 0;
+    } else {
+        while ( fl ) {
+            g_free( fl->data );
+            fl = g_slist_remove( fl, fl->data );
+        }
+    }
+    return 0;
+}
+
+int
+sp_main_gui(int argc, char const **argv)
+{
+    Gtk::Main main_instance (&argc, const_cast<char ***>(&argv));
+
+    GSList *fl = NULL;
+    int retVal = sp_common_main( argc, argv, &fl );
+    g_return_val_if_fail(retVal == 0, 1);
+
     inkscape_gtk_stock_init();
 
     /* Set default icon */
@@ -346,6 +525,7 @@ sp_main_gui(int argc, char const **argv)
         gtk_window_set_default_icon_from_file(filename, NULL);
     }
     g_free (filename);
+    filename = 0;
 
     if (!sp_global_slideshow) {
         gboolean create_new = TRUE;
@@ -388,10 +568,6 @@ sp_main_gui(int argc, char const **argv)
 int
 sp_main_console(int argc, char const **argv)
 {
-    poptContext ctx = NULL;
-    GSList * fl = NULL;
-    gchar *printer;
-
     /* We are started in text mode */
 
     /* Do this g_type_init(), so that we can use Xft/Freetype2 (Pango)
@@ -400,50 +576,19 @@ sp_main_console(int argc, char const **argv)
      * http://mail.gnome.org/archives/gtk-list/2003-December/msg00063.html
      */
     g_type_init();
+    char **argv2 = const_cast<char **>(argv);
+    gtk_init_check( &argc, &argv2 );
+    //setlocale(LC_ALL, "");
 
-    /* fixme: Move these to some centralized location (Lauris) */
-    sp_object_type_register("sodipodi:namedview", SP_TYPE_NAMEDVIEW);
-    sp_object_type_register("sodipodi:guide", SP_TYPE_GUIDE);
-
-    // temporarily switch gettext encoding to locale, so that help messages can be output properly
-    gchar const *charset;
-    g_get_charset(&charset);
-    bind_textdomain_codeset(GETTEXT_PACKAGE, charset);
-
-    ctx = poptGetContext(NULL, argc, argv, options, 0);
-    poptSetOtherOptionHelp(ctx, _("[OPTIONS...] [FILE...]\n\nAvailable options:"));
-    g_return_val_if_fail(ctx != NULL, 1);
-    fl = sp_process_args(ctx);
-    poptFreeContext(ctx);
-
-    // now switch gettext back to UTF-8 (for GUI)
-    bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
+    GSList *fl = NULL;
+    int retVal = sp_common_main( argc, argv, &fl );
+    g_return_val_if_fail(retVal == 0, 1);
 
     if (fl == NULL) {
         g_print("Nothing to do!\n");
         exit(0);
     }
 
-    /* Check for and set up printing path */
-    printer = NULL;
-    if (sp_global_printer != NULL) {
-#if 0
-        if ((sp_global_printer[0] != '|') && (sp_global_printer[0] != '/')) {
-            gchar *cwd;
-            /* Gnome-print appends relative paths to $HOME by default */
-            cwd = g_get_current_dir();
-            printer = g_build_filename(cwd, sp_global_printer, NULL);
-            g_free(cwd);
-        } else {
-            printer = g_strdup(sp_global_printer);
-        }
-#else
-        printer = g_strdup(sp_global_printer);
-#endif
-    }
-
-    /* Start up g type system, without requiring X */
-    g_type_init(); /* this is already called above... ? */
     inkscape_application_init(argv[0], false);
 
     while (fl) {
@@ -456,46 +601,104 @@ sp_main_console(int argc, char const **argv)
         if (doc == NULL) {
             g_warning("Specified document %s cannot be opened (is it valid SVG file?)", (gchar *) fl->data);
         } else {
-            if (printer) {
-                sp_print_document_to_file(doc, printer);
+            if (sp_vacuum_defs) {
+                vacuum_document(doc);
             }
-            if (sp_export_png || sp_export_id) {
+            if (sp_vacuum_defs && !sp_export_svg) {
+                // save under the name given in the command line
+                sp_repr_save_file(doc->rdoc, (gchar *)fl->data, SP_SVG_NS_URI);
+            }
+            if (sp_global_printer) {
+                sp_print_document_to_file(doc, sp_global_printer);
+            }
+            if (sp_export_png || sp_export_id || sp_export_area_drawing) {
                 sp_do_export_png(doc);
             }
             if (sp_export_svg) {
-                SPReprDoc *rdoc;
-                SPRepr *repr;
+                Inkscape::XML::Document *rdoc;
+                Inkscape::XML::Node *repr;
                 rdoc = sp_repr_document_new("svg:svg");
                 repr = rdoc->root();
                 repr = sp_document_root(doc)->updateRepr(repr, SP_OBJECT_WRITE_BUILD);
                 sp_repr_save_file(repr->document(), sp_export_svg, SP_SVG_NS_URI);
             }
+            if (sp_export_ps) {
+                do_export_ps(doc, sp_export_ps, "image/x-postscript");
+            }
+            if (sp_export_eps) {
+                do_export_ps(doc, sp_export_eps, "image/x-e-postscript");
+            }
+            if (sp_query_width || sp_query_height) {
+                do_query_dimension (doc, true, sp_query_width? NR::X : NR::Y, sp_query_id);
+            } else if (sp_query_x || sp_query_y) {
+                do_query_dimension (doc, false, sp_query_x? NR::X : NR::Y, sp_query_id);
+            }
         }
         fl = g_slist_remove(fl, fl->data);
     }
 
-    g_free(printer);
-
     inkscape_unref();
 
     return 0;
 }
 
+static void
+do_query_dimension (SPDocument *doc, bool extent, NR::Dim2 const axis, const gchar *id)
+{
+    SPObject *o = NULL;
+
+    if (id) {
+        o = doc->getObjectById(id);
+        if (o) {
+            if (!SP_IS_ITEM (o)) {
+                g_warning("Object with id=\"%s\" is not a visible item. Cannot query dimensions.", id);
+                return;
+            }
+        } else {
+            g_warning("Object with id=\"%s\" is not found. Cannot query dimensions.", id);
+            return;
+        }
+    } else {
+        o = SP_DOCUMENT_ROOT(doc);
+    }
+
+    if (o) {
+        sp_document_ensure_up_to_date (doc);
+        NR::Rect area = sp_item_bbox_desktop((SPItem *) o);
+
+        Inkscape::SVGOStringStream os;
+        if (extent) {
+            os << area.extent(axis);
+        } else {
+            os << area.min()[axis];
+        }
+        g_print ("%s\n", os.str().c_str());
+    }
+}
+
+
 static void
 sp_do_export_png(SPDocument *doc)
 {
     const gchar *filename = NULL;
     gdouble dpi = 0.0;
 
-    if (sp_export_use_hints && !sp_export_id) {
-        g_warning ("--export-use-hints can only be used with --export-id; ignored.");
+    if (sp_export_use_hints && (!sp_export_id && !sp_export_area_drawing)) {
+        g_warning ("--export-use-hints can only be used with --export-id or --export-area-drawing; ignored.");
     }
 
     GSList *items = NULL;
 
     NRRect area;
-    if (sp_export_id) {
-        SPObject *o = doc->getObjectById(sp_export_id);
+    if (sp_export_id || sp_export_area_drawing) {
+
+        SPObject *o = NULL;
+        if (sp_export_id) {
+            o = doc->getObjectById(sp_export_id);
+        } else if (sp_export_area_drawing) {
+            o = SP_DOCUMENT_ROOT (doc);
+        }
+
         if (o) {
             if (!SP_IS_ITEM (o)) {
                 g_warning("Object with id=\"%s\" is not a visible item. Nothing exported.", sp_export_id);
@@ -559,7 +762,7 @@ sp_do_export_png(SPDocument *doc)
             return;
         }
     } else {
-        /* Export the whole document */
+        /* Export the whole canvas */
         area.x0 = 0.0;
         area.y0 = 0.0;
         area.x1 = sp_document_width(doc);
@@ -624,7 +827,7 @@ sp_do_export_png(SPDocument *doc)
         bgcolor |= 0xff; // default is no opacity
     } else {
         // read from namedview
-        SPRepr *nv = sp_repr_lookup_name (doc->rroot, "sodipodi:namedview");
+        Inkscape::XML::Node *nv = sp_repr_lookup_name (doc->rroot, "sodipodi:namedview");
         if (nv && nv->attribute("pagecolor"))
             bgcolor = sp_svg_read_color(nv->attribute("pagecolor"), 0xffffff00);
         if (nv && nv->attribute("inkscape:pageopacity"))
@@ -652,7 +855,7 @@ sp_do_export_png(SPDocument *doc)
     g_print("Area %g:%g:%g:%g exported to %d x %d pixels (%g dpi)\n", area.x0, area.y0, area.x1, area.y1, width, height, dpi);
 
     g_print("Bitmap saved as: %s\n", filename);
-
+  
     if ((width >= 1) && (height >= 1) && (width < 65536) && (height < 65536)) {
         sp_export_png_file(doc, filename, area.x0, area.y0, area.x1, area.y1, width, height, bgcolor, NULL, NULL, true, sp_export_id_only ? items : NULL);
     } else {
@@ -662,6 +865,65 @@ sp_do_export_png(SPDocument *doc)
     g_slist_free (items);
 }
 
+
+/**
+ *  Perform an export of either PS or EPS.
+ *
+ *  \param doc Document to export.
+ *  \param uri URI to export to.
+ *  \param mime MIME type to export as.
+ */
+
+static void do_export_ps(SPDocument* doc, gchar const* uri, char const* mime)
+{
+    /* FIXME: I've no idea if this is the `proper' way to do this.
+    ** If anyone feels qualified to say that it is, perhaps they
+    ** could remove this comment.
+    */
+
+    Inkscape::Extension::DB::OutputList o;
+    Inkscape::Extension::db.get_output_list(o);
+    Inkscape::Extension::DB::OutputList::const_iterator i = o.begin();
+    while (i != o.end() && strcmp( (*i)->get_mimetype(), mime ) != 0) {
+        i++;
+    }
+
+    if (i == o.end())
+    {
+        g_warning ("Could not find an extension to export this file.");
+        return;
+    }
+
+    bool old_text_to_path = false;
+    bool old_bbox_page = false;
+    
+    try {
+        old_text_to_path = (*i)->get_param_bool("textToPath");
+        (*i)->set_param_bool("textToPath", sp_export_text_to_path);
+    }
+    catch (...) {
+        g_warning ("Could not set export-text-to-path option for this export.");
+    }
+
+    try {
+        old_bbox_page = (*i)->get_param_bool("pageBoundingBox");
+        (*i)->set_param_bool("pageBoundingBox", sp_export_bbox_page);
+    }
+    catch (...) {
+        g_warning ("Could not set export-bbox-page option for this export.");
+    }
+    
+    (*i)->save(doc, uri);
+
+    try {
+        (*i)->set_param_bool("textToPath", old_text_to_path);
+        (*i)->set_param_bool("pageBoundingBox", old_bbox_page);
+    }
+    catch (...) {
+        
+    }
+}
+
 static GSList *
 sp_process_args(poptContext ctx)
 {
@@ -673,18 +935,7 @@ sp_process_args(poptContext ctx)
             case SP_ARG_FILE: {
                 gchar const *fn = poptGetOptArg(ctx);
                 if (fn != NULL) {
-                    // TODO: bulia, please look over
-                    gsize bytesRead = 0;
-                    gsize bytesWritten = 0;
-                    GError *error = NULL;
-                    gchar *newFileName = g_filename_to_utf8(fn,
-                                                            -1,
-                                                            &bytesRead,
-                                                            &bytesWritten,
-                                                            &error);
-                    fl = g_slist_append(fl, ( (newFileName != NULL)
-                                              ? newFileName
-                                              : g_strdup(fn) ));
+                    fl = g_slist_append(fl, g_strdup(fn));
                 }
                 break;
             }
@@ -693,6 +944,11 @@ sp_process_args(poptContext ctx)
                 exit(0);
                 break;
             }
+            case SP_ARG_EXTENSIONDIR: {
+                printf("%s\n", INKSCAPE_EXTENSIONDIR);
+                exit(0);
+                break;
+            }
             default: {
                 break;
             }
@@ -702,19 +958,7 @@ sp_process_args(poptContext ctx)
     gchar const ** const args = poptGetArgs(ctx);
     if (args != NULL) {
         for (unsigned i = 0; args[i] != NULL; i++) {
-            // TODO: bulia, please look over
-            gsize bytesRead = 0;
-            gsize bytesWritten = 0;
-            GError *error = NULL;
-            // fixme: check to see if this will leak args[i]
-            gchar *newFileName = g_filename_to_utf8(args[i],
-                                                    -1,
-                                                    &bytesRead,
-                                                    &bytesWritten,
-                                                    &error);
-            fl = g_slist_append(fl, ( (newFileName != NULL)
-                                      ? newFileName
-                                      : (gpointer) args[i]));
+            fl = g_slist_append(fl, g_strdup(args[i]));
         }
     }
 
