@@ -6,6 +6,7 @@
  *
  * Authors:
  *   Lauris Kaplinski <lauris@kaplinski.com>
+ *   bulia byak <buliabyak@users.sf.net>
  *
  * This code is in public domain
  */
@@ -15,7 +16,7 @@
 #include "forward.h"
 #include "nodepath.h"
 struct SPKnotHolder;
-class SPSelection;
+namespace Inkscape { class Selection; }
 
 #define SP_TYPE_NODE_CONTEXT            (sp_node_context_get_type ())
 #define SP_NODE_CONTEXT(obj)            (GTK_CHECK_CAST ((obj), SP_TYPE_NODE_CONTEXT, SPNodeContext))
@@ -31,8 +32,7 @@ struct SPNodeContext {
 
 	guint drag : 1;
 
-	Path::Path *nodepath;
-	SPKnotHolder *knot_holder;
+	Inkscape::NodePath::Path *nodepath;
 
 	gboolean leftalt;
 	gboolean rightalt;
@@ -52,6 +52,6 @@ struct SPNodeContextClass {
 
 GtkType sp_node_context_get_type (void);
 
-void sp_node_context_selection_changed (SPSelection * selection, gpointer data);
+void sp_node_context_selection_changed (Inkscape::Selection * selection, gpointer data);
 
 #endif
