@@ -16,11 +16,13 @@
 #include <glib.h>
 #include <math.h>
 
+#include "sp-item.h"
 #include "sp-rect.h"
 #include "sp-ellipse.h"
 #include "sp-star.h"
 #include "sp-spiral.h"
 #include "sp-offset.h"
+#include "sp-flowtext.h"
 #include "prefs-utils.h"
 #include "document.h"
 #include <style.h>
@@ -39,9 +41,12 @@
 
 #include <libnr/nr-matrix.h>
 #include <libnr/nr-matrix-ops.h>
+#include <libnr/nr-point-matrix-ops.h>
 
 #include "xml/repr.h"
 
+#include "isnan.h"
+
 #define sp_round(v,m) (((v) < 0.0) ? ((ceil ((v) / (m) - 0.5)) * (m)) : ((floor ((v) / (m) + 0.5)) * (m)))
 
 static SPKnotHolder *sp_rect_knot_holder (SPItem *item, SPDesktop *desktop);
@@ -50,6 +55,7 @@ static SPKnotHolder *sp_star_knot_holder (SPItem *item, SPDesktop *desktop);
 static SPKnotHolder *sp_spiral_knot_holder (SPItem * item, SPDesktop *desktop);
 static SPKnotHolder *sp_offset_knot_holder (SPItem * item, SPDesktop *desktop);
 static SPKnotHolder *sp_path_knot_holder (SPItem * item, SPDesktop *desktop);
+static SPKnotHolder *sp_flowtext_knot_holder (SPItem * item, SPDesktop *desktop);
 static void sp_pat_knot_holder (SPItem * item, SPKnotHolder *knot_holder);
 
 SPKnotHolder *
@@ -67,9 +73,12 @@ sp_item_knot_holder (SPItem *item, SPDesktop *desktop)
 		return sp_spiral_knot_holder (item, desktop);
 	} else if (SP_IS_OFFSET (item)) {
 		return sp_offset_knot_holder (item, desktop);
-    } else if (SP_IS_PATH (item)) {
-        return sp_path_knot_holder (item, desktop);
-    }
+	} else if (SP_IS_PATH (item)) {
+		return sp_path_knot_holder (item, desktop);
+	} else if (SP_IS_FLOWTEXT (item) && SP_FLOWTEXT(item)->has_internal_frame()) {
+		return sp_flowtext_knot_holder (item, desktop);
+	}
+
 	return NULL;
 }
 
@@ -103,7 +112,7 @@ sp_pattern_xy_set (SPItem *item, const NR::Point &p, const NR::Point &origin, gu
 
      if (state)  {
 	 const NR::Point q = p - sp_pattern_extract_trans(pat);
-         sp_shape_adjust_pattern (item, NR::Matrix(NR::translate(q)));
+         sp_item_adjust_pattern (item, NR::Matrix(NR::translate(q)));
      }
 
      item->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
@@ -153,7 +162,7 @@ sp_pattern_angle_set (SPItem *item, const NR::Point &p, const NR::Point &origin,
     const NR::Point t = sp_pattern_extract_trans(pat);
     rot[4] = t[NR::X];
     rot[5] = t[NR::Y];
-    sp_shape_adjust_pattern (item, rot, true);
+    sp_item_adjust_pattern (item, rot, true);
     item->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
 }
 
@@ -178,7 +187,7 @@ sp_pattern_scale_set (SPItem *item, const NR::Point &p, const NR::Point &origin,
     const NR::Point t = sp_pattern_extract_trans(pat);
     rot[4] = t[NR::X];
     rot[5] = t[NR::Y];
-    sp_shape_adjust_pattern (item, rot, true);
+    sp_item_adjust_pattern (item, rot, true);
     item->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
 
 }
@@ -261,7 +270,7 @@ sp_rect_rx_knot_click (SPItem *item, guint state)
 		sp_repr_set_attr (SP_OBJECT_REPR(rect), "rx", NULL);
 		sp_repr_set_attr (SP_OBJECT_REPR(rect), "ry", NULL);
 	} else if (state & GDK_CONTROL_MASK) {
-		sp_repr_set_attr (SP_OBJECT_REPR(rect), "ry", sp_repr_attr (SP_OBJECT_REPR(rect), "rx"));
+		sp_repr_set_attr (SP_OBJECT_REPR(rect), "ry", SP_OBJECT_REPR(rect)->attribute("rx"));
 	}
 }
 
@@ -274,7 +283,7 @@ sp_rect_ry_knot_click (SPItem *item, guint state)
 		sp_repr_set_attr (SP_OBJECT_REPR(rect), "rx", NULL);
 		sp_repr_set_attr (SP_OBJECT_REPR(rect), "ry", NULL);
 	} else if (state & GDK_CONTROL_MASK) {
-		sp_repr_set_attr (SP_OBJECT_REPR(rect), "rx", sp_repr_attr (SP_OBJECT_REPR(rect), "ry"));
+		sp_repr_set_attr (SP_OBJECT_REPR(rect), "rx", SP_OBJECT_REPR(rect)->attribute("ry"));
 	}
 }
 
@@ -302,10 +311,8 @@ static NR::Point sp_rect_wh_get (SPItem *item)
 }
 
 static void
-sp_rect_wh_set (SPItem *item, const NR::Point &p, const NR::Point &origin, guint state)
+sp_rect_wh_set_internal (SPRect *rect, const NR::Point &p, const NR::Point &origin, guint state)
 {
-	SPRect *rect = SP_RECT(item);
-
 	if (state & GDK_CONTROL_MASK) {
 		// original width/height when drag started
 		gdouble w_orig = (origin[NR::X] - rect->x.computed);
@@ -352,6 +359,14 @@ sp_rect_wh_set (SPItem *item, const NR::Point &p, const NR::Point &origin, guint
 	((SPObject *)rect)->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
 }
 
+static void
+sp_rect_wh_set (SPItem *item, const NR::Point &p, const NR::Point &origin, guint state)
+{
+	SPRect *rect = SP_RECT(item);
+
+	sp_rect_wh_set_internal (rect, p, origin, state);
+}
+
 static NR::Point sp_rect_xy_get (SPItem *item)
 {
 	SPRect *rect = SP_RECT(item);
@@ -839,7 +854,7 @@ sp_spiral_outer_set (SPItem *item, const NR::Point &p, const NR::Point &origin,
 			spiral->rad = rad_new;
 			spiral->t0 = pow (r0 / spiral->rad, 1/spiral->exp);
 		}
-		if (!std::isfinite(spiral->t0)) spiral->t0 = 0.0;
+		if (!isFinite(spiral->t0)) spiral->t0 = 0.0;
 		spiral->t0 = CLAMP (spiral->t0, 0.0, 0.999);	
 	}
 
@@ -927,7 +942,7 @@ sp_offset_knot_holder (SPItem * item, SPDesktop *desktop)
 }
 
 static SPKnotHolder *
-sp_path_knot_holder (SPItem * item, SPDesktop *desktop)
+sp_path_knot_holder (SPItem * item, SPDesktop *desktop) // FIXME: eliminate, instead make a pattern-drag similar to gradient-drag
 {
     if ((SP_OBJECT(item)->style->fill.type == SP_PAINT_TYPE_PAINTSERVER)
         && SP_IS_PATTERN (SP_STYLE_FILL_SERVER (SP_OBJECT(item)->style)))
@@ -958,8 +973,28 @@ sp_pat_knot_holder (SPItem * item, SPKnotHolder *knot_holder)
         }
 }
 
+static NR::Point sp_flowtext_corner_get (SPItem *item)
+{
+	SPRect *rect = SP_RECT(item);
 
+	return NR::Point(rect->x.computed + rect->width.computed, rect->y.computed + rect->height.computed);
+}
 
+static void
+sp_flowtext_corner_set (SPItem *item, const NR::Point &p, const NR::Point &origin, guint state)
+{
+	SPRect *rect = SP_RECT(item);
 
+	sp_rect_wh_set_internal (rect, p, origin, state);
+}
 
+static SPKnotHolder *
+sp_flowtext_knot_holder (SPItem * item, SPDesktop *desktop)
+{
+	SPKnotHolder *knot_holder = sp_knot_holder_new (desktop, SP_FLOWTEXT(item)->get_frame(NULL), NULL);
 
+	sp_knot_holder_add (knot_holder, sp_flowtext_corner_set, sp_flowtext_corner_get, NULL, 
+					_("Drag to resize the <b>flowed text frame</b>"));
+
+	return knot_holder;
+}
