@@ -5,8 +5,9 @@
  *
  * Authors:
  *   Lauris Kaplinski <lauris@kaplinski.com>
+ *   bulia byak <buliabyak@users.sf.net>
  *
- * Copyright (C) 1999-2002 Authors
+ * Copyright (C) 1999-2004 Authors
  * Copyright (C) 2001-2002 Ximian, Inc.
  *
  * Released under GNU GPL, read the file 'COPYING' for more information
@@ -24,6 +25,8 @@
 #include "sp-object.h"
 #include "sp-path.h"
 #include "sp-text.h"
+#include "sp-flowtext.h"
+#include "text-editing.h"
 #include "style.h"
 #include "inkscape.h"
 #include "document.h"
@@ -34,7 +37,7 @@
 
 /* Helper functions for sp_selected_path_to_curves */
 static void sp_selected_path_to_curves0 (gboolean do_document_done, guint32 text_grouping_policy);
-static SPRepr * sp_selected_item_to_curved_repr(SPItem * item, guint32 text_grouping_policy);
+static Inkscape::XML::Node * sp_selected_item_to_curved_repr(SPItem * item, guint32 text_grouping_policy);
 enum {				
   /* Not used yet. This is the placeholder of Lauris's idea. */
 	SP_TOCURVE_INTERACTIVE       = 1 << 0,
@@ -50,7 +53,7 @@ sp_selected_path_combine (void)
 	if (!SP_IS_DESKTOP(desktop))
 	  return;
 	
-	SPSelection *selection = SP_DT_SELECTION (desktop);
+	Inkscape::Selection *selection = SP_DT_SELECTION (desktop);
 	GSList *items = (GSList *) selection->itemList();
 
 	if (g_slist_length (items) < 2) {
@@ -66,7 +69,7 @@ sp_selected_path_combine (void)
 		}
 	}
 
-	SPRepr *parent = SP_OBJECT_REPR ((SPItem *) items->data)->parent();
+	Inkscape::XML::Node *parent = SP_OBJECT_REPR ((SPItem *) items->data)->parent();
 	for (GSList *i = items; i != NULL; i = i->next) {
 		if ( SP_OBJECT_REPR ((SPItem *) i->data)->parent() != parent ) {
 		    desktop->messageStack()->flash(Inkscape::ERROR_MESSAGE, _("You cannot combine objects from <b>different groups</b> or <b>layers</b>."));
@@ -83,13 +86,13 @@ sp_selected_path_combine (void)
 	items = g_slist_sort (items, (GCompareFunc) sp_item_repr_compare_position);
 
 	// remember the position of the topmost object
-	gint topmost = sp_repr_position (SP_OBJECT_REPR ((SPItem *) g_slist_last(items)->data));
+	gint topmost = (SP_OBJECT_REPR ((SPItem *) g_slist_last(items)->data))->position();
 
 	// remember the id of the bottomost object
-	const char *id = sp_repr_attr (SP_OBJECT_REPR ((SPItem *) items->data), "id");
+	const char *id = SP_OBJECT_REPR ((SPItem *) items->data)->attribute("id");
 
 	// FIXME: merge styles of combined objects instead of using the first one's style
-	gchar *style = g_strdup (sp_repr_attr (SP_OBJECT_REPR ((SPItem *) items->data), "style"));
+	gchar *style = g_strdup (SP_OBJECT_REPR ((SPItem *) items->data)->attribute("style"));
 
 	GString *dstring = g_string_new("");
 	for (GSList *i = items; i != NULL; i = i->next) {
@@ -106,7 +109,7 @@ sp_selected_path_combine (void)
 		g_free (str);
 
 		// if this is the bottommost object,
-		if (!strcmp (sp_repr_attr (SP_OBJECT_REPR (path), "id"), id)) {
+		if (!strcmp (SP_OBJECT_REPR (path)->attribute("id"), id)) {
 			// delete it so that its clones don't get alerted; this object will be restored shortly, with the same id
 			SP_OBJECT (path)->deleteObject(false);
 		} else {
@@ -119,7 +122,7 @@ sp_selected_path_combine (void)
 
 	g_slist_free (items);
 
-	SPRepr *repr = sp_repr_new ("svg:path");
+	Inkscape::XML::Node *repr = sp_repr_new ("svg:path");
 
 	// restore id
 	sp_repr_set_attr (repr, "id", id);
@@ -131,14 +134,14 @@ sp_selected_path_combine (void)
 	g_string_free (dstring, TRUE);
 
 	// add the new group to the group members' common parent
-	sp_repr_append_child (parent, repr);
+	parent->appendChild(repr);
 
 	// move to the position of the topmost, reduced by the number of deleted items
-	sp_repr_set_position_absolute (repr, topmost > 0 ? topmost + 1 : 0);
+	repr->setPosition(topmost > 0 ? topmost + 1 : 0);
 
 	sp_document_done (SP_DT_DOCUMENT (desktop));
 
-	selection->setRepr(repr);
+	selection->set(repr);
 
 	sp_repr_unref (repr);
 }
@@ -150,7 +153,7 @@ sp_selected_path_break_apart (void)
 	if (!SP_IS_DESKTOP(desktop))
 	  return;
 	
-	SPSelection *selection = SP_DT_SELECTION (desktop);
+	Inkscape::Selection *selection = SP_DT_SELECTION (desktop);
 
 	if (selection->isEmpty()) {
 	    desktop->messageStack()->flash(Inkscape::WARNING_MESSAGE, _("Select <b>path(s)</b> to break apart."));
@@ -176,11 +179,11 @@ sp_selected_path_break_apart (void)
 
 		did = true;
 
-		SPRepr *parent = SP_OBJECT_REPR (item)->parent();
-		gint pos = sp_repr_position (SP_OBJECT_REPR (item));
-		const char *id = sp_repr_attr (SP_OBJECT_REPR (item), "id");
+		Inkscape::XML::Node *parent = SP_OBJECT_REPR (item)->parent();
+		gint pos = SP_OBJECT_REPR (item)->position();
+		const char *id = SP_OBJECT_REPR (item)->attribute("id");
 
-		gchar *style = g_strdup (sp_repr_attr (SP_OBJECT (item)->repr, "style"));
+		gchar *style = g_strdup (SP_OBJECT (item)->repr->attribute("style"));
 
 		NArtBpath *abp = nr_artpath_affine (curve->bpath, sp_item_i2root_affine (SP_ITEM (path)));
 
@@ -199,7 +202,7 @@ sp_selected_path_break_apart (void)
 		for (GSList *l = g_slist_reverse(list); l != NULL; l = l->next) {
 			curve = (SPCurve *) l->data;
 
-			SPRepr *repr = sp_repr_new ("svg:path");
+			Inkscape::XML::Node *repr = sp_repr_new ("svg:path");
 			sp_repr_set_attr (repr, "style", style);
 
 			gchar *str = sp_svg_write_path (curve->bpath);
@@ -207,16 +210,16 @@ sp_selected_path_break_apart (void)
 			g_free (str);
 
 			// add the new repr to the parent
-			sp_repr_append_child (parent, repr);
+			parent->appendChild(repr);
 
 			// move to the saved position 
-			sp_repr_set_position_absolute (repr, pos > 0 ? pos : 0);
+			repr->setPosition(pos > 0 ? pos : 0);
 
 			// if it's the first one, restore id
 			if (l == list)
 				sp_repr_set_attr (repr, "id", id);
 
-			selection->addRepr(repr);
+			selection->add(repr);
 
 			sp_repr_unref (repr);
 		}
@@ -248,7 +251,7 @@ sp_selected_path_to_curves0 (gboolean interactive, guint32 text_grouping_policy)
 	if (!SP_IS_DESKTOP(desktop))
 	  return;
 
-	SPSelection *selection = SP_DT_SELECTION (desktop);
+	Inkscape::Selection *selection = SP_DT_SELECTION (desktop);
 
 	if (selection->isEmpty()) {
 		if (interactive)
@@ -264,20 +267,20 @@ sp_selected_path_to_curves0 (gboolean interactive, guint32 text_grouping_policy)
 
 		SPItem *item = SP_ITEM (items->data);
 
-		SPRepr *repr = sp_selected_item_to_curved_repr (item, 0);
+		Inkscape::XML::Node *repr = sp_selected_item_to_curved_repr (item, 0);
 		if (!repr)
 			continue;
 		
 		did = true;
 
 		// remember the position of the item
-		gint pos = sp_repr_position (SP_OBJECT_REPR (item));
+		gint pos = SP_OBJECT_REPR (item)->position();
 		// remember parent
-		SPRepr *parent = SP_OBJECT_REPR (item)->parent();
+		Inkscape::XML::Node *parent = SP_OBJECT_REPR (item)->parent();
 		// remember id
-		const char *id = sp_repr_attr (SP_OBJECT_REPR (item), "id");
+		const char *id = SP_OBJECT_REPR (item)->attribute("id");
 
-		selection->removeItem (item);
+		selection->remove(item);
 
 		// it's going to resurrect, so we delete without advertisement
 		SP_OBJECT (item)->deleteObject(false);
@@ -285,11 +288,11 @@ sp_selected_path_to_curves0 (gboolean interactive, guint32 text_grouping_policy)
 		// restore id
 		sp_repr_set_attr (repr, "id", id);
 		// add the new repr to the parent
-		sp_repr_append_child (parent, repr);
+		parent->appendChild(repr);
 		// move to the saved position 
-		sp_repr_set_position_absolute (repr, pos > 0 ? pos : 0);
+		repr->setPosition(pos > 0 ? pos : 0);
 
-		selection->addRepr(repr);
+		selection->add(repr);
 		sp_repr_unref(repr);
 	}
 
@@ -303,7 +306,7 @@ sp_selected_path_to_curves0 (gboolean interactive, guint32 text_grouping_policy)
 	}
 }
 
-static SPRepr *
+static Inkscape::XML::Node *
 sp_selected_item_to_curved_repr(SPItem * item, guint32 text_grouping_policy)
 {
 	if (!item)
@@ -312,17 +315,17 @@ sp_selected_item_to_curved_repr(SPItem * item, guint32 text_grouping_policy)
 	SPCurve *curve = NULL;
 	if (SP_IS_SHAPE (item)) {
 		curve = sp_shape_get_curve (SP_SHAPE (item));
-	} else if (SP_IS_TEXT (item)) {
-		curve = sp_text_normalized_bpath (SP_TEXT (item));
+	} else if (SP_IS_TEXT (item) || SP_IS_FLOWTEXT (item)) {
+		curve = te_get_layout(item)->convertToCurves ();
 	}
 	
 	if (!curve)
 	  return NULL;
 	
-	SPRepr *repr = sp_repr_new ("svg:path");
+	Inkscape::XML::Node *repr = sp_repr_new ("svg:path");
 	/* Transformation */
 	sp_repr_set_attr (repr, "transform", 
-			  sp_repr_attr (SP_OBJECT_REPR (item), "transform"));
+			  SP_OBJECT_REPR (item)->attribute("transform"));
 	/* Style */
 	gchar *style_str = sp_style_write_difference (SP_OBJECT_STYLE (item), 
 					       SP_OBJECT_STYLE (SP_OBJECT_PARENT (item)));
@@ -337,56 +340,6 @@ sp_selected_item_to_curved_repr(SPItem * item, guint32 text_grouping_policy)
 	return repr;
 }
 
-void
-sp_path_cleanup (SPPath *path)
-{
-	if (strcmp (sp_repr_name (SP_OBJECT_REPR (path)), "svg:path"))
-	  return;
-
-	SPStyle *style = SP_OBJECT_STYLE (path);
-	if (style->fill.type == SP_PAINT_TYPE_NONE)
-	  return;
-
-	SPCurve *curve = sp_shape_get_curve (SP_SHAPE (path));
-	if (!curve)
-	  return;
-	
-	GSList *c = sp_curve_split (curve);
-	sp_curve_unref (curve);
-
-	gboolean dropped = FALSE;
-	GSList *curves = NULL;
-	while (c) {
-		curve = (SPCurve *) c->data;
-		if (curve->closed) {
-			curves = g_slist_prepend (curves, curve);
-		} else {
-			dropped = TRUE;
-			sp_curve_unref (curve);
-		}
-		c = g_slist_remove (c, c->data);
-	}
-	curves = g_slist_reverse (curves);
-
-	curve = sp_curve_concat (curves);
-
-	while (curves) {
-		sp_curve_unref ((SPCurve *) curves->data);
-		curves = g_slist_remove (curves, curves->data);
-	}
-
-	if (sp_curve_is_empty (curve)) {
-		SP_OBJECT (path)->deleteObject();
-	} else if (dropped) {
-		gchar *svgpath;
-		svgpath = sp_svg_write_path (curve->bpath);
-		sp_repr_set_attr (SP_OBJECT_REPR (path), "d", svgpath);
-		g_free (svgpath);
-	}
-
-	sp_curve_unref (curve);
-}
-
 void
 sp_selected_path_reverse ()
 {
@@ -394,7 +347,7 @@ sp_selected_path_reverse ()
 	if (!SP_IS_DESKTOP(desktop))
 	  return;
 	
-	SPSelection *selection = SP_DT_SELECTION (desktop);
+	Inkscape::Selection *selection = SP_DT_SELECTION (desktop);
 	GSList *items = (GSList *) selection->itemList();
 
 	if (g_slist_length (items) == 0) {
