@@ -1,8 +1,9 @@
 /** \file Pen event context. */
 
 /*
- * Initial author:
+ * Authors:
  *   Lauris Kaplinski <lauris@kaplinski.com>
+ *   bulia byak <buliabyak@users.sf.net>
  *
  * Copyright (C) 2000 Lauris Kaplinski
  * Copyright (C) 2000-2001 Ximian, Inc.
@@ -18,16 +19,19 @@
 #include "desktop.h"
 #include "desktop-affine.h"
 #include "desktop-handles.h"
+#include "selection.h"
 #include "draw-anchor.h"
 #include "draw-context.h"
 #include "prefs-utils.h"
 #include "sp-item.h"
+#include "sp-path.h"
 #include "display/canvas-bpath.h"
 #include "display/sp-canvas.h"
 #include "display/sp-ctrlline.h"
 #include "display/sodipodi-ctrl.h"
 #include <glibmm/i18n.h>
 #include "libnr/n-art-bpath.h"
+#include "snap.h"
 
 static void sp_pen_context_class_init(SPPenContextClass *klass);
 static void sp_pen_context_init(SPPenContext *pc);
@@ -200,9 +204,10 @@ sp_pen_context_set(SPEventContext *ec, gchar const *key, gchar const *val)
 static void
 spdc_endpoint_snap(SPPenContext const *const pc, NR::Point &p, guint const state)
 {
-    if (pc->npoints != 0) {
-        spdc_endpoint_snap_internal(pc, p, pc->p[0], state);
-    }
+    spdc_endpoint_snap_internal(pc, p, ( pc->npoints != 0
+                                         ? pc->p[0]
+                                         : p ),
+                                state);
 }
 
 /** Snaps new node's handle relative to the new node. */
@@ -240,7 +245,7 @@ sp_pen_context_root_handler(SPEventContext *ec, GdkEvent *event)
             break;
 
         case GDK_KEY_PRESS:
-            ret = pen_handle_key_press(pc, event->key.keyval);
+            ret = pen_handle_key_press(pc, get_group0_keyval (&event->key));
             break;
 
         default:
@@ -314,10 +319,27 @@ pen_handle_button_press(SPPenContext *const pc, GdkEventButton const &bevent)
                             if (anchor) {
                                 /* Adjust point to anchor if needed */
                                 p = anchor->dp;
+                                SP_EVENT_CONTEXT_DESKTOP(dc)->messageStack()->flash(Inkscape::NORMAL_MESSAGE, _("Continuing selected path"));
                             } else {
+
+                                // This is the first click of a new curve; deselect item so that
+                                // this curve is not combined with it (unless it is drawn from its
+                                // anchor, which is handled by the sibling branch above)
+                                if (!(bevent.state & GDK_SHIFT_MASK)) {
+                                    SP_DT_SELECTION(desktop)->clear();
+                                    SP_EVENT_CONTEXT_DESKTOP(dc)->messageStack()->flash(Inkscape::NORMAL_MESSAGE, _("Creating new path"));
+                                } else if (SP_DT_SELECTION(desktop)->singleItem() && SP_IS_PATH(SP_DT_SELECTION(desktop)->singleItem())) {
+                                    SP_EVENT_CONTEXT_DESKTOP(dc)->messageStack()->flash(Inkscape::NORMAL_MESSAGE, _("Appending to selected path"));
+                                }
+
                                 /* Create green anchor */
                                 p = event_dt;
-                                spdc_endpoint_snap(pc, p, bevent.state);
+                                /* This is the first point, so just snap it to the grid as there's
+                                ** no other points to go off.
+                                */
+                                if (!(bevent.state & GDK_SHIFT_MASK)) {
+                                    namedview_free_snap_all_types(pc->desktop->namedview, p);
+                                }
                                 pc->green_anchor = sp_draw_anchor_new(pc, pc->green_curve, TRUE, p);
                             }
                             spdc_pen_set_initial_point(pc, p);
@@ -325,17 +347,22 @@ pen_handle_button_press(SPPenContext *const pc, GdkEventButton const &bevent)
                             /* Set end anchor */
                             pc->ea = anchor;
                             NR::Point p;
-                            if (anchor) {   /* Snap node only if not hitting anchor. */
+                            if (anchor) {   
                                 p = anchor->dp;
-                            } else {
-                                p = event_dt;
-                                spdc_endpoint_snap(pc, p, bevent.state);
-                            }
-                            spdc_pen_set_subsequent_point(pc, p);
-                            if ( pc->green_anchor && pc->green_anchor->active ) {
+                                // we hit an anchor, will finish the curve (either with or without closing) in release handler
                                 pc->state = SP_PEN_CONTEXT_CLOSE;
+
+                                if ( pc->green_anchor && pc->green_anchor->active ) {
+                                    // we clicked on the current curve start, so close it even if we drag a handle away from it
+                                    dc->green_closed = TRUE; 
+                                }
+
                                 ret = TRUE;
                                 break;
+                            } else { 
+                                p = event_dt;
+                                spdc_endpoint_snap(pc, p, bevent.state); /* Snap node only if not hitting anchor. */
+                                spdc_pen_set_subsequent_point(pc, p);
                             }
                         }
                         pc->state = SP_PEN_CONTEXT_CONTROL;
@@ -470,6 +497,9 @@ pen_handle_button_release(SPPenContext *const pc, GdkEventButton const &revent)
 {
     gint ret = FALSE;
     if ( revent.button == 1 ) {
+
+        SPDrawContext *dc = SP_DRAW_CONTEXT (pc);
+
         NR::Point const event_w(revent.x,
                                 revent.y);
         /* Find desktop coordinates */
@@ -488,7 +518,6 @@ pen_handle_button_release(SPPenContext *const pc, GdkEventButton const &revent)
                                 p = anchor->dp;
                             }
                             pc->sa = anchor;
-                            spdc_endpoint_snap(pc, p, revent.state);
                             spdc_pen_set_initial_point(pc, p);
                         } else {
                             /* Set end anchor here */
@@ -536,7 +565,13 @@ pen_handle_button_release(SPPenContext *const pc, GdkEventButton const &revent)
                     case SP_PEN_CONTEXT_CLOSE:
                         spdc_endpoint_snap(pc, p, revent.state);
                         spdc_pen_finish_segment(pc, p, revent.state);
-                        spdc_pen_finish(pc, TRUE);
+                        if (pc->green_closed) {
+                            // finishing at the start anchor, close curve
+                            spdc_pen_finish(pc, TRUE);
+                        } else {
+                            // finishing at some other anchor, finish curve but not close
+                            spdc_pen_finish(pc, FALSE);
+                        }
                         break;
                     case SP_PEN_CONTEXT_STOP:
                         /* This is allowed, if we just cancelled curve */
@@ -558,7 +593,10 @@ pen_handle_button_release(SPPenContext *const pc, GdkEventButton const &revent)
         }
 
         ret = TRUE;
+
+        dc->green_closed = FALSE;
     }
+
     return ret;
 }
 
@@ -587,13 +625,16 @@ pen_handle_key_press(SPPenContext *const pc, guint const keyval)
             }
             break;
         case GDK_Escape:
-            pc->state = SP_PEN_CONTEXT_STOP;
-            spdc_reset_colors(pc);
-            sp_canvas_item_hide(pc->c0);
-            sp_canvas_item_hide(pc->c1);
-            sp_canvas_item_hide(pc->cl0);
-            sp_canvas_item_hide(pc->cl1);
-            ret = TRUE;
+            if (pc->npoints != 0) {
+                // if drawing, cancel, otherwise pass it up for deselecting
+                pc->state = SP_PEN_CONTEXT_STOP;
+                spdc_reset_colors(pc);
+                sp_canvas_item_hide(pc->c0);
+                sp_canvas_item_hide(pc->c1);
+                sp_canvas_item_hide(pc->cl0);
+                sp_canvas_item_hide(pc->cl1);
+                ret = TRUE;
+            }
             break;
         case GDK_BackSpace:
             if (sp_curve_is_empty(pc->green_curve)) {
@@ -605,7 +646,6 @@ pen_handle_key_press(SPPenContext *const pc, guint const keyval)
                 sp_canvas_item_hide(pc->cl0);
                 sp_canvas_item_hide(pc->cl1);
                 ret = TRUE;
-                break;
             } else {
                 /* Reset red curve */
                 sp_curve_reset(pc->red_curve);
