@@ -1,8 +1,9 @@
 /** \file Pencil event context. */
 
 /*
- * Initial author:
+ * Authors:
  *   Lauris Kaplinski <lauris@kaplinski.com>
+ *   bulia byak <buliabyak@users.sf.net>
  *
  * Copyright (C) 2000 Lauris Kaplinski
  * Copyright (C) 2000-2001 Ximian, Inc.
@@ -18,10 +19,12 @@
 #include "desktop.h"
 #include "desktop-affine.h"
 #include "desktop-handles.h"
+#include "selection.h"
 #include "draw-anchor.h"
 #include "draw-context.h"
 #include "modifier-fns.h"
 #include "sp-item.h"
+#include "sp-path.h"
 #include "prefs-utils.h"
 #include "snap.h"
 #include "display/bezier-utils.h"
@@ -147,9 +150,7 @@ sp_pencil_context_root_handler(SPEventContext *const ec, GdkEvent *event)
             break;
 
         case GDK_KEY_PRESS:
-            ret = pencil_handle_key_press(pc,
-                                          event->key.keyval,
-                                          event->key.state);
+            ret = pencil_handle_key_press(pc, get_group0_keyval (&event->key), event->key.state);
             break;
 
         default:
@@ -202,8 +203,21 @@ pencil_handle_button_press(SPPencilContext *const pc, GdkEventButton const &beve
                 /* Set first point of sequence */
                 if (anchor) {
                     p = anchor->dp;
-                } else if (!(bevent.state & GDK_SHIFT_MASK)) {
-                    namedview_free_snap_all_types(SP_EVENT_CONTEXT_DESKTOP(pc)->namedview, p);
+                    SP_EVENT_CONTEXT_DESKTOP(dc)->messageStack()->flash(Inkscape::NORMAL_MESSAGE, _("Continuing selected path"));
+                } else {
+
+                    if (!(bevent.state & GDK_SHIFT_MASK)) {
+
+                        // This is the first click of a new curve; deselect item so that
+                        // this curve is not combined with it (unless it is drawn from its
+                        // anchor, which is handled by the sibling branch above)
+                        SP_DT_SELECTION(desktop)->clear();
+                        SP_EVENT_CONTEXT_DESKTOP(dc)->messageStack()->flash(Inkscape::NORMAL_MESSAGE, _("Creating new path"));
+
+                        namedview_free_snap_all_types(SP_EVENT_CONTEXT_DESKTOP(pc)->namedview, p);
+                    } else if (SP_DT_SELECTION(desktop)->singleItem() && SP_IS_PATH(SP_DT_SELECTION(desktop)->singleItem())) {
+                        SP_EVENT_CONTEXT_DESKTOP(dc)->messageStack()->flash(Inkscape::NORMAL_MESSAGE, _("Appending to selected path"));
+                    }
                 }
                 pc->sa = anchor;
                 spdc_set_startpoint(pc, p);
