@@ -45,7 +45,7 @@ static char const preferences_skeleton[] =
 "  <group id=\"tools\"\n"
 "         style=\"fill:none;fill-opacity:0.75;\n"
 "                 stroke:black;stroke-opacity:1;stroke-width:1px;stroke-linejoin:miter;stroke-linecap:butt;\">\n"
-"    <group id=\"shapes\" style=\"fill-rule:evenodd;\" selcue=\"1\">\n"
+"    <group id=\"shapes\" style=\"fill-rule:evenodd;\" selcue=\"1\" gradientdrag=\"1\">\n"
 "      <eventcontext id=\"rect\" style=\"fill:blue;\" usecurrent=\"1\"/>\n"
 "      <eventcontext id=\"arc\" style=\"fill:red;\" end=\"0\" start=\"0\" usecurrent=\"1\"/>\n"
 "      <eventcontext id=\"star\" magnitude=\"5\" style=\"fill:yellow;\" usecurrent=\"1\"/>\n"
@@ -58,12 +58,13 @@ static char const preferences_skeleton[] =
 "    </group>\n"
 "    <eventcontext id=\"calligraphic\" style=\"fill:black;fill-opacity:1;fill-rule:nonzero;stroke:none;\"\n"
 "                       mass=\"0.02\" drag=\"1\" angle=\"30\" width=\"0.15\" thinning=\"0.1\" flatness=\"0.9\"  usecurrent=\"0\"/>\n"
-"    <eventcontext id=\"text\"  usecurrent=\"0\"\n"
+"    <eventcontext id=\"text\"  usecurrent=\"0\" gradientdrag=\"1\"\n"
 "                  style=\"fill:black;fill-opacity:1;stroke:none;font-family:Bitstream Vera Sans;font-style:normal;font-weight:normal;font-size:12px;\" selcue=\"1\"/>\n"
-"    <eventcontext id=\"nodes\" selcue=\"1\"/>\n"
-"    <eventcontext id=\"zoom\" selcue=\"1\"/>\n"
-"    <eventcontext id=\"dropper\" selcue=\"1\"/>\n"
-"    <eventcontext id=\"select\" selcue=\"1\"/>\n"
+"    <eventcontext id=\"nodes\" selcue=\"1\" gradientdrag=\"1\"/>\n"
+"    <eventcontext id=\"gradient\" selcue=\"1\"/>\n"
+"    <eventcontext id=\"zoom\" selcue=\"1\" gradientdrag=\"0\"/>\n"
+"    <eventcontext id=\"dropper\" selcue=\"1\" gradientdrag=\"1\"/>\n"
+"    <eventcontext id=\"select\" selcue=\"1\" gradientdrag=\"0\"/>\n"
 "  </group>\n"
 "  <group id=\"palette\">\n"
 "    <group id=\"dashes\">\n"
@@ -127,7 +128,18 @@ static char const preferences_skeleton[] =
 "    <group id=\"open\" path=\"\"/>\n"
 "    <group id=\"debug\" redirect=\"0\"/>\n"
 "    <group id=\"clonetiler\" />\n"
-"    <group id=\"extension-error\" show-on-startup=\"1\"/>\n"
+"    <group id=\"gridtiler\" />\n"
+"    <group id=\"extension-error\" show-on-startup=\"0\"/>\n"
+"    <group id=\"memory\" />\n"
+"    <group id=\"messages\" />\n"
+"    <group id=\"swatches\" />\n"
+"    <group id=\"iconpreview\" />\n"
+"    <group id=\"aboutextensions\" />\n"
+"    <group id=\"treeeditor\" />\n"
+"    <group id=\"layereditor\" />\n"
+"    <group id=\"extensioneditor\" />\n"
+"    <group id=\"trace\" />\n"
+"    <group id=\"script\" />\n"
 "  </group>\n"
 "  <group id=\"printing\">\n"
 "    <settings id=\"ps\"/>\n"
@@ -154,7 +166,7 @@ static char const preferences_skeleton[] =
 "    <group id=\"dialogsskiptaskbar\" value=\"1\"/>\n"
 "    <group id=\"arenatilescachesize\" value=\"8192\"/>\n"
 "    <group id=\"preservetransform\" value=\"0\"/>\n"
-"    <group id=\"clonecompensation\" value=\"0\"/>\n"
+"    <group id=\"clonecompensation\" value=\"1\"/>\n"
 "    <group id=\"cloneorphans\" value=\"0\"/>\n"
 "    <group id=\"stickyzoom\" value=\"0\"/>\n"
 "    <group id=\"selcue\" value=\"2\"/>\n"
@@ -164,7 +176,7 @@ static char const preferences_skeleton[] =
 "    <group id=\"createbitmap\" minsize=\"250\"/>\n"
 "  </group>\n"
 "\n"
-"  <group id=\"extensions\">"
+"  <group id=\"extensions\" show-effects-menu=\"0\">"
 "  </group>\n"
 "\n"
 "  <group id=\"desktop\""
