@@ -5,10 +5,10 @@
  *
  * Author:
  *   Lauris Kaplinski <lauris@kaplinski.com>
+ *   bulia byak <buliabyak@users.sf.net>
  *
- * Copyright (C) 2000 Lauris Kaplinski
+ * Copyright (C) 2000-2005 authors
  * Copyright (C) 2000-2001 Ximian, Inc.
- * Copyright (C) 2002 Lauris Kaplinski
  *
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
@@ -34,13 +34,13 @@
 #include "desktop-style.h"
 #include "pixmaps/cursor-rect.xpm"
 #include "rect-context.h"
-#include "sp-desktop-widget.h"
+#include "desktop-widget.h"
 #include "sp-metrics.h"
 #include <glibmm/i18n.h>
 #include "object-edit.h"
 #include "knotholder.h"
 #include "xml/repr.h"
-#include "xml/sp-repr-event-vector.h"
+#include "xml/node-event-vector.h"
 #include "prefs-utils.h"
 #include "widgets/spw-utilities.h"
 #include "selcue.h"
@@ -108,9 +108,10 @@ static void sp_rect_context_init(SPRectContext *rect_context)
     event_context->within_tolerance = false;
     event_context->item_to_select = NULL;
 
+    event_context->shape_repr = NULL;
+    event_context->shape_knot_holder = NULL;
+
     rect_context->item = NULL;
-    rect_context->repr = NULL;
-    rect_context->knot_holder = NULL;
 
     rect_context->rx = 0.0;
     rect_context->ry = 0.0;
@@ -123,6 +124,8 @@ static void sp_rect_context_dispose(GObject *object)
     SPRectContext *rc = SP_RECT_CONTEXT(object);
     SPEventContext *ec = SP_EVENT_CONTEXT(object);
 
+    ec->enableGrDrag(false);
+
     rc->sel_changed_connection.disconnect();
     rc->sel_changed_connection.~connection();
 
@@ -131,15 +134,15 @@ static void sp_rect_context_dispose(GObject *object)
         sp_rect_finish(rc);
     }
 
-    if (rc->knot_holder) {
-        sp_knot_holder_destroy(rc->knot_holder);
-        rc->knot_holder = NULL;
+    if (ec->shape_knot_holder) {
+        sp_knot_holder_destroy(ec->shape_knot_holder);
+        ec->shape_knot_holder = NULL;
     }
 
-    if (rc->repr) { // remove old listener
-        sp_repr_remove_listener_by_data(rc->repr, ec);
-        sp_repr_unref(rc->repr);
-        rc->repr = 0;
+    if (ec->shape_repr) { // remove old listener
+        sp_repr_remove_listener_by_data(ec->shape_repr, ec);
+        sp_repr_unref(ec->shape_repr);
+        ec->shape_repr = 0;
     }
 
     if (rc->_message_context) {
@@ -149,31 +152,10 @@ static void sp_rect_context_dispose(GObject *object)
     G_OBJECT_CLASS(parent_class)->dispose(object);
 }
 
-static void shape_event_attr_changed(SPRepr *repr,
-                                     gchar const *name, gchar const *old_value, gchar const *new_value,
-                                     bool const is_interactive, gpointer const data)
-{
-    SPRectContext *rc = SP_RECT_CONTEXT(data);
-    SPEventContext *ec = SP_EVENT_CONTEXT(rc);
-
-    if (rc->knot_holder) {
-        sp_knot_holder_destroy(rc->knot_holder);
-    }
-    rc->knot_holder = NULL;
-
-    SPDesktop *desktop = ec->desktop;
-
-    SPItem *item = SP_DT_SELECTION(desktop)->singleItem();
-
-    if (item) {
-        rc->knot_holder = sp_item_knot_holder(item, desktop);
-    }
-}
-
-static SPReprEventVector shape_repr_events = {
+static Inkscape::XML::NodeEventVector ec_shape_repr_events = {
     NULL, /* child_added */
     NULL, /* child_removed */
-    shape_event_attr_changed,
+    ec_shape_event_attr_changed,
     NULL, /* content_changed */
     NULL  /* order_changed */
 };
@@ -182,31 +164,31 @@ static SPReprEventVector shape_repr_events = {
 \brief  Callback that processes the "changed" signal on the selection;
 destroys old and creates new knotholder
 */
-void sp_rect_context_selection_changed(SPSelection *selection, gpointer data)
+void sp_rect_context_selection_changed(Inkscape::Selection *selection, gpointer data)
 {
     SPRectContext *rc = SP_RECT_CONTEXT(data);
     SPEventContext *ec = SP_EVENT_CONTEXT(rc);
 
-    if (rc->knot_holder) { // destroy knotholder
-        sp_knot_holder_destroy(rc->knot_holder);
-        rc->knot_holder = NULL;
+    if (ec->shape_knot_holder) { // destroy knotholder
+        sp_knot_holder_destroy(ec->shape_knot_holder);
+        ec->shape_knot_holder = NULL;
     }
 
-    if (rc->repr) { // remove old listener
-        sp_repr_remove_listener_by_data(rc->repr, ec);
-        sp_repr_unref(rc->repr);
-        rc->repr = 0;
+    if (ec->shape_repr) { // remove old listener
+        sp_repr_remove_listener_by_data(ec->shape_repr, ec);
+        sp_repr_unref(ec->shape_repr);
+        ec->shape_repr = 0;
     }
 
     SPItem *item = selection->singleItem();
     if (item) {
-        rc->knot_holder = sp_item_knot_holder(item, ec->desktop);
-        SPRepr *repr = SP_OBJECT_REPR(item);
-        if (repr) {
-            rc->repr = repr;
-            sp_repr_ref(repr);
-            sp_repr_add_listener(repr, &shape_repr_events, ec);
-            sp_repr_synthesize_events(repr, &shape_repr_events, ec);
+        ec->shape_knot_holder = sp_item_knot_holder(item, ec->desktop);
+        Inkscape::XML::Node *shape_repr = SP_OBJECT_REPR(item);
+        if (shape_repr) {
+            ec->shape_repr = shape_repr;
+            sp_repr_ref(shape_repr);
+            sp_repr_add_listener(shape_repr, &ec_shape_repr_events, ec);
+            sp_repr_synthesize_events(shape_repr, &ec_shape_repr_events, ec);
         }
     }
 }
@@ -221,13 +203,13 @@ static void sp_rect_context_setup(SPEventContext *ec)
 
     SPItem *item = SP_DT_SELECTION(ec->desktop)->singleItem();
     if (item) {
-        rc->knot_holder = sp_item_knot_holder(item, ec->desktop);
-        SPRepr *repr = SP_OBJECT_REPR(item);
-        if (repr) {
-            rc->repr = repr;
-            sp_repr_ref(repr);
-            sp_repr_add_listener(repr, &shape_repr_events, ec);
-            sp_repr_synthesize_events(repr, &shape_repr_events, ec);
+        ec->shape_knot_holder = sp_item_knot_holder(item, ec->desktop);
+        Inkscape::XML::Node *shape_repr = SP_OBJECT_REPR(item);
+        if (shape_repr) {
+            ec->shape_repr = shape_repr;
+            sp_repr_ref(shape_repr);
+            sp_repr_add_listener(shape_repr, &ec_shape_repr_events, ec);
+            sp_repr_synthesize_events(shape_repr, &ec_shape_repr_events, ec);
         }
     }
 
@@ -243,6 +225,10 @@ static void sp_rect_context_setup(SPEventContext *ec)
         ec->enableSelectionCue();
     }
 
+    if (prefs_get_int_attribute("tools.shapes", "gradientdrag", 0) != 0) {
+        ec->enableGrDrag();
+    }
+
     rc->_message_context = new Inkscape::MessageContext(SP_VIEW(ec->desktop)->messageStack());
 }
 
@@ -278,8 +264,8 @@ static gint sp_rect_context_item_handler(SPEventContext *event_context, SPItem *
             event_context->yp = (gint) event->button.y;
             event_context->within_tolerance = true;
 
-            // remember clicked item, disregarding groups
-            event_context->item_to_select = sp_desktop_item_at_point(desktop, NR::Point(event->button.x, event->button.y), TRUE);
+            // remember clicked item, disregarding groups, honoring Alt
+            event_context->item_to_select = sp_event_context_find_item (desktop, NR::Point(event->button.x, event->button.y), event->button.state, TRUE);
 
             ret = TRUE;
         }
@@ -301,6 +287,8 @@ static gint sp_rect_context_root_handler(SPEventContext *event_context, GdkEvent
     static bool dragging;
 
     SPDesktop *desktop = event_context->desktop;
+    Inkscape::Selection *selection = SP_DT_SELECTION (desktop);
+
     SPRectContext *rc = SP_RECT_CONTEXT(event_context);
 
     event_context->tolerance = prefs_get_int_attribute_limited("options.dragtolerance", "value", 0, 0, 100);
@@ -317,8 +305,8 @@ static gint sp_rect_context_root_handler(SPEventContext *event_context, GdkEvent
             event_context->yp = (gint) button_w[NR::Y];
             event_context->within_tolerance = true;
 
-            // remember clicked item, disregarding groups
-            event_context->item_to_select = sp_desktop_item_at_point(desktop, button_w, TRUE);
+            // remember clicked item, disregarding groups, honoring Alt
+            event_context->item_to_select = sp_event_context_find_item (desktop, button_w, event->button.state, TRUE);
 
             dragging = true;
             /* Position center */
@@ -367,10 +355,14 @@ static gint sp_rect_context_root_handler(SPEventContext *event_context, GdkEvent
                 sp_rect_finish(rc);
             } else if (event_context->item_to_select) {
                 // no dragging, select clicked item if any
-                SP_DT_SELECTION(desktop)->setItem(event_context->item_to_select);
+                if (event->button.state & GDK_SHIFT_MASK) {
+                    selection->toggle(event_context->item_to_select);
+                } else {
+                    selection->set(event_context->item_to_select);
+                }
             } else {
                 // click in an empty space
-                SP_DT_SELECTION(desktop)->clear();
+                selection->clear();
             }
 
             event_context->item_to_select = NULL;
@@ -380,7 +372,7 @@ static gint sp_rect_context_root_handler(SPEventContext *event_context, GdkEvent
         }
         break;
     case GDK_KEY_PRESS:
-        switch (event->key.keyval) {
+        switch (get_group0_keyval (&event->key)) {
         case GDK_Alt_L:
         case GDK_Alt_R:
         case GDK_Control_L:
@@ -422,7 +414,7 @@ static gint sp_rect_context_root_handler(SPEventContext *event_context, GdkEvent
         }
         break;
     case GDK_KEY_RELEASE:
-        switch (event->key.keyval) {
+        switch (get_group0_keyval (&event->key)) {
         case GDK_Alt_L:
         case GDK_Alt_R:
         case GDK_Control_L:
@@ -467,7 +459,7 @@ static void sp_rect_drag(SPRectContext &rc, NR::Point const pt, guint state)
         }
 
         /* Create object */
-        SPRepr *repr = sp_repr_new("svg:rect");
+        Inkscape::XML::Node *repr = sp_repr_new("svg:rect");
 
         /* Set style */
         sp_desktop_apply_style_tool (desktop, repr, "tools.shapes.rect", false);
@@ -555,7 +547,7 @@ static void sp_rect_drag(SPRectContext &rc, NR::Point const pt, guint state)
     // status text
     GString *xs = SP_PX_TO_METRIC_STRING(fabs( x1 - x0 ), sp_desktop_get_default_metric(desktop));
     GString *ys = SP_PX_TO_METRIC_STRING(fabs( y1 - y0 ), sp_desktop_get_default_metric(desktop));
-    rc._message_context->setF(Inkscape::NORMAL_MESSAGE, _("<b>Rectangle</b>: %s x %s; with <b>Ctrl</b> to make square or integer-ratio rectangle; with <b>Shift</b> to draw around the starting point"), xs->str, ys->str);
+    rc._message_context->setF(Inkscape::NORMAL_MESSAGE, _("<b>Rectangle</b>: %s &#215; %s; with <b>Ctrl</b> to make square or integer-ratio rectangle; with <b>Shift</b> to draw around the starting point"), xs->str, ys->str);
     g_string_free(xs, FALSE);
     g_string_free(ys, FALSE);
 }
@@ -571,7 +563,7 @@ static void sp_rect_finish(SPRectContext *rc)
 
         SP_OBJECT(rc->item)->updateRepr();
 
-        SP_DT_SELECTION(dt)->setItem(rc->item);
+        SP_DT_SELECTION(dt)->set(rc->item);
         sp_document_done(SP_DT_DOCUMENT(dt));
 
         rc->item = NULL;
