@@ -12,7 +12,7 @@
  */
 
 
-#include "helper/sodipodi-ctrlrect.h"
+#include "display/sodipodi-ctrlrect.h"
 #include "libnr/nr-matrix-ops.h"
 #include "desktop.h"
 #include "desktop-handles.h"
@@ -21,7 +21,7 @@
 SPDesktop * sp_rb_desktop = NULL;
 SPCtrlRect * sp_rb = NULL;
 gboolean sp_rb_dragging = FALSE;
-ArtDRect sp_rb_rect;
+NRRect sp_rb_rect;
 
 void sp_rubberband_start (SPDesktop * desktop, NR::Point const &p)
 {
