@@ -15,7 +15,9 @@
 #include <glib/gslist.h>
 #include <sigc++/sigc++.h>
 
-#include <forward.h>
+#include "forward.h"
+
+namespace Inkscape { class Selection; }
 
 enum {
         SP_SELCUE_NONE,
@@ -28,10 +30,11 @@ struct SPSelCue {
 	~SPSelCue();
 
 	SPDesktop *desktop;
-	SPSelection *selection;
+	Inkscape::Selection *selection;
 	sigc::connection sel_changed_connection;
 	sigc::connection sel_modified_connection;
 	GSList *item_bboxes;
+	GSList *text_baselines;
 };
 
 void sp_sel_cue_update_item_bboxes (SPSelCue * selcue);
