@@ -5,13 +5,16 @@
  *
  * Author:
  *   Lauris Kaplinski <lauris@kaplinski.com>
- *
- * Copyright (C) 1999-2002 Lauris Kaplinski
+ *   bulia byak <buliabyak@users.sf.net>
+  *
+ * Copyright (C) 1999-2005 authors
  *
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
 #include <math.h>
 #include <string.h>
 #include <glib.h>
@@ -26,7 +29,7 @@
 #include "desktop-affine.h"
 #include "seltrans-handles.h"
 #include "sp-cursor.h"
-#include "sp-desktop-widget.h"
+#include "desktop-widget.h"
 #include "pixmaps/cursor-select-m.xpm"
 #include "pixmaps/cursor-select-d.xpm"
 #include "pixmaps/handles.xpm"
@@ -41,8 +44,11 @@
 #include "dialogs/object-properties.h"
 #include "sp-metrics.h"
 #include "sp-item.h"
+#include "sp-item-group.h"
+#include "sp-root.h"
 #include "snap.h"
 #include "prefs-utils.h"
+#include "tools-switch.h"
 #include "message-context.h"
 #include "message-stack.h"
 
@@ -141,6 +147,9 @@ static void
 sp_select_context_dispose(GObject *object)
 {
     SPSelectContext *sc = SP_SELECT_CONTEXT(object);
+    SPEventContext * ec = SP_EVENT_CONTEXT (object);
+
+    ec->enableGrDrag(false);
 
     if (sc->grabbed) {
         sp_canvas_item_ungrab(sc->grabbed, GDK_CURRENT_TIME);
@@ -152,6 +161,15 @@ sp_select_context_dispose(GObject *object)
     delete sc->_describer;
     sc->_describer = NULL;
 
+    if (CursorSelectDragging) {
+        gdk_cursor_unref (CursorSelectDragging);
+        CursorSelectDragging = NULL;
+    }
+    if (CursorSelectMouseover) {
+        gdk_cursor_unref (CursorSelectMouseover);
+        CursorSelectMouseover = NULL;
+    }
+
     G_OBJECT_CLASS(parent_class)->dispose(object);
 }
 
@@ -172,6 +190,10 @@ sp_select_context_setup(SPEventContext *ec)
 
     sp_event_context_read(ec, "show");
     sp_event_context_read(ec, "transform");
+
+    if (prefs_get_int_attribute("tools.select", "gradientdrag", 0) != 0) {
+        ec->enableGrDrag();
+    }
 }
 
 static void
@@ -188,34 +210,12 @@ sp_select_context_set(SPEventContext *ec, gchar const *key, gchar const *val)
     } 
 }
 
-/**
-Returns item at point p in desktop; if state includes alt key mask, cyclically selects under; honors into_groups
-*/
-SPItem *
-sp_select_context_find_item (SPDesktop *desktop, NR::Point const p, int state, gboolean into_groups)
-{
-    SPItem *item;
-
-    if (state & GDK_MOD1_MASK) { // select under
-        SPItem *selected_at_point = sp_desktop_item_from_list_at_point_bottom (desktop,
-                                                                               SP_DT_SELECTION(desktop)->itemList(), p);
-        item = sp_desktop_item_at_point(desktop, p, into_groups, selected_at_point);
-        if (item == NULL) { // we may have reached bottom, flip over to the top
-            item = sp_desktop_item_at_point(desktop, p, into_groups, NULL);
-        }
-    } else 
-        item = sp_desktop_item_at_point(desktop, p, into_groups, NULL);
-
-    return item;
-}
-
-static void
+static bool
 sp_select_context_abort(SPEventContext *event_context)
 {
     SPDesktop *desktop = event_context->desktop;
     SPSelectContext *sc = SP_SELECT_CONTEXT(event_context);
     SPSelTrans *seltrans = sc->_seltrans;
-    SPSelection *selection = SP_DT_SELECTION(desktop);
 
     if (sc->dragging) {
         if (sc->moved) { // cancel dragging an object
@@ -240,6 +240,7 @@ sp_select_context_abort(SPEventContext *event_context)
             sc->item = NULL;
 
             SP_EVENT_CONTEXT(sc)->desktop->messageStack()->flash(Inkscape::NORMAL_MESSAGE, _("Move canceled."));
+            return true;
         }
     } else {
         NRRect b;
@@ -248,10 +249,22 @@ sp_select_context_abort(SPEventContext *event_context)
             sp_rubberband_stop();
             rb_escaped = 1;
             SP_EVENT_CONTEXT(sc)->desktop->messageStack()->flash(Inkscape::NORMAL_MESSAGE, _("Selection canceled."));
-        } else {
-            selection->clear();
-        }
+            return true;
+        } 
     }
+    return false;
+}
+
+bool 
+key_is_a_modifier (guint key) {
+    return (key == GDK_Alt_L ||
+                key == GDK_Alt_R ||
+                key == GDK_Control_L ||
+                key == GDK_Control_R ||
+                key == GDK_Shift_L ||
+                key == GDK_Shift_R ||
+                key == GDK_Meta_L ||  // Meta is when you press Shift+Alt (at least on my machine)
+            key == GDK_Meta_R);
 }
 
 static gint
@@ -262,7 +275,6 @@ sp_select_context_item_handler(SPEventContext *event_context, SPItem *item, GdkE
     SPDesktop *desktop = event_context->desktop;
     SPSelectContext *sc = SP_SELECT_CONTEXT(event_context);
     SPSelTrans *seltrans = sc->_seltrans;
-    SPSelection *selection = SP_DT_SELECTION(desktop);
 
     tolerance = prefs_get_int_attribute_limited("options.dragtolerance", "value", 0, 0, 100);
 
@@ -272,11 +284,6 @@ sp_select_context_item_handler(SPEventContext *event_context, SPItem *item, GdkE
     }
 
     switch (event->type) {
-        case GDK_2BUTTON_PRESS:
-            if (event->button.button == 1) {
-                ret = TRUE;
-            }
-            break;
         case GDK_BUTTON_PRESS:
             if (event->button.button == 1) {
                 /* Left mousebutton */
@@ -299,116 +306,25 @@ sp_select_context_item_handler(SPEventContext *event_context, SPItem *item, GdkE
                     sc->moved = FALSE;
 
                     // remember the clicked item in sc->item:
-                    if (event->button.state & GDK_MOD1_MASK) { // alt-click
-                        sc->item = sp_select_context_find_item (desktop, NR::Point(event->button.x, event->button.y), event->button.state, FALSE);
-                    } else { // simple click
-                        sc->item = item;
-                    }
+                    sc->item = sp_event_context_find_item (desktop, 
+                                              NR::Point(event->button.x, event->button.y), event->button.state, FALSE);
                     sp_object_ref(sc->item, NULL);
 
                     rb_escaped = drag_escaped = 0;
 
                     sp_canvas_item_grab(SP_CANVAS_ITEM(desktop->drawing),
-                                        GDK_KEY_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
+                                        GDK_KEY_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_BUTTON_PRESS_MASK |
                                         GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK,
                                         NULL, event->button.time);
                     sc->grabbed = SP_CANVAS_ITEM(desktop->drawing);
 
                     ret = TRUE;
                 }
+            } else if (event->button.button == 3) {
+                // right click; do not eat it so that right-click menu can appear, but cancel dragging & rubberband
+                sp_select_context_abort(event_context);
             }
             break;
-        case GDK_MOTION_NOTIFY:
-            if (event->motion.state & GDK_BUTTON1_MASK) {
-                /* Left mousebutton */
-                if (sc->dragging) {
-                    ret = TRUE;
-
-                    if ( within_tolerance
-                         && ( abs( (gint) event->motion.x - xp ) < tolerance )
-                         && ( abs( (gint) event->motion.y - yp ) < tolerance ) ) {
-                        break; // do not drag if we're within tolerance from origin
-                    }
-                    // Once the user has moved farther than tolerance from the original location
-                    // (indicating they intend to move the object, not click), then always process the
-                    // motion notify coordinates as given (no snapping back to origin)
-                    within_tolerance = false;
-
-                    NR::Point const motion_pt(event->motion.x, event->motion.y);
-                    NR::Point const p(sp_desktop_w2d_xy_point(desktop, motion_pt));
-                    if (!sc->moved) {
-                        NR::Point const button_pt(event->button.x, event->button.y);
-                        SPItem *item_at_point = sp_desktop_item_at_point(desktop, button_pt, TRUE);
-                        SPItem *group_at_point = sp_desktop_group_at_point(desktop, button_pt);
-                        // if neither a group nor an item (possibly in a group) at point are selected, set selection to the item passed with the event
-                        if ( ( !item_at_point || !selection->includesItem(item_at_point) )
-                             && ( !group_at_point || !selection->includesItem(group_at_point) ) 
-                             && !sc->button_press_alt) {
-                            // select what is under cursor
-                            if (!seltrans->empty)
-                                sp_sel_trans_reset_state(seltrans);
-                            if (!selection->includesItem(sc->item)) {
-                                selection->setItem(sc->item);
-                            }
-                        } // otherwise, do not change selection so that dragging selected-within-group items, as well as alt-dragging, is possible
-                        sp_sel_trans_grab(seltrans, p, -1, -1, FALSE);
-                        sc->moved = TRUE;
-                    }
-                    if (!seltrans->empty)
-                        sp_selection_moveto(seltrans, p, event->button.state);
-                    if (sp_desktop_scroll_to_point(desktop, &p)) {
-                        // unfortunately in complex drawings, gobbling results in losing grab of the object, for some mysterious reason
-                        ; //gobble_motion_events (GDK_BUTTON1_MASK);
-                    }
-                }
-            }
-            break;
-        case GDK_BUTTON_RELEASE:
-            xp = yp = 0;
-            if (event->button.button == 1) {
-                seltrans->messageContext().clear();
-                if (sc->moved) {
-                    // item has been moved
-                    sp_sel_trans_ungrab(seltrans);
-                    sc->moved = FALSE;
-                } else if (sc->item && !drag_escaped) {
-                    // item has not been moved -> do selecting
-                    if (!selection->isEmpty()) {
-                        if (event->button.state & GDK_SHIFT_MASK) { // shift-click, with previous selection
-                            sp_sel_trans_reset_state(seltrans);
-                            if (selection->includesItem(sc->item)) {
-                                selection->removeItem(sc->item);
-                            } else {
-                                selection->addItem(sc->item);
-                            }
-                        } else { // simple click, with previous selection
-                            if (selection->includesItem(sc->item)) {
-                                sp_sel_trans_increase_state(seltrans);
-                            } else {
-                                sp_sel_trans_reset_state(seltrans);
-                                selection->setItem(sc->item);
-                            }
-                        }
-                    } else { // simple or shift click, no previous selection
-                        sp_sel_trans_reset_state(seltrans);
-                        selection->setItem(sc->item);
-                    }
-                }
-                sc->dragging = FALSE;
-                if (sc->item) {
-                    sp_object_unref( SP_OBJECT(sc->item), NULL);
-                }
-                sc->item = NULL;
-                if (sc->grabbed) {
-                    sp_canvas_item_ungrab(sc->grabbed, event->button.time);
-                    sc->grabbed = NULL;
-                }
-                ret = TRUE;
-            }
-            sc->button_press_shift = false;
-            sc->button_press_ctrl = false;
-            sc->button_press_alt = false;
-            break;
 
         case GDK_ENTER_NOTIFY:
         {
@@ -423,7 +339,7 @@ sp_select_context_item_handler(SPEventContext *event_context, SPItem *item, GdkE
             break;
 
         case GDK_KEY_PRESS:
-            if (event->key.keyval == GDK_space) {
+            if (get_group0_keyval (&event->key) == GDK_space) {
                 if (sc->dragging && sc->grabbed) {
                     /* stamping mode: show content mode moving */
                     sp_sel_trans_stamp(seltrans);
@@ -447,7 +363,7 @@ sp_select_context_item_handler(SPEventContext *event_context, SPItem *item, GdkE
 static gint
 sp_select_context_root_handler(SPEventContext *event_context, GdkEvent *event)
 {
-    SPItem *item = NULL, *group = NULL;
+    SPItem *item = NULL;
     SPItem *item_at_point = NULL, *group_at_point = NULL, *item_in_group = NULL;
     gint ret = FALSE;
     NRRect b;
@@ -455,7 +371,7 @@ sp_select_context_root_handler(SPEventContext *event_context, GdkEvent *event)
     SPDesktop *desktop = event_context->desktop;
     SPSelectContext *sc = SP_SELECT_CONTEXT(event_context);
     SPSelTrans *seltrans = sc->_seltrans;
-    SPSelection *selection = SP_DT_SELECTION(desktop);
+    Inkscape::Selection *selection = SP_DT_SELECTION(desktop);
     gdouble const nudge = prefs_get_double_attribute_limited("options.nudgedistance", "value", 2, 0, 1000); // in px
     gdouble const offset = prefs_get_double_attribute_limited("options.defaultscale", "value", 2, 0, 1000);
     tolerance = prefs_get_int_attribute_limited("options.dragtolerance", "value", 0, 0, 100);
@@ -467,6 +383,25 @@ sp_select_context_root_handler(SPEventContext *event_context, GdkEvent *event)
     }
 
     switch (event->type) {
+        case GDK_2BUTTON_PRESS:
+            if (event->button.button == 1) {
+                if (!selection->isEmpty()) {
+                    SPItem *clicked_item = (SPItem *) selection->itemList()->data;
+                    if (SP_IS_GROUP (clicked_item)) { // enter group
+                        desktop->setCurrentLayer(reinterpret_cast<SPObject *>(clicked_item));
+                        SP_DT_SELECTION(desktop)->clear();
+                        sc->dragging = false;
+                    } else { // switch tool
+                        tools_switch_by_item (desktop, clicked_item);
+                    }
+                } else { // click in empty place, go up one level (but not to root)
+                    if (!SP_IS_ROOT(SP_OBJECT_PARENT(desktop->currentLayer()))) {
+                        desktop->setCurrentLayer(SP_OBJECT_PARENT(desktop->currentLayer()));
+                    }
+                }
+                ret = TRUE;
+            }
+            break;
         case GDK_BUTTON_PRESS:
             if (event->button.button == 1) {
 
@@ -493,6 +428,9 @@ sp_select_context_root_handler(SPEventContext *event_context, GdkEvent *event)
                 rb_escaped = drag_escaped = 0;
 
                 ret = TRUE;
+            } else if (event->button.button == 3) {
+                // right click; do not eat it so that right-click menu can appear, but cancel dragging & rubberband
+                sp_select_context_abort(event_context);
             }
             break;
 
@@ -528,15 +466,15 @@ sp_select_context_root_handler(SPEventContext *event_context, GdkEvent *event)
                             item_in_group = sp_desktop_item_at_point(desktop, NR::Point(event->button.x, event->button.y), TRUE);
                             group_at_point = sp_desktop_group_at_point(desktop, NR::Point(event->button.x, event->button.y));
                             // if neither a group nor an item (possibly in a group) at point are selected, set selection to the item at point
-                            if ((!item_in_group || !selection->includesItem(item_in_group)) &&
-                                (!group_at_point || !selection->includesItem(group_at_point))
+                            if ((!item_in_group || !selection->includes(item_in_group)) &&
+                                (!group_at_point || !selection->includes(group_at_point))
                                 && !sc->button_press_alt) {
                                 // select what is under cursor
                                 if (!seltrans->empty)
                                     sp_sel_trans_reset_state(seltrans);
                                 // when simply ctrl-dragging, we don't want to go into groups
-                                if (item_at_point && !selection->includesItem(item_at_point))
-                                    selection->setItem(item_at_point);
+                                if (item_at_point && !selection->includes(item_at_point))
+                                    selection->set(item_at_point);
                             } // otherwise, do not change selection so that dragging selected-within-group items, as well as alt-dragging, is possible
                             sp_sel_trans_grab(seltrans, p, -1, -1, FALSE);
                             sc->moved = TRUE;
@@ -556,7 +494,6 @@ sp_select_context_root_handler(SPEventContext *event_context, GdkEvent *event)
                 }
             }
             break;
-
         case GDK_BUTTON_RELEASE:
             xp = yp = 0;
             if ((event->button.button == 1) && (sc->grabbed)) {
@@ -565,29 +502,25 @@ sp_select_context_root_handler(SPEventContext *event_context, GdkEvent *event)
                         // item has been moved
                         sp_sel_trans_ungrab(seltrans);
                         sc->moved = FALSE;
-                    } else if (!drag_escaped) {
+                    } else if (sc->item && !drag_escaped) {
                         // item has not been moved -> simply a click, do selecting
                         if (!selection->isEmpty()) {
                             if (event->button.state & GDK_SHIFT_MASK) {
                                 // with shift, toggle selection
                                 sp_sel_trans_reset_state(seltrans);
-                                if (selection->includesItem(sc->item)) {
-                                    selection->removeItem(sc->item);
-                                } else {
-                                    selection->addItem(sc->item);
-                                }
+                                selection->toggle(sc->item);
                             } else {
                                 // without shift, increase state (i.e. toggle scale/rotation handles)
-                                if (selection->includesItem(sc->item)) {
+                                if (selection->includes(sc->item)) {
                                     sp_sel_trans_increase_state(seltrans);
                                 } else {
                                     sp_sel_trans_reset_state(seltrans);
-                                    selection->setItem(sc->item);
+                                    selection->set(sc->item);
                                 }
                             }
                         } else { // simple or shift click, no previous selection
                             sp_sel_trans_reset_state(seltrans);
-                            selection->setItem(sc->item);
+                            selection->set(sc->item);
                         }
                     }
                     sc->dragging = FALSE;
@@ -607,7 +540,7 @@ sp_select_context_root_handler(SPEventContext *event_context, GdkEvent *event)
                             selection->addList (items);
                         } else {
                             // without shift, simply select anew
-                            selection->setItemList (items);
+                            selection->setList (items);
                         }
                         g_slist_free (items);
                     } else { // it was just a click, or a too small rubberband
@@ -618,32 +551,18 @@ sp_select_context_root_handler(SPEventContext *event_context, GdkEvent *event)
                             sc->button_press_shift = false;
 
                             if (sc->button_press_ctrl) {
-                                // go into groups
-                                item = sp_select_context_find_item (desktop, NR::Point(event->button.x, event->button.y), event->button.state, TRUE);
-                                group = sp_desktop_group_at_point(desktop, NR::Point(event->button.x, event->button.y));
+                                // go into groups, honoring Alt
+                                item = sp_event_context_find_item (desktop, 
+                                                   NR::Point(event->button.x, event->button.y), event->button.state, TRUE);
                                 sc->button_press_ctrl = FALSE;
                             } else {
-                                // don't go into groups
-                                item = sp_select_context_find_item (desktop, NR::Point(event->button.x, event->button.y), event->button.state, FALSE);
+                                // don't go into groups, honoring Alt
+                                item = sp_event_context_find_item (desktop, 
+                                                   NR::Point(event->button.x, event->button.y), event->button.state, FALSE);
                             }
 
-                            // if there's both a group and an item at point, deselect group to
-                            // prevent double selection. FIXME: this only prevents double select for
-                            // top-level groups and bottom-level objects, but fails if a mid-level
-                            // group is selected without entering the top-level as a layer (though
-                            // this currently only possible by XML editor)
-                            if (group) {
-                                if (selection->includesItem(group)) {
-                                    selection->removeItem(group);
-                                }
-                            }
                             if (item) {
-                                // toggle selected status
-                                if (selection->includesItem(item)) {
-                                    selection->removeItem(item);
-                                } else {
-                                    selection->addItem(item);
-                                }
+                                selection->toggle(item);
                                 item = NULL;
                             }
 
@@ -651,21 +570,22 @@ sp_select_context_root_handler(SPEventContext *event_context, GdkEvent *event)
 
                             sc->button_press_ctrl = FALSE;
 
-                            item = sp_select_context_find_item (desktop, NR::Point(event->button.x, event->button.y), event->button.state, TRUE);
+                            item = sp_event_context_find_item (desktop, 
+                                         NR::Point(event->button.x, event->button.y), event->button.state, TRUE);
 
                             if (item) {
-                                if (selection->includesItem(item)) {
+                                if (selection->includes(item)) {
                                     sp_sel_trans_increase_state(seltrans);
                                 } else {
                                     sp_sel_trans_reset_state(seltrans);
-                                    selection->setItem(item);
+                                    selection->set(item);
                                 }
                                 item = NULL;
                             }
 
-                        } else { // click without shift, simply deselect
+                        } else { // click without shift, simply deselect, unless with Alt or something was cancelled
                             if (!selection->isEmpty()) {
-                                if (!(rb_escaped) && !(drag_escaped)) // unless something was cancelled
+                                if (!(rb_escaped) && !(drag_escaped) && !(event->button.state & GDK_MOD1_MASK))
                                     selection->clear();
                                 rb_escaped = 0;
                                 ret = TRUE;
@@ -685,20 +605,18 @@ sp_select_context_root_handler(SPEventContext *event_context, GdkEvent *event)
             break;
 
         case GDK_KEY_PRESS: // keybindings for select context
-            switch (event->key.keyval) {
-                case GDK_Alt_L:
-                case GDK_Alt_R:
-                case GDK_Control_L:
-                case GDK_Control_R:
-                case GDK_Shift_L:
-                case GDK_Shift_R:
-                case GDK_Meta_L:  // Meta is when you press Shift+Alt (at least on my machine)
-                case GDK_Meta_R:
+
+            if (!key_is_a_modifier (get_group0_keyval (&event->key))) {
+                    event_context->defaultMessageContext()->clear();
+            } else {
                     sp_event_show_modifier_tip (event_context->defaultMessageContext(), event,
                                                 _("<b>Ctrl</b>: select in groups, move hor/vert"),
                                                 _("<b>Shift</b>: toggle select, force rubberband, disable snapping"),
                                                 _("<b>Alt</b>: select under, move selected"));
                     break;
+            }
+
+            switch (get_group0_keyval (&event->key)) {
                 case GDK_Left: // move selection left
                 case GDK_KP_Left:
                 case GDK_KP_4:
@@ -760,7 +678,8 @@ sp_select_context_root_handler(SPEventContext *event_context, GdkEvent *event)
                     }
                     break;
                 case GDK_Escape:
-                    sp_select_context_abort(event_context);
+                    if (!sp_select_context_abort(event_context))
+                        selection->clear();
                     ret = TRUE;
                     break;
                 case GDK_a:
@@ -847,20 +766,9 @@ sp_select_context_root_handler(SPEventContext *event_context, GdkEvent *event)
             }
             break;
         case GDK_KEY_RELEASE:
-            switch (event->key.keyval) {
-                case GDK_Alt_L:
-                case GDK_Alt_R:
-                case GDK_Control_L:
-                case GDK_Control_R:
-                case GDK_Shift_L:
-                case GDK_Shift_R:
-                case GDK_Meta_L:  // Meta is when you press Shift+Alt
-                case GDK_Meta_R:
-                    event_context->defaultMessageContext()->clear();
-                    break;
-                default:
-                    break;
-            }
+            if (key_is_a_modifier (get_group0_keyval (&event->key)))
+                event_context->defaultMessageContext()->clear();
+            break;
         default:
             break;
     }
