@@ -3,14 +3,16 @@
  *
  * Authors:
  *   Lauris Kaplinski <lauris@kaplinski.com>
- *   bulia byak <bulia@dr.com>
+ *   bulia byak <buliabyak@users.sf.net>
  *
- * Copyright (C) 2003 authors
+ * Copyright (C) 2003-2005 authors
  *
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
 
 #include <string.h>
 #include <glib.h>
@@ -61,7 +63,7 @@
 #include "prefs-utils.h"
 #include "selection.h"
 #include "selection-chemistry.h"
-#include "sp-desktop-widget.h"
+#include "desktop-widget.h"
 #include "sp-item-transform.h"
 #include "libnr/nr-matrix.h"
 #include "libnr/nr-translate-scale-ops.h"
@@ -70,7 +72,7 @@
 #include "select-toolbar.h"
 
 static void
-sp_selection_layout_widget_update(SPWidget *spw, SPSelection *sel)
+sp_selection_layout_widget_update(SPWidget *spw, Inkscape::Selection *sel)
 {
     if (gtk_object_get_data(GTK_OBJECT(spw), "update")) {
         return;
@@ -122,7 +124,7 @@ sp_selection_layout_widget_update(SPWidget *spw, SPSelection *sel)
 
 
 static void
-sp_selection_layout_widget_modify_selection(SPWidget *spw, SPSelection *selection, guint flags, gpointer data)
+sp_selection_layout_widget_modify_selection(SPWidget *spw, Inkscape::Selection *selection, guint flags, gpointer data)
 {
     SPDesktop *desktop = (SPDesktop *) data;
     if ((desktop->selection == selection) // only respond to changes in our desktop
@@ -135,7 +137,7 @@ sp_selection_layout_widget_modify_selection(SPWidget *spw, SPSelection *selectio
 }
 
 static void
-sp_selection_layout_widget_change_selection(SPWidget *spw, SPSelection *selection, gpointer data)
+sp_selection_layout_widget_change_selection(SPWidget *spw, Inkscape::Selection *selection, gpointer data)
 {
     SPDesktop *desktop = (SPDesktop *) data;
     if (desktop->selection == selection) // only respond to changes in our desktop
@@ -160,7 +162,9 @@ sp_object_layout_any_value_changed(GtkAdjustment *adj, SPWidget *spw)
     }
     gtk_object_set_data(GTK_OBJECT(spw), "update", GINT_TO_POINTER(TRUE));
 
-    SPSelection *selection = SP_WIDGET_SELECTION(spw);
+    SPDesktop *desktop = SP_ACTIVE_DESKTOP;
+    Inkscape::Selection *selection = SP_DT_SELECTION(desktop);
+    SPDocument *document = SP_DT_DOCUMENT(desktop);
     NR::Rect bbox = selection->bounds();
 
     if (!((bbox.max()[NR::X] - bbox.min()[NR::X] > 1e-6) || (bbox.max()[NR::Y] - bbox.min()[NR::Y] > 1e-6))) {
@@ -206,15 +210,30 @@ sp_object_layout_any_value_changed(GtkAdjustment *adj, SPWidget *spw)
         }
     }
 
-    char const * const actionkey = ( fabs(x0 - bbox.min()[NR::X]) > 1e-6
-                                     ? "selector:toolbar:move:horizontal"
-                                     : fabs(x1 - bbox.max()[NR::X]) > 1e-6
-                                     ? "selector:toolbar:scale:horizontal"
-                                     : fabs(y0 - bbox.min()[NR::Y]) > 1e-6
-                                     ? "selector:toolbar:move:vertical"
-                                     : fabs(y1 - bbox.max()[NR::Y]) > 1e-6
-                                     ? "selector:toolbar:scale:vertical"
-                                     : NULL );
+    // scales and moves, in px
+    double mh = fabs(x0 - bbox.min()[NR::X]);
+    double sh = fabs(x1 - bbox.max()[NR::X]);
+    double mv = fabs(y0 - bbox.min()[NR::Y]);
+    double sv = fabs(y1 - bbox.max()[NR::Y]);
+
+    // unless the unit is %, convert the scales and moves to the unit
+    if (unit.base == SP_UNIT_ABSOLUTE || unit.base == SP_UNIT_DEVICE) {
+        mh = sp_pixels_get_units (mh, unit);
+        sh = sp_pixels_get_units (sh, unit);
+        mv = sp_pixels_get_units (mv, unit);
+        sv = sp_pixels_get_units (sv, unit);
+    }
+
+    // do the action only if one of the scales/moves is greater than half the last significant
+    // digit in the spinbox (currently spinboxes have 2 fractional digits, so that makes 0.005). If
+    // the value was changed by the user, the difference will be at least that much; otherwise it's
+    // just rounding difference between the spinbox value and actual value, so no action is
+    // performed
+    char const * const actionkey = ( mh > 5e-3 ? "selector:toolbar:move:horizontal" : 
+                                     sh > 5e-3 ? "selector:toolbar:scale:horizontal" : 
+                                     mv > 5e-3 ? "selector:toolbar:move:vertical" : 
+                                     sv > 5e-3 ? "selector:toolbar:scale:vertical" : NULL );
+
     if (actionkey != NULL) {
         NR::translate const p2o(-bbox.min());
         NR::scale scale(1, 1);
@@ -228,7 +247,7 @@ sp_object_layout_any_value_changed(GtkAdjustment *adj, SPWidget *spw)
         NR::translate const o2n(x0, y0);
         sp_selection_apply_affine(selection, p2o * scale * o2n);
 
-        sp_document_maybe_done(SP_WIDGET_DOCUMENT(spw), actionkey);
+        sp_document_maybe_done (document, actionkey);
 
         // defocus spinbuttons by moving focus to the canvas, unless "stay" is on
         spinbutton_defocus(GTK_OBJECT(spw));
@@ -243,7 +262,7 @@ sp_select_toolbox_spinbutton(gchar *label, gchar *data, float lower_limit, GtkWi
     GtkTooltips *tt = gtk_tooltips_new();
 
     GtkWidget *hb = gtk_hbox_new(FALSE, 1);
-    GtkWidget *l = gtk_label_new(_(label));
+    GtkWidget *l = gtk_label_new(Q_(label));
     gtk_tooltips_set_tip(tt, l, tooltip, NULL);
     gtk_widget_show(l);
     gtk_misc_set_alignment(GTK_MISC(l), 1.0, 0.5);
@@ -281,7 +300,7 @@ static gboolean aux_set_unit(SPUnitSelector *,
         return FALSE;
     }
 
-    SPSelection *selection = SP_DT_SELECTION(desktop);
+    Inkscape::Selection *selection = SP_DT_SELECTION(desktop);
 
     if (selection->isEmpty())
         return FALSE;
@@ -363,17 +382,17 @@ sp_select_toolbox_new(SPDesktop *desktop)
     GtkTooltips *tt = gtk_tooltips_new();
     GtkWidget *tb = gtk_hbox_new(FALSE, 0);
 
-    sp_toolbox_button_normal_new_from_verb(tb, AUX_BUTTON_SIZE, Inkscape::Verb::get(SP_VERB_OBJECT_ROTATE_90_CCW), view, tt);
-    sp_toolbox_button_normal_new_from_verb(tb, AUX_BUTTON_SIZE, Inkscape::Verb::get(SP_VERB_OBJECT_ROTATE_90_CW), view, tt);
-    sp_toolbox_button_normal_new_from_verb(tb, AUX_BUTTON_SIZE, Inkscape::Verb::get(SP_VERB_OBJECT_FLIP_HORIZONTAL), view, tt);
-    sp_toolbox_button_normal_new_from_verb(tb, AUX_BUTTON_SIZE, Inkscape::Verb::get(SP_VERB_OBJECT_FLIP_VERTICAL), view, tt);
+    sp_toolbox_button_normal_new_from_verb(tb, GTK_ICON_SIZE_SMALL_TOOLBAR, Inkscape::Verb::get(SP_VERB_OBJECT_ROTATE_90_CCW), view, tt);
+    sp_toolbox_button_normal_new_from_verb(tb, GTK_ICON_SIZE_SMALL_TOOLBAR, Inkscape::Verb::get(SP_VERB_OBJECT_ROTATE_90_CW), view, tt);
+    sp_toolbox_button_normal_new_from_verb(tb, GTK_ICON_SIZE_SMALL_TOOLBAR, Inkscape::Verb::get(SP_VERB_OBJECT_FLIP_HORIZONTAL), view, tt);
+    sp_toolbox_button_normal_new_from_verb(tb, GTK_ICON_SIZE_SMALL_TOOLBAR, Inkscape::Verb::get(SP_VERB_OBJECT_FLIP_VERTICAL), view, tt);
 
     aux_toolbox_space(tb, AUX_BETWEEN_BUTTON_GROUPS);
 
-    sp_toolbox_button_normal_new_from_verb(tb, AUX_BUTTON_SIZE, Inkscape::Verb::get(SP_VERB_SELECTION_TO_BACK), view, tt);
-    sp_toolbox_button_normal_new_from_verb(tb, AUX_BUTTON_SIZE, Inkscape::Verb::get(SP_VERB_SELECTION_LOWER), view, tt);
-    sp_toolbox_button_normal_new_from_verb(tb, AUX_BUTTON_SIZE, Inkscape::Verb::get(SP_VERB_SELECTION_RAISE), view, tt);
-    sp_toolbox_button_normal_new_from_verb(tb, AUX_BUTTON_SIZE, Inkscape::Verb::get(SP_VERB_SELECTION_TO_FRONT), view, tt);
+    sp_toolbox_button_normal_new_from_verb(tb, GTK_ICON_SIZE_SMALL_TOOLBAR, Inkscape::Verb::get(SP_VERB_SELECTION_TO_BACK), view, tt);
+    sp_toolbox_button_normal_new_from_verb(tb, GTK_ICON_SIZE_SMALL_TOOLBAR, Inkscape::Verb::get(SP_VERB_SELECTION_LOWER), view, tt);
+    sp_toolbox_button_normal_new_from_verb(tb, GTK_ICON_SIZE_SMALL_TOOLBAR, Inkscape::Verb::get(SP_VERB_SELECTION_RAISE), view, tt);
+    sp_toolbox_button_normal_new_from_verb(tb, GTK_ICON_SIZE_SMALL_TOOLBAR, Inkscape::Verb::get(SP_VERB_SELECTION_TO_FRONT), view, tt);
 
     // Create the parent widget for x y w h tracker.
     GtkWidget *spw = sp_widget_new_global(INKSCAPE);
@@ -397,18 +416,24 @@ sp_select_toolbox_new(SPDesktop *desktop)
     // four spinbuttons
 
     gtk_container_add(GTK_CONTAINER(vb),
-                      sp_select_toolbox_spinbutton("X", "X", -1e6, us, spw, _("Horizontal coordinate of selection"), TRUE));
+                      //TRANSLATORS: only translate "string" in "context|string". 
+                      // For more details, see http://developer.gnome.org/doc/API/2.0/glib/glib-I18N.html#Q-:CAPS
+                      sp_select_toolbox_spinbutton(_("select_toolbar|X"), "X", -1e6, us, spw, _("Horizontal coordinate of selection"), TRUE));
     aux_toolbox_space(vb, AUX_BETWEEN_SPINBUTTONS);
     gtk_container_add(GTK_CONTAINER(vb),
-                      sp_select_toolbox_spinbutton("Y", "Y", -1e6, us, spw, _("Vertical coordinate of selection"), FALSE));
+                      //TRANSLATORS: only translate "string" in "context|string". 
+                      // For more details, see http://developer.gnome.org/doc/API/2.0/glib/glib-I18N.html#Q-:CAPS
+                      sp_select_toolbox_spinbutton(_("select_toolbar|Y"), "Y", -1e6, us, spw, _("Vertical coordinate of selection"), FALSE));
     aux_toolbox_space(vb, AUX_BETWEEN_BUTTON_GROUPS);
 
     gtk_container_add(GTK_CONTAINER(vb),
-                      sp_select_toolbox_spinbutton("W", "width", 1e-3, us, spw, _("Width of selection"), FALSE));
+                      //TRANSLATORS: only translate "string" in "context|string". 
+                      // For more details, see http://developer.gnome.org/doc/API/2.0/glib/glib-I18N.html#Q-:CAPS
+                      sp_select_toolbox_spinbutton(_("select_toolbar|W"), "width", 1e-3, us, spw, _("Width of selection"), FALSE));
 
     // lock toggle
     GtkWidget *lockbox = gtk_vbox_new(TRUE, 0);
-    GtkWidget *lock = sp_button_new_from_data(9,
+    GtkWidget *lock = sp_button_new_from_data( GTK_ICON_SIZE_MENU,
                                               SP_BUTTON_TYPE_TOGGLE,
                                               NULL,
                                               "width_height_lock",
@@ -419,7 +444,9 @@ sp_select_toolbox_new(SPDesktop *desktop)
     gtk_object_set_data(GTK_OBJECT(spw), "lock", lock);
 
     gtk_container_add(GTK_CONTAINER(vb),
-                      sp_select_toolbox_spinbutton("H", "height", 1e-3, us, spw, _("Height of selection"), FALSE));
+                      //TRANSLATORS: only translate "string" in "context|string". 
+                      // For more details, see http://developer.gnome.org/doc/API/2.0/glib/glib-I18N.html#Q-:CAPS
+                      sp_select_toolbox_spinbutton(_("select_toolbar|H"), "height", 1e-3, us, spw, _("Height of selection"), FALSE));
 
     aux_toolbox_space(vb, 2);
 
@@ -429,7 +456,7 @@ sp_select_toolbox_new(SPDesktop *desktop)
     gtk_object_set_data(GTK_OBJECT(spw), "units", us);
 
     // Set font size.
-    sp_set_font_size(vb, AUX_FONT_SIZE);
+    sp_set_font_size_smaller (vb);
 
     // Force update when selection changes.
     gtk_signal_connect(GTK_OBJECT(spw), "modify_selection", GTK_SIGNAL_FUNC(sp_selection_layout_widget_modify_selection), desktop);
@@ -449,7 +476,7 @@ sp_select_toolbox_new(SPDesktop *desktop)
     GtkWidget *cbox = gtk_hbox_new (FALSE, 0);
 
     {
-    GtkWidget *button = sp_button_new_from_data(11,
+    GtkWidget *button = sp_button_new_from_data( GTK_ICON_SIZE_SMALL_TOOLBAR,
                                               SP_BUTTON_TYPE_TOGGLE,
                                               NULL,
                                               "transform_stroke",
@@ -461,7 +488,7 @@ sp_select_toolbox_new(SPDesktop *desktop)
     }
 
     {
-    GtkWidget *button = sp_button_new_from_data(11,
+    GtkWidget *button = sp_button_new_from_data( GTK_ICON_SIZE_SMALL_TOOLBAR,
                                               SP_BUTTON_TYPE_TOGGLE,
                                               NULL,
                                               "transform_corners",
@@ -473,7 +500,7 @@ sp_select_toolbox_new(SPDesktop *desktop)
     }
 
     {
-    GtkWidget *button = sp_button_new_from_data(11,
+    GtkWidget *button = sp_button_new_from_data( GTK_ICON_SIZE_SMALL_TOOLBAR,
                                               SP_BUTTON_TYPE_TOGGLE,
                                               NULL,
                                               "transform_gradient",
@@ -485,7 +512,7 @@ sp_select_toolbox_new(SPDesktop *desktop)
     }
 
     {
-    GtkWidget *button = sp_button_new_from_data(11,
+    GtkWidget *button = sp_button_new_from_data( GTK_ICON_SIZE_SMALL_TOOLBAR,
                                               SP_BUTTON_TYPE_TOGGLE,
                                               NULL,
                                               "transform_pattern",
