@@ -5,6 +5,7 @@
  *
  * Author:
  *   Lauris Kaplinski <lauris@kaplinski.com>
+ *   bulia byak <buliabyak@users.sf.net>
  *
  * Copyright (C) 1999-2002 Lauris Kaplinski
  *
@@ -18,6 +19,7 @@
 
 #include <libnr/nr-matrix.h>
 #include <libnr/nr-matrix-ops.h>
+#include <libnr/nr-point-matrix-ops.h>
 #include <libnr/nr-matrix-translate-ops.h>
 #include <libnr/nr-point-fns.h>
 #include <libnr/nr-rotate-ops.h>
@@ -40,7 +42,7 @@
 #include "selection.h"
 #include "select-context.h"
 #include "uri-references.h"
-#include "sp-desktop-widget.h"
+#include "desktop-widget.h"
 #include "sp-use.h"
 #include "sp-item.h"
 #include <sp-item-update-cns.h>
@@ -70,8 +72,8 @@ static void sp_sel_trans_handle_ungrab(SPKnot *knot, guint state, gpointer data)
 static void sp_sel_trans_handle_new_event(SPKnot *knot, NR::Point *position, guint32 state, gpointer data);
 static gboolean sp_sel_trans_handle_request(SPKnot *knot, NR::Point *p, guint state, gboolean *data);
 
-static void sp_sel_trans_sel_changed(SPSelection *selection, gpointer data);
-static void sp_sel_trans_sel_modified(SPSelection *selection, guint flags, gpointer data);
+static void sp_sel_trans_sel_changed(Inkscape::Selection *selection, gpointer data);
+static void sp_sel_trans_sel_modified(Inkscape::Selection *selection, guint flags, gpointer data);
 
 extern GdkPixbuf *handles[];
 
@@ -82,7 +84,7 @@ sp_seltrans_handle_event(SPKnot *knot, GdkEvent *event, gpointer)
         case GDK_MOTION_NOTIFY:
             break;
         case GDK_KEY_PRESS:
-            if (event->key.keyval == GDK_space) {
+            if (get_group0_keyval (&event->key) == GDK_space) {
                 /* stamping mode: both mode(show content and outline) operation with knot */
                 if (!SP_KNOT_IS_GRABBED(knot)) return FALSE;
                 SPDesktop *desktop = knot->desktop;
@@ -241,7 +243,7 @@ void sp_sel_trans_set_center(SPSelTrans *seltrans, NR::Point p)
 
 void sp_sel_trans_grab(SPSelTrans *seltrans, NR::Point const &p, gdouble x, gdouble y, gboolean show_handles)
 {
-    SPSelection *selection = SP_DT_SELECTION(seltrans->desktop);
+    Inkscape::Selection *selection = SP_DT_SELECTION(seltrans->desktop);
 
     g_return_if_fail(!seltrans->grabbed);
 
@@ -262,8 +264,8 @@ void sp_sel_trans_grab(SPSelTrans *seltrans, NR::Point const &p, gdouble x, gdou
 
     seltrans->point = p;
 
-    seltrans->snap_points = selection->getSnapPoints();
-    seltrans->bbox_points = selection->getBBoxPoints();
+    seltrans->snap_points = selection->getSnapPointsConvexHull ();
+    seltrans->bbox_points = selection->getBBoxPoints ();
 
     gchar const *scale_origin = prefs_get_string_attribute("tools.select", "scale_origin");
     bool const origin_on_bbox = (scale_origin == NULL || !strcmp(scale_origin, "bbox"));
@@ -332,7 +334,7 @@ void sp_sel_trans_ungrab(SPSelTrans *seltrans)
 {
     g_return_if_fail(seltrans->grabbed);
 
-    SPSelection *selection = SP_DT_SELECTION(seltrans->desktop);
+    Inkscape::Selection *selection = SP_DT_SELECTION(seltrans->desktop);
 
     bool updh = true;
     if (!seltrans->empty && seltrans->changed) {
@@ -378,7 +380,7 @@ void sp_sel_trans_ungrab(SPSelTrans *seltrans)
 
 void sp_sel_trans_stamp(SPSelTrans *seltrans)
 {
-    SPSelection *selection = SP_DT_SELECTION(seltrans->desktop);
+    Inkscape::Selection *selection = SP_DT_SELECTION(seltrans->desktop);
 
     /* stamping mode */
     if (!seltrans->empty) {
@@ -394,19 +396,19 @@ void sp_sel_trans_stamp(SPSelTrans *seltrans)
 
         while (l) {
             SPItem *original_item = SP_ITEM(l->data);
-            SPRepr *original_repr = SP_OBJECT_REPR(original_item);
+            Inkscape::XML::Node *original_repr = SP_OBJECT_REPR(original_item);
 
             // remember the position of the item
-            gint pos = sp_repr_position(original_repr);
+            gint pos = original_repr->position();
             // remember parent
-            SPRepr *parent = sp_repr_parent(original_repr);
+            Inkscape::XML::Node *parent = sp_repr_parent(original_repr);
 
-            SPRepr *copy_repr = sp_repr_duplicate(original_repr);
+            Inkscape::XML::Node *copy_repr = original_repr->duplicate();
 
             // add the new repr to the parent
-            sp_repr_append_child(parent, copy_repr);
+            parent->appendChild(copy_repr);
             // move to the saved position
-            sp_repr_set_position_absolute(copy_repr, pos > 0 ? pos : 0);
+            copy_repr->setPosition(pos > 0 ? pos : 0);
 
             SPItem *copy_item = (SPItem *) SP_DT_DOCUMENT(seltrans->desktop)->getObjectByRepr(copy_repr);
 
@@ -474,7 +476,7 @@ static void sp_sel_trans_update_handles(SPSelTrans &seltrans)
     } else {
         sp_remove_handles(seltrans.shandle, 8);
         sp_show_handles(seltrans, seltrans.rhandle, handles_rotate, 8,
-                        _("<b>Skew</b> selection; with <b>Shift</b> to skew around the opposite side"), // fixme: CTRL SNAP!!! remove "skew and scale"
+                        _("<b>Skew</b> selection; with <b>Ctrl</b> to snap angle; with <b>Shift</b> to skew around the opposite side"),
                         _("<b>Rotate</b> selection; with <b>Ctrl</b> to snap angle; with <b>Shift</b> to rotate around the opposite corner"));
     }
     if ( seltrans.state == SP_SELTRANS_STATE_SCALE ) {
@@ -487,7 +489,7 @@ static void sp_sel_trans_update_handles(SPSelTrans &seltrans)
 
 static void sp_sel_trans_update_volatile_state(SPSelTrans &seltrans)
 {
-    SPSelection *selection = SP_DT_SELECTION(seltrans.desktop);
+    Inkscape::Selection *selection = SP_DT_SELECTION(seltrans.desktop);
     seltrans.empty = selection->isEmpty();
 
     if (seltrans.empty) {
@@ -648,7 +650,7 @@ static gboolean sp_sel_trans_handle_request(SPKnot *knot, NR::Point *position, g
     return TRUE;
 }
 
-static void sp_sel_trans_sel_changed(SPSelection *selection, gpointer data)
+static void sp_sel_trans_sel_changed(Inkscape::Selection *selection, gpointer data)
 {
     SPSelTrans *seltrans = (SPSelTrans *) data;
 
@@ -660,7 +662,7 @@ static void sp_sel_trans_sel_changed(SPSelection *selection, gpointer data)
 }
 
 static void
-sp_sel_trans_sel_modified(SPSelection *selection, guint flags, gpointer data)
+sp_sel_trans_sel_modified(Inkscape::Selection *selection, guint flags, gpointer data)
 {
     SPSelTrans *seltrans;
 
@@ -668,11 +670,21 @@ sp_sel_trans_sel_modified(SPSelection *selection, guint flags, gpointer data)
 
     if (!seltrans->grabbed) {
         sp_sel_trans_update_volatile_state(*seltrans);
-        if (!seltrans->changed) { // do not reset center if the change was by seltrans itself; otherwise reset
+
+        if (
+             (flags != (SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_STYLE_MODIFIED_FLAG)) &&
+             (flags != SP_OBJECT_PARENT_MODIFIED_FLAG) &&
+             (flags != SP_OBJECT_CHILD_MODIFIED_FLAG) &&
+             !seltrans->changed) {
+            // Only reset center if object itself is modified (not style, parent or child), and this is not a local change by seltrans
+            // (still annoyingly recenters on keyboard transforms, fixme)
             seltrans->center = seltrans->box.midpoint();
-        } else {
-            seltrans->changed = FALSE;
         }
+
+        // reset internal flag
+        seltrans->changed = FALSE;
+
+        // Update the handles
         sp_sel_trans_update_handles(*seltrans);
     }
 }
@@ -887,40 +899,45 @@ gboolean sp_sel_trans_skew_request(SPSelTrans *seltrans, SPSelTransHandle const
     }
 
     skew[dim_a] = ( pt[dim_b] - point[dim_b] ) / ( point[dim_a] - norm[dim_a] );
-    skew[dim_a] = namedview_dim_snap_list_skew(desktop->namedview,
-                                               Snapper::SNAP_POINT, seltrans->snap_points,
-                                               norm, skew[dim_a], dim_b);
 
-    pt[dim_b] = ( point[dim_a] - norm[dim_a] ) * skew[dim_a] + point[dim_b];
     s[dim_a] = ( pt[dim_a] - norm[dim_a] ) / ( point[dim_a] - norm[dim_a] );
-    if (state & GDK_CONTROL_MASK) {
-        std::pair<NR::Coord, bool> sn = namedview_dim_snap_list_scale(desktop->namedview,
-                                                                      Snapper::SNAP_POINT, seltrans->snap_points,
-                                                                      norm, s[dim_a], dim_a);
-        s[dim_a] = sn.first;
+
+    if ( fabs(s[dim_a]) < 1 ) {
+        s[dim_a] = sign(s[dim_a]);
     } else {
-        if ( fabs(s[dim_a]) < NR_EPSILON ) {
-            s[dim_a] = NR_EPSILON;
-        }
-        if ( fabs(s[dim_a]) < 1 ) {
-            s[dim_a] = sign(s[dim_a]);
-        }
         s[dim_a] = floor( s[dim_a] + 0.5 );
     }
 
-    if (fabs(s[dim_a]) < NR_EPSILON) {
-        s[dim_a] = NR_EPSILON;
+    double radians = atan(skew[dim_a] / s[dim_a]);
+
+    if (state & GDK_CONTROL_MASK) {
+
+        int snaps = prefs_get_int_attribute("options.rotationsnapsperpi", "value", 12);
+
+        if (snaps) {
+            double sections = floor( radians * snaps / M_PI + .5 );
+            if (fabs(sections) >= snaps / 2) sections = sign(sections) * (snaps / 2 - 1);
+            radians = ( M_PI / snaps ) * sections;
+        }
+        skew[dim_a] = tan(radians) * s[dim_a];
+    } else {
+        skew[dim_a] = namedview_dim_snap_list_skew(desktop->namedview,
+                Snapper::SNAP_POINT, seltrans->snap_points,
+                norm, skew[dim_a], dim_b);
     }
 
+    pt[dim_b] = ( point[dim_a] - norm[dim_a] ) * skew[dim_a] + point[dim_b];
     pt[dim_a] = ( point[dim_a] - norm[dim_a] ) * s[dim_a] + norm[dim_a];
 
-    skew[dim_b] = 0;
+    /* status text */
+    double degrees = 180 / M_PI * radians;
+    if (degrees > 180) degrees -= 360;
+    if (degrees < -180) degrees += 360;
 
-    // status text
     seltrans->_message_context.setF(Inkscape::NORMAL_MESSAGE,
-                                    _("<b>Skew</b>: %0.2f%% x %0.2f%%"), // FIXME: display angle! % makes no sense here; snap with ctrl!
-                                    100 * fabs(skew[1]),
-                                    100 * fabs(skew[0]));
+                                    // TRANSLATORS: don't modify the first ";" (it will NOT be displayed as ";" - only the second one will be)
+                                    _("<b>Skew</b>: %0.2f&#176;; with <b>Ctrl</b> to snap angle"),
+                                    degrees);
 
     return TRUE;
 }
@@ -986,7 +1003,7 @@ gboolean sp_sel_trans_center_request(SPSelTrans *seltrans, SPSelTransHandle cons
         NR::Point point = seltrans->point;
         if ( fabs( point[X] - pt[X] )  >
              fabs( point[Y] - pt[Y] ) ) {
-            pt[Y] = point[X];
+            pt[Y] = point[Y];
         } else {
             pt[X] = point[X];
         }
