@@ -14,12 +14,15 @@
 
 #define noSP_ANCHOR_VERBOSE
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
 #include <string.h>
 
 #include <glibmm/i18n.h>
 #include "display/sp-canvas.h"
 #include "dialogs/object-attributes.h"
+#include "xml/quote.h"
 #include "xml/repr.h"
 #include "attributes.h"
 #include "sp-anchor.h"
@@ -30,10 +33,10 @@
 static void sp_anchor_class_init(SPAnchorClass *ac);
 static void sp_anchor_init(SPAnchor *anchor);
 
-static void sp_anchor_build(SPObject *object, SPDocument *document, SPRepr *repr);
+static void sp_anchor_build(SPObject *object, SPDocument *document, Inkscape::XML::Node *repr);
 static void sp_anchor_release(SPObject *object);
 static void sp_anchor_set(SPObject *object, unsigned int key, const gchar *value);
-static SPRepr *sp_anchor_write(SPObject *object, SPRepr *repr, guint flags);
+static Inkscape::XML::Node *sp_anchor_write(SPObject *object, Inkscape::XML::Node *repr, guint flags);
 
 static gchar *sp_anchor_description(SPItem *item);
 static gint sp_anchor_event(SPItem *item, SPEvent *event);
@@ -84,7 +87,7 @@ static void sp_anchor_init(SPAnchor *anchor)
     anchor->href = NULL;
 }
 
-static void sp_anchor_build(SPObject *object, SPDocument *document, SPRepr *repr)
+static void sp_anchor_build(SPObject *object, SPDocument *document, Inkscape::XML::Node *repr)
 {
     if (((SPObjectClass *) (parent_class))->build) {
         ((SPObjectClass *) (parent_class))->build(object, document, repr);
@@ -142,9 +145,9 @@ static void sp_anchor_set(SPObject *object, unsigned int key, const gchar *value
 }
 
 
-#define COPY_ATTR(rd,rs,key) sp_repr_set_attr((rd), (key), sp_repr_attr(rs, key));
+#define COPY_ATTR(rd,rs,key) sp_repr_set_attr((rd), (key), rs->attribute(key));
 
-static SPRepr *sp_anchor_write(SPObject *object, SPRepr *repr, guint flags)
+static Inkscape::XML::Node *sp_anchor_write(SPObject *object, Inkscape::XML::Node *repr, guint flags)
 {
     SPAnchor *anchor = SP_ANCHOR(object);
 
@@ -174,10 +177,14 @@ static SPRepr *sp_anchor_write(SPObject *object, SPRepr *repr, guint flags)
 static gchar *sp_anchor_description(SPItem *item)
 {
     SPAnchor *anchor = SP_ANCHOR(item);
-    if (anchor->href) 
-        return g_strdup_printf (_("<b>Link</b> to %s"), anchor->href);
-    else 
+    if (anchor->href) {
+        char *quoted_href = xml_quote_strdup(anchor->href);
+        char *ret = g_strdup_printf(_("<b>Link</b> to %s"), quoted_href);
+        g_free(quoted_href);
+        return ret;
+    } else {
         return g_strdup (_("<b>Link</b> without URI"));
+    }
 }
 
 /* fixme: We should forward event to appropriate container/view */
