@@ -3,9 +3,14 @@
 
 #include <glib/gtypes.h>
 
-#include "xml/xml-forward.h"
 #include "forward.h"
+
 class SPConnEnd;
+namespace Inkscape {
+namespace XML {
+class Node;
+}
+}
 
 
 class SPConnEndPair {
@@ -14,7 +19,7 @@ public:
     ~SPConnEndPair();
     void release();
     void setAttr(unsigned const key, gchar const *const value);
-    void writeRepr(SPRepr *const repr) const;
+    void writeRepr(Inkscape::XML::Node *const repr) const;
     void getAttachedItems(SPItem *[2]) const;
 
 private:
