@@ -6,6 +6,7 @@
  * Authors:
  *   Lauris Kaplinski <lauris@kaplinski.com>
  *   Mitsuru Oka
+ *   bulia byak <buliabyak@users.sf.net>
  *
  * Copyright (C) 1999-2002 Lauris Kaplinski
  * Copyright (C) 2000-2001 Ximian, Inc.
@@ -13,14 +14,17 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
 
 #include <math.h>
 #include <string.h>
 
 #include "libnr/n-art-bpath.h"
 #include "libnr/nr-matrix.h"
-#include "libnr/nr-matrix-ops.h"
+#include "libnr/nr-matrix-fns.h"
+#include "libnr/nr-point-matrix-ops.h"
 #include "svg/svg.h"
 #include "svg/stringstream.h"
 #include "xml/repr.h"
@@ -75,9 +79,9 @@ static void sp_genericellipse_update (SPObject *object, SPCtx *ctx, guint flags)
 static void sp_genericellipse_snappoints(SPItem const *item, SnapPointsIter p);
 
 static void sp_genericellipse_set_shape (SPShape *shape);
-static SPRepr *sp_genericellipse_write (SPObject *object, SPRepr *repr, guint flags);
+static Inkscape::XML::Node *sp_genericellipse_write (SPObject *object, Inkscape::XML::Node *repr, guint flags);
 
-static gboolean sp_arc_set_elliptical_path_attribute (SPArc *arc, SPRepr *repr);
+static gboolean sp_arc_set_elliptical_path_attribute (SPArc *arc, Inkscape::XML::Node *repr);
 
 static SPShapeClass *ge_parent_class;
 
@@ -138,7 +142,7 @@ sp_genericellipse_update (SPObject *object, SPCtx *ctx, guint flags)
 	if (flags & (SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_STYLE_MODIFIED_FLAG | SP_OBJECT_VIEWPORT_MODIFIED_FLAG)) {
 		SPGenericEllipse *ellipse = (SPGenericEllipse *) object;
 		SPStyle const *style = object->style;
-		double const d = 1.0 / NR_MATRIX_DF_EXPANSION(&((SPItemCtx const *) ctx)->i2vp);
+		double const d = 1.0 / NR::expansion(((SPItemCtx const *) ctx)->i2vp);
 		double const em = style->font_size.computed;
 		double const ex = em * 0.5; // fixme: get from pango or libnrtype
 		sp_svg_length_update(&ellipse->cx, em, ex, d);
@@ -256,16 +260,10 @@ static void sp_genericellipse_snappoints(SPItem const *item, SnapPointsIter p)
 
 	NR::Matrix const i2d = sp_item_i2d_affine(item);
 
-	/* Use the bounding box corners */
-	NRRect bbox;
-	sp_item_invoke_bbox(item, &bbox, i2d, TRUE);
-	*p = NR::Point(bbox.x0, bbox.y0);
-	*p = NR::Point(bbox.x1, bbox.y0);
-	*p = NR::Point(bbox.x1, bbox.y1);
-	*p = NR::Point(bbox.x0, bbox.y1);
-
-	/* And the centre */
+	/* Add the centre */
 	*p = NR::Point(ge->cx.computed, ge->cy.computed) * i2d;
+
+	// TODO: add the ends of radii
 }
 
 void
@@ -283,7 +281,7 @@ sp_genericellipse_normalize (SPGenericEllipse *ellipse)
 	/* Now we keep: 0 <= start < end <= 2*PI */
 }
 
-static SPRepr *sp_genericellipse_write (SPObject *object, SPRepr *repr, guint flags)
+static Inkscape::XML::Node *sp_genericellipse_write (SPObject *object, Inkscape::XML::Node *repr, guint flags)
 {
 	SPGenericEllipse *ellipse = SP_GENERICELLIPSE (object);
 
@@ -312,8 +310,8 @@ static SPRepr *sp_genericellipse_write (SPObject *object, SPRepr *repr, guint fl
 static void sp_ellipse_class_init (SPEllipseClass *klass);
 static void sp_ellipse_init (SPEllipse *ellipse);
 
-static void sp_ellipse_build(SPObject *object, SPDocument *document, SPRepr *repr);
-static SPRepr *sp_ellipse_write(SPObject *object, SPRepr *repr, guint flags);
+static void sp_ellipse_build(SPObject *object, SPDocument *document, Inkscape::XML::Node *repr);
+static Inkscape::XML::Node *sp_ellipse_write(SPObject *object, Inkscape::XML::Node *repr, guint flags);
 static void sp_ellipse_set(SPObject *object, unsigned int key, gchar const *value);
 static gchar *sp_ellipse_description(SPItem *item);
 
@@ -362,7 +360,7 @@ sp_ellipse_init (SPEllipse *ellipse)
 }
 
 static void
-sp_ellipse_build (SPObject *object, SPDocument *document, SPRepr *repr)
+sp_ellipse_build (SPObject *object, SPDocument *document, Inkscape::XML::Node *repr)
 {
 	if (((SPObjectClass *) ellipse_parent_class)->build)
 		(* ((SPObjectClass *) ellipse_parent_class)->build) (object, document, repr);
@@ -373,8 +371,8 @@ sp_ellipse_build (SPObject *object, SPDocument *document, SPRepr *repr)
 	sp_object_read_attr (object, "ry");
 }
 
-static SPRepr *
-sp_ellipse_write (SPObject *object, SPRepr *repr, guint flags)
+static Inkscape::XML::Node *
+sp_ellipse_write (SPObject *object, Inkscape::XML::Node *repr, guint flags)
 {
 	SPGenericEllipse *ellipse;
 
@@ -463,8 +461,8 @@ sp_ellipse_position_set (SPEllipse *ellipse, gdouble x, gdouble y, gdouble rx, g
 static void sp_circle_class_init (SPCircleClass *klass);
 static void sp_circle_init (SPCircle *circle);
 
-static void sp_circle_build(SPObject *object, SPDocument *document, SPRepr *repr);
-static SPRepr *sp_circle_write(SPObject *object, SPRepr *repr, guint flags);
+static void sp_circle_build(SPObject *object, SPDocument *document, Inkscape::XML::Node *repr);
+static Inkscape::XML::Node *sp_circle_write(SPObject *object, Inkscape::XML::Node *repr, guint flags);
 static void sp_circle_set(SPObject *object, unsigned int key, const gchar *value);
 static gchar *sp_circle_description(SPItem *item);
 
@@ -514,7 +512,7 @@ sp_circle_init (SPCircle *circle)
 }
 
 static void
-sp_circle_build (SPObject *object, SPDocument *document, SPRepr *repr)
+sp_circle_build (SPObject *object, SPDocument *document, Inkscape::XML::Node *repr)
 {
 	if (((SPObjectClass *) circle_parent_class)->build)
 		(* ((SPObjectClass *) circle_parent_class)->build) (object, document, repr);
@@ -524,8 +522,8 @@ sp_circle_build (SPObject *object, SPDocument *document, SPRepr *repr)
 	sp_object_read_attr (object, "r");
 }
 
-static SPRepr *
-sp_circle_write (SPObject *object, SPRepr *repr, guint flags)
+static Inkscape::XML::Node *
+sp_circle_write (SPObject *object, Inkscape::XML::Node *repr, guint flags)
 {
 	SPGenericEllipse *ellipse;
 
@@ -589,8 +587,8 @@ static gchar *sp_circle_description (SPItem *item)
 static void sp_arc_class_init (SPArcClass *klass);
 static void sp_arc_init (SPArc *arc);
 
-static void sp_arc_build(SPObject *object, SPDocument *document, SPRepr *repr);
-static SPRepr *sp_arc_write(SPObject *object, SPRepr *repr, guint flags);
+static void sp_arc_build(SPObject *object, SPDocument *document, Inkscape::XML::Node *repr);
+static Inkscape::XML::Node *sp_arc_write(SPObject *object, Inkscape::XML::Node *repr, guint flags);
 static void sp_arc_set(SPObject *object, unsigned int key, gchar const *value);
 static void sp_arc_modified(SPObject *object, guint flags);
 
@@ -643,12 +641,12 @@ sp_arc_init (SPArc *arc)
 }
 
 static void
-sp_arc_build (SPObject *object, SPDocument *document, SPRepr *repr)
+sp_arc_build (SPObject *object, SPDocument *document, Inkscape::XML::Node *repr)
 {
 	if (((SPObjectClass *) arc_parent_class)->build)
 		(* ((SPObjectClass *) arc_parent_class)->build) (object, document, repr);
 
-	SPVersion version = sp_object_get_sodipodi_version (object);
+	Inkscape::Version version = sp_object_get_sodipodi_version (object);
 
 	if (sp_version_inside_range (version, 0, 0, 0, 25)) {
 		/* Old spec violating arc attributes */
@@ -687,7 +685,7 @@ sp_arc_build (SPObject *object, SPDocument *document, SPRepr *repr)
  * ``F.6 Ellptical arc implementation notes'' for more detail.
  */
 static gboolean
-sp_arc_set_elliptical_path_attribute (SPArc *arc, SPRepr *repr)
+sp_arc_set_elliptical_path_attribute (SPArc *arc, Inkscape::XML::Node *repr)
 {
 	gint fa, fs;
 	gdouble  dt;
@@ -727,8 +725,8 @@ sp_arc_set_elliptical_path_attribute (SPArc *arc, SPRepr *repr)
 	return sp_repr_set_attr (repr, "d", os.str().c_str());
 }
 
-static SPRepr *
-sp_arc_write (SPObject *object, SPRepr *repr, guint flags)
+static Inkscape::XML::Node *
+sp_arc_write (SPObject *object, Inkscape::XML::Node *repr, guint flags)
 {
 	SPGenericEllipse *ge = SP_GENERICELLIPSE (object);
 	SPArc *arc = SP_ARC (object);
@@ -772,7 +770,7 @@ sp_arc_set (SPObject *object, unsigned int key, const gchar *value)
 {
 	SPGenericEllipse *ge = SP_GENERICELLIPSE (object);
 
-	SPVersion version = sp_object_get_sodipodi_version (object);
+	Inkscape::Version version = sp_object_get_sodipodi_version (object);
 
 	if (sp_version_inside_range (version, 0, 0, 0, 25)) {
 		switch (key) {
@@ -856,7 +854,19 @@ sp_arc_modified (SPObject *object, guint flags)
 
 static gchar *sp_arc_description(SPItem *item)
 {
-	return g_strdup(_("<b>Arc</b>"));
+	SPGenericEllipse *ge = SP_GENERICELLIPSE (item);
+
+	gdouble len = fmod (ge->end - ge->start, SP_2PI);
+	if (len < 0.0) len += SP_2PI;
+	if (!(fabs (len) < 1e-8 || fabs (len - SP_2PI) < 1e-8)) {
+		if (ge->closed) {
+			return g_strdup(_("<b>Segment</b>"));
+		} else {
+			return g_strdup(_("<b>Arc</b>"));
+		}
+	} else {
+		return g_strdup(_("<b>Ellipse</b>"));
+	}
 }
 
 void
