@@ -3,8 +3,11 @@
 /*
  */
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
 #include <string.h>
+#include <glibmm/i18n.h>
 
 #include <xml/repr.h>
 
@@ -17,8 +20,6 @@
 
 #include "sp-flowregion.h"
 
-#include "libnrtype/FlowDest.h"
-
 #include "display/curve.h"
 
 #include "libnr/nr-point.h"
@@ -35,11 +36,11 @@ static void sp_flowregion_class_init (SPFlowregionClass *klass);
 static void sp_flowregion_init (SPFlowregion *group);
 static void sp_flowregion_dispose (GObject *object);
 
-static void sp_flowregion_child_added (SPObject * object, SPRepr * child, SPRepr * ref);
-static void sp_flowregion_remove_child (SPObject * object, SPRepr * child);
+static void sp_flowregion_child_added (SPObject * object, Inkscape::XML::Node * child, Inkscape::XML::Node * ref);
+static void sp_flowregion_remove_child (SPObject * object, Inkscape::XML::Node * child);
 static void sp_flowregion_update (SPObject *object, SPCtx *ctx, guint flags);
 static void sp_flowregion_modified (SPObject *object, guint flags);
-static SPRepr *sp_flowregion_write (SPObject *object, SPRepr *repr, guint flags);
+static Inkscape::XML::Node *sp_flowregion_write (SPObject *object, Inkscape::XML::Node *repr, guint flags);
 
 static gchar * sp_flowregion_description (SPItem * item);
 
@@ -49,18 +50,18 @@ static void sp_flowregionexclude_class_init (SPFlowregionExcludeClass *klass);
 static void sp_flowregionexclude_init (SPFlowregionExclude *group);
 static void sp_flowregionexclude_dispose (GObject *object);
 
-static void sp_flowregionexclude_child_added (SPObject * object, SPRepr * child, SPRepr * ref);
-static void sp_flowregionexclude_remove_child (SPObject * object, SPRepr * child);
+static void sp_flowregionexclude_child_added (SPObject * object, Inkscape::XML::Node * child, Inkscape::XML::Node * ref);
+static void sp_flowregionexclude_remove_child (SPObject * object, Inkscape::XML::Node * child);
 static void sp_flowregionexclude_update (SPObject *object, SPCtx *ctx, guint flags);
 static void sp_flowregionexclude_modified (SPObject *object, guint flags);
-static SPRepr *sp_flowregionexclude_write (SPObject *object, SPRepr *repr, guint flags);
+static Inkscape::XML::Node *sp_flowregionexclude_write (SPObject *object, Inkscape::XML::Node *repr, guint flags);
 
 static gchar * sp_flowregionexclude_description (SPItem * item);
 
 static SPItemClass * flowregionexclude_parent_class;
 
 
-static void         GetDest(SPObject* child,flow_dest* computed,NR::Matrix itr_mat);
+static void         GetDest(SPObject* child,Shape **computed,NR::Matrix itr_mat);
 
 GType
 sp_flowregion_get_type (void)
@@ -111,22 +112,20 @@ sp_flowregion_class_init (SPFlowregionClass *klass)
 static void
 sp_flowregion_init (SPFlowregion *group)
 {
-	group->nbComp=group->maxComp=0;
-	group->computed=NULL;
+	new (&group->computed) std::vector<Shape*>;
 }
 
 static void
 sp_flowregion_dispose(GObject *object)
 {
 	SPFlowregion *group=(SPFlowregion *)object;
-	for (int i=0;i<group->nbComp;i++) delete group->computed[i];
-	if ( group->computed ) free(group->computed);
-	group->nbComp=group->maxComp=0;
-	group->computed=NULL;
+    for (std::vector<Shape*>::iterator it = group->computed.begin() ; it != group->computed.end() ; it++)
+        delete *it;
+    group->computed.~vector<Shape*>();
 }
 
 static void
-sp_flowregion_child_added (SPObject *object, SPRepr *child, SPRepr *ref)
+sp_flowregion_child_added (SPObject *object, Inkscape::XML::Node *child, Inkscape::XML::Node *ref)
 {
 	SPItem *item;
 	
@@ -141,7 +140,7 @@ sp_flowregion_child_added (SPObject *object, SPRepr *child, SPRepr *ref)
 /* fixme: hide (Lauris) */
 
 static void
-sp_flowregion_remove_child (SPObject * object, SPRepr * child)
+sp_flowregion_remove_child (SPObject * object, Inkscape::XML::Node * child)
 {
 	if (((SPObjectClass *) (flowregion_parent_class))->remove_child)
 		(* ((SPObjectClass *) (flowregion_parent_class))->remove_child) (object, child);
@@ -179,14 +178,9 @@ sp_flowregion_update (SPObject *object, SPCtx *ctx, unsigned int flags)
 		l = g_slist_remove (l, child);
 		if (flags || (child->uflags & (SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_CHILD_MODIFIED_FLAG))) {
 			if (SP_IS_ITEM (child)) {
-				SPItem *chi;
-				chi = SP_ITEM (child);
-
-				NRMatrix chitransform; // FIXME!!! we need to make everything NR::Matrix
-				(chi->transform).copyto (&chitransform);
-
-				nr_matrix_multiply (&cctx.i2doc, &chitransform, &ictx->i2doc);
-				nr_matrix_multiply (&cctx.i2vp, &chitransform, &ictx->i2vp);
+				SPItem const &chi = *SP_ITEM(child);
+				cctx.i2doc = chi.transform * ictx->i2doc;
+				cctx.i2vp = chi.transform * ictx->i2vp;
 				child->updateDisplay((SPCtx *)&cctx, flags);
 			} else {
 				child->updateDisplay(ctx, flags);
@@ -205,16 +199,14 @@ void             SPFlowregion::UpdateComputed(void)
 	NR::Matrix itr_mat=sp_item_i2root_affine (SP_ITEM(object));
 	itr_mat=itr_mat.inverse();
 	
-	for (int i=0;i<nbComp;i++) delete computed[i];
-	nbComp=0;
+    for (std::vector<Shape*>::iterator it = computed.begin() ; it != computed.end() ; it++)
+        delete *it;
+    computed.clear();
 	
 	for (SPObject* child = sp_object_first_child(object) ; child != NULL ; child = SP_OBJECT_NEXT(child) ) {
-		if ( nbComp >= maxComp ) {
-			maxComp=2*nbComp+1;
-			computed=(flow_dest**)realloc(computed,maxComp*sizeof(flow_dest*));
-		}
-		computed[nbComp++]=new flow_dest,
-		GetDest(child,computed[nbComp-1],itr_mat);
+        Shape *shape = NULL;
+		GetDest(child,&shape,itr_mat);
+        computed.push_back(shape);
 	}
 }
 
@@ -246,8 +238,8 @@ sp_flowregion_modified (SPObject *object, guint flags)
 	}
 }
 
-static SPRepr *
-sp_flowregion_write (SPObject *object, SPRepr *repr, guint flags)
+static Inkscape::XML::Node *
+sp_flowregion_write (SPObject *object, Inkscape::XML::Node *repr, guint flags)
 {
 	SPFlowregion *group;
 	
@@ -264,14 +256,10 @@ sp_flowregion_write (SPObject *object, SPRepr *repr, guint flags)
 }
 
 
-static gchar * sp_flowregion_description (SPItem * item)
+static gchar *sp_flowregion_description(SPItem *item)
 {
-	SPFlowregion * group;
-	
-	group = SP_FLOWREGION (item);
-	
-	//	return g_strdup_printf(_("Flow region"));
-	return g_strdup_printf("Flow region");
+	// TRANSLATORS: "Flow region" is an area where text is allowed to flow
+	return g_strdup_printf(_("Flow region"));
 }
 
 /*
@@ -327,18 +315,21 @@ sp_flowregionexclude_class_init (SPFlowregionExcludeClass *klass)
 static void
 sp_flowregionexclude_init (SPFlowregionExclude *group)
 {
-	group->computed=new flow_dest;
+	group->computed = NULL;
 }
 
 static void
 sp_flowregionexclude_dispose(GObject *object)
 {
 	SPFlowregionExclude *group=(SPFlowregionExclude *)object;
-	delete group->computed;
+    if (group->computed) {
+        delete group->computed;
+        group->computed = NULL;
+    }
 }
 
 static void
-sp_flowregionexclude_child_added (SPObject *object, SPRepr *child, SPRepr *ref)
+sp_flowregionexclude_child_added (SPObject *object, Inkscape::XML::Node *child, Inkscape::XML::Node *ref)
 {
 	SPItem *item;
 	
@@ -353,7 +344,7 @@ sp_flowregionexclude_child_added (SPObject *object, SPRepr *child, SPRepr *ref)
 /* fixme: hide (Lauris) */
 
 static void
-sp_flowregionexclude_remove_child (SPObject * object, SPRepr * child)
+sp_flowregionexclude_remove_child (SPObject * object, Inkscape::XML::Node * child)
 {
 	if (((SPObjectClass *) (flowregionexclude_parent_class))->remove_child)
 		(* ((SPObjectClass *) (flowregionexclude_parent_class))->remove_child) (object, child);
@@ -391,14 +382,9 @@ sp_flowregionexclude_update (SPObject *object, SPCtx *ctx, unsigned int flags)
 		l = g_slist_remove (l, child);
 		if (flags || (child->uflags & (SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_CHILD_MODIFIED_FLAG))) {
 			if (SP_IS_ITEM (child)) {
-				SPItem *chi;
-				chi = SP_ITEM (child);
-
-				NRMatrix chitransform; // FIXME!!! we need to make everything NR::Matrix
-				(chi->transform).copyto (&chitransform);
-
-				nr_matrix_multiply (&cctx.i2doc, &chitransform, &ictx->i2doc);
-				nr_matrix_multiply (&cctx.i2vp, &chitransform, &ictx->i2vp);
+				SPItem const &chi = *SP_ITEM(child);
+				cctx.i2doc = chi.transform * ictx->i2doc;
+				cctx.i2vp = chi.transform * ictx->i2vp;
 				child->updateDisplay((SPCtx *)&cctx, flags);
 			} else {
 				child->updateDisplay(ctx, flags);
@@ -413,12 +399,15 @@ void             SPFlowregionExclude::UpdateComputed(void)
 {
 	SPObject* object=SP_OBJECT(this);
 	
-	computed->Reset();
+	if (computed) {
+        delete computed;
+        computed = NULL;
+    }
 	NR::Matrix itr_mat=sp_item_i2root_affine (SP_ITEM(object));
 	itr_mat=itr_mat.inverse();
 	
 	for (SPObject* child = sp_object_first_child(object) ; child != NULL ; child = SP_OBJECT_NEXT(child) ) {
-		GetDest(child,computed,itr_mat);
+		GetDest(child,&computed,itr_mat);
 	}
 }
 
@@ -450,8 +439,8 @@ sp_flowregionexclude_modified (SPObject *object, guint flags)
 	}
 }
 
-static SPRepr *
-sp_flowregionexclude_write (SPObject *object, SPRepr *repr, guint flags)
+static Inkscape::XML::Node *
+sp_flowregionexclude_write (SPObject *object, Inkscape::XML::Node *repr, guint flags)
 {
 	SPFlowregionExclude *group;
 	
@@ -468,21 +457,34 @@ sp_flowregionexclude_write (SPObject *object, SPRepr *repr, guint flags)
 }
 
 
-static gchar * sp_flowregionexclude_description (SPItem * item)
+static gchar *sp_flowregionexclude_description(SPItem *item)
 {
-	SPFlowregionExclude * group;
-	
-	group = SP_FLOWREGIONEXCLUDE (item);
-	
-	//	return g_strdup_printf(_("Flow region"));
-	return g_strdup_printf("Flow excluded region");
+	/* TRANSLATORS: A region "cut out of" a flow region; text is not allowed to flow inside the
+	 * flow excluded region.  flowRegionExclude in SVG 1.2: see
+	 * http://www.w3.org/TR/2004/WD-SVG12-20041027/flow.html#flowRegion-elem and
+	 * http://www.w3.org/TR/2004/WD-SVG12-20041027/flow.html#flowRegionExclude-elem. */
+	return g_strdup_printf(_("Flow excluded region"));
 }
 
 /*
  *
  */
 
-static void         GetDest(SPObject* child,flow_dest* computed,NR::Matrix itr_mat)
+static void         UnionShape(Shape **base_shape, Shape const *add_shape)
+{
+    if (*base_shape == NULL)
+        *base_shape = new Shape;
+	if ( (*base_shape)->hasEdges() == false ) {
+		(*base_shape)->Copy(const_cast<Shape*>(add_shape));
+	} else if ( add_shape->hasEdges() ) {
+		Shape* temp=new Shape;
+		temp->Booleen(const_cast<Shape*>(add_shape), *base_shape, bool_op_union);
+		delete *base_shape;
+		*base_shape = temp;
+	}
+}
+
+static void         GetDest(SPObject* child,Shape **computed,NR::Matrix itr_mat)
 {	
 	if ( child == NULL ) return;
 	
@@ -495,7 +497,7 @@ static void         GetDest(SPObject* child,flow_dest* computed,NR::Matrix itr_m
 	if ( SP_IS_SHAPE (u_child) ) {
 		curve = sp_shape_get_curve (SP_SHAPE (u_child));
 	} else if ( SP_IS_TEXT (u_child) ) {
-	curve = sp_text_normalized_bpath (SP_TEXT (u_child));
+	curve = SP_TEXT (u_child)->getNormalizedBpath ();
 	}
 	
 	if ( curve ) {
@@ -513,7 +515,7 @@ static void         GetDest(SPObject* child,flow_dest* computed,NR::Matrix itr_m
 		} else {
 			uncross->ConvertToShape(n_shp,fill_nonZero);
 		}
-		computed->AddShape(uncross);
+		UnionShape(computed, uncross);
 		delete uncross;
 		delete n_shp;
 		delete temp;
