@@ -5,21 +5,25 @@
  *
  * Authors:
  *   Lauris Kaplinski <lauris@kaplinski.com>
+ *   bulia byak <buliabyak@users.sf.net>
  *
- * Copyright (C) 1999-2002 authors
+ * Copyright (C) 1999-2005 authors
  * Copyright (C) 2000-2001 Ximian, Inc.
  *
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
 #include <string.h>
 
 #include "display/nr-arena-group.h"
-#include <libnr/nr-matrix.h>
-#include <libnr/nr-matrix-ops.h>
-#include <xml/repr.h>
-#include <xml/repr-get-children.h>
+#include "libnr/nr-matrix.h"
+#include "libnr/nr-matrix-ops.h"
+#include "libnr/nr-matrix-fns.h"
+#include "xml/repr.h"
+#include "xml/node-fns.h"
 #include "sp-object-repr.h"
 #include "svg/svg.h"
 #include "document.h"
@@ -34,16 +38,16 @@
 
 static void sp_group_class_init (SPGroupClass *klass);
 static void sp_group_init (SPGroup *group);
-static void sp_group_build(SPObject *object, SPDocument *document, SPRepr *repr);
+static void sp_group_build(SPObject *object, SPDocument *document, Inkscape::XML::Node *repr);
 static void sp_group_release(SPObject *object);
 static void sp_group_dispose (GObject *object);
 
-static void sp_group_child_added (SPObject * object, SPRepr * child, SPRepr * ref);
-static void sp_group_remove_child (SPObject * object, SPRepr * child);
-static void sp_group_order_changed (SPObject * object, SPRepr * child, SPRepr * old_ref, SPRepr * new_ref);
+static void sp_group_child_added (SPObject * object, Inkscape::XML::Node * child, Inkscape::XML::Node * ref);
+static void sp_group_remove_child (SPObject * object, Inkscape::XML::Node * child);
+static void sp_group_order_changed (SPObject * object, Inkscape::XML::Node * child, Inkscape::XML::Node * old_ref, Inkscape::XML::Node * new_ref);
 static void sp_group_update (SPObject *object, SPCtx *ctx, guint flags);
 static void sp_group_modified (SPObject *object, guint flags);
-static SPRepr *sp_group_write (SPObject *object, SPRepr *repr, guint flags);
+static Inkscape::XML::Node *sp_group_write (SPObject *object, Inkscape::XML::Node *repr, guint flags);
 static void sp_group_set(SPObject *object, unsigned key, char const *value);
 
 static void sp_group_bbox(SPItem const *item, NRRect *bbox, NR::Matrix const &transform, unsigned const flags);
@@ -117,7 +121,7 @@ sp_group_init (SPGroup *group)
 	new (&group->_display_modes) std::map<unsigned int, SPGroup::LayerMode>();
 }
 
-static void sp_group_build(SPObject *object, SPDocument *document, SPRepr *repr)
+static void sp_group_build(SPObject *object, SPDocument *document, Inkscape::XML::Node *repr)
 {
 	sp_object_read_attr(object, "inkscape:groupmode");
 
@@ -142,7 +146,7 @@ sp_group_dispose(GObject *object)
 }
 
 static void
-sp_group_child_added (SPObject *object, SPRepr *child, SPRepr *ref)
+sp_group_child_added (SPObject *object, Inkscape::XML::Node *child, Inkscape::XML::Node *ref)
 {
 	SPItem *item;
 
@@ -151,24 +155,44 @@ sp_group_child_added (SPObject *object, SPRepr *child, SPRepr *ref)
 	if (((SPObjectClass *) (parent_class))->child_added)
 		(* ((SPObjectClass *) (parent_class))->child_added) (object, child, ref);
 
-	SPObject *ochild = sp_object_get_child_by_repr(object, child);
-	if ( ochild && SP_IS_ITEM(ochild) ) {
-		/* TODO: this should be moved into SPItem somehow */
-		SPItemView *v;
-		NRArenaItem *ac;
+    SPObject *last_child = object->lastChild();
+    if (last_child && SP_OBJECT_REPR(last_child) == child) {
+        // optimization for the common special case where the child is being added at the end
+	    SPObject *ochild = last_child;
+	    if ( SP_IS_ITEM(ochild) ) {
+		    /* TODO: this should be moved into SPItem somehow */
+		    SPItemView *v;
+		    NRArenaItem *ac;
 
-		unsigned position = sp_item_pos_in_parent(SP_ITEM(ochild));
+		    for (v = item->display; v != NULL; v = v->next) {
+			    ac = sp_item_invoke_show (SP_ITEM (ochild), NR_ARENA_ITEM_ARENA (v->arenaitem), v->key, v->flags);
 
-		for (v = item->display; v != NULL; v = v->next) {
-			ac = sp_item_invoke_show (SP_ITEM (ochild), NR_ARENA_ITEM_ARENA (v->arenaitem), v->key, v->flags);
+			    if (ac) {
+				    nr_arena_item_append_child (v->arenaitem, ac);
+				    nr_arena_item_unref (ac);
+			    }
+		    }
+	    }
+    } else {    // general case
+	    SPObject *ochild = sp_object_get_child_by_repr(object, child);
+	    if ( ochild && SP_IS_ITEM(ochild) ) {
+		    /* TODO: this should be moved into SPItem somehow */
+		    SPItemView *v;
+		    NRArenaItem *ac;
 
-			if (ac) {
-				nr_arena_item_add_child (v->arenaitem, ac, NULL);
-				nr_arena_item_set_order (ac, position);
-				nr_arena_item_unref (ac);
-			}
-		}
-	}
+		    unsigned position = sp_item_pos_in_parent(SP_ITEM(ochild));
+
+		    for (v = item->display; v != NULL; v = v->next) {
+			    ac = sp_item_invoke_show (SP_ITEM (ochild), NR_ARENA_ITEM_ARENA (v->arenaitem), v->key, v->flags);
+
+			    if (ac) {
+				    nr_arena_item_add_child (v->arenaitem, ac, NULL);
+				    nr_arena_item_set_order (ac, position);
+				    nr_arena_item_unref (ac);
+			    }
+		    }
+	    }
+    }
 
 	object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 }
@@ -176,7 +200,7 @@ sp_group_child_added (SPObject *object, SPRepr *child, SPRepr *ref)
 /* fixme: hide (Lauris) */
 
 static void
-sp_group_remove_child (SPObject * object, SPRepr * child)
+sp_group_remove_child (SPObject * object, Inkscape::XML::Node * child)
 {
 	if (((SPObjectClass *) (parent_class))->remove_child)
 		(* ((SPObjectClass *) (parent_class))->remove_child) (object, child);
@@ -185,7 +209,7 @@ sp_group_remove_child (SPObject * object, SPRepr * child)
 }
 
 static void
-sp_group_order_changed (SPObject *object, SPRepr *child, SPRepr *old_ref, SPRepr *new_ref)
+sp_group_order_changed (SPObject *object, Inkscape::XML::Node *child, Inkscape::XML::Node *old_ref, Inkscape::XML::Node *new_ref)
 {
 	if (((SPObjectClass *) (parent_class))->order_changed)
 		(* ((SPObjectClass *) (parent_class))->order_changed) (object, child, old_ref, new_ref);
@@ -272,12 +296,12 @@ sp_group_modified (SPObject *object, guint flags)
 	}
 }
 
-static SPRepr *
-sp_group_write (SPObject *object, SPRepr *repr, guint flags)
+static Inkscape::XML::Node *
+sp_group_write (SPObject *object, Inkscape::XML::Node *repr, guint flags)
 {
 	SPGroup *group;
 	SPObject *child;
-	SPRepr *crepr;
+	Inkscape::XML::Node *crepr;
 
 	group = SP_GROUP (object);
 
@@ -290,8 +314,8 @@ sp_group_write (SPObject *object, SPRepr *repr, guint flags)
 			if (crepr) l = g_slist_prepend (l, crepr);
 		}
 		while (l) {
-			sp_repr_add_child (repr, (SPRepr *) l->data, NULL);
-			sp_repr_unref ((SPRepr *) l->data);
+			sp_repr_add_child (repr, (Inkscape::XML::Node *) l->data, NULL);
+			sp_repr_unref ((Inkscape::XML::Node *) l->data);
 			l = g_slist_remove (l, l->data);
 		}
 	} else {
@@ -463,26 +487,36 @@ sp_item_group_ungroup (SPGroup *group, GSList **children, bool do_done)
 	SPObject *defs = SP_OBJECT (SP_ROOT (root)->defs);
 
 	SPItem *gitem = SP_ITEM (group);
-	SPRepr *grepr = SP_OBJECT_REPR (gitem);
-	SPCSSAttr *gstyle = sp_css_attr_from_style (SP_OBJECT (gitem));
+	Inkscape::XML::Node *grepr = SP_OBJECT_REPR (gitem);
+
+	g_return_if_fail (!strcmp (grepr->name(), "svg:g") || !strcmp (grepr->name(), "svg:a") || !strcmp (grepr->name(), "svg:switch"));
+
+      // this converts the gradient/pattern fill/stroke on the group, if any, to userSpaceOnUse
+      sp_item_adjust_paint_recursive (gitem, NR::identity(), NR::identity(), false);
 
 	SPItem *pitem = SP_ITEM (SP_OBJECT_PARENT (gitem));
-	SPRepr *prepr = SP_OBJECT_REPR (pitem);
-
-	g_return_if_fail (!strcmp (sp_repr_name (grepr), "svg:g") || !strcmp (sp_repr_name (grepr), "svg:a") || !strcmp (sp_repr_name (grepr), "svg:switch"));
+	Inkscape::XML::Node *prepr = SP_OBJECT_REPR (pitem);
 
 	/* Step 1 - generate lists of children objects */
 	GSList *items = NULL;
 	GSList *objects = NULL;
 	for (SPObject *child = sp_object_first_child(SP_OBJECT(group)) ; child != NULL; child = SP_OBJECT_NEXT(child) ) {
 
-		SPRepr *nrepr = sp_repr_duplicate (SP_OBJECT_REPR (child));
-
 		if (SP_IS_ITEM (child)) {
-			gchar affinestr[80];
 
 			SPItem *citem = SP_ITEM (child);
 
+			/* Merging of style */
+			// this converts the gradient/pattern fill/stroke, if any, to userSpaceOnUse; we need to do
+			// it here _before_ the new transform is set, so as to use the pre-transform bbox
+			sp_item_adjust_paint_recursive (citem, NR::identity(), NR::identity(), false);
+
+			sp_style_merge_from_dying_parent(SP_OBJECT_STYLE(child), SP_OBJECT_STYLE(gitem));
+			child->updateRepr();
+
+			Inkscape::XML::Node *nrepr = SP_OBJECT_REPR (child)->duplicate();
+
+			// Merging transform
 			NR::Matrix ctrans;
 			NR::Matrix const g(gitem->transform);
 			if (SP_IS_USE(citem) && sp_use_get_original (SP_USE(citem)) && 
@@ -503,39 +537,34 @@ sp_item_group_ungroup (SPGroup *group, GSList **children, bool do_done)
 			// This is just a way to temporarily remember the transform in repr. When repr is
 			// reattached outside of the group, the transform will be written more properly
 			// (i.e. optimized into the object if the corresponding preference is set)
+			gchar affinestr[80];
 			if (sp_svg_transform_write(affinestr, 79, ctrans)) {
 				sp_repr_set_attr (nrepr, "transform", affinestr);
 			} else {
 				sp_repr_set_attr (nrepr, "transform", NULL);
 			}
 
-			/* Merging of style */
-			// we do this by merging SPCSSAttrs, because there's no easy way to do this with SPStyle
-			// perhaps we need to program some sort of sp_style_combine_with_parent (SPStyle *, SPStyle *)
-			SPCSSAttr *cstyle = sp_repr_css_attr_new ();
-			sp_repr_css_merge (cstyle, gstyle);
-			sp_repr_css_merge (cstyle, sp_css_attr_from_style (SP_OBJECT (citem)));
-			sp_repr_css_change (nrepr, cstyle, "style");
-
 			items = g_slist_prepend (items, nrepr);
+
 		} else {
+			Inkscape::XML::Node *nrepr = SP_OBJECT_REPR (child)->duplicate();
 			objects = g_slist_prepend (objects, nrepr);
 		}
 	}
 
 	/* Step 2 - clear group */
 	// remember the position of the group
-	SPRepr *g_prev_sibling = sp_repr_prev(SP_OBJECT_REPR(group));
+	gint pos = SP_OBJECT_REPR(group)->position();
 
 	// the group is leaving forever, no heir, clones should take note; its children however are going to reemerge
 	SP_OBJECT (group)->deleteObject(true, false);
 
 	/* Step 3 - add nonitems */
 	if (objects) {
-	    SPRepr *last_def = sp_repr_last_child(SP_OBJECT_REPR(defs));
+	    Inkscape::XML::Node *last_def = SP_OBJECT_REPR(defs)->lastChild();
 	    while (objects) {
-		sp_repr_add_child(SP_OBJECT_REPR(defs), (SPRepr *) objects->data, last_def);
-		sp_repr_unref ((SPRepr *) objects->data);
+		sp_repr_add_child(SP_OBJECT_REPR(defs), (Inkscape::XML::Node *) objects->data, last_def);
+		sp_repr_unref ((Inkscape::XML::Node *) objects->data);
 		objects = g_slist_remove (objects, objects->data);
 	    }
 	}
@@ -543,9 +572,12 @@ sp_item_group_ungroup (SPGroup *group, GSList **children, bool do_done)
 	/* Step 4 - add items */
 	gint const preserve = prefs_get_int_attribute("options.preservetransform", "value", 0);
 	while (items) {
-		SPRepr *repr = (SPRepr *) items->data;
+		Inkscape::XML::Node *repr = (Inkscape::XML::Node *) items->data;
 		// add item
-		sp_repr_add_child(prepr, repr, g_prev_sibling);
+		prepr->appendChild(repr);
+		// restore position; since the items list was prepended (i.e. reverse), we now add
+		// all children at the same pos, which inverts the order once again
+		repr->setPosition(pos > 0 ? pos : 0);
 
 		// fill in the children list if non-null
 		SPItem *nitem = (SPItem *) doc->getObjectByRepr(repr);
@@ -599,7 +631,7 @@ sp_item_group_get_child_by_name (SPGroup *group, SPObject *ref, const gchar *nam
 {
 	SPObject *child;
 	child = (ref) ? SP_OBJECT_NEXT(ref) : sp_object_first_child(SP_OBJECT(group));
-	while ( child && strcmp (sp_repr_name (SP_OBJECT_REPR(child)), name) ) {
+	while ( child && strcmp (SP_OBJECT_REPR(child)->name(), name) ) {
 		child = SP_OBJECT_NEXT(child);
 	}
 	return child;
@@ -645,4 +677,3 @@ void SPGroup::_updateLayerMode(unsigned int display_key) {
 		}
 	}
 }
-
