@@ -6,14 +6,17 @@
 /*
  * Author:
  *   Lauris Kaplinski <lauris@kaplinski.com>
+ *   bulia byak <buliabyak@users.sf.net>
  *
- * Copyright (C) 2001-2002 Lauris Kaplinski
+ * Copyright (C) 2001-2005 authors
  * Copyright (C) 2001 Ximian, Inc.
  *
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
 
 #include <math.h>
 #include <string.h>
@@ -42,13 +45,19 @@
 #include "sp-text.h"
 #include "sp-item.h"
 #include "sp-item-rm-unsatisfied-cns.h"
+#include "sp-pattern.h"
+#include "sp-gradient.h"
+#include "gradient-chemistry.h"
 #include "prefs-utils.h"
+
 #include "libnr/nr-matrix.h"
 #include "libnr/nr-matrix-div.h"
 #include "libnr/nr-matrix-fns.h"
 #include "libnr/nr-matrix-ops.h"
 #include "libnr/nr-matrix-scale-ops.h"
 #include "libnr/nr-matrix-translate-ops.h"
+#include "libnr/nr-scale-translate-ops.h"
+#include "libnr/nr-translate-matrix-ops.h"
 #include "libnr/nr-translate-scale-ops.h"
 #include "libnr/nr-rect.h"
 #include "svg/stringstream.h"
@@ -63,11 +72,11 @@
 static void sp_item_class_init(SPItemClass *klass);
 static void sp_item_init(SPItem *item);
 
-static void sp_item_build(SPObject *object, SPDocument *document, SPRepr *repr);
+static void sp_item_build(SPObject *object, SPDocument *document, Inkscape::XML::Node *repr);
 static void sp_item_release(SPObject *object);
 static void sp_item_set(SPObject *object, unsigned key, gchar const *value);
 static void sp_item_update(SPObject *object, SPCtx *ctx, guint flags);
-static SPRepr *sp_item_write(SPObject *object, SPRepr *repr, guint flags);
+static Inkscape::XML::Node *sp_item_write(SPObject *object, Inkscape::XML::Node *repr, guint flags);
 
 static gchar *sp_item_private_description(SPItem *item);
 static void sp_item_private_snappoints(SPItem const *item, SnapPointsIter p);
@@ -230,7 +239,7 @@ void SPItem::raiseToTop() {
         SP_OBJECT_NEXT(this), NULL, &is_item
     );
     if (topmost) {
-        SPRepr *repr=SP_OBJECT_REPR(this);
+        Inkscape::XML::Node *repr=SP_OBJECT_REPR(this);
         sp_repr_change_order(sp_repr_parent(repr), repr, SP_OBJECT_REPR(topmost));
     }
 }
@@ -240,8 +249,8 @@ void SPItem::raiseOne() {
         SP_OBJECT_NEXT(this), NULL, &is_item
     );
     if (next_higher) {
-        SPRepr *repr=SP_OBJECT_REPR(this);
-        SPRepr *ref=SP_OBJECT_REPR(next_higher);
+        Inkscape::XML::Node *repr=SP_OBJECT_REPR(this);
+        Inkscape::XML::Node *ref=SP_OBJECT_REPR(next_higher);
         sp_repr_change_order(sp_repr_parent(repr), repr, ref);
     }
 }
@@ -259,8 +268,8 @@ void SPItem::lowerOne() {
     );
     if (next_lower) {
         ++next_lower;
-        SPRepr *repr=SP_OBJECT_REPR(this);
-        SPRepr *ref=( next_lower ? SP_OBJECT_REPR(&*next_lower) : NULL );
+        Inkscape::XML::Node *repr=SP_OBJECT_REPR(this);
+        Inkscape::XML::Node *ref=( next_lower ? SP_OBJECT_REPR(&*next_lower) : NULL );
         sp_repr_change_order(sp_repr_parent(repr), repr, ref);
     }
 }
@@ -279,14 +288,14 @@ void SPItem::lowerToBottom() {
     );
     if (bottom) {
         ++bottom;
-        SPRepr *repr=SP_OBJECT_REPR(this);
-        SPRepr *ref=( bottom ? SP_OBJECT_REPR(&*bottom) : NULL );
+        Inkscape::XML::Node *repr=SP_OBJECT_REPR(this);
+        Inkscape::XML::Node *ref=( bottom ? SP_OBJECT_REPR(&*bottom) : NULL );
         sp_repr_change_order(sp_repr_parent(repr), repr, ref);
     }
 }
 
 static void
-sp_item_build(SPObject *object, SPDocument *document, SPRepr *repr)
+sp_item_build(SPObject *object, SPDocument *document, Inkscape::XML::Node *repr)
 {
     sp_object_read_attr(object, "style");
     sp_object_read_attr(object, "transform");
@@ -496,8 +505,8 @@ sp_item_update(SPObject *object, SPCtx *ctx, guint flags)
     }
 }
 
-static SPRepr *
-sp_item_write(SPObject *object, SPRepr *repr, guint flags)
+static Inkscape::XML::Node *
+sp_item_write(SPObject *const object, Inkscape::XML::Node *repr, guint flags)
 {
     SPItem *item = SP_ITEM(object);
 
@@ -508,27 +517,42 @@ sp_item_write(SPObject *object, SPRepr *repr, guint flags)
         sp_repr_set_attr(repr, "transform", NULL);
     }
 
-    if (SP_OBJECT_PARENT(object)) {
-        gchar *s = NULL;
-        if (SP_IS_ITEM(SP_OBJECT_PARENT(object))) { // items have style, figure out the difference
-            s = sp_style_write_difference (SP_OBJECT_STYLE(object), SP_OBJECT_STYLE(SP_OBJECT_PARENT(object)));
-        } else { // e.g. the parent may be styleless defs
-            s = sp_style_write_string (SP_OBJECT_STYLE(object), SP_STYLE_FLAG_IFSET);
-        }
-        if (s) {
-            sp_repr_set_attr(repr, "style", (*s) ? s : NULL);
+    SPObject const *const parent = SP_OBJECT_PARENT(object);
+    /* todo: Can someone please document why this is conditional on having a parent?
+       The only parentless thing I can think of is the top-level <svg> element (SPRoot).
+       SPRoot is derived from SPGroup, and can have style.  I haven't looked at callers. */
+    if (parent) {
+        SPStyle const *const obj_style = SP_OBJECT_STYLE(object);
+        if (obj_style) {
+            gchar *s = sp_style_write_string(obj_style, SP_STYLE_FLAG_IFSET);
+            sp_repr_set_attr(repr, "style", ( *s ? s : NULL ));
             g_free(s);
+        } else {
+            /* I'm not sure what to do in this case.  Bug #1165868 suggests that it can arise, but
+             * the submitter doesn't know how to do so reliably.  The main two options are either
+             * leave repr's style attribute unchanged, or explicitly clear it.  Must also consider
+             * what to do with property attributes for the element; see below.
+             */
+            char const *style_str = repr->attribute("style");
+            if (!style_str) {
+                style_str = "NULL";
+            }
+            g_warning("Item's style is NULL; repr style attribute is %s", style_str);
         }
-        // Since differencing with parent may remove some of the properties in style=, this may
-        // unexpectedly reveal (and enable) the css attrs of the same names if they are present
-        // (they are lower priority than style= but are read if there's no corresponding inline
-        // property). So here we unset any style attrs that correspond to our object's
-        // SPStyle. This means all properties will end up in style= and the attrs will be gone.
+
+        /* We treat object->style as authoritative.  Its effects have been written to the style
+         * attribute above; any properties that are unset we take to be deliberately unset (e.g. so
+         * that clones can override the property).
+         *
+         * Note that the below has an undesirable consequence of changing the appearance on
+         * renderers that lack CSS support (e.g. SVG tiny); possibly we should write property
+         * attributes instead of a style attribute.
+         */
         sp_style_unset_property_attrs (object);
     }
 
     if (flags & SP_OBJECT_WRITE_EXT) {
-        sp_repr_set_attr(repr, "sodipodi:insensitive", item->sensitive ? NULL : "true");
+        sp_repr_set_attr(repr, "sodipodi:insensitive", ( item->sensitive ? NULL : "true" ));
     }
 
     if (((SPObjectClass *) (parent_class))->write) {
@@ -650,6 +674,11 @@ sp_item_private_description(SPItem *item)
     return g_strdup(_("Object"));
 }
 
+/**
+ * Returns a string suitable for status bar, formatted in pango markup language.
+ *
+ * Must be freed by caller.
+ */
 gchar *
 sp_item_description(SPItem *item)
 {
@@ -757,6 +786,81 @@ sp_item_invoke_hide(SPItem *item, unsigned key)
     }
 }
 
+// Adjusters
+
+void
+sp_item_adjust_pattern (SPItem *item, NR::Matrix const &postmul, bool set)
+{
+    SPStyle *style = SP_OBJECT_STYLE (item);
+
+    if (style && (style->fill.type == SP_PAINT_TYPE_PAINTSERVER)) { 
+        SPObject *server = SP_OBJECT_STYLE_FILL_SERVER (item);
+        if (SP_IS_PATTERN (server)) {
+            SPPattern *pattern = sp_pattern_clone_if_necessary (item, SP_PATTERN (server), "fill");
+            sp_pattern_transform_multiply (pattern, postmul, set);
+        }
+    }
+
+    if (style && (style->stroke.type == SP_PAINT_TYPE_PAINTSERVER)) { 
+        SPObject *server = SP_OBJECT_STYLE_STROKE_SERVER (item);
+        if (SP_IS_PATTERN (server)) {
+            SPPattern *pattern = sp_pattern_clone_if_necessary (item, SP_PATTERN (server), "stroke");
+            sp_pattern_transform_multiply (pattern, postmul, set);
+        }
+    }
+
+}
+
+void
+sp_item_adjust_gradient (SPItem *item, NR::Matrix const &postmul, bool set)
+{
+    SPStyle *style = SP_OBJECT_STYLE (item);
+
+    if (style && (style->fill.type == SP_PAINT_TYPE_PAINTSERVER)) {
+        SPObject *server = SP_OBJECT_STYLE_FILL_SERVER(item);
+        if (SP_IS_GRADIENT (server)) {
+
+            // Bbox units for a gradient are generally a bad idea because with them, you cannot
+            // preserve the relative position of the object and its gradient after rotation or
+            // skew. So now we convert them to userspace units which are easy to keep in sync just
+            // by adding the object's transform to gradientTransform.  FIXME: convert back to bbox
+            // units after transforming with the item, so as to preserve the original units.
+            SPGradient *gradient = sp_gradient_convert_to_userspace (SP_GRADIENT (server), item, "fill");
+
+            sp_gradient_transform_multiply (gradient, postmul, set);
+        }
+    }
+
+    if (style && (style->stroke.type == SP_PAINT_TYPE_PAINTSERVER)) {
+        SPObject *server = SP_OBJECT_STYLE_STROKE_SERVER(item);
+        if (SP_IS_GRADIENT (server)) {
+            SPGradient *gradient = sp_gradient_convert_to_userspace (SP_GRADIENT (server), item, "stroke");
+            sp_gradient_transform_multiply (gradient, postmul, set);
+        }
+    }
+}
+
+void
+sp_item_adjust_stroke (SPItem *item, gdouble ex)
+{
+    SPStyle *style = SP_OBJECT_STYLE (item);
+
+    if (style && style->stroke.type != SP_PAINT_TYPE_NONE && !NR_DF_TEST_CLOSE (ex, 1.0, NR_EPSILON)) {
+
+        style->stroke_width.computed *= ex;
+
+        if (style->stroke_dash.n_dash != 0) {
+            int i;
+            for (i = 0; i < style->stroke_dash.n_dash; i++) {
+                style->stroke_dash.dash[i] *= ex;
+            }
+            style->stroke_dash.offset *= ex;
+        }
+
+        SP_OBJECT(item)->updateRepr();
+    }
+}
+
 /**
 Find out the inverse of previous transform of an item (from its repr)
 */
@@ -764,7 +868,7 @@ NR::Matrix
 sp_item_transform_repr (SPItem *item)
 {
     NR::Matrix t_old(NR::identity());
-    gchar const *t_attr = sp_repr_attr(SP_OBJECT_REPR(item), "transform");
+    gchar const *t_attr = SP_OBJECT_REPR(item)->attribute("transform");
     if (t_attr) {
         NR::Matrix t;
         if (sp_svg_transform_read(t_attr, &t)) {
@@ -782,7 +886,7 @@ sp_item_transform_repr (SPItem *item)
 void
 sp_item_adjust_stroke_width_recursive(SPItem *item, double expansion)
 {
-    sp_shape_adjust_stroke (item, expansion);
+    sp_item_adjust_stroke (item, expansion);
 
     for (SPObject *o = SP_OBJECT(item)->children; o != NULL; o = o->next) {
         if (SP_IS_ITEM(o))
@@ -823,9 +927,9 @@ sp_item_adjust_paint_recursive (SPItem *item, NR::Matrix advertized_transform, N
     NR::Matrix paint_delta = t_item * t_ancestors * advertized_transform * t_ancestors.inverse() * t_item.inverse();
 
     if (is_pattern)
-        sp_shape_adjust_pattern (item, paint_delta);
+        sp_item_adjust_pattern (item, paint_delta);
     else 
-        sp_shape_adjust_gradient (item, paint_delta);
+        sp_item_adjust_gradient (item, paint_delta);
 
 // Within text, we do not fork gradients, and so must not recurse to avoid double compensation
     if (item && SP_IS_TEXT(item))
@@ -844,7 +948,7 @@ sp_item_adjust_paint_recursive (SPItem *item, NR::Matrix advertized_transform, N
 A temporary wrapper for the next function accepting the NRMatrix instead of NR::Matrix
 */
 void
-sp_item_write_transform(SPItem *item, SPRepr *repr, NRMatrix const *transform, NR::Matrix const *adv)
+sp_item_write_transform(SPItem *item, Inkscape::XML::Node *repr, NRMatrix const *transform, NR::Matrix const *adv)
 {
     if (transform == NULL)
         sp_item_write_transform(item, repr, NR::identity(), adv);
@@ -859,7 +963,7 @@ stored optimized. Send _transformed_signal. Invoke _write method so that the rep
 updated with the new transform.
  */
 void
-sp_item_write_transform(SPItem *item, SPRepr *repr, NR::Matrix const &transform, NR::Matrix const *adv)
+sp_item_write_transform(SPItem *item, Inkscape::XML::Node *repr, NR::Matrix const &transform, NR::Matrix const *adv)
 {
     g_return_if_fail(item != NULL);
     g_return_if_fail(SP_IS_ITEM(item));
@@ -887,13 +991,14 @@ sp_item_write_transform(SPItem *item, SPRepr *repr, NR::Matrix const &transform,
     // recursively compensate pattern fill if it's not to be transformed 
     if (prefs_get_int_attribute("options.transform", "pattern", 1) == 0) {
         sp_item_adjust_paint_recursive (item, advertized_transform.inverse(), NR::identity(), true);
-    }
+    } // FIXME: add the same else branch as for gradients below, to convert patterns to userSpaceOnUse as well
 
     // recursively compensate gradient fill if it's not to be transformed
     if (prefs_get_int_attribute("options.transform", "gradient", 1) == 0) {
         sp_item_adjust_paint_recursive (item, advertized_transform.inverse(), NR::identity(), false);
     } else {
-        // this converts the gradient fill, if any, to userspace; we need to do it here _before_ the new transform is set, so as to use the pre-transform bbox
+        // this converts the gradient/pattern fill/stroke, if any, to userSpaceOnUse; we need to do
+        // it here _before_ the new transform is set, so as to use the pre-transform bbox
         sp_item_adjust_paint_recursive (item, NR::identity(), NR::identity(), false);
     }
 
@@ -945,25 +1050,19 @@ sp_item_set_item_transform(SPItem *item, NR::Matrix const &transform)
 
 
 /**
- * \pre \a ancestor really is an ancestor (\>=) of \a object.
+ * \pre \a ancestor really is an ancestor (\>=) of \a object, or NULL.
  *   ("Ancestor (\>=)" here includes as far as \a object itself.)
- *
- * \pre in_same_coordsys_as_anc(object, ancestor).
  */
 NR::Matrix
 i2anc_affine(SPObject const *object, SPObject const *const ancestor) {
     NR::Matrix ret(NR::identity());
-    g_return_val_if_fail(object != NULL && ancestor != NULL, ret);
-
-    while ( object != ancestor ) {
-        g_return_val_if_fail(SP_IS_ITEM(object), ret);
-        /* g_error is correct if object is <defs> or a non-SVG element type.
-         *
-         * I wonder if there are cases (perhaps involving a nested <svg> inside a foreignObject)
-         * where some different behaviour is appropriate.  We'll wait for a bug report and example
-         * document to decide what to do.
-         */
+    g_return_val_if_fail(object != NULL, ret);
 
+    /* stop at first non-renderable ancestor */
+    while ( object != ancestor && SP_IS_ITEM(object) ) {
+        if (SP_IS_ROOT(object)) {
+            ret *= SP_ROOT(object)->c2p;
+        }
         ret *= SP_ITEM(object)->transform;
         object = SP_OBJECT_PARENT(object);
     }
@@ -981,42 +1080,13 @@ NR::Matrix SPItem::getRelativeTransform(SPObject const *dest) const {
     return i2i_affine(this, dest);
 }
 
-
 /**
  * Returns the accumulated transformation of the item and all its ancestors, including root's viewport.
  * \pre (item != NULL) and SP_IS_ITEM(item).
  */
 NR::Matrix sp_item_i2doc_affine(SPItem const *item)
 {
-    g_assert(item != NULL);
-    g_assert(SP_IS_ITEM(item));
-
-    NR::Matrix ret(NR::identity());
-#ifdef NDEBUG
-    g_assert(ret.test_identity());
-#endif
-
-    /* Note that this routine may be called for items that are not members of
-    ** the root.  For example, markers are members of a <def> object.  Hence
-    ** we terminate either when reaching the top of the tree, or when a
-    ** non-item object is reached.
-    */
-    while ( NULL != SP_OBJECT_PARENT(item) && SP_IS_ITEM(SP_OBJECT_PARENT(item)) ) {
-        ret *= item->transform;
-        item = SP_ITEM(SP_OBJECT_PARENT(item));
-    }
-
-    /* Then we only do root-related stuff if we found a root item */
-    if (SP_IS_ROOT(item)) {
-        SPRoot const *root = SP_ROOT(item);
-
-        // Viewbox is relative to root's transform:
-        // http://www.w3.org/TR/SVG11/coords.html#ViewBoxAttributeEffectOnSiblingAttributes
-        ret *= root->c2p;
-        ret *= item->transform;
-    }
-
-    return ret;
+    return i2anc_affine(item, NULL);
 }
 
 /**
@@ -1042,26 +1112,6 @@ NR::Matrix sp_item_i2root_affine(SPItem const *item)
     return ret;
 }
 
-NRMatrix *sp_item_i2doc_affine(SPItem const *item, NRMatrix *affine)
-{
-    g_return_val_if_fail(item != NULL, NULL);
-    g_return_val_if_fail(SP_IS_ITEM(item), NULL);
-    g_return_val_if_fail(affine != NULL, NULL);
-
-    *affine = sp_item_i2doc_affine(item);
-    return affine;
-}
-
-NRMatrix *sp_item_i2root_affine(SPItem const *item, NRMatrix *affine)
-{
-    g_return_val_if_fail(item != NULL, NULL);
-    g_return_val_if_fail(SP_IS_ITEM(item), NULL);
-    g_return_val_if_fail(affine != NULL, NULL);
-
-    *affine = sp_item_i2root_affine(item);
-    return affine;
-}
-
 /* fixme: This is EVIL!!! */
 
 NR::Matrix sp_item_i2d_affine(SPItem const *item)
@@ -1072,11 +1122,6 @@ NR::Matrix sp_item_i2d_affine(SPItem const *item)
     NR::Matrix const ret( sp_item_i2doc_affine(item)
                           * NR::scale(1, -1)
                           * NR::translate(0, sp_document_height(SP_OBJECT_DOCUMENT(item))) );
-#ifdef NDEBUG
-    NRMatrix tst;
-    sp_item_i2d_affine(item, &tst);
-    assert_close( ret, NR::Matrix(&tst) );
-#endif
     return ret;
 }
 
@@ -1092,19 +1137,43 @@ NR::Matrix sp_item_i2r_affine(SPItem const *item)
     return ret;
 }
 
+/**
+ * Converts a matrix \a m into the desktop coords of the \a item.
+ * Will become a noop when we eliminate the coordinate flipping.
+ */
+NR::Matrix matrix_to_desktop(NR::Matrix const m, SPItem const *item)
+{
+    NR::Matrix const ret(m
+                         * NR::translate(0, -sp_document_height(SP_OBJECT_DOCUMENT(item)))
+                         * NR::scale(1, -1));
+    return ret;
+}
+
+/**
+ * Converts a matrix \a m from the desktop coords of the \a item.
+ * Will become a noop when we eliminate the coordinate flipping.
+ */
+NR::Matrix matrix_from_desktop(NR::Matrix const m, SPItem const *item)
+{
+    NR::Matrix const ret(NR::scale(1, -1)
+                         * NR::translate(0, sp_document_height(SP_OBJECT_DOCUMENT(item)))
+                         * m);
+    return ret;
+}
+
 NRMatrix *sp_item_i2d_affine(SPItem const *item, NRMatrix *affine)
 {
     g_return_val_if_fail(item != NULL, NULL);
     g_return_val_if_fail(SP_IS_ITEM(item), NULL);
     g_return_val_if_fail(affine != NULL, NULL);
 
-    sp_item_i2doc_affine(item, affine);
+    NRMatrix const i2doc(sp_item_i2doc_affine(item));
 
     NRMatrix doc2dt;
     nr_matrix_set_scale(&doc2dt, 1, -1);
     doc2dt.c[5] = sp_document_height(SP_OBJECT_DOCUMENT(item));
 
-    nr_matrix_multiply(affine, affine, &doc2dt);
+    nr_matrix_multiply(affine, &i2doc, &doc2dt);
 
     return affine;
 }
@@ -1206,6 +1275,16 @@ sp_item_repr_compare_position(SPItem *first, SPItem *second)
                                     SP_OBJECT_REPR(second));
 }
 
+SPItem *
+sp_item_first_item_child (SPObject *obj)
+{
+    for ( SPObject *iter = sp_object_first_child(obj) ; iter ; iter = SP_OBJECT_NEXT(iter)) {
+        if (SP_IS_ITEM (iter))
+            return SP_ITEM (iter);
+    }
+    return NULL;
+}
+
 
 /*
   Local Variables:
