@@ -8,8 +8,9 @@
 /*
  * Authors:
  *   Lauris Kaplinski <lauris@kaplinski.com>
+ *   bulia byak <buliabyak@users.sf.net>
  *
- * Copyright (C) 1999-2002 authors
+ * Copyright (C) 1999-2005 authors
  * Copyright (C) 2001-2002 Ximian, Inc.
  * Copyright (C) 2004 Monash University
  *
@@ -76,11 +77,11 @@ class SPItemCtx;
 struct SPItemCtx {
 	SPCtx ctx;
 	/* Item to document transformation */
-	NRMatrix i2doc;
+	NR::Matrix i2doc;
 	/* Viewport size */
 	NRRect vp;
 	/* Item to viewport transformation */
-	NRMatrix i2vp;
+	NR::Matrix i2vp;
 };
 
 /** Abstract base class for all visible shapes. */
@@ -178,8 +179,14 @@ void sp_item_invoke_hide (SPItem *item, unsigned int key);
 
 void sp_item_snappoints(SPItem const *item, SnapPointsIter p);
 
-void sp_item_write_transform(SPItem *item, SPRepr *repr, NRMatrix const *transform, NR::Matrix const *adv = NULL);
-void sp_item_write_transform(SPItem *item, SPRepr *repr, NR::Matrix const &transform, NR::Matrix const *adv = NULL);
+void sp_item_adjust_pattern (SPItem *item, /* NR::Matrix const &premul, */ NR::Matrix const &postmul, bool set = false);
+void sp_item_adjust_gradient (SPItem *item, /* NR::Matrix const &premul, */ NR::Matrix const &postmul, bool set = false);
+void sp_item_adjust_stroke (SPItem *item, gdouble ex);
+void sp_item_adjust_stroke_width_recursive (SPItem *item, gdouble ex);
+void sp_item_adjust_paint_recursive (SPItem *item, NR::Matrix advertized_transform, NR::Matrix t_ancestors, bool is_pattern);
+
+void sp_item_write_transform(SPItem *item, Inkscape::XML::Node *repr, NRMatrix const *transform, NR::Matrix const *adv = NULL);
+void sp_item_write_transform(SPItem *item, Inkscape::XML::Node *repr, NR::Matrix const &transform, NR::Matrix const *adv = NULL);
 
 void sp_item_set_item_transform(SPItem *item, NR::Matrix const &transform);
 
@@ -198,9 +205,8 @@ NR::Matrix i2i_affine(SPObject const *src, SPObject const *dest);
 NR::Matrix sp_item_i2doc_affine(SPItem const *item);
 NR::Matrix sp_item_i2root_affine(SPItem const *item);
 
-/* Old NRMatrix version (deprecated). */
-NRMatrix *sp_item_i2doc_affine (SPItem const *item, NRMatrix *transform);
-NRMatrix *sp_item_i2root_affine(SPItem const *item, NRMatrix *transform);
+NR::Matrix matrix_to_desktop (NR::Matrix m, SPItem const *item);
+NR::Matrix matrix_from_desktop (NR::Matrix m, SPItem const *item);
 
 /* fixme: - these are evil, but OK */
 
@@ -222,4 +228,6 @@ NRMatrix *sp_item_dt2i_affine(SPItem const *item, SPDesktop *dt, NRMatrix *trans
 
 int sp_item_repr_compare_position(SPItem *first, SPItem *second);
 
+SPItem *sp_item_first_item_child (SPObject *obj);
+
 #endif
