@@ -4,7 +4,12 @@
 
 #include <glib-object.h>
 #include <glib/gtypes.h>
-class SPRepr;
+namespace Inkscape {
+namespace XML {
+class Node;
+}
+}
+
 class SPLinearGradient;
 
 #define SP_TYPE_LINEARGRADIENT (sp_lineargradient_get_type())
@@ -17,10 +22,6 @@ GType sp_lineargradient_get_type();
 
 void sp_lineargradient_set_position(SPLinearGradient *lg, gdouble x1, gdouble y1, gdouble x2, gdouble y2);
 
-/** Builds flattened repr tree of gradient - i.e. no href. */
-SPRepr *sp_lineargradient_build_repr(SPLinearGradient *lg, gboolean vector);
-
-
 #endif /* !SP_LINEAR_GRADIENT_FNS_H */
 
 /*
