@@ -19,6 +19,7 @@
 #include <stdio.h>
 
 #include "libnr/nr-matrix-fns.h"
+#include "libnr/nr-matrix-ops.h"
 #include "libnr/nr-scale-matrix-ops.h"
 #include "libnr/nr-rotate-fns.h"
 #include "svg/svg.h"
@@ -42,11 +43,11 @@ struct SPMarkerView {
 static void sp_marker_class_init (SPMarkerClass *klass);
 static void sp_marker_init (SPMarker *marker);
 
-static void sp_marker_build (SPObject *object, SPDocument *document, SPRepr *repr);
+static void sp_marker_build (SPObject *object, SPDocument *document, Inkscape::XML::Node *repr);
 static void sp_marker_release (SPObject *object);
 static void sp_marker_set (SPObject *object, unsigned int key, const gchar *value);
 static void sp_marker_update (SPObject *object, SPCtx *ctx, guint flags);
-static SPRepr *sp_marker_write (SPObject *object, SPRepr *repr, guint flags);
+static Inkscape::XML::Node *sp_marker_write (SPObject *object, Inkscape::XML::Node *repr, guint flags);
 
 static NRArenaItem *sp_marker_private_show (SPItem *item, NRArena *arena, unsigned int key, unsigned int flags);
 static void sp_marker_private_hide (SPItem *item, unsigned int key);
@@ -111,7 +112,7 @@ sp_marker_init (SPMarker *marker)
 }
 
 static void
-sp_marker_build (SPObject *object, SPDocument *document, SPRepr *repr)
+sp_marker_build (SPObject *object, SPDocument *document, Inkscape::XML::Node *repr)
 {
 	SPGroup *group;
 	SPMarker *marker;
@@ -329,8 +330,8 @@ sp_marker_update (SPObject *object, SPCtx *ctx, guint flags)
 	/* Copy parent context */
 	rctx.ctx = *ctx;
 	/* Initialize tranformations */
-	nr_matrix_set_identity (&rctx.i2doc);
-	nr_matrix_set_identity (&rctx.i2vp);
+	rctx.i2doc = NR::identity();
+	rctx.i2vp = NR::identity();
 	/* Set up viewport */
 	rctx.vp.x0 = 0.0;
 	rctx.vp.y0 = 0.0;
@@ -421,7 +422,7 @@ sp_marker_update (SPObject *object, SPCtx *ctx, guint flags)
 	nr_matrix_set_translate (&q, -marker->refX.computed, -marker->refY.computed);
 	nr_matrix_multiply (&marker->c2p, &q, &marker->c2p);
 
-	nr_matrix_multiply (&rctx.i2doc, &marker->c2p, &rctx.i2doc);
+	rctx.i2doc = marker->c2p * rctx.i2doc;
 
 	/* If viewBox is set reinitialize child viewport */
 	/* Otherwise it already correct */
@@ -430,7 +431,7 @@ sp_marker_update (SPObject *object, SPCtx *ctx, guint flags)
 		rctx.vp.y0 = marker->viewBox.y0;
 		rctx.vp.x1 = marker->viewBox.x1;
 		rctx.vp.y1 = marker->viewBox.y1;
-		nr_matrix_set_identity (&rctx.i2vp);
+		rctx.i2vp = NR::identity();
 	}
 
 	/* And invoke parent method */
@@ -447,8 +448,8 @@ sp_marker_update (SPObject *object, SPCtx *ctx, guint flags)
 	}
 }
 
-static SPRepr *
-sp_marker_write (SPObject *object, SPRepr *repr, guint flags)
+static Inkscape::XML::Node *
+sp_marker_write (SPObject *object, Inkscape::XML::Node *repr, guint flags)
 {
 	SPMarker *marker;
 
@@ -497,8 +498,8 @@ sp_marker_write (SPObject *object, SPRepr *repr, guint flags)
 		sp_repr_set_attr (repr, "orient", NULL);
 	}
 	/* fixme: */
-	sp_repr_set_attr (repr, "viewBox", sp_repr_attr (object->repr, "viewBox"));
-	sp_repr_set_attr (repr, "preserveAspectRatio", sp_repr_attr (object->repr, "preserveAspectRatio"));
+	sp_repr_set_attr (repr, "viewBox", object->repr->attribute("viewBox"));
+	sp_repr_set_attr (repr, "preserveAspectRatio", object->repr->attribute("preserveAspectRatio"));
 
 	if (((SPObjectClass *) (parent_class))->write)
 		((SPObjectClass *) (parent_class))->write (object, repr, flags);
@@ -653,4 +654,3 @@ sp_marker_view_remove (SPMarker *marker, SPMarkerView *view, unsigned int destro
 	}
 	g_free (view);
 }
-
