@@ -5,8 +5,9 @@
  *
  * Authors:
  *   Lauris Kaplinski <lauris@kaplinski.com>
+ *   bulia byak <buliabyak@users.sf.net>
  *
- * Copyright (C) 1999-2002 Authors
+ * Copyright (C) 1999-2005 Authors
  * Copyright (C) 2000-2001 Ximian, Inc.
  *
  * Released under GNU GPL, read the file 'COPYING' for more information
@@ -51,12 +52,12 @@
 static void sp_namedview_class_init (SPNamedViewClass * klass);
 static void sp_namedview_init (SPNamedView * namedview);
 
-static void sp_namedview_build (SPObject * object, SPDocument * document, SPRepr * repr);
+static void sp_namedview_build (SPObject * object, SPDocument * document, Inkscape::XML::Node * repr);
 static void sp_namedview_release (SPObject *object);
 static void sp_namedview_set (SPObject *object, unsigned int key, const gchar *value);
-static void sp_namedview_child_added (SPObject * object, SPRepr * child, SPRepr * ref);
-static void sp_namedview_remove_child (SPObject *object, SPRepr *child);
-static SPRepr *sp_namedview_write (SPObject *object, SPRepr *repr, guint flags);
+static void sp_namedview_child_added (SPObject * object, Inkscape::XML::Node * child, Inkscape::XML::Node * ref);
+static void sp_namedview_remove_child (SPObject *object, Inkscape::XML::Node *child);
+static Inkscape::XML::Node *sp_namedview_write (SPObject *object, Inkscape::XML::Node *repr, guint flags);
 
 static void sp_namedview_setup_guides (SPNamedView * nv);
 
@@ -117,6 +118,7 @@ sp_namedview_init (SPNamedView * nv)
 	nv->showgrid = FALSE;
 	nv->showguides = TRUE;
 	nv->showborder = TRUE;
+        nv->showpageshadow = TRUE;
 
 	nv->guides = NULL;
 	nv->viewcount = 0;
@@ -128,7 +130,7 @@ sp_namedview_init (SPNamedView * nv)
 }
 
 static void
-sp_namedview_build (SPObject * object, SPDocument * document, SPRepr * repr)
+sp_namedview_build (SPObject * object, SPDocument * document, Inkscape::XML::Node * repr)
 {
 	SPNamedView * nv;
 	SPObjectGroup * og;
@@ -160,6 +162,7 @@ sp_namedview_build (SPObject * object, SPDocument * document, SPRepr * repr)
 	sp_object_read_attr (object, "guidehicolor");
 	sp_object_read_attr (object, "guidehiopacity");
 	sp_object_read_attr (object, "showborder");
+	sp_object_read_attr (object, "inkscape:showpageshadow");
 	sp_object_read_attr (object, "borderlayer");
 	sp_object_read_attr (object, "bordercolor");
 	sp_object_read_attr (object, "borderopacity");
@@ -220,8 +223,6 @@ sp_namedview_set (SPObject *object, unsigned int key, const gchar *value)
 {
 	SPNamedView *nv = SP_NAMEDVIEW(object);
 
-	SPUnit const * const pt = &sp_unit_get_by_id(SP_UNIT_PT);
-
 	switch (key) {
 	case SP_ATTR_VIEWONLY:
 		nv->editable = (!value);
@@ -407,6 +408,10 @@ sp_namedview_set (SPObject *object, unsigned int key, const gchar *value)
 		nv->pageshadow = value? atoi (value) : 2; // 2 is the default
 		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
+        case SP_ATTR_SHOWPAGESHADOW:
+		nv->showpageshadow = (value) ? sp_str_to_bool (value) : TRUE;
+	        object->requestModified(SP_OBJECT_MODIFIED_FLAG);
+	        break;
 	case SP_ATTR_INKSCAPE_ZOOM:
 		nv->zoom = value? g_ascii_strtod (value, NULL) : 0; // zero means not set
 		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
@@ -460,7 +465,25 @@ sp_namedview_set (SPObject *object, unsigned int key, const gchar *value)
 		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
 		break;
 	case SP_ATTR_INKSCAPE_DOCUMENT_UNITS: {
-		SPUnit const *new_unit = pt; // old documents were pretty much pt-based, so leave it as the default
+		/* The default unit if the document doesn't override this: e.g. for files saved as
+		 * `plain SVG', or non-inkscape files, or files created by an inkscape 0.40 &
+		 * earlier.
+		 *
+		 * Here we choose `px': useful for screen-destined SVGs, and fewer bug reports
+		 * about "not the same numbers as what's in the SVG file" (at least for documents
+		 * without a viewBox attribute on the root <svg> element).  Similarly, it's also
+		 * the most reliable unit (i.e. least likely to be wrong in different viewing
+		 * conditions) for viewBox-less SVG files given that it's the unit that inkscape
+		 * uses for all coordinates.
+		 *
+		 * For documents that do have a viewBox attribute on the root <svg> element, it
+		 * might be better if we used either viewBox coordinates or if we used the unit of
+		 * say the width attribute of the root <svg> element.  However, these pose problems
+		 * in that they aren't in general absolute units as currently required by
+		 * doc_units.
+		 */
+		SPUnit const *new_unit = &sp_unit_get_by_id(SP_UNIT_PX);
+
 		if (value) {
 			SPUnit const *const req_unit = sp_unit_get_by_abbreviation(value);
 			if ( req_unit == NULL ) {
@@ -490,7 +513,7 @@ sp_namedview_set (SPObject *object, unsigned int key, const gchar *value)
 }
 
 static void
-sp_namedview_child_added (SPObject * object, SPRepr * child, SPRepr * ref)
+sp_namedview_child_added (SPObject * object, Inkscape::XML::Node * child, Inkscape::XML::Node * ref)
 {
 	SPNamedView * nv;
 	SPObject * no;
@@ -502,7 +525,7 @@ sp_namedview_child_added (SPObject * object, SPRepr * child, SPRepr * ref)
 	if (((SPObjectClass *) (parent_class))->child_added)
 		(* ((SPObjectClass *) (parent_class))->child_added) (object, child, ref);
 
-	id = sp_repr_attr (child, "id");
+	id = child->attribute("id");
 	no = object->document->getObjectById(id);
 	g_assert (SP_IS_OBJECT (no));
 
@@ -533,7 +556,7 @@ sp_namedview_child_added (SPObject * object, SPRepr * child, SPRepr * ref)
 }
 
 static void
-sp_namedview_remove_child (SPObject * object, SPRepr * child)
+sp_namedview_remove_child (SPObject * object, Inkscape::XML::Node * child)
 {
 	SPNamedView * nv;
 
@@ -557,18 +580,20 @@ sp_namedview_remove_child (SPObject * object, SPRepr * child)
 		(* ((SPObjectClass *) (parent_class))->remove_child) (object, child);
 }
 
-static SPRepr *
-sp_namedview_write (SPObject *object, SPRepr *repr, guint flags)
+static Inkscape::XML::Node *
+sp_namedview_write (SPObject *object, Inkscape::XML::Node *repr, guint flags)
 {
 	SPNamedView *nv;
 
 	nv = SP_NAMEDVIEW (object);
 
-	if (flags & SP_OBJECT_WRITE_EXT) {
+	if ( ( flags & SP_OBJECT_WRITE_EXT ) &&
+	     repr != SP_OBJECT_REPR(object) )
+	{
 		if (repr) {
-			sp_repr_merge (repr, SP_OBJECT_REPR (object), "id");
+			repr->mergeFrom(SP_OBJECT_REPR (object), "id");
 		} else {
-			repr = sp_repr_duplicate (SP_OBJECT_REPR (object));
+			repr = SP_OBJECT_REPR (object)->duplicate();
 		}
 	}
 
@@ -668,7 +693,7 @@ sp_namedview_window_from_document (SPDesktop *desktop)
 void 
 sp_namedview_document_from_window (SPDesktop *desktop)
 {
-	SPRepr *view;
+	Inkscape::XML::Node *view;
 	NRRect r;
 	gint w, h, x, y;
 	GtkWindow *win = GTK_WINDOW(gtk_object_get_data (GTK_OBJECT(desktop->owner), "window"));
@@ -724,6 +749,7 @@ sp_namedview_hide(SPNamedView *const nv, SPDesktop *const desktop)
 
 	g_assert (l);
 
+	sp_canvas_item_hide(SP_CANVAS_ITEM(l->data));
 	gtk_object_unref (GTK_OBJECT (l->data));
 	nv->gridviews = g_slist_remove (nv->gridviews, l->data);
 }
@@ -766,7 +792,7 @@ sp_namedview_setup_guides (SPNamedView *nv)
 }
 
 void
-sp_namedview_toggle_guides (SPDocument *doc, SPRepr *repr)
+sp_namedview_toggle_guides (SPDocument *doc, Inkscape::XML::Node *repr)
 {
 	unsigned int v;
 	unsigned int set = sp_repr_get_boolean (repr, "showguides", &v);
@@ -786,7 +812,7 @@ sp_namedview_toggle_guides (SPDocument *doc, SPRepr *repr)
 }
 
 void
-sp_namedview_toggle_grid (SPDocument *doc, SPRepr *repr)
+sp_namedview_toggle_grid (SPDocument *doc, Inkscape::XML::Node *repr)
 {
 	unsigned int v;
 	sp_repr_get_boolean (repr, "showgrid", &v);
@@ -939,7 +965,6 @@ sp_document_namedview (SPDocument *document, const gchar *id)
 	SPObject *nv;
 
 	g_return_val_if_fail (document != NULL, NULL);
-	g_return_val_if_fail (SP_IS_DOCUMENT (document), NULL);
 
 	nv = sp_item_group_get_child_by_name ((SPGroup *) document->root, NULL, "sodipodi:namedview");
 	g_assert (nv != NULL);
