@@ -35,6 +35,7 @@ struct SPNamedView : public SPObjectGroup {
     unsigned int showgrid : 1;
     unsigned int showguides : 1;
     unsigned int showborder : 1;
+    unsigned int showpageshadow : 1;
     unsigned int borderlayer : 2;
 
     int snap_grid_bbox;
@@ -102,8 +103,8 @@ SPNamedView *sp_document_namedview(SPDocument *document, gchar const *name);
 void sp_namedview_window_from_document(SPDesktop *desktop);
 void sp_namedview_document_from_window(SPDesktop *desktop);
 
-void sp_namedview_toggle_guides(SPDocument *doc, SPRepr *repr);
-void sp_namedview_toggle_grid(SPDocument *doc, SPRepr *repr);
+void sp_namedview_toggle_guides(SPDocument *doc, Inkscape::XML::Node *repr);
+void sp_namedview_toggle_grid(SPDocument *doc, Inkscape::XML::Node *repr);
 
 #endif /* !INKSCAPE_SP_NAMEDVIEW_H */
 
