@@ -4,8 +4,9 @@
  *
  * Authors:
  *   Lauris Kaplinski <lauris@kaplinski.com>
+ *   bulia byak <buliabyak@users.sf.net>
  *
- * Copyright (C) 1999-2002 authors
+ * Copyright (C) 1999-2005 authors
  * Copyright (C) 2001-2002 Ximian, Inc.
  *
  * Released under GNU GPL, read the file 'COPYING' for more information
@@ -15,13 +16,18 @@
 #include <list>
 
 #include "helper/sp-marshal.h"
-#include "xml/sp-repr-event-vector.h"
+#include "xml/node-event-vector.h"
 #include "attributes.h"
 #include "document.h"
 #include "style.h"
 #include "sp-object-repr.h"
 #include "sp-root.h"
+#include "streq.h"
+#include "strneq.h"
 #include "xml/repr.h"
+#include "xml/node-fns.h"
+#include "debug/event.h"
+#include "debug/event-tracker.h"
 
 #include "sp-object.h"
 #include "algorithms/longest-common-suffix.h"
@@ -49,26 +55,26 @@ static void sp_object_class_init (SPObjectClass * klass);
 static void sp_object_init (SPObject * object);
 static void sp_object_finalize (GObject * object);
 
-static void sp_object_child_added (SPObject * object, SPRepr * child, SPRepr * ref);
-static void sp_object_remove_child (SPObject * object, SPRepr * child);
-static void sp_object_order_changed (SPObject * object, SPRepr * child, SPRepr * old_ref, SPRepr * new_ref);
+static void sp_object_child_added (SPObject * object, Inkscape::XML::Node * child, Inkscape::XML::Node * ref);
+static void sp_object_remove_child (SPObject * object, Inkscape::XML::Node * child);
+static void sp_object_order_changed (SPObject * object, Inkscape::XML::Node * child, Inkscape::XML::Node * old_ref, Inkscape::XML::Node * new_ref);
 
 static void sp_object_release(SPObject *object);
-static void sp_object_build (SPObject * object, SPDocument * document, SPRepr * repr);
+static void sp_object_build (SPObject * object, SPDocument * document, Inkscape::XML::Node * repr);
 
 static void sp_object_private_set (SPObject *object, unsigned int key, const gchar *value);
-static SPRepr *sp_object_private_write (SPObject *object, SPRepr *repr, guint flags);
+static Inkscape::XML::Node *sp_object_private_write (SPObject *object, Inkscape::XML::Node *repr, guint flags);
 
 /* Real handlers of repr signals */
 
-static void sp_object_repr_attr_changed (SPRepr *repr, const gchar *key, const gchar *oldval, const gchar *newval, bool is_interactive, gpointer data);
+static void sp_object_repr_attr_changed (Inkscape::XML::Node *repr, const gchar *key, const gchar *oldval, const gchar *newval, bool is_interactive, gpointer data);
 
-static void sp_object_repr_content_changed (SPRepr *repr, const gchar *oldcontent, const gchar *newcontent, gpointer data);
+static void sp_object_repr_content_changed (Inkscape::XML::Node *repr, const gchar *oldcontent, const gchar *newcontent, gpointer data);
 
-static void sp_object_repr_child_added (SPRepr *repr, SPRepr *child, SPRepr *ref, gpointer data);
-static void sp_object_repr_child_removed (SPRepr *repr, SPRepr *child, SPRepr *ref, void *data);
+static void sp_object_repr_child_added (Inkscape::XML::Node *repr, Inkscape::XML::Node *child, Inkscape::XML::Node *ref, gpointer data);
+static void sp_object_repr_child_removed (Inkscape::XML::Node *repr, Inkscape::XML::Node *child, Inkscape::XML::Node *ref, void *data);
 
-static void sp_object_repr_order_changed (SPRepr *repr, SPRepr *child, SPRepr *old, SPRepr *newer, gpointer data);
+static void sp_object_repr_order_changed (Inkscape::XML::Node *repr, Inkscape::XML::Node *child, Inkscape::XML::Node *old, Inkscape::XML::Node *newer, gpointer data);
 
 static gchar * sp_object_get_unique_id (SPObject * object, const gchar * defid);
 
@@ -76,7 +82,7 @@ guint update_in_progress = 0; // guard against update-during-update
 
 enum {RELEASE, MODIFIED, LAST_SIGNAL};
 
-SPReprEventVector object_event_vector = {
+Inkscape::XML::NodeEventVector object_event_vector = {
 	sp_object_repr_child_added,
 	sp_object_repr_child_removed,
 	sp_object_repr_attr_changed,
@@ -190,6 +196,60 @@ sp_object_finalize (GObject * object)
 	spobject->_delete_signal.~signal();
 }
 
+namespace {
+
+Inkscape::Util::SharedCStringPtr stringify(SPObject *obj) {
+	char *temp=g_strdup_printf("%p", obj);
+	Inkscape::Util::SharedCStringPtr result=Inkscape::Util::SharedCStringPtr::copy(temp);
+	g_free(temp);
+	return result;
+}
+
+Inkscape::Util::SharedCStringPtr stringify(unsigned n) {
+	char *temp=g_strdup_printf("%u", n);
+	Inkscape::Util::SharedCStringPtr result=Inkscape::Util::SharedCStringPtr::copy(temp);
+	g_free(temp);
+	return result;
+}
+
+class RefEvent : public Inkscape::Debug::Event {
+public:
+	enum Type { REF, UNREF };
+
+	RefEvent(SPObject *object, Type type)
+	: _object(stringify(object)), _refcount(G_OBJECT(object)->ref_count),
+	  _type(type)
+	{}
+
+	static Category category() { return REFCOUNT; }
+
+	Inkscape::Util::SharedCStringPtr name() const {
+		if ( _type == REF) {
+			return Inkscape::Util::SharedCStringPtr::coerce("sp-object-ref");
+		} else {
+			return Inkscape::Util::SharedCStringPtr::coerce("sp-object-unref");
+		}
+	}
+	unsigned propertyCount() const { return 2; }
+	PropertyPair property(unsigned index) const {
+		switch (index) {
+		case 0:
+			return PropertyPair("object", _object);
+		case 1:
+			return PropertyPair("refcount", stringify( _type == REF ? _refcount + 1 : _refcount - 1 ));
+		default:
+			return PropertyPair();
+		}
+	}
+
+private:
+	Inkscape::Util::SharedCStringPtr _object;
+	unsigned _refcount;
+	Type _type;
+};
+
+}
+
 /*
  * Refcounting
  *
@@ -204,6 +264,9 @@ sp_object_ref (SPObject *object, SPObject *owner)
 	g_return_val_if_fail (SP_IS_OBJECT (object), NULL);
 	g_return_val_if_fail (!owner || SP_IS_OBJECT (owner), NULL);
 
+	Inkscape::Debug::EventTracker<> tracker;
+	tracker.set<RefEvent>(object, RefEvent::REF);
+
 	g_object_ref (G_OBJECT (object));
 
 	return object;
@@ -216,6 +279,9 @@ sp_object_unref (SPObject *object, SPObject *owner)
 	g_return_val_if_fail (SP_IS_OBJECT (object), NULL);
 	g_return_val_if_fail (!owner || SP_IS_OBJECT (owner), NULL);
 
+	Inkscape::Debug::EventTracker<> tracker;
+	tracker.set<RefEvent>(object, RefEvent::UNREF);
+
 	g_object_unref (G_OBJECT (object));
 
 	return NULL;
@@ -329,9 +395,9 @@ sp_object_compare_position(SPObject const *first, SPObject const *second)
 }
 
 
-SPObject *SPObject::appendChildRepr(SPRepr *repr) {
+SPObject *SPObject::appendChildRepr(Inkscape::XML::Node *repr) {
 	if (!SP_OBJECT_IS_CLONED(this)) {
-		sp_repr_append_child(SP_OBJECT_REPR(this), repr);
+		SP_OBJECT_REPR(this)->appendChild(repr);
 		return SP_OBJECT_DOCUMENT(this)->getObjectByRepr(repr);
 	} else {
 		g_critical("Attempt to append repr as child of cloned object");
@@ -358,7 +424,7 @@ SPObject::defaultLabel() const {
 			if (id) {
 				_default_label = g_strdup_printf("#%s", id);
 			} else {
-				_default_label = g_strdup_printf("<%s>", sp_repr_name(SP_OBJECT_REPR(this)));
+				_default_label = g_strdup_printf("<%s>", SP_OBJECT_REPR(this)->name());
 			}
 		}
 		return _default_label;
@@ -407,7 +473,7 @@ void SPObject::deleteObject(bool propagate, bool propagate_descendants)
 		this->_sendDeleteSignalRecursive();
 	}
 
-	SPRepr *repr=SP_OBJECT_REPR(this);
+	Inkscape::XML::Node *repr=SP_OBJECT_REPR(this);
 	if (repr && sp_repr_parent(repr)) {
 		sp_repr_unparent(repr);
 	}
@@ -450,6 +516,8 @@ sp_object_attach_reref (SPObject *parent, SPObject *object, SPObject *prev)
 	if (!next) {
 		parent->_last_child = object;
 	}
+    if (!object->xml_space.set)
+        object->xml_space.value = parent->xml_space.value;
 }
 
 void sp_object_reorder(SPObject *object, SPObject *prev) {
@@ -534,12 +602,14 @@ void sp_object_detach_unref (SPObject *parent, SPObject *object)
 	sp_object_unref(object, parent);
 }
 
-SPObject *sp_object_get_child_by_repr(SPObject *object, SPRepr *repr)
+SPObject *sp_object_get_child_by_repr(SPObject *object, Inkscape::XML::Node *repr)
 {
 	g_return_val_if_fail(object != NULL, NULL);
 	g_return_val_if_fail(SP_IS_OBJECT(object), NULL);
 	g_return_val_if_fail(repr != NULL, NULL);
 
+    if (object->_last_child && SP_OBJECT_REPR(object->_last_child) == repr)
+        return object->_last_child;   // optimization for common scenario
 	for ( SPObject *child = object->children ; child ; child = child->next ) {
 		if ( SP_OBJECT_REPR(child) == repr ) {
 			return child;
@@ -549,7 +619,7 @@ SPObject *sp_object_get_child_by_repr(SPObject *object, SPRepr *repr)
 	return NULL;
 }
 
-static void sp_object_child_added (SPObject * object, SPRepr * child, SPRepr * ref)
+static void sp_object_child_added (SPObject * object, Inkscape::XML::Node * child, Inkscape::XML::Node * ref)
 {
 	GType type = sp_repr_type_lookup(child);
 	if (!type) {
@@ -562,7 +632,7 @@ static void sp_object_child_added (SPObject * object, SPRepr * child, SPRepr * r
 	sp_object_invoke_build(ochild, object->document, child, SP_OBJECT_IS_CLONED(object));
 }
 
-static void sp_object_remove_child (SPObject * object, SPRepr * child)
+static void sp_object_remove_child (SPObject * object, Inkscape::XML::Node * child)
 {
 	debug("id=%x, typename=%s", object, g_type_name_from_instance((GTypeInstance*)object));
 	SPObject *ochild = sp_object_get_child_by_repr(object, child);
@@ -570,8 +640,8 @@ static void sp_object_remove_child (SPObject * object, SPRepr * child)
 	sp_object_detach_unref(object, ochild);
 }
 
-static void sp_object_order_changed (SPObject * object, SPRepr * child, SPRepr * old_ref,
-				     SPRepr * new_ref)
+static void sp_object_order_changed (SPObject * object, Inkscape::XML::Node * child, Inkscape::XML::Node * old_ref,
+				     Inkscape::XML::Node * new_ref)
 {
 	SPObject *ochild = sp_object_get_child_by_repr(object, child);
 	g_return_if_fail(ochild != NULL);
@@ -592,7 +662,7 @@ static void sp_object_release(SPObject *object)
  */
 
 static void
-sp_object_build (SPObject * object, SPDocument * document, SPRepr * repr)
+sp_object_build (SPObject * object, SPDocument * document, Inkscape::XML::Node * repr)
 {
 	/* Nothing specific here */
 	debug("id=%x, typename=%s", object, g_type_name_from_instance((GTypeInstance*)object));
@@ -601,7 +671,7 @@ sp_object_build (SPObject * object, SPDocument * document, SPRepr * repr)
 	sp_object_read_attr (object, "inkscape:label");
 	sp_object_read_attr (object, "inkscape:collect");
 
-        for (SPRepr *rchild = repr->firstChild() ; rchild != NULL; rchild = rchild->next()) {
+        for (Inkscape::XML::Node *rchild = repr->firstChild() ; rchild != NULL; rchild = rchild->next()) {
 		GType type = sp_repr_type_lookup (rchild);
 		if (!type) {
 			continue;
@@ -613,14 +683,13 @@ sp_object_build (SPObject * object, SPDocument * document, SPRepr * repr)
 }
 
 void
-sp_object_invoke_build (SPObject * object, SPDocument * document, SPRepr * repr, unsigned int cloned)
+sp_object_invoke_build (SPObject * object, SPDocument * document, Inkscape::XML::Node * repr, unsigned int cloned)
 {
 	debug("id=%x, typename=%s", object, g_type_name_from_instance((GTypeInstance*)object));
 
 	g_assert (object != NULL);
 	g_assert (SP_IS_OBJECT (object));
 	g_assert (document != NULL);
-	g_assert (SP_IS_DOCUMENT (document));
 	g_assert (repr != NULL);
 
 	g_assert (object->document == NULL);
@@ -637,9 +706,9 @@ sp_object_invoke_build (SPObject * object, SPDocument * document, SPRepr * repr,
 	if (!SP_OBJECT_IS_CLONED (object)) {
 		object->document->bindObjectToRepr(object->repr, object);
 
-		if ( object->repr->type() == SP_XML_ELEMENT_NODE ) {
+		if (Inkscape::XML::id_permitted(object->repr)) {
 			/* If we are not cloned, force unique id */
-			const gchar *id = sp_repr_attr (object->repr, "id");
+			const gchar *id = object->repr->attribute("id");
 			gchar *realid = sp_object_get_unique_id (object, id);
 			g_assert (realid != NULL);
 
@@ -713,7 +782,7 @@ sp_object_invoke_release (SPObject *object)
 }
 
 static void
-sp_object_repr_child_added (SPRepr *repr, SPRepr *child, SPRepr *ref, gpointer data)
+sp_object_repr_child_added (Inkscape::XML::Node *repr, Inkscape::XML::Node *child, Inkscape::XML::Node *ref, gpointer data)
 {
 	SPObject *object = SP_OBJECT (data);
 
@@ -722,7 +791,7 @@ sp_object_repr_child_added (SPRepr *repr, SPRepr *child, SPRepr *ref, gpointer d
 }
 
 static void
-sp_object_repr_child_removed (SPRepr *repr, SPRepr *child, SPRepr *ref, gpointer data)
+sp_object_repr_child_removed (Inkscape::XML::Node *repr, Inkscape::XML::Node *child, Inkscape::XML::Node *ref, gpointer data)
 {
 	SPObject *object = SP_OBJECT (data);
 
@@ -734,7 +803,7 @@ sp_object_repr_child_removed (SPRepr *repr, SPRepr *child, SPRepr *ref, gpointer
 /* fixme: */
 
 static void
-sp_object_repr_order_changed (SPRepr * repr, SPRepr * child, SPRepr * old, SPRepr * newer, gpointer data)
+sp_object_repr_order_changed (Inkscape::XML::Node * repr, Inkscape::XML::Node * child, Inkscape::XML::Node * old, Inkscape::XML::Node * newer, gpointer data)
 {
 	SPObject * object = SP_OBJECT (data);
 
@@ -746,15 +815,16 @@ sp_object_repr_order_changed (SPRepr * repr, SPRepr * child, SPRepr * old, SPRep
 static void
 sp_object_private_set (SPObject *object, unsigned int key, const gchar *value)
 {
-	g_assert (SP_IS_DOCUMENT (object->document));
 	g_assert (key != SP_ATTR_INVALID);
 
 	switch (key) {
 	case SP_ATTR_ID:
-		if ( !SP_OBJECT_IS_CLONED (object) && object->repr->type() == SP_XML_ELEMENT_NODE ) {
+		if ( !SP_OBJECT_IS_CLONED (object) && object->repr->type() == Inkscape::XML::ELEMENT_NODE ) {
+			SPDocument *document=object->document;
 			SPObject *conflict=NULL;
+
 			if (value) {
-				conflict = object->document->getObjectById((const char *)value);
+				conflict = document->getObjectById((const char *)value);
 			}
 			if ( conflict && conflict != object ) {
 				sp_object_ref(conflict, NULL);
@@ -766,17 +836,14 @@ sp_object_private_set (SPObject *object, unsigned int key, const gchar *value)
 			}
 
 			if (object->id) {
-				object->document->bindObjectToId(object->id, NULL);
+				document->bindObjectToId(object->id, NULL);
 				g_free (object->id);
-			} else {
-				g_message("id binding restored on bound object %s", value);
 			}
 
 			if (value) {
 				object->id = g_strdup ((const char*)value);
-				object->document->bindObjectToId(object->id, object);
+				document->bindObjectToId(object->id, object);
 			} else {
-				g_warning("id binding cleared on bound object %s", object->id);
 				object->id = NULL;
 			}
 
@@ -838,20 +905,19 @@ sp_object_read_attr (SPObject *object, const gchar *key)
 	g_assert (SP_IS_OBJECT (object));
 	g_assert (key != NULL);
 
-	g_assert (SP_IS_DOCUMENT (object->document));
 	g_assert (object->repr != NULL);
 
 	unsigned int keyid = sp_attribute_lookup (key);
 	if (keyid != SP_ATTR_INVALID) {
 		/* Retrieve the 'key' attribute from the object's XML representation */
-		const gchar *value = sp_repr_attr (object->repr, key);
+		const gchar *value = object->repr->attribute(key);
 
 		sp_object_set (object, keyid, value);
 	}
 }
 
 static void
-sp_object_repr_attr_changed (SPRepr *repr, const gchar *key, const gchar *oldval, const gchar *newval, bool is_interactive, gpointer data)
+sp_object_repr_attr_changed (Inkscape::XML::Node *repr, const gchar *key, const gchar *oldval, const gchar *newval, bool is_interactive, gpointer data)
 {
 	SPObject *object = SP_OBJECT (data);
 
@@ -865,7 +931,7 @@ sp_object_repr_attr_changed (SPRepr *repr, const gchar *key, const gchar *oldval
 }
 
 static void
-sp_object_repr_content_changed (SPRepr *repr, const gchar *oldcontent, const gchar *newcontent, gpointer data)
+sp_object_repr_content_changed (Inkscape::XML::Node *repr, const gchar *oldcontent, const gchar *newcontent, gpointer data)
 {
 	SPObject * object = SP_OBJECT (data);
 
@@ -887,11 +953,11 @@ sp_xml_get_space_string(unsigned int space)
 	}
 }
 
-static SPRepr *
-sp_object_private_write (SPObject *object, SPRepr *repr, guint flags)
+static Inkscape::XML::Node *
+sp_object_private_write (SPObject *object, Inkscape::XML::Node *repr, guint flags)
 {
 	if (!repr && (flags & SP_OBJECT_WRITE_BUILD)) {
-		repr = sp_repr_duplicate (SP_OBJECT_REPR (object));
+		repr = SP_OBJECT_REPR (object)->duplicate();
 		if (!( flags & SP_OBJECT_WRITE_EXT )) {
 			sp_repr_set_attr(repr, "inkscape:collect", NULL);
 		}
@@ -916,9 +982,9 @@ sp_object_private_write (SPObject *object, SPRepr *repr, guint flags)
 	return repr;
 }
 
-SPRepr *SPObject::updateRepr(unsigned int flags) {
+Inkscape::XML::Node *SPObject::updateRepr(unsigned int flags) {
 	if (!SP_OBJECT_IS_CLONED(this)) {
-		SPRepr *repr=SP_OBJECT_REPR(this);
+		Inkscape::XML::Node *repr=SP_OBJECT_REPR(this);
 		if (repr) {
 			return updateRepr(repr, flags);
 		} else {
@@ -931,7 +997,7 @@ SPRepr *SPObject::updateRepr(unsigned int flags) {
 	}
 }
 
-SPRepr *SPObject::updateRepr(SPRepr *repr, unsigned int flags) {
+Inkscape::XML::Node *SPObject::updateRepr(Inkscape::XML::Node *repr, unsigned int flags) {
 	if (SP_OBJECT_IS_CLONED(this)) {
 		/* cloned objects have no repr */
 		return NULL;
@@ -945,11 +1011,11 @@ SPRepr *SPObject::updateRepr(SPRepr *repr, unsigned int flags) {
 		g_warning ("Class %s does not implement ::write", G_OBJECT_TYPE_NAME (this));
 		if (!repr) {
 			if (flags & SP_OBJECT_WRITE_BUILD) {
-				repr = sp_repr_duplicate (SP_OBJECT_REPR (this));
+				repr = SP_OBJECT_REPR (this)->duplicate();
 			}
 			/* fixme: else probably error (Lauris) */
 		} else {
-			sp_repr_merge (repr, SP_OBJECT_REPR (this), "id");
+			repr->mergeFrom(SP_OBJECT_REPR (this), "id");
 		}
 		return repr;
 	}
@@ -1113,7 +1179,7 @@ sp_object_tagName_get (const SPObject *object, SPException *ex)
 	}
 
 	/* fixme: Exception if object is NULL? */
-	return sp_repr_name (object->repr);
+	return object->repr->name();
 }
 
 const gchar *
@@ -1125,7 +1191,7 @@ sp_object_getAttribute (const SPObject *object, const gchar *key, SPException *e
 	}
 
 	/* fixme: Exception if object is NULL? */
-	return (const gchar *) sp_repr_attr (object->repr, key);
+	return (const gchar *) object->repr->attribute(key);
 }
 
 void
@@ -1160,11 +1226,10 @@ sp_object_get_unique_id (SPObject * object, const gchar * id)
 	static unsigned long count = 0;
 
 	g_assert (SP_IS_OBJECT (object));
-	g_assert (SP_IS_DOCUMENT (object->document));
 
 	count ++;
 
-	const gchar *name = sp_repr_name (object->repr);
+	const gchar *name = object->repr->name();
 	g_assert (name != NULL);
 
 	const gchar *local = strchr (name, ':');
@@ -1193,19 +1258,31 @@ sp_object_get_unique_id (SPObject * object, const gchar * id)
 
 /* Style */
 
-const gchar *
-sp_object_get_style_property (SPObject *object, const gchar *key, const gchar *def)
+gchar const *
+sp_object_get_style_property(SPObject const *object, gchar const *key, gchar const *def)
 {
-	/* fixme: Use proper CSS parsing.  The current version is buggy in a number of
-	   situations where key is a substring of the style string other than as a property
-	   name (including where key is a substring of a property name), and is
-	   also buggy in its handling of inheritance for properties that aren't inherited
-	   by default. */
+	/* fixme: Use proper CSS parsing.  The current version is buggy in a number of situations
+	 * where key is a substring of the style string other than as a property name (including
+	 * where key is a substring of a property name), and is also buggy in its handling of
+	 * inheritance for properties that aren't inherited by default.  It also doesn't allow for
+	 * the case where the property is specified but with an invalid value (in which case I
+	 * believe the CSS2 error-handling behaviour applies, viz. behave as if the property hadn't
+	 * been specified).  Also, the current code doesn't use CRSelEng stuff to take a value from
+	 * stylesheets.  Also, we aren't setting any hooks to force an update for changes
+	 * in any of the inputs (i.e. in any of the elements that this function queries).
+	 *
+	 * Given that the default value for a property depends on what property it is (e.g.
+	 * whether to inherit or not), and given the above comment about ignoring invalid values,
+	 * and that the repr parent isn't necessarily the right element to inherit from
+	 * (e.g. maybe we need to inherit from the referencing <use> element instead),
+	 * we should probably make the caller responsible for ascending the repr tree as
+	 * necessary.
+	 */
 	g_return_val_if_fail (object != NULL, NULL);
 	g_return_val_if_fail (SP_IS_OBJECT (object), NULL);
 	g_return_val_if_fail (key != NULL, NULL);
 
-	const gchar *style = sp_repr_attr (object->repr, "style");
+	const gchar *style = object->repr->attribute("style");
 	if (style) {
 		size_t const len = strlen(key);
 		char const *p;
@@ -1216,11 +1293,18 @@ sp_object_get_style_property (SPObject *object, const gchar *key, const gchar *d
 			while ((*p <= ' ') && *p) p++;
 			if (*p++ != ':') break;
 			while ((*p <= ' ') && *p) p++;
-			if (*p) return p;
+			size_t const inherit_len = sizeof("inherit") - 1;
+			if (*p
+			    && !(strneq(p, "inherit", inherit_len)
+				 && (p[inherit_len] == '\0'
+				     || p[inherit_len] == ';'
+				     || g_ascii_isspace(p[inherit_len])))) {
+				return p;
+			}
 		}
 	}
-	const gchar *val = sp_repr_attr (object->repr, key);
-	if (val) {
+	const gchar *val = object->repr->attribute(key);
+	if (val && !streq(val, "inherit")) {
 		return val;
 	}
 	if (object->parent) {
@@ -1230,10 +1314,22 @@ sp_object_get_style_property (SPObject *object, const gchar *key, const gchar *d
 	return def;
 }
 
-SPVersion
+void SPObject::_requireSVGVersion(Inkscape::Version version) {
+	for ( SPObject::ParentIterator iter=this ; iter ; ++iter ) {
+		SPObject *object=iter;
+		if (SP_IS_ROOT(object)) {
+			SPRoot *root=SP_ROOT(object);
+			if ( root->version.svg < version ) {
+				root->version.svg = version;
+			}
+		}
+	}
+}
+
+Inkscape::Version
 sp_object_get_sodipodi_version (SPObject *object)
 {
-	static const SPVersion zero_version = { 0, 0 };
+	static const Inkscape::Version zero_version(0, 0);
 
 	while (object) {
 		if (SP_IS_ROOT (object)) {
@@ -1244,3 +1340,14 @@ sp_object_get_sodipodi_version (SPObject *object)
 	
 	return zero_version;
 }
+
+SPObject *
+sp_object_prev(SPObject *child)
+{
+    SPObject *parent = SP_OBJECT_PARENT(child);
+    for ( SPObject *i = sp_object_first_child(parent); i; i = SP_OBJECT_NEXT(i) ) {
+        if (SP_OBJECT_NEXT(i) == child)
+            return i;
+    }
+    return NULL;
+}
