@@ -5,6 +5,7 @@
  *
  * Author:
  *   Lauris Kaplinski <lauris@kaplinski.com>
+ *   bulia byak <buliabyak@users.sf.net>
  *
  * Copyright (C) 2002 Lauris Kaplinski
  *
@@ -64,10 +65,10 @@ struct SPPatPainter {
 static void sp_pattern_class_init (SPPatternClass *klass);
 static void sp_pattern_init (SPPattern *gr);
 
-static void sp_pattern_build (SPObject *object, SPDocument *document, SPRepr *repr);
+static void sp_pattern_build (SPObject *object, SPDocument *document, Inkscape::XML::Node *repr);
 static void sp_pattern_release (SPObject *object);
 static void sp_pattern_set (SPObject *object, unsigned int key, const gchar *value);
-static void sp_pattern_child_added (SPObject *object, SPRepr *child, SPRepr *ref);
+static void sp_pattern_child_added (SPObject *object, Inkscape::XML::Node *child, Inkscape::XML::Node *ref);
 static void sp_pattern_update (SPObject *object, SPCtx *ctx, unsigned int flags);
 static void sp_pattern_modified (SPObject *object, unsigned int flags);
 
@@ -137,7 +138,7 @@ sp_pattern_init (SPPattern *pat)
 	pat->patternContentUnits = SP_PATTERN_UNITS_USERSPACEONUSE;
 	pat->patternContentUnits_set = FALSE;
 
-	nr_matrix_set_identity (&pat->patternTransform);
+	pat->patternTransform = NR::identity();
 	pat->patternTransform_set = FALSE;
 
 	sp_svg_length_unset (&pat->x, SP_SVG_UNIT_NONE, 0.0, 0.0);
@@ -149,7 +150,7 @@ sp_pattern_init (SPPattern *pat)
 }
 
 static void
-sp_pattern_build (SPObject *object, SPDocument *document, SPRepr *repr)
+sp_pattern_build (SPObject *object, SPDocument *document, Inkscape::XML::Node *repr)
 {
 	if (((SPObjectClass *) pattern_parent_class)->build)
 		(* ((SPObjectClass *) pattern_parent_class)->build) (object, document, repr);
@@ -230,7 +231,7 @@ sp_pattern_set (SPObject *object, unsigned int key, const gchar *value)
 			pat->patternTransform = t;
 			pat->patternTransform_set = TRUE;
 		} else {
-			nr_matrix_set_identity (&pat->patternTransform);
+			pat->patternTransform = NR::identity();
 			pat->patternTransform_set = FALSE;
 		}
 		object->requestModified(SP_OBJECT_MODIFIED_FLAG);
@@ -321,7 +322,7 @@ sp_pattern_set (SPObject *object, unsigned int key, const gchar *value)
 }
 
 static void
-sp_pattern_child_added (SPObject *object, SPRepr *child, SPRepr *ref)
+sp_pattern_child_added (SPObject *object, Inkscape::XML::Node *child, Inkscape::XML::Node *ref)
 {
 	SPPattern *pat = SP_PATTERN (object);
 
@@ -449,16 +450,16 @@ SPPattern *
 pattern_chain (SPPattern *pattern)
 {
 	SPDocument *document = SP_OBJECT_DOCUMENT (pattern);
-	SPRepr *defsrepr = SP_OBJECT_REPR (SP_DOCUMENT_DEFS (document));
+	Inkscape::XML::Node *defsrepr = SP_OBJECT_REPR (SP_DOCUMENT_DEFS (document));
 
-	SPRepr *repr = sp_repr_new ("svg:pattern");
+	Inkscape::XML::Node *repr = sp_repr_new ("svg:pattern");
 	sp_repr_set_attr(repr, "inkscape:collect", "always");
-	gchar *parent_ref = g_strconcat ("#", sp_repr_attr(SP_OBJECT_REPR(pattern), "id"), NULL);
+	gchar *parent_ref = g_strconcat ("#", SP_OBJECT_REPR(pattern)->attribute("id"), NULL);
 	sp_repr_set_attr (repr, "xlink:href",  parent_ref);
 	g_free (parent_ref);
 
 	sp_repr_add_child (defsrepr, repr, NULL);
-	const gchar *child_id = sp_repr_attr(repr, "id");
+	const gchar *child_id = repr->attribute("id");
 	SPObject *child = document->getObjectById(child_id);
 	g_assert (SP_IS_PATTERN (child));
 
@@ -470,7 +471,7 @@ sp_pattern_clone_if_necessary (SPItem *item, SPPattern *pattern, const gchar *pr
 {
 	if (pattern_users(pattern) > 1) {
 		pattern = pattern_chain (pattern);
-		gchar *href = g_strconcat ("url(#", sp_repr_attr (SP_OBJECT_REPR (pattern), "id"), ")", NULL);
+		gchar *href = g_strconcat ("url(#", SP_OBJECT_REPR (pattern)->attribute("id"), ")", NULL);
 
 		SPCSSAttr *css = sp_repr_css_attr_new ();
 		sp_repr_css_set_property (css, property, href);
@@ -484,18 +485,18 @@ sp_pattern_transform_multiply (SPPattern *pattern, NR::Matrix postmul, bool set)
 {
 	// this formula is for a different interpretation of pattern transforms as described in (*) in sp-pattern.cpp
 	// for it to work, we also need    sp_object_read_attr (SP_OBJECT (item), "transform");
-	//pattern->patternTransform = premul * item->transform * NR::Matrix(pattern->patternTransform) * item->transform.inverse() * postmul;
+	//pattern->patternTransform = premul * item->transform * pattern->patternTransform * item->transform.inverse() * postmul;
 
 	// otherwise the formula is much simpler
 	if (set) {
 		pattern->patternTransform = postmul;
 	} else {
-		pattern->patternTransform = NR::Matrix(pattern_patternTransform(pattern)) * postmul;
+		pattern->patternTransform = pattern_patternTransform(pattern) * postmul;
 	}
 	pattern->patternTransform_set = TRUE;
 
 	gchar c[256];
-	if (sp_svg_transform_write(c, 256, &(pattern->patternTransform))) {
+	if (sp_svg_transform_write(c, 256, pattern->patternTransform)) {
 		sp_repr_set_attr(SP_OBJECT_REPR(pattern), "patternTransform", c);
 	} else {
 		sp_repr_set_attr(SP_OBJECT_REPR(pattern), "patternTransform", NULL);
@@ -505,9 +506,9 @@ sp_pattern_transform_multiply (SPPattern *pattern, NR::Matrix postmul, bool set)
 const gchar *
 pattern_tile (GSList *reprs, NR::Rect bounds, SPDocument *document, NR::Matrix transform, NR::Matrix move)
 {
-	SPRepr *defsrepr = SP_OBJECT_REPR (SP_DOCUMENT_DEFS (document));
+	Inkscape::XML::Node *defsrepr = SP_OBJECT_REPR (SP_DOCUMENT_DEFS (document));
 
-	SPRepr *repr = sp_repr_new ("svg:pattern");
+	Inkscape::XML::Node *repr = sp_repr_new ("svg:pattern");
 	sp_repr_set_attr (repr, "patternUnits", "userSpaceOnUse");
 	sp_repr_set_double (repr, "width", bounds.extent(NR::X));
 	sp_repr_set_double (repr, "height", bounds.extent(NR::Y));
@@ -520,18 +521,18 @@ pattern_tile (GSList *reprs, NR::Rect bounds, SPDocument *document, NR::Matrix t
 	}
 
 
-	sp_repr_append_child (defsrepr, repr);
-	const gchar *pat_id = sp_repr_attr (repr, "id");
+	defsrepr->appendChild(repr);
+	const gchar *pat_id = repr->attribute("id");
 	SPObject *pat_object = document->getObjectById(pat_id);
 
 	gchar m[256];
 
 	for (GSList *i = reprs; i != NULL; i = i->next) {
-		SPRepr *dup = sp_repr_duplicate (((SPRepr *) i->data));
+		Inkscape::XML::Node *dup = ((Inkscape::XML::Node *) i->data)->duplicate();
 		sp_repr_add_child(SP_OBJECT_REPR(pat_object), dup, NULL);
 
 		NR::Matrix dup_transform;
-		if (!sp_svg_transform_read (sp_repr_attr (dup, "transform"), &dup_transform)) 
+		if (!sp_svg_transform_read (dup->attribute("transform"), &dup_transform)) 
 			dup_transform = NR::identity();
 		dup_transform *= move;
 
@@ -581,13 +582,13 @@ guint pattern_patternContentUnits (SPPattern *pat)
 	return pat->patternContentUnits;
 }
 
-NRMatrix *pattern_patternTransform (SPPattern *pat)
+NR::Matrix const &pattern_patternTransform(SPPattern const *pat)
 {
-	for (SPPattern *pat_i = pat; pat_i != NULL; pat_i = pat_i->ref ? pat_i->ref->getObject() : NULL) {
+	for (SPPattern const *pat_i = pat; pat_i != NULL; pat_i = pat_i->ref ? pat_i->ref->getObject() : NULL) {
 		if (pat_i->patternTransform_set)
-			return &(pat_i->patternTransform);
+			return pat_i->patternTransform;
 	}
-	return &(pat->patternTransform);
+	return pat->patternTransform;
 }
 
 gdouble pattern_x (SPPattern *pat)
@@ -662,7 +663,7 @@ sp_pattern_painter_new (SPPaintServer *ps, NR::Matrix const &full_transform, NR:
 		NR::Matrix bbox2user (bbox->x1 - bbox->x0, 0.0, 0.0, bbox->y1 - bbox->y0, bbox->x0, bbox->y0);
 
 		// the final patternTransform, taking into account bbox
-		NR::Matrix ps2user = NR::Matrix (pattern_patternTransform (pat)) * bbox2user;
+		NR::Matrix const ps2user(pattern_patternTransform(pat) * bbox2user);
 
 		// see (*) comment below
 		NR::Matrix ps2px = ps2user * full_transform;
@@ -685,8 +686,8 @@ sp_pattern_painter_new (SPPaintServer *ps, NR::Matrix const &full_transform, NR:
 		// So here I comply with the majority opinion, but leave my interpretation commented out below.
 		// (To get item_transform, I subtract parent from full.)
 
-		//NR::Matrix ps2px = (full_transform / parent_transform) * NR::Matrix (pattern_patternTransform (pat)) * parent_transform;
-		NR::Matrix ps2px = NR::Matrix (pattern_patternTransform (pat)) * full_transform;
+		//NR::Matrix ps2px = (full_transform / parent_transform) * pattern_patternTransform(pat) * parent_transform;
+		NR::Matrix ps2px = pattern_patternTransform(pat) * full_transform;
 
 		ps2px.copyto (&pp->ps2px);
 	}
@@ -703,7 +704,7 @@ sp_pattern_painter_new (SPPaintServer *ps, NR::Matrix const &full_transform, NR:
 		/* Problem: What to do, if we have mixed lengths and percentages? (Lauris) */
 		/* Currently we do ignore percentages at all, but that is not good (Lauris) */
 
-		NR::Matrix vb2us = vb2ps * NR::Matrix (pattern_patternTransform (pat));
+		NR::Matrix vb2us = vb2ps * pattern_patternTransform(pat);
 
 		// see (*)
 		NR::Matrix pcs2px = vb2us * full_transform;
@@ -717,14 +718,14 @@ sp_pattern_painter_new (SPPaintServer *ps, NR::Matrix const &full_transform, NR:
 			/* BBox to user coordinate system */
 			NR::Matrix bbox2user (bbox->x1 - bbox->x0, 0.0, 0.0, bbox->y1 - bbox->y0, bbox->x0, bbox->y0);
 
-			NR::Matrix pcs2user = NR::Matrix (pattern_patternTransform (pat)) * bbox2user;
+			NR::Matrix pcs2user = pattern_patternTransform(pat) * bbox2user;
 
 			// see (*)
 			pcs2px = pcs2user * full_transform;
 		} else {
 			// see (*)
-			//pcs2px = (full_transform / parent_transform) * NR::Matrix (pattern_patternTransform (pat)) * parent_transform;
-			pcs2px = NR::Matrix (pattern_patternTransform (pat)) * full_transform;
+			//pcs2px = (full_transform / parent_transform) * pattern_patternTransform(pat) * parent_transform;
+			pcs2px = pattern_patternTransform(pat) * full_transform;
 		}
 
 		pcs2px = NR::translate (pattern_x (pat), pattern_y (pat)) * pcs2px; 
@@ -905,7 +906,7 @@ sp_pat_fill (SPPainter *painter, NRPixBlock *pb)
 			double          px_y=pb->area.y0;
 			for (int j=pb->area.y0;j<pb->area.y1;j++) {
 				unsigned char* cpx=lpx;
-				double         px_x=px_x=pb->area.x0;
+				double         px_x = pb->area.x0;
 				
 				double ps_x=pp->px2ps.c[0]*px_x+pp->px2ps.c[2]*px_y+pp->px2ps.c[4];
 				double ps_y=pp->px2ps.c[1]*px_x+pp->px2ps.c[3]*px_y+pp->px2ps.c[5];
@@ -936,7 +937,7 @@ sp_pat_fill (SPPainter *painter, NRPixBlock *pb)
 			double          px_y=pb->area.y0;
 			for (int j=pb->area.y0;j<pb->area.y1;j++) {
 				unsigned char* cpx=lpx;
-				double         px_x=px_x=pb->area.x0;
+				double         px_x = pb->area.x0;
 				
 				double ps_x=pp->px2ps.c[0]*px_x+pp->px2ps.c[2]*px_y+pp->px2ps.c[4];
 				double ps_y=pp->px2ps.c[1]*px_x+pp->px2ps.c[3]*px_y+pp->px2ps.c[5];
@@ -1000,4 +1001,3 @@ sp_pat_fill (SPPainter *painter, NRPixBlock *pb)
 		}
 	}
 }
-
