@@ -3,7 +3,12 @@
 /** \file */
 
 #include <glib-object.h>
-class SPRepr;
+namespace Inkscape {
+namespace XML {
+class Node;
+}
+}
+
 class SPRadialGradient;
 
 #define SP_TYPE_RADIALGRADIENT (sp_radialgradient_get_type())
@@ -17,9 +22,6 @@ GType sp_radialgradient_get_type();
 
 void sp_radialgradient_set_position(SPRadialGradient *rg, gdouble cx, gdouble cy, gdouble fx, gdouble fy, gdouble r);
 
-/** Builds flattened repr tree of gradient - i.e. no href. */
-SPRepr *sp_radialgradient_build_repr(SPRadialGradient *lg, gboolean vector);
-
 #endif /* !SP_RADIAL_GRADIENT_FNS_H */
 
 /*
