@@ -5,6 +5,7 @@
  *
  * Authors:
  *   Lauris Kaplinski <lauris@kaplinski.com>
+ *   bulia byak <buliabyak@users.sf.net>
  *
  * Copyright (C) 1999-2002 Lauris Kaplinski
  * Copyright (C) 2000-2001 Ximian, Inc.
@@ -12,7 +13,9 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
 
 #include <math.h>
 #include <string.h>
@@ -23,6 +26,7 @@
 #include <libnr/nr-matrix-fns.h>
 #include <libnr/nr-matrix-ops.h>
 #include <libnr/nr-point-fns.h>
+#include <libnr/nr-point-matrix-ops.h>
 
 #include "svg/svg.h"
 #include "attributes.h"
@@ -40,10 +44,10 @@
 static void sp_rect_class_init(SPRectClass *klass);
 static void sp_rect_init(SPRect *rect);
 
-static void sp_rect_build(SPObject *object, SPDocument *document, SPRepr *repr);
+static void sp_rect_build(SPObject *object, SPDocument *document, Inkscape::XML::Node *repr);
 static void sp_rect_set(SPObject *object, unsigned key, gchar const *value);
 static void sp_rect_update(SPObject *object, SPCtx *ctx, guint flags);
-static SPRepr *sp_rect_write(SPObject *object, SPRepr *repr, guint flags);
+static Inkscape::XML::Node *sp_rect_write(SPObject *object, Inkscape::XML::Node *repr, guint flags);
 
 static gchar *sp_rect_description(SPItem *item);
 static NR::Matrix sp_rect_set_transform(SPItem *item, NR::Matrix const &xform);
@@ -108,7 +112,7 @@ sp_rect_init(SPRect *rect)
 }
 
 static void
-sp_rect_build(SPObject *object, SPDocument *document, SPRepr *repr)
+sp_rect_build(SPObject *object, SPDocument *document, Inkscape::XML::Node *repr)
 {
     SPRect *rect = SP_RECT(object);
 
@@ -122,7 +126,7 @@ sp_rect_build(SPObject *object, SPDocument *document, SPRepr *repr)
     sp_object_read_attr(object, "rx");
     sp_object_read_attr(object, "ry");
 
-    SPVersion const version = sp_object_get_sodipodi_version(object);
+    Inkscape::Version const version = sp_object_get_sodipodi_version(object);
 
     if ( version.major == 0 && version.minor == 29 ) {
         if (rect->rx.set && rect->ry.set) {
@@ -196,7 +200,7 @@ sp_rect_update(SPObject *object, SPCtx *ctx, guint flags)
         SPRect *rect = (SPRect *) object;
         SPStyle *style = object->style;
         SPItemCtx const *ictx = (SPItemCtx const *) ctx;
-        double const d = NR_MATRIX_DF_EXPANSION(&ictx->i2vp);
+        double const d = NR::expansion(ictx->i2vp);
         double const w = (ictx->vp.x1 - ictx->vp.x0) / d;
         double const h = (ictx->vp.y1 - ictx->vp.y0) / d;
         double const em = style->font_size.computed;
@@ -215,8 +219,8 @@ sp_rect_update(SPObject *object, SPCtx *ctx, guint flags)
         ((SPObjectClass *) parent_class)->update(object, ctx, flags);
 }
 
-static SPRepr *
-sp_rect_write(SPObject *object, SPRepr *repr, guint flags)
+static Inkscape::XML::Node *
+sp_rect_write(SPObject *object, Inkscape::XML::Node *repr, guint flags)
 {
     SPRect *rect = SP_RECT(object);
 
@@ -399,13 +403,13 @@ sp_rect_set_transform(SPItem *item, NR::Matrix const &xform)
     sp_rect_set_shape(rect);
 
     // Adjust stroke width
-    sp_shape_adjust_stroke(item, sqrt(fabs(sw * sh)));
+    sp_item_adjust_stroke(item, sqrt(fabs(sw * sh)));
 
     // Adjust pattern fill
-    sp_shape_adjust_pattern(item, xform / ret);
+    sp_item_adjust_pattern(item, xform / ret);
 
     // Adjust gradient fill
-    sp_shape_adjust_gradient(item, xform / ret);
+    sp_item_adjust_gradient(item, xform / ret);
 
     item->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_STYLE_MODIFIED_FLAG);
 
