@@ -34,14 +34,14 @@
 static void sp_root_class_init (SPRootClass *klass);
 static void sp_root_init (SPRoot *root);
 
-static void sp_root_build (SPObject *object, SPDocument *document, SPRepr *repr);
+static void sp_root_build (SPObject *object, SPDocument *document, Inkscape::XML::Node *repr);
 static void sp_root_release (SPObject *object);
 static void sp_root_set (SPObject *object, unsigned int key, const gchar *value);
-static void sp_root_child_added (SPObject *object, SPRepr *child, SPRepr *ref);
-static void sp_root_remove_child (SPObject *object, SPRepr *child);
+static void sp_root_child_added (SPObject *object, Inkscape::XML::Node *child, Inkscape::XML::Node *ref);
+static void sp_root_remove_child (SPObject *object, Inkscape::XML::Node *child);
 static void sp_root_update (SPObject *object, SPCtx *ctx, guint flags);
 static void sp_root_modified (SPObject *object, guint flags);
-static SPRepr *sp_root_write (SPObject *object, SPRepr *repr, guint flags);
+static Inkscape::XML::Node *sp_root_write (SPObject *object, Inkscape::XML::Node *repr, guint flags);
 
 static NRArenaItem *sp_root_show (SPItem *item, NRArena *arena, unsigned int key, unsigned int flags);
 static void sp_root_bbox(SPItem const *item, NRRect *bbox, NR::Matrix const &transform, unsigned const flags);
@@ -115,7 +115,7 @@ sp_root_class_init (SPRootClass *klass)
 static void
 sp_root_init (SPRoot *root)
 {
-	static const SPVersion zero_version = { 0, 0 };
+	static const Inkscape::Version zero_version(0, 0);
 
 	sp_version_from_string (SVG_VERSION, &root->original.svg);
 	root->version.svg = root->original.svg;
@@ -137,17 +137,17 @@ sp_root_init (SPRoot *root)
 
 /** sp_root_build(object, document, repr)
  *
- *  This fills in the data for an SPObject from its SPRepr object.
+ *  This fills in the data for an SPObject from its Inkscape::XML::Node object.
  *  It fills in data such as version, x, y, width, height, etc.
  *  It then calls the object's parent class object's build function.
  */
 static void
-sp_root_build (SPObject *object, SPDocument *document, SPRepr *repr)
+sp_root_build (SPObject *object, SPDocument *document, Inkscape::XML::Node *repr)
 {
 	SPGroup *group = (SPGroup *) object;
 	SPRoot *root = (SPRoot *) object;
 
-	if (sp_repr_attr (repr, "sodipodi:docname") || sp_repr_attr (repr, "SP-DOCNAME")) {
+	if (repr->attribute("sodipodi:docname") || repr->attribute("SP-DOCNAME")) {
 		/* so we have a nonzero initial version */
 		root->original.sodipodi.major = 0;
 		root->original.sodipodi.minor = 1;
@@ -354,7 +354,7 @@ sp_root_set (SPObject *object, unsigned int key, const gchar *value)
  *  The SPRoot object is taken to be an SPGroup.
  */
 static void
-sp_root_child_added (SPObject *object, SPRepr *child, SPRepr *ref)
+sp_root_child_added (SPObject *object, Inkscape::XML::Node *child, Inkscape::XML::Node *ref)
 {
 	SPRoot *root = (SPRoot *) object;
 	SPGroup *group = (SPGroup *) object;
@@ -362,7 +362,13 @@ sp_root_child_added (SPObject *object, SPRepr *child, SPRepr *ref)
 	if (((SPObjectClass *) (parent_class))->child_added)
 		(* ((SPObjectClass *) (parent_class))->child_added) (object, child, ref);
 
-	gchar const *id = sp_repr_attr(child, "id");
+	gchar const *id = child->attribute("id");
+	if (!id) {
+		if (child->type() != Inkscape::XML::TEXT_NODE) {
+			g_error("repr has no id attribute, so cannot get corresponding object");
+		}
+		return;
+	}
 	SPObject *co = object->document->getObjectById(id);
 	g_assert (co != NULL);
 
@@ -382,7 +388,7 @@ sp_root_child_added (SPObject *object, SPRepr *child, SPRepr *ref)
  *  
  *  Removes the given child from this SPRoot object.
  */
-static void sp_root_remove_child(SPObject *object, SPRepr *child)
+static void sp_root_remove_child(SPObject *object, Inkscape::XML::Node *child)
 {
 	SPRoot *root = (SPRoot *) object;
 
@@ -531,7 +537,7 @@ sp_root_update (SPObject *object, SPCtx *ctx, guint flags)
 		root->c2p = NR::translate(-viewBox_min) * ( new_length / viewBox_length ) * NR::translate(x, y) * root->c2p;
 	}
 
-	rctx.i2doc = root->c2p * NR::Matrix(&rctx.i2doc);
+	rctx.i2doc = root->c2p * rctx.i2doc;
 
 	/* Initialize child viewport */
 	if (root->viewBox_set) {
@@ -552,7 +558,7 @@ sp_root_update (SPObject *object, SPCtx *ctx, guint flags)
 		rctx.vp.y1 = root->height.computed;
 	}
 
-	nr_matrix_set_identity (&rctx.i2vp);
+	rctx.i2vp = NR::identity();
 
 	/* And invoke parent method */
 	if (((SPObjectClass *) (parent_class))->update)
@@ -588,8 +594,8 @@ sp_root_modified (SPObject *object, guint flags)
  *
  *  This writes the object into the repr object, then calls the parent's write routine
  */
-static SPRepr *
-sp_root_write (SPObject *object, SPRepr *repr, guint flags)
+static Inkscape::XML::Node *
+sp_root_write (SPObject *object, Inkscape::XML::Node *repr, guint flags)
 {
 	SPRoot *root = SP_ROOT (object);
 
@@ -613,7 +619,7 @@ sp_root_write (SPObject *object, SPRepr *repr, guint flags)
 	sp_repr_set_double (repr, "y", root->y.computed);
 	sp_repr_set_double (repr, "width", root->width.computed);
 	sp_repr_set_double (repr, "height", root->height.computed);
-	sp_repr_set_attr (repr, "viewBox", sp_repr_attr (object->repr, "viewBox"));
+	sp_repr_set_attr (repr, "viewBox", object->repr->attribute("viewBox"));
 
 	if (((SPObjectClass *) (parent_class))->write)
 		((SPObjectClass *) (parent_class))->write (object, repr, flags);
@@ -675,4 +681,3 @@ sp_root_print (SPItem *item, SPPrintContext *ctx)
 
 	sp_print_release (ctx);
 }
-
