@@ -13,7 +13,9 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
 
 #include <glib/gmessages.h>
 #include <string.h>
@@ -24,6 +26,7 @@
 #include <libnr/nr-matrix-ops.h>
 #include <libnr/nr-matrix-translate-ops.h>
 #include <libnr/nr-point-fns.h>
+#include <libnr/nr-point-matrix-ops.h>
 #include <libnr/nr-scale-matrix-ops.h>
 
 #include <xml/repr.h>
@@ -47,9 +50,6 @@
 #include "sp-marker.h"
 #include "sp-shape.h"
 #include "sp-path.h"
-#include "sp-pattern.h"
-#include "sp-gradient.h"
-#include "gradient-chemistry.h"
 #include "sp-defs.h"
 #include "prefs-utils.h"
 
@@ -58,7 +58,7 @@
 static void sp_shape_class_init (SPShapeClass *klass);
 static void sp_shape_init (SPShape *shape);
 
-static void sp_shape_build (SPObject * object, SPDocument * document, SPRepr * repr);
+static void sp_shape_build (SPObject * object, SPDocument * document, Inkscape::XML::Node * repr);
 static void sp_shape_release (SPObject *object);
 
 static void sp_shape_update (SPObject *object, SPCtx *ctx, unsigned int flags);
@@ -128,9 +128,9 @@ sp_shape_init (SPShape *shape)
 }
 
 static void
-sp_shape_build (SPObject *object, SPDocument *document, SPRepr *repr)
+sp_shape_build (SPObject *object, SPDocument *document, Inkscape::XML::Node *repr)
 {
-	SPVersion version;
+	Inkscape::Version version;
 
 	version = sp_object_get_sodipodi_version (object);
 
@@ -227,13 +227,10 @@ sp_shape_update (SPObject *object, SPCtx *ctx, unsigned int flags)
 		SPStyle *style;
 		style = SP_OBJECT_STYLE (object);
 		if (style->stroke_width.unit == SP_CSS_UNIT_PERCENT) {
-			SPItemCtx *ictx;
-			SPItemView *v;
-			double aw;
-			ictx = (SPItemCtx *) ctx;
-			aw = 1.0 / NR_MATRIX_DF_EXPANSION (&ictx->i2vp);
+			SPItemCtx *ictx = (SPItemCtx *) ctx;
+			double const aw = 1.0 / NR::expansion(ictx->i2vp);
 			style->stroke_width.computed = style->stroke_width.value * aw;
-			for (v = ((SPItem *) (shape))->display; v != NULL; v = v->next) {
+			for (SPItemView *v = ((SPItem *) (shape))->display; v != NULL; v = v->next) {
 				nr_arena_shape_set_style ((NRArenaShape *) v->arenaitem, style);
 			}
 		}
@@ -662,7 +659,7 @@ sp_shape_print (SPItem *item, SPPrintContext *ctx)
                 if (sp_shape_marker_required (shape, m, bp)) {
 
                     SPMarker* marker = SP_MARKER (shape->marker[m]);
-                    SPItem* marker_path = SP_ITEM (shape->marker[m]->children);
+                    SPItem* marker_item = sp_item_first_item_child (SP_OBJECT (shape->marker[m]));
 
                     NR::Matrix tr(sp_shape_marker_get_transform(shape, bp));
 
@@ -670,12 +667,12 @@ sp_shape_print (SPItem *item, SPPrintContext *ctx)
                         tr = NR::scale(style->stroke_width.computed) * tr;
                     }
 
-                    tr = marker_path->transform * marker->c2p * tr;
+                    tr = marker_item->transform * marker->c2p * tr;
 
-                    NR::Matrix old_tr = marker_path->transform;
-                    marker_path->transform = tr;
-                    sp_item_invoke_print (marker_path, ctx);
-                    marker_path->transform = old_tr;
+                    NR::Matrix old_tr = marker_item->transform;
+                    marker_item->transform = tr;
+                    sp_item_invoke_print (marker_item, ctx);
+                    marker_item->transform = old_tr;
                 }
             }
         }
@@ -922,82 +919,6 @@ sp_shape_set_curve_insync (SPShape *shape, SPCurve *curve, unsigned int owner)
 	}
 }
 
-// Adjusters
-
-void
-sp_shape_adjust_pattern (SPItem *item, NR::Matrix const &postmul, bool set)
-{
-    SPStyle *style = SP_OBJECT_STYLE (item);
-
-    if (style && (style->fill.type == SP_PAINT_TYPE_PAINTSERVER)) { 
-        SPObject *server = SP_OBJECT_STYLE_FILL_SERVER (item);
-        if (SP_IS_PATTERN (server)) {
-            SPPattern *pattern = sp_pattern_clone_if_necessary (item, SP_PATTERN (server), "fill");
-            sp_pattern_transform_multiply (pattern, postmul, set);
-        }
-    }
-
-    if (style && (style->stroke.type == SP_PAINT_TYPE_PAINTSERVER)) { 
-        SPObject *server = SP_OBJECT_STYLE_STROKE_SERVER (item);
-        if (SP_IS_PATTERN (server)) {
-            SPPattern *pattern = sp_pattern_clone_if_necessary (item, SP_PATTERN (server), "stroke");
-            sp_pattern_transform_multiply (pattern, postmul, set);
-        }
-    }
-
-}
-
-void
-sp_shape_adjust_gradient (SPItem *item, NR::Matrix const &postmul, bool set)
-{
-    SPStyle *style = SP_OBJECT_STYLE (item);
-
-    if (style && (style->fill.type == SP_PAINT_TYPE_PAINTSERVER)) {
-        SPObject *server = SP_OBJECT_STYLE_FILL_SERVER(item);
-        if (SP_IS_GRADIENT (server)) {
-
-            // Bbox units for a gradient are generally a bad idea because with them, you cannot
-            // preserve the relative position of the object and its gradient after rotation or
-            // skew. So now we convert them to userspace units which are easy to keep in sync just
-            // by adding the object's transform to gradientTransform.  FIXME: convert back to bbox
-            // units after transforming with the item, so as to preserve the original units.
-            SPGradient *gradient = sp_gradient_convert_to_userspace (SP_GRADIENT (server), item, "fill");
-
-            sp_gradient_transform_multiply (gradient, postmul, set);
-        }
-    }
-
-    if (style && (style->stroke.type == SP_PAINT_TYPE_PAINTSERVER)) {
-        SPObject *server = SP_OBJECT_STYLE_STROKE_SERVER(item);
-        if (SP_IS_GRADIENT (server)) {
-            SPGradient *gradient = sp_gradient_convert_to_userspace (SP_GRADIENT (server), item, "stroke");
-            sp_gradient_transform_multiply (gradient, postmul, set);
-        }
-    }
-}
-
-void
-sp_shape_adjust_stroke (SPItem *item, gdouble ex)
-{
-    SPStyle *style = SP_OBJECT_STYLE (item);
-
-    if (style && style->stroke.type != SP_PAINT_TYPE_NONE && !NR_DF_TEST_CLOSE (ex, 1.0, NR_EPSILON)) {
-
-        style->stroke_width.computed *= ex;
-
-        if (style->stroke_dash.n_dash != 0) {
-            int i;
-            for (i = 0; i < style->stroke_dash.n_dash; i++) {
-                style->stroke_dash.dash[i] *= ex;
-            }
-            style->stroke_dash.offset *= ex;
-        }
-
-        SP_OBJECT(item)->updateRepr();
-    }
-}
-
-
 static void sp_shape_snappoints(SPItem const *item, SnapPointsIter p)
 {
     g_assert(item != NULL);
