@@ -5,6 +5,7 @@
  *
  * Author:
  *   Lauris Kaplinski <lauris@kaplinski.com>
+ *   bulia byak <buliabyak@users.sf.net>
  *
  * Copyright (C) 1999-2002 Lauris Kaplinski
  * Copyright (C) 2000-2001 Ximian, Inc.
@@ -31,13 +32,6 @@
 #include <libnr/nr-matrix-ops.h>
 #include <libnr/nr-matrix-fns.h>
 #include <libnr/nr-rotate.h>
-//#include <libnrtype/nr-typeface.h>
-//#include <libnrtype/FontFactory.h>
-//#include <libnrtype/font-instance.h>
-//#include <libnrtype/font-style-to-pos.h>
-
-#include <libnrtype/FlowDefs.h>
-//#include <libnrtype/TextWrapper.h>
 
 #include <livarot/LivarotDefs.h>
 #include <livarot/Shape.h>
@@ -68,50 +62,6 @@
 
 #include "prefs-utils.h"
 
-/*
- *
- */
-void read_length_array(int &nb, SPSVGLength *&array, char const *value)
-{
-    if (array) free(array);
-    array=NULL;
-    nb=0;
-    if (value) {
-        GList *list = sp_svg_length_list_read(value);
-        nb = g_list_length(list);
-        array = (SPSVGLength*)malloc(nb*sizeof(SPSVGLength));
-        for (int i = 0; i < nb; i++) {
-            sp_svg_length_unset(array + i, SP_SVG_UNIT_NONE, 0.0, 0.0);
-        }
-        int cur = 0;
-        for (GList *l = list; l; l = l->next) {
-            SPSVGLength *nl=(SPSVGLength*)l->data;
-            if ( cur < nb ) {
-                array[cur++]=*nl; // overcautious
-            }
-            g_free(l->data);
-        }
-        g_list_free(list);
-    }
-}
-
-char *write_length_array(int nb, SPSVGLength *array)
-{
-    if ( nb <= 0 ) return NULL;
-    gchar c[32];
-    gchar *s = NULL;
-	
-    for (int i=0;i<nb;i++) {
-        g_ascii_formatd(c, sizeof(c), "%.8g", array[i].computed);
-        if (i == 0) {
-            s = g_strdup(c);
-        }  else {
-            s = g_strjoin(" ", s, c, NULL);
-        }
-    }
-    return s;
-}
-
 /*#####################################################
 #  SPTSPAN
 #####################################################*/
@@ -119,12 +69,12 @@ char *write_length_array(int nb, SPSVGLength *array)
 static void sp_tspan_class_init(SPTSpanClass *classname);
 static void sp_tspan_init(SPTSpan *tspan);
 
-static void sp_tspan_build(SPObject * object, SPDocument * document, SPRepr * repr);
+static void sp_tspan_build(SPObject * object, SPDocument * document, Inkscape::XML::Node * repr);
 static void sp_tspan_release(SPObject *object);
 static void sp_tspan_set(SPObject *object, unsigned key, gchar const *value);
 static void sp_tspan_update(SPObject *object, SPCtx *ctx, guint flags);
 static void sp_tspan_modified(SPObject *object, unsigned flags);
-static SPRepr *sp_tspan_write(SPObject *object, SPRepr *repr, guint flags);
+static Inkscape::XML::Node *sp_tspan_write(SPObject *object, Inkscape::XML::Node *repr, guint flags);
 
 static SPItemClass *tspan_parent_class;
 
@@ -176,8 +126,7 @@ static void
 sp_tspan_init(SPTSpan *tspan)
 {
     tspan->role = SP_TSPAN_ROLE_UNSPECIFIED;
-    tspan->last_tspan = false;
-    new (&tspan->contents) div_flow_src(SP_OBJECT(tspan), txt_tline);
+    new (&tspan->attributes) TextTagAttributes;
 }
 
 static void
@@ -185,14 +134,14 @@ sp_tspan_release(SPObject *object)
 {
     SPTSpan *tspan = SP_TSPAN(object);
 
-    tspan->contents.~div_flow_src();
+    tspan->attributes.~TextTagAttributes();
 	
     if (((SPObjectClass *) tspan_parent_class)->release)
         ((SPObjectClass *) tspan_parent_class)->release(object);
 }
 
 static void
-sp_tspan_build(SPObject *object, SPDocument *doc, SPRepr *repr)
+sp_tspan_build(SPObject *object, SPDocument *doc, Inkscape::XML::Node *repr)
 {
     //SPTSpan *tspan = SP_TSPAN(object);
 	
@@ -203,19 +152,6 @@ sp_tspan_build(SPObject *object, SPDocument *doc, SPRepr *repr)
     sp_object_read_attr(object, "rotate");
     sp_object_read_attr(object, "sodipodi:role");
 	
-    bool no_content = true;
-    for (SPRepr* rch = repr->firstChild() ; rch != NULL; rch = rch->next()) {
-        if ( rch->type() == SP_XML_TEXT_NODE ) {
-            no_content = false;
-            break;
-        }
-    }
-
-    if ( no_content ) {
-        SPRepr* rch = sp_repr_new_text("");
-        sp_repr_add_child(repr, rch, NULL);
-    }
-	
     if (((SPObjectClass *) tspan_parent_class)->build)
         ((SPObjectClass *) tspan_parent_class)->build(object, doc, repr);
 }
@@ -225,60 +161,29 @@ sp_tspan_set(SPObject *object, unsigned key, gchar const *value)
 {
     SPTSpan *tspan = SP_TSPAN(object);
 	
-    /* fixme: Vectors */
-    switch (key) {
-        case SP_ATTR_X:
-            tspan->contents.SetX(value);
-            if ( tspan->contents.nb_x > 0 ) {
-                tspan->x=tspan->contents.x_s[0];
-            } else {
-                tspan->x.set=0;
-            }
-            if ( tspan->role != SP_TSPAN_ROLE_LINE ) object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
-            break;
-        case SP_ATTR_Y:
-            tspan->contents.SetY(value);
-            if ( tspan->contents.nb_y > 0 ) {
-                tspan->y=tspan->contents.y_s[0];
-            } else {
-                tspan->y.set=0;
-            }
-            if ( tspan->role != SP_TSPAN_ROLE_LINE ) object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
-            break;
-        case SP_ATTR_DX:
-            tspan->contents.SetDX(value);
-            if ( tspan->role != SP_TSPAN_ROLE_LINE ) object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
-            break;
-        case SP_ATTR_DY:
-            tspan->contents.SetDY(value);
-            if ( tspan->role != SP_TSPAN_ROLE_LINE ) object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
-            break;
-        case SP_ATTR_ROTATE:
-            tspan->contents.SetRot(value);
-            if ( tspan->role != SP_TSPAN_ROLE_LINE ) object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
-            break;
-        case SP_ATTR_SODIPODI_ROLE:
-            if (value && (!strcmp(value, "line") || !strcmp(value, "paragraph"))) {
-                tspan->role = SP_TSPAN_ROLE_LINE;
-            } else {
-                tspan->role = SP_TSPAN_ROLE_UNSPECIFIED;
-            }
-            break;
-        default:
-            if (((SPObjectClass *) tspan_parent_class)->set)
-                (((SPObjectClass *) tspan_parent_class)->set)(object, key, value);
-            break;
+    if (tspan->attributes.readSingleAttribute(key, value)) {
+        if (tspan->role != SP_TSPAN_ROLE_LINE)
+            object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
+    } else {
+        switch (key) {
+            case SP_ATTR_SODIPODI_ROLE:
+                if (value && (!strcmp(value, "line") || !strcmp(value, "paragraph"))) {
+                    tspan->role = SP_TSPAN_ROLE_LINE;
+                } else {
+                    tspan->role = SP_TSPAN_ROLE_UNSPECIFIED;
+                }
+                break;
+            default:
+                if (((SPObjectClass *) tspan_parent_class)->set)
+                    (((SPObjectClass *) tspan_parent_class)->set)(object, key, value);
+                break;
+        }
     }
 }
 
 static void
 sp_tspan_update(SPObject *object, SPCtx *ctx, guint flags)
 {
-    //SPTSpan *tspan = SP_TSPAN(object);
-    //SPStyle *style = SP_OBJECT_STYLE(object);
-    //SPItemCtx *ictx = (SPItemCtx *) ctx;
-    //GList *i;
-	
     if (((SPObjectClass *) tspan_parent_class)->update)
         ((SPObjectClass *) tspan_parent_class)->update(object, ctx, flags);
 	
@@ -311,8 +216,8 @@ sp_tspan_modified(SPObject *object, unsigned flags)
     }
 }
 
-static SPRepr *
-sp_tspan_write(SPObject *object, SPRepr *repr, guint flags)
+static Inkscape::XML::Node *
+sp_tspan_write(SPObject *object, Inkscape::XML::Node *repr, guint flags)
 {
     SPTSpan *tspan = SP_TSPAN(object);
 	
@@ -320,77 +225,24 @@ sp_tspan_write(SPObject *object, SPRepr *repr, guint flags)
         repr = sp_repr_new("svg:tspan");
     }
 	
-    bool  became_empty=false;
-    if ( tspan->role != SP_TSPAN_ROLE_UNSPECIFIED ) {
-        if ( tspan->contents.type == txt_tline && tspan->last_tspan == false && tspan->contents.utf8_st >= tspan->contents.utf8_en ) {
-            became_empty=true;
-        }
-    }
-		
-    char* nlist=NULL;
-    if ( became_empty == false ) {
-        if ( (nlist=tspan->contents.GetX()) ) {
-            sp_repr_set_attr(repr,"x",nlist);
-            g_free(nlist);
-        } else {
-            if ( tspan->x.set ) sp_repr_set_double(repr, "x", tspan->x.computed); else sp_repr_set_attr(repr, "x", NULL);
-        }
-        if ( (nlist=tspan->contents.GetY()) ) {
-            sp_repr_set_attr(repr,"y",nlist);
-            g_free(nlist);
-        } else {
-            if ( tspan->y.set ) sp_repr_set_double(repr, "y", tspan->y.computed); sp_repr_set_attr(repr, "y", NULL);
-        }
-    } else {
-        sp_repr_set_attr(repr, "x", NULL);
-        sp_repr_set_attr(repr, "y", NULL);
-    }
-    if ( (nlist=tspan->contents.GetDX()) ) {
-        sp_repr_set_attr(repr,"dx",nlist);
-        g_free(nlist);
-    } else {
-        sp_repr_set_attr(repr, "dx", NULL);
-    }
-    if ( (nlist=tspan->contents.GetDY()) ) {
-        sp_repr_set_attr(repr,"dy",nlist);
-        g_free(nlist);
-    } else {
-        sp_repr_set_attr(repr, "dy", NULL);
-    }
-    if ( (nlist=tspan->contents.GetRot()) ) {
-        sp_repr_set_attr(repr,"rotate",nlist);
-        g_free(nlist);
-    } else {
-        sp_repr_set_attr(repr, "rotate", NULL);
-    }
-    if (flags & SP_OBJECT_WRITE_EXT) {
-        if ( tspan->role != SP_TSPAN_ROLE_UNSPECIFIED ) {
-            if ( became_empty == false ) {
-                sp_repr_set_attr(repr, "sodipodi:role", "line");
-            } else {
-                sp_repr_set_attr(repr, "sodipodi:role", NULL);
-            }
-        } else {
-            sp_repr_set_attr(repr, "sodipodi:role", NULL);
-        }
-    }
+    tspan->attributes.writeTo(repr);
 	
     if ( flags&SP_OBJECT_WRITE_BUILD ) {
         GSList *l = NULL;
         for (SPObject* child = sp_object_first_child(object) ; child != NULL ; child = SP_OBJECT_NEXT(child) ) {
-            SPRepr* c_repr=NULL;
+            Inkscape::XML::Node* c_repr=NULL;
             if ( SP_IS_TSPAN(child) ) {
                 c_repr = child->updateRepr(NULL, flags);
             } else if ( SP_IS_TEXTPATH(child) ) {
                 //c_repr = child->updateRepr(NULL, flags); // shouldn't happen
             } else if ( SP_IS_STRING(child) ) {
-                c_repr = sp_repr_new_text(SP_STRING_TEXT(child));
+                c_repr = sp_repr_new_text(SP_STRING(child)->string.c_str());
             }
             if ( c_repr ) l = g_slist_prepend(l, c_repr);
         }
         while ( l ) {
-            sp_repr_add_child(repr, (SPRepr *) l->data, NULL);
-            sp_repr_unref((SPRepr *) l->data);
+            sp_repr_add_child(repr, (Inkscape::XML::Node *) l->data, NULL);
+            sp_repr_unref((Inkscape::XML::Node *) l->data);
             l = g_slist_remove(l, l->data);
         }
     } else {
@@ -400,7 +252,7 @@ sp_tspan_write(SPObject *object, SPRepr *repr, guint flags)
             } else if ( SP_IS_TEXTPATH(child) ) {
                 //c_repr = child->updateRepr(NULL, flags); // shouldn't happen
             } else if ( SP_IS_STRING(child) ) {
-                sp_repr_set_content(SP_OBJECT_REPR(child), (SP_STRING_TEXT(child))?SP_STRING_TEXT(child):"");
+                SP_OBJECT_REPR(child)->setContent(SP_STRING(child)->string.c_str());
             }
         }
     }
@@ -419,12 +271,12 @@ static void sp_textpath_class_init(SPTextPathClass *classname);
 static void sp_textpath_init(SPTextPath *textpath);
 static void sp_textpath_finalize(GObject *obj);
 
-static void sp_textpath_build(SPObject * object, SPDocument * document, SPRepr * repr);
+static void sp_textpath_build(SPObject * object, SPDocument * document, Inkscape::XML::Node * repr);
 static void sp_textpath_release(SPObject *object);
 static void sp_textpath_set(SPObject *object, unsigned key, gchar const *value);
 static void sp_textpath_update(SPObject *object, SPCtx *ctx, guint flags);
 static void sp_textpath_modified(SPObject *object, unsigned flags);
-static SPRepr *sp_textpath_write(SPObject *object, SPRepr *repr, guint flags);
+static Inkscape::XML::Node *sp_textpath_write(SPObject *object, Inkscape::XML::Node *repr, guint flags);
 
 static SPItemClass *textpath_parent_class;
 
@@ -481,8 +333,9 @@ sp_textpath_class_init(SPTextPathClass *classname)
 static void
 sp_textpath_init(SPTextPath *textpath)
 {
-    new (&textpath->contents) div_flow_src(SP_OBJECT(textpath),txt_textpath);
+    new (&textpath->attributes) TextTagAttributes;
 	
+    textpath->startOffset.set = 0;
     textpath->originalPath = NULL;
     textpath->isUpdating=false;
     // set up the uri reference
@@ -503,7 +356,7 @@ sp_textpath_release(SPObject *object)
 {
     SPTextPath *textpath = SP_TEXTPATH(object);
 	
-    textpath->contents.~div_flow_src();
+    textpath->attributes.~TextTagAttributes();
 	
     if (textpath->originalPath) delete textpath->originalPath;
     textpath->originalPath = NULL;
@@ -513,7 +366,7 @@ sp_textpath_release(SPObject *object)
 }
 
 static void
-sp_textpath_build(SPObject *object, SPDocument *doc, SPRepr *repr)
+sp_textpath_build(SPObject *object, SPDocument *doc, Inkscape::XML::Node *repr)
 {
     //SPTextPath *textpath = SP_TEXTPATH(object);
 	
@@ -522,15 +375,16 @@ sp_textpath_build(SPObject *object, SPDocument *doc, SPRepr *repr)
     sp_object_read_attr(object, "dx");
     sp_object_read_attr(object, "dy");
     sp_object_read_attr(object, "rotate");
+    sp_object_read_attr(object, "startOffset");
     sp_object_read_attr(object, "xlink:href");
 	
     bool  no_content=true;
-    for (SPRepr* rch = repr->firstChild() ; rch != NULL; rch = rch->next()) {
-        if ( rch->type() == SP_XML_TEXT_NODE ) {no_content=false;break;}
+    for (Inkscape::XML::Node* rch = repr->firstChild() ; rch != NULL; rch = rch->next()) {
+        if ( rch->type() == Inkscape::XML::TEXT_NODE ) {no_content=false;break;}
     }
 	
     if ( no_content ) {
-        SPRepr* rch = sp_repr_new_text("");
+        Inkscape::XML::Node* rch = sp_repr_new_text("");
         sp_repr_add_child(repr, rch, NULL);
     }
 	
@@ -543,45 +397,24 @@ sp_textpath_set(SPObject *object, unsigned key, gchar const *value)
 {
     SPTextPath *textpath = SP_TEXTPATH(object);
 	
-    /* fixme: Vectors */
-    switch (key) {
-        case SP_ATTR_X:
-            textpath->contents.SetX(value);
-            if ( textpath->contents.nb_x > 0 ) {
-                textpath->x=textpath->contents.x_s[0];
-            } else {
-                textpath->x.set=0;
-            }
-            object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
-            break;
-        case SP_ATTR_Y:
-            textpath->contents.SetY(value);
-            if ( textpath->contents.nb_y > 0 ) {
-                textpath->y=textpath->contents.y_s[0];
-            } else {
-                textpath->y.set=0;
-            }
-            object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
-            break;
-        case SP_ATTR_DX:
-            textpath->contents.SetDX(value);
-            object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
-            break;
-        case SP_ATTR_DY:
-            textpath->contents.SetDY(value);
-            object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
-            break;
-        case SP_ATTR_ROTATE:
-            textpath->contents.SetRot(value);
-            object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
-            break;
-        case SP_ATTR_XLINK_HREF:
-            textpath->sourcePath->link((char*)value);
-            break;
-        default:
-            if (((SPObjectClass *) textpath_parent_class)->set)
-                (((SPObjectClass *) textpath_parent_class)->set)(object, key, value);
-            break;
+    if (textpath->attributes.readSingleAttribute(key, value)) {
+        object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
+    } else {
+        switch (key) {
+            case SP_ATTR_XLINK_HREF:
+                textpath->sourcePath->link((char*)value);
+                break;
+            case SP_ATTR_STARTOFFSET:
+                if (!sp_svg_length_read(value, &textpath->startOffset)) {
+                    sp_svg_length_unset(&textpath->startOffset, SP_SVG_UNIT_NONE, 0.0, 0.0);
+                }
+                object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
+                break;
+            default:
+                if (((SPObjectClass *) textpath_parent_class)->set)
+                    (((SPObjectClass *) textpath_parent_class)->set)(object, key, value);
+                break;
+        }
     }
 }
 
@@ -589,9 +422,6 @@ static void
 sp_textpath_update(SPObject *object, SPCtx *ctx, guint flags)
 {
     SPTextPath *textpath = SP_TEXTPATH(object);
-    //SPStyle *style = SP_OBJECT_STYLE(object);
-    //SPItemCtx *ictx = (SPItemCtx *) ctx;
-    //GList *i;
 	
     textpath->isUpdating=true;
     if ( textpath->sourcePath->sourceDirty ) refresh_textpath_source(textpath);
@@ -649,8 +479,8 @@ sp_textpath_modified(SPObject *object, unsigned flags)
         }
     }
 }
-static SPRepr *
-sp_textpath_write(SPObject *object, SPRepr *repr, guint flags)
+static Inkscape::XML::Node *
+sp_textpath_write(SPObject *object, Inkscape::XML::Node *repr, guint flags)
 {
     SPTextPath *textpath = SP_TEXTPATH(object);
 	
@@ -658,55 +488,34 @@ sp_textpath_write(SPObject *object, SPRepr *repr, guint flags)
         repr = sp_repr_new("svg:textpath");
     }
 	
-    char* nlist=NULL;
-    if ( (nlist=textpath->contents.GetX()) ) {
-        sp_repr_set_attr(repr,"x",nlist);
-        g_free(nlist);
-    } else {
-        if ( textpath->x.set ) sp_repr_set_double(repr, "x", textpath->x.computed); else sp_repr_set_attr(repr, "x", NULL);
-    }
-    if ( (nlist=textpath->contents.GetY()) ) {
-        sp_repr_set_attr(repr,"y",nlist);
-        g_free(nlist);
-    } else {
-        if ( textpath->y.set ) sp_repr_set_double(repr, "y", textpath->y.computed); else sp_repr_set_attr(repr, "y", NULL);
-    }
-    if ( (nlist=textpath->contents.GetDX()) ) {
-        sp_repr_set_attr(repr,"dx",nlist);
-        g_free(nlist);
-    } else {
-        sp_repr_set_attr(repr, "dx", NULL);
-    }
-    if ( (nlist=textpath->contents.GetDY())) {
-        sp_repr_set_attr(repr,"dy",nlist);
-        g_free(nlist);
-    } else {
-        sp_repr_set_attr(repr, "dy", NULL);
-    }
-    if ( (nlist=textpath->contents.GetRot()) ) {
-        sp_repr_set_attr(repr,"rotate",nlist);
-        g_free(nlist);
-    } else {
-        sp_repr_set_attr(repr, "rotate", NULL);
+    textpath->attributes.writeTo(repr);
+    if (textpath->startOffset.set) {
+        if (textpath->startOffset.unit == SP_SVG_UNIT_PERCENT) {
+	        Inkscape::SVGOStringStream os;
+            os << (textpath->startOffset.computed * 100.0) << "%";
+            SP_OBJECT_REPR(textpath)->setAttribute("startOffset", os.str().c_str());
+        } else
+            sp_repr_set_double(repr, "startOffset", textpath->startOffset.computed);
     }
+
     if ( textpath->sourcePath->sourceHref ) sp_repr_set_attr(repr, "xlink:href", textpath->sourcePath->sourceHref);
 	
     if ( flags&SP_OBJECT_WRITE_BUILD ) {
         GSList *l = NULL;
         for (SPObject* child = sp_object_first_child(object) ; child != NULL ; child = SP_OBJECT_NEXT(child) ) {
-            SPRepr* c_repr=NULL;
+            Inkscape::XML::Node* c_repr=NULL;
             if ( SP_IS_TSPAN(child) ) {
                 c_repr = child->updateRepr(NULL, flags);
             } else if ( SP_IS_TEXTPATH(child) ) {
                 //c_repr = child->updateRepr(NULL, flags); // shouldn't happen
             } else if ( SP_IS_STRING(child) ) {
-                c_repr = sp_repr_new_text(SP_STRING_TEXT(child));
+                c_repr = sp_repr_new_text(SP_STRING(child)->string.c_str());
             }
             if ( c_repr ) l = g_slist_prepend(l, c_repr);
         }
         while ( l ) {
-            sp_repr_add_child(repr, (SPRepr *) l->data, NULL);
-            sp_repr_unref((SPRepr *) l->data);
+            sp_repr_add_child(repr, (Inkscape::XML::Node *) l->data, NULL);
+            sp_repr_unref((Inkscape::XML::Node *) l->data);
             l = g_slist_remove(l, l->data);
         }
     } else {
@@ -716,7 +525,7 @@ sp_textpath_write(SPObject *object, SPRepr *repr, guint flags)
             } else if ( SP_IS_TEXTPATH(child) ) {
                 //c_repr = child->updateRepr(NULL, flags); // shouldn't happen
             } else if ( SP_IS_STRING(child) ) {
-                sp_repr_set_content(SP_OBJECT_REPR(child), (SP_STRING_TEXT(child))?SP_STRING_TEXT(child):"");
+                SP_OBJECT_REPR(child)->setContent(SP_STRING(child)->string.c_str());
             }
         }
     }
@@ -756,9 +565,9 @@ sp_textpath_to_text(SPObject *tp)
 
     for ( GSList *i = tp_reprs ; i ; i = i->next ) {
         // make a copy of each textpath child
-        SPRepr *copy = sp_repr_duplicate((SPRepr *) i->data);
+        Inkscape::XML::Node *copy = ((Inkscape::XML::Node *) i->data)->duplicate();
         // remove the old repr from under textpath
-        sp_repr_remove_child(SP_OBJECT_REPR(tp), (SPRepr *) i->data); 
+        sp_repr_remove_child(SP_OBJECT_REPR(tp), (Inkscape::XML::Node *) i->data); 
         // put its copy into under textPath
         sp_repr_add_child(SP_OBJECT_REPR(text), copy, NULL); // fixme: copy id
     }
