@@ -5,8 +5,9 @@
  *
  * Authors:
  *   Lauris Kaplinski <lauris@kaplinski.com>
+ *   bulia byak <buliabyak@users.sf.net>
  *
- * Copyright (C) 1999-2002 Lauris Kaplinski
+ * Copyright (C) 1999-2005 authors
  * Copyright (C) 2000-2001 Ximian, Inc.
  *
  * Released under GNU GPL, read the file 'COPYING' for more information
@@ -26,6 +27,7 @@
 #include "attributes.h"
 #include "document.h"
 #include "sp-object-repr.h"
+#include "sp-flowregion.h"
 #include "uri.h"
 #include "uri-references.h"
 #include "macros.h"
@@ -44,10 +46,10 @@ static void sp_use_class_init(SPUseClass *classname);
 static void sp_use_init(SPUse *use);
 static void sp_use_finalize(GObject *obj);
 
-static void sp_use_build(SPObject *object, SPDocument *document, SPRepr *repr);
+static void sp_use_build(SPObject *object, SPDocument *document, Inkscape::XML::Node *repr);
 static void sp_use_release(SPObject *object);
 static void sp_use_set(SPObject *object, unsigned key, gchar const *value);
-static SPRepr *sp_use_write(SPObject *object, SPRepr *repr, guint flags);
+static Inkscape::XML::Node *sp_use_write(SPObject *object, Inkscape::XML::Node *repr, guint flags);
 static void sp_use_update(SPObject *object, SPCtx *ctx, guint flags);
 static void sp_use_modified(SPObject *object, guint flags);
 
@@ -146,7 +148,7 @@ sp_use_finalize(GObject *obj)
 }
 
 static void
-sp_use_build(SPObject *object, SPDocument *document, SPRepr *repr)
+sp_use_build(SPObject *object, SPDocument *document, Inkscape::XML::Node *repr)
 {
     if (((SPObjectClass *) parent_class)->build) {
         (* ((SPObjectClass *) parent_class)->build)(object, document, repr);
@@ -247,8 +249,8 @@ sp_use_set(SPObject *object, unsigned key, gchar const *value)
     }
 }
 
-static SPRepr *
-sp_use_write(SPObject *object, SPRepr *repr, guint flags)
+static Inkscape::XML::Node *
+sp_use_write(SPObject *object, Inkscape::XML::Node *repr, guint flags)
 {
     SPUse *use = SP_USE(object);
 
@@ -304,9 +306,23 @@ sp_use_description(SPItem *item)
 {
     SPUse *use = SP_USE(item);
 
+    char *ret;
     if (use->child) {
-        return g_strdup_printf(_("<b>Clone</b> of: %s"),
-                               sp_item_description(SP_ITEM(use->child)));
+        static unsigned recursion_depth = 0;
+        if (recursion_depth >= 2) {
+            /* TRANSLATORS: Used for statusbar description for long <use> chains:
+             * "Clone of: Clone of: ... in Layer 1". */
+            return g_strdup(_("..."));
+            /* We could do better, e.g. chasing the href chain until we reach something other than
+             * a <use>, and giving its description. */
+        }
+        ++recursion_depth;
+        char *child_desc = sp_item_description(SP_ITEM(use->child));
+        --recursion_depth;
+
+        ret = g_strdup_printf(_("<b>Clone</b> of: %s"), child_desc);
+        g_free(child_desc);
+        return ret;
     } else {
         return g_strdup(_("<b>Orphaned clone</b>"));
     }
@@ -437,6 +453,11 @@ sp_use_move_compensate(NR::Matrix const *mp, SPItem *original, SPUse *self)
         return;
     }
 
+    // never compensate uses which are used in flowtext 
+    if (SP_OBJECT_PARENT(self) && SP_IS_FLOWREGION(SP_OBJECT_PARENT(self))) {
+        return;
+    }
+
     guint mode = prefs_get_int_attribute("options.clonecompensation", "value", SP_CLONE_COMPENSATION_PARALLEL);
     // user wants no compensation
     if (mode == SP_CLONE_COMPENSATION_NONE)
@@ -448,6 +469,9 @@ sp_use_move_compensate(NR::Matrix const *mp, SPItem *original, SPUse *self)
     if (!(m.is_translation()))
         return;
 
+    // restore item->transform field from the repr, in case it was changed by seltrans
+    sp_object_read_attr (SP_OBJECT (self), "transform");
+
     NR::Matrix t = sp_use_get_parent_transform(self);
     NR::Matrix clone_move = t.inverse() * m * t;
 
@@ -486,7 +510,7 @@ sp_use_href_changed(SPObject *old_ref, SPObject *ref, SPUse *use)
     if (use->href) {
         SPItem *refobj = use->ref->getObject();
         if (refobj) {
-            SPRepr *childrepr = SP_OBJECT_REPR(refobj);
+            Inkscape::XML::Node *childrepr = SP_OBJECT_REPR(refobj);
             GType type = sp_repr_type_lookup(childrepr);
             g_return_if_fail(type > G_TYPE_NONE);
             if (g_type_is_a(type, SP_TYPE_ITEM)) {
@@ -513,6 +537,12 @@ sp_use_href_changed(SPObject *old_ref, SPObject *ref, SPUse *use)
 static void
 sp_use_delete_self(SPObject *deleted, SPUse *self)
 {
+    // always delete uses which are used in flowtext 
+    if (SP_OBJECT_PARENT(self) && SP_IS_FLOWREGION(SP_OBJECT_PARENT(self))) {
+        SP_OBJECT(self)->deleteObject();
+        return;
+    }
+
     guint const mode = prefs_get_int_attribute("options.cloneorphans", "value",
                                                SP_CLONE_ORPHANS_UNLINK);
 
@@ -554,7 +584,7 @@ sp_use_update(SPObject *object, SPCtx *ctx, unsigned flags)
     cctx.vp.y0 = 0.0;
     cctx.vp.x1 = use->width.computed;
     cctx.vp.y1 = use->height.computed;
-    nr_matrix_set_identity(&cctx.i2vp);
+    cctx.i2vp = NR::identity();
     flags&=~SP_OBJECT_USER_MODIFIED_FLAG_B;
 
     if (use->child) {
@@ -605,69 +635,64 @@ sp_use_unlink(SPUse *use)
 {
     if (!use) return NULL;
 
-    SPRepr *repr = SP_OBJECT_REPR(use);
+    Inkscape::XML::Node *repr = SP_OBJECT_REPR(use);
     if (!repr) return NULL;
 
-    SPRepr *parent = sp_repr_parent(repr);
+    Inkscape::XML::Node *parent = sp_repr_parent(repr);
     SPDocument *document = SP_OBJECT(use)->document;
 
-    //track the ultimate source of a chain of uses
+    // Track the ultimate source of a chain of uses.
     SPItem *orig = sp_use_root(use);
 
-    //calculate the accumulated transform, starting from the original
+    // Calculate the accumulated transform, starting from the original.
     NR::Matrix t = sp_use_get_root_transform(use);
 
-    // create copy of the original
-    SPRepr *copy = sp_repr_duplicate(SP_OBJECT_REPR(orig));
+    // Create copy of the original.
+    Inkscape::XML::Node *copy = SP_OBJECT_REPR(orig)->duplicate();
 
-    // add the duplicate repr just after the existing one
+    // Add the duplicate repr just after the existing one.
     sp_repr_add_child(parent, copy, repr);
 
-    // retrieve the SPItem of the resulting repr
+    // Retrieve the SPItem of the resulting repr.
     SPObject *unlinked = document->getObjectByRepr(copy);
 
-    // merge style from the use
-    SPStyle *unli_sty = SP_OBJECT_STYLE (unlinked);
-    SPStyle *use_sty = SP_OBJECT_STYLE (use);
-    sp_style_merge_from_parent (unli_sty, use_sty);
-    // special case for opacity: it is not inherited per the spec, but since use's opacity affected
-    // its display, we must not lose it after unlinking
-    unli_sty->opacity.value = SP_SCALE24_FROM_FLOAT (
-        SP_SCALE24_TO_FLOAT (unli_sty->opacity.value) * SP_SCALE24_TO_FLOAT (use_sty->opacity.value));
-    if (unli_sty->opacity.value != SP_SCALE24_MAX) {
-        unli_sty->opacity.set = TRUE;
-    }
-    SP_OBJECT (unlinked)->updateRepr();
+    // Merge style from the use.
+    SPStyle *unli_sty = SP_OBJECT_STYLE(unlinked);
+    SPStyle const *use_sty = SP_OBJECT_STYLE(use);
+    sp_style_merge_from_dying_parent(unli_sty, use_sty);
+    sp_style_merge_from_parent(unli_sty, unlinked->parent->style);
 
-    // hold onto our SPObject and repr for now
+    SP_OBJECT(unlinked)->updateRepr();
+
+    // Hold onto our SPObject and repr for now.
     sp_object_ref(SP_OBJECT(use), NULL);
     sp_repr_ref(repr);
 
-    // remove ourselves, not propagating delete events to avoid a
-    // chain-reaction with other elements that might reference us
+    // Remove ourselves, not propagating delete events to avoid a
+    // chain-reaction with other elements that might reference us.
     SP_OBJECT(use)->deleteObject(false);
 
-    // give the copy our old id and let go of our old repr
-    sp_repr_set_attr(copy, "id", sp_repr_attr(repr, "id"));
+    // Give the copy our old id and let go of our old repr.
+    sp_repr_set_attr(copy, "id", repr->attribute("id"));
     sp_repr_unref(repr);
 
-    // remove tiled clone attrs
+    // Remove tiled clone attrs.
     sp_repr_set_attr(copy, "inkscape:tiled-clone-of", NULL);
     sp_repr_set_attr(copy, "inkscape:tile-w", NULL);
     sp_repr_set_attr(copy, "inkscape:tile-h", NULL);
     sp_repr_set_attr(copy, "inkscape:tile-cx", NULL);
     sp_repr_set_attr(copy, "inkscape:tile-cy", NULL);
 
-    // establish the succession and let go of our object
+    // Establish the succession and let go of our object.
     SP_OBJECT(use)->setSuccessor(unlinked);
     sp_object_unref(SP_OBJECT(use), NULL);
 
     SPItem *item = SP_ITEM(unlinked);
-    // set the accummulated transform
+    // Set the accummulated transform.
     {
         NR::Matrix nomove(NR::identity());
         NRMatrix ctrans = t.operator const NRMatrix&();
-        // advertise ourselves as not moving
+        // Advertise ourselves as not moving.
         sp_item_write_transform(item, SP_OBJECT_REPR(item), &ctrans, &nomove);
     }
     return item;
