@@ -4,6 +4,7 @@
  *  Inkscape
  *
  *  Created by fred on Fri Dec 05 2003.
+ *  tweaked endlessly by bulia byak <buliabyak@users.sf.net>
  *  public domain
  *
  */
@@ -35,7 +36,7 @@
 #include "display/canvas-bpath.h"
 #include "display/curve.h"
 #include <glibmm/i18n.h>
-#include "view.h"
+#include "ui/view/view.h"
 #include "prefs-utils.h"
 
 #include "libnr/n-art-bpath.h"
@@ -49,7 +50,7 @@
 #include "livarot/LivarotDefs.h"
 
 Path   *Path_for_item (SPItem * item,bool doTransformation, bool transformFull = true);
-bool   Ancetre (SPRepr * a, SPRepr * who);
+bool   Ancetre (Inkscape::XML::Node * a, Inkscape::XML::Node * who);
 
 void sp_selected_path_boolop (bool_op bop);
 void sp_selected_path_do_offset (bool expand, double prefOffset);
@@ -99,7 +100,7 @@ sp_selected_path_boolop (bool_op bop)
     if (!SP_IS_DESKTOP (desktop))
         return;
 
-    SPSelection *selection = SP_DT_SELECTION (desktop);
+    Inkscape::Selection *selection = SP_DT_SELECTION (desktop);
   
     GSList *il = (GSList *) selection->itemList();
   
@@ -123,8 +124,8 @@ sp_selected_path_boolop (bool_op bop)
     // mettre les elements de la liste dans l'ordre pour ces operations
     if (bop == bool_op_diff || bop == bool_op_symdiff || bop == bool_op_cut || bop == bool_op_slice) {
         // check in the tree to find which element of the selection list is topmost (for 2-operand commands only)
-        SPRepr *a = SP_OBJECT_REPR (il->data);
-        SPRepr *b = SP_OBJECT_REPR (il->next->data);
+        Inkscape::XML::Node *a = SP_OBJECT_REPR (il->data);
+        Inkscape::XML::Node *b = SP_OBJECT_REPR (il->next->data);
 
         if (a == NULL || b == NULL) {
             desktop->messageStack()->flash(Inkscape::ERROR_MESSAGE, _("Unable to determine the <b>z-order</b> of the objects selected for difference, XOR, division, or path cut."));
@@ -140,18 +141,18 @@ sp_selected_path_boolop (bool_op bop)
 
             // objects are not in parent/child relationship;
             // find their lowest common ancestor
-            SPRepr *dad = LCA (a, b);
+            Inkscape::XML::Node *dad = LCA (a, b);
             if (dad == NULL) {
                 desktop->messageStack()->flash(Inkscape::ERROR_MESSAGE, _("Unable to determine the <b>z-order</b> of the objects selected for difference, XOR, division, or path cut."));
                 return;
             }
 
             // find the children of the LCA that lead from it to the a and b
-            SPRepr *as = AncetreFils (a, dad);
-            SPRepr *bs = AncetreFils (b, dad);
+            Inkscape::XML::Node *as = AncetreFils (a, dad);
+            Inkscape::XML::Node *bs = AncetreFils (b, dad);
 
             // find out which comes first
-            for (SPRepr * child = dad->firstChild(); child; child = child->next()) {
+            for (Inkscape::XML::Node * child = dad->firstChild(); child; child = child->next()) {
                 if (child == as) {
                     // a en premier->mauvais sens
                     reverseOrderForOp = true;
@@ -408,7 +409,7 @@ sp_selected_path_boolop (bool_op bop)
     }
   
     // remember important aspects of the source path, to be restored
-    SPRepr *repr_source;
+    Inkscape::XML::Node *repr_source;
     if ( bop == bool_op_diff || bop == bool_op_symdiff || bop == bool_op_cut || bop == bool_op_slice ) {
         if (reverseOrderForOp) {
              repr_source = SP_OBJECT_REPR (il->data);
@@ -420,20 +421,20 @@ sp_selected_path_boolop (bool_op bop)
         GSList *sorted = 	g_slist_copy ((GSList *) selection->reprList());
 
         sorted = g_slist_sort (sorted, (GCompareFunc) sp_repr_compare_position);
-        repr_source = ((SPRepr *) sorted->data);
+        repr_source = ((Inkscape::XML::Node *) sorted->data);
         g_slist_free (sorted);
     }
-    gint pos = sp_repr_position (repr_source);
-    SPRepr *parent = sp_repr_parent (repr_source);
-    const char *id = sp_repr_attr (repr_source, "id");
-    const char *style = sp_repr_attr (repr_source, "style");
+    gint pos = repr_source->position();
+    Inkscape::XML::Node *parent = sp_repr_parent (repr_source);
+    const char *id = repr_source->attribute("id");
+    const char *style = repr_source->attribute("style");
     
 
     // remove source paths
     selection->clear();
     for (GSList *l = il; l != NULL; l = l->next) {
         // if this is the bottommost object,
-        if (!strcmp (sp_repr_attr (SP_OBJECT_REPR (l->data), "id"), id)) {
+        if (!strcmp (SP_OBJECT_REPR (l->data)->attribute("id"), id)) {
             // delete it so that its clones don't get alerted; this object will be restored shortly, with the same id
             SP_OBJECT (l->data)->deleteObject(false);
         } else {
@@ -478,7 +479,7 @@ sp_selected_path_boolop (bool_op bop)
         for (int i=0;i<nbRP;i++) {
             gchar *d = resPath[i]->svg_dump_path ();
       
-            SPRepr *repr = sp_repr_new ("svg:path");
+            Inkscape::XML::Node *repr = sp_repr_new ("svg:path");
             sp_repr_set_attr (repr, "style", style);
             sp_repr_set_attr (repr, "d", d);
             g_free (d);
@@ -503,12 +504,12 @@ sp_selected_path_boolop (bool_op bop)
             sp_repr_set_attr (repr, "transform", transform);
 
             // add the new repr to the parent
-            sp_repr_append_child (parent, repr);
+            parent->appendChild(repr);
 
             // move to the saved position 
-            sp_repr_set_position_absolute (repr, pos > 0 ? pos : 0);
+            repr->setPosition(pos > 0 ? pos : 0);
   
-            selection->addRepr(repr);
+            selection->add(repr);
             sp_repr_unref (repr);
 
             delete resPath[i];
@@ -518,7 +519,7 @@ sp_selected_path_boolop (bool_op bop)
     } else {
         gchar *d = res->svg_dump_path ();
 
-        SPRepr *repr = sp_repr_new ("svg:path");
+        Inkscape::XML::Node *repr = sp_repr_new ("svg:path");
         sp_repr_set_attr (repr, "style", style);
 
         sp_repr_set_attr (repr, "d", d);
@@ -527,10 +528,10 @@ sp_selected_path_boolop (bool_op bop)
         sp_repr_set_attr (repr, "transform", transform);
 
         sp_repr_set_attr (repr, "id", id);
-        sp_repr_append_child (parent, repr);
-        sp_repr_set_position_absolute (repr, pos > 0 ? pos : 0);
+        parent->appendChild(repr);
+        repr->setPosition(pos > 0 ? pos : 0);
      
-        selection->addRepr(repr);
+        selection->add(repr);
         sp_repr_unref (repr);
     }
 
@@ -547,7 +548,7 @@ sp_selected_path_outline ()
     if (!SP_IS_DESKTOP (desktop))
         return;
 
-    SPSelection *selection = SP_DT_SELECTION (desktop);
+    Inkscape::Selection *selection = SP_DT_SELECTION (desktop);
 
     if (selection->isEmpty()) {
         // TRANSLATORS: "to outline" means "to convert stroke to path"
@@ -573,7 +574,7 @@ sp_selected_path_outline ()
                 continue;
         }
         if (SP_IS_TEXT (item)) {
-            curve = sp_text_normalized_bpath (SP_TEXT (item));
+            curve = SP_TEXT (item)->getNormalizedBpath ();
             if (curve == NULL)
                 continue;
         }
@@ -615,7 +616,7 @@ sp_selected_path_outline ()
         }
 
         NR::Matrix const transform(item->transform);
-        gchar *style = g_strdup (sp_repr_attr (SP_OBJECT_REPR (item), "style"));
+        gchar *style = g_strdup (SP_OBJECT_REPR (item)->attribute("style"));
 
         float o_width, o_miter;
         JoinType o_join;
@@ -625,8 +626,8 @@ sp_selected_path_outline ()
             SPStyle *i_style = SP_OBJECT (item)->style;
             int jointype, captype;
 
-            jointype = i_style->stroke_linejoin.value;
-            captype = i_style->stroke_linecap.value;
+            jointype = i_style->stroke_linejoin.computed;
+            captype = i_style->stroke_linecap.computed;
             o_width = i_style->stroke_width.computed;
 
             switch (jointype) {
@@ -701,18 +702,18 @@ sp_selected_path_outline ()
 
         sp_curve_unref (curve);
         // remember the position of the item
-        gint pos = sp_repr_position (SP_OBJECT_REPR (item));
+        gint pos = SP_OBJECT_REPR (item)->position();
         // remember parent
-        SPRepr *parent = SP_OBJECT_REPR (item)->parent();
+        Inkscape::XML::Node *parent = SP_OBJECT_REPR (item)->parent();
         // remember id
-        const char *id = sp_repr_attr (SP_OBJECT_REPR (item), "id");
+        const char *id = SP_OBJECT_REPR (item)->attribute("id");
 
-        selection->removeItem (item);
+        selection->remove(item);
         SP_OBJECT (item)->deleteObject(false);
 
         if (res->descr_cmd.size() > 1) { // if there's 0 or 1 node left, drop this path altogether
 
-            SPRepr *repr = sp_repr_new ("svg:path");
+            Inkscape::XML::Node *repr = sp_repr_new ("svg:path");
 
             // restore old style
             sp_repr_set_attr (repr, "style", style);
@@ -727,17 +728,17 @@ sp_selected_path_outline ()
             g_free (str);
 
             // add the new repr to the parent
-            sp_repr_append_child (parent, repr);
+            parent->appendChild(repr);
 
             // move to the saved position 
-            sp_repr_set_position_absolute (repr, pos > 0 ? pos : 0);
+            repr->setPosition(pos > 0 ? pos : 0);
 
             sp_repr_set_attr (repr, "id", id);
 
             SPItem *newitem = (SPItem *) SP_DT_DOCUMENT (desktop)->getObjectByRepr(repr);
             sp_item_write_transform (newitem, repr, transform);
 
-            selection->addRepr (repr);
+            selection->add (repr);
 
             sp_repr_unref (repr);
         }
@@ -817,8 +818,8 @@ void sp_selected_path_create_updating_inset ()
 void
 sp_selected_path_create_offset_object (int expand, bool updating)
 {
-    SPSelection *selection;
-    SPRepr *repr;
+    Inkscape::Selection *selection;
+    Inkscape::XML::Node *repr;
     SPItem *item;
     SPCurve *curve;
     gchar *style, *str;
@@ -849,7 +850,7 @@ sp_selected_path_create_offset_object (int expand, bool updating)
     }
     if (SP_IS_TEXT (item))
     {
-        curve = sp_text_normalized_bpath (SP_TEXT (item));
+        curve = SP_TEXT (item)->getNormalizedBpath ();
         if (curve == NULL)
             return;
     }
@@ -858,12 +859,12 @@ sp_selected_path_create_offset_object (int expand, bool updating)
 
     sp_item_write_transform(item, SP_OBJECT_REPR(item), NR::identity());
 
-    style = g_strdup (sp_repr_attr (SP_OBJECT (item)->repr, "style"));
+    style = g_strdup (SP_OBJECT (item)->repr->attribute("style"));
 
     // remember the position of the item
-    gint pos = sp_repr_position (SP_OBJECT_REPR (item));
+    gint pos = SP_OBJECT_REPR (item)->position();
     // remember parent
-    SPRepr *parent = SP_OBJECT_REPR (item)->parent();
+    Inkscape::XML::Node *parent = SP_OBJECT_REPR (item)->parent();
   
     {
         SPStyle *i_style = SP_OBJECT (item)->style;
@@ -994,7 +995,7 @@ sp_selected_path_create_offset_object (int expand, bool updating)
         g_free (str);
     
         if ( updating ) {
-            const char *id = sp_repr_attr(SP_OBJECT(item)->repr,"id");
+            const char *id = SP_OBJECT(item)->repr->attribute("id");
             const char *uri = g_strdup_printf ("#%s", id);
             sp_repr_set_attr (repr, "xlink:href", uri);
             g_free ((void *) uri);
@@ -1005,10 +1006,10 @@ sp_selected_path_create_offset_object (int expand, bool updating)
         sp_repr_set_attr (repr, "style", style);
 
         // add the new repr to the parent
-        sp_repr_append_child (parent, repr);
+        parent->appendChild(repr);
 
         // move to the saved position 
-        sp_repr_set_position_absolute (repr, pos > 0 ? pos : 0);
+        repr->setPosition(pos > 0 ? pos : 0);
 
         SPItem *nitem = (SPItem *) SP_DT_DOCUMENT (desktop)->getObjectByRepr(repr);
 
@@ -1028,7 +1029,7 @@ sp_selected_path_create_offset_object (int expand, bool updating)
 
         sp_repr_unref (repr);
 
-        selection->setItem (nitem);
+        selection->set(nitem);
     }
   
     sp_document_done (SP_DT_DOCUMENT (desktop));
@@ -1057,7 +1058,7 @@ sp_selected_path_do_offset (bool expand, double prefOffset)
     if (!SP_IS_DESKTOP (desktop))
         return;
 
-    SPSelection *selection = SP_DT_SELECTION (desktop);
+    Inkscape::Selection *selection = SP_DT_SELECTION (desktop);
 
     if (selection->isEmpty()) {
         desktop->messageStack()->flash(Inkscape::WARNING_MESSAGE, _("Select <b>path(s)</b> to inset/outset."));
@@ -1082,7 +1083,7 @@ sp_selected_path_do_offset (bool expand, double prefOffset)
                 continue;
         }
         if (SP_IS_TEXT (item)) {
-            curve = sp_text_normalized_bpath (SP_TEXT (item));
+            curve = SP_TEXT (item)->getNormalizedBpath ();
             if (curve == NULL)
                 continue;
         }
@@ -1091,7 +1092,7 @@ sp_selected_path_do_offset (bool expand, double prefOffset)
 
         sp_item_write_transform(item, SP_OBJECT_REPR(item), NR::identity());
 
-        gchar *style = g_strdup (sp_repr_attr (SP_OBJECT_REPR (item), "style"));
+        gchar *style = g_strdup (SP_OBJECT_REPR (item)->attribute("style"));
 
         float o_width, o_miter;
         JoinType o_join;
@@ -1232,13 +1233,13 @@ sp_selected_path_do_offset (bool expand, double prefOffset)
 
         sp_curve_unref (curve);
         // remember the position of the item
-        gint pos = sp_repr_position (SP_OBJECT_REPR (item));
+        gint pos = SP_OBJECT_REPR (item)->position();
         // remember parent
-        SPRepr *parent = SP_OBJECT_REPR (item)->parent();
+        Inkscape::XML::Node *parent = SP_OBJECT_REPR (item)->parent();
         // remember id
-        const char *id = sp_repr_attr (SP_OBJECT_REPR (item), "id");
+        const char *id = SP_OBJECT_REPR (item)->attribute("id");
 
-        selection->removeItem (item);
+        selection->remove(item);
         SP_OBJECT (item)->deleteObject(false);
 
         if (res->descr_cmd.size() > 1) { // if there's 0 or 1 node left, drop this path altogether
@@ -1247,7 +1248,7 @@ sp_selected_path_do_offset (bool expand, double prefOffset)
 
             tstr[79] = '\0';
 
-            SPRepr *repr = sp_repr_new ("svg:path");
+            Inkscape::XML::Node *repr = sp_repr_new ("svg:path");
 
             sp_repr_set_attr (repr, "style", style);
 
@@ -1256,10 +1257,10 @@ sp_selected_path_do_offset (bool expand, double prefOffset)
             g_free (str);
 
             // add the new repr to the parent
-            sp_repr_append_child (parent, repr);
+            parent->appendChild(repr);
 
             // move to the saved position 
-            sp_repr_set_position_absolute (repr, pos > 0 ? pos : 0);
+            repr->setPosition(pos > 0 ? pos : 0);
 
             SPItem *newitem = (SPItem *) SP_DT_DOCUMENT (desktop)->getObjectByRepr(repr);
 
@@ -1268,7 +1269,7 @@ sp_selected_path_do_offset (bool expand, double prefOffset)
 
             sp_repr_set_attr (repr, "id", id);
 
-            selection->addRepr (repr);
+            selection->add (repr);
 
             sp_repr_unref (repr);
         }
@@ -1286,7 +1287,7 @@ sp_selected_path_do_offset (bool expand, double prefOffset)
 
 //return true if we changed something, else false
 bool
-sp_selected_path_simplify_item(SPDesktop *desktop, SPSelection *selection, SPItem *item,
+sp_selected_path_simplify_item(SPDesktop *desktop, Inkscape::Selection *selection, SPItem *item,
                  float threshold,  bool justCoalesce,
                  float angleLimit, bool breakableAngles,
                  gdouble size,     bool modifySelection)
@@ -1320,7 +1321,7 @@ sp_selected_path_simplify_item(SPDesktop *desktop, SPSelection *selection, SPIte
     }
 
     if (SP_IS_TEXT (item)) {
-        curve = sp_text_normalized_bpath (SP_TEXT (item));
+        curve = SP_TEXT (item)->getNormalizedBpath ();
         if (!curve)
             return false;
     }
@@ -1335,7 +1336,7 @@ sp_selected_path_simplify_item(SPDesktop *desktop, SPSelection *selection, SPIte
     */
     sp_item_write_transform(item, SP_OBJECT_REPR(item), NR::identity());
 
-    gchar *style = g_strdup (sp_repr_attr (SP_OBJECT_REPR (item), "style"));
+    gchar *style = g_strdup(SP_OBJECT_REPR (item)->attribute("style"));
 
     Path *orig = Path_for_item (item, false);
     if (orig == NULL) {
@@ -1346,15 +1347,15 @@ sp_selected_path_simplify_item(SPDesktop *desktop, SPSelection *selection, SPIte
 
     sp_curve_unref (curve);
     // remember the position of the item
-    gint pos = sp_repr_position (SP_OBJECT_REPR (item));
+    gint pos = SP_OBJECT_REPR (item)->position();
     // remember parent
-    SPRepr *parent = SP_OBJECT_REPR (item)->parent();
+    Inkscape::XML::Node *parent = SP_OBJECT_REPR (item)->parent();
     // remember id
-    const char *id = sp_repr_attr (SP_OBJECT_REPR (item), "id");
+    const char *id = SP_OBJECT_REPR (item)->attribute("id");
 
     //If a group was selected, to not change the selection list
     if (modifySelection)
-        selection->removeItem (item);
+        selection->remove(item);
         
     SP_OBJECT (item)->deleteObject(false);
 
@@ -1365,7 +1366,7 @@ sp_selected_path_simplify_item(SPDesktop *desktop, SPSelection *selection, SPIte
         orig->Simplify (threshold * size);
     }
 
-    SPRepr *repr = sp_repr_new ("svg:path");
+    Inkscape::XML::Node *repr = sp_repr_new ("svg:path");
 
     sp_repr_set_attr (repr, "style", style);
 
@@ -1377,10 +1378,10 @@ sp_selected_path_simplify_item(SPDesktop *desktop, SPSelection *selection, SPIte
     sp_repr_set_attr (repr, "id", id);
 
     // add the new repr to the parent
-    sp_repr_append_child (parent, repr);
+    parent->appendChild(repr);
 
     // move to the saved position 
-    sp_repr_set_position_absolute (repr, pos > 0 ? pos : 0);
+    repr->setPosition(pos > 0 ? pos : 0);
 
     SPItem *newitem = (SPItem *) SP_DT_DOCUMENT (desktop)->getObjectByRepr(repr);
 
@@ -1389,7 +1390,7 @@ sp_selected_path_simplify_item(SPDesktop *desktop, SPSelection *selection, SPIte
 
     //If we are not in a selected group
     if (modifySelection)
-        selection->addRepr (repr);
+        selection->add (repr);
 
     sp_repr_unref (repr);
 
@@ -1408,7 +1409,7 @@ sp_selected_path_simplify_selection (float threshold, bool justCoalesce,
     if (!SP_IS_DESKTOP (desktop))
         return;
 
-    SPSelection *selection = SP_DT_SELECTION (desktop);
+    Inkscape::Selection *selection = SP_DT_SELECTION (desktop);
 
     if (selection->isEmpty()) {
         desktop->messageStack()->flash(Inkscape::WARNING_MESSAGE,
@@ -1489,7 +1490,7 @@ sp_selected_path_simplify (void)
 // fonctions utilitaires
 
 bool
-Ancetre (SPRepr * a, SPRepr * who)
+Ancetre (Inkscape::XML::Node * a, Inkscape::XML::Node * who)
 {
     if (who == NULL || a == NULL)
         return false;
@@ -1512,7 +1513,7 @@ Path_for_item (SPItem * item, bool doTransformation, bool transformFull)
     }
     else if (SP_IS_TEXT (item))
     {
-        curve = sp_text_normalized_bpath (SP_TEXT (item));
+        curve = SP_TEXT (item)->getNormalizedBpath ();
     }
     else
     {
