@@ -6,6 +6,7 @@
  * Authors:
  *   Mitsuru Oka <oka326@parkcity.ne.jp>
  *   Lauris Kaplinski <lauris@kaplinski.com>
+ *   bulia byak <buliabyak@users.sf.net>
  *
  * Copyright (C) 1999-2002 Lauris Kaplinski
  * Copyright (C) 2001-2002 Mitsuru Oka
@@ -38,13 +39,13 @@
 #include "desktop.h"
 #include "desktop-style.h"
 #include "pixmaps/cursor-star.xpm"
-#include "sp-desktop-widget.h"
+#include "desktop-widget.h"
 #include "sp-metrics.h"
 #include <glibmm/i18n.h>
 #include "prefs-utils.h"
 #include "knotholder.h"
 #include "xml/repr.h"
-#include "xml/sp-repr-event-vector.h"
+#include "xml/node-event-vector.h"
 #include "object-edit.h"
 #include "widgets/spw-utilities.h"
 #include <libnr/nr-point-fns.h>
@@ -113,6 +114,9 @@ sp_star_context_init (SPStarContext * star_context)
     event_context->within_tolerance = false;
     event_context->item_to_select = NULL;
 
+    event_context->shape_repr = NULL;
+    event_context->shape_knot_holder = NULL;
+
     star_context->item = NULL;
 
     star_context->magnitude = 5;
@@ -126,21 +130,22 @@ static void
 sp_star_context_dispose (GObject *object)
 {
     SPEventContext *ec = SP_EVENT_CONTEXT (object);
-
     SPStarContext *sc = SP_STAR_CONTEXT (object);
 
+    ec->enableGrDrag(false);
+
     sc->sel_changed_connection.disconnect();
     sc->sel_changed_connection.~connection();
 
-    if (sc->knot_holder) {
-        sp_knot_holder_destroy (sc->knot_holder);
-        sc->knot_holder = NULL;
+    if (ec->shape_knot_holder) {
+        sp_knot_holder_destroy (ec->shape_knot_holder);
+        ec->shape_knot_holder = NULL;
     }
 
-    if (sc->repr) { // remove old listener
-        sp_repr_remove_listener_by_data (sc->repr, ec);
-        sp_repr_unref (sc->repr);
-        sc->repr = 0;
+    if (ec->shape_repr) { // remove old listener
+        sp_repr_remove_listener_by_data (ec->shape_repr, ec);
+        sp_repr_unref (ec->shape_repr);
+        ec->shape_repr = 0;
     }
 
     /* fixme: This is necessary because we do not grab */
@@ -153,29 +158,10 @@ sp_star_context_dispose (GObject *object)
     G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
-static void shape_event_attr_changed (SPRepr * repr, const gchar * name, const gchar * old_value, const gchar * new_value, bool is_interactive, gpointer data)
-{
-    SPStarContext *sc = SP_STAR_CONTEXT(data);
-    SPEventContext *ec = SP_EVENT_CONTEXT(sc);
-
-    if (sc->knot_holder) {
-        sp_knot_holder_destroy (sc->knot_holder);
-    }
-    sc->knot_holder = NULL;
-
-    SPDesktop *desktop = ec->desktop;
-
-    SPItem *item = SP_DT_SELECTION(desktop)->singleItem();
-
-    if (item) {
-        sc->knot_holder = sp_item_knot_holder (item, desktop);
-    }
-}
-
-static SPReprEventVector shape_repr_events = {
+static Inkscape::XML::NodeEventVector ec_shape_repr_events = {
     NULL, /* child_added */
     NULL, /* child_removed */
-    shape_event_attr_changed,
+    ec_shape_event_attr_changed,
     NULL, /* content_changed */
     NULL  /* order_changed */
 };
@@ -186,33 +172,33 @@ destroys old and creates new knotholder
 \param  selection Should not be NULL.
 */
 void
-sp_star_context_selection_changed (SPSelection * selection, gpointer data)
+sp_star_context_selection_changed (Inkscape::Selection * selection, gpointer data)
 {
     g_assert (selection != NULL);
 
     SPStarContext *sc = SP_STAR_CONTEXT (data);
     SPEventContext *ec = SP_EVENT_CONTEXT (sc);
 
-    if (sc->knot_holder) { // desktroy knotholder
-        sp_knot_holder_destroy (sc->knot_holder);
-        sc->knot_holder = NULL;
+    if (ec->shape_knot_holder) { // desktroy knotholder
+        sp_knot_holder_destroy (ec->shape_knot_holder);
+        ec->shape_knot_holder = NULL;
     }
 
-    if (sc->repr) { // remove old listener
-        sp_repr_remove_listener_by_data (sc->repr, ec);
-        sp_repr_unref (sc->repr);
-        sc->repr = 0;
+    if (ec->shape_repr) { // remove old listener
+        sp_repr_remove_listener_by_data (ec->shape_repr, ec);
+        sp_repr_unref (ec->shape_repr);
+        ec->shape_repr = 0;
     }
 
     SPItem *item = selection->singleItem();
     if (item) {
-        sc->knot_holder = sp_item_knot_holder (item, ec->desktop);
-        SPRepr *repr = SP_OBJECT_REPR (item);
-        if (repr) {
-            sc->repr = repr;
-            sp_repr_ref (repr);
-            sp_repr_add_listener (repr, &shape_repr_events, ec);
-            sp_repr_synthesize_events (repr, &shape_repr_events, ec);
+        ec->shape_knot_holder = sp_item_knot_holder (item, ec->desktop);
+        Inkscape::XML::Node *shape_repr = SP_OBJECT_REPR (item);
+        if (shape_repr) {
+            ec->shape_repr = shape_repr;
+            sp_repr_ref (shape_repr);
+            sp_repr_add_listener (shape_repr, &ec_shape_repr_events, ec);
+            sp_repr_synthesize_events (shape_repr, &ec_shape_repr_events, ec);
         }
     }
 }
@@ -231,17 +217,17 @@ sp_star_context_setup (SPEventContext *ec)
     sp_event_context_read (ec, "rounded");
     sp_event_context_read (ec, "randomized");
 
-    SPSelection *selection = SP_DT_SELECTION(ec->desktop);
+    Inkscape::Selection *selection = SP_DT_SELECTION(ec->desktop);
 
     SPItem *item = selection->singleItem();
         if (item) {
-            sc->knot_holder = sp_item_knot_holder (item, ec->desktop);
-            SPRepr *repr = SP_OBJECT_REPR (item);
-            if (repr) {
-                sc->repr = repr;
-                sp_repr_ref (repr);
-                sp_repr_add_listener (repr, &shape_repr_events, ec);
-                sp_repr_synthesize_events (repr, &shape_repr_events, ec);
+            ec->shape_knot_holder = sp_item_knot_holder (item, ec->desktop);
+            Inkscape::XML::Node *shape_repr = SP_OBJECT_REPR (item);
+            if (shape_repr) {
+                ec->shape_repr = shape_repr;
+                sp_repr_ref (shape_repr);
+                sp_repr_add_listener (shape_repr, &ec_shape_repr_events, ec);
+                sp_repr_synthesize_events (shape_repr, &ec_shape_repr_events, ec);
             }
         }
 
@@ -252,6 +238,10 @@ sp_star_context_setup (SPEventContext *ec)
         ec->enableSelectionCue();
     }
 
+    if (prefs_get_int_attribute("tools.shapes", "gradientdrag", 0) != 0) {
+        ec->enableGrDrag();
+    }
+
     sc->_message_context = new Inkscape::MessageContext(SP_VIEW(ec->desktop)->messageStack());
 }
 
@@ -283,6 +273,8 @@ sp_star_context_root_handler (SPEventContext * event_context, GdkEvent * event)
     static gboolean dragging;
 
     SPDesktop *desktop = event_context->desktop;
+    Inkscape::Selection *selection = SP_DT_SELECTION (desktop);
+
     SPStarContext *sc = SP_STAR_CONTEXT (event_context);
 
     event_context->tolerance = prefs_get_int_attribute_limited ("options.dragtolerance", "value", 0, 0, 100);
@@ -298,8 +290,8 @@ sp_star_context_root_handler (SPEventContext * event_context, GdkEvent * event)
             event_context->yp = (gint) event->button.y;
             event_context->within_tolerance = true;
 
-           // remember clicked item, disregarding groups
-            event_context->item_to_select = sp_desktop_item_at_point (desktop, NR::Point(event->button.x, event->button.y), TRUE);
+           // remember clicked item, disregarding groups, honoring Alt
+            event_context->item_to_select = sp_event_context_find_item (desktop, NR::Point(event->button.x, event->button.y), event->button.state, TRUE);
 
             dragging = TRUE;
             /* Position center */
@@ -340,10 +332,14 @@ sp_star_context_root_handler (SPEventContext * event_context, GdkEvent * event)
             sp_star_finish (sc);
             } else if (event_context->item_to_select) {
                 // no dragging, select clicked item if any
-                SP_DT_SELECTION(desktop)->setItem(event_context->item_to_select);
+                if (event->button.state & GDK_SHIFT_MASK) {
+                    selection->toggle(event_context->item_to_select);
+                } else {
+                    selection->set(event_context->item_to_select);
+                }
             } else {
                 // click in an empty space
-                SP_DT_SELECTION(desktop)->clear();
+                selection->clear();
             }
 
             event_context->item_to_select = NULL;
@@ -352,7 +348,7 @@ sp_star_context_root_handler (SPEventContext * event_context, GdkEvent * event)
         }
         break;
     case GDK_KEY_PRESS:
-        switch (event->key.keyval) {
+        switch (get_group0_keyval (&event->key)) {
         case GDK_Alt_R:
         case GDK_Control_L:
         case GDK_Control_R:
@@ -391,7 +387,7 @@ sp_star_context_root_handler (SPEventContext * event_context, GdkEvent * event)
         }
         break;
     case GDK_KEY_RELEASE:
-        switch (event->key.keyval) {
+        switch (get_group0_keyval (&event->key)) {
         case GDK_Alt_L:
         case GDK_Alt_R:
         case GDK_Control_L:
@@ -438,7 +434,7 @@ sp_star_drag(SPStarContext *sc, NR::Point p, guint state)
         }
 
         /* Create object */
-        SPRepr *repr = sp_repr_new ("svg:path");
+        Inkscape::XML::Node *repr = sp_repr_new ("svg:path");
         sp_repr_set_attr (repr, "sodipodi:type", "star");
 
         /* Set style */
@@ -472,8 +468,8 @@ sp_star_drag(SPStarContext *sc, NR::Point p, guint state)
     GString *rads = SP_PX_TO_METRIC_STRING(r1, sp_desktop_get_default_metric(desktop));
     sc->_message_context->setF(Inkscape::NORMAL_MESSAGE,
                                       ( sc->isflatsided?
-                                      _("<b>Polygon</b>: radius %s, angle %5g; with <b>Ctrl</b> to snap angle")
-                                          : _("<b>Star</b>: radius %s, angle %5g; with <b>Ctrl</b> to snap angle") ),
+                                      _("<b>Polygon</b>: radius %s, angle %5g&#176;; with <b>Ctrl</b> to snap angle")
+                                          : _("<b>Star</b>: radius %s, angle %5g&#176;; with <b>Ctrl</b> to snap angle") ),
                                       rads->str, sp_round ((arg1)*180/M_PI, 0.0001));
     g_string_free(rads, FALSE);
 }
@@ -491,7 +487,7 @@ sp_star_finish (SPStarContext * sc)
 
         object->updateRepr(NULL, SP_OBJECT_WRITE_EXT);
 
-        SP_DT_SELECTION(desktop)->setItem(sc->item);
+        SP_DT_SELECTION(desktop)->set(sc->item);
         sp_document_done(SP_DT_DOCUMENT(desktop));
 
         sc->item = NULL;
