@@ -8,7 +8,7 @@
 
 /* Extracted mechanically from http://www.w3.org/TR/SVG11/types.html#ColorKeywords:
  *
- *   tidy -wrap 999 < tmp/types.html 2> /dev/null |
+ *   tidy -wrap 999 < types.html 2> /dev/null |
  *     egrep '(prop|color-keyword)-value' |
  *     sed 's,<td><span class="prop-value">,    {",;s/<td><span class="color-keyword-value">rgb(/", {/;s%).*%}},@%;s%</span></td>%%' |
  *     tr -d \\n |
@@ -324,13 +324,25 @@ font_family_val(char const prop[], char const *const dummy_vals[])
     /* todo: unrecognized fonts, comma-separated lists. */
 }
 
+/**
+ * "unitful length" is used for font-size, which we always write with an explicit unit (typically
+ * `px') for better CSS interoperability.
+ */
+static void
+unitful_length_val(char const prop[], char const *const dummy_vals[])
+{
+    /* todo */
+    assert(dummy_vals == NULL);
+}
+
 static void
 length_val(char const prop[], char const *const dummy_vals[])
 {
     assert(dummy_vals == NULL);
     number_val(prop, 0., 6., 1e-7);
-    /* todo: units */
-    /* todo: exponential notation */
+    unitful_length_val(prop, dummy_vals);
+    /* todo: exponential notation.  See http://www.w3.org/TR/SVG11/types.html#DataTypeNumber for
+       where exponential notation is/isn't allowed. */
 }
 
 static void
@@ -383,7 +395,7 @@ test_style()
         {"fill-rule", "nonzero", "nonzero", enum_val, fill_rule_vals, true},
         {"font-family", "Bitstream Vera Sans", "Bitstream Vera Sans", font_family_val, NULL, true},
         // initial value depends on user agent
-        {"font-size", "medium", "medium", length_val, NULL, true},
+        {"font-size", "medium", "medium", unitful_length_val, NULL, true},
         // TODO: abs, rel, pcnt.
         {"font-stretch", "normal", "normal", enum_val, font_stretch_vals, true},
         {"font-style", "normal", "normal", enum_val, font_style_vals, true},
@@ -412,7 +424,44 @@ test_style()
         {"writing-mode", "lr-tb", "lr-tb", enum_val, writing_mode_vals, true}
     };
 
-    char const str0_all_exp[] = "font-size:medium;font-style:normal;font-variant:normal;font-weight:normal;font-stretch:normal;opacity:1.0000000;color:#000000;fill:#000000;fill-opacity:1.0000000;fill-rule:nonzero;stroke:none;stroke-width:1.0000000;stroke-linecap:butt;stroke-linejoin:miter;marker:none;marker-start:none;marker-mid:none;marker-end:none;stroke-miterlimit:4.0000000;stroke-dasharray:none;stroke-dashoffset:0;stroke-opacity:1.0000000;visibility:visible;display:inline;overflow:visible;font-family:Bitstream Vera Sans;letter-spacing:normal;text-anchor:start;writing-mode:lr-tb";
+    char const str0_all_exp[] =
+        "font-size:medium;"
+        "font-style:normal;"
+        "font-variant:normal;"
+        "font-weight:normal;"
+        "font-stretch:normal;"
+        "text-indent:0.0000000;"
+        "text-align:start;"
+        "text-decoration:none;"
+        "line-height:normal;"
+        "letter-spacing:normal;"
+        "word-spacing:normal;"
+        "text-transform:none;"
+        "direction:ltr;"
+        "block-progression:tb;"
+        "writing-mode:lr-tb;"
+        "text-anchor:start;"
+        "opacity:1.0000000;"
+        "color:#000000;"
+        "fill:#000000;"
+        "fill-opacity:1.0000000;"
+        "fill-rule:nonzero;"
+        "stroke:none;"
+        "stroke-width:1.0000000;"
+        "stroke-linecap:butt;"
+        "stroke-linejoin:miter;"
+        "marker:none;"
+        "marker-start:none;"
+        "marker-mid:none;"
+        "marker-end:none;"
+        "stroke-miterlimit:4.0000000;"
+        "stroke-dasharray:none;"
+        "stroke-dashoffset:0;"
+        "stroke-opacity:1.0000000;"
+        "visibility:visible;"
+        "display:inline;"
+        "overflow:visible;"
+        "font-family:Bitstream Vera Sans";
 
     utest_start("style");
     UTEST_TEST("sp_style_new, sp_style_write_string") {
@@ -421,6 +470,7 @@ test_style()
         gchar *str0_set = sp_style_write_string(style, SP_STYLE_FLAG_IFSET);
         UTEST_ASSERT(*str0_set == '\0');
         UTEST_ASSERT(streq(str0_all, str0_all_exp));
+        printf(str0_all);
         g_free(str0_all);
         g_free(str0_set);
         sp_style_unref(style);
@@ -472,6 +522,26 @@ test_style()
         g_free(str0_set);
     }
     g_log_remove_handler(NULL, log_handler_id);
+
+    /* The following tests involve invalid style, but aren't expected to trigger
+       g_log calls. */
+    /* invalid color setting */
+    {
+        char const *bad[] = {"#4321", "currentColor", "#87654321", "#42", "aquam"};
+        for (unsigned i = 0; i < G_N_ELEMENTS(bad); ++i) {
+            gchar *tst_name = g_strdup_printf("invalid color setting: %s", bad[i]);
+            gchar *style_str = g_strdup_printf("color:%s;color:#123;color:%s",
+                                               bad[i], bad[i]);
+            UTEST_TEST(tst_name) {
+                gchar *str0_set = merge_then_write_string(style_str, SP_STYLE_FLAG_IFSET);
+                UTEST_ASSERT(streq(str0_set, "color:#112233"));
+                g_free(str0_set);
+            }
+            g_free(style_str);
+            g_free(tst_name);
+        }
+    }
+
     /* End of invalid style string examples. */
 
 
