@@ -23,11 +23,15 @@ class SPSVGViewClass;
 #define SP_IS_SVG_VIEW_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), SP_TYPE_SVG_VIEW))
 
 #include "display/display-forward.h"
-#include "view.h"
+#include "ui/view/view.h"
 
 
+void sp_svg_view_set_scale (SPSVGView *view, gdouble hscale, gdouble vscale);
+void sp_svg_view_set_rescale (SPSVGView *view, gboolean rescale, gboolean keepaspect, gdouble width, gdouble height);
+
 
 struct SPSVGView : public SPView {
+    public:
 	unsigned int dkey;
 
 	SPCanvasGroup *parent;
@@ -38,6 +42,16 @@ struct SPSVGView : public SPView {
 	/* Whether to rescale automatically */
 	gboolean rescale, keepaspect;
 	gdouble width, height;
+
+    // C++ Wrappers to functions
+    void setScale(gdouble hscale, gdouble vscale) {
+	sp_svg_view_set_scale(this, hscale, vscale);
+    }
+
+    void setRescale(gboolean rescale, gboolean keepAspect, gdouble width, gdouble height) {
+	sp_svg_view_set_rescale(this, rescale, keepAspect, width, height);
+    }
+
 };
 
 struct SPSVGViewClass {
@@ -48,9 +62,6 @@ GtkType sp_svg_view_get_type (void);
 
 SPView *sp_svg_view_new (SPCanvasGroup *parent);
 
-void sp_svg_view_set_scale (SPSVGView *view, gdouble hscale, gdouble vscale);
-void sp_svg_view_set_rescale (SPSVGView *view, gboolean rescale, gboolean keepaspect, gdouble width, gdouble height);
-
 /* SPSVGViewWidget */
 
 class SPSVGViewWidget;
@@ -62,7 +73,11 @@ class SPSVGViewWidgetClass;
 #define SP_IS_SVG_VIEW_WIDGET(obj) (GTK_CHECK_TYPE ((obj), SP_TYPE_SVG_VIEW_WIDGET))
 #define SP_IS_SVG_VIEW_WIDGET_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), SP_TYPE_SVG_VIEW_WIDGET))
 
+void sp_svg_view_widget_set_resize (SPSVGViewWidget *vw, gboolean resize, gdouble width, gdouble height);
+
+
 struct SPSVGViewWidget {
+    public:
 	SPViewWidget widget;
 
 	GtkWidget *sw;
@@ -71,6 +86,11 @@ struct SPSVGViewWidget {
 	/* Whether to resize automatically */
 	gboolean resize;
 	gdouble maxwidth, maxheight;
+
+    // C++ Wrappers
+    void setResize(gboolean resize, gdouble width, gdouble height) {
+	sp_svg_view_widget_set_resize(this, resize, width, height);
+    }
 };
 
 struct SPSVGViewWidgetClass {
@@ -81,8 +101,5 @@ GtkType sp_svg_view_widget_get_type (void);
 
 GtkWidget *sp_svg_view_widget_new (SPDocument *doc);
 
-void sp_svg_view_widget_set_resize (SPSVGViewWidget *vw, gboolean resize, gdouble width, gdouble height);
-
-
 
 #endif
