@@ -31,13 +31,17 @@ void sp_commands_toolbox_set_desktop(GtkWidget *toolbox, SPDesktop *desktop);
 void show_aux_toolbox(GtkWidget *toolbox);
 
 GtkWidget *sp_toolbox_button_normal_new_from_verb(GtkWidget *t,
-                                                  unsigned size,
+                                                  GtkIconSize size,
                                                   Inkscape::Verb * verb,
                                                   SPView *view,
                                                   GtkTooltips *tt);
 
 void aux_toolbox_space(GtkWidget *tb, gint space);
 
+// utility
+void sp_toolbox_add_label(GtkWidget *tbl, gchar const *title, bool wide = true);
+
+
 
 #endif /* !SEEN_TOOLBOX_H */
 
