@@ -11,7 +11,9 @@
 
 #include <string.h>
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
 
 #include "inkscape-private.h"
 #include "file.h"
@@ -28,15 +30,23 @@
 
 #include "select-context.h"
 #include "node-context.h"
+#include "sp-path.h"
 #include "rect-context.h"
+#include "sp-rect.h"
 #include "arc-context.h"
+#include "sp-ellipse.h"
 #include "star-context.h"
+#include "sp-star.h"
 #include "spiral-context.h"
+#include "sp-spiral.h"
 #include "draw-context.h"
 #include "dyna-draw-context.h"
 #include "pen-context.h"
 #include "pencil-context.h"
 #include "text-context.h"
+#include "sp-text.h"
+#include "sp-flowtext.h"
+#include "gradient-context.h"
 #include "zoom-context.h"
 #include "dropper-context.h"
 
@@ -54,6 +64,7 @@ static char const *tool_names[] = {
   "tools.freehand.pen",
   "tools.calligraphic",
   "tools.text",
+  "tools.gradient",
   "tools.zoom",
   "tools.dropper",
   NULL
@@ -71,6 +82,7 @@ static char const *tool_ids[] = {
   "pen",
   "calligraphic",
   "text",
+  "gradient",
   "zoom",
   "dropper",
   NULL
@@ -94,27 +106,23 @@ tools_isactive (SPDesktop *dt, unsigned num)
 {
 	g_assert( num < G_N_ELEMENTS(tool_ids) );
 	if (SP_IS_EVENT_CONTEXT(dt->event_context))
-		return (!strcmp (sp_repr_attr (dt->event_context->repr, "id"), tool_ids[num]));
+		return (!strcmp (dt->event_context->prefs_repr->attribute("id"), tool_ids[num]));
 	else return FALSE;
 }
 
 int
 tools_active (SPDesktop *dt)
 {
-	return (tools_id2num (sp_repr_attr (dt->event_context->repr, "id")));
+	return (tools_id2num (dt->event_context->prefs_repr->attribute("id")));
 }
 
 void
 tools_switch (SPDesktop *dt, int num)
 {
-
-  if (dt)
-    {
-      dt->_tool_changed.emit(num);
-    }
-  
-  
-  
+	if (dt) {
+		dt->_tool_changed.emit(num);
+	}
+ 
 	switch (num) {
 	case TOOLS_SELECT:
 		sp_desktop_set_event_context (dt, SP_TYPE_SELECT_CONTEXT, tool_names[num]);
@@ -156,13 +164,13 @@ tools_switch (SPDesktop *dt, int num)
 		sp_desktop_set_event_context (dt, SP_TYPE_PENCIL_CONTEXT, tool_names[num]);
 		sp_desktop_activate_guides (dt, FALSE);
 		inkscape_eventcontext_set (SP_DT_EVENTCONTEXT (dt));
-		SP_VIEW(dt)->tipsMessageContext()->set(Inkscape::NORMAL_MESSAGE, _("<b>Drag</b> to create a freehand line. Press <b>a</b> to toggle Append/New."));
+		SP_VIEW(dt)->tipsMessageContext()->set(Inkscape::NORMAL_MESSAGE, _("<b>Drag</b> to create a freehand line. Start drawing with <b>Shift</b> to append to selected path."));
 		break;
 	case TOOLS_FREEHAND_PEN:
 		sp_desktop_set_event_context (dt, SP_TYPE_PEN_CONTEXT, tool_names[num]);
 		sp_desktop_activate_guides (dt, FALSE);
 		inkscape_eventcontext_set (SP_DT_EVENTCONTEXT (dt));
-		SP_VIEW(dt)->tipsMessageContext()->set(Inkscape::NORMAL_MESSAGE, _("<b>Click</b> to create a node, <b>click and drag</b> to create a smooth node. Press <b>a</b> to toggle Append/New."));
+		SP_VIEW(dt)->tipsMessageContext()->set(Inkscape::NORMAL_MESSAGE, _("<b>Click</b> to create a node, <b>click and drag</b> to create a smooth node. Start drawing with <b>Shift</b> to append to selected path."));
 		break;
 	case TOOLS_CALLIGRAPHIC:
 		sp_desktop_set_event_context (dt, SP_TYPE_DYNA_DRAW_CONTEXT, tool_names[num]);
@@ -174,7 +182,13 @@ tools_switch (SPDesktop *dt, int num)
 		sp_desktop_set_event_context (dt, SP_TYPE_TEXT_CONTEXT, tool_names[num]);
 		sp_desktop_activate_guides (dt, FALSE);
 		inkscape_eventcontext_set (SP_DT_EVENTCONTEXT (dt));
-		SP_VIEW(dt)->tipsMessageContext()->set(Inkscape::NORMAL_MESSAGE, _("<b>Click</b> to select or create text object, then type."));
+		SP_VIEW(dt)->tipsMessageContext()->set(Inkscape::NORMAL_MESSAGE, _("<b>Click</b> to select or create text, <b>drag</b> to create flowed text; then type."));
+		break;
+	case TOOLS_GRADIENT:
+		sp_desktop_set_event_context (dt, SP_TYPE_GRADIENT_CONTEXT, tool_names[num]);
+		sp_desktop_activate_guides (dt, FALSE);
+		inkscape_eventcontext_set (SP_DT_EVENTCONTEXT (dt));
+		SP_VIEW(dt)->tipsMessageContext()->set(Inkscape::NORMAL_MESSAGE, _("<b>Drag</b> or <b>double click</b> to create a gradient on selected objects, <b>drag handles</b> to adjust gradients."));
 		break;
 	case TOOLS_ZOOM:
 		sp_desktop_set_event_context (dt, SP_TYPE_ZOOM_CONTEXT, tool_names[num]);
@@ -186,7 +200,7 @@ tools_switch (SPDesktop *dt, int num)
 		sp_desktop_set_event_context (dt, SP_TYPE_DROPPER_CONTEXT, tool_names[num]);
 		sp_desktop_activate_guides (dt, FALSE);
 		inkscape_eventcontext_set (SP_DT_EVENTCONTEXT (dt));
-		SP_VIEW(dt)->tipsMessageContext()->set(Inkscape::NORMAL_MESSAGE, _("<b>Click</b> to set fill color, <b>Shift+click</b> to set stroke color. <b>Click and drag</b> to pick the average color of an area. <b>Ctrl+C</b> to copy the color under mouse to clipboard."));
+		SP_VIEW(dt)->tipsMessageContext()->set(Inkscape::NORMAL_MESSAGE, _("<b>Click</b> to set fill, <b>Shift+click</b> to set stroke; <b>drag</b> to average color in area; with <b>Alt</b> to pick inverse color; <b>Ctrl+C</b> to copy the color under mouse to clipboard"));
 		break;
 	}
 }
@@ -198,3 +212,20 @@ tools_switch_current (int num)
 	dt = SP_ACTIVE_DESKTOP;
 	if (dt) tools_switch (dt, num);
 }
+
+void tools_switch_by_item (SPDesktop *dt, SPItem *item)
+{
+	if (SP_IS_RECT(item)) {
+		tools_switch (dt, TOOLS_SHAPES_RECT);
+	} else if (SP_IS_GENERICELLIPSE(item)) {
+		tools_switch (dt, TOOLS_SHAPES_ARC);
+	} else if (SP_IS_STAR(item)) {
+		tools_switch (dt, TOOLS_SHAPES_STAR);
+	} else if (SP_IS_SPIRAL(item)) {
+		tools_switch (dt, TOOLS_SHAPES_SPIRAL);
+	} else if (SP_IS_PATH(item)) {
+		tools_switch (dt, TOOLS_NODES);
+	} else if (SP_IS_TEXT(item) || SP_IS_FLOWTEXT(item))  {
+		tools_switch (dt, TOOLS_TEXT);
+	}
+}
