@@ -26,6 +26,7 @@ enum {
   TOOLS_FREEHAND_PEN,
   TOOLS_CALLIGRAPHIC,
   TOOLS_TEXT,
+  TOOLS_GRADIENT,
   TOOLS_ZOOM,
   TOOLS_DROPPER
 };
@@ -34,7 +35,7 @@ int tools_isactive(SPDesktop *dt, unsigned num);
 int tools_active(SPDesktop *dt);
 void tools_switch(SPDesktop *dt, int num);
 void tools_switch_current(int num);
-
+void tools_switch_by_item (SPDesktop *dt, SPItem *item);
 
 #endif /* !SEEN_TOOLS_SWITCH_H */
 
