@@ -7,7 +7,7 @@
  * Copyright (C) 2004 Bob Jamison
  *
  * Released under GNU GPL, read the file 'COPYING' for more information
- * 
+ *
  * Potrace, the wonderful tracer located at http://potrace.sourceforge.net,
  * is provided by the generosity of Peter Selinger, to whom we are grateful.
  *
@@ -16,12 +16,6 @@
 #include "inkscape-potrace.h"
 
 
-#include "main.h"
-#include "greymap.h"
-#include "curve.h"
-#include "path.h"
-#include "bitmap.h"
-
 #include "trace/filterset.h"
 #include "trace/imagemap-gdk.h"
 
@@ -34,12 +28,14 @@
 #include <svg/stringstream.h>
 #include <xml/repr.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
+#include <glibmm/i18n.h>
 
 #include <gtkmm.h>
 
-/** BOB added these two small items **/
-PotraceStatusFunc potraceStatusFunc = NULL;
-void   *potraceStatusUserData       = NULL;
+#include "greymap.h"
+#include "curve.h"
+#include "potracelib.h"
+#include "bitmap.h"
 
 static void updateGui()
 {
@@ -51,28 +47,23 @@ static void updateGui()
 }
 
 
-static int potraceStatus(char *msg, void *userData)
+static void potraceStatusCallback(double progress, void *userData) /* callback fn */
 {
     updateGui();
 
-    if (!msg || !userData)
-        return 0;
+    if (!userData)
+        return;
 
-    //g_message("msg:%s\n", msg);
-
-    Inkscape::Trace::Potrace::PotraceTracingEngine *engine =
-          (Inkscape::Trace::Potrace::PotraceTracingEngine *)userData;
-    
-    return engine->keepGoing;
+    //g_message("progress: %f\n", progress);
 
+    //Inkscape::Trace::Potrace::PotraceTracingEngine *engine =
+    //      (Inkscape::Trace::Potrace::PotraceTracingEngine *)userData;
 }
 
 
 
 
 //required by potrace
-struct info info;
-
 namespace Inkscape {
 
 namespace Trace {
@@ -97,41 +88,6 @@ PotraceTracingEngine::PotraceTracingEngine()
     cannyHighThreshold   = 0.65;
 
 
-    //##### Potrace's defaults
-    //backend_lookup("eps", &info.backend);
-#define UNDEF ((double)(1e30))   /* a value to represent "undefined" */
-    info.debug        = 0;
-    info.width_d.x    = UNDEF;
-    info.height_d.x   = UNDEF;
-    info.rx           = UNDEF;
-    info.ry           = UNDEF;
-    info.sx           = UNDEF;
-    info.sy           = UNDEF;
-    info.stretch      = 1;
-    info.lmar_d.x     = UNDEF;
-    info.rmar_d.x     = UNDEF;
-    info.tmar_d.x     = UNDEF;
-    info.bmar_d.x     = UNDEF;
-    info.angle        = 0;
-    info.paperwidth   = DEFAULT_PAPERWIDTH;
-    info.paperheight  = DEFAULT_PAPERHEIGHT;
-    info.turdsize     = 2;
-    info.unit         = 10;
-    info.compress     = 0; //not applicable
-    info.pslevel      = 2;
-    info.color        = 0x000000;
-    info.turnpolicy   = POLICY_MINORITY;
-    info.gamma        = 2.2;
-    info.opticurve    = 1;
-    info.longcoding   = 0;
-    info.alphamax     = 1.0;
-    info.opttolerance = 0.2;
-    info.outfile      = NULL;
-    info.blacklevel   = 0.5;
-    info.invert       = 0;
-    info.opaque       = 0;
-    info.group        = 0;
-    info.fillcolor    = 0xffffff;
 }
 
 
@@ -164,20 +120,22 @@ hasPoint(std::vector<Point> &points, double x, double y)
 /**
  *  Recursively descend the path_t node tree, writing paths in SVG
  *  format into the output stream.  The Point vector is used to prevent
- *  redundant paths.
+ *  redundant paths.  Returns number of paths processed.
  */
-static void
-writePaths(PotraceTracingEngine *engine, path_t *plist,
+static long
+writePaths(PotraceTracingEngine *engine, potrace_path_t *plist,
             Inkscape::SVGOStringStream& data, std::vector<Point> &points)
 {
+    long nodeCount = 0L;
 
-    path_t *node;
+    potrace_path_t *node;
     for (node=plist; node ; node=node->sibling)
         {
+        potrace_curve_t *curve = &(node->curve);
         //g_message("node->fm:%d\n", node->fm);
-        if (!node->fm)
+        if (!curve->n)
             continue;
-        dpoint_t *pt = node->fcurve[node->fm -1].c;
+        dpoint_t *pt = curve->c[curve->n - 1];
         double x0 = 0.0;
         double y0 = 0.0;
         double x1 = 0.0;
@@ -197,24 +155,26 @@ writePaths(PotraceTracingEngine *engine, path_t *plist,
             points.push_back(p);
             }
         data << "M " << x2 << " " << y2 << " ";
-        for (int i=0 ; i<node->fm ; i++)
+        nodeCount++;
+
+        for (int i=0 ; i<curve->n ; i++)
             {
-            if (!potraceStatus("wp", (void *)engine))
-                return;
-            pt = node->fcurve[i].c;
+            if (!engine->keepGoing)
+                return 0L;
+            pt = curve->c[i];
             x0 = pt[0].x;
             y0 = pt[0].y;
             x1 = pt[1].x;
             y1 = pt[1].y;
             x2 = pt[2].x;
             y2 = pt[2].y;
-            switch (node->fcurve[i].tag)
+            switch (curve->tag[i])
                 {
-                case CORNER:
+                case POTRACE_CORNER:
                     data << "L " << x1 << " " << y1 << " " ;
                     data << "L " << x2 << " " << y2 << " " ;
                 break;
-                case CURVETO:
+                case POTRACE_CURVETO:
                     data << "C " << x0 << " " << y0 << " "
                                  << x1 << " " << y1 << " "
                                  << x2 << " " << y2 << " ";
@@ -223,14 +183,18 @@ writePaths(PotraceTracingEngine *engine, path_t *plist,
                 default:
                 break;
                 }
+            nodeCount++;
             }
         data << "z";
 
         for (path_t *child=node->childlist; child ; child=child->sibling)
             {
-            writePaths(engine, child, data, points);
+            nodeCount += writePaths(engine, child, data, points);
             }
         }
+
+    return nodeCount;
+
 }
 
 
@@ -257,7 +221,7 @@ filter(PotraceTracingEngine &engine, GdkPixbuf * pixbuf)
         }
 
     /*### Brightness threshold ###*/
-    else if ( engine.getTraceType() == TRACE_BRIGHTNESS || 
+    else if ( engine.getTraceType() == TRACE_BRIGHTNESS ||
               engine.getTraceType() == TRACE_BRIGHTNESS_MULTI )
         {
         GrayMap *gm = gdkPixbufToGrayMap(pixbuf);
@@ -401,18 +365,25 @@ PotraceTracingEngine::preview(GdkPixbuf * pixbuf)
 
 
 //*This is the core inkscape-to-potrace binding
-char *PotraceTracingEngine::grayMapToPath(GrayMap *grayMap)
+char *PotraceTracingEngine::grayMapToPath(GrayMap *grayMap, long *nodeCount)
 {
 
-    bitmap_t *bm = bm_new(grayMap->width, grayMap->height);
-    bm_clear(bm, 0);
+    /* get default parameters */
+    potrace_param_t *potraceParams = potrace_param_default();
+
+    potraceParams->progress.callback = potraceStatusCallback;
+    potraceParams->progress.data     = (void *)this;
+
+    potrace_bitmap_t *potraceBitmap = bm_new(grayMap->width, grayMap->height);
+    bm_clear(potraceBitmap, 0);
 
     //##Read the data out of the GrayMap
     for (int y=0 ; y<grayMap->height ; y++)
         {
         for (int x=0 ; x<grayMap->width ; x++)
             {
-            BM_UPUT(bm, x, y, grayMap->getPixel(grayMap, x, y) ? 0 : 1);
+            BM_UPUT(potraceBitmap, x, y, 
+                  grayMap->getPixel(grayMap, x, y) ? 0 : 1);
             }
         }
 
@@ -429,30 +400,18 @@ char *PotraceTracingEngine::grayMapToPath(GrayMap *grayMap)
         return NULL;
         }
 
-    /* process the image */
-    path_t *plist;
-    int ret = bm_to_pathlist(bm, &plist);
-    if (ret)
-        {
-        g_warning("Potrace::convertImageToPath: trouble tracing temp image\n");
-        return NULL;
-        }
+    /* trace a bitmap*/
+    potrace_state_t *potraceState = potrace_trace(potraceParams, 
+			                          potraceBitmap);
 
     //## Free the Potrace bitmap
-    bm_free(bm);
-
-    ret = process_path(plist);
-    if (ret)
-        {
-        g_warning("Potrace::convertImageToPath: trouble processing trace\n");
-        return NULL;
-        }
-
+    bm_free(potraceBitmap);
 
     if (!keepGoing)
         {
         g_warning("aborted");
-        pathlist_free(plist);
+        potrace_state_free(potraceState);
+        potrace_param_free(potraceParams);
         return NULL;
         }
 
@@ -462,15 +421,19 @@ char *PotraceTracingEngine::grayMapToPath(GrayMap *grayMap)
 
     //## copy the path information into our d="" attribute string
     std::vector<Point> points;
-    writePaths(this, plist, data, points);
+    long thisNodeCount = writePaths(this, potraceState->plist, data, points);
 
-    //# we are done with the pathlist
-    pathlist_free(plist);
+    /* free a potrace items */
+    potrace_state_free(potraceState);
+    potrace_param_free(potraceParams);
 
     if (!keepGoing)
         return NULL;
 
-    char *d     = strdup((char *)data.str().c_str());
+    char *d = strdup((char *)data.str().c_str());
+    if ( nodeCount)
+        *nodeCount = thisNodeCount;
+
     
     return d;
 
@@ -489,29 +452,30 @@ PotraceTracingEngine::traceSingle(GdkPixbuf * thePixbuf, int *nrPaths)
         return NULL;
 
     brightnessFloor = 0.0; //important to set this
-    
+
     GrayMap *grayMap = filter(*this, thePixbuf);
     if (!grayMap)
         return NULL;
 
-    char *d = grayMapToPath(grayMap);
-    
+    long nodeCount;
+    char *d = grayMapToPath(grayMap, &nodeCount);
+
     grayMap->destroy(grayMap);
-    
+
     if (!d)
         {
         *nrPaths = 0;
         return NULL;
         }
     char *style = "fill:#000000";
-    
+
     //g_message("### GOT '%s' \n", d);
-    TracingEngineResult *result = new TracingEngineResult(style, d);
+    TracingEngineResult *result = new TracingEngineResult(style, d, nodeCount);
 
     free(d);
-    
+
     *nrPaths = 1;
-    
+
     return result;
 }
 
@@ -525,44 +489,47 @@ PotraceTracingEngine::traceBrightnessMulti(GdkPixbuf * thePixbuf, int *nrPaths)
 
     if (!thePixbuf)
         return NULL;
-        
+
     double low     = 0.2; //bottom of range
     double high    = 0.9; //top of range
     double delta   = (high - low ) / ((double)multiScanNrColors);
-    
+
     brightnessFloor = 0.0; //Set bottom to black
 
+    int traceCount = 0;
+
     TracingEngineResult *results = NULL;
     for ( brightnessThreshold = low ;
           brightnessThreshold <= high ;
           brightnessThreshold += delta)
-    
+
         {
-        
+
         GrayMap *grayMap = filter(*this, thePixbuf);
         if (!grayMap)
             return NULL;
 
-        char *d = grayMapToPath(grayMap);
-    
+        long nodeCount;
+        char *d = grayMapToPath(grayMap, &nodeCount);
+
         grayMap->destroy(grayMap);
-    
+
         if (!d)
             {
             *nrPaths = 0;
             return NULL;
             }
-            
+
         int grayVal = (int)(256.0 * brightnessThreshold);
         char style[31];
         sprintf(style, "fill-opacity:1.0;fill:#%02x%02x%02x",
                     grayVal, grayVal, grayVal);
-    
+
         //g_message("### GOT '%s' \n", d);
-        TracingEngineResult *result = new TracingEngineResult(style, d);
-        
+        TracingEngineResult *result = new TracingEngineResult(style, d, nodeCount);
+
         free(d);
-    
+
         if (!results)
             {
             results = result; //first one
@@ -579,12 +546,19 @@ PotraceTracingEngine::traceBrightnessMulti(GdkPixbuf * thePixbuf, int *nrPaths)
         if (!multiScanStack)
             brightnessFloor = brightnessThreshold;
 
+        SPDesktop *desktop = SP_ACTIVE_DESKTOP;
+        if (desktop)
+            {
+            gchar *msg = g_strdup_printf(_("Trace: %d.  %ld nodes"), traceCount++, nodeCount);
+            desktop->messageStack()->flash(Inkscape::NORMAL_MESSAGE, msg);
+            g_free(msg);
+            }
         }
-    
+
     //report the count of paths processed
     *nrPaths = multiScanNrColors;
 
-    
+
     return results;
 }
 
@@ -604,17 +578,17 @@ PotraceTracingEngine::traceQuant(GdkPixbuf * thePixbuf, int *nrPaths)
         return NULL;
 
     TracingEngineResult *results = NULL;
-    
+
     //Create and clear a gray map
     GrayMap *gm = GrayMapCreate(iMap->width, iMap->height);
     for (int row=0 ; row<gm->height ; row++)
         for (int col=0 ; col<gm->width ; col++)
             gm->setPixel(gm, col, row, GRAYMAP_WHITE);
 
-    
+
     for (int colorIndex=0 ; colorIndex<iMap->nrColors ; colorIndex++)
         {
-        
+
         /*Make a gray map for each color index */
         for (int row=0 ; row<iMap->height ; row++)
             {
@@ -634,24 +608,25 @@ PotraceTracingEngine::traceQuant(GdkPixbuf * thePixbuf, int *nrPaths)
             }
 
         //## Now we have a traceable graymap
-        char *d = grayMapToPath(gm);
-    
+        long nodeCount;
+        char *d = grayMapToPath(gm, &nodeCount);
+
         if (!d)
             {
             *nrPaths = 0;
             return NULL;
             }
-        
+
         //### get style info
         char style[13];
         RGB rgb = iMap->clut[colorIndex];
         sprintf(style, "fill:#%02x%02x%02x", rgb.r, rgb.g, rgb.b);
-    
+
         //g_message("### GOT '%s' \n", d);
-        TracingEngineResult *result = new TracingEngineResult(style, d);
+        TracingEngineResult *result = new TracingEngineResult(style, d, nodeCount);
 
         free(d);
-    
+
         if (!results)
             {
             results = result; //first one
@@ -663,7 +638,7 @@ PotraceTracingEngine::traceQuant(GdkPixbuf * thePixbuf, int *nrPaths)
             result->next = results;
             results = result;
             */
-            
+
             //append
             TracingEngineResult *r;
             for (r=results ; r->next ; r=r->next)
@@ -672,15 +647,23 @@ PotraceTracingEngine::traceQuant(GdkPixbuf * thePixbuf, int *nrPaths)
             }
 
 
+        SPDesktop *desktop = SP_ACTIVE_DESKTOP;
+        if (desktop)
+            {
+            gchar *msg = g_strdup_printf(_("Trace: %d.  %ld nodes"), colorIndex, nodeCount);
+            desktop->messageStack()->flash(Inkscape::NORMAL_MESSAGE, msg);
+            g_free(msg);
+            }
+
 
         }
-    
+
     //report the count of paths processed
     *nrPaths = iMap->nrColors;
 
     gm->destroy(gm);
-    iMap->destroy(iMap);  
-    
+    iMap->destroy(iMap);
+
     return results;
 }
 
@@ -697,8 +680,6 @@ PotraceTracingEngine::trace(GdkPixbuf * thePixbuf, int *nrPaths)
 
     //Set up for messages
     keepGoing             = 1;
-    potraceStatusFunc     = potraceStatus;
-    potraceStatusUserData = (void *)this;
 
     if ( traceType == TRACE_QUANT_COLOR ||
          traceType == TRACE_QUANT_MONO   )
