@@ -15,7 +15,7 @@
 
 #include "trace/trace.h"
 #include "trace/potrace/inkscape-potrace.h"
-#include <dialogs/tracedialog.h>
+#include "ui/dialog/tracedialog.h"
 
 #include <inkscape.h>
 #include <desktop.h>
@@ -35,26 +35,6 @@ namespace Inkscape {
 
 namespace Trace {
 
-/**
- *
- */
-Tracer::Tracer()
-{
-    engine = NULL;
-    selectedItem = NULL;
-}
-
-
-
-/**
- *
- */
-Tracer::~Tracer()
-{
-}
-
-
-
 /**
  *
  */
@@ -68,7 +48,7 @@ Tracer::getSelectedSPImage()
         return NULL;
         }
 
-    SPSelection *sel = desktop->selection;
+    Inkscape::Selection *sel = desktop->selection;
     if (!sel)
         {
         char *msg = _("Select an <b>image</b> to trace");
@@ -147,7 +127,7 @@ void Tracer::traceThread()
         return;
         }
 
-    SPSelection *selection = SP_DT_SELECTION (desktop);
+    Inkscape::Selection *selection = SP_DT_SELECTION (desktop);
 
     if (!SP_ACTIVE_DOCUMENT)
         {
@@ -182,7 +162,7 @@ void Tracer::traceThread()
     int nrPaths;
     TracingEngineResult *results = engine->trace(pixbuf, &nrPaths);
     //printf("nrPaths:%d\n", nrPaths);
-    
+
     //### Check if we should stop
     if (!keepGoing || !results || nrPaths<1)
         {
@@ -191,8 +171,8 @@ void Tracer::traceThread()
         }
 
     //### Get pointers to the <image> and its parent
-    SPRepr *imgRepr   = SP_OBJECT(img)->repr;
-    SPRepr *par       = sp_repr_parent(imgRepr);
+    Inkscape::XML::Node *imgRepr   = SP_OBJECT(img)->repr;
+    Inkscape::XML::Node *par       = sp_repr_parent(imgRepr);
 
     //### Get some information for the new transform()
     double x      = 0.0;
@@ -218,7 +198,7 @@ void Tracer::traceThread()
 
     double iwscale = width  / iwidth;
     double ihscale = height / iheight;
-    
+
     NR::Matrix scal(NR::scale(iwscale, ihscale));
 
     //# Convolve scale, translation, and the original transform
@@ -229,7 +209,7 @@ void Tracer::traceThread()
 
     //#OK.  Now let's start making new nodes
 
-    SPRepr *groupRepr = NULL;
+    Inkscape::XML::Node *groupRepr = NULL;
 
     //# if more than 1, make a <g>roup of <path>s
     if (nrPaths > 1)
@@ -238,10 +218,14 @@ void Tracer::traceThread()
         sp_repr_add_child(par, groupRepr, imgRepr);
         }
 
+    long totalNodeCount = 0L;
+
     for (TracingEngineResult *result=results ;
                   result ; result=result->next)
         {
-        SPRepr *pathRepr = sp_repr_new("svg:path");
+	    totalNodeCount += result->getNodeCount();
+
+        Inkscape::XML::Node *pathRepr = sp_repr_new("svg:path");
         sp_repr_set_attr(pathRepr, "style", result->getStyle());
         sp_repr_set_attr(pathRepr, "d",     result->getPathData());
 
@@ -260,7 +244,7 @@ void Tracer::traceThread()
         if (nrPaths == 1)
             {
             selection->clear();
-            selection->addRepr(pathRepr);
+            selection->add(pathRepr);
             }
         sp_repr_unref (pathRepr);
         }
@@ -272,7 +256,7 @@ void Tracer::traceThread()
     if (nrPaths > 1)
         {
         selection->clear();
-        selection->addRepr(groupRepr);
+        selection->add(groupRepr);
         sp_repr_unref (groupRepr);
         }
 
@@ -281,6 +265,10 @@ void Tracer::traceThread()
 
     engine = NULL;
 
+    char *msg = g_strdup_printf(_("Trace: Done. %ld nodes created"), totalNodeCount);
+    desktop->messageStack()->flash(Inkscape::NORMAL_MESSAGE, msg);
+    g_free(msg);
+
 }
 
 /**
