@@ -129,7 +129,6 @@ uri_to_id(SPDocument *document, const gchar *uri)
 	gint len;
 
 	g_return_val_if_fail (document != NULL, NULL);
-	g_return_val_if_fail (SP_IS_DOCUMENT (document), NULL);
 
 	if (!uri) return NULL;
 	/* fixme: xpointer, everything */
@@ -138,7 +137,7 @@ uri_to_id(SPDocument *document, const gchar *uri)
 	e = uri + 5;
 	while (*e) {
 		if (*e == ')') break;
-		if (!isalnum (*e) && (*e != '_') && (*e != '-')) return NULL;
+		if (!isalnum (*e) && (*e != '_') && (*e != '-') && (*e != ':') && (*e != '.')) return NULL;
 		e += 1;
 		if (!*e) return NULL;
 	}
