@@ -10,11 +10,12 @@
  */
 
 /*
- * Author:
+ * Authors:
  *   Lauris Kaplinski <lauris@kaplinski.com>
  *   Ted Gould <ted@gould.cx>
  *   MenTaLguY <mental@rydia.net>
  *   David Turner <novalis@gnu.org>
+ *   bulia byak <buliabyak@users.sf.net>
  *
  * This code is in public domain.
  */
@@ -26,7 +27,9 @@
 
 #include <gtk/gtkstock.h>
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
 
 #include "helper/action.h"
 #include <glibmm/i18n.h>
@@ -34,7 +37,6 @@
 #include "dialogs/text-edit.h"
 #include "dialogs/export.h"
 #include "dialogs/xml-tree.h"
-#include "dialogs/align.h"
 #include "dialogs/transformation.h"
 #include "dialogs/object-properties.h"
 #include "dialogs/desktop-properties.h"
@@ -42,10 +44,13 @@
 #include "dialogs/item-properties.h"
 #include "dialogs/find.h"
 #include "dialogs/debugdialog.h"
-#include "dialogs/scriptdialog.h"
-#include "dialogs/tracedialog.h"
 #include "dialogs/layer-properties.h"
 #include "dialogs/clonetiler.h"
+#include "ui/widget/panel.h"
+#include "ui/dialog/dialog.h"
+#include "dialogs/iconpreview.h"
+#include "dialogs/extensions.h"
+#include "dialogs/swatches.h"
 
 #include "extension/effect.h"
 
@@ -62,7 +67,7 @@
 #include "text-chemistry.h"
 #include "shortcuts.h"
 #include "toolbox.h"
-#include "view.h"
+#include "ui/view/view.h"
 #include "interface.h"
 #include "prefs-utils.h"
 #include "splivarot.h"
@@ -98,7 +103,43 @@ sp_action_get_title (const SPAction *action)
 } // end of sp_action_get_title()
 
 
-namespace Inkscape {
+namespace Inkscape { 
+
+// !!!FIXME:: kill this, use DialogManager instead!!!
+
+class PanelDialog : public Inkscape::UI::Dialog::Dialog
+{
+public:
+
+    PanelDialog(char const *prefs_path, int const verb_num) : Dialog(prefs_path, verb_num) {}
+/*
+    virtual Glib::ustring getName() const {return "foo";}
+    virtual Glib::ustring getDesc() const {return "bar";}
+*/
+};
+
+/** \brief Utility function to get a panel displayed. */
+static void show_panel( Inkscape::UI::Widget::Panel &panel, char const *prefs_path, int const verb_num )
+{
+    Gtk::Container* container = panel.get_toplevel();
+    if ( &panel == container ) { // safe check?
+        //g_message("Creating new dialog to hold it");
+        PanelDialog* dia = new PanelDialog(prefs_path, verb_num);
+        Gtk::VBox *mainVBox = dia->get_vbox();
+        mainVBox->pack_start(panel);
+        dia->show_all_children();
+        dia->present();
+        dia->read_geometry();
+    } else {
+        Gtk::Dialog* dia = dynamic_cast<Gtk::Dialog*>(container);
+        if ( dia ) {
+            //g_message("Found an existing dialog");
+            dia->present();
+        } else {
+            g_message("Failed to find an existing dialog");
+        }
+    }
+}
 
 /** \brief A class to encompass all of the verbs which deal with
            file operations. */
@@ -481,7 +522,7 @@ SPAction *
 Verb::make_action_helper (SPView * view, SPActionEventVector * vector, void * in_pntr)
 {
     SPAction *action;
-    
+
     //std::cout << "Adding action: " << _code << std::endl;
     action = sp_action_new(view, _id, _(_name),
                            _(_tip), _image, this);
@@ -535,9 +576,29 @@ Verb::get_action (SPView * view)
         _actions->insert(ActionTable::value_type(view, action));
     }
 
+    // sp_action_set_sensitive(action, _default_sensitive ? 1 : 0);
+
     return action;
 }
 
+void
+Verb::sensitive (SPDocument * in_doc, bool in_sensitive)
+{
+    for (ActionTable::iterator cur_action = _actions->begin();
+             cur_action != _actions->end();
+             cur_action++) {
+        if (in_doc == NULL || cur_action->first->doc == in_doc) {
+            sp_action_set_sensitive(cur_action->second, in_sensitive ? 1 : 0);
+        }
+    }
+    
+    if (in_doc == NULL) {
+        _default_sensitive = in_sensitive;
+    }
+
+    return;
+}
+
 /** \brief  A function to remove the action associated with a view.
     \param  view  Which view's actions should be removed.
     \return None
@@ -683,14 +744,11 @@ FileVerb::perform (SPAction *action, void * data, void *pdata)
 void
 EditVerb::perform (SPAction *action, void * data, void * pdata)
 {
-    SPDesktop *dt;
-    SPEventContext *ec;
-
-    dt = SP_DESKTOP (sp_action_get_view (action));
+    SPDesktop *dt = SP_DESKTOP (sp_action_get_view (action));
     if (!dt)
         return;
 
-    ec = dt->event_context;
+    SPEventContext *ec = dt->event_context;
 
     switch (reinterpret_cast<std::size_t>(data)) {
         case SP_VERB_EDIT_UNDO:
@@ -740,7 +798,7 @@ EditVerb::perform (SPAction *action, void * data, void * pdata)
             break;
         case SP_VERB_EDIT_SELECT_ALL:
             if (tools_isactive (dt, TOOLS_NODES)) {
-                sp_nodepath_select_all (SP_NODE_CONTEXT(ec)->nodepath);
+                sp_nodepath_select_all_from_subpath (SP_NODE_CONTEXT(ec)->nodepath);
             } else {
                 sp_edit_select_all();
             }
@@ -753,7 +811,11 @@ EditVerb::perform (SPAction *action, void * data, void * pdata)
             }
             break;
         case SP_VERB_EDIT_SELECT_ALL_IN_ALL_LAYERS:
-            sp_edit_select_all_in_all_layers();
+            if (tools_isactive (dt, TOOLS_NODES)) {
+                sp_nodepath_select_all (SP_NODE_CONTEXT(ec)->nodepath);
+            } else {
+                sp_edit_select_all_in_all_layers();
+            }
             break;
         case SP_VERB_EDIT_INVERT_IN_ALL_LAYERS:
             sp_edit_invert_in_all_layers();
@@ -863,14 +925,11 @@ SelectionVerb::perform (SPAction *action, void * data, void * pdata)
         case SP_VERB_SELECTION_SIMPLIFY:
             sp_selected_path_simplify ();
             break;
-        case SP_VERB_SELECTION_CLEANUP:
-            sp_selection_cleanup ();
-            break;
         case SP_VERB_SELECTION_REVERSE:
             sp_selected_path_reverse ();
             break;
-        case SP_VERB_SELECTION_POTRACE:
-            Inkscape::UI::Dialogs::TraceDialog::showInstance();
+        case SP_VERB_SELECTION_TRACE:
+            dt->_dlg_mgr->showDialog("Trace");
             break;
         case SP_VERB_SELECTION_CREATE_BITMAP:
             sp_selection_create_bitmap_copy ();
@@ -882,6 +941,9 @@ SelectionVerb::perform (SPAction *action, void * data, void * pdata)
         case SP_VERB_SELECTION_BREAK_APART:
             sp_selected_path_break_apart ();
             break;
+        case SP_VERB_SELECTION_GRIDTILE:
+            dt->_dlg_mgr->showDialog("TileDialog");
+            break;
         default:
             break;
     }
@@ -912,7 +974,6 @@ LayerVerb::perform (SPAction *action, void *data, void *pdata)
             SPObject *next=Inkscape::next_layer(dt->currentRoot(), dt->currentLayer());
             if (next) {
                 dt->setCurrentLayer(next);
-                // TODO move selected objects to top of next
                 sp_document_done(SP_DT_DOCUMENT(dt));
                 dt->messageStack()->flash(Inkscape::NORMAL_MESSAGE, _("Moved to next layer."));
             } else {
@@ -924,7 +985,6 @@ LayerVerb::perform (SPAction *action, void *data, void *pdata)
             SPObject *prev=Inkscape::previous_layer(dt->currentRoot(), dt->currentLayer());
             if (prev) {
                 dt->setCurrentLayer(prev);
-                // TODO move selected objects to top of prev
                 sp_document_done(SP_DT_DOCUMENT(dt));
                 dt->messageStack()->flash(Inkscape::NORMAL_MESSAGE, _("Moved to previous layer."));
             } else {
@@ -932,6 +992,14 @@ LayerVerb::perform (SPAction *action, void *data, void *pdata)
             }
             break;
         }
+        case SP_VERB_LAYER_MOVE_TO_NEXT: {
+            sp_selection_to_next_layer ();
+            break;
+        }
+        case SP_VERB_LAYER_MOVE_TO_PREV: {
+            sp_selection_to_prev_layer ();
+            break;
+        }
         case SP_VERB_LAYER_TO_TOP:
         case SP_VERB_LAYER_TO_BOTTOM:
         case SP_VERB_LAYER_RAISE:
@@ -966,11 +1034,11 @@ LayerVerb::perform (SPAction *action, void *data, void *pdata)
                 switch (verb) {
                     case SP_VERB_LAYER_TO_TOP:
                     case SP_VERB_LAYER_RAISE:
-                        message = g_strdup_printf (_("Raised layer <b>%s</b>."), layer->label());
+                        message = g_strdup_printf (_("Raised layer <b>%s</b>."), layer->defaultLabel());
                         break;
                     case SP_VERB_LAYER_TO_BOTTOM:
                     case SP_VERB_LAYER_LOWER:
-                        message = g_strdup_printf (_("Lowered layer <b>%s</b>."), layer->label());
+                        message = g_strdup_printf (_("Lowered layer <b>%s</b>."), layer->defaultLabel());
                         break;
                 };
                 sp_document_done(SP_DT_DOCUMENT(dt));
@@ -1019,11 +1087,12 @@ void
 ObjectVerb::perform ( SPAction *action, void *data, void *pdata )
 {
     SPDesktop *dt = SP_DESKTOP(sp_action_get_view(action));
-
     if (!dt)
         return;
 
-    SPSelection *sel = SP_DT_SELECTION(dt);
+    SPEventContext *ec = dt->event_context;
+
+    Inkscape::Selection *sel = SP_DT_SELECTION(dt);
 
     if (sel->isEmpty())
         return;
@@ -1043,17 +1112,29 @@ ObjectVerb::perform ( SPAction *action, void *data, void *pdata )
         case SP_VERB_OBJECT_TO_CURVE:
             sp_selected_path_to_curves ();
             break;
+        case SP_VERB_OBJECT_FLOW_TEXT:
+            text_flow_into_shape();
+            break;
+        case SP_VERB_OBJECT_UNFLOW_TEXT:
+            text_unflow();
+            break;
         case SP_VERB_OBJECT_FLOWTEXT_TO_TEXT:
-            convert_to_text ();
+            SPFlowtext::convert_to_text ();
             break;
         case SP_VERB_OBJECT_FLIP_HORIZONTAL:
-            // TODO: make tool-sensitive, in node edit flip selected node(s)
-            sp_selection_scale_relative(sel, center, NR::scale(-1.0, 1.0));
+            if (tools_isactive (dt, TOOLS_NODES)) {
+                sp_nodepath_flip (SP_NODE_CONTEXT(ec)->nodepath, NR::X);
+            } else {
+                sp_selection_scale_relative(sel, center, NR::scale(-1.0, 1.0));
+            }
             sp_document_done (SP_DT_DOCUMENT (dt));
             break;
         case SP_VERB_OBJECT_FLIP_VERTICAL:
-            // TODO: make tool-sensitive, in node edit flip selected node(s)
-            sp_selection_scale_relative(sel, center, NR::scale(1.0, -1.0));
+            if (tools_isactive (dt, TOOLS_NODES)) {
+                sp_nodepath_flip (SP_NODE_CONTEXT(ec)->nodepath, NR::Y);
+            } else {
+                sp_selection_scale_relative(sel, center, NR::scale(1.0, -1.0));
+            }
             sp_document_done (SP_DT_DOCUMENT (dt));
             break;
         default:
@@ -1119,6 +1200,9 @@ ContextVerb::perform (SPAction *action, void * data, void * pdata)
         case SP_VERB_CONTEXT_TEXT:
             tools_switch_current (TOOLS_TEXT);
             break;
+        case SP_VERB_CONTEXT_GRADIENT:
+            tools_switch_current (TOOLS_GRADIENT);
+            break;
         case SP_VERB_CONTEXT_ZOOM:
             tools_switch_current (TOOLS_ZOOM);
             break;
@@ -1180,6 +1264,11 @@ ContextVerb::perform (SPAction *action, void * data, void * pdata)
             prefs_set_int_attribute ("dialogs.preferences", "page_tools", PREFS_PAGE_TOOLS_TEXT);
             sp_display_dialog ();
             break;
+        case SP_VERB_CONTEXT_GRADIENT_PREFS:
+            prefs_set_int_attribute ("dialogs.preferences", "page_top", PREFS_PAGE_TOOLS);
+            prefs_set_int_attribute ("dialogs.preferences", "page_tools", PREFS_PAGE_TOOLS_GRADIENT);
+            sp_display_dialog ();
+            break;
         case SP_VERB_CONTEXT_ZOOM_PREFS:
             prefs_set_int_attribute ("dialogs.preferences", "page_top", PREFS_PAGE_TOOLS);
             prefs_set_int_attribute ("dialogs.preferences", "page_tools", PREFS_PAGE_TOOLS_ZOOM);
@@ -1209,13 +1298,13 @@ ZoomVerb::perform (SPAction *action, void * data, void * pdata)
 
     SPDocument *doc = SP_DT_DOCUMENT (dt);
 
-    SPRepr *repr = SP_OBJECT_REPR (dt->namedview);
+    Inkscape::XML::Node *repr = SP_OBJECT_REPR (dt->namedview);
 
     gdouble zoom_inc =
         prefs_get_double_attribute_limited ( "options.zoomincrement",
                                              "value", 1.414213562, 1.01, 10 );
 
-    switch (reinterpret_cast<std::size_t>(data)) {
+    switch (GPOINTER_TO_INT(data)) {
         case SP_VERB_ZOOM_IN:
             sp_desktop_get_display_area (dt, &d);
             sp_desktop_zoom_relative ( dt, (d.x0 + d.x1) / 2,
@@ -1282,6 +1371,9 @@ ZoomVerb::perform (SPAction *action, void * data, void * pdata)
         case SP_VERB_VIEW_NEW_PREVIEW:
             sp_ui_new_view_preview ();
             break;
+        case SP_VERB_VIEW_ICON_PREVIEW:
+            show_panel( Inkscape::UI::Dialogs::IconPreviewPanel::getInstance(), "dialogs.iconpreview", SP_VERB_VIEW_ICON_PREVIEW );
+            break;
         default:
             break;
     }
@@ -1292,11 +1384,14 @@ ZoomVerb::perform (SPAction *action, void * data, void * pdata)
 void
 DialogVerb::perform (SPAction *action, void * data, void * pdata)
 {
-    if ((int) data != SP_VERB_DIALOG_TOGGLE) {
+    if (reinterpret_cast<std::size_t>(data) != SP_VERB_DIALOG_TOGGLE) {
         // unhide all when opening a new dialog
         inkscape_dialogs_unhide ();
     }
 
+    SPDesktop *dt = SP_DESKTOP (sp_action_get_view (action));
+    g_assert(dt->_dlg_mgr != NULL);
+
     switch (reinterpret_cast<std::size_t>(data)) {
         case SP_VERB_DIALOG_DISPLAY:
             sp_display_dialog ();
@@ -1307,11 +1402,15 @@ DialogVerb::perform (SPAction *action, void * data, void * pdata)
         case SP_VERB_DIALOG_FILL_STROKE:
             sp_object_properties_dialog ();
             break;
+        case SP_VERB_DIALOG_SWATCHES:
+            show_panel( Inkscape::UI::Dialogs::SwatchesPanel::getInstance(), "dialogs.swatches", SP_VERB_DIALOG_SWATCHES);
+            break;
         case SP_VERB_DIALOG_TRANSFORM:
-            sp_transformation_dialog_move ();
+            //sp_transformation_dialog_move ();
+            dt->_dlg_mgr->showDialog("Transformation");
             break;
         case SP_VERB_DIALOG_ALIGN_DISTRIBUTE:
-            sp_quick_align_dialog ();
+            dt->_dlg_mgr->showDialog("AlignAndDistribute");
             break;
         case SP_VERB_DIALOG_TEXT:
             sp_text_edit_dialog ();
@@ -1323,10 +1422,10 @@ DialogVerb::perform (SPAction *action, void * data, void * pdata)
             sp_find_dialog ();
             break;
         case SP_VERB_DIALOG_DEBUG:
-            Inkscape::UI::Dialogs::DebugDialog::showInstance();
+            dt->_dlg_mgr->showDialog("Messages");
             break;
         case SP_VERB_DIALOG_SCRIPT:
-            Inkscape::UI::Dialogs::ScriptDialog::showInstance();
+            dt->_dlg_mgr->showDialog("Script");
             break;
         case SP_VERB_DIALOG_TOGGLE:
             inkscape_dialogs_toggle ();
@@ -1340,20 +1439,41 @@ DialogVerb::perform (SPAction *action, void * data, void * pdata)
         default:
             break;
     }
-
 } // end of sp_verb_action_dialog_perform()
 
 /** \brief  Decode the verb code and take appropriate action */
 void
 HelpVerb::perform (SPAction *action, void * data, void * pdata)
 {
+    SPDesktop *dt = SP_DESKTOP (sp_action_get_view (action));
+    g_assert(dt->_dlg_mgr != NULL);
+
     switch (reinterpret_cast<std::size_t>(data)) {
         case SP_VERB_HELP_KEYS:
+            /* TRANSLATORS: If you have translated the keys.svg file to your language, then
+               translate this string as "keys.LANG.svg" (where LANG is your language code);
+               otherwise leave as "keys.svg". */
             sp_help_open_screen (_("keys.svg"));
             break;
         case SP_VERB_HELP_ABOUT:
             sp_help_about ();
             break;
+        case SP_VERB_HELP_ABOUT_EXTENSIONS:
+        {
+            Inkscape::UI::Dialogs::ExtensionsPanel * panel = new Inkscape::UI::Dialogs::ExtensionsPanel();
+            panel->set_full(true);
+            show_panel( *panel, "dialogs.aboutextensions", SP_VERB_HELP_ABOUT_EXTENSIONS );
+        }
+        break;
+
+        case SP_VERB_SHOW_LICENSE:
+            // TRANSLATORS: See "tutorial-basic.svg" comment.
+            sp_help_open_tutorial(NULL, (gpointer) _("gpl-2.svg"));
+            break;
+
+        case SP_VERB_HELP_MEMORY:
+            dt->_dlg_mgr->showDialog("Memory");
+            break;
     default:
         break;
     }
@@ -1365,25 +1485,34 @@ TutorialVerb::perform (SPAction *action, void * data, void * pdata)
 {
     switch (reinterpret_cast<std::size_t>(data)) {
         case SP_VERB_TUTORIAL_BASIC:
+            /* TRANSLATORS: If you have translated the tutorial-basic.svg file to your language,
+               then translate this string as "tutorial-basic.LANG.svg" (where LANG is your language
+               code); otherwise leave as "tutorial-basic.svg". */
             sp_help_open_tutorial (NULL, (gpointer)_("tutorial-basic.svg"));
             break;
         case SP_VERB_TUTORIAL_SHAPES:
+            // TRANSLATORS: See "tutorial-basic.svg" comment.
             sp_help_open_tutorial (NULL, (gpointer)_("tutorial-shapes.svg"));
             break;
         case SP_VERB_TUTORIAL_ADVANCED:
+            // TRANSLATORS: See "tutorial-basic.svg" comment.
             sp_help_open_tutorial (NULL, (gpointer)_("tutorial-advanced.svg"));
             break;
         case SP_VERB_TUTORIAL_TRACING:
+            // TRANSLATORS: See "tutorial-basic.svg" comment.
             sp_help_open_tutorial (NULL, (gpointer)_("tutorial-tracing.svg"));
             break;
         case SP_VERB_TUTORIAL_CALLIGRAPHY:
+            // TRANSLATORS: See "tutorial-basic.svg" comment.
             sp_help_open_tutorial (NULL, (gpointer)_("tutorial-calligraphy.svg"));
             break;
         case SP_VERB_TUTORIAL_DESIGN:
-            sp_help_open_tutorial (NULL, (gpointer)_("elementsofdesign.svg"));
+            // TRANSLATORS: See "tutorial-basic.svg" comment.
+            sp_help_open_tutorial (NULL, (gpointer)_("tutorial-elements.svg"));
             break;
         case SP_VERB_TUTORIAL_TIPS:
-            sp_help_open_tutorial (NULL, (gpointer)_("tipsandtricks.svg"));
+            // TRANSLATORS: See "tutorial-basic.svg" comment.
+            sp_help_open_tutorial (NULL, (gpointer)_("tutorial-tips.svg"));
             break;
     default:
         break;
@@ -1477,6 +1606,7 @@ public:
                    gchar const * tip,
                    gchar const * image) :
             Verb(code, id, name, tip, image) {
+        // sensitive(NULL, false);
     }
 }; /* EffectLastVerb class */
 
@@ -1599,9 +1729,9 @@ Verb * Verb::_base_verbs[] = {
         N_("Delete all objects from document"), NULL),
     new EditVerb(SP_VERB_EDIT_SELECT_ALL, "EditSelectAll", N_("Select Al_l"),
         N_("Select all objects or all nodes"), "selection_select_all"),
-    new EditVerb(SP_VERB_EDIT_SELECT_ALL_IN_ALL_LAYERS, "EditSelectAllInAllLayers", N_("Select All in All Layers"),
+    new EditVerb(SP_VERB_EDIT_SELECT_ALL_IN_ALL_LAYERS, "EditSelectAllInAllLayers", N_("Select All in All La_yers"),
         N_("Select all objects in all visible and unlocked layers"), NULL),
-    new EditVerb(SP_VERB_EDIT_INVERT, "EditInvert", N_("Invert Selection"),
+    new EditVerb(SP_VERB_EDIT_INVERT, "EditInvert", N_("In_vert Selection"),
         N_("Invert selection (unselect what is selected and select everything else)"), NULL),
     new EditVerb(SP_VERB_EDIT_INVERT_IN_ALL_LAYERS, "EditInvertInAllLayers", N_("Invert in All Layers"),
         N_("Invert selection in all visible and unlocked layers"), NULL),
@@ -1627,7 +1757,9 @@ Verb * Verb::_base_verbs[] = {
     new SelectionVerb(SP_VERB_SELECTION_TEXTFROMPATH, "SelectionTextFromPath", N_("_Remove from Path"),
         N_("Remove text from path"), NULL),
     new SelectionVerb(SP_VERB_SELECTION_REMOVE_KERNS, "SelectionTextRemoveKerns", N_("Remove Manual _Kerns"),
-        N_("Remove all manual kerns from a text object"), NULL),
+        // TRANSLATORS: "glyph": An image used in the visual representation of characters;
+        //  roughly speaking, how a character looks. A font is a set of glyphs.
+        N_("Remove all manual kerns and glyph rotations from a text object"), NULL),
 
     new SelectionVerb(SP_VERB_SELECTION_UNION, "SelectionUnion", N_("_Union"),
         N_("Union of selected objects"), "union"),
@@ -1646,25 +1778,25 @@ Verb * Verb::_base_verbs[] = {
     // TRANSLATORS: "outset": expand a shape by offsetting the object's path,
     // i.e. by displacing it perpendicular to the path in each point.
     // See also the Advanced Tutorial for explanation.
-    new SelectionVerb(SP_VERB_SELECTION_OFFSET, "SelectionOffset", N_("Ou_tset"),
+    new SelectionVerb(SP_VERB_SELECTION_OFFSET, "SelectionOffset", N_("Outs_et"),
         N_("Outset selected path(s)"), "outset_path"),
-    new SelectionVerb(SP_VERB_SELECTION_OFFSET_SCREEN, "SelectionOffsetScreen", 
-        N_("O_utset Path by 1px"),
-        N_("Outset selected path(s) by 1px"), NULL),
-    new SelectionVerb(SP_VERB_SELECTION_OFFSET_SCREEN_10, "SelectionOffsetScreen10", 
-        N_("O_utset Path by 10px"),
-        N_("Outset selected path(s) by 10px"), NULL),
+    new SelectionVerb(SP_VERB_SELECTION_OFFSET_SCREEN, "SelectionOffsetScreen",
+        N_("O_utset Path by 1 px"),
+        N_("Outset selected path(s) by 1 px"), NULL),
+    new SelectionVerb(SP_VERB_SELECTION_OFFSET_SCREEN_10, "SelectionOffsetScreen10",
+        N_("O_utset Path by 10 px"),
+        N_("Outset selected path(s) by 10 px"), NULL),
     // TRANSLATORS: "inset": contract a shape by offsetting the object's path,
     // i.e. by displacing it perpendicular to the path in each point.
     // See also the Advanced Tutorial for explanation.
     new SelectionVerb(SP_VERB_SELECTION_INSET, "SelectionInset", N_("I_nset"),
         N_("Inset selected path(s)"), "inset_path"),
-    new SelectionVerb(SP_VERB_SELECTION_INSET_SCREEN, "SelectionInsetScreen", 
-        N_("I_nset Path by 1px"),
-        N_("Inset selected path(s) by 1px"), NULL),
-    new SelectionVerb(SP_VERB_SELECTION_INSET_SCREEN_10, "SelectionInsetScreen", 
-        N_("I_nset Path by 10px"),
-        N_("Inset selected path(s) by 10px"), NULL),
+    new SelectionVerb(SP_VERB_SELECTION_INSET_SCREEN, "SelectionInsetScreen",
+        N_("I_nset Path by 1 px"),
+        N_("Inset selected path(s) by 1 px"), NULL),
+    new SelectionVerb(SP_VERB_SELECTION_INSET_SCREEN_10, "SelectionInsetScreen",
+        N_("I_nset Path by 10 px"),
+        N_("Inset selected path(s) by 10 px"), NULL),
     new SelectionVerb(SP_VERB_SELECTION_DYNAMIC_OFFSET, "SelectionDynOffset",
         N_("D_ynamic Offset"), N_("Create a dynamic offset object"), "dynamic_offset"),
     new SelectionVerb(SP_VERB_SELECTION_LINKED_OFFSET, "SelectionLinkedOffset",
@@ -1675,12 +1807,10 @@ Verb * Verb::_base_verbs[] = {
         N_("Convert selected stroke(s) to path(s)"), "stroke_tocurve"),
     new SelectionVerb(SP_VERB_SELECTION_SIMPLIFY, "SelectionSimplify", N_("Si_mplify"),
         N_("Simplify selected path(s) by removing extra nodes"), "simplify"),
-    new SelectionVerb(SP_VERB_SELECTION_CLEANUP, "SelectionCleanup", N_("Cl_eanup"),
-        N_("Clean up selected path(s)"), "selection_cleanup"),
     new SelectionVerb(SP_VERB_SELECTION_REVERSE, "SelectionReverse", N_("_Reverse"),
         N_("Reverses the direction of selected path(s); useful for flipping markers"), "selection_reverse"),
     // TRANSLATORS: "to trace" means "to convert a bitmap to vector graphics" (to vectorize)
-    new SelectionVerb(SP_VERB_SELECTION_POTRACE, "SelectionPotrace", N_("_Trace Bitmap"),
+    new SelectionVerb(SP_VERB_SELECTION_TRACE, "SelectionTrace", N_("_Trace Bitmap"),
         N_("Convert bitmap object to paths"), "selection_trace"),
     new SelectionVerb(SP_VERB_SELECTION_CREATE_BITMAP, "SelectionCreateBitmap", N_("_Make a Bitmap Copy"),
         N_("Export selection to a bitmap and insert it into document"), "selection_bitmap" ),
@@ -1690,26 +1820,31 @@ Verb * Verb::_base_verbs[] = {
     // Advanced tutorial for more info
     new SelectionVerb(SP_VERB_SELECTION_BREAK_APART, "SelectionBreakApart", N_("Break _Apart"),
         N_("Break selected path(s) into subpaths"), "selection_break"),
-
+    new SelectionVerb(SP_VERB_SELECTION_GRIDTILE, "SelectionGridTile", N_("Gri_d Arrange..."),
+        N_("Arrange selection in grid pattern"), "grid_arrange"),
     /* Layer */
-    new LayerVerb(SP_VERB_LAYER_NEW, "LayerNew", N_("_New Layer..."),
-        N_("Create a new layer"), NULL),
-    new LayerVerb(SP_VERB_LAYER_RENAME, "LayerRename", N_("Ren_ame Layer..."),
-        N_("Rename the current layer"), NULL),
-    new LayerVerb(SP_VERB_LAYER_NEXT, "LayerNext", N_("Switch to Next Layer"),
-        N_("Switch to the next layer in the document"), NULL),
-    new LayerVerb(SP_VERB_LAYER_PREV, "LayerPrev", N_("Switch to Previous Layer"),
-        N_("Switch to the previous layer in the document"), NULL),
+    new LayerVerb(SP_VERB_LAYER_NEW, "LayerNew", N_("_Add Layer..."),
+        N_("Create a new layer"), "new_layer"),
+    new LayerVerb(SP_VERB_LAYER_RENAME, "LayerRename", N_("Re_name Layer..."),
+        N_("Rename the current layer"), "rename_layer"),
+    new LayerVerb(SP_VERB_LAYER_NEXT, "LayerNext", N_("Switch to Layer Abov_e"),
+        N_("Switch to the layer above the current"), "switch_to_layer_above"),
+    new LayerVerb(SP_VERB_LAYER_PREV, "LayerPrev", N_("Switch to Layer Belo_w"),
+        N_("Switch to the layer below the current"), "switch_to_layer_below"),
+    new LayerVerb(SP_VERB_LAYER_MOVE_TO_NEXT, "LayerMoveToNext", N_("Move Selection to Layer Abo_ve"),
+        N_("Move selection to the layer above the current"), "move_selection_above"),
+    new LayerVerb(SP_VERB_LAYER_MOVE_TO_PREV, "LayerMoveToPrev", N_("Move Selection to Layer Bel_ow"),
+        N_("Move selection to the layer below the current"), "move_selection_below"),
     new LayerVerb(SP_VERB_LAYER_TO_TOP, "LayerToTop", N_("Layer to _Top"),
-        N_("Raise the current layer to the top"), NULL),
+        N_("Raise the current layer to the top"), "layer_to_top"),
     new LayerVerb(SP_VERB_LAYER_TO_BOTTOM, "LayerToBottom", N_("Layer to _Bottom"),
-        N_("Lower the current layer to the bottom"), NULL),
+        N_("Lower the current layer to the bottom"), "layer_to_bottom"),
     new LayerVerb(SP_VERB_LAYER_RAISE, "LayerRaise", N_("_Raise Layer"),
-        N_("Raise the current layer"), NULL),
+        N_("Raise the current layer"), "raise_layer"),
     new LayerVerb(SP_VERB_LAYER_LOWER, "LayerLower", N_("_Lower Layer"),
-        N_("Lower the current layer"), NULL),
+        N_("Lower the current layer"), "lower_layer"),
     new LayerVerb(SP_VERB_LAYER_DELETE, "LayerDelete", N_("_Delete Current Layer"),
-        N_("Delete the current layer"), NULL),
+        N_("Delete the current layer"), "delete_layer"),
 
     /* Object */
     new ObjectVerb(SP_VERB_OBJECT_ROTATE_90_CW, "ObjectRotate90", N_("Rotate _90 deg CW"),
@@ -1720,8 +1855,12 @@ Verb * Verb::_base_verbs[] = {
         N_("Remove transformations from object"), "object_reset"),
     new ObjectVerb(SP_VERB_OBJECT_TO_CURVE, "ObjectToCurve", N_("_Object to Path"),
         N_("Convert selected object(s) to path(s)"), "object_tocurve"),
-    new ObjectVerb(SP_VERB_OBJECT_FLOWTEXT_TO_TEXT, "ObjectFlowtextToText", N_("_Unflow Text"),
-        N_("Convert selected flowed text(s) to regular text objects"), NULL),
+    new ObjectVerb(SP_VERB_OBJECT_FLOW_TEXT, "ObjectFlowText", N_("_Flow into Frame"),
+        N_("Put text into frame(s)"), NULL),
+    new ObjectVerb(SP_VERB_OBJECT_UNFLOW_TEXT, "ObjectUnFlowText", N_("_Unflow"),
+        N_("Remove text from frame (creates a single-line text object)"), NULL),
+    new ObjectVerb(SP_VERB_OBJECT_FLOWTEXT_TO_TEXT, "ObjectFlowtextToText", N_("_Convert to Text"),
+        N_("Convert flowed text(s) to regular text objects (preserves appearance)"), NULL),
     new ObjectVerb(SP_VERB_OBJECT_FLIP_HORIZONTAL, "ObjectFlipHorizontally",
         N_("Flip _Horizontally"), N_("Flip selection horizontally"),
         "object_flip_hor"),
@@ -1750,6 +1889,8 @@ Verb * Verb::_base_verbs[] = {
         N_("Draw calligraphic lines"), "draw_calligraphic"),
     new ContextVerb(SP_VERB_CONTEXT_TEXT, "DrawText", N_("Text"),
         N_("Create and edit text objects"), "draw_text"),
+    new ContextVerb(SP_VERB_CONTEXT_GRADIENT, "DrawGradient", N_("Gradient"),
+        N_("Create and edit gradients"), "draw_gradient"),
     new ContextVerb(SP_VERB_CONTEXT_ZOOM, "DrawZoom", N_("Zoom"),
         N_("Zoom in or out"), "draw_zoom"),
     new ContextVerb(SP_VERB_CONTEXT_DROPPER, "DrawDropper", N_("Dropper"),
@@ -1776,6 +1917,8 @@ Verb * Verb::_base_verbs[] = {
         N_("Open Inkscape Preferences for the Calligraphy tool"), NULL),
     new ContextVerb(SP_VERB_CONTEXT_TEXT_PREFS, "TextPrefs", N_("Text Preferences"),
         N_("Open Inkscape Preferences for the Text tool"), NULL),
+    new ContextVerb(SP_VERB_CONTEXT_GRADIENT_PREFS, "GradientPrefs", N_("Gradient Preferences"),
+        N_("Open Inkscape Preferences for the Gradient tool"), NULL),
     new ContextVerb(SP_VERB_CONTEXT_ZOOM_PREFS, "ZoomPrefs", N_("Zoom Preferences"),
         N_("Open Inkscape Preferences for the Zoom tool"), NULL),
     new ContextVerb(SP_VERB_CONTEXT_DROPPER_PREFS, "DropperPrefs", N_("Dropper Preferences"),
@@ -1802,11 +1945,13 @@ Verb * Verb::_base_verbs[] = {
     new ZoomVerb(SP_VERB_FULLSCREEN, "FullScreen", N_("_Fullscreen"), N_("Stretch this document window to full screen"),
         "fullscreen"),
 #endif /* HAVE_GTK_WINDOW_FULLSCREEN */
-    new ZoomVerb(SP_VERB_VIEW_NEW, "ViewNew", N_("D_uplicate Window"), N_("Open a new window with the same document"),
+    new ZoomVerb(SP_VERB_VIEW_NEW, "ViewNew", N_("Duplic_ate Window"), N_("Open a new window with the same document"),
         "view_new"),
     new ZoomVerb(SP_VERB_VIEW_NEW_PREVIEW, "ViewNewPreview", N_("_New View Preview"),
          N_("New View Preview"), NULL/*"view_new_preview"*/),
-    new ZoomVerb(SP_VERB_ZOOM_PAGE, "ZoomPage", N_("_Page"), 
+    new ZoomVerb(SP_VERB_VIEW_ICON_PREVIEW, "ViewIconPreview", N_("Ico_n Preview"),
+         N_("Open a window to preview items at different icon resolutions"), NULL/*"view_icon_preview"*/),
+    new ZoomVerb(SP_VERB_ZOOM_PAGE, "ZoomPage", N_("_Page"),
        N_("Zoom to fit page in window"), "zoom_page"),
     new ZoomVerb(SP_VERB_ZOOM_PAGE_WIDTH, "ZoomPageWidth", N_("Page _Width"),
         N_("Zoom to fit page width in window"), "zoom_pagewidth"),
@@ -1822,9 +1967,12 @@ Verb * Verb::_base_verbs[] = {
         N_("Preferences saved with the document"), GTK_STOCK_PROPERTIES ),
     new DialogVerb(SP_VERB_DIALOG_FILL_STROKE, "DialogFillStroke", N_("_Fill and Stroke..."),
         N_("Fill and Stroke dialog"), "fill_and_stroke"),
+    // TRANSLATORS: "Swatches" means: color samples
+    new DialogVerb(SP_VERB_DIALOG_SWATCHES, "DialogSwatches", N_("S_watches..."),
+        N_("View color swatches"), GTK_STOCK_SELECT_COLOR),
     new DialogVerb(SP_VERB_DIALOG_TRANSFORM, "DialogTransform", N_("Transfor_m..."),
         N_("Transform dialog"), "object_trans"),
-    new DialogVerb(SP_VERB_DIALOG_ALIGN_DISTRIBUTE, "DialogAlignDistribute", N_("_Align and Distribute..."), 
+    new DialogVerb(SP_VERB_DIALOG_ALIGN_DISTRIBUTE, "DialogAlignDistribute", N_("_Align and Distribute..."),
         N_("Align and Distribute dialog"), "object_align"),
     new DialogVerb(SP_VERB_DIALOG_TEXT, "Dialogtext", N_("_Text and Font..."),
         N_("Text and Font dialog"), "object_font"),
@@ -1834,9 +1982,9 @@ Verb * Verb::_base_verbs[] = {
         N_("Find objects in document"), GTK_STOCK_FIND ),
     new DialogVerb(SP_VERB_DIALOG_DEBUG, "DialogDebug", N_("_Messages..."),
         N_("View debug messages"), NULL),
-    new DialogVerb(SP_VERB_DIALOG_SCRIPT, "DialogScript", N_("_Scripts..."),
+    new DialogVerb(SP_VERB_DIALOG_SCRIPT, "DialogScript", N_("S_cripts..."),
         N_("Run scripts"), NULL),
-    new DialogVerb(SP_VERB_DIALOG_TOGGLE, "DialogsToggle", N_("Show/_Hide Dialogs"),
+    new DialogVerb(SP_VERB_DIALOG_TOGGLE, "DialogsToggle", N_("Show/Hide D_ialogs"),
         N_("Show or hide all active dialogs"), "dialog_toggle"),
     // TRANSLATORS: "Tile clones" means: "Create tiled clones"
     new DialogVerb(SP_VERB_DIALOG_CLONETILER, "DialogsClonetiler", N_("Tile clones..."),
@@ -1847,8 +1995,14 @@ Verb * Verb::_base_verbs[] = {
     /* Help */
     new HelpVerb(SP_VERB_HELP_KEYS, "HelpKeys", N_("_Keys and Mouse"),
         N_("Key and mouse shortcuts reference"), "help_keys"),
+    new HelpVerb(SP_VERB_HELP_ABOUT_EXTENSIONS, "HelpAboutExtensions", N_("About E_xtensions"),
+        N_("About Extensions..."), NULL),
+    new HelpVerb(SP_VERB_HELP_MEMORY, "HelpAboutMemory", N_("About _Memory"),
+        N_("About Memory..."), NULL),
     new HelpVerb(SP_VERB_HELP_ABOUT, "HelpAbout", N_("_About Inkscape"),
         N_("About Inkscape"), /*"help_about"*/"inkscape_options"),
+    new HelpVerb(SP_VERB_SHOW_LICENSE, "ShowLicense", N_("Modifying or Redistributing Inkscape"),
+        N_("Show license to modify and/or redistribute Inkscape: GNU GPL"), NULL),
 
     /* Tutorials */
     new TutorialVerb(SP_VERB_TUTORIAL_BASIC, "TutorialsBasic", N_("Inkscape: _Basic"),
@@ -1870,7 +2024,7 @@ Verb * Verb::_base_verbs[] = {
     /* Effect */
     new EffectLastVerb(SP_VERB_EFFECT_LAST, "EffectLast", N_("Previous Effect"),
         N_("Repeat the last effect with the same settings"), NULL/*"tutorial_tips"*/),
-    new EffectLastVerb(SP_VERB_EFFECT_LAST_PREF, "EffectLastPref", N_("Previous Effect..."),
+    new EffectLastVerb(SP_VERB_EFFECT_LAST_PREF, "EffectLastPref", N_("Previous Effect Settings..."),
         N_("Repeat the last effect with new settings"), NULL/*"tutorial_tips"*/),
 
     /* Footer */
