@@ -1,5 +1,5 @@
-#ifndef __SP_VERBS_H__
-#define __SP_VERBS_H__
+#ifndef SEEN_SP_VERBS_H
+#define SEEN_SP_VERBS_H
 
 /**
  * \brief Frontend to actions
@@ -86,17 +86,19 @@ enum {
     SP_VERB_SELECTION_LINKED_OFFSET,
     SP_VERB_SELECTION_OUTLINE,
     SP_VERB_SELECTION_SIMPLIFY,
-    SP_VERB_SELECTION_CLEANUP,
     SP_VERB_SELECTION_REVERSE,
-    SP_VERB_SELECTION_POTRACE,
+    SP_VERB_SELECTION_TRACE,
     SP_VERB_SELECTION_CREATE_BITMAP,
     SP_VERB_SELECTION_COMBINE,
     SP_VERB_SELECTION_BREAK_APART,
+    SP_VERB_SELECTION_GRIDTILE,
     /* Layer */
     SP_VERB_LAYER_NEW,
     SP_VERB_LAYER_RENAME,
     SP_VERB_LAYER_NEXT,
     SP_VERB_LAYER_PREV,
+    SP_VERB_LAYER_MOVE_TO_NEXT,
+    SP_VERB_LAYER_MOVE_TO_PREV,
     SP_VERB_LAYER_TO_TOP,
     SP_VERB_LAYER_TO_BOTTOM,
     SP_VERB_LAYER_RAISE,
@@ -107,6 +109,8 @@ enum {
     SP_VERB_OBJECT_ROTATE_90_CCW,
     SP_VERB_OBJECT_FLATTEN,
     SP_VERB_OBJECT_TO_CURVE,
+    SP_VERB_OBJECT_FLOW_TEXT,
+    SP_VERB_OBJECT_UNFLOW_TEXT,
     SP_VERB_OBJECT_FLOWTEXT_TO_TEXT,
     SP_VERB_OBJECT_FLIP_HORIZONTAL,
     SP_VERB_OBJECT_FLIP_VERTICAL,
@@ -121,6 +125,7 @@ enum {
     SP_VERB_CONTEXT_PEN,
     SP_VERB_CONTEXT_CALLIGRAPHIC,
     SP_VERB_CONTEXT_TEXT,
+    SP_VERB_CONTEXT_GRADIENT,
     SP_VERB_CONTEXT_ZOOM,
     SP_VERB_CONTEXT_DROPPER,
     /* Tool preferences */
@@ -134,6 +139,7 @@ enum {
     SP_VERB_CONTEXT_PEN_PREFS,
     SP_VERB_CONTEXT_CALLIGRAPHIC_PREFS,
     SP_VERB_CONTEXT_TEXT_PREFS,
+    SP_VERB_CONTEXT_GRADIENT_PREFS,
     SP_VERB_CONTEXT_ZOOM_PREFS,
     SP_VERB_CONTEXT_DROPPER_PREFS,
     /* Zooming and desktop settings */
@@ -153,6 +159,7 @@ enum {
 #endif /* HAVE_GTK_WINDOW_FULLSCREEN */
     SP_VERB_VIEW_NEW,
     SP_VERB_VIEW_NEW_PREVIEW,
+    SP_VERB_VIEW_ICON_PREVIEW,
     SP_VERB_ZOOM_PAGE,
     SP_VERB_ZOOM_PAGE_WIDTH,
     SP_VERB_ZOOM_DRAWING,
@@ -161,6 +168,7 @@ enum {
     SP_VERB_DIALOG_DISPLAY,
     SP_VERB_DIALOG_NAMEDVIEW,
     SP_VERB_DIALOG_FILL_STROKE,
+    SP_VERB_DIALOG_SWATCHES,
     SP_VERB_DIALOG_TRANSFORM,
     SP_VERB_DIALOG_ALIGN_DISTRIBUTE,
     SP_VERB_DIALOG_TEXT,
@@ -173,7 +181,10 @@ enum {
     SP_VERB_DIALOG_ITEM,
     /* Help */
     SP_VERB_HELP_KEYS,
+    SP_VERB_HELP_ABOUT_EXTENSIONS,
+    SP_VERB_HELP_MEMORY,
     SP_VERB_HELP_ABOUT,
+    SP_VERB_SHOW_LICENSE,
     /* Tutorials */
     SP_VERB_TUTORIAL_BASIC,
     SP_VERB_TUTORIAL_SHAPES,
@@ -191,31 +202,7 @@ enum {
 
 gchar *sp_action_get_title (const SPAction *action);
 
-/* FIXME !!! we should probably go ahead and use GHashTables, actually -- more portable */
-
-#if defined(__GNUG__) && (__GNUG__ >= 3)
-# include <ext/hash_map>
-using __gnu_cxx::hash_map;
-#else
-# include <hash_map.h>
-#endif
-
-#if defined(__GNUG__) && (__GNUG__ >= 3)
-namespace __gnu_cxx {
-#endif
-
-template <>
-class hash<SPView *> {
-    typedef SPView *T;
-public:
-    size_t operator()(const T& x) const {
-        return (size_t)g_direct_hash((gpointer)x);
-    }
-};
-
-#if defined(__GNUG__) && (__GNUG__ >= 3)
-}  /* namespace __gnu_cxx */
-#endif
+#include <map>
 
 namespace Inkscape {
 
@@ -226,7 +213,7 @@ namespace Inkscape {
 class Verb {
 private:
     /** \brief An easy to use defition of the table of verbs by code. */
-    typedef hash_map<unsigned int, Inkscape::Verb *> VerbTable;
+    typedef std::map<unsigned int, Inkscape::Verb *> VerbTable;
     /** \brief A table of all the dynamically created verbs. */
     static VerbTable _verbs;
     /** \brief The table of statically created verbs which are mostly
@@ -235,7 +222,7 @@ private:
     /* Plus one because there is an entry for SP_VERB_LAST */
 
     /** \brief A simple typedef to make using the action table easier. */
-    typedef hash_map<SPView *, SPAction *> ActionTable;
+    typedef std::map<SPView *, SPAction *> ActionTable;
     /** \brief A list of all the actions that have been created for this
                verb.  It is referenced by the view that they are created for. */
     ActionTable * _actions;
@@ -253,6 +240,9 @@ private:
                file. */
     unsigned int  _code;
 
+    /** \brief Whether this verb is set to default to sensitive or
+               insensitive when new actions are created. */
+    bool _default_sensitive;
 public:
     /** \brief Accessor to get the internal variable. */
     unsigned int get_code (void) { return _code; }
@@ -274,7 +264,7 @@ public:
         This function also sets \c _actions to NULL.
 
         \warning NO DATA IS COPIED BY CALLING THIS FUNCTION.
-        
+
         In many respects this is very bad object oriented design, but it
         is done for a reason.  All verbs today are of two types: 1) static
         or 2) created for extension.  In the static case all of the
@@ -288,7 +278,7 @@ public:
          gchar const * name,
          gchar const * tip,
          gchar const * image) :
-        _actions(NULL), _id(id), _name(name), _tip(tip), _image(image), _code(code) {
+        _actions(NULL), _id(id), _name(name), _tip(tip), _image(image), _code(code), _default_sensitive(true) {
     }
     Verb (gchar const * id, gchar const * name, gchar const * tip, gchar const * image);
     virtual ~Verb (void);
@@ -318,12 +308,14 @@ public:
 
     static void delete_all_view (SPView * view);
     void delete_view (SPView * view);
+
+    void sensitive (SPDocument * in_doc = NULL, bool in_sensitive = true);
 }; /* Verb class */
 
 
 }  /* Inkscape namespace */
 
-#endif
+#endif // SEEN_SP_VERBS_H
 
 /*
   Local Variables:
