@@ -6,6 +6,7 @@
  * Authors:
  *   Chris Lahey <clahey@ximian.com>
  *   Lauris Kaplinski <lauris@kaplinski.com>
+ *   bulia byak <buliabyak@users.sf.net>
  *
  * Copyright (C) 1999-2001 Ximian, Inc.
  * Copyright (C) 2002 Lauris Kaplinski
@@ -210,7 +211,7 @@ sp_font_selector_init (SPFontSelector *fsel)
 	gtk_combo_set_value_in_list (GTK_COMBO (fsel->size), FALSE, FALSE);
 	gtk_combo_set_use_arrows (GTK_COMBO (fsel->size), TRUE);
 	gtk_combo_set_use_arrows_always (GTK_COMBO (fsel->size), TRUE);
-	gtk_widget_set_size_request (fsel->size, 64, -1);
+	gtk_widget_set_size_request (fsel->size, 90, -1);
 	gtk_signal_connect (GTK_OBJECT (GTK_COMBO (fsel->size)->entry), "changed", GTK_SIGNAL_FUNC (sp_font_selector_size_changed), fsel);
 	gtk_box_pack_end (GTK_BOX (hb), fsel->size, FALSE, FALSE, 0);
 	
@@ -320,8 +321,10 @@ sp_font_selector_emit_set (SPFontSelector *fsel)
 {
 	font_instance *font;
 
-	if (static_cast< unsigned int > (fsel->styleidx) < fsel->styles.length) {
-		font = (font_factory::Default())->FaceFromDescr ((gchar *) ((fsel->styles.records)[fsel->styleidx].descr));
+	if (static_cast< unsigned int > (fsel->styleidx) < fsel->styles.length
+        && static_cast< unsigned int > (fsel->familyidx) < fsel->families.length) {
+		font = (font_factory::Default())->FaceFromDescr ((gchar *) ((fsel->families.names)[fsel->familyidx]),
+                                                         (gchar *) ((fsel->styles.records)[fsel->styleidx].name));
 	} else {
 		font = NULL;
 	}
@@ -359,7 +362,7 @@ sp_font_selector_set_font (SPFontSelector *fsel, font_instance *font, double siz
 	fcl = GTK_CLIST (fsel->family);
 	scl = GTK_CLIST (fsel->style);
 	
-	if (font) {
+	if (font && (fsel->font != font || size != fsel->fontsize)) {
 		{ // select family in the list
 			gchar family[256];
 			font->Family (family, 256);
@@ -402,10 +405,12 @@ sp_font_selector_set_font (SPFontSelector *fsel, font_instance *font, double siz
 			gtk_clist_moveto (scl, best_i, 0, 0.66, 0.0);
 		}
 		
-		gchar s[8];
-		g_snprintf (s, 8, "%.5g", size); // UI, so printf is ok
-		gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (fsel->size)->entry), s);
-		fsel->fontsize = size;
+		if (size != fsel->fontsize) {
+			gchar s[8];
+			g_snprintf (s, 8, "%.5g", size); // UI, so printf is ok
+			gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (fsel->size)->entry), s);
+			fsel->fontsize = size;
+		}
 	}
 }
 
@@ -541,6 +546,10 @@ sp_font_preview_expose (GtkWidget *widget, GdkEventExpose *event)
 			if (fprev->phrase) {
 				p = fprev->phrase;
 			} else {
+				/* TRANSLATORS: Test string used in text and font dialog (when no
+				 * text has been entered) to get a preview of the font.  Choose
+				 * some representative characters that users of your locale will be
+				 * interested in. */
 				p = _("AaBbCcIiPpQq12368.;/()");
 			}
 			int len = 0;
@@ -680,9 +689,10 @@ sp_font_preview_new (void)
 void
 sp_font_preview_set_font (SPFontPreview *fprev, font_instance *font, SPFontSelector *fsel)
 {
-	if (font)  font->Ref();
-	if (fprev->font) fprev->font->Unref();
-	fprev->font = font;
+	if (font != fprev->font) {
+		if (font)  font->Ref();
+		if (fprev->font) fprev->font->Unref();
+		fprev->font = font;
 	
 		if (fprev->rfont) {
 			fprev->rfont->Unref();
@@ -694,6 +704,7 @@ sp_font_preview_set_font (SPFontPreview *fprev, font_instance *font, SPFontSelec
 			fprev->rfont = fprev->font->RasterFont(flip, 0);
 		}
 		if (GTK_WIDGET_DRAWABLE (fprev)) gtk_widget_queue_draw (GTK_WIDGET (fprev));
+	}
 }
 
 void
