@@ -3,8 +3,9 @@
 /*
  * Gradient vector selection widget
  *
- * Author:
+ * Authors:
  *   Lauris Kaplinski <lauris@kaplinski.com>
+ *   bulia byak <buliabyak@users.sf.net>
  *
  * Copyright (C) 2001-2002 Lauris Kaplinski
  * Copyright (C) 2001 Ximian, Inc.
@@ -15,7 +16,9 @@
  *
  */
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
@@ -149,8 +152,6 @@ sp_gradient_vector_selector_new (SPDocument *doc, SPGradient *gr)
 {
 	GtkWidget *gvs;
 
-	g_return_val_if_fail (!doc || SP_IS_DOCUMENT (doc), NULL);
-	g_return_val_if_fail (!gr || (doc != NULL), NULL);
 	g_return_val_if_fail (!gr || SP_IS_GRADIENT (gr), NULL);
 	g_return_val_if_fail (!gr || (SP_OBJECT_DOCUMENT (gr) == doc), NULL);
 
@@ -172,7 +173,6 @@ sp_gradient_vector_selector_set_gradient (SPGradientVectorSelector *gvs, SPDocum
 
 	g_return_if_fail (gvs != NULL);
 	g_return_if_fail (SP_IS_GRADIENT_VECTOR_SELECTOR (gvs));
-	g_return_if_fail (!doc || SP_IS_DOCUMENT (doc));
 	g_return_if_fail (!gr || (doc != NULL));
 	g_return_if_fail (!gr || SP_IS_GRADIENT (gr));
 	g_return_if_fail (!gr || (SP_OBJECT_DOCUMENT (gr) == doc));
@@ -408,13 +408,13 @@ sp_gvs_defs_modified (SPObject *defs, guint flags, SPGradientVectorSelector *gvs
 #include "../widgets/widget-sizes.h"
 #include "../desktop-handles.h"
 #include "../selection.h"
-#include "../xml/sp-repr-event-vector.h"
+#include "../xml/node-event-vector.h"
 #include "../svg/svg.h"
 
 
 #define PAD 4
 
-static GtkWidget *sp_gradient_vector_widget_new (SPGradient *gradient);
+static GtkWidget *sp_gradient_vector_widget_new (SPGradient *gradient, SPStop *stop);
 
 static void sp_gradient_vector_widget_load_gradient (GtkWidget *widget, SPGradient *gradient);
 static gint sp_gradient_vector_dialog_delete (GtkWidget *widget, GdkEvent *event, GtkWidget *dialog);
@@ -429,7 +429,7 @@ static void update_stop_list( GtkWidget *mnu, SPGradient *gradient, SPStop *new_
 
 static gboolean blocked = FALSE;
 
-static void grad_edit_dia_stop_added_or_removed (SPRepr *repr, SPRepr *child, SPRepr *ref, gpointer data)
+static void grad_edit_dia_stop_added_or_removed (Inkscape::XML::Node *repr, Inkscape::XML::Node *child, Inkscape::XML::Node *ref, gpointer data)
 {
    GtkWidget *vb = GTK_WIDGET(data);
    GtkWidget *mnu = (GtkWidget *)g_object_get_data (G_OBJECT(vb), "stopmenu");
@@ -440,7 +440,7 @@ static void grad_edit_dia_stop_added_or_removed (SPRepr *repr, SPRepr *child, SP
 //FIXME!!! We must also listen to attr changes on all children (i.e. stops) too,
 //otherwise the dialog does not reflect undoing color or offset change. This is a major
 //hassle, unless we have a "one of the descendants changed in some way" signal.
-static SPReprEventVector grad_edit_dia_repr_events =
+static Inkscape::XML::NodeEventVector grad_edit_dia_repr_events =
 {
     grad_edit_dia_stop_added_or_removed, /* child_added */
     grad_edit_dia_stop_added_or_removed, /* child_removed */
@@ -469,7 +469,7 @@ verify_grad(SPGradient *gradient)
 		Inkscape::SVGOStringStream os;
 		os << "stop-color:" << c << ";stop-opacity:" << 1.0 << ";";
 
-		SPRepr *child;
+		Inkscape::XML::Node *child;
 
 		child = sp_repr_new ("svg:stop");
 		sp_repr_set_double (child, "offset", 0.0);
@@ -483,40 +483,27 @@ verify_grad(SPGradient *gradient)
 	}
 	if (i < 2) {
 		sp_repr_set_double (SP_OBJECT_REPR(stop), "offset", 0.0);
-		SPRepr *child = sp_repr_duplicate(SP_OBJECT_REPR(stop));
+		Inkscape::XML::Node *child = SP_OBJECT_REPR(stop)->duplicate();
 		sp_repr_set_double (child, "offset", 1.0);
 		sp_repr_add_child (SP_OBJECT_REPR(gradient), child, SP_OBJECT_REPR (stop));
 	}
 }
 
-static SPStop*
-sp_prev_stop(SPStop *stop, SPGradient *gradient)
+static void
+select_stop_in_list( GtkWidget *mnu, SPGradient *gradient, SPStop *new_stop)
 {
-	if (sp_object_first_child(SP_OBJECT(gradient)) == SP_OBJECT(stop)) 
-		return NULL;
-	SPObject *found = NULL;
+	int i = 0;
 	for ( SPObject *ochild = sp_object_first_child(SP_OBJECT(gradient)) ; ochild != NULL ; ochild = SP_OBJECT_NEXT(ochild) ) {
 		if (SP_IS_STOP (ochild)) {
-			found = ochild;
-		}
-		if (SP_OBJECT_NEXT(ochild) == SP_OBJECT(stop) || SP_OBJECT(ochild) == SP_OBJECT(stop)) {
-			break;
+			if (SP_OBJECT (ochild) == SP_OBJECT(new_stop)) {
+				gtk_option_menu_set_history (GTK_OPTION_MENU (mnu), i);
+				break;
+			} 
+			i++;
 		}
 	}
-	return SP_STOP(found);
 }
 
-static SPStop*
-sp_next_stop(SPStop *stop)
-{
-  for (SPObject *ochild = SP_OBJECT_NEXT(stop); ochild != NULL; ochild = SP_OBJECT_NEXT(ochild)) {
-	if (SP_IS_STOP (ochild))
-		return SP_STOP(ochild);
-  }
-  return NULL;
-}
-
-
 static void
 update_stop_list( GtkWidget *mnu, SPGradient *gradient, SPStop *new_stop)
 {
@@ -558,12 +545,12 @@ update_stop_list( GtkWidget *mnu, SPGradient *gradient, SPStop *new_stop)
 				gtk_widget_show (i);
 				g_object_set_data (G_OBJECT (i), "stop", stop);
 				GtkWidget *hb = gtk_hbox_new (FALSE, 4);
-				GtkWidget *cpv = sp_color_preview_new (sp_color_get_rgba32_falpha (&stop->color, stop->opacity));
+				GtkWidget *cpv = sp_color_preview_new(sp_stop_get_rgba32(stop));
 				gtk_widget_show (cpv);
 				gtk_container_add ( GTK_CONTAINER (hb), cpv );
 				g_object_set_data ( G_OBJECT (i), "preview", cpv );
-				SPRepr *repr = SP_OBJECT_REPR((SPItem *) sl->data);
-				GtkWidget *l = gtk_label_new (sp_repr_attr(repr,"id"));
+				Inkscape::XML::Node *repr = SP_OBJECT_REPR((SPItem *) sl->data);
+				GtkWidget *l = gtk_label_new (repr->attribute("id"));
 				gtk_widget_show (l);
 				gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
 				gtk_box_pack_start (GTK_BOX (hb), l, TRUE, TRUE, 0);
@@ -576,20 +563,14 @@ update_stop_list( GtkWidget *mnu, SPGradient *gradient, SPStop *new_stop)
 		gtk_widget_set_sensitive (mnu, TRUE);
 	}
 	gtk_option_menu_set_menu (GTK_OPTION_MENU (mnu), m);
+
 	/* Set history */
-	if (new_stop == NULL) gtk_option_menu_set_history (GTK_OPTION_MENU (mnu), 0);
-	else {
-		int i = 0;
-		for ( SPObject *ochild = sp_object_first_child(SP_OBJECT(gradient)) ; ochild != NULL ; ochild = SP_OBJECT_NEXT(ochild) ) {
-			if (SP_IS_STOP (ochild)) {
-				if (SP_OBJECT (ochild) == SP_OBJECT(new_stop)) {
-					gtk_option_menu_set_history (GTK_OPTION_MENU (mnu), i);
-					break;
-				} 
-				i++;
-			}
-		}
+	if (new_stop == NULL) {
+		gtk_option_menu_set_history (GTK_OPTION_MENU (mnu), 0);
+	} else {
+		select_stop_in_list (mnu, gradient, new_stop);
 	}
+
 	blocked = FALSE;
 }
 
@@ -607,7 +588,7 @@ sp_grad_edit_select (GtkOptionMenu *mnu,  GtkWidget *tbl)
 	blocked = TRUE;
 
 	SPColorSelector *csel = (SPColorSelector*)g_object_get_data (G_OBJECT (tbl), "cselector");
-	guint32 c = sp_color_get_rgba32_falpha (&stop->color, stop->opacity);
+	guint32 const c = sp_stop_get_rgba32(stop);
 	csel->base->setAlpha(SP_RGBA32_A_F (c));
 	SPColor color;
 	sp_color_set_rgb_float (&color, SP_RGBA32_R_F (c), SP_RGBA32_G_F (c), SP_RGBA32_B_F (c));
@@ -701,7 +682,7 @@ sp_grd_ed_add_stop (GtkWidget *widget,  GtkWidget *vb)
 	if (stop == NULL) 
 		return;
 
-	SPRepr *new_stop_repr = NULL;
+	Inkscape::XML::Node *new_stop_repr = NULL;
 
 	SPStop *next = sp_next_stop (stop);
 
@@ -714,11 +695,11 @@ sp_grd_ed_add_stop (GtkWidget *widget,  GtkWidget *vb)
 	}
 
 	if (next != NULL) {
-		new_stop_repr = sp_repr_duplicate(SP_OBJECT_REPR(stop));
+		new_stop_repr = SP_OBJECT_REPR(stop)->duplicate();
 		sp_repr_add_child (SP_OBJECT_REPR(gradient), new_stop_repr, SP_OBJECT_REPR(stop));
 	} else {
 		next = stop;
-		new_stop_repr = sp_repr_duplicate(SP_OBJECT_REPR(sp_prev_stop(stop, gradient)));
+		new_stop_repr = SP_OBJECT_REPR(sp_prev_stop(stop, gradient))->duplicate();
 		sp_repr_add_child (SP_OBJECT_REPR(gradient), new_stop_repr, SP_OBJECT_REPR(sp_prev_stop(stop, gradient)));
 	}
 
@@ -726,8 +707,8 @@ sp_grd_ed_add_stop (GtkWidget *widget,  GtkWidget *vb)
    
 	newstop->offset = (stop->offset + next->offset) * 0.5 ;
 
-	guint32 c1 = sp_color_get_rgba32_falpha (&stop->color, stop->opacity);
-	guint32 c2 = sp_color_get_rgba32_falpha (&next->color, next->opacity);
+	guint32 const c1 = sp_stop_get_rgba32(stop);
+	guint32 const c2 = sp_stop_get_rgba32(next);
 	guint32 cnew = sp_average_color (c1, c2);
 
 	Inkscape::SVGOStringStream os;
@@ -750,22 +731,38 @@ sp_grd_ed_add_stop (GtkWidget *widget,  GtkWidget *vb)
 static void
 sp_grd_ed_del_stop (GtkWidget *widget,  GtkWidget *vb)
 {
-    SPGradient *gradient = (SPGradient *)g_object_get_data (G_OBJECT(vb), "gradient");
+	SPGradient *gradient = (SPGradient *)g_object_get_data (G_OBJECT(vb), "gradient");
 
-    GtkOptionMenu *mnu = (GtkOptionMenu *)g_object_get_data (G_OBJECT(vb), "stopmenu");
-    if (!g_object_get_data (G_OBJECT(gtk_menu_get_active (GTK_MENU(gtk_option_menu_get_menu (mnu)))), "stop")) return;
-    SPStop *stop = SP_STOP(g_object_get_data (G_OBJECT(gtk_menu_get_active (GTK_MENU(gtk_option_menu_get_menu (mnu)))), "stop"));
-    if (stop->offset>0 && stop->offset<1) {
-        sp_repr_remove_child (SP_OBJECT_REPR(gradient), SP_OBJECT_REPR(stop));
-        sp_gradient_vector_widget_load_gradient (vb, gradient);
-        update_stop_list(GTK_WIDGET(mnu), gradient, NULL);
-        sp_document_done (SP_OBJECT_DOCUMENT (gradient));
-    }
+	GtkOptionMenu *mnu = (GtkOptionMenu *)g_object_get_data (G_OBJECT(vb), "stopmenu");
+	if (!g_object_get_data (G_OBJECT(gtk_menu_get_active (GTK_MENU(gtk_option_menu_get_menu (mnu)))), "stop")) return;
+	SPStop *stop = SP_STOP(g_object_get_data (G_OBJECT(gtk_menu_get_active (GTK_MENU(gtk_option_menu_get_menu (mnu)))), "stop"));
+	if (gradient->vector.stops.size() > 2) { // 2 is the minimum
+
+		// if we delete first or last stop, move the next/previous to the edge
+		if (stop->offset == 0) {
+			SPStop *next = sp_next_stop (stop);
+			if (next) {
+				next->offset = 0;
+				sp_repr_set_double (SP_OBJECT_REPR (next), "offset", 0);
+			}
+		} else if (stop->offset == 1) {
+			SPStop *prev = sp_prev_stop (stop, gradient);
+			if (prev) {
+				prev->offset = 1;
+				sp_repr_set_double (SP_OBJECT_REPR (prev), "offset", 1);
+			}
+		}
+
+		sp_repr_remove_child (SP_OBJECT_REPR(gradient), SP_OBJECT_REPR(stop));
+		sp_gradient_vector_widget_load_gradient (vb, gradient);
+		update_stop_list(GTK_WIDGET(mnu), gradient, NULL);
+		sp_document_done (SP_OBJECT_DOCUMENT (gradient));
+	}
 
 }
 
 static GtkWidget *
-sp_gradient_vector_widget_new (SPGradient *gradient)
+sp_gradient_vector_widget_new (SPGradient *gradient, SPStop *select_stop)
 {
 	GtkWidget *vb, *w, *f, *csel;
 
@@ -873,13 +870,16 @@ sp_gradient_vector_widget_new (SPGradient *gradient)
 
 	sp_gradient_vector_widget_load_gradient (vb, gradient);
 
+	if (select_stop)
+		select_stop_in_list (GTK_WIDGET(mnu), gradient, select_stop);
+
 	return vb;
 }
 
 
 
 GtkWidget *
-sp_gradient_vector_editor_new (SPGradient *gradient)
+sp_gradient_vector_editor_new (SPGradient *gradient, SPStop *stop)
 {
 	GtkWidget *wid;
 
@@ -912,7 +912,7 @@ sp_gradient_vector_editor_new (SPGradient *gradient)
 
 		gtk_container_set_border_width (GTK_CONTAINER (dlg), PAD);
 
-		wid = (GtkWidget*)sp_gradient_vector_widget_new (gradient);
+		wid = (GtkWidget*)sp_gradient_vector_widget_new (gradient, stop);
 		g_object_set_data (G_OBJECT (dlg), "gradient-vector-widget", wid);
 		/* Connect signals */
 		gtk_widget_show (wid);
@@ -934,12 +934,7 @@ sp_gradient_vector_editor_new (SPGradient *gradient)
 		g_object_unref (G_OBJECT (event.window));
 
 		g_assert (dlg == NULL);
-		sp_gradient_vector_editor_new (gradient);
-
-		// The old code which crashes when you "add" (i.e. copy) a three-stop gradient, "edit" it, and add a stop
-		//		gtk_window_present ((GtkWindow *) dlg);
-		//		wid = (GtkWidget*)g_object_get_data (G_OBJECT (dlg), "gradient-vector-widget");
-		//		sp_gradient_vector_widget_load_gradient (wid, gradient);
+		sp_gradient_vector_editor_new (gradient, stop);
 	}
 
 	return dlg;
@@ -970,7 +965,7 @@ sp_gradient_vector_widget_load_gradient (GtkWidget *widget, SPGradient *gradient
 
 		GtkOptionMenu *mnu = (GtkOptionMenu *)g_object_get_data (G_OBJECT(widget), "stopmenu");
 		SPStop *stop = SP_STOP(g_object_get_data (G_OBJECT(gtk_menu_get_active (GTK_MENU(gtk_option_menu_get_menu (mnu)))), "stop"));
-		guint32 c = sp_color_get_rgba32_falpha (&stop->color, stop->opacity);
+		guint32 const c = sp_stop_get_rgba32(stop);
 
 		/// get the color selector 
 		SPColorSelector *csel = SP_COLOR_SELECTOR(g_object_get_data (G_OBJECT (widget), "cselector"));
@@ -990,7 +985,7 @@ sp_gradient_vector_widget_load_gradient (GtkWidget *widget, SPGradient *gradient
 	update_stop_list (GTK_WIDGET(mnu), gradient, NULL);
 
 	// Once the user edits a gradient, it stops being auto-collectable
-	if (sp_repr_attr (SP_OBJECT_REPR(gradient), "inkscape:collect")) {
+	if (SP_OBJECT_REPR(gradient)->attribute("inkscape:collect")) {
 		SPDocument *document = SP_OBJECT_DOCUMENT (gradient);
 		gboolean saved = sp_document_get_undo_sensitive(document);
 		sp_document_set_undo_sensitive (document, FALSE);
@@ -1079,32 +1074,30 @@ static void sp_gradient_vector_color_dragged(SPColorSelector *csel, GtkObject *o
     SPStop *stop = SP_STOP(g_object_get_data (G_OBJECT(gtk_menu_get_active (GTK_MENU(gtk_option_menu_get_menu (mnu)))), "stop"));
 
 
-    csel->base->getColorAlpha(stop->color, &stop->opacity);
+    csel->base->getColorAlpha(stop->specified_color, &stop->opacity);
+    stop->currentColor = false;
 
 	blocked = FALSE;
     SPColorPreview *cpv = (SPColorPreview *)g_object_get_data (G_OBJECT(gtk_menu_get_active (GTK_MENU(gtk_option_menu_get_menu (mnu)))), "preview");
-    sp_color_preview_set_rgba32 (cpv, sp_color_get_rgba32_falpha (&stop->color, stop->opacity));
+    sp_color_preview_set_rgba32(cpv, sp_stop_get_rgba32(stop));
 
 }
 
 static void
 sp_gradient_vector_color_changed (SPColorSelector *csel, GtkObject *object)
 {
-	SPGradient *gradient, *ngr;
-	gdouble start, end;
-	SPObject *child;
 	SPColor color;
 	float alpha;
 	guint32 rgb;
 
 	if (blocked) return;
 
-	gradient = (SPGradient*)g_object_get_data (G_OBJECT (object), "gradient");
+	SPGradient *gradient = (SPGradient*)g_object_get_data (G_OBJECT (object), "gradient");
 	if (!gradient) return;
 
 	blocked = TRUE;
 
-	ngr = sp_gradient_ensure_vector_normalized (gradient);
+	SPGradient *ngr = sp_gradient_ensure_vector_normalized (gradient);
 	if (ngr != gradient) {
 		/* Our master gradient has changed */
 		sp_gradient_vector_widget_load_gradient (GTK_WIDGET (object), ngr);
@@ -1112,15 +1105,9 @@ sp_gradient_vector_color_changed (SPColorSelector *csel, GtkObject *object)
 
 	sp_gradient_ensure_vector (ngr);
 
-	start = ngr->vector->start;
-	end = ngr->vector->end;
-
 	/* Set start parameters */
 	/* We rely on normalized vector, i.e. stops HAVE to exist */
-	for ( child = sp_object_first_child(SP_OBJECT(ngr)) ; child != NULL ; child = SP_OBJECT_NEXT(child) ) {
-		if (SP_IS_STOP (child)) break;
-	}
-	g_return_if_fail (child != NULL);
+	g_return_if_fail (sp_first_stop(ngr) != NULL);
 
 	GtkOptionMenu *mnu = (GtkOptionMenu *)g_object_get_data (G_OBJECT(object), "stopmenu");
 	SPStop *stop = SP_STOP(g_object_get_data (G_OBJECT(gtk_menu_get_active (GTK_MENU(gtk_option_menu_get_menu (mnu)))), "stop"));
@@ -1142,4 +1129,3 @@ sp_gradient_vector_color_changed (SPColorSelector *csel, GtkObject *object)
 
 	blocked = FALSE;
 }
-
