@@ -36,7 +36,7 @@
 #include "dialogs/layer-properties.h"
 
 #include "xml/repr.h"
-#include "xml/sp-repr-event-vector.h"
+#include "xml/node-event-vector.h"
 
 namespace Inkscape {
 namespace Widgets {
@@ -45,16 +45,16 @@ namespace {
 
 class AlternateIcons : public Gtk::HBox {
 public:
-    AlternateIcons(unsigned size, gchar const *a, gchar const *b)
+    AlternateIcons(GtkIconSize size, gchar const *a, gchar const *b)
     : _a(NULL), _b(NULL)
     {
         if (a) {
-            _a = Gtk::manage(new Gtk::Image(PixBufFactory::get().getIcon(a,size)));
+            _a = Gtk::manage(sp_icon_get_icon(a, size));
             _a->set_no_show_all(true);
             add(*_a);
         }
         if (b) {
-            _b = Gtk::manage(new Gtk::Image(PixBufFactory::get().getIcon(b,size)));
+            _b = Gtk::manage(sp_icon_get_icon(b,size));
             _b->set_no_show_all(true);
             add(*_b);
         }
@@ -99,7 +99,7 @@ LayerSelector::LayerSelector(SPDesktop *desktop)
 {
     AlternateIcons *label;
 
-    label = Gtk::manage(new AlternateIcons(11, "visible", "hidden"));
+    label = Gtk::manage(new AlternateIcons(GTK_ICON_SIZE_MENU, "visible", "hidden"));
     _visibility_toggle.add(*label);
     _visibility_toggle.signal_toggled().connect(
         sigc::compose(
@@ -107,7 +107,7 @@ LayerSelector::LayerSelector(SPDesktop *desktop)
             sigc::mem_fun(_visibility_toggle, &Gtk::ToggleButton::get_active)
         )
     );
-    _hide_toggled_connection = _visibility_toggle.signal_toggled().connect(
+    _visibility_toggled_connection = _visibility_toggle.signal_toggled().connect(
         sigc::compose(
             sigc::mem_fun(*this, &LayerSelector::_hideLayer),
             sigc::mem_fun(_visibility_toggle, &Gtk::ToggleButton::get_active)
@@ -119,7 +119,7 @@ LayerSelector::LayerSelector(SPDesktop *desktop)
     _tooltips.set_tip(_visibility_toggle, _("Toggle current layer visibility"));
     pack_start(_visibility_toggle, Gtk::PACK_EXPAND_PADDING);
 
-    label = Gtk::manage(new AlternateIcons(11, "lock_unlocked", "width_height_lock"));
+    label = Gtk::manage(new AlternateIcons(GTK_ICON_SIZE_MENU, "lock_unlocked", "width_height_lock"));
     _lock_toggle.add(*label);
     _lock_toggle.signal_toggled().connect(
         sigc::compose(
@@ -231,6 +231,7 @@ private:
 /** Selects the given layer in the dropdown selector.  
  */
 void LayerSelector::_selectLayer(SPObject *layer) {
+    using Inkscape::Util::List;
     using Inkscape::Util::cons;
     using Inkscape::Util::reverse_list;
 
@@ -242,7 +243,7 @@ void LayerSelector::_selectLayer(SPObject *layer) {
         _layer_model->erase(first_row);
     }
 
-    SPObject *root(_desktop->currentRoot());
+    SPObject *root=_desktop->currentRoot();
 
     if (_layer) {
         sp_object_unref(_layer, NULL);
@@ -250,9 +251,12 @@ void LayerSelector::_selectLayer(SPObject *layer) {
     }
 
     if (layer) {
-        _buildEntries(0, cons(*root,
-            reverse_list<SPObject::ParentIterator>(layer, root)
-        ));
+        List<SPObject &> hierarchy=reverse_list<SPObject::ParentIterator>(layer, root);
+        if ( layer == root ) {
+            _buildEntries(0, cons(*root, hierarchy));
+        } else if (hierarchy) {
+            _buildSiblingEntries(0, *root, hierarchy);
+        }
 
         Gtk::TreeIter row(
             std::find_if(
@@ -309,8 +313,8 @@ void LayerSelector::_buildEntries(unsigned depth,
 
     _buildEntry(depth, *hierarchy);
 
-    List<SPObject &> remainder(rest(hierarchy));
-    if ( remainder && rest(remainder) ) {
+    List<SPObject &> remainder=rest(hierarchy);
+    if (remainder) {
         _buildEntries(depth+1, remainder);
     } else {
         _buildSiblingEntries(depth+1, *hierarchy, remainder);
@@ -355,35 +359,36 @@ struct Callbacks {
     sigc::slot<void> update_list;
 };
 
-void attribute_changed(SPRepr *repr, gchar const *name,
+void attribute_changed(Inkscape::XML::Node *repr, gchar const *name,
                        gchar const *old_value, gchar const *new_value,
                        bool is_interactive, void *data) 
 {
-    if ( !std::strcmp(name, "id") || !std::strcmp(name, "inkscape:label") ) {
+    if ( !std::strcmp(name, "inkscape:groupmode") ) {
+        reinterpret_cast<Callbacks *>(data)->update_list();
+    } else {
         reinterpret_cast<Callbacks *>(data)->update_row();
-    } else if ( !std::strcmp(name, "inkscape:groupmode") ) {
-        reinterpret_cast<Callbacks *>(data)->update_list();
     }
 }
 
-void node_added(SPRepr *parent, SPRepr *child, SPRepr *ref, void *data) {
-    gchar const *mode=sp_repr_attr(child, "inkscape:groupmode");
+void node_added(Inkscape::XML::Node *parent, Inkscape::XML::Node *child, Inkscape::XML::Node *ref, void *data) {
+    gchar const *mode=child->attribute("inkscape:groupmode");
     if ( mode && !std::strcmp(mode, "layer") ) {
         reinterpret_cast<Callbacks *>(data)->update_list();
     }
 }
 
-void node_removed(SPRepr *parent, SPRepr *child, SPRepr *ref, void *data) {
-    gchar const *mode=sp_repr_attr(child, "inkscape:groupmode");
+void node_removed(Inkscape::XML::Node *parent, Inkscape::XML::Node *child, Inkscape::XML::Node *ref, void *data) {
+    gchar const *mode=child->attribute("inkscape:groupmode");
     if ( mode && !std::strcmp(mode, "layer") ) {
         reinterpret_cast<Callbacks *>(data)->update_list();
     }
 }
 
-void node_reordered(SPRepr *parent, SPRepr *child,
-                    SPRepr *old_ref, SPRepr *new_ref, void *data)
+void node_reordered(Inkscape::XML::Node *parent, Inkscape::XML::Node *child,
+                    Inkscape::XML::Node *old_ref, Inkscape::XML::Node *new_ref,
+                    void *data)
 {
-    gchar const *mode=sp_repr_attr(child, "inkscape:groupmode");
+    gchar const *mode=child->attribute("inkscape:groupmode");
     if ( mode && !std::strcmp(mode, "layer") ) {
         reinterpret_cast<Callbacks *>(data)->update_list();
     }
@@ -412,27 +417,43 @@ void rebuild_all_rows(sigc::slot<void, SPObject *> rebuild, SPDesktop *desktop)
 
 }
 
+void LayerSelector::_protectUpdate(sigc::slot<void> slot) {
+    bool visibility_blocked=_visibility_toggled_connection.blocked();
+    bool lock_blocked=_lock_toggled_connection.blocked();
+    _visibility_toggled_connection.block(true);
+    _lock_toggled_connection.block(true);
+    slot();
+    _visibility_toggled_connection.block(visibility_blocked);
+    _lock_toggled_connection.block(lock_blocked);
+}
+
 /** Builds and appends a row in the layer model object.
  */
 void LayerSelector::_buildEntry(unsigned depth, SPObject &object) {
-    SPReprEventVector *vector;
+    Inkscape::XML::NodeEventVector *vector;
 
     Callbacks *callbacks=new Callbacks();
 
     callbacks->update_row = sigc::bind(
-        sigc::ptr_fun(&update_row_for_object),
-        &object, _model_columns.object, _layer_model
+        sigc::mem_fun(*this, &LayerSelector::_protectUpdate),
+        sigc::bind(
+            sigc::ptr_fun(&update_row_for_object),
+            &object, _model_columns.object, _layer_model
+        )
     );
 
     SPObject *layer=_desktop->currentLayer();
     if ( &object == layer || &object == SP_OBJECT_PARENT(layer) ) {
         callbacks->update_list = sigc::bind(
-            sigc::ptr_fun(&rebuild_all_rows),
-            sigc::mem_fun(*this, &LayerSelector::_selectLayer),
-            _desktop
+            sigc::mem_fun(*this, &LayerSelector::_protectUpdate),
+            sigc::bind(
+                sigc::ptr_fun(&rebuild_all_rows),
+                sigc::mem_fun(*this, &LayerSelector::_selectLayer),
+                _desktop
+            )
         );
 
-        SPReprEventVector events = {
+        Inkscape::XML::NodeEventVector events = {
             &node_added,
             &node_removed,
             &attribute_changed,
@@ -440,9 +461,9 @@ void LayerSelector::_buildEntry(unsigned depth, SPObject &object) {
             &node_reordered
         };
 
-        vector = new SPReprEventVector(events);
+        vector = new Inkscape::XML::NodeEventVector(events);
     } else {
-        SPReprEventVector events = {
+        Inkscape::XML::NodeEventVector events = {
             NULL,
             NULL,
             &attribute_changed,
@@ -450,7 +471,7 @@ void LayerSelector::_buildEntry(unsigned depth, SPObject &object) {
             NULL
         };
 
-        vector = new SPReprEventVector(events);
+        vector = new Inkscape::XML::NodeEventVector(events);
     }
 
     Gtk::ListStore::iterator row(_layer_model->append());
@@ -477,7 +498,7 @@ void LayerSelector::_destroyEntry(Gtk::ListStore::iterator const &row) {
     if (object) {
         sp_object_unref(object, NULL);
     }
-    SPRepr *repr=row->get_value(_model_columns.repr);
+    Inkscape::XML::Node *repr=row->get_value(_model_columns.repr);
     if (repr) {
         sp_repr_remove_listener_by_data(repr, callbacks);
         sp_repr_unref(repr);
@@ -502,32 +523,31 @@ void LayerSelector::_prepareLabelRenderer(
         SPObject *layer=( _desktop ? _desktop->currentLayer() : NULL );
         SPObject *root=( _desktop ? _desktop->currentRoot() : NULL );
 
-        bool isroot = !( layer && SP_OBJECT_PARENT(object) == SP_OBJECT_PARENT(layer) ||
-                layer == root && SP_OBJECT_PARENT(object) == root);
+        bool isancestor = !( layer && SP_OBJECT_PARENT(object) == SP_OBJECT_PARENT(layer) || layer == root && SP_OBJECT_PARENT(object) == root);
 
         bool iscurrent = ( object == layer && object != root );
 
         gchar *format = g_strdup_printf (
             "<span size=\"smaller\" %s><tt>%*s%s</tt>%s%s%s%%s%s%s%s</span>",
-            (_desktop && _desktop->itemIsHidden (SP_ITEM(object))) ? "foreground=\"gray50\"" : "",
-            depth ? (depth) : 0, "", iscurrent ? "&#8226;" : (depth? " " : ""),
-            iscurrent? "<b>" : "",
-            (SP_ITEM(object)->isLocked()) ? "[" : "",
-            isroot ? "<small>" : "",
-            isroot ? "</small>" : "",
-            (SP_ITEM(object)->isLocked()) ? "]" : "",
-            iscurrent? "</b>" : ""
+            ( _desktop && _desktop->itemIsHidden (SP_ITEM(object)) ? "foreground=\"gray50\"" : "" ),
+            depth, "", ( iscurrent ? "&#8226;" : " " ),
+            ( iscurrent ? "<b>" : "" ),
+            ( SP_ITEM(object)->isLocked() ? "[" : "" ),
+            ( isancestor ? "<small>" : "" ),
+            ( isancestor ? "</small>" : "" ),
+            ( SP_ITEM(object)->isLocked() ? "]" : "" ),
+            ( iscurrent ? "</b>" : "" )
             );
 
         gchar const *label;
-        if (depth) {
+        if ( object != root ) {
             label = object->label();
             if (!label) {
                 label = object->defaultLabel();
                 label_defaulted = true;
             }
         } else {
-            label = "(root)";
+            label = _("(root)");
         }
 
         gchar *text = g_markup_printf_escaped(format, label); 
