@@ -5,12 +5,15 @@
  */
 
 /*
- * Copyright (C) Lauris 2002
- */
+ * Copyright (C) Lauris Kaplinski 2002
+ *   bulia byak <buliabyak@users.sf.net>
+*/
 
 #define noSP_PS_VERBOSE
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
 
 #include <math.h>
 #include <string.h>
@@ -29,7 +32,7 @@
 #include <gtk/gtkframe.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkoptionmenu.h>
-#include "menu.h"
+#include <gtk/gtktooltips.h>
 #include <gtk/gtkmenuitem.h>
 #include <gtk/gtkimage.h>
 
@@ -50,18 +53,21 @@
 #include <inkscape.h>
 #include <document-private.h>
 #include <desktop-handles.h>
+#include <desktop-style.h>
 #include <selection.h>
 #include <style.h>
+#include "svg/svg.h"
+#include "svg/stringstream.h"
 
 #include "paint-selector.h"
 
-
 enum {
 	MODE_CHANGED,
 	GRABBED,
 	DRAGGED,
 	RELEASED,
 	CHANGED,
+	FILLRULE_CHANGED,
 	LAST_SIGNAL
 };
 
@@ -69,16 +75,17 @@ static void sp_paint_selector_class_init (SPPaintSelectorClass *klass);
 static void sp_paint_selector_init (SPPaintSelector *slider);
 static void sp_paint_selector_destroy (GtkObject *object);
 
-static GtkWidget *sp_paint_selector_style_button_add (SPPaintSelector *psel, const gchar *px, SPPaintSelectorMode mode, GtkRadioButton *last, GtkTooltips *tt, const gchar *tip);
+static GtkWidget *sp_paint_selector_style_button_add (SPPaintSelector *psel, const gchar *px, SPPaintSelectorMode mode, GtkTooltips *tt, const gchar *tip);
 static void sp_paint_selector_style_button_toggled (GtkToggleButton *tb, SPPaintSelector *psel);
+static void sp_paint_selector_fillrule_toggled (GtkToggleButton *tb, SPPaintSelector *psel);
 
 static void sp_paint_selector_set_mode_empty (SPPaintSelector *psel);
 static void sp_paint_selector_set_mode_multiple (SPPaintSelector *psel);
-static void sp_paint_selector_set_mode_clone (SPPaintSelector *psel);
 static void sp_paint_selector_set_mode_none (SPPaintSelector *psel);
 static void sp_paint_selector_set_mode_color (SPPaintSelector *psel, SPPaintSelectorMode mode);
 static void sp_paint_selector_set_mode_gradient (SPPaintSelector *psel, SPPaintSelectorMode mode);
 static void sp_paint_selector_set_mode_pattern (SPPaintSelector *psel, SPPaintSelectorMode mode);
+static void sp_paint_selector_set_mode_unset (SPPaintSelector *psel);
 
 
 static void sp_paint_selector_set_style_buttons (SPPaintSelector *psel, GtkWidget *active);
@@ -145,6 +152,12 @@ sp_paint_selector_class_init (SPPaintSelectorClass *klass)
 						 GTK_SIGNAL_OFFSET (SPPaintSelectorClass, changed),
 						 gtk_marshal_NONE__NONE,
 						 GTK_TYPE_NONE, 0);
+	psel_signals[FILLRULE_CHANGED] = gtk_signal_new ("fillrule_changed",
+						 (GtkSignalRunType)(GTK_RUN_FIRST | GTK_RUN_NO_RECURSE),
+						 GTK_CLASS_TYPE(object_class),
+						 GTK_SIGNAL_OFFSET (SPPaintSelectorClass, fillrule_changed),
+						 gtk_marshal_NONE__UINT,
+						 GTK_TYPE_NONE, 1, GTK_TYPE_UINT);
 
 	object_class->destroy = sp_paint_selector_destroy;
 }
@@ -167,15 +180,46 @@ sp_paint_selector_init (SPPaintSelector *psel)
 
 	/* Buttons */
 	psel->none = sp_paint_selector_style_button_add (psel, INKSCAPE_STOCK_FILL_NONE,
-							 SP_PAINT_SELECTOR_MODE_NONE, NULL, tt, _("No paint"));
+							 SP_PAINT_SELECTOR_MODE_NONE, tt, _("No paint"));
 	psel->solid = sp_paint_selector_style_button_add (psel, INKSCAPE_STOCK_FILL_SOLID,
-							  SP_PAINT_SELECTOR_MODE_COLOR_RGB, GTK_RADIO_BUTTON (psel->none), tt, _("Flat color"));
+							  SP_PAINT_SELECTOR_MODE_COLOR_RGB, tt, _("Flat color"));
 	psel->gradient = sp_paint_selector_style_button_add (psel, INKSCAPE_STOCK_FILL_GRADIENT,
-						     SP_PAINT_SELECTOR_MODE_GRADIENT_LINEAR, GTK_RADIO_BUTTON (psel->solid), tt, _("Linear gradient"));
+						     SP_PAINT_SELECTOR_MODE_GRADIENT_LINEAR, tt, _("Linear gradient"));
 	psel->radial = sp_paint_selector_style_button_add (psel, INKSCAPE_STOCK_FILL_RADIAL,
-							 SP_PAINT_SELECTOR_MODE_GRADIENT_RADIAL, GTK_RADIO_BUTTON (psel->gradient), tt, _("Radial gradient")),
+							 SP_PAINT_SELECTOR_MODE_GRADIENT_RADIAL, tt, _("Radial gradient")),
 	psel->pattern = sp_paint_selector_style_button_add (psel, INKSCAPE_STOCK_FILL_PATTERN,
-                               SP_PAINT_SELECTOR_MODE_PATTERN, GTK_RADIO_BUTTON (psel->radial), tt, _("Pattern"));
+                               SP_PAINT_SELECTOR_MODE_PATTERN, tt, _("Pattern"));
+	psel->unset = sp_paint_selector_style_button_add (psel, INKSCAPE_STOCK_FILL_UNSET,
+                               SP_PAINT_SELECTOR_MODE_UNSET, tt, _("Unset paint (make it undefined so it can be inherited)"));
+
+	/* Fillrule */
+	{
+	psel->fillrulebox = gtk_hbox_new (FALSE, 0);
+	gtk_box_pack_end (GTK_BOX (psel->style), psel->fillrulebox, FALSE, FALSE, 0);
+
+	GtkWidget *w;
+	psel->evenodd = gtk_radio_button_new (NULL);
+	gtk_button_set_relief (GTK_BUTTON (psel->evenodd), GTK_RELIEF_NONE);
+	gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON(psel->evenodd), FALSE);
+    // TRANSLATORS: for info, see http://www.w3.org/TR/2000/CR-SVG-20000802/painting.html#FillRuleProperty
+	gtk_tooltips_set_tip (tt, psel->evenodd, _("Any path self-intersections or subpaths create holes in the fill (fill-rule: evenodd)"), NULL);
+	gtk_object_set_data (GTK_OBJECT (psel->evenodd), "mode", GUINT_TO_POINTER (SP_PAINT_SELECTOR_FILLRULE_EVENODD));
+	w = sp_icon_new (GTK_ICON_SIZE_MENU, "fillrule_evenodd");
+	gtk_container_add (GTK_CONTAINER (psel->evenodd), w);
+	gtk_box_pack_start (GTK_BOX (psel->fillrulebox), psel->evenodd, FALSE, FALSE, 0);
+	gtk_signal_connect (GTK_OBJECT (psel->evenodd), "toggled", GTK_SIGNAL_FUNC (sp_paint_selector_fillrule_toggled), psel);
+
+	psel->nonzero = gtk_radio_button_new (gtk_radio_button_group (GTK_RADIO_BUTTON (psel->evenodd)));
+	gtk_button_set_relief (GTK_BUTTON (psel->nonzero), GTK_RELIEF_NONE);
+	gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON(psel->nonzero), FALSE);
+    // TRANSLATORS: for info, see http://www.w3.org/TR/2000/CR-SVG-20000802/painting.html#FillRuleProperty
+	gtk_tooltips_set_tip (tt, psel->nonzero, _("Fill is solid unless a subpath is counterdirectional (fill-rule: nonzero)"), NULL);
+	gtk_object_set_data (GTK_OBJECT (psel->nonzero), "mode", GUINT_TO_POINTER (SP_PAINT_SELECTOR_FILLRULE_NONZERO));
+	w = sp_icon_new (GTK_ICON_SIZE_MENU, "fillrule_nonzero");
+	gtk_container_add (GTK_CONTAINER (psel->nonzero), w);
+	gtk_box_pack_start (GTK_BOX (psel->fillrulebox), psel->nonzero, FALSE, FALSE, 0);
+	gtk_signal_connect (GTK_OBJECT (psel->nonzero), "toggled", GTK_SIGNAL_FUNC (sp_paint_selector_fillrule_toggled), psel);
+	}
 
 	/* Frame */
 	psel->frame = gtk_frame_new ("");
@@ -201,20 +245,22 @@ sp_paint_selector_destroy (GtkObject *object)
 }
 
 static GtkWidget *
-sp_paint_selector_style_button_add (SPPaintSelector *psel, const gchar *pixmap, SPPaintSelectorMode mode, GtkRadioButton *last, GtkTooltips *tt, const gchar *tip)
+sp_paint_selector_style_button_add (SPPaintSelector *psel, const gchar *pixmap, SPPaintSelectorMode mode, GtkTooltips *tt, const gchar *tip)
 {
 	GtkWidget *b, *w;
 
-	b = gtk_radio_button_new ((last) ? gtk_radio_button_group (last) : NULL);
+	b = gtk_toggle_button_new ();
 	gtk_tooltips_set_tip (tt, b, tip, NULL);
 	gtk_widget_show (b);
 
 	gtk_container_set_border_width (GTK_CONTAINER (b), 0);
 
+	gtk_button_set_relief (GTK_BUTTON (b), GTK_RELIEF_NONE);
+
 	gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (b), FALSE);
 	gtk_object_set_data (GTK_OBJECT (b), "mode", GUINT_TO_POINTER (mode));
 
-	w = sp_icon_new (SP_ICON_SIZE_BUTTON, pixmap);
+	w = sp_icon_new (GTK_ICON_SIZE_BUTTON, pixmap);
 	gtk_widget_show (w);
 	gtk_container_add (GTK_CONTAINER (b), w);
 
@@ -232,8 +278,31 @@ sp_paint_selector_style_button_toggled (GtkToggleButton *tb, SPPaintSelector *ps
 	}
 }
 
+static void
+sp_paint_selector_fillrule_toggled (GtkToggleButton *tb, SPPaintSelector *psel)
+{
+	if (!psel->update && gtk_toggle_button_get_active (tb)) {
+		SPPaintSelectorFillRule fr = (SPPaintSelectorFillRule)GPOINTER_TO_UINT (gtk_object_get_data (GTK_OBJECT (tb), "mode"));
+		gtk_signal_emit (GTK_OBJECT (psel), psel_signals[FILLRULE_CHANGED], fr);
+	}
+}
+
+void
+sp_paint_selector_show_fillrule (SPPaintSelector *psel, bool is_fill)
+{
+	if (psel->fillrulebox) {
+		if (is_fill) {
+			gtk_widget_show_all (psel->fillrulebox);
+		} else {
+			gtk_widget_destroy (psel->fillrulebox);
+			psel->fillrulebox = NULL;
+		}
+	}
+}
+
+
 GtkWidget *
-sp_paint_selector_new (void)
+sp_paint_selector_new (bool is_fill)
 {
 	SPPaintSelector *psel;
 
@@ -241,6 +310,11 @@ sp_paint_selector_new (void)
 
 	sp_paint_selector_set_mode (psel, SP_PAINT_SELECTOR_MODE_MULTIPLE);
 
+     // This silliness is here because I don't know how to pass a parameter to the
+     // GtkObject "constructor" (sp_paint_selector_init). Remove it when paint_selector
+     // becomes a normal class.
+	sp_paint_selector_show_fillrule (psel, is_fill); 
+
 	return GTK_WIDGET (psel);
 }
 
@@ -273,8 +347,8 @@ sp_paint_selector_set_mode (SPPaintSelector *psel, SPPaintSelectorMode mode)
 		case SP_PAINT_SELECTOR_MODE_PATTERN:
 			sp_paint_selector_set_mode_pattern (psel, mode);
 			break;
-		case SP_PAINT_SELECTOR_MODE_CLONE:
-			sp_paint_selector_set_mode_clone (psel);
+		case SP_PAINT_SELECTOR_MODE_UNSET:
+			sp_paint_selector_set_mode_unset (psel);
 			break;
 		default:
 			g_warning ("file %s: line %d: Unknown paint mode %d", __FILE__, __LINE__, mode);
@@ -286,6 +360,15 @@ sp_paint_selector_set_mode (SPPaintSelector *psel, SPPaintSelectorMode mode)
 	}
 }
 
+void
+sp_paint_selector_set_fillrule (SPPaintSelector *psel, SPPaintSelectorFillRule fillrule)
+{
+	if (psel->fillrulebox) {
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (psel->evenodd), (fillrule == SP_PAINT_SELECTOR_FILLRULE_EVENODD));
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (psel->nonzero), (fillrule == SP_PAINT_SELECTOR_FILLRULE_NONZERO));
+	}
+}
+
 void
 sp_paint_selector_set_color_alpha (SPPaintSelector *psel, const SPColor *color, float alpha)
 {
@@ -328,20 +411,6 @@ sp_paint_selector_set_gradient_linear (SPPaintSelector *psel, SPGradient *vector
 	sp_gradient_selector_set_vector (gsel, (vector) ? SP_OBJECT_DOCUMENT (vector) : NULL, vector);
 }
 
-void
-sp_paint_selector_set_lgradient_position (SPPaintSelector *psel, gdouble x0, gdouble y0, gdouble x1, gdouble y1)
-{
-	SPGradientSelector *gsel;
-
-	g_return_if_fail (psel != NULL);
-	g_return_if_fail (SP_IS_PAINT_SELECTOR (psel));
-	g_return_if_fail (psel->mode == SP_PAINT_SELECTOR_MODE_GRADIENT_LINEAR);
-
-	gsel = (SPGradientSelector*)gtk_object_get_data (GTK_OBJECT (psel->selector), "gradient-selector");
-
-	sp_gradient_selector_set_lgradient_position (gsel, x0, y0, x1, y1);
-}
-
 void
 sp_paint_selector_set_gradient_radial (SPPaintSelector *psel, SPGradient *vector)
 {
@@ -357,70 +426,6 @@ sp_paint_selector_set_gradient_radial (SPPaintSelector *psel, SPGradient *vector
 	sp_gradient_selector_set_vector (gsel, (vector) ? SP_OBJECT_DOCUMENT (vector) : NULL, vector);
 }
 
-void
-sp_paint_selector_set_rgradient_position (SPPaintSelector *psel, gdouble cx, gdouble cy, gdouble fx, gdouble fy, gdouble r)
-{
-	SPGradientSelector *gsel;
-
-	g_return_if_fail (psel != NULL);
-	g_return_if_fail (SP_IS_PAINT_SELECTOR (psel));
-	g_return_if_fail (psel->mode == SP_PAINT_SELECTOR_MODE_GRADIENT_RADIAL);
-
-	gsel = (SPGradientSelector*)gtk_object_get_data (GTK_OBJECT (psel->selector), "gradient-selector");
-
-	sp_gradient_selector_set_rgradient_position (gsel, cx, cy, fx, fy, r);
-}
-
-void
-sp_paint_selector_set_gradient_bbox (SPPaintSelector *psel, gdouble x0, gdouble y0, gdouble x1, gdouble y1)
-{
-	SPGradientSelector *gsel;
-
-	g_return_if_fail (psel != NULL);
-	g_return_if_fail (SP_IS_PAINT_SELECTOR (psel));
-	g_return_if_fail ((psel->mode == SP_PAINT_SELECTOR_MODE_GRADIENT_LINEAR) ||
-			  (psel->mode == SP_PAINT_SELECTOR_MODE_GRADIENT_RADIAL));
-
-	gsel = (SPGradientSelector*)gtk_object_get_data (GTK_OBJECT (psel->selector), "gradient-selector");
-
-	sp_gradient_selector_set_bbox (gsel, x0, y0, x1, y1);
-}
-
-void
-sp_paint_selector_set_gradient_gs2d_matrix_f(SPPaintSelector *psel, NRMatrix const *gs2d)
-{
-	sp_paint_selector_set_gradient_gs2d_matrix(psel, *gs2d);
-}
-
-void
-sp_paint_selector_set_gradient_gs2d_matrix(SPPaintSelector *psel, NR::Matrix const &gs2d)
-{
-	g_return_if_fail (psel != NULL);
-	g_return_if_fail (SP_IS_PAINT_SELECTOR (psel));
-	g_return_if_fail ((psel->mode == SP_PAINT_SELECTOR_MODE_GRADIENT_LINEAR) ||
-			  (psel->mode == SP_PAINT_SELECTOR_MODE_GRADIENT_RADIAL));
-
-	SPGradientSelector *gsel = (SPGradientSelector*) gtk_object_get_data(GTK_OBJECT(psel->selector),
-									     "gradient-selector");
-
-	sp_gradient_selector_set_gs2d_matrix_f (gsel, gs2d);
-}
-
-void
-sp_paint_selector_get_gradient_gs2d_matrix_f(SPPaintSelector const *psel, NRMatrix *gs2d)
-{
-	SPGradientSelector *gsel;
-
-	g_return_if_fail (psel != NULL);
-	g_return_if_fail (SP_IS_PAINT_SELECTOR (psel));
-	g_return_if_fail ((psel->mode == SP_PAINT_SELECTOR_MODE_GRADIENT_LINEAR) ||
-			  (psel->mode == SP_PAINT_SELECTOR_MODE_GRADIENT_RADIAL));
-
-	gsel = (SPGradientSelector*)gtk_object_get_data (GTK_OBJECT (psel->selector), "gradient-selector");
-
-	sp_gradient_selector_get_gs2d_matrix_f (gsel, gs2d);
-}
-
 void
 sp_paint_selector_set_gradient_properties (SPPaintSelector *psel, SPGradientUnits units, SPGradientSpread spread)
 {
@@ -475,134 +480,6 @@ sp_paint_selector_get_gradient_vector (SPPaintSelector *psel)
 	return sp_gradient_selector_get_vector (gsel);
 }
 
-void
-sp_paint_selector_get_gradient_position_floatv (SPPaintSelector *psel, gfloat *pos)
-{
-	SPGradientSelector *gsel;
-
-	g_return_if_fail ((psel->mode == SP_PAINT_SELECTOR_MODE_GRADIENT_LINEAR) ||
-			  (psel->mode == SP_PAINT_SELECTOR_MODE_GRADIENT_RADIAL));
-
-	gsel = (SPGradientSelector*)gtk_object_get_data (GTK_OBJECT (psel->selector), "gradient-selector");
-
-	if (psel->mode == SP_PAINT_SELECTOR_MODE_GRADIENT_LINEAR) {
-		sp_gradient_selector_get_lgradient_position_floatv (gsel, pos);
-	} else {
-		sp_gradient_selector_get_rgradient_position_floatv (gsel, pos);
-	}
-}
-
-void
-sp_paint_selector_write_lineargradient (SPPaintSelector *psel, SPLinearGradient *lg, SPItem *item)
-{
-	g_return_if_fail (psel->mode == SP_PAINT_SELECTOR_MODE_GRADIENT_LINEAR);
-
-	/* Units have to be set before position calculations */
-	SPGradientUnits units;
-	SPGradientSpread spread;
-	sp_paint_selector_get_gradient_properties (psel, &units, &spread);
-	sp_gradient_set_units (SP_GRADIENT (lg), units);
-	sp_gradient_set_spread (SP_GRADIENT (lg), spread);
-
-	/* Get positions from the selector*/
-	gfloat p[4];
-	sp_paint_selector_get_gradient_position_floatv (psel, p);
-	NR::Point p1(p[0], p[1]);
-	NR::Point p2(p[2], p[3]);
-
-	/* Calculate raw gradient transform */
-	NRRect fbb_;
-	sp_item_invoke_bbox(item, &fbb_, NR::identity(), TRUE);
-	NR::Rect fbb(fbb_);
-
-	NR::Matrix fctm = sp_item_i2doc_affine (item);
-
-	NRMatrix gs2d;
-	sp_paint_selector_get_gradient_gs2d_matrix_f (psel, &gs2d);
-
-	NR::Matrix g2d = sp_gradient_get_g2d_matrix (SP_GRADIENT (lg), fctm, fbb);
-
-	NR::Matrix gs2g = NR::Matrix(gs2d) / g2d;
-
-	/* Normalize transform */
-	NR::Point gp1 = p1 * gs2g;
-	NR::Point gp2 = p2 * gs2g;
-
-	double e = NR::expansion(gs2g);
-	if (e < 0.001) e = 0.001;
-	gs2g[0] /= e;
-	gs2g[1] /= e;
-	gs2g[2] /= e;
-	gs2g[3] /= e;
-	gs2g[4] = 0.0;
-	gs2g[5] = 0.0;
-
-	gs2d = gs2g * NR::Matrix(g2d);
-	sp_gradient_set_gs2d_matrix (SP_GRADIENT (lg), fctm, fbb, gs2d);
-
-	p1 = gp1 * gs2g.inverse();
-	p2 = gp2 * gs2g.inverse();
-
-	sp_lineargradient_set_position (lg, p1[NR::X], p1[NR::Y], p2[NR::X], p2[NR::Y]);
-}
-
-void
-sp_paint_selector_write_radialgradient (SPPaintSelector *psel, SPRadialGradient *rg, SPItem *item)
-{
-	g_return_if_fail (psel->mode == SP_PAINT_SELECTOR_MODE_GRADIENT_RADIAL);
-
-	SPGradientUnits units;
-	SPGradientSpread spread;
-	sp_paint_selector_get_gradient_properties (psel, &units, &spread);
-	sp_gradient_set_units (SP_GRADIENT (rg), units);
-	sp_gradient_set_spread (SP_GRADIENT (rg), spread);
-
-	/* Now position */
-	gfloat p[5];
-	sp_paint_selector_get_gradient_position_floatv (psel, p);
-	NR::Point p1(p[0], p[1]);
-	NR::Point p2(p[2], p[3]);
-	gdouble r = p[4];
-
-	/* Calculate raw gradient transform */
-	NRRect fbb_;
-	sp_item_invoke_bbox(item, &fbb_, NR::identity(), TRUE);
-	NR::Rect fbb(fbb_);
-
-	NR::Matrix fctm = sp_item_i2doc_affine (item);
-
-	NRMatrix gs2d;
-	sp_paint_selector_get_gradient_gs2d_matrix_f (psel, &gs2d);
-
-	NR::Matrix g2d = sp_gradient_get_g2d_matrix (SP_GRADIENT (rg), fctm, fbb);
-
-	NR::Matrix gs2g = NR::Matrix(gs2d) / g2d;
-
-	/* Normalize transform */
-	NR::Point gp1 = p1 * gs2g;
-	NR::Point gp2 = p2 * gs2g;
-
-	double e = NR::expansion(gs2g);
-	if (e < 0.001) e = 0.001;
-	gs2g[0] /= e;
-	gs2g[1] /= e;
-	gs2g[2] /= e;
-	gs2g[3] /= e;
-	gs2g[4] = 0.0;
-	gs2g[5] = 0.0;
-
-	gs2d = gs2g * NR::Matrix(g2d);
-	sp_gradient_set_gs2d_matrix (SP_GRADIENT (rg), fctm, fbb, gs2d);
-
-	p1 = gp1 * gs2g.inverse();
-	p2 = gp2 * gs2g.inverse();
-	r *= e;
-
-	sp_radialgradient_set_position (rg, p1[NR::X], p1[NR::Y], p2[NR::X], p2[NR::Y], r);
-
-
-}
-
 void
 sp_gradient_selector_attrs_to_gradient (SPGradient *gr, SPPaintSelector *psel)
 {
@@ -664,19 +541,14 @@ sp_paint_selector_set_mode_multiple (SPPaintSelector *psel)
 }
 
 static void
-sp_paint_selector_set_mode_clone (SPPaintSelector *psel)
+sp_paint_selector_set_mode_unset (SPPaintSelector *psel)
 {
-    sp_paint_selector_set_style_buttons (psel, NULL);
-    gtk_widget_set_sensitive (psel->style, FALSE);
+    sp_paint_selector_set_style_buttons (psel, psel->unset);
+    gtk_widget_set_sensitive (psel->style, TRUE);
 
     sp_paint_selector_clear_frame(psel);
 
-    // TRANSLATORS: "Clone" is a noun here. This message is to notify the user
-    // that the currently selected object is a clone.
-    gtk_frame_set_label (GTK_FRAME (psel->frame), _("Clone selected"));
-
-    // FIXME: we must enable it to edit the style of clone, so that it's inherited into
-    // the clone object for those properties that are not set in the original
+    gtk_frame_set_label (GTK_FRAME (psel->frame), _("Paint is undefined"));
 }
 
 static void
@@ -840,6 +712,7 @@ sp_paint_selector_set_style_buttons (SPPaintSelector *psel, GtkWidget *active)
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (psel->gradient), (active == psel->gradient));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (psel->radial), (active == psel->radial));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (psel->pattern), (active == psel->pattern));
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (psel->unset), (active == psel->unset));
 }
 
 static void
@@ -896,8 +769,8 @@ ink_pattern_menu (GtkWidget *mnu)
 				//        g_signal_connect (G_OBJECT (i), "activate", G_CALLBACK (sp_gvs_gradient_activate), gvs);
 				GtkWidget *hb = gtk_hbox_new (FALSE, 4);
 				gtk_widget_show (hb);
-				SPRepr *repr = SP_OBJECT_REPR((SPItem *) pl->data);
-				GtkWidget *l = gtk_label_new (sp_repr_attr(repr,"id"));
+				Inkscape::XML::Node *repr = SP_OBJECT_REPR((SPItem *) pl->data);
+				GtkWidget *l = gtk_label_new (repr->attribute("id"));
 				gtk_widget_show (l);
 				gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
 				gtk_box_pack_start (GTK_BOX (hb), l, TRUE, TRUE, 0);
@@ -938,7 +811,7 @@ sp_update_pattern_list ( SPPaintSelector *psel,  SPPattern *pattern)
 
 		gtk_object_set_data(GTK_OBJECT(mnu), "update", GINT_TO_POINTER(TRUE));
 
-		gchar *patname = (gchar *) sp_repr_attr(SP_OBJECT_REPR(pattern),"id");
+		gchar *patname = (gchar *) SP_OBJECT_REPR(pattern)->attribute("id");
 
 		GtkMenu *m = GTK_MENU(gtk_option_menu_get_menu (GTK_OPTION_MENU(mnu)));
 		GList *kids = GTK_MENU_SHELL(m)->children;
@@ -947,7 +820,7 @@ sp_update_pattern_list ( SPPaintSelector *psel,  SPPattern *pattern)
 		int i = 0;
 
 		for (; kids != NULL; kids = kids->next) {
-			gchar *men_pat = (gchar *) sp_repr_attr(SP_OBJECT_REPR(g_object_get_data(G_OBJECT(kids->data), "pattern")),"id");
+			gchar *men_pat = (gchar *) SP_OBJECT_REPR(g_object_get_data(G_OBJECT(kids->data), "pattern"))->attribute("id");
 			if ( strcmp(men_pat, patname) == 0 ) {
 				patpos = i;
 			}
@@ -1035,4 +908,71 @@ sp_paint_selector_get_pattern (SPPaintSelector *psel)
 	return pat;
 }
 
+void
+sp_paint_selector_set_flat_color (SPPaintSelector *psel, SPDesktop *desktop, const gchar *color_property, const gchar *opacity_property)
+{
+    SPCSSAttr *css = sp_repr_css_attr_new ();
 
+    SPColor color;
+    gfloat alpha;
+    sp_paint_selector_get_color_alpha (psel, &color, &alpha);
+    guint32 rgba = sp_color_get_rgba32_falpha (&color, alpha);
+
+    gchar b[64];
+    sp_svg_write_color (b, 64, rgba);
+
+    sp_repr_css_set_property (css, color_property, b);
+    Inkscape::SVGOStringStream osalpha;
+    osalpha << alpha;
+    sp_repr_css_set_property (css, opacity_property, osalpha.str().c_str());
+
+    sp_desktop_set_style (desktop, css);
+
+    sp_repr_css_attr_unref (css);
+}
+
+SPPaintSelectorMode
+sp_style_determine_paint_selector_mode (SPStyle *style, bool isfill)
+{
+    unsigned set = isfill? style->fill.set : style->stroke.set;
+    if (!set)
+        return SP_PAINT_SELECTOR_MODE_UNSET;
+
+    unsigned type = isfill? style->fill.type : style->stroke.type;
+    switch (type) {
+
+        case SP_PAINT_TYPE_NONE:
+        {
+            return SP_PAINT_SELECTOR_MODE_NONE;
+        }
+
+        case SP_PAINT_TYPE_COLOR:
+        {
+            return SP_PAINT_SELECTOR_MODE_COLOR_RGB; // so far only rgb can be read from svg
+        }
+
+        case SP_PAINT_TYPE_PAINTSERVER:
+        {
+            SPPaintServer *server = isfill? SP_STYLE_FILL_SERVER (style) : SP_STYLE_STROKE_SERVER (style);
+    
+            if (SP_IS_LINEARGRADIENT (server)) {
+                return SP_PAINT_SELECTOR_MODE_GRADIENT_LINEAR;
+            } else if (SP_IS_RADIALGRADIENT (server)) {
+                return SP_PAINT_SELECTOR_MODE_GRADIENT_RADIAL;
+            } else if (SP_IS_PATTERN (server)) {
+                return SP_PAINT_SELECTOR_MODE_PATTERN;
+            }
+
+            g_warning ( "file %s: line %d: Unknown paintserver",
+                        __FILE__, __LINE__ );
+            return SP_PAINT_SELECTOR_MODE_NONE;
+        }
+
+        default:
+            g_warning ( "file %s: line %d: Unknown paint type %d",
+                        __FILE__, __LINE__, type );
+            break;
+    }
+
+    return SP_PAINT_SELECTOR_MODE_NONE;
+}
