@@ -36,9 +36,14 @@ typedef enum {
 	SP_PAINT_SELECTOR_MODE_GRADIENT_LINEAR,
     SP_PAINT_SELECTOR_MODE_GRADIENT_RADIAL,
     SP_PAINT_SELECTOR_MODE_PATTERN,
-    SP_PAINT_SELECTOR_MODE_CLONE
+    SP_PAINT_SELECTOR_MODE_UNSET
 } SPPaintSelectorMode;
 
+typedef enum {
+	SP_PAINT_SELECTOR_FILLRULE_NONZERO,
+	SP_PAINT_SELECTOR_FILLRULE_EVENODD
+} SPPaintSelectorFillRule;
+
 struct SPPaintSelector {
 	GtkVBox vbox;
 
@@ -47,7 +52,11 @@ struct SPPaintSelector {
 	SPPaintSelectorMode mode;
 
 	GtkWidget *style;
-    GtkWidget *none, *solid, *gradient, *radial, *pattern;
+	GtkWidget *none, *solid, *gradient, *radial, *pattern, *unset;
+
+	GtkWidget *fillrulebox;
+	GtkWidget *evenodd, *nonzero;
+
 	GtkWidget *frame, *selector;
 
 	SPColor color;
@@ -63,27 +72,21 @@ struct SPPaintSelectorClass {
 	void (* dragged) (SPPaintSelector *psel);
 	void (* released) (SPPaintSelector *psel);
 	void (* changed) (SPPaintSelector *psel);
+	void (* fillrule_changed) (SPPaintSelector *psel, SPPaintSelectorFillRule fillrule);
 };
 
 GtkType sp_paint_selector_get_type (void);
 
-GtkWidget *sp_paint_selector_new (void);
+GtkWidget *sp_paint_selector_new (bool is_fill);
 
 void sp_paint_selector_set_mode (SPPaintSelector *psel, SPPaintSelectorMode mode);
+void sp_paint_selector_set_fillrule (SPPaintSelector *psel, SPPaintSelectorFillRule fillrule);
 
 void sp_paint_selector_set_color_alpha (SPPaintSelector *psel, const SPColor *color, float alpha);
 
 void sp_paint_selector_set_gradient_linear (SPPaintSelector *psel, SPGradient *vector);
-void sp_paint_selector_set_lgradient_position (SPPaintSelector *psel, gdouble x0, gdouble y0, gdouble x1, gdouble y1);
 
 void sp_paint_selector_set_gradient_radial (SPPaintSelector *psel, SPGradient *vector);
-void sp_paint_selector_set_rgradient_position (SPPaintSelector *psel, gdouble cx, gdouble cy, gdouble fx, gdouble fy, gdouble r);
-
-void sp_paint_selector_set_gradient_bbox (SPPaintSelector *psel, gdouble x0, gdouble y0, gdouble x1, gdouble y1);
-
-void sp_paint_selector_set_gradient_gs2d_matrix(SPPaintSelector *psel, NR::Matrix const &gs2d);
-void sp_paint_selector_set_gradient_gs2d_matrix_f(SPPaintSelector *psel, NRMatrix const *gs2d);
-void sp_paint_selector_get_gradient_gs2d_matrix_f(SPPaintSelector const *psel, NRMatrix *gs2d);
 
 void sp_paint_selector_set_gradient_properties (SPPaintSelector *psel, SPGradientUnits units, SPGradientSpread spread);
 void sp_paint_selector_get_gradient_properties (SPPaintSelector *psel, SPGradientUnits *units, SPGradientSpread *spread);
@@ -94,15 +97,14 @@ void sp_paint_selector_get_color_alpha (SPPaintSelector *psel, SPColor *color, g
 
 SPGradient *sp_paint_selector_get_gradient_vector (SPPaintSelector *psel);
 
-void sp_paint_selector_get_gradient_position_floatv (SPPaintSelector *psel, gfloat *pos);
-
-void sp_paint_selector_write_lineargradient (SPPaintSelector *psel, SPLinearGradient *lg, SPItem *item);
-void sp_paint_selector_write_radialgradient (SPPaintSelector *psel, SPRadialGradient *rg, SPItem *item);
-
 void sp_paint_selector_system_color_set (SPPaintSelector *psel, const SPColor *color, float opacity);
 
 SPPattern * sp_paint_selector_get_pattern (SPPaintSelector *psel);
 
 void sp_update_pattern_list ( SPPaintSelector *psel, SPPattern *pat);
 
+void sp_paint_selector_set_flat_color (SPPaintSelector *psel, SPDesktop *desktop, const gchar *color_property, const gchar *opacity_property);
+
+SPPaintSelectorMode sp_style_determine_paint_selector_mode (SPStyle *style, bool isfill);
+
 #endif
