@@ -29,27 +29,16 @@
 
 struct SPWidget {
 	GtkBin bin;
-
 	Inkscape::Application *inkscape;
-	SPObject *object;
-	SPRepr *repr;
-
-	/* fixme: We do not need these probably (Lauris) */
-	/* fixme: Or maybe configurable? For text widget? (Lauris) */
-	guint dirty : 1;
-	guint autoupdate : 1;
 };
 
 struct SPWidgetClass {
 	GtkBinClass bin_class;
 	void (* construct) (SPWidget *spw);
 	/* Selection change handlers */
-	void (* modify_selection) (SPWidget *spw, SPSelection *selection, guint flags);
-	void (* change_selection) (SPWidget *spw, SPSelection *selection);
-	void (* set_selection) (SPWidget *spw, SPSelection *selection);
-	void (* attr_changed) (SPWidget *spw, const guchar *key, const guchar *oldval, const guchar *newval);
-	/* Signal */
-	void (* set_dirty) (SPWidget *spw, gboolean dirty);
+	void (* modify_selection) (SPWidget *spw, Inkscape::Selection *selection, guint flags);
+	void (* change_selection) (SPWidget *spw, Inkscape::Selection *selection);
+	void (* set_selection) (SPWidget *spw, Inkscape::Selection *selection);
 };
 
 GtkType sp_widget_get_type (void);
@@ -58,18 +47,5 @@ GtkType sp_widget_get_type (void);
 /* Generic constructor for global widget */
 GtkWidget *sp_widget_new_global (Inkscape::Application *inkscape);
 GtkWidget *sp_widget_construct_global (SPWidget *spw, Inkscape::Application *inkscape);
-GtkWidget *sp_widget_construct_repr (SPWidget *spw, SPRepr *repr);
-
-void sp_widget_set_dirty (SPWidget *spw, gboolean dirty);
-void sp_widget_set_autoupdate (SPWidget *spw, gboolean autoupdate);
-
-const GSList *sp_widget_get_item_list (SPWidget *spw);
-
-/* fixme: Do clean way (Lauris) */
-#define SP_WIDGET_DOCUMENT(spw) SP_ACTIVE_DOCUMENT
-#define SP_WIDGET_DESKTOP(spw) SP_ACTIVE_DESKTOP
-#define SP_WIDGET_SELECTION(spw) SP_DT_SELECTION (SP_ACTIVE_DESKTOP)
-
-
 
 #endif
