@@ -1,4 +1,4 @@
-#define TOOL_BUTTON_SIZE 28
+// #define TOOL_BUTTON_SIZE 28
 
 // GTK uses 24 for icon sizes by default.  Spacing adjust to keep the
 // toolbar the same as other GTK applications.  If we want that, use
@@ -6,12 +6,11 @@
 //#define AUX_BUTTON_SIZE 24
 //#define AUX_SPACING 2
 
-#define AUX_BUTTON_SIZE 20
+// #define AUX_BUTTON_SIZE 20
 #define AUX_SPACING 3
 
 #define AUX_BETWEEN_BUTTON_GROUPS 7
 #define AUX_BETWEEN_SPINBUTTONS 0
-#define AUX_FONT_SIZE 8000
 #define AUX_SPINBUTTON_WIDTH 57
 #define AUX_SPINBUTTON_HEIGHT 20
 #define AUX_OPTION_MENU_WIDTH 55
@@ -38,7 +37,7 @@
 
 #define RULER_WIDTH           12
 #define RULER_HEIGHT          12
-#define RULER_FONT_SIZE 7000
+#define RULER_FONT_SIZE 6500
 #define RULER_FONT_VERTICAL_SPACING 1.2
 
 /*
